function [PredPaths,PredMean,MeanShocks,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ,KernelX,KernelY] = BVARCondPredictionPathsPostMode(Psi,Pi,Omega,DSGEModel,CurrINI,DPred,Z,U,h,NumPaths,AnnualizeData)
% BVARCondPredictionPathsPostMode: Computes prediction paths of a certain length for the Bayesian VAR model using
%                                  given estimates of the parameters and a set of conditioning assumptions on the
%                                  predictions of the observed variables.
%
% USAGE:
%
%       [PredPaths,PredMean,MeanShocks,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ,...
%                  KernelX,KernelY] = BVARCondPredictionPathsPostMode(Psi,Pi,Omega,DSGEModel,CurrINI,DPred,Z,U,h,...
%                                                                     NumPaths,AnnualizeData)
%
% REQUIRED INPUT:  Psi (pxq) matrix with steady state parameters.
%
%                  Pi (pxpk) matrix with the parameters on lagged enogenous variables.
%
%                  Omega (pxp) covariance matrix of the VAR residuals.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure= with initialization information.
%
%                  DPred (matrix) of dimension qxh with data for the exogenous variables over the prediction sample.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                    predictions on.
%
%                  U (vector) of dimension mxg with initial values for the mapping between the observed
%                    data and the conditioning data.
%
%                  h (integer) with the length of the prediction sample.
%
%                  NumPaths (integer) with the number of prediction paths to compute
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
% REQUIRED OUTPUT: PredPaths (p x h x NumPaths) 3D matrix with the NumPaths possible paths for the p
%                    endogenous variables.
%
%                  PredMean (pxh) matrix with the mean predictions at the posterior mode for the p
%                    endogenous variables.
%
%                  MeanShocks (matrix) of dimension pxh with the mean values of the shocks at the
%                    population mean prediction.
%
%                  PredEventData (p x 8) matrix where the first column holds the number of times
%                    the PredPaths data satisfies the event, the second hold the number of times
%                    the PredPaths data is greater than the event, the third is the number of times
%                    the data is above the upper bound, while the fourth column holds the number
%                    of times the event has been tested. The fifth and sixth column measure the
%                    average deviation from the lower bound when below the lower bound and the square
%                    of this, while the seventh and eighth columns hold similar data for the upper
%                    bound of the event when the variables are above the upper bound.
%
%                  YObsEventData (p x 8) matrix where the first column hold the number of times
%                    the PredPaths (adjusted such that the mean is equal to the realized value)
%                    satisfies the event, the second holds the number of times the adjusted paths
%                    data is greater than the event, the third the number of time the data is above
%                    the upper bound, while the fourth column holds the number of times the event has
%                    been tested. The fifth and sixth column measure the average deviation from the
%                    lower bound when below the lower bound and the square of this, while the seventh
%                    and eighth columns hold similar data for the upper bound of the event when the
%                    variables are above the upper bound.
%
%                  MultiModestyStat (matrix) with NumPaths x 2 elements. The first column is the the
%                    multivariate modesty statistic based on the conditioning consistent shocks and
%                    the second is the value of this statistic for randomly drawn shocks.
%
%                  UniModestyStat (matrix) with NumPaths x p elements. Each column gives the univariate
%                    modesty for a particular observed variable.
%
%                  UniModestyStatLZ (vector) with the values of the univariate modesty statistics suggested
%                    by Leeper-Zha (other shocks are zero over examined prediction horizon) for all
%                    observed variables.
%
%                  KernelX (3D matrix) of dimension p x 2^8 x h with the x-axis values.
%
%                  KernelY (3D matrix) of dimension p x 2^8 x h with the y-axis values.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 4, 2011.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 07-03-2011: Finished the function from paths computations and onwards.
%
% * 02-01-2012: Updated the documentation.
%
% * 17-01-2012: The K2 matrices can now be lag-varying.
%
% * 07-02-2012: Added support for missing observations in the conditioning assumptions.
%
% * 09-03-2012: Made sure that missing observations don't affect the prediction event calculations.
%
% * 13-03-2012: Fixed a bug with initialization of "PsiTg".
%
% * 16-03-2012: Removed a disp command.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 09-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
PredPaths = [];
PredMean = [];
MeanShocks = [];
PredEventData = [];
YObsEventData = [];
KernelX = [];
KernelY = [];
%
% set up the data matrices
%
p = length(DSGEModel.BVARY);
k = DSGEModel.BVARLags;
q = length(DSGEModel.BVARX);
[m,g] = size(Z);
%
if AnnualizeData==0;
   MultiModestyStat = zeros(NumPaths,2);
   UniModestyStat = zeros(NumPaths,p);
   UniModestyStatLZ = zeros(1,p);
else;
   MultiModestyStat = [];
   UniModestyStat = [];
   UniModestyStatLZ = [];
end;
%
% Determine the sample
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood with the Kalman filter
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Check if we should adjust this based on k, the lag order
%
if FirstPeriod+KalmanFirstObservation-1<k+1;
   KalmanFirstObservation = k+2-FirstPeriod;
end;
%
% rewrite the VAR on VAR form
%
if k>1;
   B = [Pi;eye(p*(k-1)) zeros(p*(k-1),p)];
   J = [eye(p) zeros(p,p*(k-1))]';
else;
   B = Pi;
   J = eye(p);
end;
%
% setup the initial value for Y at period T
%
T = LastPeriod-(FirstPeriod+KalmanFirstObservation-1)+1;
X_T = zeros(p*k,1);
for i=1:k;
   X_T(1+(p*(i-1)):p*i,1) = DSGEModel.Y(DSGEModel.BVARY,LastPeriod-i+1);
end;
%
% and now for the exogenous variables
%
if isempty(DSGEModel.BVARX)==0;
   d = DPred;
   dLags = zeros(q*k,1);
   for i=1:k;
      dLags(1+(q*(i-1)):q*i,:) = DSGEModel.X(DSGEModel.BVARX,LastPeriod-i+1);
   end;
   Y_T = X_T-(kron(eye(k),Psi)*dLags);
else;
   d = [];
   Y_T = X_T;
end;
%
% setup an initialization data for the endogenous variables
%
Xinit = [];
if AnnualizeData==1;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Xinit = DSGEModel.Y(DSGEModel.BVARY,LastPeriod-2:LastPeriod);
   elseif strcmp(DSGEModel.DataFrequency,'m')==1;
      Xinit = DSGEModel.Y(DSGEModel.BVARY,LastPeriod-10:LastPeriod);
   end;
end;
%
% conditioning mapping
%
K1 = DSGEModel.K1(DSGEModel.BVARY,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(DSGEModel.BVARY,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(DSGEModel.BVARY,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
%
% check if we have enough sources of variation
%
if rank(Omega)<m;
   txt = ['The number of unique shocks that can be used for the BVAR is ' int2str(rank(Omega)) ' (equal to the rank of the residual covariance matrix) while the required number of conditioning shocks is ' int2str(m) '. You must therefore drop some conditioning assumptions.'];
   About(txt,'error','Error - Conditional Shocks',200,500,CurrINI);
   PredPaths = [];
   return;
end;
%
% setup exogenous variables and the G-matrix on YT
%
PsiTg = zeros(p*g,1);
G = zeros(p*g,p*k);
D = eye(p*g);
ZTg = zeros(m*g,1);
UT = zeros(m*g,1);
BExp = B;
K = zeros(p*g,m*g);
for i=1:g;
   Ind = (1+((g-i)*p):(g-i+1)*p);
   Ind1 = Ind;
   Ind2 = (1+((g-i)*m):(g-i+1)*m);
   if q>0;
      PsiTg(Ind) = Psi*d(:,i);
   end;
   G(Ind,:) = J'*BExp;
   K(Ind,Ind2) = K1;
   Bi = B;
   for j=1:i-1;
      Ind1 = Ind1+p;
      Ind3 = Ind+(j*p);
      D(Ind,Ind1) = J'*Bi*J;
      if K2length==2;
         K(Ind3,Ind2) = K2;
      else;
         if j<=size(K2,3);
            K(Ind3,Ind2) = K2(:,:,j);
         end;
      end;
      Bi = Bi*B;
   end;
   ZTg(Ind2) = Z(:,i);
   UT(Ind2) = U;
   BExp = BExp*B;
end;
%
% create a vector with positions for the conditioning assumptions that should be used
%
PosVec = (1:m*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
%
% adjust for missing restrictions
%
K = K(:,PosVec);
ZTg = ZTg(PosVec,1);
UT = UT(PosVec,1);
%
% compute the mean predictions at the posterior mode
%
PredMean = zeros(p,h);
MeanShocks = zeros(p,h);
if AnnualizeData==0;
   BTilde = zeros(p*k,p*g);
   Bi = eye(p*k);
   SigmaBarY = zeros(p*k,p*k);
   SigmaBarLZ = zeros(p*k,p*k);
end;
kTg = ZTg-UT-(K'*(PsiTg+(G*Y_T)));
YLast = Y_T;
IgOmega = kron(eye(g),Omega);
InvKDIgOmega = eye(length(PosVec))/(K'*D*IgOmega*D'*K);
muETg = IgOmega*D'*K*InvKDIgOmega*kTg;
MeanShocks(:,1:g) = fliplr(reshape(muETg,p,g));
SigmaETg = IgOmega-(IgOmega*D'*K*InvKDIgOmega*K'*D*IgOmega);
SigmaETg = diag(abs(diag(SigmaETg)))+tril(SigmaETg,-1)+tril(SigmaETg,-1)';
%
[SE,LE] = schur(SigmaETg);
StdEShocks = SE*sqrt(abs(LE));
CovMatShocks = Omega*K1*((K1'*Omega*K1)\K1')*Omega;
%
if q>0;
   PsiDTh = Psi*d;
else;
   PsiDTh = zeros(p,h);
end;
for i=1:h;
   if i<=g
      YLast = (B*YLast)+J*MeanShocks(:,i);
      if AnnualizeData==0;
         BTilde(:,1+((i-1)*p):p*i) = Bi*J;
         Bi = Bi*B;
         SigmaBarY = (J*Omega*J')+(B*SigmaBarY*B');
         SigmaBarLZ = (J*CovMatShocks*J')+(B*SigmaBarLZ*B');
         if i==g;
            %
            % compute Leeper-Zha type of univariate modesty statistics
            %
            PhiDiff = J'*BTilde*muETg;
            StdBarLZ = sqrt(abs(diag(J'*SigmaBarLZ*J)));
            StdBarLZ = StdBarLZ+(StdBarLZ==0);
            UniModestyStatLZ = (PhiDiff./StdBarLZ)';
            SigmaY = J'*SigmaBarY*J;
         end;
      end;
   else;
      YLast = (B*YLast);
   end;
   PredMean(:,i) = PsiDTh(:,i)+(J'*YLast);
end;
%
% check if we should annualize the population mean predictions
%
if AnnualizeData==1;
   PredMean = Annualize([Xinit PredMean],DSGEModel.annual(DSGEModel.BVARY),DSGEModel.annualscale(DSGEModel.BVARY));
end;
%
% set up the output matrix
%
PredPaths = zeros(p,h,NumPaths);
if (DSGEModel.RunPredictionEvent==1)&(size(DSGEModel.Y,2)>=LastPeriod+h);
   YObserved = DSGEModel.Y(DSGEModel.BVARY,LastPeriod+1:LastPeriod+h);
   if AnnualizeData==1;
      YObserved = Annualize([Xinit YObserved],DSGEModel.annual(DSGEModel.BVARY),DSGEModel.annualscale(DSGEModel.BVARY));;
   end;
else;
   YObserved = [];
end;
%
% and the covariance matrix
%
OmegaStMat = chol(Omega)';
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(NumPaths) ' Prediction Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = 'BVAR Conditional Predictive Dist - Posterior Mode';
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
end;
%
% start the iterations
%
it = 1;
abort = '';
for it=1:NumPaths
   %
   % check if we should update the progress dialog
   %
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         PredPaths = [];
         break;
      else;
         ProgressDLG([it/NumPaths it]);
      end;
   end;
   %
   % draw the residuals
   %
   UpsilonMat = randn(p,h);
   EpsilonMat = OmegaStMat*UpsilonMat;
   UpsilonMat(:,1:g) = fliplr(reshape(muETg+(StdEShocks*vec(fliplr(UpsilonMat(:,1:g)))),p,g));
   if h>g;
      UpsilonMat(:,g+1:h) = EpsilonMat(:,g+1:h);
   end;
   %
   % create the path
   %
   X = zeros(p,h);
   YLast = Y_T;
   for i=1:h;
      %
      % update the Y-vector
      %
      YLast = (B*YLast)+(J*UpsilonMat(:,i));
      %
      % calculate the X-vector
      %
      X(:,i) = PsiDTh(:,i)+(J'*YLast);
      %
      % deal with modesty statistics
      %
      if (AnnualizeData==0)&(i==g);
         PhiDiff = J'*BTilde*vec(fliplr(UpsilonMat(:,1:g)));
         PhiDiffC = J'*BTilde*vec(fliplr(EpsilonMat(:,1:g)));
         MultiModestyStat(it,1) = PhiDiff'*(SigmaY\PhiDiff);
         MultiModestyStat(it,2) = PhiDiffC'*(SigmaY\PhiDiffC);
         UniModestyStat(it,:) = (PhiDiff./sqrt(diag(SigmaY)))';
      end;
   end;
   if AnnualizeData==0;
      PredPaths(:,:,it) = X;
   else;
      PredPaths(:,:,it) = Annualize([Xinit X],DSGEModel.annual(DSGEModel.BVARY),DSGEModel.annualscale(DSGEModel.BVARY));
   end;
   drawnow;
end;
%
% close the wait handle
%
if DSGEModel.ShowProgress==1;
   set(WaitHandle,'UserData','done');
   close(WaitHandle);
   drawnow;
   pause(0.02);
end;
if (strcmp(abort,'cancel')==0)&(DSGEModel.RunPredictionEvent==1);
   MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
   if DSGEModel.AdjustPredictionPaths==1;
      %
      % we need to fix the paths such that the mean is correct
      %
      AdjustMean = PredMean-MeanPrediction;
      for i=1:NumPaths;
         PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
   else;
      PEPredPaths = PredPaths;
   end;
   %
   % compute the prediction events results
   %
   PredEventData = CalculatePredictionEvents(PEPredPaths,DSGEModel.PredictionEvent(DSGEModel.BVARY,:));
   %
   if isempty(YObserved)==0;
      AdjustMean = YObserved-MeanPrediction;
      %
      % make sure that missing observations don't affect adjustment
      %
      AdjustMean(isnan(AdjustMean)) = 0;
      for i=1:NumPaths;
         YPEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
      YObsEventData = CalculatePredictionEvents(YPEPredPaths,DSGEModel.PredictionEvent(DSGEModel.BVARY,:));
   end;
end;
%
% check if we should compute kernel density estimates
%
if (strcmp(abort,'cancel')==0)&(NumPaths>=500);
   if DSGEModel.RunPredictionEvent==0;
      if DSGEModel.AdjustPredictionPaths==1;
         MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
         AdjustMean = PredMean-MeanPrediction;
         for i=1:NumPaths;
            PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
         end;
      else;
         PEPredPaths = PredPaths;
      end;
   end;
   %
   [KernelX,KernelY] = PredictionKernelDensity(PEPredPaths);
end;

%
% end of BVARCondPredictionPathsPostMode.m
%
