function ParameterNames = CheckUpdateParameterNames(ParameterNames,ModelParameters,theta,thetaPositions,DSGEModel)
% CheckUpdateParameterNames: Checks if the suggested selection of calibrated and updated (additional) parameter
%                            names are correct based on a slight change in the value of theta, the estimated
%                            parameters.
%
% USAGE:
%
%       ParameterNames = CheckUpdateParameterNames(ParameterNames,ModelParameters,theta,thetaPositions,DSGEModel)
%
% REQUIRED INPUT:  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                                 "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all", "estimated",
%                                 "additional", and "calibrated". Each field returns a string matrix with the parameter names.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the DSGE model.
%
%                  theta (vector) whose entries correspond to values of the parameters of the DSGE model that are estimated
%
%                  thetaPositions (structure) with fields parameter and distribution. This function needs the former field
%                                 to know which parameter name corresponds to position i in theta.
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                                 input files, variable and sample data.
%
% REQUIRED OUTPUT: ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                                 "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all", "estimated",
%                                 "additional", and "calibrated". Each field returns a string matrix with the parameter names.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 23, 2017.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 24-10-2017: Updated the documentation.
%
% * 11-01-2018: Updated the documentation.
%
% * 31-10-2018: Made sure that the new theta value is not proportional to the old value
%               and that it is sufficiently larger in absolute terms.
%
% * 02-11-2018: Made sure that the new theta update parameters we compare have real values.
%
% * 21-01-2019: Updated the documentation.
%
% * 19-03-2019: Added the check if the additional parameters are actually calibrated.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 21-02-2022: Changed eval calls to feval.
%
% * 03-01-2023: Updated the documentation.
%
% * 01-09-2023: Changed the increase of theta in the test for having update parameters from 100 times
%               theta plus a random number to 2 times this value.
%
% * 24-10-2023: Made sure that the previous change also applied to the additional parameters.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% first check if there the calibrated parameters are actually updated
%
if isempty(ParameterNames.calibrated)==0;
   %
   % change the values of the theta vector and update the ModelParameters structure
   % accordingly
   %
   Newtheta = 2*(theta+randn(length(theta),1));
   NewModelParameters = ThetaToModelParameters(ModelParameters,Newtheta,thetaPositions);
   %
   % now run the update parameters file
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      NewModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),NewModelParameters);
   else;
      return;
   end;
   %
   % now we can check if the fields in NewModelParameters related to the calibrated parameters
   % have changed or not
   %
   Positions = [];
   NumCal = size(ParameterNames.calibrated,1);
   for i=1:NumCal;
      CalParName = StringTrim(ParameterNames.calibrated(i,:));
      if abs(real(getfield(NewModelParameters,CalParName))-getfield(ModelParameters,CalParName))>0;
         %
         % add the parameter name to additional
         %
         Positions = [Positions i];
         ParameterNames.additional = strvcat(ParameterNames.additional,CalParName);
      end;
   end;
   %
   % check if we need to remove parameters from calibrated
   %
   if isempty(Positions)==0;
      KeepEntries = setdiff((1:NumCal),Positions);
      NumKE = length(KeepEntries);
      calibrated = '';
      for i=1:NumKE;
         calibrated = strvcat(calibrated,StringTrim(ParameterNames.calibrated(KeepEntries(i),:)));
      end;
      ParameterNames.calibrated = calibrated;
   end;
end;
%
% second check if some of the additional entries are actually calibrated
%
if isempty(ParameterNames.additional)==0;
   if exist('NewModelParameters')==0;
      %
      % change the values of the theta vector and update the ModelParameters structure
      % accordingly
      %
      Newtheta = 2*(theta+randn(length(theta),1));
      NewModelParameters = ThetaToModelParameters(ModelParameters,Newtheta,thetaPositions);
   end;
   %
   % now run the update parameters file
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      NewModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),NewModelParameters);
   else;
      return;
   end;
   %
   % now we can check if the fields in NewModelParameters related to the updated parameters
   % have changed or not
   %
   Positions = [];
   NumAdd = size(ParameterNames.additional,1);
   for i=1:NumAdd;
      AddParName = StringTrim(ParameterNames.additional(i,:));
      if getfield(NewModelParameters,AddParName)==getfield(ModelParameters,AddParName);
         %
         % add the parameter name to additional
         %
         Positions = [Positions i];
         ParameterNames.calibrated = strvcat(ParameterNames.calibrated,AddParName);
      end;
   end;
   %
   % check if we need to remove parameters from calibrated
   %
   if isempty(Positions)==0;
      KeepEntries = setdiff((1:NumAdd),Positions);
      NumKE = length(KeepEntries);
      additional = '';
      for i=1:NumKE;
         additional = strvcat(additional,StringTrim(ParameterNames.additional(KeepEntries(i),:)));
      end;
      ParameterNames.additional = additional;
   end;
end;

%
% end of CheckUpdateParameterNames.m
%
