function status = VerifyPredeterminedVariables(PredeterStateVarPos,PredeterStateEQPos,NumLead,DSGEModel,CurrINI)
% VerifyPredeterminedVariables: Tests if the specified predetermined variables can indeed be treated as predetermined
%                               in a DSGE model based on solving it with the Klein solver.
%
% USAGE:
%
%       status = VerifyPredeterminedVariables(PredeterStateVarPos,PredeterStateEQPos,NumLead,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  PredeterStateVarPos (vector) with the positions of the specified predetermined
%                                      variables among the set of state variables.
%
%                  PredeterStateEQPos (vector) with the positions of the state equations which
%                                      are the equations for the predetermined variables.
%
%                  NumLead (integer) with the number of leads in the DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: status (boolean) that takes the value 1 if the specified predetermined variables
%                                      can be treated as predetermined variables according to the
%                                      Klein solver of a DSGE model.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2014 European Central Bank.
%
%                       First version: August 11, 2010.
%                        This version: August 11, 2014.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
status = 0;
%
% get some necessary parameter data
%
[InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames] = InitializeDSGEModelSimulation(DSGEModel,CurrINI,0);
if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
   if FileExist(ModeFile)==1;
      ErrorStr = '';
      try;
         ModeData = load(ModeFile);
      catch;
         ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
      end;
      if isempty(ErrorStr)==1;
         theta = ModeData.theta;
         thetaPositions = ModeData.thetaPositions;
         ModelParameters = ModeData.ModelParameters;
         ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
         ParameterNames = ModeData.ParameterNames;
      else;
         About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         return;
      end;
   end;
end;
%
% shorten some variables
%
mcode = 0;
condn = 10^(-DSGEModel.AIMTolerance-2);
StateVariablePositions = DSGEModel.StateVariablePositions;
StateEquationPositions = DSGEModel.StateEquationPositions;
StateShockPositions = DSGEModel.StateShockPositions;
%
% Construct structural coefficient matrices.
%
[GMatrix,HMatrix] = compute_aim_matrices(ModelParameters);
if isreal(GMatrix)==0;
   mcode = 8;
end;
if isreal(HMatrix)==0;
   mcode = 8;
end;
if mcode==0;
   [rh,ch] = size(HMatrix);
   [rg,cg] = size(GMatrix);
   H = zeros(rh,ch);
   H(1:rg,1:cg) = GMatrix;
   H = H+HMatrix;
   Hlag = H(:,1:rh);
   Hexp = H(:,(2*rh)+1:(2+NumLead)*rh);
   H = H(:,rh+1:2*rh);
   StateVarPos = StateVariablePositions;
   if NumLead>1;
      for j=2:NumLead;
         StateVarPos = [StateVarPos StateVariablePositions+(rh*(j-1))];
      end;
   end;
   Hlag = Hlag(StateEquationPositions,StateVariablePositions);
   Hexp = Hexp(StateEquationPositions,StateVarPos);
   D = -H(StateEquationPositions,StateShockPositions);
   H = H(StateEquationPositions,StateVariablePositions);
   [k,l] = size(Hexp);

   disp(H)





   mcode = 1;
end;
%
% check mcode
%
if mcode~=1;
   if mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   txt = ['The Klein solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information','DSGE Model To VAR Model - Posterior Mode',120,500,CurrINI);
   return;
end;

%
% end of VerifyPredeterminedVariables.m
%
