function DSGEModel = VerifyStateScenarioInformation(DSGEModel,DataStructure)
% VerifyStateScenarioInformation: Checks if there is valid information for running state
%                                 variable scenarios from the DataStructure structure.
%
% USAGE:
%
%       DSGEModel = VerifyStateScenarioInformation(DSGEModel,DataStructure)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  DataStrcuture (structure) where this function looks at the fields State.S,
%                            State.data, State.names, and State.startperiod.
%
% REQUIRED OUTPUT: DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 15, 2009.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-04-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 24-06-2010: Fixed some bugs related to SData dimensions.
%
% * 04-08-2010: The field "ConstrainedStateVariables" is now also supported.
%
% * 05-08-2010: The field "ConstrainedStateValue" is now also supported.
%
% * 17-08-2010: The function now also checks that the number of rows in S matches
%               the number of state variables.
%
% * 27-10-2010: The function no longer checks the rank of the S-matrix. Moreover, it no longer
%               requires that the number of possible state scenario restrictions is less than
%               or equal to the number of structural shocks. Both these changes make it possible
%               to have alternative scenario paths for a given state variable.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DSGEModel.S = [];
DSGEModel.SData = [];
DSGEModel.SVariableNames = '';
%
if isfield(DSGEModel,'StateScenarioShocks')==0;
   DSGEModel.StateScenarioShocks = [];
end;
if isfield(DSGEModel,'StateScenarioVariables')==0;
   DSGEModel.StateScenarioVariables = [];
end;
if isfield(DSGEModel,'ConstrainedStateVariables')==0;
   DSGEModel.ConstrainedStateVariables = [];
end;
if isfield(DataStructure,'State')==0;
   DSGEModel.StateScenarioShocks = [];
   DSGEModel.StateScenarioVariables = [];
   DSGEModel.ConstrainedStateVariables = [];
   DSGEModel.ConstrainedStateValue = 1;
   DSGEModel.ScenarioStateValue = 1;
   DSGEModel.AnnScenarioStateValue = 1;
   return;
end;
if isfield(DataStructure.State,'S')==0;
   DSGEModel.StateScenarioShocks = [];
   DSGEModel.StateScenarioVariables = [];
   DSGEModel.ConstrainedStateVariables = [];
   DSGEModel.ConstrainedStateValue = 1;
   DSGEModel.ScenarioStateValue = 1;
   DSGEModel.AnnScenarioStateValue = 1;
   return;
end;
if isfield(DataStructure.State,'data')==0;
   DSGEModel.StateScenarioShocks = [];
   DSGEModel.StateScenarioVariables = [];
   DSGEModel.ConstrainedStateVariables = [];
   DSGEModel.ConstrainedStateValue = 1;
   DSGEModel.ScenarioStateValue = 1;
   DSGEModel.AnnScenarioStateValue = 1;
   return;
end;
if isfield(DataStructure.State,'names')==0;
   DSGEModel.StateScenarioShocks = [];
   DSGEModel.StateScenarioVariables = [];
   DSGEModel.ConstrainedStateVariables = [];
   DSGEModel.ConstrainedStateValue = 1;
   DSGEModel.ScenarioStateValue = 1;
   DSGEModel.AnnScenarioStateValue = 1;
   return;
end;
%
% all fields are available.
%
% check if dimensions match for S, SVariableNames, and SData
%
S = DataStructure.State.S;
SData = DataStructure.State.data;
SVariableNames = char(DataStructure.State.names);
[r s] = size(S);
sv = size(SVariableNames,1);
%
% check if the data on the state variable assumptions has the wrong dimension
% and should be transposed.
%
if (size(SData,1)>size(SData,2))&(size(SData,1)~=s);
   SData = SData';
end;
[sd Ts] = size(SData);
if (Ts==0)|(s==0)|(s~=sd)|(s~=sv)|(r~=size(DSGEModel.StateVariableNames,1));
   DSGEModel.StateScenarioShocks = [];
   DSGEModel.StateScenarioVariables = [];
   DSGEModel.ConstrainedStateVariables = [];
   DSGEModel.ConstrainedStateValue = 1;
   DSGEModel.ScenarioStateValue = 1;
   DSGEModel.AnnScenarioStateValue = 1;
   return;
end;
%
% we're OK
%
DSGEModel.S = S;
DSGEModel.SData = SData;
DSGEModel.SVariableNames = SVariableNames;
%
m = min(s,length(DSGEModel.StateShockPositions));
if (isempty(DSGEModel.StateScenarioVariables)==1)|(max(DSGEModel.StateScenarioVariables)>s)|(length(DSGEModel.StateScenarioVariables)>m);
   DSGEModel.StateScenarioVariables = 1;
end;
if (isempty(DSGEModel.ConstrainedStateVariables)==1)|(max(DSGEModel.ConstrainedStateVariables)>s)|(length(DSGEModel.ConstrainedStateVariables)>m);
   DSGEModel.ConstrainedStateVariables = 1;
end;
if (isempty(DSGEModel.StateScenarioShocks)==1)|(max(DSGEModel.StateScenarioShocks)>length(DSGEModel.StateShockPositions));
   DSGEModel.StateScenarioShocks = 1;
end;
%
% check if there's information about the scenario start value
%
if (DSGEModel.ScenarioStateValue==1)&(isfield(DataStructure.State,'startperiod')==1)&(length(DataStructure.State.startperiod)==2)&(isnumeric(DataStructure.State.startperiod)==1);
   %
   % check if we have a valid entry
   %
   DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
   DateVector = DateVector(DSGEModel.KalmanFirstObservation:length(DateVector));
   if strcmp(lower(DSGEModel.DataFrequency),'q')==1;
      StartValue = DataStructure.State.startperiod(1)+(((2*DataStructure.State.startperiod(2))-1)/8);
   elseif strcmp(lower(DSGEModel.DataFrequency),'m')==1;
      StartValue = DataStructure.State.startperiod(1)+(((2*DataStructure.State.startperiod(2))-1)/24);
   else;
      StartValue = DataStructure.State.startperiod(1);
   end;
   StartPos = (DateVector==StartValue)*(1:length(DateVector))';
   if StartPos>0;
      DSGEModel.ScenarioStateValue = length(DateVector)-StartPos+1;
      if DSGEModel.AnnScenarioStateValue==1;
         DSGEModel.AnnScenarioStateValue = DSGEModel.ScenarioStateValue;
      end;
   end;
end;

%
% end of VerifyStateScenarioInformation.m
%
