function UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,WindowNameStr,Weights,Share)
% WeightBasedPosteriorDraws: Determines which of the NumDraws to use based on the Weights
%                            and random draws satisfying the bins implied by accumulating
%                            weights and dividing by the number of weights.
%
% USAGE:
%
%       UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,Weights,Share)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (strcuture) with initialization information.
%
%                  WindowNameStr (string vector) with the name to use for the selection
%                            dialog.
%
%                  Weights (vector) of dimension NumDraws x 1 with the weights used for
%                            the posterior sampler.
%
%                  Share (scalar) with the percent of posterior draws to keep.
%
% REQUIRED OUTPUT: UseDraws (vector) with the positions of the posterior draws to use.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: May 16, 2022.
%                        This version: January 6, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-05-2022: Added the forgotten random number initialization.
%
% * 18-05-2022: Removed use of randn as it it not needed.
%
% * 06-01-2023: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

NumDraws = length(Weights);
UseDraws = (1:NumDraws);
%
if isempty(Share)==1;
   SelectionOptions = strvcat('0.1%','0.2%','0.3%','0.4%','0.5%','0.6%','0.7%','0.8%','0.9%','1%','2%','3%','4%','5%','6%','7%','8%','9%','10%','11%','12%','13%','14%','15%','16%','17%','18%','19%','20%','21%','22%','23%','24%','25%','26%','27%','28%','29%','30%','31%','32%','33%','34%','35%','36%','37%','38%','39%','40%','41%','42%','43%','44%','45%','46%','47%','48%','49%','50%','51%','52%','53%','54%','55%','56%','57%','58%','59%','60%','61%','62%','63%','64%','65%','66%','67%','68%','69%','70%','71%','72%','73%','74%','75%','76%','77%','78%','79%','80%','81%','82%','83%','84%','85%','86%','87%','88%','89%','90%','91%','92%','93%','94%','95%','96%','97%','98%','99%','100%');
   [action,SelectedOption] = SelectionDlg(SelectionOptions,19,'Select the percent of posterior draws to use:','',WindowNameStr,'','',CurrINI);
   if strcmp(lower(action),'cancel')==1;
      UseDraws = [];
      return;
   end;
   Share = str2num(strrep(StringTrim(SelectionOptions(SelectedOption,:)),'%',''))/100;
end;
if Share==1;
   return;
end;
WeightBins = cumsum(Weights)/NumDraws;
NDraws = floor(NumDraws*Share);
%
% initialize the random number generator
%
if DSGEModel.RandomNumberValue==1;
   rand('state',0);
else;
   rand('state',sum(100*clock));
end;
Draws = rand(NDraws,1);
UseDraws = zeros(NDraws,1);
PosVec = (1:NumDraws)';
%
% collect the positions of the draws to use
%
for i=1:NDraws;
   UseDraws(i) = max((WeightBins<=Draws(i)).*PosVec);
end;
UseDraws = sort(UseDraws);

%
% end of WeightBasedPosteriorDraws.m
%