function LogQ = MixedDistributionPDF(phiNew,phi,phiTilde,SigmaphiTilde,alpha,cstar)
% MixedDistributionPDF: Computes the log height of the mixed normal distribution for the
%                       SMC algorithm.
%
% USAGE:
%
%       LogQ = MixedDistributionPDF(phi,phiTilde,SigmaphiTilde,alpha,cstar)
%
% REQUIRED INPUT:  phiNew (vector) of dimension NumParam x 1 with the parameter value to
%                         evaluate the mixture of normals at.
%
%                  phi (vector) of dimension NumParam x 1 with location values
%                      for the parameter draw.
%
%                  phiTilde (vector) of dimension NumParam X 1 with other location values
%                      for the parameter draw.
%
%                  SigmaphiTilde (matrix) of dimension NumParam x NumParam with the
%                      basic covariance matrix for the parameter draw.
%
%                  alpha (scalar) with the mixing weight for the draws. Takes values between
%                      zero and unity.
%
%                  cstar (scalar) with the scaling factor for the covariance matrix for the
%                      parameter draw.
%
% REQUIRED OUTPUT: LogQ (scalar) with the log height of the mixture of normals.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 18, 2017.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-08-2017: Updated the documentation.
%
% * 12-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
cstar2 = cstar^2;
NumParam = length(phi);
LogDetSigma = LogDet(cstar2*SigmaphiTilde);
InvSigma = (1/cstar2)*eye(NumParam)/SigmaphiTilde;
LogQ = logMultiNormalPDF(phiNew,phi,LogDetSigma,InvSigma);
DiagSigma = diag(diag(SigmaphiTilde));
LogQ1 = logMultiNormalPDF(phiNew,phi,LogDet(cstar2*DiagSigma),eye(NumParam)/(cstar2*DiagSigma));
LogQ2 = logMultiNormalPDF(phiNew,phiTilde,LogDetSigma,InvSigma);
LogQ = (alpha*LogQ)+(((1-alpha)/2)*(LogQ1+LogQ2));

%
% MixedDistributionPDF.m
%
