function DSGEVARCheckPosteriorMode(DSGEModel,CurrINI,controls)
% DSGEVARCheckPosteriorMode: Opens graphs of the posterior mode for DSGE-VAR. If the graphs already exists on disk,
%                            these are shown. Otherwise, new ones are created based on the information
%                            stored to disk about the posterior mode.
%
% USAGE:
%
%       DSGEVARCheckPosteriorMode(DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT: DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 CurrINI (structure), with initialization information.
%
%                 controls (structure), whose fields provide handles to all the UI
%                           controls in YADA's main dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 22, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 23-02-2010: Updated the documentation.
%
% * 10-03-2010: Made sure that controls.tab6 gets focus when the matlab version supports this.
%
% * 16-12-2010: Took into account that "logJointPosteriorPhiDSGEVAR", "logJointPosteriorThetaDSGEVAR",
%               "logPosteriorPhiDSGEVAR", and "logPosteriorThetaDSGEVAR" now take the input variable
%               "OrderQZ" from the CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy call to "CopyFile" calls.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 06-07-2011: Took into account that "logPosteriorThetaDSGEVAR" and "logJointPosteriorThetaDSGEVAR"
%               now require the "ParameterBounds" input variable.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 03-12-2014: Took into account that "VillaniCheckOptimum" takes the boolean input variable "ShowFigs".
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 23-05-2017: Took into account that VillaniCheckOptimum now requires an additional input variable.
%
% * 12-06-2017: The function now supports the system prior.
%
% * 30-08-2017: Took into account that "logPosteriorPhiDSGEVAR" and "logPosteriorThetaDSGEVAR" require
%               the input variable "NumWorkers".
%
% * 16-01-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 14-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 01-04-2020: Took into account that VillaniCheckOptimum now requires an additional input variable.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 08-02-2021: Made sure that figures are not opened twice!
%
% * 04-01-2022: Updated the documentation.
%
% * 13-01-2022: Took DSGEModel.ShowEstimationLog into account.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

drawnow;
pause(0.02);
%
% determine string with 1's and 0's for the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% check if joint or marginal mode
%
if gcbo==controls.dsgevarview.checkpostmode.margmode;
   [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
   ModeType = 'Marginal Posterior Mode';
   FileAddOn = 'Marginal';
   MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
else;
   [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
   ModeType = 'Joint Posterior Mode';
   FileAddOn = 'Joint';
   MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
end;
%
% get suitable data for the selection dialog
%
[LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
[action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['DSGE-VAR - Check ' ModeType],'','',CurrINI);
if strcmp(lower(action),'ok')==0;
   return;
end;
%
lambdai = lambda(value);
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
file = FixFilePath([GraphDirectory '\' FileAddOn 'PosteriorModeOptimum-DSGEVAR-' num2str(lambdai,'%0.4f')]);
if FileExist([file '-phi.fig'])==1;
   txt = ['YADA has located old plots of the posterior mode in the file "' GetFilename(file) '-phi.fig". Would you like to open this file?'];
   answer = Query(txt,'question',200,['DSGE-VAR - Check ' ModeType],500,'yes',CurrINI);
elseif FileExist([file '-phi-1.fig'])==1;
   txt = ['YADA has located old plots of the posterior mode in the files "' GetFilename(file) '-phi-x.fig", where x=1,2,... Would you like to open these files?'];
   answer = Query(txt,'question',200,['DSGE-VAR - Check ' ModeType],500,'yes',CurrINI);
else;
   answer = 'No';
end;
drawnow;
pause(0.02);
%
% check if we should make new figures
%
if strcmp(lower(answer),'no')==1;
   %
   % create new figures
   %
   oldtab = get(controls.tab1,'UserData');
   maingui = get(controls.tab1,'Parent');
   if DSGEModel.ShowEstimationLog==1;
      set(controls.about,'UserData',0);
      set(maingui,'CurrentObject',controls.tab6);
      YADAGUI('tab');
      if MatlabNumber>=7;
         uicontrol(controls.tab6);
      end;
      set(controls.about,'UserData',1);
   end;
   %
   % check if estimation log is empty
   %
   LogText = get(controls.estimation_log,'String');
   if isempty(LogText)==0;
      SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
   end;
   %
   % we need to load data from mat-file, and run the check optimum functions
   %
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\' FileAddOn 'PosteriorMode-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
   %
   % Retrieving data from ModeFile
   %
   ErrorStr = '';
   if MatlabNumber>=8.0;
      try;
         ModeData = load(ModeFile);
      catch ME;
         ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
      end;
   else;
      try;
         ModeData = load(ModeFile);
      catch;
         ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
      end;
   end;
   if isempty(ErrorStr)==1;
      %
      % copy files that we need to have on the path
      %
      [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
      [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.InitializeParameterFile)==1;
         [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.SystemPriorFile)==1;
         [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
      end;
      %
      % check if we should copy model solver file
      %
      if DSGEModel.ModelSolver==4;
         if FileExist(DSGEModel.ModelSolverFile)==1;
            [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
         end;
      end;
      %
      % and the AIMData file
      %
      if MatlabNumber>=8.0;
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch ME;
            ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
         end;
      else;
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch;
            ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
         end;
      end;
      if isempty(ErrorStr)==1;
         if DSGEModel.UseOwnInitialState==1;
            if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
               DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
            else;
               DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
            end;
         else;
            DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
         end;
         %
         % Set up the sub-sample
         %
         [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
         %
         SetLogText(strvcat(' ','Checking the optimum...',' * Transformed parameters'));
         %
         NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
         NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
         %
         % check if the graphs directory exists or not
         %
         GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
         stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
         if stat==1;
            MinMaxGrid = zeros(length(ModeData.phiMode),2);
            MinMaxGrid(:,1) = ModeData.phiMode-(DSGEModel.GridWidth*sqrt(diag(ModeData.InverseHessian)));
            MinMaxGrid(:,2) = ModeData.phiMode+(DSGEModel.GridWidth*sqrt(diag(ModeData.InverseHessian)));
            %
            % try to load data from the mat-file.
            %
            n = ModeData.n;
            p = DSGEModel.BVARLags;
            k = DSGEModel.k;
            npk = (n*p)+k;
            if isfield(ModeData,'GammaHatyy')==1;
               T = ModeData.T;
               GammaHatyy = ModeData.GammaHatyy;
               GammaHatyY = ModeData.GammaHatyY;
               GammaHatYY = ModeData.GammaHatYY;
               DetProductMoments = ModeData.DetProductMoments;
               HSample = ModeData.HSample;
               if isfield(ModeData,'logGPR')==1;
                  logGPR = ModeData.logGPR;
               else;
                  logGPR = [];
               end;
            else;
               [y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
               T = size(y,2);
               if k>0;
                  DetProductMoments = CalcDeterministicProductMoments(x,X);
               else;
                  DetProductMoments = [];
               end;
               GammaHatyy = (1/T)*y*y';
               GammaHatyY = (1/T)*y*Y';
               GammaHatYY = (1/T)*Y*Y';
               if (strcmp(lower(FileAddOn),'marginal')==1)&(isinf(lambdai)==0);
                  logGPR = LogGammaProductRatio(n,((1+lambdai)*T)-npk,lambdai*T-npk);
               else;
                  logGPR = [];
               end;
            end;
            %
            % try to run the optimum function
            %
            if strcmp(lower(FileAddOn),'marginal')==1;
               phiFittedStd = VillaniCheckOptimum('logPosteriorPhiDSGEVAR',ModeData.phiMode,[],MinMaxGrid,ModeData.InverseHessian,NumGridPoints,0,[],ModeData.ParameterNames.estimated,'Checking Curvature at Marginal Posterior Mode - Transformed Parameters',CurrINI,GraphDirectory,'phi',['DSGEVAR-' num2str(lambdai,'%0.4f') '-'],'Marginal ',1,1,0, ...
                        ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,1);
               if DSGEModel.CheckTransformedOptimum==0;
                  SetLogText(' * Original parameters');
                  if isfield(ModeData,'ParameterBounds')==1;
                     ParameterBounds = ModeData.ParameterBounds;
                  else;
                     ParameterBounds = CreateBounds(ModeData.thetaDist,ModeData.LowerBound,ModeData.UniformBounds);
                     ParameterBounds = ParameterBounds(:,1:2);
                  end;
                  MinMaxGrid(:,1) = PhiToTheta(MinMaxGrid(:,1),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound);
                  MinMaxGrid(:,2) = PhiToTheta(MinMaxGrid(:,2),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound);
                  ThetaPhiPartial = PartialThetaPartialPhi(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
                  thetaFittedStd = VillaniCheckOptimum('logPosteriorThetaDSGEVAR',ModeData.thetaMode,[],MinMaxGrid,ThetaPhiPartial*ModeData.InverseHessian*ThetaPhiPartial,NumGridPoints,0,[],ModeData.ParameterNames.estimated,'Checking Curvature at Marginal Posterior Mode - Original Parameters',CurrINI,GraphDirectory,'theta',['DSGEVAR-' num2str(lambdai,'%0.4f') '-'],'Marginal ',1,1,0, ...
                           ModeData.thetaIndex,ParameterBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,1);
               else;
                  thetaFittedStd = NaN;
               end;
            else;
               phiFittedStd = VillaniCheckOptimum('logJointPosteriorPhiDSGEVAR',ModeData.phiMode,[vec(ModeData.PhiMode);vech(ModeData.SigmaModeEpsilon)],MinMaxGrid,ModeData.InverseHessian,NumGridPoints,0,[],ModeData.ParameterNames.estimated,'Checking Curvature at Joint Posterior Mode - Transformed Parameters',CurrINI,GraphDirectory,'phi',['DSGEVAR-' num2str(lambdai,'%0.4f') '-'],'Joint ',1,1,0, ...
                        ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,CurrINI.OrderQZ);
               if DSGEModel.CheckTransformedOptimum==0;
                  SetLogText(' * Original parameters');
                  if isfield(ModeData,'ParameterBounds')==1;
                     ParameterBounds = ModeData.ParameterBounds;
                  else;
                     ParameterBounds = CreateBounds(ModeData.thetaDist,ModeData.LowerBound,ModeData.UniformBounds);
                     ParameterBounds = ParameterBounds(:,1:2);
                  end;
                  MinMaxGrid(:,1) = PhiToTheta(MinMaxGrid(:,1),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound);
                  MinMaxGrid(:,2) = PhiToTheta(MinMaxGrid(:,2),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound);
                  ThetaPhiPartial = PartialThetaPartialPhi(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
                  thetaFittedStd = VillaniCheckOptimum('logJointPosteriorThetaDSGEVAR',ModeData.thetaMode,[vec(ModeData.PhiMode);vech(ModeData.SigmaModeEpsilon)],MinMaxGrid,ThetaPhiPartial*ModeData.InverseHessian*ThetaPhiPartial,NumGridPoints,0,[],ModeData.ParameterNames.estimated,'Checking Curvature at Joint Posterior Mode - Original Parameters',CurrINI,GraphDirectory,'theta',['DSGEVAR-' num2str(lambdai,'%0.4f') '-'],'Joint ',1,1,0, ...
                           ModeData.thetaIndex,ParameterBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,CurrINI.OrderQZ);
               else;
                  thetaFittedStd = NaN;
               end;
            end;
            %
            % save to mat-file
            %
            if isfield(ModeData,'GammaHatyy')==0;
               %
               % update the posterior mode file
               %
               if strcmp(lower(FileAddOn),'marginal')==1;
                  save(ModeFile,'T','GammaHatyy','GammaHatyY','GammaHatYY','DetProductMoments','HSample','logGPR','-APPEND');
               else;
                  save(ModeFile,'T','GammaHatyy','GammaHatyY','GammaHatYY','DetProductMoments','HSample','-APPEND');
               end;
            end;
            save(ModeFile,'phiFittedStd','thetaFittedStd','-APPEND');
            SetLogText('All done!');
            return;
         else;
            txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\graphs". Check ' lower(ModeType) ' has therefore been aborted.'];
            About(txt,'information','YADA - Directory Creation Problem',300,600,CurrINI);
            drawnow;
            pause(0.02);
            return;
         end;
      else;
         About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         drawnow;
         pause(0.02);
         return;
      end;
   else;
      About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
      drawnow;
      pause(0.02);
      return;
   end;
end;
%
% try to open the figures
%
if (FileExist(FixFilePath([GraphDirectory '\' FileAddOn 'PosteriorModeOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-phi.fig']))==1)&(strcmp(lower(answer),'yes')==1);
   %
   % Files of the optimum already exists. Open them
   %
   ErrorStr = '';
   if MatlabNumber>=8.0;
      try;
         open(FixFilePath([GraphDirectory '\' FileAddOn 'PosteriorModeOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-phi.fig']));
      catch ME;
         ErrorStr = ['Unable to load the file "' FileAddOn 'PosteriorModeOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-phi.fig' '". Message caught is: ' YADALastError(ME)];
      end;
   else;
      try;
         open(FixFilePath([GraphDirectory '\' FileAddOn 'PosteriorModeOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-phi.fig']));
      catch;
         ErrorStr = ['Unable to load the file "' FileAddOn 'PosteriorModeOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-phi.fig' '". Message caught is: ' YADALastError];
      end;
   end;
   if isempty(ErrorStr)==1;
      set(gcf,'Visible','off');
      SetToolbarImages(gcf,CurrINI.images);
      set(gcf,'Visible','on');
      drawnow;
      pause(0.02);
      if FileExist(FixFilePath([GraphDirectory '\' FileAddOn 'CompLikePostOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-phi.fig']))==1;
         open(FixFilePath([GraphDirectory '\' FileAddOn 'CompLikePostOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-phi.fig']));
         set(gcf,'Visible','off');
         SetToolbarImages(gcf,CurrINI.images);
         set(gcf,'Visible','on');
         drawnow;
         pause(0.02);
      end;
      if DSGEModel.CheckTransformedOptimum==0;
         if FileExist(FixFilePath([GraphDirectory '\' FileAddOn 'PosteriorModeOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-theta.fig']))==1;
            open(FixFilePath([GraphDirectory '\' FileAddOn 'PosteriorModeOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-theta.fig']));
            set(gcf,'Visible','off');
            SetToolbarImages(gcf,CurrINI.images);
            set(gcf,'Visible','on');
            drawnow;
            pause(0.02);
         end;
         if FileExist(FixFilePath([GraphDirectory '\' FileAddOn 'CompLikePostOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-theta.fig']))==1;
            open(FixFilePath([GraphDirectory '\' FileAddOn 'CompLikePostOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-theta.fig']));
            set(gcf,'Visible','off');
            SetToolbarImages(gcf,CurrINI.images);
            set(gcf,'Visible','on');
            drawnow;
            pause(0.02);
         end;
      end;
   else;
      About(ErrorStr,'information','YADA - Bad FIG File',500,600,CurrINI);
   end;
elseif FileExist(FixFilePath([GraphDirectory '\' FileAddOn 'PosteriorModeOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-phi-1.fig']))==1;
   %
   % There are multiple files to open
   %
   done = 0;
   num = 1;
   ErrorStr = '';
   while done==0;
      %
      % we close the loop once a figure doesn't exist
      %
      file = FixFilePath([GraphDirectory '\' FileAddOn 'PosteriorModeOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-phi-' int2str(num) '.fig']);
      if FileExist(file)==1;
         if MatlabNumber>=8.0;
            try;
               open(file);
            catch ME;
               ErrorStr = ['Unable to load the file "' GetFilename(file) '". Message caught is: ' YADALastError(ME)];
               done = 1;
            end;
         else;
            try;
               open(file);
            catch;
               ErrorStr = ['Unable to load the file "' GetFilename(file) '". Message caught is: ' YADALastError];
               done = 1;
            end;
         end;
         if isempty(ErrorStr)==1;
            set(gcf,'Visible','off');
            SetToolbarImages(gcf,CurrINI.images);
            set(gcf,'Visible','on');
            drawnow;
            pause(0.02);
            %
            NextFile = FixFilePath([GraphDirectory '\' FileAddOn 'CompLikePostOptimum-DSGEVAR-' num2str(lambdai,'%0.4f') '-phi-' int2str(num) '.fig']);
            if FileExist(NextFile)==1;
               open(NextFile);
               set(gcf,'Visible','off');
               SetToolbarImages(gcf,CurrINI.images);
               set(gcf,'Visible','on');
               drawnow;
               pause(0.02);
            end;
            if DSGEModel.CheckTransformedOptimum==0;
               file = strrep(file,'phi','theta');
               if FileExist(file)==1;
                  open(file);
                  set(gcf,'Visible','off');
                  SetToolbarImages(gcf,CurrINI.images);
                  set(gcf,'Visible','on');
                  drawnow;
                  pause(0.02);
               else;
                  done = 1;
               end;
               %
               if done==0;
                  NextFile = strrep(NextFile,'phi','theta');
                  if FileExist(NextFile)==1;
                     open(NextFile);
                     set(gcf,'Visible','off');
                     SetToolbarImages(gcf,CurrINI.images);
                     set(gcf,'Visible','on');
                     drawnow;
                     pause(0.02);
                  else;
                     done = 1;
                  end;
               end;
            end;
            num = num+1;
         else;
            About(ErrorStr,'information','YADA - Bad FIG File',500,600,CurrINI);
            done = 1;
         end;
      else;
         done = 1;
      end;
   end;
end;

%
% end of DSGEVARCheckPosteriorMode.m
%
