function [SOmega,Omega,Quantiles] = DSGEVARCoherenceDistribution(UsedDraws,lambda,PME,CurrChain,IsPosterior,Weights,DSGEModel,CurrINI,controls)
% DSGEVARCoherenceDistribution: Computes the marginal distributions of the coherence statistics for the observed
%                               variables based on parameter draws from the prior or the posterior distribution.
%                               of a DSGE-VAR model.
%
% USAGE:
%
%       [SOmega,Omega,Quantiles] = DSGEVARCoherenceDistribution(UsedDraws,lambda,PME,CurrChain,IsPosterior,Weights,...
%                                                               DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  UsedDraws (integer) with the number of parameter draws used for the predictions.
%
%                  lambda (scalar)  with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                      are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                      used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                      when generating the posterior draws.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                      are used and 0 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension UsedDraws x 1 with the SMC based weights. Is empty for
%                      all MCMC samplers.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                      model, such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                      main dialog.
%
% REQUIRED OUTPUT: SOmega (matrix) of dimension m(m-1)/2 x f with the m(m-1)/2 unique coherence statistics for
%                          f frequencies from 0 to pi of the m variables.
%
%                  Omega (vector) of dimension f with the frequencies from 0 to pi.
%
%                  Quantiles (vector structure) with fields "SOmega" and "percent".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 22, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 23-06-2010: Updated the documentation.
%
% * 22-10-2010: The wait dialog now display the number of lags of the DSGE-VAR.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 28-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 04-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 11-09-2017: Added the input variable "Weights".
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if isempty(Weights)==1;
   Weights = ones(UsedDraws,1);
end;
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% initialize index for saving
%
SaveAfterDraws = min(UsedDraws,1000);
NumFiles = ceil(UsedDraws/SaveAfterDraws);
p = DSGEModel.BVARLags;
n = DSGEModel.n;
NumQuants = length(DSGEModel.Percentiles);
%
%
% load the first file to determine Omega
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGEVAR-Coherence-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-1-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGEVAR-Coherence-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-1-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
end;
%
CoherenceData = load(file);
Omega = CoherenceData.Omega;
f = length(Omega);
%
% intialize output
%
SOmega = zeros(n*(n-1)/2,f);
for i=1:NumQuants;
   Quantiles(i).SOmega = zeros(n*(n-1)/2,f);
   Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
Weightsf = Weights*ones(1,f);
%
% set up a wait dialog
%
if IsPosterior==1;
   txt = ['Please wait while YADA computes the posterior mean coherence statistics as well as percentile values for the observed variables based on a DSGE-VAR(' num2str(lambda,'%0.4f') ') model with ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current variable pair: (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(DSGEModel.VariableNames(2,:)) ')'];
else;
   txt = ['Please wait while YADA computes the prior mean coherence statistics as well as percentile values for the observed variables based on a DSGE-VAR(' num2str(lambda,'%0.4f') ') model with ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current variable pair: (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(DSGEModel.VariableNames(2,:)) ')'];
end;
WaitHandle = WaitDLG(txt,'information',['Coherence Distribution' PriorHeader],500,300,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
num = 0;
for i=1:n-1;
   for j=i+1:n;
      num = num+1;
      if IsPosterior==1;
         txt = ['Please wait while YADA computes the posterior mean coherence statistics as well as percentile values for the observed variables based on a DSGE-VAR(' num2str(lambda,'%0.4f') ') model with ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current variable pair: (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(DSGEModel.VariableNames(j,:)) ')'];
      else;
         txt = ['Please wait while YADA computes the prior mean coherence statistics as well as percentile values for the observed variables based on a DSGE-VAR(' num2str(lambda,'%0.4f') ') model with ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current variable pair: (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(DSGEModel.VariableNames(j,:)) ')'];
      end;
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         TempVar = zeros(UsedDraws,f);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGEVAR-Coherence-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGEVAR-Coherence-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
            end;
            CoherenceData = load(file);
            Coherence = CoherenceData.Coherence;
            NumSaves = length(Coherence);
            for s=1:NumSaves;
               TempVar(CurrSaves+s,:) = Coherence(s).YCoherence(num,:);
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % compute the mean
         %
         SOmega(num,:) = (1/UsedDraws)*sum(TempVar.*Weightsf,1);
         %
         % sort each column
         %
         TempVar = sort(TempVar,1);
         %
         if DSGEModel.ConfidenceRegionMethod==1;
            for k=1:NumQuants;
               if DSGEModel.Percentiles(k)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(k)/100)*UsedDraws);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(k)/100)*UsedDraws);
               end;
               Quantiles(k).SOmega(num,:) = TempVar(QuantVal,:);
            end;
         else;
            for k=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-k+1)-DSGEModel.Percentiles(k))*UsedDraws/100);
               for t=1:f;
                  Bounds = HighestProbabilityDensityBand(TempVar(:,t),ConfDraws);
                  Quantiles(k).SOmega(num,t) = Bounds(1);
                  Quantiles(NumQuants-k+1).SOmega(num,t) = Bounds(2);
               end;
            end;
         end;
      catch
         for t=1:f;
            TempVar = zeros(UsedDraws,1);
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGEVAR-Coherence-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\coherence\DSGEVAR-Coherence-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
               end;
               CoherenceData = load(file);
               Coherence = CoherenceData.Coherence;
               NumSaves = length(Coherence);
               for s=1:NumSaves;
                  TempVar(CurrSaves+s,1) = Coherence(s).YCoherence(num,t);
               end;
               CurrSaves = CurrSaves+NumSaves;
               drawnow;
            end;
            %
            % compute the mean
            %
            SOmega(num,t) = (1/UsedDraws)*sum(TempVar.*Weights,1);
            %
            % sort each column
            %
            TempVar = sort(TempVar,1);
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for k=1:NumQuants;
                  if DSGEModel.Percentiles(k)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(k)/100)*UsedDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(k)/100)*UsedDraws);
                  end;
                  Quantiles(k).SOmega(num,t) = TempVar(QuantVal,1);
               end;
            else;
               for k=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-k+1)-DSGEModel.Percentiles(k))*UsedDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
                  Quantiles(k).SOmega(num,t) = Bounds(1);
                  Quantiles(NumQuants-k+1).SOmega(num,t) = Bounds(2);
               end;
            end;
         end;
      end;
      drawnow;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEVARCoherenceDistribution.m
%
