function CondCorr = DSGEVARConditionalCorrsParam(Phi,A0,lambda,ShockNames,NumPaths,EstStr,DSGEModel,CurrINI)
% DSGEVARConditionalCorrsParam: Computes the conditional correlations for the observed variables
%                               with a DSGE-VAR model for fixed parameter values. Both sample- and
%                               population-based correlations are supported.
%
% USAGE:
%
%       CondCorr = DSGEVARConditionalCorrsParam(Phi,A0,lambda,ShockNames,NumPaths,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nx(np+k) with parameters on exogenous variable and
%                      lagged endogenous variables.
%
%                  A0 (matrix) of dimension nxn with parameters on the structural shocks.
%
%                  lambda (scalar) with the lambda hyperparameter of the DSGE-VAR.
%
%                  ShockNames (string matrix) with the names of the structural shocks.
%
%                  NumPaths (integer) with the number of sample paths to generate; 0 means that
%                      population-based correlations are calculated, while a positive value
%                      means that sample-based correlations are computed.
%
%                  EstStr (string) that takes the values "Initial Values", "Marginal Posterior mode", or
%                      "Joint Posterior Mode".
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                      input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: CondCorr (structure) with fields Mean and Quantiles and ShockNames.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 20, 2011.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-01-2011: Updated the documentation.
%
% * 01-03-2011: Changed the calculation of the square-root matrix of "SigmaZ".
%
% * 03-01-2012: Updated the documentation.
%
% * 19-01-2012: Added try-catch code around the call to "DoublingAlgorithmLyapunov".
%
% * 29-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
CondCorr = [];
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
CondCorr(1).ShockNames = ShockNames;
%
% remove parameters on the exogenous variables
%
if k>0;
   Phi0 = Phi(:,1:k);
else;
   Phi0 = zeros(n,1);
end;
Phi = Phi(:,k+1:(n*p)+k);
%
% setup the parameter matriuces
%
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
% check if population or sample based correlations should be computed
%
if NumPaths==0;
   %
   % population-based correlations
   %
   for j=1:n;
      %
      % compute the conditional state population covariance matrix when there is
      % only shock i using the doubling algorithm
      %
      try;
         [SigmaZj,stat] = DoublingAlgorithmLyapunov(Phi,J*A0(:,j)*A0(:,j)'*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      catch;
         vSigmaZj = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(A0(:,j)*A0(:,j)');
         SigmaZj = reshape(vSigmaZj,n*p,n*p);
      end;
      SigmaYj = SigmaZj(1:n,1:n);
      CorrVec = sqrt(abs(diag(SigmaYj)));
      CorrVec(CorrVec<1.0e-12) = 1;
      CondCorr(j).Mean = SigmaYj./(CorrVec*CorrVec');
   end;
else;
   %
   % initialize the random number generator
   %
   if DSGEModel.RandomNumberValue==1;
      randn('state',0);
      rand('state',0);
   else;
      randn('state',sum(100*clock));
      rand('state',sum(100*clock));
   end;
   %
   Dn = DuplicationMatrix(n);
   abort = '';
   MeanEstimationTime = 0;
   StopEstimation = 0;
   %
   % get the data
   %
   [y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
   T = size(y,2);
   %
   txt = ['Please wait while YADA computes the distributions for the sample-based conditional correlations of the observed variables using a DSGE-VAR model with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Current shock: ' StringTrim(ShockNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',['Conditional Correlations - ' EstStr],500,200,CurrINI,1);
   WaitControls = get(WaitHandle,'UserData');
   %
   % sample-base correlations
   %
   for j=1:n;
      %
      % simulate data with only shock j
      %
      if isempty(WaitControls.cancel)==0;
         %
         % check if the cancel button was pressed
         %
         if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
            StopEstimation = 1;
            break;
         end;
      end;
      %
      txt = ['Please wait while YADA computes the distributions for the sample-based conditional correlations of the observed variables using a DSGE-VAR model with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Current shock: ' StringTrim(ShockNames(j,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      % initial values for Y
      %
      A0j = A0(:,j);
      try;
         [SigmaZ,stat] = DoublingAlgorithmLyapunov(Phi,J*A0j*A0j'*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      catch;
         vSigmaZ = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(A0j*A0j');
         SigmaZ = reshape(vSigmaZ,n*p,n*p);
      end;
      [SE,LE] = schur(SigmaZ);
      CholSigmaZ = SE*sqrt(abs(LE));
      if k==0;
         muY = CholSigmaZ*randn(n*p,NumPaths);
      elseif (k==1)&(max(X)==min(X));
         muY = (kron(ones(p,1),J'*((eye(n*p)-Phi)\J)*Phi0)*ones(1,NumPaths))+(CholSigmaZ*randn(n*p,NumPaths));
      else;
         muY = Y(:,1)*ones(1,NumPaths);
      end;
      %
      % intialize the mean value and the draws
      %
      CondCorr(j).Mean = zeros(n,n);
      QuantileData = zeros(NumPaths,n*(n+1)/2);
      for i=1:NumPaths;
         %
         % simulate the data
         %
         Epsilon = A0j*randn(1,T);
         ymat = zeros(n,T);
         Y = muY(:,i);
         if k>0;
            for t=1:T;
               Y = (J*Phi0*x(:,t))+(Phi*Y)+(J*Epsilon(:,t));
               ymat(:,t) = J'*Y;
            end;
         else;
            for t=1:T;
               Y = (Phi*Y)+(J*Epsilon(:,t));
               ymat(:,t) = J'*Y;
            end;
         end;
         if k>0;
            MeanY = (ymat*x')/(x*x');
            ymat = ymat-(MeanY*x);
         else;
            MeanY = (1/T)*sum(ymat,2);
            ymat = ymat-(MeanY*ones(1,T));
         end;
         SigmaYj = (1/T)*ymat*ymat';
         CorrVec = sqrt(abs(diag(SigmaYj)));
         CorrVec(CorrVec<1.0e-12) = 1;
         CCorr = SigmaYj./(CorrVec*CorrVec');
         CondCorr(j).Mean = CondCorr(j).Mean+((1/NumPaths)*CCorr);
         QuantileData(i,:) = vech(CCorr)';
         drawnow;
      end;
      %
      % sort each row of the draws
      %
      QuantileData = sort(QuantileData);
      %
      % deal with the quantiles
      %
      NumQuants = length(DSGEModel.Percentiles);
      if DSGEModel.ConfidenceRegionMethod==1;
         for i=1:NumQuants;
            %
            % initialize
            %
            CondCorr(j).Quantiles(i).percent = DSGEModel.Percentiles(i);
            if DSGEModel.Percentiles(i)<50;
               QuantVal = ceil((DSGEModel.Percentiles(i)/100)*NumPaths);
            else;
               QuantVal = floor((DSGEModel.Percentiles(i)/100)*NumPaths);
            end;
            CondCorr(j).Quantiles(i).Mean = reshape(Dn*QuantileData(QuantVal,:)',n,n);
         end;
      else;
         for i=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-i+1)-DSGEModel.Percentiles(i))*NumPaths/100);
            HMat = zeros(n*(n+1)/2,2);
            for t=1:n*(n+1)/2;
               HMat(t,:) = HighestProbabilityDensityBand(QuantileData(:,t),ConfDraws);
            end;
            CondCorr(j).Quantiles(i).Mean = reshape(Dn*HMat(:,1),n,n);
            CondCorr(j).Quantiles(NumQuants-i+1).Mean = reshape(Dn*HMat(:,2),n,n);
            CondCorr(j).Quantiles(i).percent = DSGEModel.Percentiles(i);
            CondCorr(j).Quantiles(NumQuants-i+1).percent = DSGEModel.Percentiles(NumQuants-i+1);
         end;
      end;
   end;
   if StopEstimation==1;
      CondCorr = [];
   end;
   %
   % close the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;

%
% end of DSGEVARConditionalCorrsParam.m
%
