function status = DSGEVARCorrelationsExist(UsedDraws,CurrChain,IsPosterior,PME,lambda,NumPaths,DSGEModel,controls)
% DSGEVARCorrelationsExist: Tests if the observed variable correlations have already been calculated and
%                           stored to disk.
%
% USAGE:
%
%       status = DSGEVARCorrelationsExist(UsedDraws,CurrChain,IsPosterior,PME,lambda,NumPaths,DSGEModel,controls)
%
% REQUIRED INPUT:  UsedDraws (integer) with the number of parameter draws used for the correlations.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                              are used and 0 if draws from the prior are supplied.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                              are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                              used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                              when generating the posterior draws.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  NumPaths (integer) with the number of simulated paths per parameter value.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                              model, such as input files, variable and sample data.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                              main dialog.
%
% REQUIRED OUTPUT: status (boolean) that indicates if the correlations have already been calculated or not.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 17, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-06-2010: Added the "NumPaths" input variable.
%
% * 22-06-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 0;
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% initialize index for saving
%
SaveAfterDraws = min(UsedDraws,1000);
if NumPaths==0;
   NumFiles = ceil(UsedDraws/SaveAfterDraws);
else;
   NumFiles = UsedDraws;
end;
p = DSGEModel.BVARLags;
%
if NumPaths==0;
   for ThisSave=1:NumFiles;
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
      end;
      if FileExist(file)==0;
         return;
      end;
   end;
else;
   for ThisSave=1:NumFiles;
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(NumPaths) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGEVAR-CorrY-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumPaths) '.mat']);
      end;
      if FileExist(file)==0;
         return;
      end;
   end;
end;
status = 1;

%
% end of DSGEVARCorrelationsExist.m
%
