function [MeanResp,StdResp,CorrResp] = DSGEVARMomentsResponseTheta(Phi,SigmaEpsilon,ParameterNames,ParamNumber,theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,lambda,DSGEModel,CurrINI)
% DSGEVARMomentsResponseTheta: Computes the numerical partial derivatives of the means, standard deviations, and correlations
%                              for a DSGE-VAR model with respect to a change in one structural parameters.
%
% USAGE:
%
%       [MeanResp,StdResp,CorrResp] = DSGEVARMomentsResponseTheta(Phi,SigmaEpsilon,ParameterNames,ParamNumber,...
%                                                                 theta,thetaPositions,ModelParameters,PriorDist,...
%                                                                 thetaDist,LowerBound,lambda,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nx(np+k) with the parameters on the exogenous and lagged endogenous variables
%                      of the VAR.
%
%                  SigmaEpsilon (matrix) of dimension nxn with the residual covariances of the VAR.
%
%                  ParameterNames (string matrix) with the names of the parameters.
%
%                  ParamNumber (integer) with the position of the parameter in theta to change.
%
%                  theta (vector) with the values of the original DSGE model parameters.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                      of the model parameter in position i of theta. The field "distribution" has a string vector
%                      giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                      DSGE model.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                      "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                      parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                      and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                      it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                      truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                      values are the lower and the upper bound. For the student-t they are the location (mode),
%                      the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                      and scale parameters are needed. For the Pareto the parameter values are for the mode
%                      and the shape parameter.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                      truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                      and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  lambda (scalar) with the value of the lambda hyperparameter.
%
%                  DSGEModel (structure) whose fields contain all required information about the model,
%                      such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: MeanResp (matrix) of dimension nxk with the responses of the endogenous variables from the change
%                      change in one structural parameter.
%
%                  StdResp (vector) of dimension n for observed variables with the responses of the standard deviations
%                      of the variables.
%
%                  CorrResp (vector) of dimension n(n-1)/2 for observed variables with the responses of the correlations
%                      (stacking of elements below the diagonal of the correlation matrix).
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 16, 2019.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-04-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize output
%
MeanResp = [];
StdResp = [];
CorrResp = [];
%
% first we compute the mean and covariance matrix at the original parameter values
%
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
%
% remove parameters on the exogenous variables
%
if k>0;
   Phi0 = Phi(:,1:k);
else;
   Phi0 = zeros(n,1);
end;
Phi = Phi(:,k+1:(n*p)+k);
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
% we approximate the mean by J'*inv(I-Phi)*J*Phi0
%
MeanMat = J'*((eye(n*p)-Phi)\J)*Phi0;
%
% compute the autocovariances for the stacked Z vector
%
try;
   [SigmaZ,stat] = DoublingAlgorithmLyapunov(Phi,J*SigmaEpsilon*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
catch;
   vSigmaZ = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(SigmaEpsilon);
   SigmaZ = reshape(vSigmaZ,n*p,n*p);
end;
CovMat = SigmaZ(1:n,1:n);
StdVec = real(sqrt(diag(CovMat)));
if length(StdVec)>1;
   CorrVec = vechl(CovMat./(StdVec*StdVec'));
end;
%
% now make the parameter change
%
thetaChange = 0.001*abs(theta);
IdentMat = eye(length(theta));
ParamDiff = thetaChange(ParamNumber);
thetaNew = theta+(ParamDiff*IdentMat(:,ParamNumber));
%
% compute the new DSGE-VAR parameters
%
[PhiNew,SigmaEpsilonNew,A0,IdentifyStatus,ModeStatus] = GetDSGEVARParameters(thetaNew,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,lambda,DSGEModel,CurrINI);
if ModeStatus~=1;
   thetaChange = 0.0001*abs(theta);
   IdentMat = eye(length(theta));
   ParamDiff = thetaChange(ParamNumber);
   thetaNew = theta+(ParamDiff*IdentMat(:,ParamNumber));
   [PhiNew,SigmaEpsilonNew,A0,IdentifyStatus,ModeStatus] = GetDSGEVARParameters(thetaNew,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,lambda,DSGEModel,CurrINI);
   if ModeStatus~=1;
      txt = ['YADA was unable to compute the numerical partial derivatives of the first and second moments of the ' VarStr ' variables since the VAR parameter couldn''t be determined from the DSGE model with the tested new values of the "' StringTrim(ParameterNames(ParamNumber,:)) '" parameter.'];
      About(ErrorStr,'information','YADA - Bad Numerical Partial Derivatives',500,600,CurrINI);
      return;
   end;
end;
%
% now we can compute the mean and the covariance using the new parameter values
%
if k>0;
   Phi0New = PhiNew(:,1:k);
else;
   Phi0New = zeros(n,1);
end;
PhiNew = PhiNew(:,k+1:(n*p)+k);
if p>1;
   PhiNew = [PhiNew;eye(n*(p-1)) zeros(n*(p-1),n)];
end;
%
% we approximate the mean by J'*inv(I-Phi)*J*Phi0
%
MeanMatNew = J'*((eye(n*p)-PhiNew)\J)*Phi0New;
if k>0;
   MeanResp = (MeanMatNew-MeanMat)./ParamDiff;
else;
   MeanResp = zeros(n,1);
end;
%
% compute the autocovariances for the stacked Z vector
%
try;
   [SigmaZ,stat] = DoublingAlgorithmLyapunov(PhiNew,J*SigmaEpsilonNew*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
catch;
   vSigmaZ = (eye(n*n*p*p)-kron(PhiNew,PhiNew))\kron(J,J)*vec(SigmaEpsilonNew);
   SigmaZ = reshape(vSigmaZ,n*p,n*p);
end;
CovMatNew = SigmaZ(1:n,1:n);
StdVecNew = real(sqrt(diag(CovMatNew)));
if length(StdVecNew)>1;
   CorrVecNew = vechl(CovMatNew./(StdVecNew*StdVecNew'));
end;
%
% now compute the change in standard deviations and correlations
%
StdResp = (StdVecNew-StdVec)./ParamDiff;
if length(StdVecNew)>1;
   CorrResp = (CorrVecNew-CorrVec)./ParamDiff;
end;

%
% end of DSGEVARMomentsResponseTheta.m
%
