function DSGEVARPriorSampling(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,NumPriorDraws,DSGEModel,CurrINI,controls)
% DSGEVARPriorSampling: Draws values of the parameters to be estimated from the prior
%                       distribution.
%
% USAGE:
%
%       DSGEVARPriorSampling(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,NumPriorDraws,DSGEModel,CurrINI)
%
% REQUIRED INPUT: theta (vector) with the initial values of the original DSGE model parameters.
%
%                 thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                                of the model parameter in position i of theta. The field "distribution" has a string
%                                vector giving the assumed prior distribution of the parameter.
%
%                 ModelParameters (structure) whose fields have names given by the parameter names in the
%                                DSGE model.
%
%                 PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                                "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                                parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                                and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                                it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                                truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                                values are the lower and the upper bound. For the student-t they are the location (mode),
%                                the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                                and scale parameters are needed. For the Pareto the parameter values are for the mode
%                                and the shape parameter.
%
%                 thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                                truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                                and 11 if pareto.
%
%                 ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                                "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated".
%                                Each field returns a string matrix with the parameter names.
%
%                 thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                                0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of
%                                transformation that may need to be performed for the theta parameters.
%
%                 UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                                for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%                 LowerBound (vector) with lower bound values for the theta vector.
%
%                 NumPriorDraws (integer) with the number of draws to take from the prior distribution.
%
%                 DSGEModel (structure) whose fields contain all required information about the model,
%                                such as input files, variable and sample data.
%
%                 CurrINI (structure) with initialization information.
%
%                 controls (structure) whose fields provide handles to all the UI controls in YADA's main dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 18, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-02-2010: Updated the documentation.
%
% * 09-03-2010: Fixed a bug where SigmaEpsilon was set to its mode before sampling from the
%               inverted Wishart.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ"
%               from the CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 12-01-2011: Added additional support for unix and mac paths.
%
% * 14-12-2011: Made sure that "DSGEPriorSampling" received the "HeaderStr" input variable.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 26-06-2017: Began work on supporting the system prior. This means that the input variables ParameterNames,
%               thetaIndex, UniformBounds, and controls were added.
%
% * 13-12-2017: Made sure that a subset of prior draws can be used when the model includes a system prior. In
%               addition, the function will use previously computed system prior draws if the number of such
%               available draws matches the desired number of draws.
%
% * 16-01-2018: Updated the documentation.
%
% * 25-10-2018: Made sure that the name of the prior draws file reflects using a system prior.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-05-2020: Took into account that "DSGEPriorSampling" and "DSGESystemPriorSampling" take the input variable
%               "IsAdaptiveLearning".
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'priordraws');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\priordraws". The DSGE-VAR prior sampling has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/priordraws". The DSGE-VAR prior sampling has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% start by drawing the DSGE model parameters
%
if FileExist(DSGEModel.SystemPriorFile)==1;
   %
   % check if the desired number of prior draws already exist on disk
   %
   IsSystemPrior = 1;
   UsePriorDraws = 0;
   PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
   if FileExist(PriorDrawsFile)==1;
      PriorData = load(PriorDrawsFile);
      thetaDraws = PriorData.thetaDraws;
      if size(thetaDraws,2)==NumPriorDraws;
         UsePriorDraws = 1;
      else;
         clear('thetaDraws');
      end;
   end;
   if UsePriorDraws==0;
      thetaDraws = DSGESystemPriorSampling(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,NumPriorDraws,0,DSGEModel,CurrINI,controls);
   end;
   %
   % check if we should select a subset of the prior draws
   %
   if DSGEModel.PosteriorSampler>7;
      [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','DSGE-VAR Prior Sampling','','',CurrINI);
      if strcmp(lower(action),'ok')==1;
         DSGEModel.PostDrawsPercentValue = SelectedOption;
      end;
   else;
      action = 'ok';
   end;
   if strcmp(lower(action),'ok')==1;
      UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
      if length(UseDraws)<NumPriorDraws;
         thetaDraws = thetaDraws(:,UseDraws);
         NumPriorDraws = length(UseDraws);
      end;
   end;
else;
   thetaDraws = DSGEPriorSampling(PriorDist,thetaDist,LowerBound,NumPriorDraws,0,'DSGE-VAR Prior Sampling',DSGEModel,CurrINI);
   IsSystemPrior = 0;
end;
%
% compute the prior parameters at theta
%
[PhiModeInitVal,SigmaEpsilonInitVal,A,HInitVal,R,F,B0InitVal,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
if Status==0;
   return;
end;
%
% initialize dimensions
%
p = DSGEModel.BVARLags;
k = DSGEModel.k;
n = DSGEModel.n;
npk = (n*p)+k;
%
% get the lambda values we can check
%
[lambda,T] = GetLambda(DSGEModel);
%
% check if we can identify the structural shocks
%
[A0InitVal,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilonInitVal,HInitVal,B0InitVal,DSGEModel.DSGEVARShocks,HSample);
%
% open a wait dialog
%
OriginalTime = StringTrim(datestr(now,14));
txt = ['Please wait while YADA draws ' int2str(NumPriorDraws) ' values from the prior distribution of the DSGE-VAR parameters. Current value of lambda is ' num2str(lambda(1),'%0.4f') '. Sampling first started at: ' OriginalTime '. Sampling for current lambda value started at: ' StringTrim(datestr(now,14)) '.'];
WaitHandle = WaitDLG(txt,'information',['DSGE-VAR Prior Sampling - ' int2str(NumPriorDraws) ' Draws'],500,300,CurrINI,1);
WaitControls = get(WaitHandle,'UserData');
drawnow;
%
%
% determine string with 1's and 0's for the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
StopEstimation = 0;
for i=1:length(lambda);
   lambdai = lambda(i);
   %
   % update the wait dialog
   %
   txt = ['Please wait while YADA draws ' int2str(NumPriorDraws) ' values from the prior distribution of the DSGE-VAR parameters. Current value of lambda is ' num2str(lambdai,'%0.4f') '. Sampling first started at: ' OriginalTime '. Sampling for current lambda value started at: ' StringTrim(datestr(now,14)) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   % initialize the output
   %
   PriorDraws = [];
   for j=1:NumPriorDraws;
      %
      % check if the cancel button was pressed
      %
      if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
         StopEstimation = 1;
         break;
      end;
      %
      % try to solve the DSGE model at the current thetaj
      %
      thetaj = thetaDraws(:,j);
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaj,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status==1;
         [Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
         if PriorStatus==1;
            Phi = GammayY/GammaYY;
            SigmaEpsilon = Gammayy-(Phi*GammayY');
         else;
            Phi = PhiModeInitVal;
            SigmaEpsilon = SigmaEpsilonInitVal;
            H = HInitVal;
            B0 = B0InitVal;
         end;
      else;
         Phi = PhiModeInitVal;
         SigmaEpsilon = SigmaEpsilonInitVal;
         H = HInitVal;
         B0 = B0InitVal;
      end;
      %
      % compute the SigmaEpsilon and Phi
      %
      if isinf(lambdai)==0;
         InvGammaYY = GammaYY\eye(npk);
         %
         % draw a value of SigmaEpsilon from the inverted Wishart
         %
         SigmaEpsilon = InvWishartRndFcn(lambdai*T*SigmaEpsilon,(lambdai*T)-npk);
         Phi = reshape(MultiNormalRndFcn(vec(Phi),(1/(lambdai*T))*kron(InvGammaYY,SigmaEpsilon),1),n,npk);
      end;
      %
      % check if we can compute A0
      %
      if IdentifyStatus==1;
         [A0,IdStat] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
      else;
         A0 = [];
      end;
      PriorDraws(j).Phi = Phi;
      PriorDraws(j).SigmaEpsilon = SigmaEpsilon;
      PriorDraws(j).A0 = A0;
   end;
   if StopEstimation==1;
      break;
   end;
   %
   % save draws to file
   %
   if IsSystemPrior==1;
      MatFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
   else;
      MatFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-DSGEVAR-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
   end;
   save(MatFile,'PriorDraws','thetaDraws','lambdai','T','p');
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
txt = ['YADA has created ' int2str(NumPriorDraws) ' draws from the prior distributions of the DSGE-VAR parameters for the different lambda values. The draws are stored in the directory "' GetPath(MatFile) '" in mat-files with names "PriorDraws-DSGEVAR-x.xxxx-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat", where x.xxxx is replaced with a lambda value.'];
About(txt,'information','DSGE-VAR Prior Sampling',300,600,CurrINI);

%
% end of DSGEVARPriorSampling.m
%
