function DSGEVARShockGraphDLG(selector,DSGEModel,CurrINI,ShockNames,Shocks,Quantiles,DateVector,SampleStrMat,NameStr)
% DSGEVARShockGraphDLG: Displays graphs of state or measurement error shocks
%
% USAGE:
%
%       DSGEVARShockGraphDLG(selector,DSGEModel,CurrINI,ShockNames,Shocks,Quantiles,DateVector,SampleStrMat,NameStr)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: DSGEModel (structure) whose fields contain all required information about the
%                           model, such as input files, variable and sample data. Actual observations
%                           are not stored in this file.
%
%                 CurrINI (structure), with initialization information.
%
%                 ShockNames (string matrix) with the names of the structural shocks.
%
%                 Shocks (matrix) with the estimated structural shocks.
%
%                 Quantiles (structure) with quantile data.
%
%                 DateVector (vector) with numeric values for the dates.
%
%                 SampleStrMat (string matrix) with all the sample dates.
%
%                 NameStr (string vector) taking on the values "Posterior Mode" or "Initial Values".
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 25, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-02-2010: Updated the documentation.
%
% * 27-04-2010: Added the "DSGEModel" and "Quantiles" input variables.
%
% * 25-05-2010: Corrected a text issue on the dialog.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 23-03-2018: Fixed an issue to make legend compatible with matlab 2017a and later.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   EstData.Shocks = Shocks;
   EstData.Quantiles = Quantiles;
   EstData.DateVector = DateVector;
   EstData.NameStr = NameStr;
   EstData.DSGEModel = DSGEModel;
else;
   ShockGraphGUI = findobj('Type','figure','Tag','DSGEVARShockGraphDLG');
   Controls = get(ShockGraphGUI,'UserData');
   CurrINI = get(Controls.variablespopup,'UserData');
   EstData = get(Controls.variables,'UserData');
end;
%
switch selector;
   case 'init';
      ShockGraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','DSGEVARShockGraphDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-120)/2 500 120], ...
         'Tag','DSGEVARShockGraphDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['DSGE-VAR Structural Shocks - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','DSGEVARShockGraphDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(ShockGraphGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 498 110],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Sample selection controls
      %
      Controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      Controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[185 55 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','DSGEVARShockGraphDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      Controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[278 55 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','DSGEVARShockGraphDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SampleStrMat,1));
      %
      % Select variable
      %
      Controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 160 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',EstData, ...
         'String','Select shock:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariablesText');
      %
      Controls.variablespopup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[185 20 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',ShockNames, ...
         'HorizontalAlignment','center', ...
         'Tag','VariablePopup',...
         'Value',1);
      %
      % Create graph button
      %
      Controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','DSGEVARShockGraphDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      Controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','DSGEVARShockGraphDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(ShockGraphGUI,'UserData',Controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(Controls.samplebeg,'Value');
      sampleend = get(Controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(Controls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(Controls.samplebeg,'Value');
      sampleend = get(Controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(Controls.samplebeg,'Value',sampleend);
      end;
   case 'showgraph'
      VarValue = get(Controls.variablespopup,'Value');
      VarString = get(Controls.variablespopup,'String');
      CurrVariable = StringTrim(VarString(VarValue,:));
      if VarValue<size(VarString,1);
         set(Controls.variablespopup,'Value',VarValue+1);
      else;
         set(Controls.variablespopup,'Value',1);
      end;
      %
      % get the selected sample
      %
      StartSample = get(Controls.samplebeg,'Value');
      EndSample = get(Controls.sampleend,'Value');
      Quantiles = EstData.Quantiles;
      DSGEModel = EstData.DSGEModel;
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                         'Tag',CurrVariable, ...
                         'Name',[EstData.NameStr ' Estimates - ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % check if we have quantiles
      %
      if isempty(Quantiles)==0;
         set(FigHandle,'Name',[EstData.NameStr ' - ' CurrVariable])
         NumBounds = floor(length(Quantiles)/2);
         Handles = [];
         LegendStr = '';
         for nb=1:NumBounds;
            Lower = Quantiles(nb).u(VarValue,StartSample:EndSample);
            Upper = Quantiles(length(DSGEModel.Percentiles)-nb+1).u(VarValue,StartSample:EndSample);
            PatchColor = ((1+(2*(nb-1)))/length(DSGEModel.Percentiles))*DSGEModel.ConfidenceBandBaseColor;
            ConfBound = Quantiles(length(DSGEModel.Percentiles)-nb+1).percent-Quantiles(nb).percent;
            ConfPatch = patch([EstData.DateVector(StartSample:EndSample) fliplr(EstData.DateVector(StartSample:EndSample))],[Upper fliplr(Lower)],PatchColor);
            set(ConfPatch,'FaceColor',PatchColor,'EdgeColor',PatchColor);
            hold('on');
            Handles = [Handles ConfPatch];
            LegendStr = strvcat(LegendStr,[num2str(ConfBound,'%0.0f') ' confidence']);
         end;
         set(gca,'Layer','Top');
      end;
      %
      % plot a zero line
      %
      ZeroHandle = plot([EstData.DateVector(StartSample) EstData.DateVector(EndSample)],[0 0],'-','Color',[0 0 0]);
      hold('on');
      %
      % plot the selected shock
      %
      if isempty(Quantiles)==1;
         LineHandle = plot([EstData.DateVector(StartSample:EndSample)],EstData.Shocks(VarValue,StartSample:EndSample),'.-','LineWidth',2,'Color',[1 0 0]);
      else;
         LineHandle = plot([EstData.DateVector(StartSample:EndSample)],EstData.Shocks(VarValue,StartSample:EndSample),'.-','LineWidth',2,'Color',[0.75 0 0]);
      end;
      hold('on');
      %
      set(gca,'XLim',[EstData.DateVector(StartSample) EstData.DateVector(EndSample)]);
      %
      if isempty(Quantiles)==1;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' estimates at ' lower(EstData.NameStr)]);
      else;
         SetAxesFonts(CurrINI,'Title',CurrVariable);
      end;
      %
      % check if we should add a legend
      %
      if isempty(Quantiles)==0;
         if MatlabNumber>=7;
            legHandle = legend([LineHandle Handles],strvcat('Mean',LegendStr),'Location','NorthWest');
         else;
            legHandle = legend([LineHandle Handles],strvcat('Mean',LegendStr),2);
         end;
         SetAxesFonts(CurrINI,legHandle);
      end;
      %
      SetXTickIntegers(gca);
      %
      % Fix the toolbar
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      %
      % display figures
      %
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(Controls.done,'UserData','done');
      delete(ShockGraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(ShockGraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(ShockGraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(Controls.done);
   end;
   waitfor(Controls.done,'UserData','done');
end;

%
% end of DSGEVARShockGraphDLG.m
%
