function DSGEVARSpectralDecompDLG(selector,Omega,SyOmega,OriginalVariance,SyOmegaAnnual,AnnualVariance,lambda,ShockNames,ShockGroups,EstStr,DSGEModel,CurrINI)
% DSGEVARSpectralDecompDLG: Displays graphs of spectral decompositions for the observed variables when estimated via a DSGE-VAR.
%
% USAGE:
%
%       DSGEVARSpectralDecompDLG(selector,Omega,SyOmega,OriginalVariance,SyOmegaAnnual,AnnualVariance,lambda,...
%                                ShockNames,ShockGroups,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT: selector (string) (string vector) taking on the values "init", "lowerperiod", "upperperiod",
%                          "showgraph", and "done".
%
% OPTIONAL INPUT: Omega (vector) of dimension f with the frequencies from 0 to pi.
%
%                 SyOmega (cell array) of dimension nxf with nxn matrices with the conditional
%                          spectral density for the f frequencies and the n structural shocks.
%
%                 OriginalVariance (vector) with the DSGE-VAR model based population variances for
%                          the endogenous variables.
%
%                 SyOmegaAnnual (cell array) of dimension nxf with nxn matrices with the annualized
%                          conditional spectral density for the f frequencies and the n structural
%                          shocks.
%
%                 AnnualVariance (vector) with the DSGE-VAR model based population variances for
%                          the annualized endogenous variables.
%
%                 lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                 ShockNames (string matrix) with the names of the structural shocks of the DSGE-VAR.
%
%                 ShockGroups (vector) with integer values for the different shock groups.
%
%                 EstStr (string) that takes the value 'Initial Values', 'Marginal Posterior Mode',
%                          or 'Joint Posterior Mode'.
%
%                 DSGEModel (structure) whose fields contain all required information about the model, such as
%                          input files, variable and sample data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 31, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 01-06-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 05-04-2011: Added support for "ShockColors" and "ShockAliases".
%
% * 18-04-2011: "ShockColors" can now be initialized from the information in "ShockGroupsColors"
%               provided that there are more shocks and shock groups.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.ShockNames = ShockNames;
   DSGEModel.ShockGroups = ShockGroups;
   DSGEModel.Omega = Omega;
   DSGEModel.lambda = lambda;
   DSGEModel.SyOmega = SyOmega;
   DSGEModel.OriginalVariance = OriginalVariance;
   DSGEModel.SyOmegaAnnual = SyOmegaAnnual;
   DSGEModel.AnnualVariance = AnnualVariance;
   DSGEModel.EstStr = EstStr;
   if max(DSGEModel.annual)>1
      AddHeight = 20;
   else;
      AddHeight = 0;
   end;
%   if strcmp(lower(VarStr),'observed variables')==1;
%      VariableNames = DSGEModel.VariableNames;
%      DSGEModel.IsObsVar = 1;
%      AddHeight = 20;
%   else;
%      VariableNames = DSGEModel.StateVariableNames;
%      DSGEModel.IsObsVar = 0;
%      AddHeight = 0;
%   end;
   %
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      PeriodStrMat = '1 quarter';
      for j=2:80;
         PeriodStrMat = strvcat(PeriodStrMat,[int2str(j) ' quarters']);
      end;
      LowerValue = 4;
      UpperValue = 32;
   elseif strcmp(DSGEModel.DataFrequency,'m')==1;
      PeriodStrMat = '1 month';
      for j=2:240;
         PeriodStrMat = strvcat(PeriodStrMat,[int2str(j) ' months']);
      end;
      LowerValue = 12;
      UpperValue = 96;
   else;
      PeriodStrMat = '1 year';
      for j=2:20;
         PeriodStrMat = strvcat(PeriodStrMat,[int2str(j) ' years']);
      end;
      LowerValue = 1;
      UpperValue = 8;
   end;
else;
   SpectralDecompGUI = findobj('Type','figure','Tag','DSGEVARSpectralDecompDLG');
   controls = get(SpectralDecompGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
% create the dialog
%
switch selector;
   case 'init';
      SpectralDecompGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','DSGEVARSpectralDecompDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-520)/2 (CurrINI.scrsz(4)-175-AddHeight)/2 520 175+AddHeight], ...
         'Tag','DSGEVARSpectralDecompDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['DSGE-VAR Spectral Decompositions - ' EstStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','DSGEVARSpectralDecompDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(SpectralDecompGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 518 165+AddHeight],'Graphics Selections',45,[0.5 1],'on',CurrINI);
      %
      % select business cycle frequencies
      %
      controls.frequencytxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 120+AddHeight 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select cyclical period:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');
      %
      controls.lowerperiod = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 125+AddHeight 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',PeriodStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','DSGEVARSpectralDecompDLG lowerperiod', ...
         'Tag','LowerPeriodPopup',...
         'Value',LowerValue);
      %
      controls.upperperiod = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[303 125+AddHeight 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',PeriodStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','DSGEVARSpectralDecompDLG upperperiod', ...
         'Tag','UpperPeriodPopup',...
         'Value',UpperValue);
      %
      % Select variable
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 90+AddHeight 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 95+AddHeight 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','VariablePopup',...
         'Value',1);
      %
      % plots for shock groups
      %
      controls.shockgroups = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 60+AddHeight 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Use shock groups', ...
            'Value',0, ...
            'Enable','off', ...
            'Tag','ShockgroupsCheckBox');
      %
      if length(unique(ShockGroups))<length(ShockGroups);
         set(controls.shockgroups,'Enable','on','Value',1);
      end;
      %
      % normalize by variance
      %
      controls.normalize = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 40+AddHeight 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Normalize spectral density height with variance', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','NormalizeCheckBox');
      %
      % black edge patches
      %
      controls.blackedgepatch = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20+AddHeight 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Draw patches with black edges', ...
            'Value',1, ...
            'Enable','on', ...
            'Tag','BlackEdgePatchCheckBox');
      %
      % check if we should show spectral decompositions for annualized data or not
      %
      if max(DSGEModel.annual)>1;
         controls.annualize = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Plot decompositions for annualized observed variables', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','AnnualizedCheckBox');
      else;
         controls.annualize = [];
      end;
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[415 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','DSGEVARSpectralDecompDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[415 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','DSGEVARSpectralDecompDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(SpectralDecompGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'lowerperiod'
      lowervalue = get(controls.lowerperiod,'Value');
      uppervalue = get(controls.upperperiod,'Value');
      h = size(get(controls.upperperiod,'String'),1);
      if lowervalue>=uppervalue;
         if lowervalue<h
            set(controls.upperperiod,'Value',lowervalue+1);
         else;
            set(controls.lowerperiod,'Value',h-1);
            set(controls.upperperiod,'Value',h);
         end;
      end;
   case 'upperperiod'
      lowervalue = get(controls.lowerperiod,'Value');
      uppervalue = get(controls.upperperiod,'Value');
      if lowervalue>=uppervalue;
         if uppervalue>1
            set(controls.lowerperiod,'Value',uppervalue-1);
         else;
            set(controls.lowerperiod,'Value',1);
            set(controls.upperperiod,'Value',2);
         end;
      end;
   case 'showgraph'
      VariableValue = get(controls.variables,'Value');
      VariableNames = get(controls.variables,'String');
      CurrVariable = StringTrim(VariableNames(VariableValue,:));
      if VariableValue<size(VariableNames,1);
         set(controls.variables,'Value',VariableValue+1);
      else;
         set(controls.variables,'Value',1);
      end;
      %
      UseBlackEdge = get(controls.blackedgepatch,'Value');
      LowerFrequency = 2*pi/get(controls.upperperiod,'Value');
      UpperFrequency = 2*pi/get(controls.lowerperiod,'Value');
      %
      % collect the data
      %
      Omega = DSGEModel.Omega;
      f = length(Omega);
      n = DSGEModel.n;
      %
      Shocks = zeros(n,f);
      AnnStr = '';
      if (isempty(controls.annualize)==1)|(get(controls.annualize,'Value')==0);
         %
         % original data decompositions
         %
         for k=1:f;
            for j=1:n;
               ShockMat = DSGEModel.SyOmega{j,k};
               Shocks(j,k) = ShockMat(VariableValue,VariableValue);
            end;
         end;
         CurrVariance = DSGEModel.OriginalVariance(VariableValue);
      else;
         %
         % annualized data
         %
         for k=1:f;
            for j=1:n;
               ShockMat = DSGEModel.SyOmegaAnnual{j,k};
               Shocks(j,k) = ShockMat(VariableValue,VariableValue);
            end;
         end;
         AnnStr = 'Annual ';
         CurrVariance = DSGEModel.AnnualVariance(VariableValue);
      end;
      DataSpectrum = real(sum(Shocks,1));
      Shocks = real(Shocks);
      %
      NormalizeStr = '';
      if (get(controls.normalize,'Value')==1)&(CurrVariance>0);
         Shocks = (1/CurrVariance)*Shocks;
         DataSpectrum = (1/CurrVariance)*DataSpectrum;
         NormalizeStr = ' (normalized)';
      end;
      %
      % check if we should look at shock groups instead of all shocks
      %
      if get(controls.shockgroups,'Value')==1;
         Shocks = SubIdentityMatrix(DSGEModel.ShockGroups)'*Shocks;
         GroupNames = DSGEModel.ShockGroupNames(unique(DSGEModel.ShockGroups),:);
         ShockGroupNames = '';
         for j=1:size(GroupNames,1);
            ShockGroupNames = strvcat(ShockGroupNames,strrep(GroupNames(j,:),'_',' '));
         end;
         ShockGroupColors = DSGEModel.ShockGroupColors(unique(DSGEModel.ShockGroups),:);
      else;
         ShockGroupNames = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
         for j=1:size(ShockGroupNames,1);
            ShockGroupNames(j,:) = strrep(ShockGroupNames(j,:),'_',' ');
         end;
         if isempty(DSGEModel.ShockColors)==0;
            ShockGroupColors = DSGEModel.ShockColors(DSGEModel.DSGEVARShocks,:);
         else;
            if max(DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks))==n;
               ShockGroupColors = CreateDefaultColors(n);
            else;
               ShockGroupColors = CreateShockColors(DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks),DSGEModel.ShockGroupColors);
            end;
         end;
      end;
      %
      % accumulate across shocks for a given frequency to obtain bounds for patches
      %
      Shocks = [zeros(1,f);cumsum(Shocks,1)];
      %
      % create the graphs
      %
      FigHandle = figure('Visible','off', ...
                     'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                     'Tag',CurrVariable, ...
                     'Name',['Spectral Density Decomposition - ' AnnStr CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % mark up the cyclical area
      %
      UpperYValue = 2*max(DataSpectrum);
      BgPatchColor = [0.9 0.9 0.9];
      BgPatchHandle = patch([LowerFrequency UpperFrequency UpperFrequency LowerFrequency],[UpperYValue UpperYValue 0 0],BgPatchColor);
      set(BgPatchHandle,'FaceColor',BgPatchColor,'EdgeColor',BgPatchColor);
      hold('on');
      %
      % plot the patches with
      %
      Handles = [];
      for j=2:size(Shocks,1);
         Upper = Shocks(j,:);
         Lower = Shocks(j-1,:);
         PatchColor = ShockGroupColors(j-1,:);
         ConfPatch = patch([Omega fliplr(Omega)],[Upper fliplr(Lower)],PatchColor);
         if UseBlackEdge==1;
            EdgeColor = zeros(1,3);
         else;
            EdgeColor = PatchColor;
         end;
         set(ConfPatch,'FaceColor',PatchColor,'EdgeColor',EdgeColor);
         hold('on');
         Handles = [Handles ConfPatch];
      end;
      set(gca,'Layer','Top');
      %
      % draw dashed edges for the cyclical area
      %
      plot(LowerFrequency*ones(1,2),[0 UpperYValue],':','Color',[0 0 0],'LineWidth',0.5);
      hold('on');
      plot(UpperFrequency*ones(1,2),[0 UpperYValue],':','Color',[0 0 0],'LineWidth',0.5);
      hold('on');
      %
      % draw a line for the data
      %
      DataHandle = plot(Omega,DataSpectrum,'-','Color',[0 0 0],'LineWidth',2);
      hold('on');
      %
      % set the limits
      %
      set(gca,'XLim',[0 pi],'YLim',[0 1.1*max(DataSpectrum)]);
      hold('off');
      %
      % set the title
      %
      SetAxesFonts(CurrINI,'Title',['Spectral decomposition of ' lower(AnnStr) CurrVariable ' at ' lower(DSGEModel.EstStr) ' for lambda=' num2str(DSGEModel.lambda,'%0.4f')],'XLabel','Frequency in radians (\omega)','YLabel',['Spectral density' NormalizeStr]);
      %
      LegHandle = legend([Handles DataHandle],strvcat(ShockGroupNames,CurrVariable));
      SetAxesFonts(CurrINI,LegHandle);
      %
      SetXTickIntegers(gca);
      %
      % make the figure visible
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(controls.done,'UserData','done');
      delete(SpectralDecompGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(SpectralDecompGUI,'Visible','on');
   drawnow;
   SetYADAIcon(SpectralDecompGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of DSGEVARSpectralDecompDLG.m
%
