function DoneCalc = DSGEVARStationaryPredictiveVariance(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,lambda,n,p,X,XT,h,IsPosterior,Weights,NumDraws,CurrChain,PME,DSGEModel,CurrINI)
% DSGEVARStationaryPredictiveVariance: Computes the h-step-ahead stationary predictive variances for prior or
%                                      posterior draws and decomposes them into the shares due to the initial
%                                      state, the shocls, the measurement errors and parameter uncertainty for
%                                      a DSGE-VAR model.
%
% USAGE:
%
%       DoneCalc = DSGEVARStationaryPredictiveVariance(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,lambda,...
%                                                      n,p,X,XT,h,IsPosterior,Weights,NumDraws,CurrChain,PME,...
%                                                      DSGEModel,CurrINI)
%
% REQUIRED INPUT:  PhiDraws (matrix) of dimension UsedDraws x n(np+k) with the draws from the
%                           VAR parameters. Each row holds vec(Phi(0) Phi(1) ... Phi(p))'.
%
%                  SigmaDraws (matrix) of dimension UsedDraws x n(n+1)/2 with the unique draws
%                           of the SigmaEpsilon parameters. Each row holds vech(Sigma)'.
%
%                  PhiMode (matrix) of dimension nx(np+k) with fixed values for the Phi parameters
%                           that can be used as replacements for PhiDraws.
%
%                  SigmaModeEpsilon (matrix) of dimension nxn with fixed values of the SigmaEpsilon
%                           parameters that can be used as replacements for SigmaDraws.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  n (integer) with the number of endogenous variables.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  XT (vector) of dimension kx1 with the exogenous variables in the last historical period.
%
%                  h (integer) with the prediction horizon.
%
%                  IsPosterior (boolean)  that is 1 if draws from the posterior distribution
%                           are used and 0 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension UsedDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
%                  NumDraws (integer) with the original number of parameter draws for the DSGE model.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                           are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                           used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                           when generating the posterior draws.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 27, 2019.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-03-2019: Finished the function.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
UsedDraws = size(PhiDraws,1);
if isempty(Weights)==1;
   Weights = ones(UsedDraws,1);
end;
%
ShockCov = zeros(n,n,h);
ParameterCov = ShockCov;
MeanPredictY = zeros(n,h);
TotalCov = ShockCov;
%
% determine the backup values
%
k = DSGEModel.k;
if k>0;
   Phi0star = PhiMode(:,1:k);
   Phistar = PhiMode(:,k+1:(n*p)+k);
else;
   Phi0star = [];
   Phistar = PhiMode;
end;
J = eye(n);
if p>1;
   Phistar = [Phistar;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
Dn = DuplicationMatrix(n);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% initial conditions for the forecasts where all variables
% are at their means; backup values.
%
if k>0;
   YTstar = (eye(n*p)-Phistar)\(J*Phi0star*XT);
else;
   YTstar = zeros(n*p,1);
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(UsedDraws) ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Draw Number:';
   %
   ProgressStructure.name = ['Stationary Predictive Variance' PriorHeader ' - ' int2str(h) '-Steps-Ahead'];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes stationary predictive variances for the original data of a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Stationary Predictive Variance ' PriorHeader ' - ' int2str(h) '-Steps-Ahead'],600,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
%
for ParamIndex=1:UsedDraws;
   %
   % get the new parameters
   %
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if strcmp(abort,'cancel')==1;
         StopEstimation = 1;
         break;
      else;
         ProgressDLG([ParamIndex/UsedDraws ParamIndex]);
      end;
   end;
   SigmaEpsilon = reshape(Dn*SigmaDraws(ParamIndex,:)',n,n);
   if IsPositiveDefinite(SigmaEpsilon)==1;
      Phi = reshape(PhiDraws(ParamIndex,:)',n,(n*p)+k);
      if k>0;
         Phi0 = Phi(:,1:k);
         Phi = Phi(:,k+1:(n*p)+k);
      else;
         Phi0 = [];
      end;
      if p>1;
         Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
      end;
      if k>0;
         YT = (eye(n*p)-Phi)\(J*Phi0*XT);
      else;
         YT = zeros(n*p,1);
      end;
   else;
      SigmaEpsilon = SigmaModeEpsilon;
      Phi = Phistar;
      Phi0 = Phi0star;
      YT = YTstar;
   end;
   %
   % compute the population mean at the current parameter values
   %
   XTh = zeros(n,h);
   PredictY = zeros(n,h);
   Xbar = zeros(n*p,1);
   Phii = Phi;
   for i=1:h;
      if k>0;
         Xbar = (J*Phi0*X(:,i))+(Phi*Xbar);
         XTh(:,i) = J'*Xbar;
      end;
      PredictY(:,i) = XTh(:,i)+(J'*Phii*YT);
      Phii = Phii*Phi;
   end;
   %
   % compute the parameter dependent estimators of the two forecast uncertainty terms
   %
   SigmaBarY = zeros(n*p,n*p);
   for i=1:h;
      SigmaBarY = (J*SigmaEpsilon*J')+(Phi*SigmaBarY*Phi');
      ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/UsedDraws)*J'*SigmaBarY*J);
      ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/UsedDraws)*PredictY(:,i)*PredictY(:,i)');
   end;
   %
   MeanPredictY = MeanPredictY+((Weights(ParamIndex)/UsedDraws)*PredictY);
   drawnow;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
for i=1:h;
   ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
   TotalCov(:,:,i) = ShockCov(:,:,i)+ParameterCov(:,:,i);
end;
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-StationaryPredictiveVariance-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-StationaryPredictiveVariance-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(h) '.' int2str(UsedDraws) '.mat']);
end;
save(file,'MeanPredictY','ShockCov','ParameterCov','TotalCov');
DoneCalc = 1;

%
% end of DSGEVARStationaryPredictiveVariance.m
%
