function [FEVDs,LRVD,VarShares] = DSGEVARVarianceDecompositions(Phi,A0,p,h,VDType,levels)
% DSGEVARVarianceDecompositions: Conmputes variance decompositions at the given parameter values for
%                                the original data or for the levels data.
%
% USAGE:
%
%       [FEVDs,LRVD,VarShares] = DSGEVARVarianceDecompositions(Phi,A0,p,h,VDType,levels)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nxnp with the coefficient on lagged endogenous
%                      variables in the VAR.
%
%                  A0 (matrix) of dimension nxn with the contemporaneous responses in the
%                      endogenous variables from unit shocks.
%
%                  p (integer) with the lag order of the VAR.
%
%                  h (integer) with the response horizon.
%
%                  VDType (integer) that takes the value 1 when using the original data and
%                      2 when using the levels.
%
%                  levels (vector) of dimension n which indicates if a variable appears
%                      in levels (1) or first differences (0).
%
% REQUIRED OUTPUT: FEVDs (matrix) of dimension nxnxh with the variance decompositions for the
%                      n endogenous variables (rows) of the n structural shocks (columns) over
%                      the h forecast periods.
%
%                  LRVD (matrix) of dimension nxn with the long-run variance decompositions.
%                      These are based on max(h,200) for the original data and asymptotes for
%                      the levels data.
%
% OPTIONAL OUTPUT: VarShares (vector) with the h-steps ahead forecast error variances relative
%                      to the long-run forecast error variances of the endogenous variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 20, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-05-2010: The output variable "VarShares" is now optional rather than required.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
n = size(Phi,1);
FEVDs = zeros(n,n,h);
LRVD = zeros(n,n);
%
% set small responses to zero
%
Apos = (A0>1.0e-12);
Aneg = (A0<-1.0e-12);
A0 = (A0 .* Apos)+(A0 .* Aneg);
%
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
In = eye(n);
DenomOld = zeros(n,n);
NumerOld = zeros(n,n);
Phii = eye(n*p);
if VDType==1;
   for i=1:h;
      Ri = J'*Phii*J*A0;
      Ripos = (Ri>1.0e-12);
      Rineg = (Ri<-1.0e-12);
      Ri = (Ri .* Ripos)+(Ri .* Rineg);
      DenomOld = DenomOld+((Ri*Ri').*In);
      NumerOld = NumerOld+(Ri.*Ri);
      FEVDs(:,:,i) = DenomOld\NumerOld;
      Phii = Phi*Phii;
   end;
   if nargout==3;
      VarShares = diag(DenomOld);
   end;
   %
   % and now for the long run
   %
   if h<200;
      for i=h+1:200;
         Ri = J'*Phii*J*A0;
         Ripos = (Ri>1.0e-12);
         Rineg = (Ri<-1.0e-12);
         Ri = (Ri .* Ripos)+(Ri .* Rineg);
         DenomOld = DenomOld+((Ri*Ri').*In);
         NumerOld = NumerOld+(Ri.*Ri);
         Phii = Phi*Phii;
      end;
      LRVD = DenomOld\NumerOld;
   else;
      LRVD = FEVDs(:,:,h);
   end;
   if nargout==3;
      VarShares = VarShares./diag(DenomOld);
   end;
else;
   RiOld = zeros(n,n);
   AccMat = diag(1-levels);
   PhiSum = zeros(n*p);
   for i=1:h;
      Ri = RiOld+(J'*Phii*J*A0);
      Ripos = (Ri>1.0e-12);
      Rineg = (Ri<-1.0e-12);
      Ri = (Ri .* Ripos)+(Ri .* Rineg);
      RiOld = AccMat*Ri;
      DenomOld = DenomOld+((Ri*Ri').*In);
      NumerOld = NumerOld+(Ri.*Ri);
      FEVDs(:,:,i) = DenomOld\NumerOld;
      Phii = Phi*Phii;
   end;
   if nargout==3;
      VarShares = diag((AccMat*DenomOld/h)+((In-AccMat)*DenomOld));
   end;
   %
   % and now for the long run
   %
   if h<200;
      for i=h+1:200;
         Ri = RiOld+(J'*Phii*J*A0);
         Ripos = (Ri>1.0e-12);
         Rineg = (Ri<-1.0e-12);
         Ri = (Ri .* Ripos)+(Ri .* Rineg);
         RiOld = AccMat*Ri;
         DenomOld = DenomOld+((Ri*Ri').*In);
         NumerOld = NumerOld+(Ri.*Ri);
         Phii = Phi*Phii;
      end;
   end;
   RiSum = AccMat*J'*((eye(n*p)-Phi)\J)*A0;
   DenomNew = (AccMat*(RiSum*RiSum') .* In)+((In-AccMat)*DenomOld);
   NumerNew = (AccMat*(RiSum .* RiSum))+((In-AccMat)*NumerOld);
   LRVD = DenomNew\NumerNew;
   if nargout==3;
      VarShares = VarShares./diag(DenomNew);
   end;
end;

%
% end of DSGEVARVarianceDecompositions.m
%
