function GraphVARDSGEPosteriorDensity(ParamStr,PostSample,ParamMode,lambda,PME,CurrChain,DSGEModel,CurrINI)
% GraphVARDSGEPosteriorDensity: Creates plots of the posterior densities for a certain subset of the
%                               parameters of the DSGE-VAR.
%
% USAGE:
%
%       GraphVARDSGEPosteriorDensity(ParamStr,PostSample,ParamMode,lambda,PME,CurrChain,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ParamStr (string) that takes the values 'VAR', 'Sigma', and 'A0'.
%
%                 PostSample (matrix) of dimension NumDrawsxNumParam with the posterior draws of the
%                            parameters determined by ParamStr.
%
%                 ParamMode (matrix) of dimension nx(np+k) if ParamStr is 'VAR' and of dimension
%                            nxn otherwise. Holds the posterior mode estimates of the parameters.
%
%                 lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                 PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                 DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 9, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 10-03-2010: Finished the function.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 13-01-2011: Fixed path errors for the posterior mode estimated when "PME" is 2 or 3.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[NumDraws,NumParam] = size(PostSample);
%
% first we display a wait dialog
%
if DSGEModel.PosteriorDensityValue>2;
   WaittxtAdd = [' Estimation started at: ' StringTrim(datestr(now,14)) '. This may take a very long time...'];
   ShowCancel = 1;
else;
   WaittxtAdd = [' Estimation started at: ' StringTrim(datestr(now,14)) '.'];
   ShowCancel = 0;
end;
txt = ['Please wait while YADA computes kernel density estimates of the marginal posterior densities of the ' ParamStr ' parameters.' WaittxtAdd ' Current parameter: - 1 out of ' int2str(NumParam) '.'];
WaitHandle = WaitDLG(txt,'information','Kernel Density Estimation - Posterior',500,150,CurrINI,ShowCancel);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% now we try to construct the x-axis and y-axis data for the posterior densities
%
StopEstimation = 0;
for i=1:NumParam;
   txt = ['Please wait while YADA computes kernel density estimates of the marginal posterior densities of the ' ParamStr ' parameters.' WaittxtAdd ' Current parameter: ' int2str(i) ' out of ' int2str(NumParam) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   if DSGEModel.PosteriorDensityValue==1;
      if exist('YADAksdensity')==2;
         [f,x] = YADAksdensity(PostSample(:,i),'npoints',256);
      else;
         if exist('ksdensity')==2;
            [f,x] = ksdensity(PostSample(:,i),'npoints',256);
         else;
            [x,f] = KernelDensityPlot(PostSample(:,i),'knorm');
         end;
      end;
   elseif DSGEModel.PosteriorDensityValue==2;
      [x,f] = YADAMHDensEst(PostSample(:,i),2^7,0,'gaussian',[]);
   elseif DSGEModel.PosteriorDensityValue==3;
      [x,f] = YADAMHDensEst(PostSample(:,i),2^7,-1,'gaussian',WaitControls.cancel);
   elseif DSGEModel.PosteriorDensityValue==4;
      [x,f] = YADAMHDensEst(PostSample(:,i),2^7,-2,'gaussian',WaitControls.cancel);
   end;
   if isempty(x)==1;
      StopEstimation = 1;
      break;
   end;
   if i==1;
      Xposterior = zeros(NumParam,length(x));
      Yposterior = zeros(NumParam,length(x));
   end;
   if size(x,1)>size(x,2);
      Xposterior(i,:) = x';
   else;
      Xposterior(i,:) = x;
   end;
   if size(f,1)>size(f,2);
      Yposterior(i,:) = f';
   else;
      Yposterior(i,:) = f;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
if StopEstimation==1;
   return;
end;
%
% load data from the underlying file with posterior mode results
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
if PME==1;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
elseif PME==2;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
elseif PME==3;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
end;
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% setup data needed to compute the prior parameters
%
theta = ModeData.theta;
thetaPositions = ModeData.thetaPositions;
ModelParameters = ModeData.ModelParameters;
PriorDist = ModeData.PriorDist;
thetaDist = ModeData.thetaDist;
LowerBound = ModeData.LowerBound;
%
[Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
%
[LambdaVector,T] = GetLambda(DSGEModel);
if isinf(lambda)==0;
   SigmaModeEpsilon = ((lambda*T)/((lambda*T)+DSGEModel.n+1))*SigmaEpsilon;
else;
   SigmaModeEpsilon = SigmaEpsilon;
end;
[A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaModeEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
%
[Xdata,Ydata] = GraphVARDSGEPriorDensity(ParamStr,Phi,SigmaModeEpsilon,A0,T,lambda,H,B0,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
%
% setup data for plots
%
ParameterNames = '';
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
if strcmp(ParamStr,'VAR')==1;
   Xprior = Xdata.PhiXMatrix';
   Yprior = Ydata.PhiYMatrix';
   JointMode = vec(ParamMode);
   if k>0;
      for j=1:k;
         for i=1:n;
            ParameterNames = strvcat(ParameterNames,['Phi(' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(DSGEModel.XVariableNames(j,:)) ')']);
         end;
      end;
   end;
   for l=1:p;
      for j=1:n;
         for i=1:n;
            ParameterNames = strvcat(ParameterNames,['Phi(' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(DSGEModel.VariableNames(j,:)) ';lag=' int2str(l) ')']);
         end;
      end;
   end;
   ParamTestValue = 1.0e-02;
elseif strcmp(ParamStr,'Sigma')==1;
   Xprior = Xdata.SigmaXMatrix';
   Yprior = Ydata.SigmaYMatrix';
   JointMode = vech(ParamMode);
   VariancePosition = zeros(1,n);
   CurrPos = 0;
   VarPos = 0;
   for j=1:n;
      for i=j:n;
         CurrPos = CurrPos+1;
         if i==j;
            VarPos = VarPos+1;
            VariancePosition(VarPos) = CurrPos;
         end;
         ParameterNames = strvcat(ParameterNames,['Sigma(' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(DSGEModel.VariableNames(j,:)) ')']);
      end;
   end;
   ParamTestValue = 1.0e-03;
elseif strcmp(ParamStr,'A0')==1;
   ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
   Xprior = Xdata.A0XMatrix';
   Yprior = Ydata.A0YMatrix';
   JointMode = vec(ParamMode);
   for j=1:n;
      for i=1:n;
         ParameterNames = strvcat(ParameterNames,['A0(' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ')']);
      end;
   end;
   ParamTestValue = 1.0e-02;
end;
clear('Xdata','Ydata');
%
% we now have all information needed for the plots
%
%
% Directory where the plots are stored
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
% Compute the number of figures (max 16 plots per graph)
%
NumParam = length(JointMode);
NumFigs = ceil(NumParam/16);
%
if NumFigs==1;
   [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumParam);
else;
   NumRows = 4;
   NumCols = 4;
end;
%
% setting up the main loop
%
NumPlots = 0;
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
for i=1:NumFigs;
   if i>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>CurrINI.scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = CurrINI.scrsz(4)-700;
      end;
   end;
   FigHandles{i} = figure('Visible','off','Position',pos);
   if NumFigs==1;
      set(FigHandles{i},'Name',['Posterior Densities Of ' ParamStr ' Parameters']);
   else;
      set(FigHandles{i},'Name',['Posterior Densities Of ' ParamStr ' Parameters - Figure ' int2str(i)]);
   end;
   SetFigureProperties(FigHandles{i},CurrINI);
   %
   s = 0;
   for r=1:NumRows;
      for c=1:NumCols;
         NumPlots = NumPlots+1;
         if NumPlots<=NumParam;
            s = s+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
            subplot(NumRows,NumCols,s);
            %
            % plot the posterior density
            %
            PosteriorHandle = plot(Xposterior(NumPlots,:),Yposterior(NumPlots,:),'-','Color',[0 0 0.75],'LineWidth',2);
            hold('on');
            %
            % set the title
            %
            SetAxesFonts(CurrINI,'Title',StringTrim(ParameterNames(NumPlots,:)));
            axis('tight');
            XLim = get(gca,'XLim');
            YLim = get(gca,'YLim');
            AddOn = max(0.1*(YLim(2)-YLim(1)),0.01);
            YLim = [0 YLim(2)+AddOn];
            if YLim(2)>10000;
               YLim(2) = 1000;
            end;
            %
            % find the marginal mode
            %
            [MargMode,Ind] = max(Yposterior(NumPlots,:));
            XMargMode = Xposterior(NumPlots,Ind);
            MarginalModeHandle = plot(XMargMode*ones(1,2),[0 YLim(2)],'-','Color',[0 0 0],'LineWidth',1);
            hold('on');
            %
            % the joint mode
            %
            JointModeHandle = plot(JointMode(NumPlots)*ones(1,2),[0 YLim(2)],':','Color',[0 0 0],'LineWidth',1);
            hold('on');
            %
            % determine proper X axis scale
            %
            Ycheck = (Yposterior(NumPlots,:)/MargMode>ParamTestValue) .* (1:length(Yposterior(NumPlots,:)));
            Ycheck = Ycheck(Ycheck>0);
            if isempty(Ycheck)==0;
               XLimTest = [Xposterior(NumPlots,min(Ycheck)) Xposterior(NumPlots,max(Ycheck))];
               if XLimTest(2)-XLimTest(1)>0.01;
                  XLim = XLimTest;
               else;
                  XLim(1) = mean(Xposterior(NumPlots,:))-0.1;
                  XLim(2) = mean(Xposterior(NumPlots,:))+0.1;
               end;
            else;
               XLim(1) = mean(Xposterior(NumPlots,:))-0.1;
               XLim(2) = mean(Xposterior(NumPlots,:))+0.1;
            end;
            %
            % check if we have a variance term
            %
            if strcmp(ParamStr,'Sigma')==1;
               if (sum(VariancePosition==NumPlots)==1)&(XLim(1)<0);
                  XLim(1) = 0;
               end;
            end;
            %
            % and now we can deal with the prior
            %
            PriorHandle = plot(Xprior(NumPlots,:),Yprior(NumPlots,:),'--','Color',[0.75 0 0],'LineWidth',2);
            hold('on');
            %
            % set the axes limits
            %
            set(gca,'YLim',YLim,'XLim',XLim);
            hold('off');
         end;
      end;
   end;
   %
   % set a legend
   %
   legHandle = legend([PriorHandle PosteriorHandle MarginalModeHandle JointModeHandle],strvcat('Prior','Posterior','Marginal Mode','Joint Mode'));
   %
   SetAxesFonts(CurrINI,legHandle);
   %
   % transform units temporarily into pixels
   %
   OldAxesUnits = get(gca,'Units');
   OldLegUnits = get(legHandle,'Units');
   set(gca,'Units','pixels');
   set(legHandle,'Units','pixels');
   %
   % check if we can move legend
   %
   LegPos = get(legHandle,'Position');
   AxPos = get(gca,'Position');
   if LegPos(1)<AxPos(1)+AxPos(3);
      if AxPos(1)+AxPos(3)+10+LegPos(3)<1000;
         LegPos(1) = AxPos(1)+AxPos(3)+10;
         set(legHandle,'Position',LegPos);
      end;
   end;
   set(gca,'Units',OldAxesUnits);
   set(legHandle,'Units',OldLegUnits);
   %
   % save to file
   %
   if DirExist(GraphDirectory)==1;
      if NumFigs>1;
         file = FixFilePath([GraphDirectory '\PosteriorDensity-DSGEVAR-' ParamStr '-' num2str(lambda,'%0.4f') '-' int2str(T) '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel  '-' int2str(i) '.fig']);
      else;
         file = FixFilePath([GraphDirectory '\PosteriorDensity-DSGEVAR-' ParamStr '-' num2str(lambda,'%0.4f') '-' int2str(T) '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.fig']);
      end;
      saveas(FigHandles{i},file);
   end;
   %
   % Fix the toolbar cdata
   %
   SetToolbarImages(FigHandles{i},CurrINI.images);
end;
%
% display the images
%
for i=1:NumFigs;
   set(FigHandles{i},'Visible','on');
   drawnow;
   pause(0.02);
end;

%
% end of GraphVARDSGEPosteriorDensity.m
%
