function [Xdata,Ydata] = GraphVARDSGEPriorDensity(ParamStr,PhiInit,SigmaEpsilonInit,A0Init,T,lambda,HInit,B0Init,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI)
% GraphVARDSGEPriorDensity: Creates plots of the prior densities for a certain subset of the
%                           parameters of the DSGE-VAR.
%
% USAGE:
%
%       [Xdata,Ydata] = GraphVARDSGEPriorDensity(ParamStr,PhiInit,SigmaEpsilonInit,A0Init,T,lambda,HInit,...
%                                                B0Init,thetaPositions,ModelParameters,PriorDist,thetaDist,...
%                                                LowerBound,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  ParamStr (string) that takes the values 'VAR', 'Sigma', and 'A0'.
%
%                  PhiInit (matrix) of dimension nx(np+k) with the parameters on the exogenous and lagged endogenous variables
%                          of the VAR.
%
%                  SigmaEpsilonInit (matrix) of dimension nxn with the residual covariances of the VAR.
%
%                  A0Init (matrix) of dimension nxn with the contemporaneous effects of the n structural
%                          shocks in the DSGE-VAR model.
%
%                  T (integer) with the sample size of the data.
%
%                  lambda (scalar) with the lambda hyperparameter.
%
%                  HInit (matrix) of dimension rxn or rxnxT (if time-varying) with the parameters on the state variables
%                          in the measurement equation of the state-space representation of the DSGE model.
%
%                  B0Init (matrix) of dimension rxq with the parameters on the iid N(0,1) structural shocks in the
%                          state equation of the state-space representation of the DSGE model.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                          of the model parameter in position i of theta. The field "distribution" has a string
%                          vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                          DSGE model.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                          "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                          parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                          and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                          it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                          truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                          values are the lower and the upper bound. For the student-t they are the location (mode),
%                          the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                          and scale parameters are needed. For the Pareto the parameter values are for the mode
%                          and the shape parameter.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                          truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                          and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                          input files, variable and sample  data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% OPTIONAL OUTPUT: Xdata (structure) with 3 fields: "PhiXmatrix", "SigmaXMatrix" and "A0XMatrix" with the X axis data
%                          for plotting prior densities of the parameter groups of the DSGE-VAR.
%
%                  Ydata (structure) with 3 fields: "PhiYmatrix", "SigmaYMatrix" and "A0YMatrix" with the Y axis data
%                          for plotting prior densities of the parameter groups of the DSGE-VAR.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 9, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 10-03-2010: Finished the function.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 12-01-2011: Added additional support for unix and mac paths.
%
% * 14-12-2011: Made sure that "DSGEPriorSampling" receives the "HeaderStr" input variable.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-05-2020: Took into account that "DSGEPriorSampling" and "DSGESystemPriorSampling" take the input variable
%               "IsAdaptiveLearning".
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
if nargout==2;
   Xdata = [];
   Ydata = [];
end;
%
% the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% check if we can load the results from file
%
EstimateDensities = 1;
MatFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDensities-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(T) '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
if FileExist(MatFile)==1;
   if nargout==2;
      answer = 'Yes';
   else;
      txt = ['YADA has located previously computed kernel density estimates of the ' ParamStr ' parameters. Would you like to use them?'];
      answer = Query(txt,'question',200,[ParamStr ' Prior Densities'],500,'no',CurrINI);
   end;
   if strcmp(lower(answer),'yes')==1;
      ErrorStr = '';
      if MatlabNumber>=8.0;
         try;
            MatData = load(MatFile);
         catch ME;
            ErrorStr = ['Unable to load the file "' MatFile '". Message caught is: ' YADALastError(ME)];
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
      else;
         try;
            MatData = load(MatFile);
         catch;
            ErrorStr = ['Unable to load the file "' MatFile '". Message caught is: ' YADALastError];
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
      end;
      if isempty(ErrorStr)==1;
         PhiXMatrix = MatData.PhiXMatrix;
         SigmaXMatrix = MatData.SigmaXMatrix;
         A0XMatrix = MatData.A0XMatrix;
         PhiYMatrix = MatData.PhiYMatrix;
         SigmaYMatrix = MatData.SigmaYMatrix;
         A0YMatrix = MatData.A0YMatrix;
         EstimateDensities = 0;
      end;
   end;
end;
%
% initialize dimensions
%
p = DSGEModel.BVARLags;
k = DSGEModel.k;
n = DSGEModel.n;
npk = (n*p)+k;
%
% check if we should estimate the densities
%
if EstimateDensities==1;
   %
   % create the output directory
   %
   stat = MakeDir(DSGEModel.OutputDirectory,'priordraws');
   if stat~=1;
      if ispc==1;
         txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\priordraws". The prior density computation has therefore been aborted.'];
      else;
         txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/priordraws". The prior density computation has therefore been aborted.'];
      end;
      About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
      return;
   end;
   %
   % load the AIMData file
   %
   ErrorStr = '';
   if MatlabNumber>=8.0;
      try;
         AIMData = load(DSGEModel.AIMDataFile);
      catch ME;
         ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
      end;
   else;
      try;
         AIMData = load(DSGEModel.AIMDataFile);
      catch;
         ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
      end;
   end;
   if isempty(ErrorStr)==0;
      About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
      return;
   end;
   %
   % get the sample information
   %
   [y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
   %
   % and the product moments for the exogenous variables
   %
   DetProductMoments = CalcDeterministicProductMoments(x,X);
   %
   % start by drawing the DSGE model parameters
   %
   NumDraws = 10000;
   thetaDraws = DSGEPriorSampling(PriorDist,thetaDist,LowerBound,NumDraws,0,'DSGE-VAR Prior Sampling',DSGEModel,CurrINI);
   %
   % initialize the output
   %
   if strcmp(ParamStr,'Sigma')==1;
      ParamType = 2;
   elseif strcmp(ParamStr,'VAR')==1;
      ParamType = 1;
   elseif strcmp(ParamStr,'A0')==1;
      ParamType = 3;
   end;
   %
   % check if we can sample 5000 draws from the prior of the VAR parameters
   %
   try;
      PhiDraws = zeros(NumDraws,n*npk);
   catch;
      %
      % this should work
      %
      NumDraws = 1000;
      PhiDraws = zeros(NumDraws,n*npk);
      thetaDraws = thetaDraws((1:10:10000),:);
   end;
   SigmaDraws = zeros(NumDraws,(n*(n+1))/2);
   if isempty(A0Init)==0;
      A0Draws = zeros(NumDraws,n*n);
   end;
   %
   % update the wait dialog
   %
   txt = strvcat(['Please wait while YADA draws ' int2str(NumDraws) ' values from the prior distribution of the DSGE-VAR parameters with lambda=' num2str(lambda,'%0.4f') '.'],' ',['Sampling started at: ' StringTrim(datestr(now,14)) '.']);
   WaitHandle = WaitDLG(txt,'information',[ParamStr ' Prior Densities'],500,300,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   %
   for j=1:NumDraws;
      %
      % try to solve the DSGE model at the current thetaj
      %
      thetaj = thetaDraws(:,j);
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaj,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status==1;
         [Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
         if PriorStatus==1;
            Phi = GammayY/GammaYY;
            SigmaEpsilon = Gammayy-(Phi*GammayY');
         else;
            Phi = PhiInit;
            SigmaEpsilon = SigmaEpsilonInit;
            H = HInit;
            B0 = B0Init;
         end;
      else;
         Phi = PhiInit;
         SigmaEpsilon = SigmaEpsilonInit;
         H = HInit;
         B0 = B0Init;
      end;
      %
      % compute the SigmaEpsilon and Phi
      %
      if isinf(lambda)==0;
         InvGammaYY = GammaYY\eye(npk);
         %
         % draw a value of SigmaEpsilon from the inverted Wishart
         %
         SigmaEpsilon = InvWishartRndFcn(lambda*T*SigmaEpsilon,(lambda*T)-npk);
         PhiDraws(j,:) = MultiNormalRndFcn(vec(Phi),(1/(lambda*T))*kron(InvGammaYY,SigmaEpsilon),1)';
         SigmaDraws(j,:) = vech(SigmaEpsilon)';
      else;
         SigmaDraws(j,:) = vech(SigmaEpsilon)';
         PhiDraws(j,:) = vec(Phi)';
      end;
      if isempty(A0Init)==0;
         [A0,IdStat] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
         A0Draws(j,:) = vec(A0)';
      end;
      drawnow;
   end;
   clear('thetaDraws');
   %
   % estimate with kernel density methods
   %
   PhiXMatrix = zeros(1024,n*npk);
   PhiYMatrix = zeros(1024,n*npk);
   for j=1:n*npk;
      txt = strvcat(['Kernel density estimation of the Phi(i,j) parameters. Current parameters: ' int2str(j) ' of ' int2str(n*npk) '.'],' ','NOTE: The data will be stored to disk for possible reuse.');
      set(WaitControls.text,'String',txt);
      drawnow;
      [XValue,YValue] = YADAMHDensEst(PhiDraws(:,j),2^10,0,'gaussian',[]);
      PhiXMatrix(:,j) = XValue;
      PhiYMatrix(:,j) = YValue;
   end;
   clear('PhiDraws');
   SigmaXMatrix = zeros(1024,(n*(n+1))/2);
   SigmaYMatrix = zeros(1024,(n*(n+1))/2);
   for j=1:(n*(n+1))/2;
      txt = strvcat(['Kernel density estimation of the Sigma(i,j) parameters. Current parameters: ' int2str(j) ' of ' int2str((n*(n+1))/2) '.'],' ','NOTE: The data will be stored to disk for possible reuse.');
      set(WaitControls.text,'String',txt);
      drawnow;
      [XValue,YValue] = YADAMHDensEst(SigmaDraws(:,j),2^10,0,'gaussian',[]);
      SigmaXMatrix(:,j) = XValue;
      SigmaYMatrix(:,j) = YValue;
   end;
   clear('SigmaDraws');
   if isempty(A0Init)==0;
      A0XMatrix = zeros(1024,n*n);
      A0YMatrix = zeros(1024,n*n);
      for j=1:n*n;
         txt = strvcat(['Kernel density estimation of the A0(i,j) parameters. Current parameters: ' int2str(j) ' of ' int2str(n*n) '.'],' ','NOTE: The data will be stored to disk for possible reuse.');
         set(WaitControls.text,'String',txt);
         drawnow;
         [XValue,YValue] = YADAMHDensEst(A0Draws(:,j),2^10,0,'gaussian',[]);
         A0XMatrix(:,j) = XValue;
         A0YMatrix(:,j) = YValue;
      end;
   else;
      A0XMatrix = [];
      A0YMatrix = [];
   end;
   clear('A0Draws');
   %
   % save data to disk
   %
   save(MatFile,'PhiXMatrix','PhiYMatrix','SigmaXMatrix','SigmaYMatrix','A0XMatrix','A0YMatrix');
   %
   % close the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if we should exit
%
if nargout==2;
   Xdata.PhiXMatrix = PhiXMatrix;
   Xdata.SigmaXMatrix = SigmaXMatrix;
   Xdata.A0XMatrix = A0XMatrix;
   Ydata.PhiYMatrix = PhiYMatrix;
   Ydata.SigmaYMatrix = SigmaYMatrix;
   Ydata.A0YMatrix = A0YMatrix;
   return;
end;
%
% plot the densities
%
ParameterNames = '';
if strcmp(ParamStr,'VAR')==1;
   XValues = PhiXMatrix';
   YValues = PhiYMatrix';
   PriorMode = vec(PhiInit);
   if k>0;
      for j=1:k;
         for i=1:n;
            ParameterNames = strvcat(ParameterNames,['Phi(' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(DSGEModel.XVariableNames(j,:)) ')']);
         end;
      end;
   end;
   for l=1:p;
      for j=1:n;
         for i=1:n;
            ParameterNames = strvcat(ParameterNames,['Phi(' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(DSGEModel.VariableNames(j,:)) ';lag=' int2str(l) ')']);
         end;
      end;
   end;
   ParamTestValue = 1.0e-02;
elseif strcmp(ParamStr,'Sigma')==1;
   XValues = SigmaXMatrix';
   YValues = SigmaYMatrix';
   PriorMode = vech(SigmaEpsilonInit);
   VariancePosition = zeros(1,n);
   CurrPos = 0;
   VarPos = 0;
   for j=1:n;
      for i=j:n;
         CurrPos = CurrPos+1;
         if i==j;
            VarPos = VarPos+1;
            VariancePosition(VarPos) = CurrPos;
         end;
         ParameterNames = strvcat(ParameterNames,['Sigma(' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(DSGEModel.VariableNames(j,:)) ')']);
      end;
   end;
   ParamTestValue = 1.0e-03;
elseif strcmp(ParamStr,'A0')==1;
   ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
   XValues = A0XMatrix';
   YValues = A0YMatrix';
   PriorMode = vec(A0Init);
   for j=1:n;
      for i=1:n;
         ParameterNames = strvcat(ParameterNames,['A0(' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ')']);
      end;
   end;
   ParamTestValue = 1.0e-02;
end;
%
% Directory where the plots are stored
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
% Compute the number of figures (max 16 plots per graph)
%
NumParam = length(PriorMode);
NumFigs = ceil(NumParam/16);
%
if NumFigs==1;
   [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumParam);
else;
   NumRows = 4;
   NumCols = 4;
end;
%
% setting up the main loop
%
NumPlots = 0;
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
for i=1:NumFigs;
   if i>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>CurrINI.scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = CurrINI.scrsz(4)-700;
      end;
   end;
   FigHandles{i} = figure('Visible','off','Position',pos);
   if NumFigs==1;
      set(FigHandles{i},'Name',['Prior Densities Of ' ParamStr ' Parameters']);
   else;
      set(FigHandles{i},'Name',['Prior Densities Of ' ParamStr ' Parameters - Figure ' int2str(i)]);
   end;
   SetFigureProperties(FigHandles{i},CurrINI);
   %
   s = 0;
   for r=1:NumRows;
      for c=1:NumCols;
         NumPlots = NumPlots+1;
         if NumPlots<=NumParam;
            s = s+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
            subplot(NumRows,NumCols,s);
            PriorHandle = plot(XValues(NumPlots,:),YValues(NumPlots,:),'-','Color',[0.75 0 0],'LineWidth',2);
            hold('on');
            %
            % find the marginal mode
            %
            [MargMode,Ind] = max(YValues(NumPlots,:));
            if (length(Ind)==1)&(Ind>1)&(Ind<1024);
               XMargMode = XValues(NumPlots,Ind);
               ModeHandle = plot(XMargMode*ones(1,2),[0 MargMode],'-','Color',[0 0 0],'LineWidth',1);
               hold('on');
            else;
               ModeHandle = [];
            end;
            %
            % set the title
            %
            SetAxesFonts(CurrINI,'Title',StringTrim(ParameterNames(NumPlots,:)));
            axis('tight');
            XLim = get(gca,'XLim');
            YLim = get(gca,'YLim');
            AddOn = max(0.1*(YLim(2)-YLim(1)),0.01);
            YLim = [0 YLim(2)+AddOn];
            if YLim(2)>10000;
               YLim(2) = 1000;
            end;
            %
            % determine proper X axis scale
            %
            Ycheck = (YValues(NumPlots,:)/MargMode>ParamTestValue) .* (1:length(YValues(NumPlots,:)));
            Ycheck = Ycheck(Ycheck>0);
            if isempty(Ycheck)==0;
               XLimTest = [XValues(NumPlots,min(Ycheck)) XValues(NumPlots,max(Ycheck))];
               if XLimTest(2)-XLimTest(1)>0.01;
                  XLim = XLimTest;
               else;
                  XLim(1) = mean(XValues(NumPlots,:))-0.1;
                  XLim(2) = mean(XValues(NumPlots,:))+0.1;
               end;
            else;
               XLim(1) = mean(XValues(NumPlots,:))-0.1;
               XLim(2) = mean(XValues(NumPlots,:))+0.1;
            end;
            %
            % check if we have a variance term
            %
            if strcmp(ParamStr,'Sigma')==1;
               if (sum(VariancePosition==NumPlots)==1)&(XLim(1)<0);
                  XLim(1) = 0;
               end;
            end;
            set(gca,'YLim',YLim,'XLim',XLim);
            hold('off');
         end;
      end;
   end;
   %
   % save to file
   %
   if DirExist(GraphDirectory)==1;
      if NumFigs>1;
         file = FixFilePath([GraphDirectory '\PriorDensity-DSGEVAR-' ParamStr '-' num2str(lambda,'%0.4f') '-' int2str(T) '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel  '-' int2str(i) '.fig']);
      else;
         file = FixFilePath([GraphDirectory '\PriorDensity-DSGEVAR-' ParamStr '-' num2str(lambda,'%0.4f') '-' int2str(T) '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.fig']);
      end;
      saveas(FigHandles{i},file);
   end;
   %
   % Fix the toolbar cdata
   %
   SetToolbarImages(FigHandles{i},CurrINI.images);
end;
%
% display the images
%
for i=1:NumFigs;
   set(FigHandles{i},'Visible','on');
   drawnow;
   pause(0.02);
end;

%
% end of GraphVARDSGEPriorDensity.m
%
