function PredEventData = LoadDSGEVARPredictionPaths(PredictionEvent,UsedDraws,h,IsConditional,ShockMethod,DataType,Weights,lambda,PME,CurrChain,NumPaths,DSGEModel,CurrINI,controls)
% LoadDSGEVARPredictionPaths:  Loads prediction paths for a DSGE-VAR model from disk and computes the prediction event data.
%
% USAGE:
%
%       PredEventData = LoadDSGEVARPredictionPaths(PredictionEvent,UsedDraws,h,IsConditional,ShockMethod,DataType,...
%                                                  Weights,lambda,PME,CurrChain,NumPaths,DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  PredictionEvent (matrix) of dimension nx3 with the prediction event information. The first
%                                  column holds the upper bound, the second the lower bound for the event,
%                                  while the third holds the event length in periods.
%
%                  UsedDraws (integer) with the number of parameter draws used for the predictions.
%
%                  h (integer) with the maximum forecast horizon.
%
%                  IsConditional (boolean) that is 1 if the predictions are conditional and 0 otherwise.
%
%                  ShockMethod (string) that is either "Direct-", "WaggonerZha-", or "".
%
%                  DataType (string) that is "Original", "Annualized", or "Transformed".
%
%                  Weights (vector) of dimension UsedDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
%                  lambda (scalar)  with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                                  are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                                  used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                                  when generating the posterior draws.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  NumPaths (integer) with the number of forecast paths to compute per parameter value.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                                  model, such as input files, variable and sample data. Actual observations
%                                  are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                                  main dialog.
%
% REQUIRED OUTPUT: PredEventData (matrix) of dimension nx8 where the first column holds the number of times
%                                  the PredPaths data satisfies the event, the second hold the number
%                                  of times the PredPaths data is greater than the event, the third
%                                  the number of time the data is above the upper bound, while the
%                                  fourth column holds the number of times the event has been tested.
%                                  The fifth and sixth column measure the average deviation from the
%                                  lower bound when below the lower bound and the square of this, while
%                                  the seventh and eighth columns hold similar data for the upper bound
%                                  of the event when the variables are above the upper bound.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 9, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-06-2010: Updated the documentation.
%
% * 18-11-2010: Took into account that "PredEventData" now has 7 columns.
%
% * 19-11-2010: And now 8.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 01-02-2011: Corrected an error in the filename when loading the paths. The "ShockMethod" input
%               variable was missing.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 11-09-2017: Added the input variable "Weights".
%
% * 15-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
n = size(PredictionEvent,1);
p = DSGEModel.BVARLags;
PredEventData = zeros(n,8);
CondStr = '';
if IsConditional==1;
   CondStr = 'Cond';
end;
if isempty(Weights)==1;
   Weights = ones(UsedDraws,1);
end;
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% check if prediction event results already exist on disk.
%
file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-' CondStr 'Prediction-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' ShockMethod DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(UsedDraws) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
%
% load the data
%
PredData = load(file);
if (isfield(PredData,'PredictionEvent')==1)&(isfield(PredData,'PredEventData')==1)&(isempty(PredData.PredEventData)==0)&(size(PredData.PredEventData,2)==8)&(isequal(PredictionEvent,PredData.PredictionEvent)==1);
   %
   % the PredEventData has already been calculated.
   %
   PredEventData = PredData.PredEventData;
   return;
end;
%
%
if IsConditional==0;
   HeadStr = 'Predictive Distributions';
else;
   HeadStr = 'Conditional Predictive Distributions';
end;
%
% set up a wait dialog
%
txt = ['Please wait while YADA computes the prediction event probabilities for the observed variables via a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') '. The data are loaded sequentially from disk and it may therefore take some time. Calculation started at: ' StringTrim(datestr(now,14)) '.'];
WaitHandle = WaitDLG(txt,'information',HeadStr,500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
%
for ParamIndex=1:UsedDraws;
   file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-' CondStr 'Prediction-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' ShockMethod DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
   %
   % load the data
   %
   PredData = load(file);
   PredPaths = PredData.PredPaths;
   %
   % compute the prediction event data
   %
   PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,PredictionEvent,Weights(ParamIndex));
   %
   if ParamIndex==UsedDraws;
      %
      % append PredictionEvent and PredEventData to file.
      %
      save(file,'PredEventData','PredictionEvent','-APPEND');
   end;
   clear('PredData');
   drawnow;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of LoadDSGEVARPredictionPaths.m
%
