function [Status,value,CurrChain,PME,Lambda,IsSubSample,RecomputePosteriorKernel] = SequentialMarginalLikelihoodDSGEVAR(DSGEModel,CurrINI,controls,value,CurrChain,PME,Lambda,IsSubSample,RecomputePosteriorKernel,RecomputeML)
% SequentialMarginalLikelihoodDSGEVAR: Computes or collects an already estimated marginal likelihood for
%                                      a DSGE-VAR model.
%
% USAGE:
%
%       [Status,value,CurrChain,PME,Lambda,IsSubSample,RecomputePosteriorKernel] = ...
%                                          SequentialMarginalLikelihoodDSGEVAR(DSGEModel,CurrINI,controls,value,...
%                                                                              CurrChain,PME,Lambda,IsSubSample,...
%                                                                              RecomputePosteriorKernel,RecomputeML)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information 
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                  value (integer) with the selected position for the available lambda hyperparameter.
%
%                  CurrChain (integer) with the value of the MCMC chain to use.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  Lambda (vector) with the lambda hyperparameter values that can be used.
%
%                  IsSubSample (boolean) that takes the value 1 if a subsample of the posterior
%                            draws should be used and 0 if all post-burnin draws should be used.
%                            When empty, the function checks which alternative should be considered.
%
%                  RecomputePosteriorKernel (boolean) that takes the value 1 if the posterior kernel
%                            should be recomputed, and 0 otherwise. When empty, the function asks which
%                            case the user would like to use.
%
%                  RecomputeML (boolean) that takes the value 1 if the marginal likelihood should be
%                            recomputed, provided that it has already been computed and saved to disk;
%                            and 0 otherwise.
%
% REQUIRED OUTPUT: Status (boolean) that takes the value 1 if the computations could be successfully
%                            carries out, and 0 otherwise.
%
%                  value (integer) with the selected position for the available lambda hyperparameter.
%
%                  CurrChain (integer) with the value of the MCMC chain to use.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  Lambda (vector) with the lambda hyperparameter values that can be used.
%
%                  IsSubSample (boolean) that takes the value 1 if a subsample of the posterior
%                            draws should be used and 0 if all post-burnin draws should be used.
%
%                  RecomputePosteriorKernel (boolean) that takes the value 1 if the posterior kernel
%                            should be recomputed, and 0 otherwise.
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 14, 2019.
%                        This version: March 13, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-02-2019: Finished the function after reading the posterior mode file.
%
% * 08-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 03-02-2022: Added saving results to text file.
%
% * 04-02-2022: Finished the additions.
%
% * 06-01-2023: Updated the documentation.
%
% * 13-03-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Status = 0;
%
if isempty(value)==1;
   [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('Marginal Likelihood Estimation',DSGEModel,controls,CurrINI,1);
end;
if (isempty(value)==1)|(value==0);
   SetLogText('Unable to load posterior data. Aborting calculation.');
   value = [];
   CurrChain = [];
   PME = [];
   Lambda = [];
   return;
end;
lambda = Lambda(value);
%
% and now the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% load data from the underlying file with posterior mode results
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% number of lags of the DSGE-VAR
%
p = DSGEModel.BVARLags;
%
% load data needed to rerun the marginal likelihood estimation functions
%
if PME==1;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
elseif PME==2;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
elseif PME==3;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
end;
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   SetLogText('Unable to load posterior mode data. Aborting calculation.');
   return;
end;
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% Coverage probabilities for Geweke''s Harmonic Mean estimator of
% the marginal likelihood
%
CovStartStr = get(controls.dsge.coveragestart,'String');
CovStartValue = str2num(StringTrim(CovStartStr(DSGEModel.CoverageStart,:)));
CovIncStr = get(controls.dsge.coverageincrement,'String');
CovIncValue = str2num(StringTrim(CovIncStr(DSGEModel.CoverageIncrement,:)));
CovEndStr = get(controls.dsge.coverageend,'String');
CovEndValue = str2num(StringTrim(CovEndStr(DSGEModel.CoverageEnd,:)));
CovProb = (CovStartValue:CovIncValue:CovEndValue);
%
if (isempty(CurrChain)==1)&(NumChains>1);
   %
   % ask which chain to load
   %
   ChainsStr = '';
   for i=1:NumChains;
      ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
   end;
   [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC\SMC Chain Selection','Marginal Likelihood Estimation','','',CurrINI);
elseif (isempty(CurrChain)==0)&(CurrChain>NumChains);
   action = 'cancel';
   SetLogText(['The selected chain number (' int2str(CurrChain) ') is greater than the available number of posterior draws chains.']);
else;
   CurrChain = 1;
   action = 'ok';
end;
if strcmp(lower(action),'cancel')==1;
   SetLogText('Aborting calculation.');
   return;
end;
%
% check if sub-sample should be
%
if DSGEModel.PosteriorSampler<=7;
   if DSGEModel.MarginalLikelihoodValue~=4;
      if isempty(IsSubSample)==1;
         txt = 'Would you like to use a sub-sample of the posterior draws when estimating the marginal likelihood?';
         answer = Query(txt,'question',400,'Marginal Likelihood Estimation',500,'no',CurrINI);
         if strcmp(lower(answer),'yes')==1;
            IsSubSample = 1;
         else;
            IsSubSample = 0;
         end;
      end;
   end;
   %
   % check if the log posterior kernel should be recomputed
   %
   if isempty(RecomputePosteriorKernel)==1;
      txt = 'Would you like to recompute the value of the log posterior at the posterior draws?';
      answer = Query(strvcat(txt,' ','NOTE: This is primarily useful if the posterior draws are taken from another data sample.'),'question',400,'Marginal Likelihood Estimation',500,'no',CurrINI);
      if strcmp(lower(answer),'yes')==1;
         RecomputePosteriorKernel = 1;
      else;
         RecomputePosteriorKernel = 0;
      end;
   end;
   %
   % set the backup marginal likelihood value to Geweke's modified harmonic
   %
   if (DSGEModel.MarginalLikelihoodValue==1)|(DSGEModel.MarginalLikelihoodValue==5);
      MarginalLikelihoodValue = 2;
   else;
      MarginalLikelihoodValue = DSGEModel.MarginalLikelihoodValue;
   end;
else;
   IsSubSample = 0;
   RecomputePosteriorKernel = 0;
end;
%
% check if there is a system prior
%
if isfield(ModeData,'LogMargLikeSystemLaplace')==1;
   LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
else;
   LogMargLikeSystemLaplace = 0;
end;
%
% check if an SMC or IS posterior sampler is used
%
if DSGEModel.PosteriorSampler<=7;
   if VerifyDSGEVARPosteriorDraws(DSGEModel,CurrChain,controls)==1;
      %
      % load posterior draws
      %
      [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
      clear('thetaPostSample');
      %
      % select which posterior draws to use
      %
      if DSGEModel.MarginalLikelihoodValue~=4;
         LogPost = LogPost(NumBurnin+1:NumPostDraws,:);
         PostSample = PostSample(NumBurnin+1:NumPostDraws,:);
         AcceptedDraws = AcceptedDraws(NumBurnin+1:NumPostDraws);
         NumDraws = NumPostDraws-NumBurnin;
         %
         if IsSubSample==1;
            %
            % check which draws we should use
            %
            UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
            LogPost = LogPost(UseDraws);
            PostSample = PostSample(UseDraws,:);
            AcceptedDraws = AcceptedDraws(UseDraws);
         end;
      end;
      NumDraws = length(LogPost);
      %
      % check if old data is available and if they should be used
      %
      MargLikeEstValue = MarginalLikelihoodValue-1;
      if RecomputeML==0;
         MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-MarginalLikelihood-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.PosteriorSampler) '.' int2str(MargLikeEstValue) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumDraws) '.' int2str(NumPostDraws) '.mat']);
         if FileExist(MatFile)==1;
            MLData = load(MatFile);
            LogMargLike = MLData.LogMargLike;
            LogMargLikeSystemLaplace = MLData.LogMargLikeSystemLaplace;
            MargLikeEstimator = MLData.MargLikeEstimator;
         end;
      end;
      %
      % compute the marginal likelihood
      %
      if MarginalLikelihoodValue==2;
         SetLogText(strvcat('Computing Geweke''s modified harmonic mean estimator',[' * Time:                               ' StringTrim(datestr(now,14))]));
         tic;
         %
         % check if the log posterior kernel should be recomputed
         %
         if RecomputePosteriorKernel==1;
            LogPost = DSGEVARLogPosterior(PostSample,AcceptedDraws,ModeData,lambda,'Geweke''s Harmonic Mean',DSGEModel,CurrINI);
         end;
         [LogMargsMH,CovProb,NWStdErr] = MargLikeModifiedHarmonic(PostSample,LogPost,0,CovProb,DSGEModel);
         ModHarmTime = toc;
         SetLogText(strvcat('Estimation completed!',[' * Estimation time:                    ' SecondsToHoursStr(ModHarmTime) ' (hrs:min:sec)']));
         MargLikeEstimator = 'Geweke''s modified harmonic mean estimator (median)';
         LogMargLike = median(LogMargsMH(1,2:size(LogMargsMH,2)));
      elseif MarginalLikelihoodValue==3;
         SetLogText(strvcat('Computing Sims-Waggoner-Zha''s modified harmonic mean estimator',[' * Time:                               ' StringTrim(datestr(now,14))]));
         tic;
         %
         % check if the log posterior kernel should be recomputed
         %
         if RecomputePosteriorKernel==1;
            LogPost = DSGEVARLogPosterior(PostSample,AcceptedDraws,ModeData,lambda,'SWZ''s Harmonic Mean',DSGEModel,CurrINI);
         end;
         [LogMargsSWZ,qL,LBound,NWStdErr] = MargLikeSWZModifiedHarmonic(PostSample,LogPost,NumPostDraws,90,0,ModeData,[],DSGEModel,CurrINI);
         ModHarmTime = toc;
         SetLogText(strvcat('Estimation completed!',[' * Estimation time:                    ' SecondsToHoursStr(ModHarmTime) ' (hrs:min:sec)']));
         MargLikeEstimator = 'Sims-Waggoner-Zha''s modified harmonic mean estimator';
         LogMargLike = LogMargsSWZ(1,2);
      elseif MarginalLikelihoodValue==4;
         %
         % check if we should temporarily change the state of the random number generators
         %
         if DSGEModel.RandomNumberValue==1;
            StateValue = randn('state');
            randn('state',0);
         end;
         SetLogText(strvcat('Computing the Chib and Jeliazkov''s estimator',[' * Time:                               ' StringTrim(datestr(now,14))]));
         tic;
         if RecomputePosteriorKernel==1;
            LogPost = DSGEVARLogPosterior(PostSample(NumBurnin+1:NumPostDraws,:),AcceptedDraws(NumBurnin+1:NumPostDraws),ModeData,lambda,'Chib & Jeliazkov',DSGEModel,CurrINI);
         end;
         TmpDSGEModel = DSGEModel;
         TmpDSGEModel.SequentialML = 0;
         %
         LogMargsCJ = PlotMargLikeChibJeliazkov(PostSample,LogPost,NumBurnin,CurrChain,TmpDSGEModel,CurrINI);
         clear('TmpDSGEModel');
         CJTime = toc;
         SetLogText(strvcat('Estimation completed!',[' * Estimation time:                    ' SecondsToHoursStr(CJTime) ' (hrs:min:sec)']));
         if DSGEModel.RandomNumberValue==1;
            randn('state',StateValue);
         end;
         MargLikeEstimator = 'Chib and Jeliazkov''s estimator';
         LogMargLike = LogMargsCJ(2);
      end;
   else;
      SetLogText(strvcat('Unable to locate posterior draws',' '));
      return;
   end;
elseif DSGEModel.PosteriorSampler==8;
   %
   % 3. Number of tempering stages
   %
   TempStagesStr = get(controls.posterior.temperingstages,'String');
   Ntau = str2num(StringTrim(strrep(TempStagesStr(DSGEModel.TemperingStagesValue,:),',','')));
   %
   % load marginal likelihood from file
   %
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(Ntau) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   PSData = load(PostSampleFile);
   LogMargLike = PSData.LogMargLikeSMC;
   %
   % remember LogMargLikeSystemLaplace
   %
   MargLikeEstimator = 'SMC estimator';
   MargLikeEstValue = 4;
   NumDraws = NumPostDraws;
elseif DSGEModel.PosteriorSampler==9;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   PSData = load(PostSampleFile);
   LogMargLike = PSData.LogMargLikeSMC;
   %
   % remember LogMargLikeSystemLaplace
   %
   MargLikeEstimator = 'SMC estimator';
   MargLikeEstValue = 4;
   NumDraws = NumPostDraws;
elseif DSGEModel.PosteriorSampler==10;
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\is\IS-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PostModeEstimator) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' num2str(CurrChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumPostDraws,'%0.0f') '.mat']);
   PSData = load(PostSampleFile);
   LogMargLike = PSData.LogMargLikeIS;
   StErrLogMargLike = ParData.StErrLogMargLikeIS;
   %
   % remember LogMargLikeSystemLaplace
   %
   MargLikeEstimator = 'IS estimator';
   MargLikeEstValue = 5;
   NumDraws = NumPostDraws;
end;
%
% print results to text-file
%
if MargLikeEstValue==1;
   %
   % write results to a text file
   %
   [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
   %
   TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ModifiedHarmonicMean-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' int2str(NumDraws) '.' int2str(NumPostDraws) '.txt']);
   fid = fopen(TextFile,'wt');
   prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
   %
   fprintf(fid,'********************************************************************************\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'********************************************************************************\n\n');
   fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
   fprintf(fid,'Number of used posterior draws:   %0.0f\n',NumDraws);
   fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
   if NumChains>1;
      fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
   end;
   fprintf(fid,'lambda:                           %0.4f\n',lambda);
   fprintf(fid,'Number of lags:                   %0.0f\n',p);
   %
   % sample information
   %
   fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
   fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
   %
   % Geweke's modified harmonic mean estimator
   %
   prt_str = 'Geweke''s Harmonic Mean';
   top_str = '                      ';
   std_str = 'Num. standard errors: ';
   for i=1:length(CovProb);
      top_str = [top_str '        ' num2str(CovProb(i)*100,'%0.0f') ' percent'];
      prt_str = [prt_str '   ' prt_val];
      std_str = [std_str '   ' prt_val];
   end;
   TopStr = MultiCharStr('=',length(top_str));
   MidStr = MultiCharStr('-',length(top_str));
   fprintf(fid,[TopStr '\n']);
   fprintf(fid,'                              Coverage probabilities for the estimator\n');
   fprintf(fid,[top_str '\n']);
   fprintf(fid,[MidStr '\n']);
   fprintf(fid,[prt_str '\n'],LogMargsMH(size(LogMargsMH,1),2:size(LogMargsMH,2))');
   fprintf(fid,[std_str '\n'],NWStdErr');
   fprintf(fid,[MidStr '\n']);
   fprintf(fid,['Mean:                    ' prt_val '\n'],sum(LogMargsMH(1,2:size(LogMargsMH,2)))/length(CovProb));
   fprintf(fid,['Median:                  ' prt_val '\n'],median(LogMargsMH(1,2:size(LogMargsMH,2))));
   fprintf(fid,[TopStr '\n\n']);
   %
   % Some information about the filename and its location
   %
   fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
   fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
   fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
   %
   fclose(fid);
elseif MargLikeEstValue==2;
   NumBurnin = OldNumBurnin;
   %
   % retrieve estimation sample start year and period
   %
   [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
   %
   TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ChibJeliazkov-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
   fid = fopen(TextFile,'wt');
   prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
   %
   fprintf(fid,'********************************************************************************\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'********************************************************************************\n\n');
   fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
   fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
   if NumChains>1;
      fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
   end;
   fprintf(fid,'lambda:                           %0.4f\n',lambda);
   fprintf(fid,'Number of lags:                   %0.0f\n',p);
   %
   % sample information
   %
   fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
   fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
   fprintf(fid,'==========================================================\n');
   fprintf(fid,'                          point estimate        std. error\n');
   fprintf(fid,'----------------------------------------------------------\n');
   fprintf(fid,['Chib & Jeliazkov         ' prt_val '   ' prt_val '\n'],LogMargsCJ(1,2),LogMargsCJ(1,3));
   fprintf(fid,'==========================================================\n\n');
   %
   % Some information about the filename and its location
   %
   fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
   fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
   fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
   %
   fclose(fid);
elseif MargLikeEstValue==3;
   %
   % write results to a text file
   %
   [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
   %
   TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ModifiedHarmonicMean-SWZ-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' int2str(NumDraws) '.' int2str(NumPostDraws) '.txt']);
   fid = fopen(TextFile,'wt');
   prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
   %
   fprintf(fid,'********************************************************************************\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'********************************************************************************\n\n');
   fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
   fprintf(fid,'Number of used posterior draws:   %0.0f\n',NumDraws);
   fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
   if NumChains>1;
      fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
   end;
   fprintf(fid,'lambda:                           %0.4f\n',lambda);
   fprintf(fid,'Number of lags:                   %0.0f\n',p);
   %
   % sample information
   %
   fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
   fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
   %
   % SWZ's modified harmonic mean estimator
   %
   fprintf(fid,'===============================================================\n');
   fprintf(fid,'                              point estimate        std. error\n');
   fprintf(fid,'---------------------------------------------------------------\n');
   fprintf(fid,['Sims, Waggoner & Zha         ' prt_val '   ' prt_val '\n'],LogMargsSWZ(1,2),NWStdErr);
   fprintf(fid,'---------------------------------------------------------------\n');
   fprintf(fid,['Lower Bound of log posterior kernel:           ' prt_val ' \n'],LBound);
   fprintf(fid,['Fraction of iid draws from elliptical > lower bound:  %0.6f \n'],qL);
   fprintf(fid,['Fraction of posterior draws > lower bound:            %0.6f \n'],PValue/100);
   fprintf(fid,'===============================================================\n\n');
   fprintf(fid,'NOTE: The "fraction of iid draws from elliptical > lower bound" notation refers to the fraction of\n      the parameter draws which have a log posterior kernel greater than the lower bound. The "fraction\n      of posterior draws > lower bound" notation also refers to the log posterior kernel values.\n\n');
   %
   fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
   fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
   fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
   %
   fclose(fid);
elseif MargLikeEstValue==4;
   %
   % write results to a text file
   %
   [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
   %
   TextFile = strrep(strrep(PostSampleFile,'smc\','MarginalLikelihood-'),'.mat',['-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
   fid = fopen(TextFile,'wt');
   prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
   %
   fprintf(fid,'********************************************************************************\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'********************************************************************************\n\n');
   fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
   if NumChains>1;
      fprintf(fid,'SMC chain number:                 %0.0f\n',CurrChain);
   end;
   fprintf(fid,'lambda:                           %0.4f\n',lambda);
   fprintf(fid,'Number of lags:                   %0.0f\n',p);
   %
   % sample information
   %
   fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
   fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
   %
   % Marginal likelihod estimate
   %
   fprintf(fid,'    Log marginal likelihood estimate = %0.6f \n\n',LogMargLike);
   %
   % Some information about the filename and its location
   %
   fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
   fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
   fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
   %
   fclose(fid);
elseif MargLikeEstValue==5;
   %
   % write results to a text file
   %
   [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
   %
   TextFile = strrep(strrep(PostSampleFile,'is\','MarginalLikelihood-'),'.mat',['-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
   fid = fopen(TextFile,'wt');
   prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
   %
   fprintf(fid,'********************************************************************************\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'********************************************************************************\n\n');
   fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
   if NumChains>1;
      fprintf(fid,'Importance sampling chain number: %0.0f\n',CurrChain);
   end;
   fprintf(fid,'lambda:                           %0.4f\n',lambda);
   fprintf(fid,'Number of lags:                   %0.0f\n',p);
   %
   % sample information
   %
   fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
   fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
   %
   % Marginal likelihood estimate
   %
   fprintf(fid,'    Log marginal likelihood estimate = %0.6f \n',LogMargLike);
   fprintf(fid,'    Numerical standard error         = %0.6f \n\n',StErrLogMargLike);
   %
   % Some information about the filename and its location
   %
   fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
   fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
   fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
   %
   fclose(fid);
end;
%
% save results to mat-file
%
MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-MarginalLikelihood-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.PosteriorSampler) '.' int2str(MargLikeEstValue) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumDraws) '.' int2str(NumPostDraws) '.mat']);
if (RecomputeML==1)|(FileExist(MatFile)==0);
   save(MatFile,'LogMargLike','LogMargLikeSystemLaplace','MargLikeEstimator');
end;
if LogMargLikeSystemLaplace==0;
   SetLogText(strvcat(' ',['Marginal likelihood estimator: ' MargLikeEstimator],['Log ML:                        ' num2str(LogMargLike,'%0.4f')],'Output written to:',['Mat file:  ' GetFilename(MatFile)],['Directory: ' GetPath(MatFile)],' '));
else;
   SetLogText(strvcat(' ',['Marginal likelihood estimator: ' MargLikeEstimator],['Log ML:                        ' num2str(LogMargLike,'%0.4f')],['Log system prior ML (Laplace): ' num2str(LogMargLikeSystemLaplace,'%0.4f')],' ','Output written to:',['Mat file:  ' GetFilename(MatFile)],['Directory: ' GetPath(MatFile)],' '));
end;
Status = 1;

%
% end of SequentialMarginalLikelihoodDSGEVAR.m
%
