function [SelMode,Sellambdai] = DSGEVARSMCDataTemperingPosteriorSampler(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai)
% DSGEVARSMCDataTemperingPosteriorSampler: Runs the sequential Monte Carlo with data tempering algorithm
%                                          to sample from the posterior distribution of the parameters of
%                                          the DSGE model through the lense of a DSGE-VAR.
%
% USAGE:
%
%       [SelMode,Sellambdai] = DSGEVARSMCDataTemperingPosteriorSampler(DSGEModel,CurrINI,maingui,controls,...
%                                                                      SelectedMode,lambdai)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure), with initialization information.
%
%                  maingui (handle) to the YADA main dialog.
%
%                  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% OPTIONAL INPUT:  SelectedMode (integer) which is equal to 1 for the DSGE model posterior
%                            mode, 2 for the DSGE-VAR model marginal posterior mode, and 3
%                            for the DSGE-VAR model joint posterior mode.
%
%                  lambdai (positive scalar) with the value of the lambda hyperparameter
%                            for the DSGE-VAR.
%
% OPTIONAL OUTPUT: SelMode (integer) which is equal to 1 for the DSGE model posterior
%                            mode, 2 for the DSGE-VAR model marginal posterior mode, and 3
%                            for the DSGE-VAR model joint posterior mode.
%
%                  Sellambdai (positive scalar) with the value of the lambda hyperparameter
%                            for the DSGE-VAR.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 16, 2018.
%                        This version: March 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-10-2018: Began the real work on the function.
%
% * 22-10-2018: Continued work on the function from the end of the correction step.
%
% * 25-10-2018: Made sure that the name of the prior draws file reflects using a system prior.
%
% * 10-12-2019: Took into account that the DSGEModel.ModelSolverFile now requires the three additional
%               input variables "DSGEModel.StateVariableNames", "DSGEModel.StateEquationNames" and 
%               "DSGEModel.StateShockNames".
%
% * 08-01-2020: Updated the documentation.
%
% * 02-04-2020: Took into account that "ComputeSystemPriorMode" takes the input variable "IsAdaptiveLearning".
%
% * 25-05-2020: Took into account that "DSGEPriorSampling" and "DSGESystemPriorSampling" take the input variable
%               "IsAdaptiveLearning".
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 13-01-2022: Took DSGEModel.ShowEstimationLog into account.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed additional eval calls to feval.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
% * 15-03-2024: Changed the lower vertical position of the TextGUI function from 32 pixels to 50.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
if nargin==4;
   SelectedMode = [];
   lambdai = [];
end;
if nargout==2;
   SelMode = SelectedMode;
   Sellambdai = lambdai;
end;
%
% clean up any error data
%
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
% check which posterior mode estimates to choose between
%
PostModeStr = '';
DSGEModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
if FileExist(DSGEModeFile)==1;
   PostModeStr = 'DSGE model posterior mode';
end;
if VerifyDSGEVARMarginalMode(DSGEModel)==1;
   PostModeStr = strvcat(PostModeStr,'DSGE-VAR marginal posterior mode');
end;
if VerifyDSGEVARJointMode(DSGEModel)==1;
   PostModeStr = strvcat(PostModeStr,'DSGE-VAR joint posterior mode');
end;
if isempty(SelectedMode)==1;
   if size(PostModeStr,1)>1;
      [action,SelectedMode] = SelectionDlg(PostModeStr,1,'Select the posterior mode estimator:','Estimator Selection','DSGE-VAR Posterior Sampling','','',CurrINI);
   else;
      action = 'OK';
      SelectedMode = 1;
   end;
   if strcmp(lower(action),'ok')==0;
      return;
   end;
end;
%
if nargout==2;
   SelMode = SelectedMode;
end;
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% now check which lambda to use
%
if strcmp(StringTrim(PostModeStr(SelectedMode,:)),'DSGE model posterior mode')==1;
   %
   % we use the DSGE model estimator
   %
   PostModeEstimator = 1;
   lambda = GetLambda(DSGEModel);
   Files = '';
   LambdaValuesStr = '';
   for i=1:length(lambda);
      LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
   end;
   DefaultValue = length(lambda);
elseif strcmp(StringTrim(PostModeStr(SelectedMode,:)),'DSGE-VAR marginal posterior mode')==1;
   %
   % marginal posterior mode
   %
   PostModeEstimator = 2;
   [Status,lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
   if length(lambda)>1;
      MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
      [LambdaValuesStr,DefaultValue] = GetLambdaSelectionData(MatFile,lambda);
   end;
elseif strcmp(StringTrim(PostModeStr(SelectedMode,:)),'DSGE-VAR joint posterior mode')==1;
   %
   % joint posterior mode
   %
   PostModeEstimator = 3;
   [Status,lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
   if length(lambda)>1;
      MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
      [LambdaValuesStr,DefaultValue] = GetLambdaSelectionData(MatFile,lambda);
   end;
end;
%
% now select a value for lambda
%
if isempty(lambdai)==1;
   if length(lambda)>1;
      [action,value] = SelectionDlg(LambdaValuesStr,DefaultValue,'Select the value of the lambda hyperparameter:','Lambda Selection','DSGE-VAR Posterior Sampling','','',CurrINI);
   else;
      action = 'OK';
      value = 1;
   end;
   if strcmp(lower(action),'ok')==0;
      return;
   end;
else;
   value = (lambda==lambdai);
   if length(value)>1;
      value = value.*(1:length(value));
      value = value(value>0);
   end;
   if (isempty(value)==1)|(value==0);
      SetLogText(['The current DSGE model doesn''t support lambda=' num2str(lambdai,'%0,4f') ' for DSGE-VARs';]);
      return;
   end;
end;
%
if DSGEModel.ShowEstimationLog==1;
   set(controls.about,'UserData',0);
   drawnow;
   oldtab = get(controls.tab1,'UserData');
   set(maingui,'CurrentObject',controls.tab6);
   YADAGUI('tab');
   if MatlabNumber>=7;
      uicontrol(controls.tab6);
   end;
   set(controls.about,'UserData',1);
   drawnow;
end;
%
% check if we should add some dividing line to the log-text control
%
if isempty(CurrINI.Sequence)==1;
   LogText = get(controls.estimation_log,'String');
   if isempty(LogText)==0;
      SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
   end;
end;
%
% determine the selected value of lambda
%
if isempty(lambdai)==1;
   lambdai = lambda(value);
end;
if nargout==2;
   Sellambdai = lambdai;
end;
%
% determine the mode file
%
if PostModeEstimator==1;
   ModeFile = DSGEModeFile;
else;
   ModeFile = StringTrim(Files(value,:));
end;
%
% First we load the posterior mode estimates
%
SetLogText(strvcat(' ','Initializing posterior sampling using sequential Monte Carlo','with data tempering...',' * Attempting to load posterior mode data file...'));
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==1;
   SetLogText(strvcat('   STATUS: OK',' * Attempting to load DSGE model data file...'));
else;
   SetLogText(strvcat(' ',['Unable to load the file "' GetFilename(ModeFile) '".']));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
phiMode = ModeData.phiMode;
NumParam = length(phiMode);
%
% load additional information about the model.
%
thetaIndex = ModeData.thetaIndex;
UniformBounds = ModeData.UniformBounds;
LowerBound = ModeData.LowerBound;
thetaPositions = ModeData.thetaPositions;
thetaDist = ModeData.thetaDist;
PriorDist = ModeData.PriorDist;
ModelParameters = ModeData.ModelParameters;
ParameterNames = ModeData.ParameterNames;
thetaMode = ModeData.thetaMode;
theta = ModeData.theta;
thetaInit = theta;
phi = ModeData.phi;
if isfield(ModeData,'LogMargLikeSystemLaplace')==1;
   LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
else;
   LogMargLikeSystemLaplace = 0;
end;
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ParameterNames.additional)==0);
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   %
   % ask if posterior distribition of parameters that are updated should be computed
   %
   if isempty(CurrINI.Sequence)==1;
      txt = ['The model "' DSGEModel.NameOfModel '" has parameters that are a function of the estimated parameters. Should YADA compute the posterior distribution for these additional parameters?'];
      answer = Query(txt,'question',200,'DSGE Posterior Sampling - Additional Parameters',500,'no',CurrINI);
      drawnow;
   else;
      answer = CurrINI.Sequence.QueryAnswers{2};
   end;
   if strcmp(lower(answer),'yes')==1;
      CalcUpdatePosterior = 1;
   else;
      CalcUpdatePosterior = 0;
   end;
else;
   CalcUpdatePosterior = 0;
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   IsSystemPrior = 1;
else;
   IsSystemPrior = 0;
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
ErrorFile = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
end;
%
% Test if there are any problems
%
firstUpdatePos = 0;
lastUpdatePos = 0;
firstInitializePos = 0;
lastInitializePos = 0;
if isempty(ErrorStr)==1;
   if DSGEModel.UseOwnInitialState==1;
      if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
         DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
      else;
         DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
      end;
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
      if MatlabNumber>=8.0;
         try;
            firstUpdatePos = length(fieldnames(ModelParameters))+1;
            ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
            lastUpdatePos = length(fieldnames(ModelParameters));
         catch ME;
            ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError(ME)];
            ErrorFile = DSGEModel.UpdateParameterFile;
         end;
      else;
         try;
            firstUpdatePos = length(fieldnames(ModelParameters))+1;
            ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
            lastUpdatePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError];
            ErrorFile = DSGEModel.UpdateParameterFile;
         end;
      end;
   end;
end;
%
% Check if we should exit
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
% Try to solve the DSGE model at the posterior mode
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE model has a unique convergent','   solution for posterior mode parameter values...'));
if DSGEModel.ModelSolver==1;
   SolverStr = 'AiM';
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein';
   [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Gensys';
   [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==4;
   SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
if mcode~=1;
   %
   % There doesn't exist a unique convergent solution for the initial parameter values.
   %
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" The posterior mode values are no longer valid.'];
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% do it again to measure the time
%
tic;
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==4;
   [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
SolveTime = toc;
if isempty(DSGEModel.UnitRootStates)==1;
   F1 = F;
else;
   KeepPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
   F1 = F(KeepPos,KeepPos);
end;
if (((max(abs(eig(F1)))>=1)|(IsAlmostEqual(max(abs(eig(F1))),1)==1))&(DSGEModel.AllowUnitRoot==0))|((isempty(DSGEModel.UnitRootStates)==1)&(DSGEModel.AllowUnitRoot==1)&(max(abs(eig(F1)))>1));
   SetLogText(strvcat('   STATUS: Non-stationary state equation',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The largest eigenvalue of the state equation is ' num2str(max(abs(eig(F1)))) '. The state equation is therefore non-stationary. The posterior mode values are no longer valid.'];
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check the measurement equation file for internal validity
%
if isempty(ErrorStr)==1;
   %
   % add the values of F and B0 to the ModelParameters.YADA field
   %
   [B0Alt,KeepVar] = RemoveRedundantColumns(B0);
   ModelParameters.YADA.F = F;
   ModelParameters.YADA.B0 = B0Alt;
   SetLogText(strvcat('   STATUS: OK',['   DSGE model solver:      ' SolverStr],['   Maximum absolute error: ' num2str(MaxAbsError)],['   Solution time:          ' num2str(SolveTime) ' seconds'],[' * Checking ' GetName(DSGEModel.MeasurementEquationFile) '...']));
   if MatlabNumber>=8.0;
      try;
         [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
      catch ME;
         ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
         ErrorFile = DSGEModel.MeasurementEquationFile;
      end;
   else;
      try;
         [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
      catch;
         ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
         ErrorFile = DSGEModel.MeasurementEquationFile;
      end;
   end;
end;
%
% checking the system prior file if it exists
%
if (IsSystemPrior==1)&(isempty(ErrorStr)==1);
   SetLogText(strvcat('   STATUS: OK',' * Checking if the system prior can be calculated...'));
   if MatlabNumber>=8.0;
      try;
         lnSP = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);
      catch ME;
         ErrorStr  = ['Unable to calculate the log of the system prior. Message caught is: ' YADALastError(ME)];
         ErrorFile = DSGEModel.SystemPriorFile;
         SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
         if isempty(CurrINI.Sequence)==1;
            About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
            drawnow;
         end;
         return;
      end;
   else;
      try;
         lnSP = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);
      catch;
         ErrorStr  = ['Unable to calculate the log of the system prior. Message caught is: ' YADALastError];
         ErrorFile = DSGEModel.SystemPriorFile;
         SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
         if isempty(CurrINI.Sequence)==1;
            About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
            drawnow;
         end;
         return;
      end;
   end;
   SetLogText(strvcat('   STATUS: OK',' * Checking if the system prior mode has been calculated...'));
   SystemPriorModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
   if FileExist(SystemPriorModeFile)==1;
      SystemPriorModeData = load(SystemPriorModeFile);
      %
      % we only need the Laplace approximation of the log marginal likelihood of the system prior
      %
      LogMargLikeSystemLaplace = SystemPriorModeData.LogMargLikeSystemLaplace;
      SetLogText(strvcat('   STATUS: OK',' * Checking if prior draws are available from the system prior...'));
   else;
      %
      % we need to first compute the system prior mode
      %
      [SystemPriorStatus,LogMargLikeSystemLaplace] = ComputeSystemPriorMode(ModeData.theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,[firstUpdatePos lastUpdatePos firstInitializePos lastInitializePos],0,AIMData,controls,DSGEModel,CurrINI);
      if SystemPriorStatus==0;
         SetLogText('Prior mode estimation has been aborted.');
         return;
      end;
   end;
   %
   % checking if the system prior draws are available
   %
   PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
   if FileExist(PriorDrawsFile)==1;
      PDData = load(PriorDrawsFile);
      if isfield(PDData,'LogMargsMH')==1;
         SPLogMargsMH = PDData.LogMargsMH;
         SPNWStdErrMH = PDData.NWStdErrMH;
         SPCovProb = PDData.CovProb;
         if LogMargLikeSystemLaplace==0;
            LogMargLikeSystemLaplace = PDData.LogMargLikeSystemLaplace;
         end;
         SetLogText('   STATUS: Data available');
      else;
         SPLogMargsMH = [];
         SPNWStdErrMH = [];
         SPCovProb = [];
         SetLogText('   STATUS: Data unavailable');
      end;
   else;
      SPLogMargsMH = [];
      SPNWStdErrMH = [];
      SPCovProb = [];
      SetLogText('   STATUS: Data unavailable');
   end;
else;
   LogMargLikeSystemLaplace = 0;
   SPLogMargsMH = [];
   SPNWStdErrMH = [];
   SPCovProb = [];
end;
%
% Check if we should exit
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check if we can compute the DSGE-VAR prior
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE-VAR prior can be calculated...'));
%
% initialize dimensions
%
p = DSGEModel.BVARLags;
k = DSGEModel.k;
n = DSGEModel.n;
npk = (n*p)+k;
%
% create sample dates
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Check if we should adjust this based on p, the lag order
%
IVAdjusted = 0;
if FirstPeriod+KalmanFirstObservation-1<p+1;
   KalmanFirstObservation = p+2-FirstPeriod;
   IVAdjusted = 1;
end;
%
% actual sample that we will use
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,KalmanFirstObservation-1);
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
DSGEModel.KalmanFirstObservation = KalmanFirstObservation;
%
% get the data
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
T = size(y,2);
if k>0;
   DetProductMoments = CalcDeterministicProductMoments(x,X);
else;
   DetProductMoments = [];
end;
%
% check if we can compute the prior
%
[Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
if PriorStatus==0;
   SetLogText(strvcat('   STATUS: Prior moment matrix GammaYY is not invertible.',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The sample average of the implied non-central population moment matrix GammaYY(theta) is not invertible at the ' StringTrim(PostModeStr(SelectedMode,:)) ' values. This matrix is required for setting up the DSGE-VAR prior. YADA has therefore aborted from psoterior sampling of the DSGE-VAR model.'];
      About(txt,'information','YADA - DSGE Model Problem',300,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the log posterior can be calculated...'));
%
% compute sample product moments
%
GammaHatyy = (1/T)*y*y';
GammaHatyY = (1/T)*y*Y';
GammaHatYY = (1/T)*Y*Y';
%
if isinf(lambdai)==0;
   logGPR = LogGammaProductRatio(n,((1+lambdai)*T)-npk,(lambdai*T)-npk);
else;
   logGPR = [];
end;
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      logPost = logPosteriorPhiDSGEVAR(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
   catch ME;
      ErrorStr = ['Unable to calculate the log posterior of the DSGE-VAR at ' StringTrim(PostModeStr(SelectedMode,:)) ' values for lambda=' num2str(lambdai,'%0.4f') '. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      logPost = logPosteriorPhiDSGEVAR(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
   catch;
      ErrorStr = ['Unable to calculate the log posterior of the DSGE-VAR at ' StringTrim(PostModeStr(SelectedMode,:)) ' values for lambda=' num2str(lambdai,'%0.4f') '. Message caught is: ' YADALastError];
   end;
end;
if ((exist('logPost')==0)|(logPost==1000000))&(isempty(ErrorStr)==1);
   %
   % report error
   %
   SetLogText(strvcat('   STATUS: Unable to compute log posterior at posterior mode',['           for lambda=' num2str(lambdai,'%0.4f')],' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['Unable to compute the log posterior of the DSGE-VAR at the ' StringTrim(PostModeStr(SelectedMode,:)) ' values for lambda=' num2str(lambdai,'%0.4f') '.'];
      About(txt,'information','YADA - DSGE Model Problem',300,500,CurrINI);
      drawnow;
   end;
   return;
elseif isempty(ErrorStr)==0;
   SetLogText(strvcat('   STATUS: Unable to compute log posterior at posterior mode',['           for lambda=' num2str(lambdai,'%0.4f')],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error','Error - Log Posterior',500,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
tic;
logPostMode = -logPosteriorPhiDSGEVAR(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
LogPostTime = toc;
%
SetLogText(strvcat(['   STATUS: OK for lambda=' num2str(lambdai,'%0.4f')],['   Computation time:       ' num2str(LogPostTime) ' seconds'],' * Checking if the VAR parameters can be computed...'));
%
% check if we can identify the structural shocks
%
[PhiMode,SigmaEpsilon,Gyy,GyY,GYY,Status] = DSGEVARParameters(A,H,R,F,B0,lambdai,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
if Status==0;
   SetLogText(strvcat('   STATUS: Unable to compute VAR parameters at posterior mode',['           for lambda=' num2str(lambdai,'%0.4f')],' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['YADA wasn''t able to calculated the VAR parameters at the posterior mode estimates of the DSGE model parameters. The matrix GammaHatYY with a mixture of non-central prior product moments and non-central data product moments is not invertible. Posterior sampling has therefore been aborted.'];
      About(txt,'error','Error - VAR Parameters',300,500,CurrINI);
      drawnow;
   end;
   return;
end;
clear('Gyy','GyY','GYY');
SetLogText(strvcat('   STATUS: OK',' * Checking if the structural shocks can be identified...'));
%
if isinf(lambdai)==0;
   SigmaModeEpsilon = ((lambdai*T)/((lambdai*T)+n+1))*SigmaEpsilon;
else;
   SigmaModeEpsilon = SigmaEpsilon;
end;
[A0Mode,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaModeEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
if IdentifyStatus==1;
   SetLogText('   STATUS: OK');
else;
   SetLogText('   STATUS: Unable to identify structural shocks');
end;
%
% Get the base values number of draws, etc.
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% test sufficient memory
%
SetLogText(' * Testing if memory is sufficient...');
tic;
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      SampleMatrix = zeros(NumPostDraws,NumParam);
   catch ME;
      ErrorStr = ['There was an error creating the matrix to store the posterior draws. YADA caught the following error message: ' YADALastError(ME)];
   end;
else;
   try;
      SampleMatrix = zeros(NumPostDraws,NumParam);
   catch;
      ErrorStr = ['There was an error creating the matrix to store the posterior draws. YADA caught the following error message: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   SampleMatrixTime = toc;
   SetLogText(strvcat(' ','Too many total draws: memory problem','Posterior sampling was aborted.'));
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','YADA - Memory Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
SampleMatrixTime = toc;
SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(SampleMatrixTime) ' seconds']));
clear('SampleMatrix');
%
% The number of Metropolis-Hastings steps for the mutation step
%
SMCNumMHSteps = DSGEModel.SMCNumMHSteps;
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 3. Number of parameter blocks for mutation step
%
NumBlocks = min(DSGEModel.NumParamBlocks,NumParam);
ParamPerBlock = NumParam/NumBlocks;
if ParamPerBlock-floor(ParamPerBlock)>=0.5;
   ParamPerBlock = ceil(ParamPerBlock);
else;
   ParamPerBlock = floor(ParamPerBlock);
end;
%
% 4. Mixing weight for proposal density of mutation step
%
MixedWeightStr = get(controls.posterior.mixeddistweight,'String');
alpha = str2num(StringTrim(MixedWeightStr(DSGEModel.MixedDistWeightValue,:)));
%
% 5. Initial value of scale factor for proposal density of mutation step
%
SMCInitialScaleStr = get(controls.posterior.smcinitialscale,'String');
cstar = str2num(StringTrim(SMCInitialScaleStr(DSGEModel.SMCInitialScaleFactor,:)));
cstarinit = cstar;
%
% 6. target acceptance rate
%
TargetAcceptRateStr = get(controls.posterior.targetacceptancerate,'String');
pstar = str2num(StringTrim(TargetAcceptRateStr(DSGEModel.TargetAcceptanceRateValue,:)));
%
% 7. resampling algorithm for selection step
%
ResamplingAlgorithmStr = get(controls.posterior.resamplingalgorithm,'String');
ResamplingAlgorithmStr = StringTrim(ResamplingAlgorithmStr(DSGEModel.ResamplingAlgorithm,:));
%
% 8. threshold value of effective sample size for resampling
%
ResamplingThresholdStr = get(controls.posterior.resamplingthreshold,'String');
ResamplingThreshold = floor((str2num(StringTrim(strrep(ResamplingThresholdStr(DSGEModel.ResamplingThresholdValue,:),'percent','')))/100)*NumPostDraws);
%
% make sure the output directory is exists
%
stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
if stat~=1;
   if ispc==1;
      SetLogText(strvcat(' ','Couldn''t create the directory:',['  ' DSGEModel.OutputDirectory '\rwm'],'Posterior sampling was aborted.'));
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\rwm". Posterior sampling has therefore been aborted.'];
   else;
      SetLogText(strvcat(' ','Couldn''t create the directory:',['  ' DSGEModel.OutputDirectory '/rwm'],'Posterior sampling was aborted.'));
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/rwm". Posterior sampling has therefore been aborted.'];
   end;
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
if IsSystemPrior==1;
   %
   % determine the number of prior draws
   %
   if NumPostDraws<10000;
      NumPriorDraws = min(100*NumPostDraws,500000);
   elseif (NumPostDraws>=10000)&(NumPostDraws<100000);
      NumPriorDraws = min(10*NumPostDraws,500000);
   else;
      NumPriorDraws = NumPostDraws;
   end;
end;
%
% compute the log posterior kernel at the initial values. Needed as backup value when a prior draw
% does not give a unique convergent solution.
%
[logPostIV,LogLikeIV] = logPosteriorPhiDSGEVAR(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
LogPriorIV = -logPostIV-LogLikeIV;
%
SetLogText(strvcat(' ',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Draws (particles) per chain:        ' int2str(NumPostDraws)],[' * Number of parallel chains:          ' int2str(NumChains)],' ','Correction Step of SMC with Data Tempering Algorithm:',[' * Threshold of effective sample size: ' int2str(ResamplingThreshold)],' ','Selection Step of SMC with Data Tempering Algorithm:',[' * Resampling algorithm:               ' ResamplingAlgorithmStr],' ','Mutation Step of SMC with Data Tempering Algorithm:',[' * Number of MH steps:                 ' int2str(SMCNumMHSteps)],[' * Number of parameter blocks:         ' int2str(NumBlocks)],[' * Mixing weight (alpha):              ' num2str(alpha)],[' * Initial scale factor for proposal:  ' num2str(cstar)],[' * Target acceptance rate:             ' num2str(pstar)],' ',[' * Number of parameters to estimate:   ' num2str(NumParam)],[' * Total number of parameters:         ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Full estimation sample:             ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:       ' int2str(n)],[' * Number of lags:                     ' int2str(p)],[' * Number of exogenous variables:      ' int2str(k)],[' * Hyperparameter lambda:              ' num2str(lambdai,'%0.4f')],' '));
%
% deal with the random number generator: we do this after each stage along the tempreing schedule
%
if DSGEModel.RandomNumberValue==1;
   rand('state',0);
   randn('state',0);
else;
   %
   % we set the state based on the current time
   %
   rand('state',sum(100*clock));
   randn('state',sum(100*clock));
end;
%
% initialize the PostSampleTime variable
%
abort = '';
tic;
PostSampleTime = toc;
T = size(y,2);
tauvec = zeros(1,T+1);
%
% the tempering schedule is selected adaptively based on the effective sample size in the correction step
% as suggested by Durham and Geweke (2014), "Adaptive posterior simulators for m,assively parallel computing
% environments" in Advances in Econometrics, volume 34, chapter 6, 1-44.
%
% this means we need to remove trailing zeros from the schedule as its maximum length is T, but more than
% one data point may be added during a correction step
%
for NChains=1:NumChains;
   %
   % we begin by drawing from the prior distribution
   %
   tic;
   if IsSystemPrior==0;
      PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\smc\PriorDraws-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(NumPostDraws) '-' int2str(NChains) '.' int2str(NumChains) '.mat']);
   else;
      PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SystemPriorDraws-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(NumPostDraws) '-' int2str(NChains) '.' int2str(NumChains) '.mat']);
   end;
   if (DSGEModel.OverwriteDraws==0)&(FileExist(PriorDrawsFile)==1);
      SetLogText(strvcat(' * Data end point:     0',['   Loading prior data: ' GetFilename(PriorDrawsFile)]));
      %
      % load prior draws from file and check if they are sufficient
      %
      PriorDrawsData = load(PriorDrawsFile);
      PostSample = PriorDrawsData.phiDraws';
      thetaPostSample = PriorDrawsData.thetaDraws';
      LogPrior = PriorDrawsData.LogPrior;
      LogLike = PriorDrawsData.LogLike;
      MaxLogPost = max(LogLike+LogPrior);
      SetLogText(strvcat(['   Max logPost kernel: ' num2str(MaxLogPost,'%0.4f')],' '));
   else;
      %
      % need to compute the prior draws
      %
      PostSample = [];
      thetaPostSample = [];
      LogPrior = [];
      LogLike = [];
   end;
   %
   % check if we should draw from the prior
   %
    if isempty(PostSample)==1;
      %
      SetLogText(strvcat(' * Data end point:     0','   Drawing from the prior distribution...'));
      %
      % generate prior draws
      %
      if IsSystemPrior==0;
         thetaDraws = DSGEPriorSampling(PriorDist,thetaDist,LowerBound,NumPostDraws,0,['SMC Prior Sampling - Chain ' int2str(NChains) ' of ' int2str(NumChains)],DSGEModel,CurrINI);
      else;
         thetaDraws = DSGESystemPriorSampling(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,NumPriorDraws,DSGEModel,CurrINI,controls);
         if isempty(thetaDraws)==1;
            abort = 'cancel';
            break;
         end;
         if NumPriorDraws>NumPostDraws;
            %
            % make sure we just take a subsample of the prior draws available
            %
            DrawFreq = round(NumPriorDraws/NumPostDraws);
            UseDraws = (1:DrawFreq:NumPriorDraws);
            if length(UseDraws)+1==NumPostDraws;
               UseDraws = [UseDraws NumPriorDraws];
            elseif length(UseDraws)>NumPostDraws;
               UseDraws = UseDraws(1:NumPostDraws);
            end;
            thetaDraws = thetaDraws(:,UseDraws);
         end;
      end;
      %
      % transform to phiDraws and compute the log posterior kernel values
      %
      SetLogText('   Computing the log prior and log likelihood...');
      phiDraws = zeros(NumParam,NumPostDraws);
      LogPrior = zeros(NumPostDraws,1);
      LogLike = LogPrior;
      for i=1:NumPostDraws;
         phiD = ThetaToPhi(thetaDraws(:,i),thetaIndex,UniformBounds,LowerBound);
         %
         % compute the LogPrior(i) and LogLike(i) values with the log posterior kernel function directly
         %
         [logPost,logLike] = logPosteriorPhiDSGEVAR(phiD,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
         if logPost~=1000000;
            phiDraws(:,i) = phiD;
            LogPrior(i) = -logPost-logLike;
            LogLike(i) = logLike;
         else;
            thetaDraws(:,i) = theta;
            phiDraws(:,i) = phi;
            LogPrior(i) = LogPriorIV;
            LogLike(i) = LogLikeIV;
         end;
         drawnow;
      end;
      %
      % save the prior draws to disk
      %
      MaxLogPost = max(LogLike+LogPrior);
      SetLogText(strvcat(['   Max logPost kernel: ' num2str(MaxLogPost,'%0.4f')],['   Saving prior data:  ' GetFilename(PriorDrawsFile)],' '));
      save(PriorDrawsFile,'phiDraws','thetaDraws','LogPrior','LogLike');
      PostSample = phiDraws';
      thetaPostSample = thetaDraws';
   end;
   %
   % reset the random number generators
   %
   if NChains==1;
      if DSGEModel.RandomNumberValue==1;
         rand('state',0);
         randn('state',0);
      else;
         %
         % we set the state based on the current time
         %
         rand('state',sum(100*clock));
         randn('state',sum(100*clock));
      end;
   end;
   %
   % check if we should show a progress dialog
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' int2str(T) ' observations'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Data value:';
      %
      ProgressStructure.name = ['SMC With Data Tempering - Chain ' int2str(NChains) ' of ' int2str(NumChains)];
      ProgressStructure.CurrINI = CurrINI;
      mywait = ProgressDLG(0,ProgressStructure,600);
      set(mywait,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   else;
      %
      % show a wait dialog
      %
      txt = ['Please wait while YADA runs the sequential Monte Carlo with data tempering posterior sampler for ' int2str(NChains) ' of ' int2str(NumChains) ' chains for lambda=' num2str(lambdai,'%0.4f') ' and p=' int2str(p) '. The total number of posterior draws per chain is ' int2str(NumPostDraws) '. The current chain was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
      WaitHandle = WaitDLG(txt,'information',['SMC With Data Tempering - Chain ' int2str(NChains) ' of ' int2str(NumChains)],500,200,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % now we can start the recursion
   %
   LogMargLikeSMC = 0;
   NumberAcceptedDraws = zeros(T+1,1);
   NumberAcceptedDraws(1) = length(unique(LogPrior));
   ScaleParameter = zeros(T+1,1);
   ScaleParameter(1) = NaN;
   ScaleParameter(2) = cstar;
   EffectiveSampleSize = zeros(T+1,1);
   EffectiveSampleSize(1) = NumPostDraws;
   %
   taun1 = 0;
   nvalue = 1;
   OldLikeValue = ones(NumPostDraws,1);
   while taun1<T;
      %
      % update the progress dialog
      %
      if DSGEModel.ShowProgress==1;
         abort = get(mywait,'UserData');
         if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
            break;
         else;
            if taun1<=T;
               ProgressDLG([taun1/T taun1]);
            end;
         end;
      end;
      %
      % Check if we already have computed the particles for iteration Nn
      %
      SetLogText(strvcat([' * Recursion number:   ' int2str(nvalue)],['   Old data end point: ' int2str(taun1)]));
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMCDT-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(taun1) '.' int2str(nvalue) '-' int2str(NChains) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
      if (DSGEModel.OverwriteDraws==0)&(FileExist(PostSampleFile)==1);
         %
         % load particles from file
         %
         SetLogText(strvcat(['   Loading data:       ' GetFilename(PostSampleFile)]));
         PDData = load(PostSampleFile);
         PostSample = PDData.PostSample;
         thetaPostSample = PDData.thetaPostSample;
         LogPrior = PDData.LogPrior;
         LogLike = PDData.LogLike;
         MaxLogPost = max(LogLike+LogPrior);
         LogMargLikeSMC = PDData.LogMargLikeSMC;
         OldLikeValue = PDData.OldLikeValue;
         tauvec = PDData.tauvec;
         MargLikeInc = PDData.MargLikeInc;
         AcceptedDraws = PDData.AcceptedDraws;
         ESS = PDData.ESS;
         NK = PDData.NK;
         NumTrials = PDData.NumTrials;
         nvalue = nvalue+1;
         taun1 = tauvec(nvalue);
         taun = tauvec(nvalue);
         NumberAcceptedDraws(nvalue) = AcceptedDraws(NumPostDraws);
         EffectiveSampleSize(nvalue) = ESS;
         SetLogText(strvcat(['   New data end point: ' int2str(taun)],['   Eff. sample size:   ' num2str(ESS,'%0.4f')],['   Max logPost kernel: ' num2str(MaxLogPost,'%0.4f')],['   Acceptance rate:    ' num2str(AcceptedDraws(NumPostDraws)/NumPostDraws,'%0.4f')],['   Target rate:        ' num2str(pstar,'%0.4f')]));
         %
         % check if we should deal with random number generators
         %
         if DSGEModel.RandomNumberValue==1;
            %
            % check if the selection step involved resampling
            %
            RSAlgo = [];
            if DSGEModel.ResamplingAlgorithm<=3;
               RSAlgo = rand(NumPostDraws,1);
            elseif DSGEModel.ResamplingAlgorithm==4;
               if NK>0;
                  RSAlgo = rand(NK,1);
               end;
            end;
            %
            % now deal with the mutation step
            %
            % 1. the MH acceptance test
            %
            ATemp = rand(NumTrials,1);
            if alpha==1;
               %
               % 2. for drawing from the proposal
               %
               BTemp = randn(NumTrials,NumParam);
            else;
               BTemp = randn(NumTrials,3*NumParam);
            end;
            %
            % take the number of block draws into account
            %
            if NumBlocks>1;
               CTemp = rand(NumParam,NumPostDraws);
            end;
            clear('RSAlgo','ATemp','BTemp','CTemp');
         end;
      else;
         %
         % compute the particles
         %
         % 1. Correction step: compute weights and reflect the density of the particles
         %                     for current iteration
         %
         % initialize the effective sample size
         %
         ESS = NumPostDraws;
         taun = taun1;
         while (ESS>=ResamplingThreshold)&(taun<T);
            taun = taun+1;
            SetLogText(['   Try data end point: ' int2str(taun)]);
            drawnow;
            %
            % determine the sub-sample data
            %
            ytaun = y(:,1:taun);
            Ytaun = Y(:,1:taun);
            HSampletaun = [HSample(1) HSample(1)+taun-1];
            if isempty(x)==0;
               xtaun = x(:,1:taun);
               Xtaun = X(:,1:taun+p);
               DetProductMomentstaun = CalcDeterministicProductMoments(xtaun,Xtaun);
            else;
               xtaun = [];
               Xtaun = [];
               DetProductMomentstaun = [];
            end;
            %
            % check if we can compute the prior
            %
            [Gammayytaun,GammaYYtaun,GammayYtaun,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSampletaun);
            %
            % we only try this sub-sample if PriorStatus equals 1, otherwise we increase by 1 data point directly
            %
            if PriorStatus==1;
               %
               % compute sample product moments
               %
               GammaHatyytaun = (1/taun)*ytaun*ytaun';
               GammaHatyYtaun = (1/taun)*ytaun*Ytaun';
               GammaHatYYtaun = (1/taun)*Ytaun*Ytaun';
               if isinf(lambdai)==0;
                  logGPRtaun = LogGammaProductRatio(n,((1+lambdai)*taun)-npk,(lambdai*taun)-npk);
               else;
                  logGPRtaun = [];
               end;
               %
               % compute the log posterior kernel with the particles from the previous iteration
               %
               LogLike = zeros(NumPostDraws,1);
               for s=1:NumPostDraws;
                  phiD = PostSample(s,:)';
                  [logPosttaun,logLiketaun] = logPosteriorPhiDSGEVAR(phiD,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,taun,n,p,npk,GammaHatyytaun,GammaHatyYtaun,GammaHatYYtaun,DetProductMomentstaun,HSampletaun,logGPRtaun,CurrINI.OrderQZ,CurrINI.NumWorkers);
                  if logPosttaun~=1000000;
                     LogLike(s) = logLiketaun;
                  end;
               end;
               %
               % compute new weights
               %
               NewLikeValue = exp(LogLike);
               Weights = NewLikeValue./OldLikeValue;
               MargLikeInc = (1/NumPostDraws)*sum(Weights);
               Weights = (1/MargLikeInc)*Weights;
               ESS = NumPostDraws/((1/NumPostDraws)*sum(Weights.*Weights));
            end;
         end;
         LogMargLikeSMC = LogMargLikeSMC+log(MargLikeInc);
         AcceptedDraws = zeros(NumPostDraws,1);
         NK = 0;
         %
         % check if we should first comute the proposal density mean and covariance
         %
         if ESS<=NumParam;
            ExpandWeights = (Weights*ones(1,NumParam))';
            phiTilde = (1/NumPostDraws)*sum(PostSample'.*ExpandWeights,2);
            PostSampleDev = PostSample'-(phiTilde*ones(1,NumPostDraws));
            SigmaphiTilde = (1/NumPostDraws)*(PostSampleDev.*ExpandWeights)*PostSampleDev';
         end;
         %
         % 2. Selection step: resample
         %
         SetLogText(strvcat(['   New data end point: ' int2str(taun)],['   Eff. sample size:   ' num2str(ESS,'%0.4f')],['   New selection:      ' lower(ResamplingAlgorithmStr)]));
         %
         % resample the particles using the user selected algorithm
         %
         if DSGEModel.ResamplingAlgorithm==1;
            %
            % multinomial resampling
            %
            Indexes = MultinomialResampling(Weights,NumPostDraws);
         elseif DSGEModel.ResamplingAlgorithm==2;
            %
            % stratified resampling
            %
            Indexes = StratifiedResampling(Weights,NumPostDraws);
         elseif DSGEModel.ResamplingAlgorithm==3;
            %
            % systematic resampling
            %
            Indexes = SystematicResampling(Weights,NumPostDraws);
         elseif DSGEModel.ResamplingAlgorithm==4;
            %
            % residual resampling
            %
            [Indexes,NK] = ResidualResampling(Weights,NumPostDraws);
         end;
         PostSample = PostSample(Indexes,:);
         thetaPostSample = thetaPostSample(Indexes,:);
         LogPrior = LogPrior(Indexes);
         LogLike = LogLike(Indexes);
         %
         % 3. Mutation step: propagate the particles via fixed block Metropolis-Hastings
         %
         % Compute the importance sampling estimates of the mean and the covariance
         % matrix of the parameters
         %
         if ESS>NumParam;
            phiTilde = (1/NumPostDraws)*sum(PostSample,1)';
            PostSampleDev = PostSample'-(phiTilde*ones(1,NumPostDraws));
            SigmaphiTilde = (1/NumPostDraws)*(PostSampleDev*PostSampleDev');
         end;
         if NumBlocks>1;
            %
            % determine the parameter blocks
            %
            BlockDraws = rand(NumParam,1);
            ParameterBlock = GetBlockingParameterPositions(BlockDraws,ParamPerBlock,NumBlocks);
            for b=1:NumBlocks;
               if rank(SigmaphiTilde(ParameterBlock{b},ParameterBlock{b}))>=length(ParameterBlock{b});
                  CholSigmaphiTilde{b} = chol(SigmaphiTilde(ParameterBlock{b},ParameterBlock{b}))';
               else;
                  CholSigmaphiTilde{b} = diag(sqrt(diag(SigmaphiTilde(ParameterBlock{b},ParameterBlock{b}))));
               end;
               CholDiagSigma{b} = diag(sqrt(diag(SigmaphiTilde(ParameterBlock{b},ParameterBlock{b}))));
            end;
         else;
            if rank(SigmaphiTilde)>=NumParam;
               CholSigmaphiTilde = chol(SigmaphiTilde)';
            else;
               CholSigmaphiTilde = diag(sqrt(diag(SigmaphiTilde)));
            end;
            CholDiagSigma = diag(sqrt(diag(SigmaphiTilde)));
            ParameterBlock = [];
         end;
         %
         SetLogText(['   Mutation step:      ' int2str(NumPostDraws) ' particles']);
         NumTrials = 0;
         for s=1:NumPostDraws;
            %
            % initialize phi
            %
            phi = PostSample(s,:)';
            theta = thetaPostSample(s,:)';
            LogLikeOld = LogLike(s);
            LogPriorOld = LogPrior(s);
            AcceptDraw = 0;
            for m=1:SMCNumMHSteps;
               %
               % check how many blocks there are
               %
               if NumBlocks==1;
                  %
                  % draw from a mixture distribution
                  %
                  phiProp = MixedDistributionDraw(phi,phiTilde,CholSigmaphiTilde,CholDiagSigma,alpha,cstar,NumParam);
                  %
                  % compute the log likelihood and the log prior at the proposal value
                  %
                  [logPostProp,LogLikeProp] = logPosteriorPhiDSGEVAR(phiProp,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,taun,n,p,npk,GammaHatyytaun,GammaHatyYtaun,GammaHatYYtaun,DetProductMomentstaun,HSampletaun,logGPRtaun,CurrINI.OrderQZ,CurrINI.NumWorkers);
                  if logPostProp~=1000000;
                     LogPriorProp = -logPostProp-LogLikeProp;
                     NumTrials = NumTrials+1;
                     if alpha==1;
                        LogQNew = 0;
                        LogQOld = 0;
                     else;
                        LogQNew = MixedDistributionPDF(phiProp,phi,phiTilde,SigmaphiTilde,alpha,cstar);
                        LogQOld = MixedDistributionPDF(phi,phiProp,phiTilde,SigmaphiTilde,alpha,cstar);
                     end;
                     %
                     % compute the probability ratio for the acceptance test
                     %
                     LogMHRatio = LogLikeProp-LogLikeOld+(LogPriorProp-LogPriorOld)+(LogQOld-LogQNew);
                     AcceptProb = min(1,exp(LogMHRatio));
                     TestProb = rand;
                     if TestProb<AcceptProb;
                        %
                        % update the results
                        %
                        AcceptDraw = 1;
                        phi = phiProp;
                        theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
                        LogLikeOld = LogLikeProp;
                        LogPriorOld = LogPriorProp;
                     end;
                  end;
               else;
                  for b=1:NumBlocks;
                     %
                     % initialize proposal
                     %
                     phiProp = phi;
                     phiProp(ParameterBlock{b}) = MixedDistributionDraw(phi(ParameterBlock{b}),phiTilde(ParameterBlock{b}),CholSigmaphiTilde{b},CholDiagSigma{b},alpha,cstar,length(ParameterBlock{b}));
                     %
                     % compute the log likelihood and the log prior at the proposal value
                     %
                     [logPostProp,LogLikeProp] = logPosteriorPhiDSGEVAR(phiProp,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambdai,taun,n,p,npk,GammaHatyytaun,GammaHatyYtaun,GammaHatYYtaun,DetProductMomentstaun,HSampletaun,logGPRtaun,CurrINI.OrderQZ,CurrINI.NumWorkers);
                     if logPostProp~=1000000;
                        LogPriorProp = -logPostProp-LogLikeProp;
                        NumTrials = NumTrials+1;
                        if alpha==1;
                           LogQNew = 0;
                           LogQOld = 0;
                        else;
                           LogQNew = MixedDistributionPDF(phiProp,phi,phiTilde,SigmaphiTilde,alpha,cstar);
                           LogQOld = MixedDistributionPDF(phi,phiProp,phiTilde,SigmaphiTilde,alpha,cstar);
                        end;
                        %
                        % compute the probability ratio for the acceptance test
                        %
                        LogMHRatio = LogLikeProp-LogLikeOld+(LogPriorProp-LogPriorOld)+(LogQOld-LogQNew);
                        AcceptProb = min(1,exp(LogMHRatio));
                        TestProb = rand;
                        if TestProb<AcceptProb;
                           %
                           % update the results
                           %
                           AcceptDraw = 1;
                           phi = phiProp;
                           theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
                           LogLikeOld = LogLikeProp;
                           LogPriorOld = LogPriorProp;
                        end;
                     end;
                  end;
               end;
            end;
            %
            % store the results
            %
            if s>1;
               AcceptedDraws(s) = AcceptedDraws(s-1)+AcceptDraw;
            else;
               AcceptedDraws(s) = AcceptDraw;
            end;
            LogLike(s) = LogLikeOld;
            LogPrior(s) = LogPriorOld;
            PostSample(s,:) = phi';
            thetaPostSample(s,:) = theta';
            drawnow;
         end;
         %
         % save the results to disk
         %
         OldLikeValue = exp(LogLike);
         nvalue = nvalue+1;
         EffectiveSampleSize(nvalue) = ESS;
         tauvec(nvalue) = taun;
         taun1 = tauvec(nvalue);
         MaxLogPost = max(LogLike+LogPrior);
         NumberAcceptedDraws(nvalue) = AcceptedDraws(NumPostDraws);
         SetLogText(strvcat(['   Max logPost kernel: ' num2str(MaxLogPost,'%0.4f')],['   Acceptance rate:    ' num2str(AcceptedDraws(NumPostDraws)/NumPostDraws,'%0.4f')],['   Target rate:        ' num2str(pstar,'%0.4f')],' ','   Saving draws to disk',['   File:               ' GetFilename(PostSampleFile)],['   Time:               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate]));
         save(PostSampleFile,'PostSample','thetaPostSample','LogPrior','LogLike','OldLikeValue','tauvec','ESS','NK','NumTrials','AcceptedDraws','LogMargLikeSMC','MargLikeInc','phiTilde','SigmaphiTilde');
      end;
      %
      % handle adaption of certain parameters
      %
      if taun<T;
         ExponentRate = exp(16*((AcceptedDraws(NumPostDraws)/NumPostDraws)-pstar));
         cstar = cstar*(0.95+(0.1*(ExponentRate/(1+ExponentRate))));
         ScaleParameter(nvalue+1) = cstar;
         SetLogText(strvcat(['   New scale factor:   ' num2str(cstar,'%0.4f')],' '));
      end;
      drawnow;
   end;
   PostSampleTime = toc;
   %
   % close the current waitbar
   %
   if DSGEModel.ShowProgress==1;
      if IsHandle(mywait)==1;
         set(mywait,'UserData','done');
         close(mywait);
         drawnow;
         pause(0.02);
      end;
   else;
      if IsHandle(WaitHandle)==1;
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
      end;
   end;
   %
   % if we have finished properly, save to the following file
   %
   if strcmp(abort,'cancel')==0;
      if NumChains==1;
         SetLogText(strvcat(' ','SMC with data tempering completed!',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Posterior sampling time:            ' SecondsToHoursStr(PostSampleTime) ' (hrs:min:sec)'],[' * Acceptance rate:                    ' num2str(AcceptedDraws(NumPostDraws,1)/NumPostDraws)],[' * Number of tempering stages:         ' int2str(nvalue)]));
      elseif NChains<NumChains;
         SetLogText(strvcat(' ',['SMC with data tempering chain no. ' num2str(NChains,'%0.0f') ' completed.'],[' * Time:                                  ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Posterior sampling time:               ' SecondsToHoursStr(PostSampleTime) ' (hrs:min:sec)'],[' * Acceptance rate:                       ' num2str(AcceptedDraws(NumPostDraws,1)/NumPostDraws)],[' * Number of tempering stages:         ' int2str(nvalue)],' '));
      else;
         SetLogText(strvcat(' ',['SMC with data tempering chain no. ' num2str(NChains,'%0.0f') ' completed.'],[' * Time:                                  ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Posterior sampling time:               ' SecondsToHoursStr(PostSampleTime) ' (hrs:min:sec)'],[' * Acceptance rate:                       ' num2str(AcceptedDraws(NumPostDraws,1)/NumPostDraws)],[' * Number of tempering stages:         ' int2str(nvalue)]));
      end;
      %
      % lower the dimension of certain outputs
      %
      tauvec = tauvec(1:nvalue);
      EffectiveSampleSize = EffectiveSampleSize(1:nvalue);
      NumberAcceptedDraws = NumberAcceptedDraws(1:nvalue);
      ScaleParameter = ScaleParameter(1:nvalue);
      %
      LogPost = LogLike+LogPrior;
      BatchPostSample = PostSample;
      BatchthetaPostSample = thetaPostSample;
      BatchLogPost = LogPost;
      BatchAcceptedDraws = AcceptedDraws;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(NChains) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
      save(PostSampleFile,'PostSample','thetaPostSample','LogLike','LogPrior','LogPost','NumberAcceptedDraws','tauvec','EffectiveSampleSize','ScaleParameter','ResamplingThreshold','pstar','LogMargLikeSMC','AcceptedDraws','PostSampleTime','BatchPostSample','BatchthetaPostSample','BatchLogPost','BatchAcceptedDraws');
      clear('BatchPostSample','BatchthetaPostSample','BatchLogPost','BatchAcceptedDraws');
   else;
      SetLogText('Posterior sampling aborted.');
      break;
   end;
end;
%
% clean up the tmp directory for AiM related error data.
%
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
if strcmp(abort,'cancel')==1;
   return;
end;
%
% check if we have just run 1 chain. if so display certain information.
%
for CurrChain=1:NumChains;
   %
   % make sure that we load the correct results
   %
   if NumChains>1;
      %
      % load the data
      %
      SetLogText(strvcat(' ','Reloading posterior draws data...'));
      if exist('PostSample')==1;
         %
         % make sure the workspace is cleared from the last chain draws before loading the draws
         % from the first chain.
         %
         clear('PostSample','thetaPostSample','LogPost','AcceptedDraws');
      end;
      [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambdai,PostModeEstimator,CurrChain);
      %
      % temporarily remove sequential estimation
      %
      DSGEModel.SequentialML = 0;
      SetLogText(strvcat('Computing summary information about the parameters...',[' * Chain number:                       ' num2str(CurrChain,'%0.0f')],[' * Time:                               ' StringTrim(datestr(now,14))]));
   else;
      NumBurnin = 0;
      SetLogText(strvcat(' ','Computing summary information about the parameters...',[' * Time:                               ' StringTrim(datestr(now,14))]));
   end;
   NumAcceptedDraws = AcceptedDraws(NumPostDraws,1)/NumPostDraws;
   clear('AcceptedDraws');
   %
   % compute summary information about the parameter draws for the original parameters
   %
   tic;
   ParData = ParameterSummary(thetaPostSample);
   ParTime = toc;
   SetLogText(strvcat('Computation completed!',[' * Estimation time:                    ' SecondsToHoursStr(ParTime) ' (hrs:min:sec)']));
   %
   % check if we have parameters to update
   %
   if (FileExist(DSGEModel.UpdateParameterFile)==1)&(CalcUpdatePosterior==1);
      tic;
      SetLogText(strvcat(' ','Preparing posterior values for parameters to update...',[' * Time:                               ' StringTrim(datestr(now,14))]));
      [UpdatePostSample,UpdateMode,UpdateNames] = GetDSGEVARUpdatePosteriorDraws(thetaPostSample,lambdai,PostModeEstimator,ModeFile,NumBurnin,CurrChain,NumChains,0,DSGEModel,CurrINI);
      SetLogText(strvcat('Computing summary information...',[' * Time:                               ' StringTrim(datestr(now,14))]));
      UpdateParData = ParameterSummary(UpdatePostSample);
      UpParTime = toc;
      SetLogText(strvcat('Computation completed!',[' * Total preparation time:             ' SecondsToHoursStr(UpParTime) ' (hrs:min:sec)']));
   else;
      UpdatePostSample = [];
      UpdateParData = [];
      UpdateMode = [];
      UpdateNames = '';
   end;
   %
   % try to estimate the posterior mode of the marginal distribution.
   %
   if DSGEModel.PosteriorDensityValue==1;
      ModeAlgorithm = ' * Algorithm:                          Gaussian';
   elseif DSGEModel.PosteriorDensityValue==2;
      ModeAlgorithm = strvcat(' * Algorithm:                          Silverman-type and','                                       Skld-Roberts correction');
   elseif DSGEModel.PosteriorDensityValue==3;
      ModeAlgorithm = ' * Algorithm:                          Sheather-Jones bandwidth';
   elseif DSGEModel.PosteriorDensityValue==4;
      ModeAlgorithm = ' * Algorithm:                          Bump killing bandwidth';
   end;
   SetLogText(strvcat(' ','Computing the posterior mode of the marginal distribution...',ModeAlgorithm,[' * Time:                               ' StringTrim(datestr(now,14))]));
   tic;
   [thetaMargMode,UpdateMargMode] = GraphDSGEVARPosteriorDensity(thetaPostSample,lambdai,PostModeEstimator,ModeFile,NumBurnin,CurrChain,UpdatePostSample,UpdateMode,UpdateNames,[],[],[],DSGEModel,CurrINI,controls,1);
   ModeTime = toc;
   if isempty(thetaMargMode)==1;
      thetaMargMode = thetaMode;
      UpdateMargMode = UpdateMode;
      SetLogText(strvcat('Estimation aborted.',' * Using the posterior mode of the joint distribution','   of the DSGE model parameters.'));
   else;
      SetLogText(strvcat('Estimation completed!',[' * Estimation time:                    ' SecondsToHoursStr(ModeTime) ' (hrs:min:sec)']));
   end;
   %
   % clear original parameter draws from memory
   %
   clear('thetaPostSample','UpdatePostSample','PostSample','LogPost');
   %
   % save parameter data to file
   %
   MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\smc\MargLikeData-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   save(MargLikeFile,'ParData','thetaMargMode','UpdateParData','UpdateMargMode');
   %
   % load marginal likelihood estimates
   %
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   PSData = load(PostSampleFile);
   ParData.NumResamples = length(PSData.tauvec)-1;
   ParData.LogMargLikeSMC = PSData.LogMargLikeSMC;
   %
   clear('PSData');
   ParData.NumBlocks = NumBlocks;
   ParData.SMCNumMHSteps = SMCNumMHSteps;
   ParData.alpha = alpha;
   ParData.cstar = cstar;
   ParData.cstarinit = cstarinit;
   ParData.pstar = pstar;
   ParData.ResamplingAlgorithmStr = ResamplingAlgorithmStr;
   %
   % check if we have the DSGE model marginal likelihood as well
   %
   DSGEPostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   if FileExist(DSGEPostSampleFile)==1;
      PSData = load(DSGEPostSampleFile);
      ParData.DSGELogMargLikeSMC = PSData.LogMargLikeSMC;
      clear('PSData');
   else;
      ParData.DSGELogMargLikeSMC = [];
   end;
   save(MargLikeFile,'ParData','-APPEND');
   %
   % write sample statistics to file
   %
   PostSampleTextFile = FixFilePath([DSGEModel.OutputDirectory '\SMCPosteriorSamplingSummary-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws)  '.txt']);
   SetLogText(strvcat(' ','Writing posterior sampling summary results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: ' GetFilename(PostSampleTextFile)]));
   PrintDSGEVARPosteriorSamplingSummary(PostSampleTextFile,DSGEModel,CurrINI,ParameterNames,PriorDist,thetaDist,ParData,thetaMargMode,UpdateParData,UpdateMargMode,UpdateNames,NumPostDraws,NumBurnin,[],[],PostSampleTime,NumAcceptedDraws,[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],CurrChain,NumChains,StringTrim(PostModeStr(SelectedMode,:)),thetaInit,[],[],n,p,k,lambdai,IsSystemPrior,LogMargLikeSystemLaplace,SPLogMargsMH,SPCovProb,SPNWStdErrMH);
   %
   % display the results
   %
   if (isempty(CurrINI.Sequence)==1)&(CurrChain==NumChains);
      TextGUI(PostSampleTextFile,['DSGE Posterior Sampling - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
   elseif (CurrChain==NumChains);
      SetLogText(' ');
   end;
end;

%
% end of DSGEVARSMCDataTemperingPosteriorSampler.m
%
