function DoneCalc = DSGEVARPredictiveLikelihood(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,lambda,n,p,npk,HSample,YData,X,SelectedYData,IsOriginal,IsPosterior,IsMarginal,CurrChain,PME,DSGEModel,CurrINI,controls)
% DSGEVARPredictiveLikelihood: Computes the joint or the marginal predictive likelihood for selected subset of variables
%                              for prior or posterior draws of parameters such that the data can be used by the importance
%                              sampling estimator.
%
% USAGE:
%
%       DoneCalc = DSGEVARPredictiveLikelihood(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,lambda,n,p,npk,HSample, ...
%                                              YData,X,SelectedYData,IsOriginal,IsPosterior,IsMarginal,CurrChain, ...
%                                              PME,DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  PhiDraws (matrix) of dimension UsedDraws x n(np+k) with the draws from the
%                           VAR parameters. Each row holds vec(Phi(0) Phi(1) ... Phi(p))'.
%
%                  SigmaDraws (matrix) of dimension UsedDraws x n(n+1)/2 with the unique draws
%                           of the SigmaEpsilon parameters. Each row holds vech(Sigma)'.
%
%                  PhiMode (matrix) of dimension nx(np+k) with fixed values for the Phi parameters
%                           that can be used as replacements for PhiDraws.
%
%                  SigmaModeEpsilon (matrix) of dimension nxn with fixed values of the SigmaEpsilon
%                           parameters that can be used as replacements for SigmaDraws.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  n (integer) with the number of endogenous variables.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  npk (integer) with the number of regressors in the VAR, equal to the number of endogenous
%                           variables times the number of lags plus the number of exogenous variables.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                           are located.
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                           sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  SelectedYData (integer) with the selected set of actuals for the observed variables.
%
%                  IsOriginal (boolean) that takes the value 1 if original values should be predicted and 0 if
%                           annualized values should be predicted.
%
%                  IsPosterior (boolean) that takes the value 1 if the parameter draws are taken from the posterior
%                           distribution and 0 otherwise.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive likelihood should be
%                           computed and 0 if the joint predictive likelihood is the object of interest.
%
%                  CurrChain (integer) for the selected MCMC chain when posterior draws are used.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                           are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                           used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                           when generating the posterior draws.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  controls (structure) with handles to the controls on YADA's GUI window.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 11, 2012.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-05-2012: Finished the original data part of the function.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 21-08-2012: Took into account that an importance sampling estimator is used for the predictive likelihood.
%
% * 19-10-2012: Continued work on the function adding annualized predictive likelihood computations.
%
% * 22-10-2012: Finished the function.
%
% * 23-10-2012: Removed unnecessary input variables.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 20-01-2015: Adapted the function for use with YADA's sequential tools.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 15-06-2015: The function now takes the input variable "SelectedYData".
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 11-05-2022: At the 10th anniversary the function now supports parallel computations.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

DoneCalc = 0;
NumDraws = size(PhiDraws,1);
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
TotalDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
if IsOriginal==1;
   TypeStr = 'Original';
else;
   TypeStr = 'Annualized';
end;
if IsMarginal==1;
   MargStr = 'Marginal';
else;
   MargStr = 'Joint';
end;
SelectedVarStr = ismember((1:n),DSGEModel.PredictionVariables);
SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if isempty(CurrINI.Sequence)==1;
      if ispc==1;
         txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of the ' lower(DataType) ' predictive likelihood has therefore been aborted.'];
      else;
         txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of the ' lower(DataType) ' predictive likelihood has therefore been aborted.'];
      end;
      About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   else;
      if ispc==1;
         DirText = ['"' DSGEModel.OutputDirectory '\predictions"'];
      else;
         DirText = ['"' DSGEModel.OutputDirectory '/predictions"'];
      end;
      SetLogText(['Unable to create the directory: ' DirText]);
   end;
   return;
end;
%
% Get the sub-sample
%
h = size(YData,2);
%
% initialize matrices
%
Y = vec(fliplr(DSGEModel.Y(:,HSample(2)-p+1:HSample(2))));
J = zeros(n*p,n);
J(1:n,:) = eye(n);
k = DSGEModel.k;
Dn = DuplicationMatrix(n);
%
if IsOriginal==1;
   [lnPLmode,PredVars] = logPredLikePriorDSGEVAR(YData,X,h,IsMarginal,DSGEModel,n,p,k,npk,J,Y,PhiMode,SigmaModeEpsilon);
   YDataAnnual = [];
else;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = DSGEModel.Y(:,HSample(2)-2:HSample(2));
      DSGEModel.Frequency = 4;
   else;
      Yinit = DSGEModel.Y(:,HSample(2)-10:HSample(2));
      DSGEModel.Frequency = 12;
   end;
   DSGEModel.Yinit = Yinit;
   DSGEModel.DiffPos = DSGEModel.annual'==DSGEModel.Frequency;
   DSGEModel.LevPos = DSGEModel.annual'==1;
   %
   % no need to make sure that we take NaN's over the historical sample into account appropriately
   % since the DSGE-VAR doesn't support them
   %
   YDataAnnual = Annualize([Yinit YData],DSGEModel.annual,ones(n,1));
   [lnPLmode,PredVars] = logPredLikePriorDSGEVARAnnual(YData,YDataAnnual,X,h,IsMarginal,DSGEModel,n,p,k,npk,J,Y,PhiMode,SigmaModeEpsilon);
end;
%
% we can now run the routine for all draws
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should use parallel computations or not
%
if CurrINI.NumWorkers<2;
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' int2str(NumDraws) ' Draws'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Path Number:';
      %
      ProgressStructure.name = [MargStr ' Predictive Likelihood' PriorHeader];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   else;
      txt = ['Please wait while YADA computes the ' lower(MargStr) ' predictive likelihood of a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',['Predictive Likelihood ' PriorHeader ' - ' TypeStr ' Data'],600,200,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % begin the loop
   %
   abort = '';
   StopEstimation = 0;
   %
   LogPosteriorData = zeros(NumDraws,h);
   %
   for ParamIndex=1:NumDraws;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([ParamIndex/NumDraws ParamIndex]);
         end;
      end;
      %
      SigmaEpsilon = reshape(Dn*SigmaDraws(ParamIndex,:)',n,n);
      if IsPositiveDefinite(SigmaEpsilon)==1;
         Phi = reshape(PhiDraws(ParamIndex,:)',n,npk);
      else;
         SigmaEpsilon = SigmaModeEpsilon;
         Phi = PhiMode;
      end;
      if IsOriginal==1;
         [lnPL,PredVars] = logPredLikePriorDSGEVAR(YData,X,h,IsMarginal,DSGEModel,n,p,k,npk,J,Y,Phi,SigmaEpsilon);
      else;
         [lnPL,PredVars] = logPredLikePriorDSGEVARAnnual(YData,YDataAnnual,X,h,IsMarginal,DSGEModel,n,p,k,npk,J,Y,Phi,SigmaEpsilon);
      end;
      LogPosteriorData(ParamIndex,:) = lnPL';
      drawnow;
   end;
   %
   if StopEstimation==0;
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEVARPredictiveLikelihood-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEVARPredictiveLikelihood-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.mat']);
      end;
      save(file,'LogPosteriorData','PredVars');
   end;
   %
   % close the wait dialog
   %
   if DSGEModel.ShowProgress==1;
      if IsHandle(WaitHandle)==1;
         set(WaitHandle,'UserData','done');
         close(WaitHandle);
         drawnow;
         pause(0.02);
      end;
   else;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   %
   % we can do parallel computations over the parameter draws
   %
   txt = ['Please wait while YADA computes the ' lower(MargStr) ' predictive likelihood of a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Predictive Likelihood ' PriorHeader ' - ' TypeStr ' Data'],600,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   % begin the loop
   %
   StopEstimation = 0;
   %
   LogPosteriorData = zeros(NumDraws,h);
   %
   parfor ParamIndex=1:NumDraws;
      %
      SigmaEpsilon = reshape(Dn*SigmaDraws(ParamIndex,:)',n,n);
      if IsPositiveDefinite(SigmaEpsilon)==1;
         Phi = reshape(PhiDraws(ParamIndex,:)',n,npk);
      else;
         SigmaEpsilon = SigmaModeEpsilon;
         Phi = PhiMode;
      end;
      if IsOriginal==1;
         [lnPL,PredVars] = logPredLikePriorDSGEVAR(YData,X,h,IsMarginal,DSGEModel,n,p,k,npk,J,Y,Phi,SigmaEpsilon);
      else;
         [lnPL,PredVars] = logPredLikePriorDSGEVARAnnual(YData,YDataAnnual,X,h,IsMarginal,DSGEModel,n,p,k,npk,J,Y,Phi,SigmaEpsilon);
      end;
      LogPosteriorData(ParamIndex,:) = lnPL';
      drawnow;
   end;
   %
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEVARPredictiveLikelihood-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' MargStr 'DSGEVARPredictiveLikelihood-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' TypeStr '-' SelectedVarStr '-' int2str(h) '-' int2str(NumDraws) '.mat']);
   end;
   save(file,'LogPosteriorData','PredVars');
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
if StopEstimation==1;
   return;
end;
%
% we made it to the end
%
DoneCalc = 1;

%
% end of DSGEVARPredictiveLikelihood.m
%
