function StructureForData = DSFFDataConstFile
% DSFFDataConstFile: Shows how the data construction output should be provided for YADA. This example is based
%                    on the Del Negro, Giannoni and Schorfheide (2013) which combines the Smets and Wouters (2007,AER)
%                    model with BGG-type of financial frictions.
%
% USAGE:
%
%       StructureForData = DSFFDataConstFile
%
% where StructureForData is a proper structure in matlab. The file is run in matlab via the eval
% function.
%
% INPUTS:          None!
%
% REQUIRED OUTPUT: StructureForData (structure). Any name for this structure will do since YADA renames it internally.
%                  REQUIRED FIELDS:  Y.data              (nxT) data-matrix of the n observable variables that should be evaluated by the
%                                                        Kalman filter
%                                    X.data              (kxT) data-matrix with exogenous data, may be an empty matrix, but must be stated
%                                    Y.names             (n dimensional cellstring) with variable names
%                                    X.names             (k dimensional cellstring) with names for exogenous variables (may be empty)
%                                    sample              (1x4 matrix) containing [start-year start-period end-yer end-period]. Must match
%                                                        the number of observations T.
%                                    frequency           (string) data frequency. Valid entries are: a(nnual), q(uarterly), m(onthly)
%                                                        if invalid frequency is reported, it is set to annual with sample being set
%                                                        to [1 1 T 1]
%                  OPTIONAL FIELDS:  Lambda              (vector) with lambda values to use for the DSGE-VAR models.
%                                    Y.levels            (1xn) vector, where 0 indicates a first differenced variable and 1 a levels
%                                                        variable
%                                    Y.annual            (1xn) vector, where 1 indicates that the variable should be annualized based
%                                                        on the data frequency, while 0 means that it should not be annualized (i.e.,
%                                                        can already be regarded as annualized).
%
% NOTE: and many more fields are supported.
%
%
% NOTICE:
%         * Any valid matlab functions on the path can be used by the function. How you transform any data you read in from
%           a spreadsheet, text file, of matlab mat-file is up to you.
%         * Any file format supported by matlab can be used for your data.
%
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 11, 2014.
%                        This version: January 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-02-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 15-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 15-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% read data from an excel spreadsheet
%
FFData = xlsread([pwd '/example/DelNegroSchorfheide/data/DelNegroSchorfheideFFData.xls']);
T = size(FFData,1);
%
% annualize the federal funds rate
%
FFData(:,5) = 4*FFData(:,5);
%
% annualize the spread between Moody's Baa corporate bond yield over the 10-year treasury note
% yield at constant maturity:
%                              SP = (1/4)*(BaaCorporate-10yearTreasuty)
% accorirdning to Del Negro, Giannoni and Schorfheide (2013), Appendix A
%
FFData(:,8) = 4*FFData(:,8);
%
% make sure that hours in 1992Q3 matches hours from Smets and Wouters (2007, AER) for the same period
%
FFData(:,2) = FFData(:,2)-FFData(114,2)+(-1.400048556);
%
% the spread parametere in the measurement equation should be close mean(FFData(:,8))/4 + 1
%
% transpose the data since YADA prefers it that way
%
StructureForData.Y.data = FFData';
StructureForData.Y.names = {'DY' 'L' 'DW' 'PIY' 'FFR' 'DC' 'DI' 'SP'};
StructureForData.Lambda = [0.25 0.5 0.75 1 1.25 1.5 1.75 2 2.5 3 3.5 4 4.5 5 6 7 8 9 10 25 Inf];
StructureForData.Y.levels = [0 1 0 1 1 0 0 1];
StructureForData.Y.annual = [1 0 1 0 0 1 1 0];
StructureForData.Y.annualscale = [1 0 1 0 0 1 1 0];
StructureForData.X.data = ones(1,T);
StructureForData.X.extradata = ones(1,120);
StructureForData.X.names = {'const'};
StructureForData.sample = [1964 2 2008 3];
StructureForData.frequency = 'q';
StructureForData.percentiles = [2.5 5 15 25 75 85 95 97.5];

%
% end of DSFFDataConstFile.m
%
