function [A,H,R] = DSFFMeasureEqFile(ModelParameters,StateVarNames,VariableNames,XVariableNames,n,r,k)
% DSFFMeasureEqFile: Shows how to set up the measurement equations for the Smets-Wouters (2007, AER) model
%                    with stochastic detrending and with financial frictions, as in Del Negro and
%                    Schorfheide (2013, Handbook of Economic Forecasting).
%
% USAGE:
%
%       [A,H,R] = DSFFMeasureEqFile(ModelParameters,StateVarNames,VariableNames,XVariableNames,n,r,k)
%
%
% REQUIRED INPUTS:  ModelParameters (structure) with fields given by all the model parameters
%                                   and the fields taking scalar values. All model parameters
%                                   includes the parameters to be estimated, the parameters that
%                                   are calibrated, and the parameters that are functions of
%                                   the above.
%
%                   StateVarNames (string matrix) with rows being the names of the variables
%                                   as they have been assigned in the AiM model file. This will
%                                   typically include all state variables, as well as the shocks
%                                   to the state variables (and possibly a constant).
%
%                   VariableNames (string matrix) where each row contains a name of an observed
%                                   variables.
%
%                   XVariableNames (string matrix) where each row contains a name of an exogenous
%                                   variable.
%
%                   n (integer) with the number of observed variables.
%
%                   r (integer) with the number of state variables.
%
%                   k (integer) with the number of exogenous variables.
%
% REQUIRED OUTPUTS: A (kxn) matrix with parameters on the exogenous variables. Notice that k=
%                                   number of exogenous variables, n=number of observed variables
%                                   in the measurement equation.
%
%                   H (rxn) matrix with parameters on the state variables. Here, r=number of state
%                                   variables.
%
%                   R (nxn) matrix with covariances for the measurement errors. This matrix has
%                                   to be positive semi-definite!
%
%
% NOTICE: The measurement equations are expressed as:
%
%             y(t) = A'*x(t) + H'*xi(t) + w(t)
%
%         where y (nx1) is the vector of observed variable, x (kx1) is the vector of exogenous
%         variables, xi (rx1) the vector of state variables, and w (nx1) the vector of measurement
%         errors, with covariance matrix R.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 12, 2014.
%                        This version: January 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-02-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 15-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 15-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% The constant (exogenous)
%
A = zeros(k,n);
A(loc(XVariableNames,'const'),loc(VariableNames,'DY')) = ModelParameters.gammabar;
A(loc(XVariableNames,'const'),loc(VariableNames,'DC')) = ModelParameters.gammabar;
A(loc(XVariableNames,'const'),loc(VariableNames,'DI')) = ModelParameters.gammabar;
A(loc(XVariableNames,'const'),loc(VariableNames,'DW')) = ModelParameters.gammabar;
A(loc(XVariableNames,'const'),loc(VariableNames,'PIY')) = ModelParameters.pibar;
A(loc(XVariableNames,'const'),loc(VariableNames,'FFR')) = 4*ModelParameters.rbar;
A(loc(XVariableNames,'const'),loc(VariableNames,'L')) = ModelParameters.lbar;
A(loc(XVariableNames,'const'),loc(VariableNames,'SP')) = 4*ModelParameters.sbar;

%
% The parameters on the state variables.
%
H = zeros(r,n);
H(loc(StateVarNames,'dy'),loc(VariableNames,'DY')) = 1;
H(loc(StateVarNames,'tau'),loc(VariableNames,'DY')) = 1;
H(loc(StateVarNames,'dc'),loc(VariableNames,'DC')) = 1;
H(loc(StateVarNames,'tau'),loc(VariableNames,'DC')) = 1;
H(loc(StateVarNames,'di'),loc(VariableNames,'DI')) = 1;
H(loc(StateVarNames,'tau'),loc(VariableNames,'DI')) = 1;
H(loc(StateVarNames,'dw'),loc(VariableNames,'DW')) = 1;
H(loc(StateVarNames,'tau'),loc(VariableNames,'DW')) = 1;
H(loc(StateVarNames,'piy'),loc(VariableNames,'PIY')) = 1;
H(loc(StateVarNames,'r'),loc(VariableNames,'FFR')) = 4;
H(loc(StateVarNames,'l'),loc(VariableNames,'L')) = 1;
H(loc(StateVarNames,'Etret1'),loc(VariableNames,'SP')) = 4;
H(loc(StateVarNames,'r'),loc(VariableNames,'SP')) = -4;

%
% The covariance matrix for the measurement errors. Here, we have no measurement errors!
%
R = zeros(n,n);

%
% end of DSFFMeasureEqFile.m
%
