function ModelParameters = UpdateDSFFParameters(ModelParameters)
% UpdateDSFFParameters: Shows how to define new parameters from previously defined parameters
%
% USAGE:
%
%       ModelParameters = UpdateDSFFParameters(ModelParameters)
%
% REQUIRED INPUT:  ModelParameters (structure) with fields corresponding to the parameters
%                                   you need for defining new parameters that should be
%                                   added to the structure.
%
% REQUIRED OUTPUT: ModelParameters (structure) with additions to the input structure.
%
% NOTE: The financial frictions calculations are based on the code provided by Del Negro, Giannoni and
%       Schorfheide (2013), available from Frank Schorfheide's website at the University of Pennsylvania.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 12, 2014.
%                        This version: January 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-02-2014: Finished the function.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 06-02-2017: Fixed a bug in the zetabomegafcn where the denominator was
%               incorrect.
%
% * 07-02-2017: Corrected two sign errors in zetanq.
%
% * 08-02-2017: Added zetantau as output.
%
% * 15-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 15-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% stuff to delete
%
%if nargin==0;
%   ModelParameters.varphi = 5.48;
%   ModelParameters.sigmac = 1.39;
%   ModelParameters.lambda = 0.71;
%   ModelParameters.xiw = 0.73;
%   ModelParameters.sigmal = 1.92;
%   ModelParameters.xip = 0.65;
%   ModelParameters.imathw = 0.59;
%   ModelParameters.imathp = 0.22;
%   ModelParameters.psi = 0.54;
%   ModelParameters.phip = 1.61;
%   ModelParameters.rpi = 2.03;
%   ModelParameters.rho = 0.81;
%   ModelParameters.ry = 0.08;
%   ModelParameters.rdy = 0.22;
%   ModelParameters.pibar = 0.81;
%   ModelParameters.betainv = 0.16;
%   ModelParameters.lbar = -0.1;
%   ModelParameters.gammabar = 0.43;
%   ModelParameters.alpha = 0.19;
%   ModelParameters.spstar = 2;
%   ModelParameters.zetaspb = 0.05;
%   ModelParameters.delta = 0.025;
%   ModelParameters.phiw = 1.5;
%   ModelParameters.gy = 0.18;
%   ModelParameters.epsilonp = 10;
%   ModelParameters.epsilonw = 10;
%   ModelParameters.fbar = 0.0075;
%   ModelParameters.kappae = 0.99;
%end;
%
% Parameter functions related to the steady-state of the Smets and Wouters model
%
ModelParameters.gamma = (ModelParameters.gammabar/100)+1;
ModelParameters.pistar = (ModelParameters.pibar/100)+1;
ModelParameters.beta = 1/((ModelParameters.betainv/100)+1);
ModelParameters.betabar = ModelParameters.beta*(ModelParameters.gamma^(-ModelParameters.sigmac));
ModelParameters.rbar = 100*((ModelParameters.pistar/ModelParameters.betabar)-1);
ModelParameters.sprd = (1+(ModelParameters.spstar/100))^(1/4);
ModelParameters.sbar = 100*log(ModelParameters.sprd);
ModelParameters.rstar = ModelParameters.pistar/ModelParameters.betabar;
ModelParameters.restar = ModelParameters.sprd*ModelParameters.rstar;
%
% steady-state rental rate of capital
%
ModelParameters.rkstar = (ModelParameters.sprd/ModelParameters.betabar)+ModelParameters.delta-1;
%
% steady-state real wages
%
ModelParameters.w = (ModelParameters.alpha^ModelParameters.alpha*(1-ModelParameters.alpha)^(1-ModelParameters.alpha)/(ModelParameters.phip*ModelParameters.rkstar^ModelParameters.alpha))^(1/(1-ModelParameters.alpha));
%
% steady-state investment-capital ratio
%
ModelParameters.ik = ModelParameters.gamma+ModelParameters.delta-1;
%
% steady-state labor-capital ratio
%
ModelParameters.lk = ((1-ModelParameters.alpha)/ModelParameters.alpha)*(ModelParameters.rkstar/ModelParameters.w);
%
% steady-state capital-output ratio
%
ModelParameters.ky = ModelParameters.phip*(ModelParameters.lk^(ModelParameters.alpha-1));
%
% steady-state investment-output ratio
%
ModelParameters.iy = ModelParameters.ik*ModelParameters.ky;
%
% steady-state consumption-output ratio
%
ModelParameters.cy = 1-ModelParameters.gy-ModelParameters.iy;
%
% steady-state rental rate of capital times capital output ratio
%
% zy = rkstar*ky = alpha
%
ModelParameters.zy = ModelParameters.alpha;
%
% steady-state real-wage bill to consumption ratio 
%
ModelParameters.whlcstar = (1-ModelParameters.alpha)/(ModelParameters.phiw*ModelParameters.cy);
%
% compute the financial frictions parameters
%
zw = norminv(ModelParameters.fbar);
%
% solve for sigmaomega: all other FF parameters can thereafter be computed
%
sigmaomega = fzero(@(sigma)zetaspbfcn(zw,sigma,ModelParameters.sprd)-ModelParameters.zetaspb,0.5);
%
% compute the remaining steady-state paramaters
%
omegabar = omegafcn(zw,sigmaomega);
nk = nkfcn(zw,sigmaomega,ModelParameters.sprd);
mue = mufcn(zw,sigmaomega,ModelParameters.sprd);
ModelParameters.varrho = (1-nk)/nk;
%
% evaluate all BGG function elasticities
%
Gstar = Gfcn(zw,sigmaomega);
Gammastar = Gammafcn(zw,sigmaomega);
dGdomega = dGdomegafcn(zw,sigmaomega);
d2Gdomega2 = d2Gdomega2fcn(zw,sigmaomega);
dGammadomega = dGammadomegafcn(zw);
d2Gammadomega2 = d2Gammadomega2fcn(zw,sigmaomega);
dGdsigma = dGdsigmafcn(zw,sigmaomega);
d2Gdomegadsigma = d2Gdomegadsigmafcn(zw,sigmaomega);
dGammadsigma = dGammadsigmafcn(zw,sigmaomega);
d2Gammadomegadsigma = d2Gammadomegadsigmafcn(zw,sigmaomega);
%
% a couple of combinations
%
GammamuG = Gammastar-(mue*Gstar);
GammamuGprime = dGammadomega-(mue*dGdomega);
%
% elasticities wrt omegabar
%
zeta_bomega = zetabomegafcn(zw,sigmaomega,ModelParameters.sprd);
zeta_zomega = zetazomegafcn(zw,sigmaomega,ModelParameters.sprd);
zeta_bomega_zomega = zeta_bomega/zeta_zomega;
%
% elasticities wrt sigmaomega
%
Astar = (((1-(mue*(dGdsigma/dGammadsigma)))/(1-(mue*(dGdomega/dGammadomega))))-1)*dGammadsigma*ModelParameters.sprd;
Bstar = ((mue*nk*((dGdomega*d2Gammadomegadsigma)-(dGammadomega*d2Gdomegadsigma)))/(GammamuGprime^2));
Cstar = ((1-Gammastar)*ModelParameters.sprd)+((dGammadomega/GammamuGprime)*(1-nk));
zeta_bsigma = (sigmaomega*(Astar+Bstar))/Cstar;
%
zeta_zsigma = sigmaomega*(dGammadsigma-(mue*dGdsigma))/GammamuG;
ModelParameters.zetaspsigma = ((zeta_bomega_zomega*zeta_zsigma)-zeta_bsigma)/(1-zeta_bomega_zomega);
%
% elasticities of G
%
zeta_Gomega = omegabar*(dGdomega/Gstar);
zeta_Gsigma = sigmaomega*(dGdsigma/Gstar);
%
% parameters of the net worth equation
%
ModelParameters.zetane = ModelParameters.kappae*(ModelParameters.restar/(ModelParameters.pistar*ModelParameters.gamma*nk))*...
                         (1-(mue*Gstar*(1-(zeta_Gomega/zeta_zomega))));
ModelParameters.zetanr = (ModelParameters.kappae/(ModelParameters.beta*nk))*(1-nk+(mue*Gstar*ModelParameters.sprd*(1-(zeta_Gomega/zeta_zomega))));
ModelParameters.zetanq = (ModelParameters.kappae*(ModelParameters.restar/(ModelParameters.pistar*ModelParameters.gamma*nk))*...
                         (1-(mue*Gstar*(1+((zeta_Gomega*nk)/(zeta_zomega*(1-nk)))))))...
                         - (ModelParameters.kappae/(ModelParameters.beta*nk));
ModelParameters.zetann = (ModelParameters.kappae/ModelParameters.beta)+...
                         (ModelParameters.kappae*(ModelParameters.restar/(ModelParameters.pistar*ModelParameters.gamma))*...
                         mue*Gstar*(zeta_Gomega/(zeta_zomega*(1-nk))));
ModelParameters.zetansigma = ModelParameters.kappae*(ModelParameters.restar/(ModelParameters.pistar*ModelParameters.gamma*nk))*...
                             mue*Gstar*(zeta_Gsigma-((zeta_zsigma*zeta_Gomega)/zeta_zomega));
ModelParameters.zetantau = (ModelParameters.kappae/ModelParameters.beta)+...
                           ((ModelParameters.kappae/(ModelParameters.beta*nk))*((ModelParameters.sprd*(1-(mue*Gstar)))-1));
%
% store additional financial frictions parameters
%
ModelParameters.sigmaomega = sigmaomega;
ModelParameters.omegabar = omegabar;
ModelParameters.nk = nk;
%
% additional parameters
%
ModelParameters.c1 = (ModelParameters.lambda/ModelParameters.gamma)/(1+(ModelParameters.lambda/ModelParameters.gamma));
ModelParameters.c2 = ((ModelParameters.sigmac-1)*ModelParameters.whlcstar)/(ModelParameters.sigmac*(1+(ModelParameters.lambda/ModelParameters.gamma)));
ModelParameters.c3 = (1-(ModelParameters.lambda/ModelParameters.gamma))/(ModelParameters.sigmac*(1+(ModelParameters.lambda/ModelParameters.gamma)));
ModelParameters.i1 = 1/(1+(ModelParameters.betabar*ModelParameters.gamma));
ModelParameters.i2 = ModelParameters.i1/(ModelParameters.gamma^2*ModelParameters.varphi);
ModelParameters.q1 = (1-ModelParameters.delta)/(ModelParameters.rkstar+1-ModelParameters.delta);
ModelParameters.q1f = ModelParameters.betabar*(1-ModelParameters.delta);
ModelParameters.c3inv = 1/ModelParameters.c3;
ModelParameters.z1 = (1-ModelParameters.psi)/ModelParameters.psi;
ModelParameters.k1 = (1-ModelParameters.delta)/ModelParameters.gamma;
ModelParameters.k2 = (1-ModelParameters.k1)*(1+(ModelParameters.betabar*ModelParameters.gamma))*ModelParameters.gamma^2*ModelParameters.varphi;
ModelParameters.pi1 = ModelParameters.imathp/(1+(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.imathp));
ModelParameters.pi2 = (ModelParameters.betabar*ModelParameters.gamma)/(1+(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.imathp));
ModelParameters.pi3 = ((1-ModelParameters.xip)*(1-(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.xip)))/((1+(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.imathp))*ModelParameters.xip*((ModelParameters.phip-1)*ModelParameters.epsilonp+1));
ModelParameters.m1 = ModelParameters.sigmal;
ModelParameters.m2 = 1/(1-(ModelParameters.lambda/ModelParameters.gamma));
ModelParameters.m3 = ModelParameters.m2*ModelParameters.lambda/ModelParameters.gamma;
ModelParameters.w1 = 1/(1+(ModelParameters.betabar*ModelParameters.gamma));
ModelParameters.w2 = ModelParameters.w1*(1+(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.imathw));
ModelParameters.w3 = ModelParameters.w1*ModelParameters.imathw;
ModelParameters.w4 = (ModelParameters.w1*(1-ModelParameters.xiw)*(1-(ModelParameters.betabar*ModelParameters.gamma*ModelParameters.xiw)))/(ModelParameters.xiw*((ModelParameters.phiw-1)*ModelParameters.epsilonw+1));

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% local functions
%
% compute the elasticity of the spread wrt leverage 
%
function f = zetaspbfcn(z,sigma,sprd)
%
zetaratio = zetabomegafcn(z,sigma,sprd)/zetazomegafcn(z,sigma,sprd);
nk = nkfcn(z,sigma,sprd);
f = -(zetaratio/(1-zetaratio))*(nk/(1-nk));

%
% compute the elasticity zeta_{b,omegabar}
%
function f = zetabomegafcn(z,sigma,sprd)
%
nk = nkfcn(z,sigma,sprd);
mue = mufcn(z,sigma,sprd);
omega = omegafcn(z,sigma);
Gammastar = Gammafcn(z,sigma);
Gstar = Gfcn(z,sigma);
dGammadomega = dGammadomegafcn(z);
dGdomega = dGdomegafcn(z,sigma);
d2Gammadomega2 = d2Gammadomega2fcn(z,sigma);
d2Gdomega2 = d2Gdomega2fcn(z,sigma);
f = (omega*mue*nk*((d2Gammadomega2*dGdomega)-(d2Gdomega2*dGammadomega)))/...
    (((dGammadomega-(mue*dGdomega))^2)*(1-Gammastar+(dGammadomega*(Gammastar-(mue*Gstar))/(dGammadomega-(mue*dGdomega))))*sprd);

%
% compute the elasticity zeta_{z,omegabar}
%
function f = zetazomegafcn(z,sigma,sprd)
%
mue = mufcn(z,sigma,sprd);
f = omegafcn(z,sigma)*(dGammadomegafcn(z)-(mue*dGdomegafcn(z,sigma)))/...
    (Gammafcn(z,sigma)-(mue*Gfcn(z,sigma)));

%
% compute the steady-state net worth to capital ratio, which
% is related to leverage, i.e., steady-state leverage is (1-nk)/nk
% 
%
function f = nkfcn(z,sigma,sprd)
%
f = 1-(Gammafcn(z,sigma)-mufcn(z,sigma,sprd)*Gfcn(z,sigma))*sprd;

%
% compute the steady-state fraction related to bankruptcy costs
%
function f = mufcn(z,sigma,sprd)
%
f = (1-1/sprd)/(dGdomegafcn(z,sigma)/dGammadomegafcn(z)*(1-Gammafcn(z,sigma))+Gfcn(z,sigma));

%
% compute the steady-state omegabar, the value of omega below which entrepreneurs defaults on their debt
%
function f = omegafcn(z,sigma)
%
f = exp(sigma*z-1/2*sigma^2);

%
% compute the G function at omegabar
%
function f = Gfcn(z,sigma)
%
f = normcdf(z-sigma);

%
% compute the Gamma function at omegabar
%
function f = Gammafcn(z,sigma)
%
f = omegafcn(z,sigma)*(1-normcdf(z))+normcdf(z-sigma);

%
% compute the derivative of G wrt omegabar
%
function f = dGdomegafcn(z,sigma)
%
f = normpdf(z)/sigma;

%
% compute the second derivative of G wrt omegebar twice
%
function f = d2Gdomega2fcn(z,sigma)
%
f = -z*normpdf(z)/omegafcn(z,sigma)/sigma^2;

%
% compute the derivative of Gamma wrt omegbar
%
function f = dGammadomegafcn(z)
f = 1-normcdf(z);

%
% compute the second derivative of Gamma wrt omegabar twice
%
function f = d2Gammadomega2fcn(z,sigma)
%
f = -normpdf(z)/omegafcn(z,sigma)/sigma;

%
% compute the derivative of G wrt sigma
%
function f = dGdsigmafcn(z,sigma)
%
f = -z*normpdf(z-sigma)/sigma;

%
% compute the second derivate of G wrt omegabar and sigma
%
function f = d2Gdomegadsigmafcn(z,sigma)
%
f = -normpdf(z)*(1-z*(z-sigma))/sigma^2;

%
% compute the derivative of Gamma wrt sigma
%
function f = dGammadsigmafcn(z,sigma)
f = -normpdf(z-sigma);

%
% compute the second derivative of Gamma wrt omegabar and sigma
%
function f = d2Gammadomegadsigmafcn(z,sigma)
%
f = (z/sigma-1)*normpdf(z);

%
% end of UpdateDSFFParameters.m
%
