function StructureForData = FLMDataConstFile
% SWDataConstFile: Shows how the data construction output should be provided for YADA. This example is based
%                  on the model in Fagan, Lothian and McNelis (2012, JAE).
%
% USAGE:
%
%       StructureForData = FLMDataConstFile
%
% where StructureForData is a proper structure in matlab. The file is run in matlab via the eval
% function.
%
% INPUTS:          None!
%
% REQUIRED OUTPUT: StructureForData (structure). Any name for this structure will do since YADA renames it internally.
%                  REQUIRED FIELDS:  Y.data              (nxT) data-matrix of the n observable variables that should be evaluated by the
%                                                        Kalman filter
%                                    X.data              (kxT) data-matrix with exogenous data, may be an empty matrix, but must be stated
%                                    Y.names             (n dimensional cellstring) with variable names
%                                    X.names             (k dimensional cellstring) with names for exogenous variables (may be empty)
%                                    sample              (1x4 matrix) containing [start-year start-period end-yer end-period]. Must match
%                                                        the number of observations T.
%                                    frequency           (string) data frequency. Valid entries are: a(nnual), q(uarterly), m(onthly)
%                                                        if invalid frequency is reported, it is set to annual with sample being set
%                                                        to [1 1 T 1]
%                  OPTIONAL FIELDS:  Lambda              (vector) with lambda values to use for the DSGE-VAR models.
%                                    Y.levels            (1xn) vector, where 0 indicates a first differenced variable and 1 a levels
%                                                        variable
%                                    Y.annual            (1xn) vector, where 1 indicates that the variable should be annualized based
%                                                        on the data frequency, while 0 means that it should not be annualized (i.e.,
%                                                        can already be regarded as annualized).
%
% NOTE: and many more fields are supported.
%
%
% NOTICE:
%         * Any valid matlab functions on the path can be used by the function. How you transform any data you read in from
%           a spreadsheet, text file, of matlab mat-file is up to you.
%         * Any file format supported by matlab can be used for your data.
%
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 21, 2012.
%                        This version: January 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-01-2013: Took the changes to the monetary base data provided by Paul McNelis into account.
%
% * 30-04-2013: Renamed the HP-filter function to ensure it doesn't clash with the existing matlab function.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 15-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 15-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% Read data from a file. Make sure that the full path is given since this file will be moved
% into the sub-driectory tmp of pwd (pwd = working directory, i.e. the directory where YADA.m
% is located)
%

StructureForData = [];

[GoldData,GoldText] = wk1read([pwd '/example/FaganLothianMcNelis/data/GoldData.wk1']);
[d1,d2] = size(GoldData);
GoldData = GoldData(2:d1,2:d2);
[T,n] = size(GoldData);
%
% transform the variables
%
% 1. real GDP
%
Y = log(GoldData(:,1));
Y = Y-HPFilter(Y,1600);
Y = Y(2:T);
%
% 2. GDP deflator inflation
%
PIY = diff(log(GoldData(:,2)));
PIY = PIY-mean(PIY);
%
% 3. commercial paper rate
%
R = GoldData(:,3);
R = (R-mean(R))/400;
R = R(2:T);
%
% 4. monetary base growth
%
M = GoldData(:,4);
%M(32) = 1;
%M(140) = 2.70;
M = M/4;
%
% based on paragraph 1 of Section 2 in Fagan, Lothian and McNelis
%
DM = diff(log(M));
DM = DM-mean(DM);
%
% setup the output structure
%
StructureForData.Y.data = [Y PIY R DM]';
StructureForData.Y.names = {'Y' 'PIY' 'R' 'DM'};
StructureForData.Y.levels = [1 0 1 0];
StructureForData.Y.annual = [0 1 0 1];
StructureForData.Y.annualscale = ones(1,4);
StructureForData.X.data = [];
StructureForData.X.names = {''};
StructureForData.sample = [1878 2 1914 4];
StructureForData.frequency = 'q';
StructureForData.percentiles = [5 15 85 95];

%
% local function
%
function s = HPFilter(y,lambda)
%
T = length(y);
d = repmat([lambda -4*lambda ((6*lambda+1)/2)], T, 1);
d(1,2) = -2*lambda;      d(T-1,2) = -2*lambda;
d(1,3) = (1+lambda)/2;   d(T,3) = (1+lambda)/2;
d(2,3) = (5*lambda+1)/2; d(T-1,3) = (5*lambda+1)/2;
B = spdiags(d, -2:0, T, T);
B = B+B';
s = B\y;

%
% end of FLMDataConstFile.m
%
