function StructureForData = LPTDataConstFile
% DataConstFile: Shows how the data construction output should be provided for YADA. This example is based
%                on the fiscal policy DSGE model in Leeper, Plante and Traum (2010, J of Econometrics).
%
% USAGE:
%
%       StructureForData = LPTDataConstFile
%
% where StructureForData is a proper structure in matlab. The file is run in matlab via the eval
% function.
%
% INPUTS:          None!
%
% REQUIRED OUTPUT: StructureForData (structure). Any name for this structure will do since YADA renames it internally.
%                  REQUIRED FIELDS:  Y.data              (nxT) data-matrix of the n observable variables that should be evaluated by the
%                                                        Kalman filter
%                                    X.data              (kxT) data-matrix with exogenous data, may be an empty matrix, but must be stated
%                                    Y.names             (n dimensional cellstring) with variable names
%                                    X.names             (k dimensional cellstring) with names for exogenous variables (may be empty)
%                                    sample              (1x4 matrix) containing [start-year start-period end-yer end-period]. Must match
%                                                        the number of observations T.
%                                    frequency           (string) data frequency. Valid entries are: a(nnual), q(uarterly), m(onthly)
%                                                        if invalid frequency is reported, it is set to annual with sample being set
%                                                        to [1 1 T 1]
%                  OPTIONAL FIELDS:  Lambda              (vector) with lambda values to use for the DSGE-VAR models.
%                                    Y.levels            (1xn) vector, where 0 indicates a first differenced variable and 1 a levels
%                                                        variable
%                                    Y.annual            (1xn) vector, where 1 indicates that the variable should be annualized based
%                                                        on the data frequency, while 0 means that it should not be annualized (i.e.,
%                                                        can already be regarded as annualized).
%                                    Y.annualscale       (1xn) vector with scaling coefficient to multiply the annualized variables
%                                                        with.
%                                    Y.BVAR              (1xn1) vector where n1<=n that gives the indices for the variables in Y that
%                                                        should be included in a Bayesian VAR.
%                                    X.BVAR              (1xk1) vector where k1<=k that gives the indices for the variables in X that
%                                                        should be included in a Bayesian VAR.
%                                    X.extradata         (kxT1) data matrix with out-of-sample exogenous data.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: July 7, 2015.
%                        This version: January 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-07-2015: Continued work on the function to load data correctly from the excel file.
%
% * 09-07-2015: Continued work on the function with the individual variables.
%
% * 13-07-2015: Continued work from the labor tax variable LTAX.
%
% * 15-07-2015: Updated the documentation.
%
% * 17-07-2015: Changed the names of four observed variables to simplify the notation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 15-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 15-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%


StructureForData = [];
%
% open the excel spreadsheet
%
if isempty(findstr(lower(pwd),'example'))==1;
   file = [pwd '\example\LeeperPlanteTraum\data\LPTData-HerbstSchorfheide-FRED.xls'];
else;
   file = [pwd '\data\LPTData-HerbstSchorfheide-FRED.xls'];
end;
[OrigData,OrigText,RawData] = xlsread(file);
[w1,w2] = size(OrigText);
%
if isempty(RawData)==0;
   [r,c] = size(RawData);
   Data = zeros(r,c);
   for i=1:r;
      for j=1:c;
         if ischar(RawData{i,j})==0;
            Data(i,j) = cell2mat(RawData(i,j));
         end;
      end;
   end;
else;
   Data = zeros(w1,w2);
   d2 = size(OrigData,2);
   OrigData = [zeros(1,d2);OrigData];
   for i=1:w1;
      for j=1:w2;
         if isempty(OrigText{i,j})==1;
            Data(i,j:j+d2-1) = OrigData(:,i);
            break;
         end;
      end;
   end;
end;
for i=1:w1;
   %
   % skip the first column
   %
   VariableNames = '';
   for j=2:w2;
      VariableNames = strvcat(VariableNames,OrigText{i,j});
   end;
   if isempty(VariableNames)==0;
      StartRow = i;
      break;
   end;
end;
%
% determine the dates
%
SampleMatrix = '';
Pos1992Q3 = [];
Pos1960Q1 = [];
for i=2:w1;
   SampleMatrix = strvcat(SampleMatrix,strtrim(OrigText{i,1}));
   if strcmp(OrigText{i,1},'1960Q1')==1;
      Pos1960Q1 = i-1;
   end;
   if strcmp(OrigText{i,1},'1992Q3')==1;
      Pos1992Q3 = i-1;
   end;
end;
%
% delete the first column and row of Data
%
Data = Data(2:w1,2:w2);
%
% collect the data into individual variables
%
for i=1:w2-1;
   if strcmp(strtrim(VariableNames(i,:)),'PCED (PCDG)')==1;
      % Personal Consumption Expenditures: Durable Goods
      PCED = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'P (DPCERD3Q086SBEA)')==1;
      % Personal consumption expenditures (implicit price deflator)
      P = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'POP (CNP16OV)')==1;
      % Civilian Noninstitutional Population
      POP = Data(:,i);
      POP = POP/POP(Pos1992Q3);
   elseif strcmp(strtrim(VariableNames(i,:)),'PCE1 (PCEND)')==1;
      % Personal Consumption Expenditures: Nondurable Goods
      PCE1 = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'PCE2 (PCES)')==1;
      % Personal Consumption Expenditures: Services
      PCE2 = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'HOURS (PRS85006023)')==1;
      % Nonfarm Business Sector: Average Weekly Hours
      HOURS = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'EMP (CE16OV)')==1;
      % civilian employment
      EMP = Data(:,i);
      EMP = EMP/EMP(Pos1992Q3);
   elseif strcmp(strtrim(VariableNames(i,:)),'CTAX (W007RC1Q027SBEA)')==1;
      % Federal government current tax receipts: Taxes on production and imports
      CTAX = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'IT (A074RC1Q027SBEA)')==1;
      % Federal government current tax receipts: Personal current taxes
      IT = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'W (WASCUR)')==1;
      % Compensation of Employees: Wages & Salary Accruals
      W = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'PRI1 (A045RC1Q027SBEA)')==1;
      % Proprietors' income with inventory valuation and capital consumption adjustments: Nonfarm
      PRI1 = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'PRI2 (A043RC1Q027SBEA)')==1;
      % Proprietors' income with IVA: Farm
      PRI2 = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'RENT (RENTIN)')==1;
      % Rental Income of Persons with Capital Consumption Adjustment (CCAdj)
      RENT = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'PROF (CPROFIT)')==1;
      % Corporate Profits with Inventory Valuation Adjustment (IVA) and Capital Consumption Adjustment (CCAdj)
      PROF = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'INT (W255RC1Q027SBEA)')==1;
      % Net interest and miscellaneous payments on assets
      INT = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'CSI (W780RC1Q027SBEA)')==1;
      % Federal government current receipts: Contributions for government social insurance
      CSI = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'EC (A576RC1Q027SBEA)')==1;
      % Compensation of employees: Wages and salaries
      EC = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'CT (B075RC1Q027SBEA)')==1;
      % Federal government current tax receipts: Taxes on corporate income
      CT = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'PT (B249RC1Q027SBEA)')==1;
      % State and local government current tax receipts: Taxes on production and imports: Property taxes
      PT = Data(:,i);
      PT(isnan(PT)==1) = 0;
   elseif strcmp(strtrim(VariableNames(i,:)),'GC (FGEXPND)')==1;
      % Federal Government: Current Expenditures
      GC = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'GI (A787RC1Q027SBEA)')==1;
      % Gross government investment: Federal
      GI = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'GP (AD08RC1Q027SBEA)')==1;
      % Federal government total expenditures: Net purchases of nonproduced assets
      GP = Data(:,i);
      GP(isnan(GP)==1) = 0;
   elseif strcmp(strtrim(VariableNames(i,:)),'GCK (A918RC1Q027SBEA)')==1;
      % Consumption of fixed capital: Government: Federal
      GCK = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'TRANSPAY (W014RC1Q027SBEA)')==1;
      % Government current transfer payments
      TRANSPAY = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'TRANSREC (W011RC1Q027SBEA)')==1;
      % Personal current transfer receipts
      TRANSREC = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'CAPTRANSPAY (W020RC1Q027SBEA)')==1;
      % Government capital transfer payments
      CAPTRANSPAY = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'CAPTRANSREC (B232RC1Q027SBEA)')==1;
      % Federal government capital transfer receipts
      CAPTRANSREC = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'TAXREC (W006RC1Q027SBEA)')==1;
      % Federal government current tax receipts
      TAXREC = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'INCREC (W009RC1Q027SBEA)')==1;
      % Personal income receipts on assets
      INCREC = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'GOVSRP (B097RC1Q027SBEA)')==1;
      % Current surplus of government enterprises
      GOVSRP = Data(:,i);
%      GOVSRP(isnan(GOVSRP)==1) = 0;
   elseif strcmp(strtrim(VariableNames(i,:)),'SUBSIDIES (B096RC1Q027SBEA)')==1;
      SUBSIDIES = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'INTPAY (A091RC1Q027SBEA)')==1;
      % Federal government current expenditures: Interest payments
      INTPAY = Data(:,i);
   elseif strcmp(strtrim(VariableNames(i,:)),'M (AMBSL)')==1;
      % St. Louis Adjusted Monetary Base
      M = Data(:,i);
   end;
end;
T = w1-1;
%
%
% 1. Real investment for the sample 1960Q1-2014Q4
%
lnI = 100*log(PCED(Pos1960Q1:T)./(P(Pos1960Q1:T).*POP(Pos1960Q1:T)));
lnI = DTrend(lnI);
%
% 2. Real consumption
%
PCE = PCE1+PCE2;
lnC = 100*log(PCE(Pos1960Q1:T)./(P(Pos1960Q1:T).*POP(Pos1960Q1:T)));
lnC = DTrend(lnC);
%
% 3. Real hours worked
%
lnH = 100*log((HOURS(Pos1960Q1:T).*EMP(Pos1960Q1:T))./POP(Pos1960Q1:T));
lnH = DTrend(lnH);
%
% 4. Real consumption tax revenues
%
lnCTR = 100*log(CTAX(Pos1960Q1:T)./(P(Pos1960Q1:T).*POP(Pos1960Q1:T)));
lnCTR = DTrend(lnCTR);
%
% 5. Real labor tax revenues
%
% proprietors' income
PRI = PRI1+PRI2;
% capital income
CI = RENT+PROF+INT+(PRI/2);
% average personal income tax rate
tauP = IT./(W+(PRI/2)+CI);
% average labor income tax rate
tauL = ((tauP.*(W+PRI))+CSI)./(EC+(PRI/2));
% tax base
BASE = PCE+PCED;
% total labor tax revenues
LTAX = tauL.*BASE;
%
lnLTR = 100*log(LTAX(Pos1960Q1:T)./(P(Pos1960Q1:T).*POP(Pos1960Q1:T)));
lnLTR = DTrend(lnLTR);
%
% 6. Real capital tax revenues
%
% average capital income tax
tauK = ((tauP.*CI)+CT)./(CI+PT);
% total capital tax revenues
KTAX = tauK.*BASE;
%
lnKTR = 100*log(KTAX(Pos1960Q1:T)./(P(Pos1960Q1:T).*POP(Pos1960Q1:T)));
lnKTR = DTrend(lnKTR);
%
% 7. Real government expenditure
%
G = GC+GI+GP-GCK;
%
lnG = 100*log(G(Pos1960Q1:T)./(P(Pos1960Q1:T).*POP(Pos1960Q1:T)));
lnG = DTrend(lnG);
%
% 8. Real government transfers
%
% net current transfers
CURRTRANS = TRANSPAY-TRANSREC;
% net capital transfers
CAPTRANS = CAPTRANSPAY-CAPTRANSREC;
% total tax revenue
TAX = CTAX+LTAX+KTAX;
% tax residual
TAXRESID = TAXREC+INCREC+CSI+GOVSRP-TAX;
% transfers
TR = CURRTRANS+CAPTRANS+SUBSIDIES-TAXRESID;
%
lnTR = 100*log(TR(Pos1960Q1:T)./(P(Pos1960Q1:T).*POP(Pos1960Q1:T)));
lnTR = DTrend(lnTR);
%
% 9. Real government debt
%
NB = G+INTPAY+TR-TAX;
B = zeros(T,1);
%
% market value of government debt in 1947Q1 is taken from Cox and Hirschhorn (1983), Table 7
% in billions of US dollars. It's the average of their monthly figure, i.e., the average of
% Jan 47 266.960 ; Feb 47 268.794 ; Mar 47 266.745
%
B(1) = 267.5;
for t=2:T;
   B(t) = NB(t) - M(t) + M(t-1) + B(t-1);
end;
%
lnB = 100*log(B(Pos1960Q1:T)./(P(Pos1960Q1:T).*POP(Pos1960Q1:T)));
lnB = DTrend(lnB);
%
% now we can setup the required matrices
%
StructureForData.Y.data = [lnC lnI lnH lnB lnG lnKTR lnLTR lnCTR lnTR]';
StructureForData.Y.names = {'lnC' 'lnI' 'lnH' 'lnB' 'lnG' 'lnTk' 'lnTl' 'lnTc' 'lnZ'};
StructureForData.Y.levels = ones(1,9);
StructureForData.Y.annual = zeros(1,9);
StructureForData.X.data = [];
StructureForData.X.names = '';
StructureForData.frequency = 'q';
FP = strrep(lower(SampleMatrix(Pos1960Q1,:)),'q',' ');
LP = strrep(lower(SampleMatrix(T,:)),'q',' ');
StructureForData.sample = [str2num(FP) str2num(LP)];
StructureForData.percentiles = [5 15 85 95];
StructureForData.Lambda = [0.5 1 1.25 1.5 2 2.5 3 3.5 4 4.5 5 6 6.5 7.5 10 15 25 50 Inf];

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% local function: Remove deterministic part
%
function z = DTrend(y)

T = length(y);
x = [ones(T,1) (1:T)'];
%
% estimate the coefficients
b = (x'*x)\(x'*y);
z = y-(x*b);

%
% LPTDataConstFile.m
%