function [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = ChandrasekharRecursions(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod)
% ChandrasekharRecursions: Calculates the value of the log-likelihood function (based on Gaussian errors and initial entry
%                          for the state vector) for the observed data using the state-space representation via the so called
%                          Chandrasekhar recursions.
%
% USAGE:
%
%       [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1] = ChandrasekharRecursions(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter, ...
%                                                                         Tolerance,StartPeriod)
%
% REQUIRED INPUT:  Y (nxT) matrix with observed data.
%
%                  X (kxT) matrix with exogenous of predetermined data.
%
%                  A (kxn) matrix with parameter values on the k X variables in the measurement equation.
%
%                  H (rxn) matrix with parameter values on the r state variables (ksi) in the measurement equation.
%
%                  F (rxr) matrix with parameter values on lagged state variables in the state equation.
%
%                  Q (rxr) positive semidefinite matrix with covariances for the innovations in the state equation.
%
%                  R (nxn) positive semidefinite matrix with covariances for the innovations in the measurement equation.
%
%                  KsiLast (rx1) vector with initial values for the state vector.
%
%                  initP (integer) indicating method of determining initial value for P(1|0), the MSE matrix for
%                        ksi(1|0). If initP=1 (default), the P(1|0) is given by the unconditional covariance matrix of ksi,
%                        calculated through the vec operator. Since this operation can be excessively slow for large systems
%                        (when r^2 is a large number) the Doubling Algorthm can be chosen to approximate the unconditional
%                        variance. This algorithm is called if initP=2.
%
%                  MaxIter (integer), the maximum number of iterations to use with the doubling algorithm
%                        divided by 100. Must be multiplied by 100 when calling DoublingAlgorithmLyapunov.
%
%                  Tolerance (integer), determines the tolerance value for the doubling algorithm.
%
%                  StartPeriod, positive integer indicating first observation to use when calculating the log-likelihood value. The
%                        default value is 1.
%
% REQUIRED OUTPUT: lnL, scalar value of the log-likelihood function.
%
%                  status (boolean) indicating the covariance stationarity of the state variables. If all the
%                        eigenvalues of F are inside the unit circle, then status=0 and the log-likelihood function
%                        is calculated. If not, then status=1 and initP is set to 3. In addition, status can be
%                        -1 if ths forecast covariance matrix is not positive definite for some time period.
%
% OPTIONAL OUTPUT: lnLt (1x(T-StartPeriod+1)) vector with the time t values of the log-densities that sum to the value lnL.
%
%                  Yhat (nx(T-StartPeriod+1)) matrix with the predictions [y(StartPeriod|StartPeriod-1) ... y(T|T-1).
%
%                  MSEY (nxnx(T-StartPeriod+1)) 3 dimensional matrix with MSEY(:,:,t-StartPeriod+1) = H'*P(t|t-1)*H + R, the mean
%                        squared error matrix for y(t|t-1).
%
%                  Ksitt1 (rx(T-StartPeriod+1)) matrix with the predictions [ksi(StartPeriod|StartPeriod-1) ... ksi(T|T-1)].
%
%                  Ptt1 (rxrx(T-StartPeriod+1)) 3 dimensional matrix with Ptt1(:,:,t-StartPeriod+1) = P(t|t-1)
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: November 15, 2012.
%                        This version: January 16, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-11-2012: Finished the function.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 13-01-2015: Updated the documentation.
%
% * 02-09-2015: The recursive estimation breaks if the log-likelihood is a NaN.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initializing
%
lnL = 0;
status = 0;
%
% Determine dimensions. Checking if dimensions match is not undertaken!
%
[n,T] = size(Y);
if isempty(X)==0;
   Ax = A'*X;
else;
   Ax = zeros(n,T);
end;
HT = H';
In = eye(n);
r = size(H,1);
if initP==1;
   PLast = reshape((eye(r*r)-kron(F,F))\vec(Q),r,r);
elseif initP==2;
   %
   % The maximum number of iterations is set to MaxIter*100, while the convergence value is 1.0e-8
   %
   [PLast,stat] = DoublingAlgorithmLyapunov(F,Q,MaxIter*100,10^(-(Tolerance+1)));
end;
%
% Initialize temporary matrices
%
if nargout>2;
   YhatTmp = zeros(n,T);
   MSEYTmp = zeros(n,n,T);
   Ksitt1Tmp = zeros(r,T);
   Ksitt1Tmp(:,1) = KsiLast;
   Ptt1Tmp = zeros(r,r,T);
end;
lnLtTmp = zeros(1,T);
%
% initialize the W and M matrices
%
%
% Now we can compute the Kalman filtered series
%
lnLConst = -n*log(2*pi);
PLastH = PLast*H;
MSEY = (HT*PLastH)+R;
KTilde = F*PLastH;
W = KTilde;
if rank(Q)+rank(R)>=n;
   InvMSEY = In/MSEY;
   M = -InvMSEY;
   if nargout==2;
      for t=1:T;
         yPred = Ax(:,t) + (HT*KsiLast);
         yError = Y(:,t)-yPred;
         %
         % compute the log-likelihood
         %
         lnLtTmp(1,t) = lnLConst+LogDet(InvMSEY)-(yError'*InvMSEY*yError);
         if isnan(lnLtTmp(1,t))==1;
            break;
         end;
         %
         % update KsiLast
         %
         KsiLast = (F*KsiLast)+(KTilde*InvMSEY*yError);
         %
         % common matrix multiplications
         %
         HTW = HT*W;
         FW = F*W;
         MWTH = M*HTW';
         %
         % now the Chandrasekhar recursions for P(t+1|t)
         %
         MSEY = MSEY + (HTW*MWTH);
         KTilde = KTilde + (FW*MWTH);
         M = M + (M*HTW'*InvMSEY*HTW*M);
         InvMSEY = In/MSEY;
         W = FW - (KTilde*InvMSEY*HTW);
      end;
   else;
      for t=1:T;
         yPred = Ax(:,t) + (HT*KsiLast);
         yError = Y(:,t)-yPred;
         %
         % compute the log-likelihood
         %
         lnLtTmp(1,t) = lnLConst+LogDet(InvMSEY)-(yError'*InvMSEY*yError);
         if isnan(lnLtTmp(1,t))==1;
            break;
         end;
         %
         % store values if asked to do so
         %
         YhatTmp(:,t) = yPred;
         MSEYTmp(:,:,t) = MSEY;
         Ksitt1Tmp(:,t) = KsiLast;
         Ptt1Tmp(:,:,t) = PLast;
         %
         % update KsiLast
         %
         KsiLast = (F*KsiLast)+(KTilde*InvMSEY*yError);
         %
         % common matrix multiplications
         %
         HTW = HT*W;
         FW = F*W;
         MWTH = M*HTW';
         %
         % now the Chandrasekhar recursions for P(t+1|t)
         %
         MSEY = MSEY + (HTW*MWTH);
         KTilde = KTilde + (FW*MWTH);
         M = M + (M*HTW'*InvMSEY*HTW*M);
         InvMSEY = In/MSEY;
         W = FW - (KTilde*InvMSEY*HTW);
         %
         % update PLast
         %
         PLast = PLast + (W*M*W');
      end;
   end;
else;
   InvMSEY = pinv(MSEY);
   M = -InvMSEY;
   if nargout==2;
      for t=1:T;
         yPred = Ax(:,t) + (HT*KsiLast);
         yError = Y(:,t)-yPred;
         %
         % compute the log-likelihood
         %
         lnLtTmp(1,t) = lnLConst+LogDet(InvMSEY)-(yError'*InvMSEY*yError);
         if isnan(lnLtTmp(1,t))==1;
            break;
         end;
         %
         % update KsiLast
         %
         KsiLast = (F*KsiLast)+(KTilde*InvMSEY*yError);
         %
         % common matrix multiplications
         %
         HTW = HT*W;
         FW = F*W;
         MWTH = M*HTW';
         %
         % now the Chandrasekhar recursions for P(t+1|t)
         %
         MSEY = MSEY + (HTW*MWTH);
         KTilde = KTilde + (FW*MWTH);
         M = M + (M*HTW'*InvMSEY*HTW*M);
         InvMSEY = pinv(MSEY);
         W = FW - (KTilde*InvMSEY*HTW);
      end;
   else;
      for t=1:T;
         yPred = Ax(:,t) + (HT*KsiLast);
         yError = Y(:,t)-yPred;
         %
         % compute the log-likelihood
         %
         lnLtTmp(1,t) = lnLConst+LogDet(InvMSEY)-(yError'*InvMSEY*yError);
         if isnan(lnLtTmp(1,t))==1;
            break;
         end;
         %
         % store values if asked to do so
         %
         YhatTmp(:,t) = yPred;
         MSEYTmp(:,:,t) = MSEY;
         Ksitt1Tmp(:,t) = KsiLast;
         Ptt1Tmp(:,:,t) = PLast;
         %
         % update KsiLast
         %
         KsiLast = (F*KsiLast)+(KTilde*InvMSEY*yError);
         %
         % common matrix multiplications
         %
         HTW = HT*W;
         FW = F*W;
         MWTH = M*HTW';
         %
         % now the Chandrasekhar recursions for P(t+1|t)
         %
         MSEY = MSEY + (HTW*MWTH);
         KTilde = KTilde + (FW*MWTH);
         M = M + (M*HTW'*InvMSEY*HTW*M);
         InvMSEY = pinv(MSEY);
         W = FW - (KTilde*InvMSEY*HTW);
         %
         % update PLast
         %
         PLast = PLast + (W*M*W');
      end;
   end;
end;
lnL = (1/2)*sum(lnLtTmp(StartPeriod:T));
%
% test if the log-likelihood is real
%
if (isreal(lnL)==0)|(isnan(lnL)==1);
   status = -1;
end;
%
% Fix output to match StartPeriod
%
if nargout==3;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
elseif nargout==4;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
elseif nargout==5;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
elseif nargout==6;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
elseif nargout==7;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
end;

%
% end of ChandrasekharRecursions.m
%
