function [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = DiffuseStateSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaRank,SmoothData)
% DiffuseStateSmoother: Computes smoothed and updated values of the state variables from the standard Kalman filter,
%                       as well as the MSE matrices for these predictors when diffuse initialization has been used
%                       for the Kalman filter.
%
% USAGE:
%
%       [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = DiffuseStateSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaRank,SmoothData)
%
% REQUIRED INPUT:  Y (matrix) of dimension nxT with observed data.
%
%                  Yhat (matrix) of dimension nxT with the forecasted data (1 step ahead forecasts).
%
%                  Ksitt1 (matrix) of dimension rxT with the forecasted state vector (1 step ahead).
%
%                  Ptt1 (matrix) of dimension rxrxT with the MSEs for the forecasted state vector.
%
%                  H (matrix) of dimension rxn with parameter values on the r state variables (ksi)
%                    in the measurement equation.
%
%                  F (matrix) of dimension rxr with parameter values on lagged state variables in the state equation.
%
%                  R (matrix) of dimension nxn with covariances for the innovations in the measurement equation.
%
%                  B0 (matrix) of dimension rxq with coefficients on the structural shocks in the state equation.
%
%                  SigmaRank (vector) of dimension d with the rank of \Sigma_\infty during the initialization
%                    phase.
%
%                  SmoothData (structure) with fields containing data that were used by the diffuse Kalman smoother
%                    during the initialization phase. The fields are given by:
%
%                              SigmaInf (matrix) of dimension nxnxd with the values of \Sigma(\infty,t|t-1) during the
%                                       initialization sample.
%
%                              SigmaStar (matrix) of dimension nxnxd with the values of \Sigma(\star,t|t-1) during the
%                                       initialization sample.
%
%                              PInf (matrix) of dimension rxrxd with the values of P(\infty,t|t-1) during the initialization
%                                       sample.
%
%                              PStar (matrix) of dimension rxrxd with the values of P(\star,t|t-1) during the initialization
%                                       sample.
%
%                              KInfMat (matrix) of dimension rxnxd with the values of K(\infty,t) during the initialization
%                                       sample.
%
%                              LInfMat (matrix) of dimension rxrxd with the values of L(\infty,t) during the initialization
%                                       sample.
%
%                              KStarMat (matrix) of dimension rxnxd with the values of K(\star,t) during the initialization
%                                       sample.
%
%                              LStarMat (matrix) of dimension rxrxd with the values of L(\star,t) during the initialization
%                                       sample.
%
%                              rInfMat (matrix) of dimension rxd with the values of r(\infty,t|t) during the initialization
%                                       sample.
%
%                              rStarMat (matrix) of dimension rxTtot with the values of r(\star,t|t) during the full sample.
%
%                              PInfttMat (matrix) of dimension rxrxd with the values of P(\infty,t|t) during the
%                                       initialization sample.
%
%                              PStarttMat (matrix) of dimension rxrxTtot with the values of P(\star,t|t) during the full
%                                       sample.
%
%                              KsittMat (matrix) of dimension rxTtot with the values of Ksi(t|t) during the full sample.
%
%                              sigma2iInfCell (cell array) of dimension 1xd with vectors of dimension n containing the
%                                       values of Sigma(\infty,t,i).
%
%                              sigma2iStarCell (cell array) of dimension 1xd with vectors of dimension n containing the
%                                       values of Sigma(\star,t,i).
%
%                              KtiInfCell (cell array) of dimension 1xd with matrices of dimension rxn with the Kalman
%                                       gain vectors K(\infty,t,i) in its columns.
%
%                              KtiStarCell (cell array) of dimension 1xd with matrices of dimension rxn with the Kalman
%                                       gain vectors K(\star,t,i) in its columns.
%
%                              ztiCell (cell array) of dimension 1xd with vectors of dimension n with the prediction
%                                       errors z(t,i).
%
%                              HtiCell (cell array) of dimension 1xd with matrices of dimension rxn with the transformed
%                                       H matrices.
%
% REQUIRED OUTPUT: Ksitt (matrix) of dimension rxT with the updated predictions of the state vector.
%
%                  Ptt (matrix) of dimension rxrxT with the MSEs of the updated predictions of the state vector.
%
%                  KsitT (matrix) of dimension rxT with the smooth predictions of the state vector.
%
%                  PtT (matrix) of dimension rxrxT with the MSEs of the smooth predictions of the state vector.
%
%                  rttMat (matrix) of dimension rxT with standardized forecast errors.
%
% OPTIONAL OUTPUT: rTMat (matrix) of dimension rxT with the r(t|T)-vectors.
%
%                  NtMat (matrix) of dimension rxrxT with the Nt matrices.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 25, 2011.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-10-2011: Continued work on the function.
%
% * 27-10-2011: Added the smoothing code for the initialization sample.
%
% * 28-10-2011: Made sure that Sigmatilde has the correct dimension when rSI is zero.
%
% * 03-01-2012: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 06-03-2019: Fixed an error when computing Ksit1t over the initialization sample
%               and the SigmaInf matrix is nonsingular.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
% * 26-02-2024: Changed the rtvec output variable to rTMat.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% initialize
%
[r,T] = size(Ksitt1);
n = size(R,1);
Q = B0*B0';
HT = H';
In = eye(n);
Ir = eye(r);
d = length(SigmaRank);
Ttot = size(SmoothData.KsittMat,2);
%
% check how many obs there are in the sample for the original smoother
%
Td = max(T-Ttot+d,0);
%
% initialie output
%
Ksitt = zeros(r,T);
Ptt = zeros(r,r,T);
KsitT = zeros(r,T);
PtT = zeros(r,r,T);
rttMat = zeros(r,T);
rTMatTmp = zeros(r,T);
NtMatTmp = zeros(r,r,T);
rt = zeros(r,1);
Nt = zeros(r,r);
%
% we loop from period T and backwards to t=1
%
if rank(B0)+rank(R)>=n;
   for t=(T:-1:Td+1);
      Ptt1H = Ptt1(:,:,t)*H;
      InvMSEY = In/((HT*Ptt1H)+R);
      %
      Ksitt(:,t) = SmoothData.KsittMat(:,Ttot-T+t);
      Ptt(:,:,t) = SmoothData.PStarttMat(:,:,Ttot-T+t);
      if t<T;
         KalmanGain = F*Ptt1H*InvMSEY;
         Lt = F-(KalmanGain*HT);
         rt = (H*InvMSEY*(Y(:,t)-Yhat(:,t)))+(Lt'*rt);
         Nt = (H*InvMSEY*HT)+(Lt'*Nt*Lt);
         KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
         PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
      else;
         %
         % Updates and smooth predictions and MSEs have the same values for t=T
         %
         KsitT(:,T) = Ksitt(:,T);
         PtT(:,:,T) = Ptt(:,:,T);
         rt = H*InvMSEY*(Y(:,t)-Yhat(:,t));
         Nt = H*InvMSEY*HT;
      end;
      rttMat(:,t) = H*InvMSEY*(Y(:,t)-Yhat(:,t));
      rTMatTmp(:,t) = rt;
      NtMatTmp(:,:,t) = Nt;
   end;
   %
   % initialize for initial sample
   %
   rtilde = [rt;zeros(r,1)];
   Ntilde = [Nt zeros(r,r);zeros(r,2*r)];
   Htilde = kron(eye(2),H);
   Ftilde = kron(eye(2),F);
   FtildeT = Ftilde';
   %
   % deal with the initial sample
   %
   for t=(Td:-1:1);
      rSI = SigmaRank(1,d+t-Td);
      PInft = SmoothData.PInf(:,:,d+t-Td);
      PStart = SmoothData.PStar(:,:,d+t-Td);
      Ptilde = [PStart PInft];
      if rSI==n;
         %
         % SigmaInf has full rank
         %
         ztilde = [Y(:,t)-Yhat(:,t);zeros(n,1)];
         LInft = SmoothData.LInfMat(:,:,d+t-Td);
         KStart = SmoothData.KStarMat(:,:,d+t-Td);
         InvSigInft = In/SmoothData.SigmaInf(:,:,d+t-Td);
         SigStart = SmoothData.SigmaStar(:,:,d+t-Td);
         Ltilde = [LInft -KStart*HT;zeros(r,r) LInft];
         Sigmatilde = [zeros(n,n) InvSigInft;InvSigInft -InvSigInft*SigStart*InvSigInft];
         %
         rtilde = (Htilde*Sigmatilde*ztilde)+(Ltilde'*rtilde);
         Ntilde = (Htilde*Sigmatilde*Htilde')+(Ltilde'*Ntilde*Ltilde);
         stilde = Htilde*Sigmatilde*ztilde;
%         stilde = [zeros(r,1);H*InvSigInft*(Y(:,t)-Yhat(:,t))];
         Mtilde = Htilde*Sigmatilde*Htilde';
      elseif rSI==0;
         %
         % SigmaInf is zero
         %
         ztilde = [Y(:,t)-Yhat(:,t);zeros(n,1)];
         Ltilde = [SmoothData.LStarMat(:,:,d+t-Td) zeros(r,r);zeros(r,r) F];
         Sigmatilde = zeros(2*n,2*n);
         Sigmatilde(1:n,1:n) = In/SmoothData.SigmaStar(:,:,d+t-Td);
         %
         rtilde = (Htilde*Sigmatilde*ztilde)+(Ltilde'*rtilde);
         Ntilde = (Htilde*Sigmatilde*Htilde')+(Ltilde'*Ntilde*Ltilde);
         stilde = Htilde*Sigmatilde*ztilde;
%         stilde = [H*Sigmatilde(1:n,1:n)*(Y(:,t)-Yhat(:,t));zeros(r,1)];
         Mtilde = Htilde*Sigmatilde*Htilde';
      else;
         %
         % univariate initial smoothing
         %
         sigma2iInf = SmoothData.sigma2iInfCell{d+t-Td};
         sigma2iStar = SmoothData.sigma2iStarCell{d+t-Td};
         KtiInf = SmoothData.KtiInfCell{d+t-Td};
         KtiStar = SmoothData.KtiStarCell{d+t-Td};
         zti = SmoothData.ztiCell{d+t-Td};
         Hti = SmoothData.HtiCell{d+t-Td};
         %
         % handle transitions from rtilde(t+1,0) to
         % rtilde(t,n) and similary for Ntilde,
         % where rtilde(t+1,0) = rtilde(t+1|T)
         % and Ntilde(t+1,0) = Ntilde(t+1|T)
         %
         rtilde = FtildeT*rtilde;
         Ntilde = FtildeT*Ntilde*Ftilde;
         stilde = zeros(2*r,1);
         Mtilde = zeros(2*r,2*r);
         for i=(n:-1:1);
            Hi = Hti(:,i);
            zi = zti(i);
            if sigma2iInf(i)>1.0e-10;
               LInfti = Ir-(KtiInf(:,i)*Hi'*(1/sigma2iInf(i)));
               L0ti = ((KtiInf(:,i)*(sigma2iStar(i)/sigma2iInf(i)))-KtiStar(:,i))*Hi'*(1/sigma2iInf(i));
               atilde = [zeros(r,1);Hi*(zi/sigma2iInf(i))];
               btilde = [zeros(r,r) Hi*Hi'*(1/sigma2iInf(i));Hi*Hi'*(1/sigma2iInf(i)) Hi*Hi'*(1/sigma2iInf(i)^2)];
               Ltilde = [LInfti L0ti;zeros(r,r) LInfti];
            else;
               LStarti = Ir-(KtiStar(:,i)*Hi'*(1/sigma2iStar(i)));
               Ltilde = [LStarti zeros(r,r);zeros(r,r) LStarti];
               atilde = [Hi*(zi/sigma2iStar(i));zeros(r,1)];
               btilde = [Hi*Hi'*(1/sigma2iStar(i)) zeros(r,r);zeros(r,2*r)];
            end;
            rtilde = atilde+(Ltilde'*rtilde);
            Ntilde = btilde+(Ltilde'*Ntilde*Ltilde);
            stilde = atilde+(Ltilde'*stilde);
            Mtilde = btilde+(Ltilde'*Mtilde*Ltilde);
         end;
      end;
      %
      % disturbance smoother estimates
      %
      rt = rtilde(1:r,1);
      Nt = Ntilde(1:r,1:r);
      %
      % compute the smooth estimates for the state variables
      %
      KsitT(:,t) = Ksitt1(:,t)+(Ptilde*rtilde);
      PtT(:,:,t) = PStart-(Ptilde*Ntilde*Ptilde');
      Ksitt(:,t) = Ksitt1(:,t)+(Ptilde*stilde);
      Ptt(:,:,t) = PStart-(Ptilde*Mtilde*Ptilde');
      if rSI<n;
         rttMat(:,t) = stilde(1:r,1);
      else;
         rttMat(:,t) = stilde(r+1:2*r,1);
      end;
      rTMatTmp(:,t) = rt;
      NtMatTmp(:,:,t) = Nt;
   end;
else;
   for t=(T:-1:Td+1);
      Ptt1H = Ptt1(:,:,t)*H;
      InvMSEY = pinv((HT*Ptt1H)+R);
      %
      Ksitt(:,t) = SmoothData.KsittMat(:,Ttot-T+t);
      Ptt(:,:,t) = SmoothData.PStarttMat(:,:,Ttot-T+t);
      if t<T;
         KalmanGain = F*Ptt1H*InvMSEY;
         Lt = F-(KalmanGain*HT);
         rt = (H*InvMSEY*(Y(:,t)-Yhat(:,t)))+(Lt'*rt);
         Nt = (H*InvMSEY*HT)+(Lt'*Nt*Lt);
         KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
         PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
      else;
         %
         % Updates and smooth predictions and MSEs have the same values for t=T
         %
         KsitT(:,T) = Ksitt(:,T);
         PtT(:,:,T) = Ptt(:,:,T);
         rt = H*InvMSEY*(Y(:,t)-Yhat(:,t));
         Nt = H*InvMSEY*HT;
      end;
      rttMat(:,t) = H*InvMSEY*(Y(:,t)-Yhat(:,t));
      rTMatTmp(:,t) = rt;
      NtMatTmp(:,:,t) = Nt;
   end;
   %
   % initialize for initial sample
   %
   rtilde = [rt;zeros(r,1)];
   Ntilde = [Nt zeros(r,r);zeros(r,2*r)];
   Htilde = kron(eye(2),H);
   Ftilde = kron(eye(2),F);
   FtildeT = Ftilde';
   %
   % deal with the initial sample
   %
   for t=(Td:-1:1);
      rSI = SigmaRank(1,d+t-Td);
      PInft = SmoothData.PInf(:,:,d+t-Td);
      PStart = SmoothData.PStar(:,:,d+t-Td);
      Ptilde = [PStart PInft];
      if rSI==n;
         %
         % SigmaInf has full rank
         %
         ztilde = [Y(:,t)-Yhat(:,t);zeros(n,1)];
         LInft = SmoothData.LInfMat(:,:,d+t-Td);
         KStart = SmoothData.KStarMat(:,:,d+t-Td);
         InvSigInft = pinv(SmoothData.SigmaInf(:,:,d+t-Td));
         SigStart = SmoothData.SigmaStar(:,:,d+t-Td);
         Ltilde = [LInft -KStart*HT;zeros(r,r) LInft];
         Sigmatilde = [zeros(n,n) InvSigInft;InvSigInft -InvSigInft*SigStart*InvSigInft];
         %
         rtilde = (Htilde*Sigmatilde*ztilde)+(Ltilde'*rtilde);
         Ntilde = (Htilde*Sigmatilde*Htilde')+(Ltilde'*Ntilde*Ltilde);
         stilde = Htilde*Sigmatilde*ztilde;
         Mtilde = Htilde*Sigmatilde*Htilde';
      elseif rSI==0;
         %
         % SigmaInf is zero
         %
         ztilde = [Y(:,t)-Yhat(:,t);zeros(n,1)];
         Ltilde = [SmoothData.LStarMat(:,:,d+t-Td) zeros(r,r);zeros(r,r) F];
         Sigmatilde = zeros(2*n,2*n);
         Sigmatilde(1:n,1:n) = pinv(SmoothData.SigmaStar(:,:,d+t-Td));
         %
         rtilde = (Htilde*Sigmatilde*ztilde)+(Ltilde'*rtilde);
         Ntilde = (Htilde*Sigmatilde*Htilde')+(Ltilde'*Ntilde*Ltilde);
         stilde = Htilde*Sigmatilde*ztilde;
         Mtilde = Htilde*Sigmatilde*Htilde';
      else;
         %
         % univariate initial smoothing
         %
         sigma2iInf = SmoothData.sigma2iInfCell{d+t-Td};
         sigma2iStar = SmoothData.sigma2iStarCell{d+t-Td};
         KtiInf = SmoothData.KtiInfCell{d+t-Td};
         KtiStar = SmoothData.KtiStarCell{d+t-Td};
         zti = SmoothData.ztiCell{d+t-Td};
         Hti = SmoothData.HtiCell{d+t-Td};
         %
         % handle transitions from rtilde(t+1,0) to
         % rtilde(t,n) and similary for Ntilde,
         % where rtilde(t+1,0) = rtilde(t+1|T)
         % and Ntilde(t+1,0) = Ntilde(t+1|T)
         %
         rtilde = FtildeT*rtilde;
         Ntilde = FtildeT*Ntilde*Ftilde;
         stilde = zeros(2*r,1);
         Mtilde = zeros(2*r,2*r);
         for i=(n:-1:1);
            Hi = Hti(:,i);
            zi = zti(i);
            if sigma2iInf(i)>1.0e-10;
               LInfti = Ir-(KtiInf(:,i)*Hi'*(1/sigma2iInf(i)));
               L0ti = ((KtiInf(:,i)*(sigma2iStar(i)/sigma2iInf(i)))-KtiStar(:,i))*Hi'*(1/sigma2iInf(i));
               atilde = [zeros(r,1);Hi*(zi/sigma2iInf(i))];
               btilde = [zeros(r,r) Hi*Hi'*(1/sigma2iInf(i));Hi*Hi'*(1/sigma2iInf(i)) Hi*Hi'*(1/sigma2iInf(i)^2)];
               Ltilde = [LInfti L0ti;zeros(r,r) LInfti];
            else;
               LStarti = Ir-(KtiStar(:,i)*Hi'*(1/sigma2iStar(i)));
               Ltilde = [LStarti zeros(r,r);zeros(r,r) LStarti];
               atilde = [Hi*(zi/sigma2iStar(i));zeros(r,1)];
               btilde = [Hi*Hi'*(1/sigma2iStar(i)) zeros(r,r);zeros(r,2*r)];
            end;
            rtilde = atilde+(Ltilde'*rtilde);
            Ntilde = btilde+(Ltilde'*Ntilde*Ltilde);
            stilde = atilde+(Ltilde'*stilde);
            Mtilde = btilde+(Ltilde'*Mtilde*Ltilde);
         end;
      end;
      %
      % disturbance smoother estimates
      %
      rt = rtilde(1:r,1);
      Nt = Ntilde(1:r,1:r);
      %
      % compute the smooth estimates for the state variables
      %
      KsitT(:,t) = Ksitt1(:,t)+(Ptilde*rtilde);
      PtT(:,:,t) = PStart-(Ptilde*Ntilde*Ptilde');
      Ksitt(:,t) = Ksitt1(:,t)+(Ptilde*stilde);
      Ptt(:,:,t) = PStart-(Ptilde*Mtilde*Ptilde');
      if rSI<n;
         rttMat(:,t) = stilde(1:r,1);
      else;
         rttMat(:,t) = stilde(r+1:2*r,1);
      end;
      rTMatTmp(:,t) = rt;
      NtMatTmp(:,:,t) = Nt;
   end;
end;
%
if nargout>=6;
   rTMat = rTMatTmp;
   if nargout==7;
      NtMat = NtMatTmp;
   end;
end;

%
% end of DiffuseStateSmoother.m
%
