function [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaRank,SmoothData] = DiffuseUnivariateKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos)
% DiffuseUnivariateKalmanFilterMO: Calculates the value of the log-likelihood function for the observed data using the
%                                  state-space representation using diffuse initialization of the univariate Kalman filter,
%                                  allowing for missing observations.
%
% USAGE:
%
%       [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaRank,SmoothData] = DiffuseUnivariateKalmanFilterMO(Y,X,A,H,F,Q,R,...
%                                                                    KsiLast,StartPeriod,AllowUnitRoot,StationaryPos)
%
% REQUIRED INPUT:  Y (matrix) of dimension nxT with data on the observed variables.
%
%                  X (matrix) of dimension kxT with data on the exogenous/deterministic variables.
%
%                  A (matrix) of dimension kxn with parameter values on the k X variables in the measurement equation.
%
%                  H (matrix) of dimension rxn with parameter values on the r state variables (ksi) in the
%                    measurement equation.
%
%                  F (matrix) of dimension rxr with parameters for the transitions in the state equation.
%
%                  Q (matrix) of dimension rxr with covariance parameters for the shocks to the state equation.
%
%                  R (matrix) of dimension nxn with covariance parameters for the measurement errors.
%
%                  KsiLast (vector) o dimension r with initial values for the state variables.
%
%                  StartPeriod (integer) indicating first observation to use when calculating the log-likelihood value.
%
%                  AllowUnitRoot (boolean) which is 1 if unit roots should be allowed for 0 otherwise.
%
%                  StationaryPos (vector) with positions in the state vector of the stationary variables.
%
% REQUIRED OUTPUT: lnL (scalar) with the value of the log-likelihood function.
%
%                  status (integer) ndicating the covariance stationarity of the state variables. If all the
%                    eigenvalues of F are inside the unit circle, then status=0. If not, then status=1. In
%                    addition, status can be -1 if ths forecast covariance matrix is not positive definite for
%                    some time period after the initialization of the filter has finished.
%
% OPTIONAL OUTPUT: lnLt (vector) of dimension (T-StartPeriod+1) with the time t values of the log-likelihood
%                    function.
%
%                  Yhat (matrix) of dimension nx(T-StartPeriod+1) with the 1-step ahead forecasts of the
%                    observed variables.
%
%                  MSEY (matrix) of dimension nxnx(T-StartPeriod+1) with the 1-step ahead MSE-matrix for the
%                    observed variables.
%
%                  Ksitt1 (matrix) of dimension rx(T-StartPeriod+1) with the 1-step ahead forecasts of the
%                    state variables.
%
%                  Ptt1 (matrix) of dimension rxrx(T-StartPeriod+1) with the 1-step ahead MSE-matrix for the
%                    state variables.
%
%                  SigmaRank (matrix) of dimension nxd with the rank of \Sigma(\infty,t,i) during the initialization
%                    phase.
%
%                  SmoothData (structure) with fields containing data that were used by the diffuse Kalman smoother
%                    during the initialization phase. The fields are given by:
%
%                              PInf (matrix) of dimension rxrxd with the values of P(\infty,t|t-1) during the initialization
%                                       sample.
%
%                              PStar (matrix) of dimension rxrxd with the values of P(\star,t|t-1) during the initialization
%                                       sample.
%
%                              sigma2iInfCell (cell array) of dimension 1xd with vectors of dimension n containing the
%                                       values of Sigma(\infty,t,i).
%
%                              sigma2iStarCell (cell array) of dimension 1xT with vectors of dimension n containing the
%                                       values of Sigma(\star,t,i).
%
%                              KtiInfCell (cell array) of dimension 1xd with matrices of dimension rxn with the Kalman
%                                       gain vectors K(\infty,t,i) in its columns.
%
%                              KtiStarCell (cell array) of dimension 1xT with matrices of dimension rxn with the Kalman
%                                       gain vectors K(\star,t,i) in its columns.
%
%                              ztiCell (cell array) of dimension 1xT with vectors of dimension n with the prediction
%                                       errors z(t,i).
%
%                              HtiCell (cell array) of dimension 1xT with matrices of dimension rxn with the transformed
%                                       H matrices.
%
%                              d (integer) with the length of the initialization sample.
%
%                              T (integer) with the length of the full sample.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 25, 2011.
%                        This version: January 16, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-10-2011: Updated the documentation.
%
% * 08-11-2011: Added the fields "PInf" and "PStar" to "SmoothData".
%
% * 22-12-2011: Made sure that all fields in SmoothData ar available also when d=T. Added an
%               additional requirement for PInfLast to be nonzero.
%
% * 03-01-2012: Updated the documentation.
%
% * 20-03-2012: The function now initializes the variance of known unit-root processes to unity.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 02-09-2015: The recursive estimation breaks if the log-likelihood is a NaN.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% initializing
%
lnL = 0;
status = 0;
%
% check the covariance stationarity of the state vector.
%
if max(abs(eig(F(StationaryPos,StationaryPos))))>=1;
   if AllowUnitRoot==0;
      status = 1;
   elseif (AllowUnitRoot==1)&(max(abs(eig(F(StationaryPos,StationaryPos))))>1)
      status = 1;
   end;
end;
%
% Determine dimensions. Checking if dimensions match is not undertaken!
%
[n,T] = size(Y);
if isempty(X)==0;
   Ax = A'*X;
else;
   Ax = zeros(n,T);
end;
Y = Y-Ax;
HT = H';
r = size(H,1);
d = T;
%
% Adjust data for measurement error covariance matrix if we need to
% use the univariate Kalman filter during initialization
%
if isequal(R,diag(diag(R)))==1;
   Hs = H;
   S = eye(n);
   Rstar = R;
else;
   [S,L] = eig(R);
   Hs = H*S;
   Rstar = L;
end;
HsT = Hs';
%
% initialize additional output variables
%
if nargout>2;
   MSEYTmp = NaN*ones(n,n,T);
   YhatTmp = NaN*ones(n,T);
   Ksitt1Tmp = zeros(r,T);
   Ptt1Tmp = zeros(r,r,T);
   SigmaRankTmp = NaN*ones(n,T);
   sigma2iInfCell = cell(1,T);
   sigma2iStarCell = cell(1,T);
   KtiInfCell = cell(1,T);
   KtiStarCell = cell(1,T);
   ztiCell = cell(1,T);
   HtiCell = cell(1,T);
   SmoothData = [];
end;
lnLtTmp = zeros(1,T);
%
if nargout==2;
   %
   if r==length(StationaryPos);
      PStarLast = zeros(r,r);
      PInfLast = eye(r);
   else;
      PInfLast = zeros(r,1);
      PInfLast(StationaryPos) = 1;
      PInfLast = diag(PInfLast);
      PStarLast = zeros(r,1);
      UnitRootPos = setdiff((1:r),StationaryPos);
      PStarLast(UnitRootPos,1) = 1;
      PStarLast = diag(PStarLast);
   end;
   t = 0;
   InitConv = 0;
   while (InitConv==0)&(t<T);
      t = t+1;
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      lnLConst = -n1*log(2*pi);
      if n1>0;
         Yt = S(pos,pos)'*Y(pos,t);
         Ht = Hs(:,pos);
         Rt = Rstar(pos,pos);
      end;
      Done = 0;
      for i=1:n1;
         Hi = Ht(:,i);
         sigma2Star = (Hi'*PStarLast*Hi)+Rt(i,i);
         sigma2Inf = Hi'*PInfLast*Hi;
         KtInf = PInfLast*Hi;
         KtStar = PStarLast*Hi;
         zt = Yt(i)-(Hi'*KsiLast);
         %
         if sigma2Inf>1.0e-10;
            lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2Inf);
            KsiLast = KsiLast+((zt/sigma2Inf)*KtInf);
            PInfLast = PInfLast-((1/sigma2Inf)*KtInf*KtInf');
            PStarLast = PStarLast+((sigma2Star/(sigma2Inf^2))*KtInf*KtInf')-((1/sigma2Inf)*((KtStar*KtInf')+(KtInf*KtStar')));
            if (rank(PInfLast,1.0e-6)==0)|(max(diag(PInfLast))<=1.0e-6);
               Done = 1;
               iStar = i;
            end;
         else;
            lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2Star)+((zt^2)/sigma2Star);
            KsiLast = KsiLast+((zt/sigma2Star)*KtStar);
            PInfLast = PInfLast;
            PStarLast = PStarLast-((1/sigma2Star)*KtStar*KtStar');
         end;
      end;
      lnLtTmp(1,t) = lnLConst-lnLtTmp(1,t);
      if isnan(lnLtTmp(1,t))==1;
         break;
      end;
      %
      % transition stage
      %
      KsiLast = F*KsiLast;
      PInfLast = F*PInfLast*F';
      PStarLast = (F*PStarLast*F')+Q;
      if Done==1;
         InitConv = 1;
         d = t;
      end;
   end;
   %
   % run the univariate Kalman filter for missing observations
   %
   PLast = PStarLast;
   for t=d+1:T;
      %
      % compute forecast error and MSE-matrix
      %
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      lnLConst = -n1*log(2*pi);
      %
      if n1>0;
         Yt = S(pos,pos)'*Y(pos,t);
         Ht = Hs(:,pos);         
         Rt = Rstar(pos,pos);
      end;
      %
      % initialize
      %
      for i=1:n1;
         Hi = Ht(:,i);
         sigma2 = (Hi'*PLast*Hi)+ Rt(i,i);
         if sigma2>0;
            Kt = PLast*Hi;
            zt = Yt(i)-(Hi'*KsiLast);
            %
            lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2)+((zt^2)/sigma2);
            %
            % update the state and the cov matrix
            %
            KsiLast = KsiLast+(Kt*(zt/sigma2));
            PLast = PLast-((1/sigma2)*Kt*Kt');
         end;
      end;
      lnLtTmp(1,t) = lnLConst-lnLtTmp(1,t);
      if isnan(lnLtTmp(1,t))==1;
         break;
      end;
      %
      % transition stage
      %
      KsiLast = F*KsiLast;
      PLast = (F*PLast*F')+Q;
   end;
else;
   %
   if r==length(StationaryPos);
      PStarLast = zeros(r,r);
      PInfLast = eye(r);
   else;
      PInfLast = zeros(r,1);
      PInfLast(StationaryPos) = 1;
      PInfLast = diag(PInfLast);
      PStarLast = zeros(r,1);
      UnitRootPos = setdiff((1:r),StationaryPos);
      PStarLast(UnitRootPos,1) = 1;
      PStarLast = diag(PStarLast);
   end;
   PInf = zeros(r,r,T);
   PStar = zeros(r,r,T);
   t = 0;
   InitConv = 0;
   while (InitConv==0)&(t<T);
      t = t+1;
      Ksitt1Tmp(:,t) = KsiLast;
      Ptt1Tmp(:,:,t) = Inf;
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      lnLConst = -n1*log(2*pi);
      if n1>0;
         if rank(HsT(pos,:)*PInfLast*Hs(:,pos),10e-10)>=1;
            MSEYTmp(pos,pos,t) = Inf;
         else;
            MSEYTmp(pos,pos,t) = (HT(pos,:)*PStarLast*H(:,pos))+R(pos,pos);
         end;
         YhatTmp(pos,t) = Ax(pos,t)+(HT(pos,:)*KsiLast);
         Yt = S(pos,pos)'*Y(pos,t);
         Ht = Hs(:,pos);
         Rt = Rstar(pos,pos);
      end;
      PStar(:,:,t) = PStarLast;
      PInf(:,:,t) = PInfLast;
      Done = 0;
      %
      % initialize variables to store in cell arrays
      %
      sigma2Infit = zeros(1,n1);
      sigma2Starit = zeros(1,n1);
      KtInfit = zeros(r,n1);
      KtStarit = zeros(r,n1);
      ztit = zeros(1,n1);
      for i=1:n1;
         Hi = Ht(:,i);
         sigma2Star = (Hi'*PStarLast*Hi)+Rt(i,i);
         sigma2Inf = Hi'*PInfLast*Hi;
         KtInf = PInfLast*Hi;
         KtStar = PStarLast*Hi;
         zt = Yt(i)-(Hi'*KsiLast);
         %
         % store the intermediate results
         %
         sigma2Starit(i) = sigma2Star;
         KtInfit(:,i) = KtInf;
         KtStarit(:,i) = KtStar;
         ztit(i) = zt;
         %
         if sigma2Inf>1.0e-10;
            SigmaRankTmp(i,t) = 1;
            sigma2Infit(i) = sigma2Inf;
            lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2Inf);
            KsiLast = KsiLast+((zt/sigma2Inf)*KtInf);
            PInfLast = PInfLast-((1/sigma2Inf)*KtInf*KtInf');
            PStarLast = PStarLast+((sigma2Star/(sigma2Inf^2))*KtInf*KtInf')-((1/sigma2Inf)*((KtStar*KtInf')+(KtInf*KtStar')));
            if (rank(PInfLast,1.0e-6)==0)|(max(diag(PInfLast))<=1.0e-6);
               Done = 1;
               iStar = i;
            end;
         else;
            lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2Star)+((zt^2)/sigma2Star);
            KsiLast = KsiLast+((zt/sigma2Star)*KtStar);
            PInfLast = PInfLast;
            PStarLast = PStarLast-((1/sigma2Star)*KtStar*KtStar');
         end;
      end;
      lnLtTmp(1,t) = lnLConst-lnLtTmp(1,t);
      if isnan(lnLtTmp(1,t))==1;
         break;
      end;
      %
      % store the intermediate results
      %
      sigma2iInfCell{t} = sigma2Infit;
      sigma2iStarCell{t} = sigma2Starit;
      KtiInfCell{t} = KtInfit;
      KtiStarCell{t} = KtStarit;
      ztiCell{t} = ztit;
      HtiCell{t} = Hs;
      %
      % transition stage
      %
      KsiLast = F*KsiLast;
      PInfLast = F*PInfLast*F';
      PStarLast = (F*PStarLast*F')+Q;
      if Done==1;
         InitConv = 1;
         d = t;
         SmoothData.PInf = PInf(:,:,1:d);
         SmoothData.PStar = PStar(:,:,1:d);
      elseif (Done==0)&(t==T);
         SmoothData.PInf = PInf(:,:,1:d);
         SmoothData.PStar = PStar(:,:,1:d);
      end;
   end;
   %
   % run the univariate Kalman filter for missing observations
   %
   PLast = PStarLast;
   for t=d+1:T;
      %
      % compute forecast error and MSE-matrix
      %
      Ksitt1Tmp(:,t) = KsiLast;
      Ptt1Tmp(:,:,t) = PLast;
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      lnLConst = -n1*log(2*pi);
      %
      if n1>0;
         Yt = S(pos,pos)'*Y(pos,t);
         Ht = Hs(:,pos);         
         Rt = Rstar(pos,pos);
         MSEYTmp(pos,pos,t) = (HT(pos,:)*PLast*H(:,pos))+R(pos,pos);
         YhatTmp(pos,t) = Ax(pos,t)+(HT(pos,:)*KsiLast);
      end;
      %
      % initialize
      %
      sigma2it = zeros(1,n1);
      Ktit = zeros(r,n1);
      ztit = zeros(1,n1);
      for i=1:n1;
         Hi = Ht(:,i);
         sigma2 = (Hi'*PLast*Hi)+ Rt(i,i);
         if sigma2>0;
            Kt = PLast*Hi;
            zt = Yt(i)-(Hi'*KsiLast);
            sigma2it(i) = sigma2;
            Ktit(:,i) = Kt;
            ztit(i) = zt;
            %
            lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2)+((zt^2)/sigma2);
            %
            % update the state and the cov matrix
            %
            KsiLast = KsiLast+(Kt*(zt/sigma2));
            PLast = PLast-((1/sigma2)*Kt*Kt');
         end;
      end;
      %
      % store the intermediate results
      %
      lnLtTmp(1,t) = lnLConst-lnLtTmp(1,t);
      if isnan(lnLtTmp(1,t))==1;
         break;
      end;
      sigma2iStarCell{t} = sigma2it;
      KtiStarCell{t} = Ktit;
      ztiCell{t} = ztit;
      HtiCell{t} = Ht;
      %
      % transition stage
      %
      KsiLast = F*KsiLast;
      PLast = (F*PLast*F')+Q;
   end;
end;
lnL = (1/2)*sum(lnLtTmp(1,StartPeriod:T));
%
% test if the log-likelihood is real
%
if (isreal(lnL)==0)|(isnan(lnL)==1);
   status = -1;
end;
%
% Fix output to match StartPeriod
%
if nargout>2;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   SigmaRank = SigmaRankTmp(:,1:d);
   SmoothData.sigma2iStarCell = sigma2iStarCell;
   SmoothData.KtiStarCell = KtiStarCell;
   SmoothData.ztiCell = ztiCell;
   SmoothData.HtiCell = HtiCell;
   SmoothData.sigma2iInfCell = sigma2iInfCell(1:d);
   SmoothData.KtiInfCell = KtiInfCell(1:d);
   SmoothData.d = d;
   SmoothData.T = T;
end;

%
% end of DiffuseUnivariateKalmanFilterMO.m
%
