function [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = SquareRootKalmanFilterMOHt(Y,X,A,H,F,B0,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c)
% SquareRootKalmanFilterMOHt: Computes the square root Kalman filter for a state-space model with the time-varying matrix on
%                             the state variables in the measurement equation. The function allows for missing observations.
%
% USAGE:
%
%       [lnL,status,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = SquareRootKalmanFilterMOHt(Y,X,A,H,F,B0,R, ...
%                                                                   KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c)
%
% REQUIRED INPUT:  Y (matrix) of dimension nxT with observed data.
%
%                  X (matrix) of dimension kxT with exogenous data.
%
%                  A (matrix) of dimension kxn with parameter values on the k X variables in the measurement equation.
%
%                  H (matrix) of dimension rxnxT with parameter values on the r state variables (ksi) in the measurement equation.
%
%                  F (matrix) of dimension rxr with parameter values on lagged state variables in the state equation.
%
%                  B0 (matrix) of dimension rxq with parameters on the standardized innovations in the state equation.
%
%                  R (matrix) of dimension nxn with covariances for the measurement errors.
%
%                  KsiLast (vector) of dimension rx1 with initial values for the state vector.
%
%                  initP (integer) indicating the method of determining initial value for P(1|0), the MSE matrix for
%                        ksi(1|0). If initP=1 (default), the P(1|0) is given by the unconditional covariance matrix of ksi,
%                        calculated through the vec operator. Since this operation can be excessively slow for large systems
%                        (when r^2 is a large number) the Doubling Algorthm can be chosen to approximate the unconditional
%                        variance. This algorithm is called if initP=2. Finally, if initP=3, then P(1|0) = c*eye(r*r).
%                  NOTE: If F has eigenvalues outside the unit circle, then initP is always set to 3!
%
%                  MaxIter (integer) with the maximum number of iterations to use with the doubling algorithm
%                        divided by 100. Must be multiplied by 100 when calling DoublingAlgorithmLyapunov.
%
%                  Tolerance (integer) which determines the tolerance value for the doubling algorithm.
%
%                  StartPeriod (integer) which determines the first observation to use when calculating the log-likelihood
%                        value.
%
%                  AllowUnitRoot (boolean) which is 1 if unit roots should be allowed for 0 otherwise.
%
%                  c (scalar) indicating the variance of P(1|0) when initP=3. The default value is 1000.
%
% REQUIRED OUTPUT: lnL (scalar) with the overall value of the log-likelihood function.
%
%                  status (integer) indicating the covariance stationarity of the state variables. If all the
%                        eigenvalues of F are inside the unit circle, then status=0 and the log-likelihood function
%                        is calculated. If not, then status=1 and initP is set to 3. In addition, status can be
%                        -1 if ths forecast covariance matrix is not positive definite for some time period.
%
% OPTIONAL OUTPUT: lnLt (vector) of dimension 1x(T-StartPeriod+1) with the time t values of the log-densities that
%                        sum to the value lnL.
%
%                  Yerror (matrix) of dimension nx(T-StartPeriod+1) with the 1-step ahead forecast errors of the
%                        observed variables.
%
%                  Yhat (matrix) of dimension nx(T-StartPeriod+1) with the 1-step ahead forecasts of the observed
%                        variables.
%
%                  SigmaSqRoot (matrix) of dimension nxnx(T-StartPeriod+1) with the lower traingular square root
%                        of the inverse of the 1-step ahead forecast error covariance matrix of the observed
%                        variables.
%
%                  MSEY (matrix) of dimension nxnx(T-StartPeriod+1) with the 1-step aheaf forecast error covariances
%                        of the observed variables.
%
%                  InvMSEY (matrix) of dimension nxnxx(T-StartPeriod+1) with the inverted 1-step ahead forecast error
%                        covariances of the observed variables.
%
%                  KalmanGain (matrix) of dimension rxnxx(T-StartPeriod+1) with the Kalman gain estimates from
%                        the square root filter.
%
%                  Ksitt1 (matrix) of dimension rxx(T-StartPeriod+1) with the 1-step ahead forecasts of the
%                        state variables.
%
%                  Ptt1 (matrix) of dimension rxrxx(T-StartPeriod+1) with the 1-step ahead forecast error
%                        covariances of the state variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 14, 2011.
%                        This version: January 16, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-03-2011: Updated the documentation.
%
% * 29-03-2011: Corrected the nargout checks.
%
% * 03-01-2012: Updated the documentation.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 03-01-2013: Updated the documentation.
%
% * 26-06-2013: The predicted values of the observables now also cover variables that are missing.
%
% * 27-01-2014: Updated the documentation.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 13-01-2015: Updated the documentation.
%
% * 02-09-2015: The recursive estimation breaks if the log-likelihood is a NaN.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 17-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initializing
%
lnL = 0;
status = 0;
if nargout>2;
   lnLt = [];
end;
%
% check the covariance stationarity of the state vector.
%
if max(abs(eig(F)))>=1;
   if AllowUnitRoot==0;
      status = 1;
   elseif (AllowUnitRoot==1)&(max(abs(eig(F)))>1)
      status = 1;
   end;
   initP = 3;
end;
%
% Determine dimensions. Checking if dimensions match is not undertaken!
%
[n,T] = size(Y);
if isempty(X)==0;
   Ax = A'*X;
else;
   Ax = zeros(n,T);
end;
r = size(H,1);
Y = Y-Ax;
q = size(B0,2);
%
% Compute initial values
%
if initP==1;
   PLast = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r);
elseif initP==2;
   %
   % The maximum number of iterations is set to MaxIter*100, while the convergence value is 1.0e-8
   %
   [PLast,stat] = DoublingAlgorithmLyapunov(F,B0*B0',MaxIter*100,10^(-(Tolerance+1)));
else;
   if c<=0;
      c = 1000;
   end;
   PLast = c*eye(r);
end;
if size(R,1)<n;
   R = zeros(n,n);
end;
%
% compute lower triangular PLast via eigendecomp and QR factorization
%
PSq = PSDCholeski(PLast);
%
% compute the lower triangular square-root matrix of R
%
RSq = PSDCholeski(R);
%
% initialize
%
lnLtTmp = zeros(1,T);
%
% start the filter
%
ln2pi = -log(2*pi);
if nargout<=2;
   for t=1:T;
      %
      % compute the forecast error
      %
      HT = H(:,:,t)';
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      if n1>0;
         Yerror = Y(pos,t)-(HT(pos,:)*KsiLast);
         %
         % set up the umatrix
         %
         U = [HT(pos,:)*PSq RSq(pos,pos) zeros(n1,q);F*PSq zeros(r,n1) B0];
         %
         % compute a QR factorization of U'
         %
         [G,US] = qr(U');
         US = US';
         Ustar1 = pinv(US(1:n1,1:n1));
         Ustar2 = US(n1+1:n1+r,1:n1);
         PSq = US(n1+1:n1+r,n1+1:n1+r);
         %
         % compute the new state variable forecast
         %
         KsiLast = (F*KsiLast)+(Ustar2*Ustar1*Yerror);
         %
         % compute the log-likelihood
         %
         InvMSEY = Ustar1'*Ustar1;
         lnLtTmp(1,t) = (n1*ln2pi)+LogDet(InvMSEY)-(Yerror'*InvMSEY*Yerror);
         if isnan(lnLtTmp(1,t))==1;
            break;
         end;
      else;
         U = [F*PSq B0];
         [G,US] = qr(U');
         US = US';
         PSq = US(:,1:r);
         KsiLast = F*KsiLast;
      end;
   end;
else;
   %
   % initialize
   %
   Yerror = NaN*ones(n,T);
   Yhat = NaN*ones(n,T);
   SigmaSqRoot = NaN*ones(n,n,T);
   InvMSEY = NaN*ones(n,n,T);
   MSEY = NaN*ones(n,n,T);
   KalmanGain = NaN*ones(r,n,T);
   Ksitt1 = zeros(r,T);
   Ptt1 = zeros(r,r,T);
   for t=1:T;
      %
      % store the state forecasts
      %
      HT = H(:,:,t)';
      Ksitt1(:,t) = KsiLast;
      Ptt1(:,:,t) = PSq*PSq';
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      %
      % compute the forecast errors of the observed variables
      %
      if n1>0;
         Yerror(pos,t) = Y(pos,t)-(HT(pos,:)*KsiLast);
         Yhat(:,t) = Ax(:,t)+HT*KsiLast;
         %Yhat(pos,t) = Y(pos,t)+Ax(pos,t)-Yerror(pos,t);
         %
         % set up the umatrix
         %
         U = [HT(pos,:)*PSq RSq(pos,pos) zeros(n1,q);F*PSq zeros(r,n1) B0];
         %
         % compute a QR factorization of U'
         %
         [G,US] = qr(U');
         US = US';
         Ustar1 = pinv(US(1:n1,1:n1));
         SigmaSqRoot(pos,pos,t) = Ustar1;
         Ustar2 = US(n1+1:n1+r,1:n1);
         PSq = US(n1+1:n1+r,n1+1:n1+r);
         %
         % compute the Kalman gain
         %
         KalmanGain(:,pos,t) = Ustar2*Ustar1;
         %
         % compute the new state variable forecast
         %
         KsiLast = (F*KsiLast)+(KalmanGain(:,pos,t)*Yerror(pos,t));
         %
         % compute the log-likelihood
         %
         MSEY(pos,pos,t) = US(1:n1,1:n1)*US(1:n1,1:n1)';
         InvMSEY(pos,pos,t) = Ustar1'*Ustar1;
         lnLtTmp(1,t) = (n1*ln2pi)+LogDet(InvMSEY(pos,pos,t))-(Yerror(pos,t)'*InvMSEY(pos,pos,t)*Yerror(pos,t));
         if isnan(lnLtTmp(1,t))==1;
            break;
         end;
      else;
         Yhat(:,t) = Ax(:,t)+HT*KsiLast;
         U = [F*PSq B0];
         [G,US] = qr(U');
         US = US';
         PSq = US(:,1:r);
         KsiLast = F*KsiLast;
      end;
   end;
end;
lnL = (1/2)*sum(lnLtTmp(1,StartPeriod:T));
%
% test if the log-likelihood is real
%
if (isreal(lnL)==0)|(isnan(lnL)==1);
   status = -1;
end;
%
if nargout>=3;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = Yerror(:,StartPeriod:T);
   Yhat = Yhat(:,StartPeriod:T);
   SigmaSqRoot = SigmaSqRoot(:,:,StartPeriod:T);
   MSEY = MSEY(:,:,StartPeriod:T);
   InvMSEY = InvMSEY(:,:,StartPeriod:T);
   KalmanGain = KalmanGain(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1(:,StartPeriod:T);
   Ptt1 = Ptt1(:,:,StartPeriod:T);
end;

%
% end of SquareRootKalmanFilterMOHt.m
%
