function [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = StateSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0)
% StateSmoother: Calculates smoothed and updated values of the state variables from the Kalman filter, as well as
%                the MSE matrices for these predictors.
%
% USAGE:
%
%       [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = StateSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0)
%
% The measurement equation is given by:
%
%                      y(t) = A'*x(t) + H'*ksi(t) + w(t)
%
% while the state equation is:
%
%                      ksi(t) = F*ksi(t-1) + v(t)
%
% while w(t) and v(t) are white noise with zero mean and covariance matrices R and Q, respectively. The vector x(t) is
% predetermined. Furthermore, ksi(1) is uncorrelated with v(t) and w(t).
%
% The updated and smooth predictions along with their MSE matrices are calculated using the formulas from
% Koopman & Harvey (2003, JEDC). They are also given in Warne (2009).
%
%
% REQUIRED INPUTS:  Y (nxT) matrix with observed data.
%
%                   Yhat (nxT) matrix with the forecasted data (1 step ahead forecasts).
%
%                   Ksitt1 (rxT) matrix with the forecasted state vector (1 step ahead).
%
%                   Ptt1 (rxrxT) 3 dimensional matrix with the MSEs for the forecasted state vector.
%
%                   H (rxn) matrix with parameter values on the r state variables (ksi) in the measurement equation.
%
%                   F (rxr) matrix with parameter values on lagged state variables in the state equation.
%
%                   R (nxn) positive semidefinite matrix with covariances for the innovations in the measurement equation.
%
%                   B0 (rxq) matrix with coefficients on the structural shocks in the state equation.
%
% REQUIRED OUTPUTS: Ksitt (rxT) matrix with the updated predictions of the state vector.
%
%                   Ptt (rxrxT) 3 dimensional matrix with the MSEs of the updated predictions of the state vector.
%
%                   KsitT (rxT) matrix with the smooth predictions of the state vector.
%
%                   PtT (rxrxT) 3 dimensional matrix with the MSEs of the smooth predictions of the state vector.
%
%                   rttMat (rxT) matrix with standardized forecast errors.
%
% OPTIONAL OUTPUTS: rTMat (rxT) matrix with the r(t|T)-vectors.
%
%                   NtMat (rxrxT) matrix with the Nt matrices.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: July 14, 2006.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-07-2006: Updated function information.
%
% * 22-08-2006: Added feature to handle valid reduced rank issues of the Ptt1 matrix.
%
% * 17-10-2006: Added call to DecomposeStateMSE function both for P10 matrix, and for
%               the Ptt1 matrix. This deals with the case when Ptt1 has less than full
%               rank and is different from P10 for some t.
%
% * 06-11-2006: Changed the computation of the J_t matrix such that the m*m' appearing
%               before and after Ptt(:,:,t) are removed.
%
% * 08-11-2006: Took the name change of DecomposeStateMSE into SchurDecomposition into account.
%
% * 14-11-2006: Added the optional inputs P01, Ksi01, Yerr01 and the optional output Ksi0T. Moreover, the
%               rank of Ptt1 at time t+1 always determines m matrix from the possible Schur decomposition.
%
% * 15-11-2006: Made some checks, No changes to the function.
%
% * 23-11-2006: Updated the documentation.
%
% * 04-01-2007: Removed the output "Ksi0T" and the inputs "P10", "P01", "Ksi01", "Yerr01" as they are no longer needed.
%
% * 20-02-2007: Added the calculation of "rttMat", i.e., a 1-step smoother for all time periods.
%
% * 05-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 04-07-2008: Updated the documentation.
%
% * 18-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 27-08-2009: Changed the algorithm for the Kalman smoother to the one from Koopman & Harvey (2003).
%
% * 12-10-2009: Introduced the Q and HT matrices. The "Ptt1H" is now also used by the "InvMSEY"
%               matrix. These changes should improve the speed a bit.
%
% * 05-01-2010: Updated the documentation.
%
% * 17-02-2010: Added the optional "NtMat" output variable.
%
% * 24-08-2010: The function now tests if the rank of Q and R supports a standard inverse or requires
%               a pseudo-inverse of the 1-step forecast covariance matrix of the observed variables.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 17-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
% * 26-02-2024: Changed the rtvec output variable to rTMat.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[r,T] = size(Ksitt1);
n = size(R,1);
Q = B0*B0';
HT = H';
In = eye(n);
%
% Initializa the output
%
Ksitt = zeros(r,T);
Ptt = zeros(r,r,T);
KsitT = zeros(r,T);
PtT = zeros(r,r,T);
rttMat = zeros(r,T);
rTMatTmp = zeros(r,T);
NtMatTmp = zeros(r,r,T);
%
% we loop from period T and backwards to t=1
%
if rank(B0)+rank(R)>=n;
   for t=(T:-1:1);
      Ptt1H = Ptt1(:,:,t)*H;
      InvMSEY = In/((HT*Ptt1H)+R);
      %
      % calculate the updated state prediction and its MSE
      %
      Ksitt(:,t) = Ksitt1(:,t)+(Ptt1H*InvMSEY*(Y(:,t)-Yhat(:,t)));
      Ptt(:,:,t) = Ptt1(:,:,t)-(Ptt1H*InvMSEY*Ptt1H');
      if t<T;
         KalmanGain = F*Ptt1H*InvMSEY;
         Lt = F-(KalmanGain*HT);
         rt = (H*InvMSEY*(Y(:,t)-Yhat(:,t)))+(Lt'*rt);
         Nt = (H*InvMSEY*HT)+(Lt'*Nt*Lt);
         KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
         PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
      else;
         %
         % Updates and smooth predictions and MSEs have the same values for t=T
         %
         KsitT(:,T) = Ksitt(:,T);
         PtT(:,:,T) = Ptt(:,:,T);
         rt = H*InvMSEY*(Y(:,t)-Yhat(:,t));
         Nt = H*InvMSEY*HT;
      end;
      rttMat(:,t) = H*InvMSEY*(Y(:,t)-Yhat(:,t));
      rTMatTmp(:,t) = rt;
      NtMatTmp(:,:,t) = Nt;
   end;
else;
   for t=(T:-1:1);
      Ptt1H = Ptt1(:,:,t)*H;
      InvMSEY = pinv((HT*Ptt1H)+R);
      %
      % calculate the updated state prediction and its MSE
      %
      Ksitt(:,t) = Ksitt1(:,t)+(Ptt1H*InvMSEY*(Y(:,t)-Yhat(:,t)));
      Ptt(:,:,t) = Ptt1(:,:,t)-(Ptt1H*InvMSEY*Ptt1H');
      if t<T;
         KalmanGain = F*Ptt1H*InvMSEY;
         Lt = F-(KalmanGain*HT);
         rt = (H*InvMSEY*(Y(:,t)-Yhat(:,t)))+(Lt'*rt);
         Nt = (H*InvMSEY*HT)+(Lt'*Nt*Lt);
         KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
         PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
      else;
         %
         % Updates and smooth predictions and MSEs have the same values for t=T
         %
         KsitT(:,T) = Ksitt(:,T);
         PtT(:,:,T) = Ptt(:,:,T);
         rt = H*InvMSEY*(Y(:,t)-Yhat(:,t));
         Nt = H*InvMSEY*HT;
      end;
      rttMat(:,t) = H*InvMSEY*(Y(:,t)-Yhat(:,t));
      rTMatTmp(:,t) = rt;
      NtMatTmp(:,:,t) = Nt;
   end;
end;
if nargout>=6;
   rTMat = rTMatTmp;;
   if nargout==7;
      NtMat = NtMatTmp;
   end;
end;

%
% end of StateSmoother.m
%
