function [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = StateSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0)
% StateSmootherMO: Calculates smoothed and updated values of the state variables from the Kalman filter, as well as
%                  the MSE matrices for these predictors. Allows for missing observations.
%
% USAGE:
%
%       [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = StateSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0)
%
% REQUIRED INPUT:  Y (matrix) of dimension nxT with observed data.
%
%                  Yhat (matrix) of dimension nxT with the forecasted data (1 step ahead forecasts).
%
%                  Ksitt1 (matrix) of dimension rxT with the forecasted state vector (1 step ahead).
%
%                  Ptt1 (matrix) of dimension rxrxT with the MSEs for the forecasted state vector.
%
%                  H (matrix) of dimension rxn with parameter values on the r state variables (ksi) in the measurement equation.
%
%                  F (matrix) of dimension rxr with parameter values on lagged state variables in the state equation.
%
%                  R (matrix) of dimension nxn with covariances for the innovations in the measurement equation.
%
%                  B0 (matrix) of dimension rxq with coefficients on the structural shocks in the state equation.
%
% REQUIRED OUTPUT: Ksitt (matrix) of dimension rxT with the updated predictions of the state vector.
%
%                  Ptt (matrix) of dimension rxrxT with the MSEs of the updated predictions of the state vector.
%
%                  KsitT (matrix) of dimension rxT with the smooth predictions of the state vector.
%
%                  PtT (matrix) of dimension rxrxT with the MSEs of the smooth predictions of the state vector.
%
%                  rttMat (matrix) of dimension rxT with standardized forecast errors.
%
% OPTIONAL OUTPUT: rTMat (matrix) of dimension rxT with the r(t|T)-vectors.
%
%                  NtMat (matrix) of dimension rxrxT with the Nt matrices.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 15, 2011.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-03-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
% * 26-02-2024: Changed the rtvec output variable to rTMat.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[r,T] = size(Ksitt1);
n = size(R,1);
Q = B0*B0';
HT = H';
In = eye(n);
%
% Initializa the output
%
Ksitt = zeros(r,T);
Ptt = zeros(r,r,T);
KsitT = zeros(r,T);
PtT = zeros(r,r,T);
rttMat = zeros(r,T);
rTMatTmp = zeros(r,T);
NtMatTmp = zeros(r,r,T);
%
% we loop from period T and backwards to t=1
%
if rank(B0)+rank(R)>=n;
   for t=(T:-1:1);
      %
      % check for missing observations
      %
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      if n1>0;
         Ptt1H = Ptt1(:,:,t)*H(:,pos);
         InvMSEY = In(pos,pos)/((HT(pos,:)*Ptt1H)+R(pos,pos));
         %
         % calculate the updated state prediction and its MSE
         %
         Ksitt(:,t) = Ksitt1(:,t)+(Ptt1H*InvMSEY*(Y(pos,t)-Yhat(pos,t)));
         Ptt(:,:,t) = Ptt1(:,:,t)-(Ptt1H*InvMSEY*Ptt1H');
         if t<T;
            KalmanGain = F*Ptt1H*InvMSEY;
            Lt = F-(KalmanGain*HT(pos,:));
            rt = (H(:,pos)*InvMSEY*(Y(pos,t)-Yhat(pos,t)))+(Lt'*rt);
            Nt = (H(:,pos)*InvMSEY*HT(pos,:))+(Lt'*Nt*Lt);
            KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
            PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
         else;
            %
            % Updates and smooth predictions and MSEs have the same values for t=T
            %
            KsitT(:,T) = Ksitt(:,T);
            PtT(:,:,T) = Ptt(:,:,T);
            rt = H(:,pos)*InvMSEY*(Y(pos,t)-Yhat(pos,t));
            Nt = H(:,pos)*InvMSEY*HT(pos,:);
         end;
         rttMat(:,t) = H(:,pos)*InvMSEY*(Y(pos,t)-Yhat(pos,t));
      else;
         Ksitt(:,t) = Ksitt1(:,t);
         Ptt(:,:,t) = Ptt1(:,:,t);
         if t<T;
            rt = F'*rt;
            Nt = F'*Nt*F;
            KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
            PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
         else;
            KsitT(:,T) = Ksitt(:,T);
            PtT(:,:,T) = Ptt(:,:,T);
            rt = zeros(r,1);
            Nt = zeros(r,r);
         end;
      end;
      rTMatTmp(:,t) = rt;
      NtMatTmp(:,:,t) = Nt;
   end;
else;
   for t=(T:-1:1);
      %
      % check for missing observations
      %
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      if n1>0;
         Ptt1H = Ptt1(:,:,t)*H(:,pos);
         InvMSEY = pinv((HT(pos,:)*Ptt1H)+R(pos,pos));
         %
         % calculate the updated state prediction and its MSE
         %
         Ksitt(:,t) = Ksitt1(:,t)+(Ptt1H*InvMSEY*(Y(pos,t)-Yhat(pos,t)));
         Ptt(:,:,t) = Ptt1(:,:,t)-(Ptt1H*InvMSEY*Ptt1H');
         if t<T;
            KalmanGain = F*Ptt1H*InvMSEY;
            Lt = F-(KalmanGain*HT(pos,:));
            rt = (H(:,pos)*InvMSEY*(Y(pos,t)-Yhat(pos,t)))+(Lt'*rt);
            Nt = (H(:,pos)*InvMSEY*HT(pos,:))+(Lt'*Nt*Lt);
            KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
            PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
         else;
            %
            % Updates and smooth predictions and MSEs have the same values for t=T
            %
            KsitT(:,T) = Ksitt(:,T);
            PtT(:,:,T) = Ptt(:,:,T);
            rt = H(:,pos)*InvMSEY*(Y(pos,t)-Yhat(pos,t));
            Nt = H(:,pos)*InvMSEY*HT(pos,:);
         end;
         rttMat(:,t) = H(:,pos)*InvMSEY*(Y(pos,t)-Yhat(pos,t));
      else;
         Ksitt(:,t) = Ksitt1(:,t);
         Ptt(:,:,t) = Ptt1(:,:,t);
         if t<T;
            rt = F'*rt;
            Nt = F'*Nt*F;
            KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
            PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
         else;
            KsitT(:,T) = Ksitt(:,T);
            PtT(:,:,T) = Ptt(:,:,T);
            rt = zeros(r,1);
            Nt = zeros(r,r);
         end;
      end;
      rTMatTmp(:,t) = rt;
      NtMatTmp(:,:,t) = Nt;
   end;
end;
if nargout>=6;
   rTMat = rTMatTmp;
   if nargout==7;
      NtMat = NtMatTmp;
   end;
end;

%
% end of StateSmootherMO.m
%
