function CorrelationDecompositionDLG(selector,CorrDec,VariableNames,VarStr,ModelStr,EstStr,DSGEModel,CurrINI)
% CorrelationDecompositionDLG: Displays correlation decompositions for the observed variables or the state
%                              variables.
%
% USAGE:
%
%       CorrelationDecompositionDLG(selector,CorrDec,VariableNames,VarStr,ModelStr,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT: selector (string) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: CorrDec (structure) with fields "Y", "Xi", "R", "AutoCovHorizon", "ShockNames", "ShockGroups",
%                         "ShockGroupNames", "ShockColors", and "ShockGroupColors". All optional variables are
%                         required when selector is 'init'
%
%                 VariableNames (string matrix) with the names of the variables for which the decompositions
%                         are performed.
%
%                 VarStr (string) that takes the values 'Observed Variables' or 'State Variables'.
%
%                 ModelStr (string) that takes the values 'DSGE' or 'DSGE-VAR'.
%
%                 EstStr (string) that takes the values 'Initial Values', 'Posterior Mode', 'Marginal Posterior
%                         Mode', or 'Joint Posterior Mode'.
%
%                 DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 18, 2011.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-01-2011: Made sure that the full bars are always visible, e.g., when a linear combination
%               of shares is greater than unity.
%
% * 05-04-2011: Added support for "DSGEModel.ShockColors".
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 10-09-2015: The function is now ready for Matlab 2014b and later.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 08-04-2021: Allowed the use of shock groups also when the number of such groups is
%               equal to the number of shocks.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.CorrDec = CorrDec;
   DSGEModel.VarStr = VarStr;
   DSGEModel.ModelStr = ModelStr;
   DSGEModel.EstStr = EstStr;
else;
   CorrelationDecompositionGUI = findobj('Type','figure','Tag','CorrelationDecompositionDLG');
   controls = get(CorrelationDecompositionGUI,'UserData');
   CurrINI = get(controls.firstvariable,'UserData');
   DSGEModel = get(controls.firstvariabletext,'UserData');
end;
%
% display the GUI
%
switch selector;
   case 'init';
      CorrelationDecompositionGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','CorrelationDecompositionDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-525)/2 (CurrINI.scrsz(4)-180)/2 525 180], ...
         'Tag','CorrelationDecompositionDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Correlation Decompositions - ' EstStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','CorrelationDecompositionDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(CorrelationDecompositionGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 523 170],[VarStr ' - ' ModelStr ' Model'],45,[0.5 1],'on',CurrINI);
      %
      % Select variable 1
      %
      controls.firstvariabletext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 120 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select first variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVar1Text');
      %
      controls.firstvariable = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 125 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVar1Popup',...
         'Value',1);
      %
      % select variable 2
      %
      controls.secondvariabletext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 90 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select second variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVar2Text');
      %
      controls.secondvariable = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 95 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVar2Popup',...
         'Value',1);
      %
      % plots for shock groups
      %
      controls.shockgroups = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 60 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Use shock groups', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','ShockgroupsCheckBox');
      %
      % black edge patches
      %
      controls.blackedgepatch = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 40 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Draw patches with black edges', ...
            'Value',1, ...
            'Enable','on', ...
            'Tag','BlackEdgePatchCheckBox');
      %
      % include measurement errors
      %
      controls.measureerror = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Include measurement errors when non-zero', ...
            'Value',0, ...
            'Enable','off', ...
            'Tag','MeasureErrorCheckBox');
      %
      if (strcmp(lower(VarStr),'observed variables')==1)&(strcmp(ModelStr,'DSGE')==1)&(rank(CorrDec.R)>0);
         set(controls.measureerror,'Enable','on');
      end;
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','CorrelationDecompositionDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','CorrelationDecompositionDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(CorrelationDecompositionGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'showgraph'
      FirstValue = get(controls.firstvariable,'Value');
      SecondValue = get(controls.secondvariable,'Value');
      VarStrMat = get(controls.firstvariable,'String');
      n = size(VarStrMat,1);
      FirstVariable = StringTrim(VarStrMat(FirstValue,:));
      SecondVariable = StringTrim(VarStrMat(SecondValue,:));
      if FirstValue<n;
         NewSecondValue = SecondValue;
         NewFirstValue = FirstValue+1;
      else;
         if SecondValue<n;
            NewFirstValue = SecondValue+1;
            NewSecondValue = SecondValue+1;
         else;
            NewFirstValue = 1;
            NewSecondValue = 1;
         end;
      end;
      set(controls.firstvariable,'Value',NewFirstValue);
      set(controls.secondvariable,'Value',NewSecondValue);
      %
      % determine which entry we should take
      %
      NumberMat = reshape(DuplicationMatrix(n)*(1:((n*(n+1))/2))',n,n);
      %
      % check if we have to flip the autocorrelations or not
      %
      if FirstValue>=SecondValue;
         AutoCovValue = NumberMat(FirstValue,SecondValue);
         DoFlip = 0;
      else;
         AutoCovValue = NumberMat(SecondValue,FirstValue);
         DoFlip = 1;
      end;
      %
      % determine the data to display
      %
      CorrDec = DSGEModel.CorrDec;
      AutoCovHorizon = (-CorrDec.AutoCovHorizon:CorrDec.AutoCovHorizon);
      if strcmp(lower(DSGEModel.VarStr),'observed variables')==1;
         CorrObs = CorrDec.Y;
         if strcmp(DSGEModel.ModelStr,'DSGE')==1;
            q = size(CorrObs,3)-1;
         else;
            q = size(CorrObs,3);
         end;
      else;
         CorrObs = CorrDec.Xi;
         q = size(CorrObs,3);
      end;
      % IMatrix = SubIdentityMatrix(CorrDec.ShockGroups);
      %
      % get the current autocorrelations
      %
      CorrVals = sum(CorrObs(AutoCovValue,:,:),3);
      DecompVals = shiftdim(CorrObs(AutoCovValue,:,:))';
      if DoFlip==1;
         CorrVals = fliplr(CorrVals);
         DecompVals = fliplr(DecompVals);
      end;
      Shocks = DecompVals(1:q,:);
      if get(controls.measureerror,'Value')==1;
         MError = DecompVals(q+1,:);
         MEStr = 'Measurement Error';
      else;
         MError = [];
         MEStr = '';
      end;
      %
      % check if we should look at shock groups instead of all shocks
      %
      if get(controls.shockgroups,'Value')==1;
         Shocks = SubIdentityMatrix(CorrDec.ShockGroups)'*Shocks;
         ShockGroupNames = '';
         for j=1:size(CorrDec.ShockGroupNames,1);
            ShockGroupNames = strvcat(ShockGroupNames,strrep(CorrDec.ShockGroupNames(j,:),'_',' '));
         end;
         ShockGroupColors = CorrDec.ShockGroupColors;
      else;
         ShockGroupNames = CorrDec.ShockAliases;
         for j=1:size(ShockGroupNames,1);
            ShockGroupNames(j,:) = strrep(ShockGroupNames(j,:),'_',' ');
         end;
         if isempty(CorrDec.ShockColors)==0;
            ShockGroupColors = CorrDec.ShockColors;
         else;
            ShockGroupColors = CreateDefaultColors(q);
         end;
      end;
      Shocks = [Shocks;MError];
      ShockGroupNames = strvcat(ShockGroupNames,MEStr);
      if isempty(MError)==0;
         ShockGroupColors = [ShockGroupColors;0.7 0.7 0.7];
      end;
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                     'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                     'Tag',[FirstVariable '-' SecondVariable], ...
                     'Name',['Correlation Decomposition - ' FirstVariable '(t-h) and ' SecondVariable '(t)']);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % Create the bars and collect the handles
      %
      if MatlabNumber>8.3;
         BarHandles = bar(AutoCovHorizon',Shocks','stacked');
      else;
         if MatlabNumber>=7.0;
            %
            % make sure the bars are written as patches
            %
            BarHandles = bar('v6',AutoCovHorizon',Shocks','stacked');
         else;
            BarHandles = bar(AutoCovHorizon',Shocks','stacked');
         end;
      end;
      %
      % remake the bar patches such that all bars are displayed properly.
      %
      BarHandles = RemakeBars(BarHandles,ShockGroupColors,get(controls.blackedgepatch,'Value'));
      %
      % plot the population autocorrelation
      %
      LineHandle = plot(AutoCovHorizon,CorrVals,'.-','Color',[0 0 0],'LineWidth',2);
      hold('on');
      %
      % set the title
      %
      SetAxesFonts(CurrINI,'Title',['Correlation decomposition for ' FirstVariable '(t-h) and ' SecondVariable '(t) at ' lower(DSGEModel.EstStr)],'XLabel','horizon (h)');
      axis('tight');
      %
      % see if we should add a zero line
      %
      ylim = get(gca,'YLim');
      add_on = 0.1*(ylim(2)-ylim(1));
      ylim = [ylim(1)-add_on ylim(2)+add_on];
      set(gca,'YLim',ylim);
      if (ylim(1)<0)&(ylim(2)>0);
         hold('on');
         plot([AutoCovHorizon(1) AutoCovHorizon(length(AutoCovHorizon))],[0 0],'-','Color',[0 0 0]);
         %
         % make sure the zero line is displayed below the observed variable
         %
         kids = get(gca,'Children');
         if size(kids,1)>size(kids,2);
            kids = kids';
         end;
         kids = [kids(2:length(kids)) kids(1)]';
         set(gca,'Children',kids);
      end;
      %
      % add a length
      %
      legHandle = legend([BarHandles LineHandle],strvcat(ShockGroupNames,'Correlations'));
      SetAxesFonts(CurrINI,legHandle);
      hold('off');
      %
      SetXTickIntegers(gca);
      %
      % make the figure visible
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(controls.done,'UserData','done');
      delete(CorrelationDecompositionGUI);
      drawnow;
      pause(0.02);
end;
if strcmp(selector,'init')==1;
   set(CorrelationDecompositionGUI,'Visible','on');
   drawnow;
   SetYADAIcon(CorrelationDecompositionGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of CorrelationDecompositionDLG.m
%
