function GraphLaplacePosteriorDensity(thetaMode,thetaInverseHessian,UpdateMode,theta,thetaDist,LowerBound,PriorDist,ParameterNames,thetaPositions,ModelParameters,lambda,EstStr,IsAdaptiveLearning,controls,DSGEModel,CurrINI)
% GraphLaplacePosteriorDensity: Computes Laplace approximations of the posterior densities and plots these against
%                               the prior densities.
%
% USAGE:
%
%       GraphLaplacePosteriorDensity(thetaMode,thetaInverseHessian,UpdateMode,theta,thetaDist,LowerBound,PriorDist,...
%                                    ParameterNames,thetaPositions,ModelParameters,lambda,EstStr,IsAdaptiveLearning,...
%                                    controls,DSGEModel,CurrINI)
%
% REQUIRED INPUT: thetaMode (vector) of dimension NumParam with the posterior mode estimates of the original parameters.
%
%                 thetaInverseHessian (matrix) of dimension NumParamxNumParam with the covariances of the original
%                                     parameters. The posterior distribution is assumed to be normal with mean equal
%                                     to the mode.
%
%                 UpdateMode (vector) with the posterior mode estimates of the parameters that are a function of the
%                                     estimated parameters.
%
%                 theta (vector) with initial values of the parameters.
%
%                 thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if truncnormal,
%                                     6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel, and 11 if
%                                     pareto.
%
%                 LowerBound (vector) with lower bound values.
%
%                 PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                                     "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are
%                                     matrices with the parameter values for the distributions. In the case of the
%                                     normal, beta, gamma, logistic and gumbel these parameter values are the mean
%                                     and the standard deviation. For the invgamma it is the s and q parameters, where
%                                     q is a positive integer (degrees of freedom). For the left truncated normal there's
%                                     a third parameter, the lower bound. For the uniform the parameter values are the
%                                     lower and the upper bound. For the student-t they are the location (mode), the
%                                     scale and the degrees of freedom parameter, while for the Cauchy only the location
%                                     and scale parameters are needed. For the Pareto the parameter values are for the mode
%                                     and the shape parameter.
%
%                 ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                                     "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated".
%                                     Each field returns a string matrix with the parameter names.
%
%                 thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name of the
%                                     model parameter in position i of theta. The field "distribution" has a string vector
%                                     giving the assumed prior distribution of the parameter.
%
%                 ModelParameters (structure) whose fields have names given by the parameter names in the DSGE model.
%
%                 lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR model.
%
%                 EstStr (string) that takes the values 'Posterior Mode', 'DSGE Posterior Mode', 'Marginal Posterior Mode',
%                                     or 'Joint Posterior Mode'.
%
%                 IsAdaptiveLearning (boolean) that takes the value 1 if the adaptive learning based solution of the
%                                     DSGE model is used, and 0 otherwise.
%
%                 controls (structure), whose fields provide handles to all the UI controls in YADA's main dialog.
%
%                 DSGEModel (structure) whose fields contain all required information about the model, such as input files,
%                                     variable and sample data.
%
%                 CurrINI (structure), with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 3, 2010.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-12-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 07-02-2011: Made sure that update parameters will only be available when the field
%               "additional" to the "ParameterNames" structure is not empty.
%
% * 08-02-2011: Took into account that the output variable "UpdateParameters" is given by
%               the "GetUpdatePriorDensity" function.
%
% * 09-02-2011: Trying to determine which bounds may detect fixed update parameters from
%               varying.
%
% * 10-02-2011: Removed this code.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 07-03-2014: Added the input variables "EstStr" and "lambda" to allow for DSGE-VAR models.
%
% * 04-04-2014: Dealt with the issue when Xposterior for a given parameter is NaN.
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
% * 08-06-2017: The function now supports the system prior.
%
% * 08-01-2018: Updated the documentation.
%
% * 25-10-2018: Made sure that the name of the prior draws file reflects using a system prior.
%
% * 21-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 09-02-2021: Added the input variable "IsAdaptiveLearning".
%
% * 03-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
NumParam = length(theta);
NumUpdateParam = length(UpdateMode);
%
NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
NumGridPointsHalf = NumGridPoints/2;
%
% directory for saving graphs
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
% setup various string variables used to distinguish the different models
%
if strcmp(EstStr(1:4),'Post')==1;
   ModelStr = 'Adaptive Learning Posterior';
   EStr = '';
   AddStr = '';
elseif strcmp(EstStr(1:4),'DSGE')==1;
   ModelStr = 'DSGE';
   EStr = '';
   AddStr = '';
elseif strcmp(EstStr(1:8),'Marginal')==1;
   ModelStr = ['DSGE-VAR(' num2str(lambda,'%0.4f') ')'];
   EStr = 'Marginal ';
   ModeType = 'MargMode';
   AddStr = ' ';
else;
   ModelStr = ['DSGE-VAR(' num2str(lambda,'%0.4f') ')'];
   EStr = 'Joint ';
   ModeType = 'JointMode';
   AddStr = ' ';
end;
if isempty(lambda)==0;
   SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
   SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
end;
%
% check if we have a system prior
%
if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
   IsSystemPrior = 0;
else;
   IsSystemPrior = 1;
end;
%
% check if we should compute prior and laplace approx posterior for additional parameters
%
if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ParameterNames.additional)==0);
   %
   % ask if the user wants to load additional parameters
   %
   txt = ['The model "' DSGEModel.NameOfModel '" has parameters that are a function of the estimated parameters. Would you also like to plot Laplace approximations of the posterior densities for these additional parameters?'];
   answer = Query(txt,'question',200,'Laplace Posterior Densities - Additional Parameters',500,'no',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      UpdateNames = ParameterNames.additional;
      %
      % get the prior densities for the additional parameters
      %
      TmpCurrINI = CurrINI;
      TmpCurrINI.theta = theta;
      TmpCurrINI.thetaDist = thetaDist;
      TmpCurrINI.LowerBound = LowerBound;
      TmpCurrINI.PriorDist = PriorDist;
      TmpCurrINI.ParameterNames = ParameterNames;
      TmpCurrINI.thetaPositions = thetaPositions;
      TmpCurrINI.ModelParameters = ModelParameters;
      TmpCurrINI.UpdateNames = UpdateNames;
      if IsSystemPrior==0;
         if IsAdaptiveLearning==0;
            [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = GetUpdatePriorDensity(DSGEModel,TmpCurrINI,controls,0,0,0,0);
         else;
            [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = AdaptiveLearningGetUpdatePriorDensity(DSGEModel,TmpCurrINI,controls,0,0,0,0);
         end;
      else;
         %
         % we need to deal with an update system prior density
         %
         if IsAdaptiveLearning==0;
            [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = GetUpdateSystemPriorDensity(DSGEModel,TmpCurrINI,controls,0,0,0,0);
         else;
            [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = AdaptiveLearningGetUpdateSystemPriorDensity(DSGEModel,TmpCurrINI,controls,0,0,0,0)
         end;
         if isempty(UpdatePriorX)==1;
            return;
         end;
      end;
      %
      % test if the results were obtained from an old mat-file
      %
      if max(UpdateParameters)>NumUpdateParam;
         if IsAdaptiveLearning==0;
            if IsSystemPrior==0;
               AddFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-Additional-' DSGEModel.NameOfModel '.mat']);
            else;
               AddFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-Additional-' DSGEModel.NameOfModel '.mat']);
            end;
            delete(AddFile);
            if IsSystemPrior==0;
               [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = GetUpdatePriorDensity(DSGEModel,TmpCurrINI,controls,0,0,0,0);
            else;
               [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = GetUpdateSystemPriorDensity(DSGEModel,TmpCurrINI,controls,0,0,0,0);
            end;
         else;
            if IsSystemPrior==0;
               AddFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-Additional-' DSGEModel.NameOfModel '.mat']);
            else;
               AddFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-Additional-' DSGEModel.NameOfModel '.mat']);
            end;
            delete(AddFile);
            if IsSystemPrior==0;
               [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = AdaptiveLearningGetUpdatePriorDensity(DSGEModel,TmpCurrINI,controls,0,0,0,0);
            else;
               [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = AdaptiveLearningGetUpdateSystemPriorDensity(DSGEModel,TmpCurrINI,controls,0,0,0,0);
            end;
         end;
      end;
      clear('TmpCurrINI');
      if isempty(UpdatePriorX)==0;
         %
         % make sure that all variables used for the plots are based only on UpdateParameters
         %
         if NumUpdateParam>length(UpdateParameters);
            UpdateNames = UpdateNames(UpdateParameters,:);
            UpdateMode = UpdateMode(UpdateParameters);
            NumUpdateParam = length(UpdateParameters);
         end;
         %
         % compute the matrix with partial derivatives of the update paramaters wrt the the original parameters
         %
         UpdatePartial = GetUpdatePartial(UpdateMode,thetaMode,UpdateNames,ModelParameters,thetaPositions,DSGEModel.UpdateParameterFile);
         UpdateStd = sqrt(diag(UpdatePartial*thetaInverseHessian*UpdatePartial'));
         %
         % compute grid density estimates of the posterior density
         %
         UpdateLB = UpdateMode-(DSGEModel.GridWidth*UpdateStd);
         Dev = (DSGEModel.GridWidth*UpdateStd)/NumGridPointsHalf;
         UpdatePosteriorX = zeros(NumUpdateParam,NumGridPoints+1);
         UpdatePosteriorY = zeros(NumUpdateParam,NumGridPoints+1);
         for j=1:NumUpdateParam;
            UpdateGrid = (UpdateLB(j):Dev(j):UpdateLB(j)+(NumGridPoints*Dev(j)));
            %
            % compute the height of the normal density
            %
            DensityVals = exp(logNormalPDF(UpdateGrid,UpdateMode(j),UpdateStd(j)));
            %
            % make sure that values outside the prior bounds are skipped
            %
            BadPos = ((UpdateGrid>UpdatePriorBounds(j,2))+(UpdateGrid<UpdatePriorBounds(j,1))+(isinf(DensityVals))).*(1:length(UpdateGrid));
            BadPos = BadPos(BadPos>0);
            if isempty(BadPos)==0;
               UpdateGrid(BadPos) = NaN;
               DensityVals(BadPos) = NaN;
            end;
            UpdatePosteriorX(j,:) = UpdateGrid;
            UpdatePosteriorY(j,:) = DensityVals;
         end;
      else;
         txt = 'YADA has determined that all parameters that are supposed to be functions of the estimated parameters are, in fact, fixed. YADA will therefore skip these parameters in the plots.';
         About(txt,'information','Laplace Posterior Densities - Additional Parameters',300,500,CurrINI);
         UpdateNames = '';
         UpdateMode = [];
         UpdatePriorX = [];
         UpdatePriorY = [];
         UpdatePriorBounds = [];
         UpdatePosteriorX = [];
         UpdatePosteriorY = [];
         NumUpdateParam = 0;
      end;
   else;
      UpdateNames = '';
      UpdateMode = [];
      UpdatePriorX = [];
      UpdatePriorY = [];
      UpdatePriorBounds = [];
      UpdatePosteriorX = [];
      UpdatePosteriorY = [];
      NumUpdateParam = 0;
   end;
else;
   UpdateNames = '';
   UpdateMode = [];
   UpdatePriorX = [];
   UpdatePriorY = [];
   UpdatePriorBounds = [];
   UpdatePosteriorX = [];
   UpdatePosteriorY = [];
   NumUpdateParam = 0;
end;
%
% now we compute the prior densities of the original parameters
%
if IsAdaptiveLearning==0;
   if IsSystemPrior==0;
      [Xprior,Yprior,BoundsMatrix] = GraphPriorDensity(theta,thetaDist,LowerBound,PriorDist,ParameterNames,thetaPositions,ModelParameters,controls,DSGEModel,CurrINI);
   else;
      [XMargPrior,YMargPrior,BoundsMatrix,Xprior,Yprior] = GraphSystemPriorDensity(DSGEModel,CurrINI);
      if isempty(Xprior)==1;
         return;
      end;
   end;
else;
   if IsSystemPrior==0;
      [Xprior,Yprior,BoundsMatrix] = AdaptiveLearningGraphPriorDensity(theta,thetaDist,LowerBound,PriorDist,ParameterNames,thetaPositions,ModelParameters,controls,DSGEModel,CurrINI);
   else;
      [XMargPrior,YMargPrior,BoundsMatrix,Xprior,Yprior] = AdaptiveLearningGraphSystemPriorDensity(DSGEModel,CurrINI);
      if isempty(Xprior)==1;
         return;
      end;
   end;
end;
%
% and now the Laplace approximation
%
thetaStd = sqrt(diag(thetaInverseHessian));
thetaLB = thetaMode-(DSGEModel.GridWidth*thetaStd);
Dev = (DSGEModel.GridWidth*thetaStd)/NumGridPointsHalf;
Xposterior = zeros(NumParam,NumGridPoints+1);
Yposterior = zeros(NumParam,NumGridPoints+1);
for j=1:NumParam;
   XGrid = (thetaLB(j):Dev(j):thetaLB(j)+(NumGridPoints*Dev(j)));
   %
   % compute the height of the normal density
   %
   DensityVals = exp(logNormalPDF(XGrid,thetaMode(j),thetaStd(j)));
   %
   % make sure that values outside the prior bounds are skipped
   %
   BadPos = ((XGrid>BoundsMatrix(j,2))+(XGrid<BoundsMatrix(j,1))+(isinf(DensityVals))).*(1:1:(NumGridPoints+1));
   BadPos = BadPos(BadPos>0);
   if isempty(BadPos)==0;
      XGrid(BadPos) = NaN;
      DensityVals(BadPos) = NaN;
   end;
   Xposterior(j,:) = XGrid;
   Yposterior(j,:) = DensityVals;
end;
%
% setup data matrices for the posterior that can be saved and plotted
%
ParameterNames = strvcat(ParameterNames.estimated,UpdateNames);
Xposterior = [Xposterior;UpdatePosteriorX];
Yposterior = [Yposterior;UpdatePosteriorY];
thetaMode = [thetaMode;UpdateMode];
%
% ask if the user wants to save the plot data to mat-file
%
txt = ['Would you like to save the Laplace approximation of the posterior densities based on the ' lower(EStr) 'posterior mode of the ' ModelStr ' model and the prior densities to a mat-file?'];
answer = Query(txt,'question',200,'Save - Laplace Posterior Densities',500,'no',CurrINI);
if strcmp(lower(answer),'yes')==1;
   %
   if IsAdaptiveLearning==0;
      if isempty(lambda)==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\LaplacePosteriorPriorData-' int2str(NumGridPoints) '.' int2str(DSGEModel.GridWidth) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\LaplacePosteriorPriorData-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ModeType '-' int2str(NumGridPoints) '.' int2str(DSGEModel.GridWidth) '.mat']);
      end;
      %
      % and now the m-file
      %
      if isempty(lambda)==1;
         mfile = FixFilePath([DSGEModel.OutputDirectory '\LaplacePostData' int2str(NumGridPoints) '_' int2str(DSGEModel.GridWidth) '.m']);
      else;
         mfile = FixFilePath([DSGEModel.OutputDirectory '\LaplacePostDataDSGEVAR' strrep(num2str(lambda,'%0.4f'),'.','_') '_' ModeType '_' int2str(NumGridPoints) '_' int2str(DSGEModel.GridWidth) '.m']);
      end;
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\LaplacePosteriorPriorData-AdaptiveLearning-' int2str(NumGridPoints) '.' int2str(DSGEModel.GridWidth) '.mat']);
      mfile = FixFilePath([DSGEModel.OutputDirectory '\LaplacePostDataAL' int2str(NumGridPoints) '_' int2str(DSGEModel.GridWidth) '.m']);
   end;
   %
   save(file,'Xprior','Yprior','UpdatePriorX','UpdatePriorY','Xposterior','Yposterior','ParameterNames','BoundsMatrix','thetaDist','UpdatePriorBounds','thetaMode');
   %
   fid = fopen(mfile,'wt');
   fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
   fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
   fprintf(fid,'%%\n%% x-axis values for prior density of estimated parameters\n%%\n');
   fprintf(fid,'Xprior = DataStructure.Xprior;\n');
   fprintf(fid,'%%\n%% y-axis values for prior density of estimated parameters\n%%\n');
   fprintf(fid,'Yprior = DataStructure.Yprior;\n');
   fprintf(fid,'%%\n%% x-axis values for prior density of updated parameters\n%%\n');
   fprintf(fid,'UpdatePriorX = DataStructure.UpdatePriorX;\n');
   fprintf(fid,'%%\n%% y-axis values for prior density of updated parameters\n%%\n');
   fprintf(fid,'UpdatePriorY = DataStructure.UpdatePriorY;\n');
   fprintf(fid,'%%\n%% x-axis values for posterior density of all parameters\n%% (estimated on top, updated below)\n%%\n');
   fprintf(fid,'Xposterior = DataStructure.Xposterior;\n');
   fprintf(fid,'%%\n%% y-axis values for posterior density of all parameters\n%%\n');
   fprintf(fid,'Yposterior = DataStructure.Yposterior;\n');
   fprintf(fid,'%%\n%% string matrix with names of all parameters\n%%\n');
   fprintf(fid,'ParameterNames = DataStructure.ParameterNames;\n');
   fprintf(fid,'%%\n%% matrix with possible lower and upper bounds of the estimated parameters\n%%\n');
   fprintf(fid,'BoundsMatrix = DataStructure.BoundsMatrix;\n');
   fprintf(fid,'%%\n%% matrix with guestimated lower and upper bounds of the update parameters\n%%\n');
   fprintf(fid,'UpdatePriorBounds = DataStructure.UpdatePriorBounds;\n');
   fprintf(fid,'%%\n%% vector which indicates prior distribution used: 1 gamma, 2 beta, 3 normal,\n%% 4 invgamma, 5 truncnormal, 6 uniform, 7 student, 8 cauchy, 9 logistic\n%% 10 gumbel, 11 pareto\n%%\n');
   fprintf(fid,'thetaDist = DataStructure.thetaDist;\n');
   fprintf(fid,'%%\n%% vector with joint mode of all parameters\n%%\n');
   fprintf(fid,'thetaMode = DataStructure.thetaMode;\n');
   fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
   %
   % end the script file
   %
   fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
   fclose(fid);
   %
   txt = ['The Laplace approximation of the posterior densities and the prior densities have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 11 entries: Xprior and Yprior (prior density data for estimated parameters), UpdatePriorX and UpdatePriorY (prior density data for updated parameters), Xposterior and Yposterior (Laplace approximation of the posterior density data for all parameters), ParameterNames (string matrix with the names of all parameters), BoundsMatrix (matrix with potential lower and upper bounds for the estimated parameters), UpdatePriorBounds (matrix with guestimated lower and upper bounds for the updated parameters), thetaDist (vector indicating type of prior distribution of estimated parameters), and thetaMode (vector with joint mode of all parameters). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
   About(txt,'information',['Laplace Posterior Densities - ' EstStr],500,600,CurrINI);
end;
%
% now we can start plotting the prior and posterior densities
%
%
% Compute the number of figures (max 16 plots per graph)
%
NumFigs = ceil((NumParam+NumUpdateParam)/16);
%
if NumFigs==1;
   [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumParam+NumUpdateParam);
else;
   NumRows = 4;
   NumCols = 4;
end;
NumPlots = 0;
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
%
%
% now we can start the loops
%
for i=1:NumFigs;
   if i>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>CurrINI.scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = CurrINI.scrsz(4)-700;
      end;
   end;
   FigHandles{i} = figure('Visible','off','Position',pos);
   if NumFigs==1;
      set(FigHandles{i},'Name',['Laplace Posterior Densities - ' EstStr]);
   else;
      set(FigHandles{i},'Name',['Laplace Posterior Densities - ' EstStr ' - Figure ' int2str(i)]);
   end;
   SetFigureProperties(FigHandles{i},CurrINI);
   %
   k = 0;
   for r=1:NumRows;
      for c=1:NumCols;
         NumPlots = NumPlots+1;
         if NumPlots<=NumParam+NumUpdateParam;
            k = k+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
            subplot(NumRows,NumCols,k);
            %
            % plot the prior density
            %
            if NumPlots<=NumParam;
               PriorHandle = plot(Xprior(NumPlots,:),Yprior(NumPlots,:),'--','Color',[0.75 0 0],'LineWidth',2);
            else;
               PriorHandle = plot(UpdatePriorX(NumPlots-NumParam,:),UpdatePriorY(NumPlots-NumParam,:),'--','Color',[0.75 0 0],'LineWidth',2);
            end;
            hold('on');
            %
            % plot the posterior density
            %
            PosteriorHandle = plot(Xposterior(NumPlots,:),Yposterior(NumPlots,:),'-','Color',[0 0 0.75],'LineWidth',2);
            hold('on');
            SetAxesFonts(CurrINI,'Title',StringTrim(ParameterNames(NumPlots,:)));
            axis('tight');
            xlim = [min(Xposterior(NumPlots,:)) max(Xposterior(NumPlots,:))];
            add_x = 0.1*(xlim(2)-xlim(1));
            x_lim = [xlim(1)-add_x xlim(2)+add_x];
            if x_lim(1)==x_lim(2);
               x_lim(1) = x_lim(1)-0.5;
               x_lim(2) = x_lim(2)+0.5;
            end;
            if max(isnan(x_lim))==0;
               set(gca,'XLim',x_lim);
            end;
            %
            ylim = [0 1.2*max(Yposterior(NumPlots,:))];
            if NumPlots<=NumParam;
               if thetaDist(NumPlots)==1;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = BoundsMatrix(NumPlots,1);
                  end;
               elseif thetaDist(NumPlots)==2;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = BoundsMatrix(NumPlots,1);
                  end;
                  if x_lim(2)>BoundsMatrix(NumPlots,2);
                     x_lim(2) = BoundsMatrix(NumPlots,2);
                  end;
               elseif thetaDist(NumPlots)==4;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = 0;
                  end;
               elseif thetaDist(NumPlots)==5;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = BoundsMatrix(NumPlots,1);
                  end;
               elseif thetaDist(NumPlots)==6;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = BoundsMatrix(NumPlots,1);
                  end;
                  if x_lim(2)>BoundsMatrix(NumPlots,2);
                     x_lim(2) = BoundsMatrix(NumPlots,2);
                  end;
               elseif thetaDist(NumPlots)==11;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = BoundsMatrix(NumPlots,1);
                  end;
               end;
            else;
               if x_lim(1)<UpdatePriorBounds(NumPlots-NumParam,1);
                  x_lim(1) = UpdatePriorBounds(NumPlots-NumParam,1);
               end;
               if x_lim(2)>UpdatePriorBounds(NumPlots-NumParam,2);
                  x_lim(2) = UpdatePriorBounds(NumPlots-NumParam,2);
               end;
            end;
            %
            % plot the joint posterior mode estimate
            %
            JointModeHandle = plot(thetaMode(NumPlots)*ones(1,2),ylim,'-','Color',[0 0 0],'LineWidth',1);
            %
            if (max(isnan(x_lim))==0)&(max(isnan(ylim))==0);
               set(gca,'XLim',x_lim,'YLim',ylim);
            elseif max(isnan(ylim))==0;
               set(gca,'YLim',ylim);
            end;
            hold('off');
         end;
      end;
   end;
   if IsSystemPrior==0;
      legHandle = legend([PriorHandle PosteriorHandle JointModeHandle],strvcat('Prior','Approx. Posterior',[ModelStr AddStr StringTrim(EStr) ' Mode']));
   else;
      legHandle = legend([PriorHandle PosteriorHandle JointModeHandle],strvcat('System Prior','Approx. Posterior',[ModelStr AddStr StringTrim(EStr) ' Mode']));
   end;
   %
   SetAxesFonts(CurrINI,legHandle);
   %
   % transform units temporarily into pixels
   %
   OldAxesUnits = get(gca,'Units');
   OldLegUnits = get(legHandle,'Units');
   set(gca,'Units','pixels');
   set(legHandle,'Units','pixels');
   %
   % check if we can move legend
   %
   LegPos = get(legHandle,'Position');
   AxPos = get(gca,'Position');
   if LegPos(1)<AxPos(1)+AxPos(3);
      if AxPos(1)+AxPos(3)+10+LegPos(3)<1000;
         LegPos(1) = AxPos(1)+AxPos(3)+10;
         set(legHandle,'Position',LegPos);
      end;
   end;
   set(gca,'Units',OldAxesUnits);
   set(legHandle,'Units',OldLegUnits);
   %
   % save it to disk before
   %
   if DirExist(GraphDirectory);
      if IsAdaptiveLearning==0;
         if isempty(lambda)==1;
            if NumFigs>1;
               file = FixFilePath([GraphDirectory '\LaplacePosteriorDensity-' int2str(NumGridPoints) '.' int2str(DSGEModel.GridWidth) '-' int2str(i) '.fig']);
            else;
               file = FixFilePath([GraphDirectory '\LaplacePosteriorDensity-' int2str(NumGridPoints) '.' int2str(DSGEModel.GridWidth) '.fig']);
            end;
         else;
            if NumFigs>1;
               file = FixFilePath([GraphDirectory '\LaplacePosteriorDensity-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ModeType '-' int2str(NumGridPoints) '.' int2str(DSGEModel.GridWidth) '-' int2str(i) '.fig']);
            else;
               file = FixFilePath([GraphDirectory '\LaplacePosteriorDensity-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ModeType '-' int2str(NumGridPoints) '.' int2str(DSGEModel.GridWidth) '.fig']);
            end;
         end;
      else;
         if NumFigs>1;
            file = FixFilePath([GraphDirectory '\LaplacePosteriorDensity-AdaptiveLearning-' int2str(NumGridPoints) '.' int2str(DSGEModel.GridWidth) '-' int2str(i) '.fig']);
         else;
            file = FixFilePath([GraphDirectory '\LaplacePosteriorDensity-AdaptiveLearning-' int2str(NumGridPoints) '.' int2str(DSGEModel.GridWidth) '.fig']);
         end;
      end;
      saveas(FigHandles{i},file);
   end;
   %
   % Fix the toolbar cdata
   %
   SetToolbarImages(FigHandles{i},CurrINI.images);
   %
   % Display the current figure
   %
   set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;

%
% end of GraphLaplacePosteriorDensity.m
%
