function GraphMonteCarloFilteringDensity(SolutionCode,thetaDraws,ParameterNames,IsAdaptiveLearning,DSGEModel,CurrINI)
% GraphMonteCarloFilteringDensity: Creates plots of the three possible densities from Monte Carlo filtering of the prior draws.
%
% USAGE:
%
%       GraphMonteCarloFilteringDensity(SolutionCode,thetaDraws,ParameterNames,IsAdaptiveLearning,DSGEModel,CurrINI)
%
% REQUIRED INPUT: SolutionCode (vector) of dimension NumDraws with integer 1 for unique, convergent solution,
%                              2 for indeterminacy, and 0 for no stable solution.
%
%                 thetaDraws (matrix) of dimension NumParamxNumDraws with values of the parameters of the
%                              DSGE model.
%
%                 ParameterNames (string matrix) with the names of the estimated parameters.
%
%                 IsAdaptiveLearning (boolean) which takes the value 1 if the adaptive learning is used
%                              and 0 otherwise.
%
%                 DSGEModel (structure) whose fields contain all required information about the model,
%                              such as input files, variable and sample data.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: November 2, 2023.
%                        This version: November 3, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 03-11-2023: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[NumParam,NumDraws] = size(thetaDraws);
%
% locate unique and convergent solutions
%
OnePos = (SolutionCode==1);
TwoPos = (SolutionCode==2);
ZeroPos = (SolutionCode==0);
NumUniqueConv = sum(OnePos);
NumIndeterminacy = sum(TwoPos);
NumNoSolution = sum(ZeroPos);
NumOther = NumDraws-NumUniqueConv;
%
if IsAdaptiveLearning==1;
   LearningText = ' - Adaptive Learning';
else;
   LearningText = '';
end;
%
if DSGEModel.PosteriorDensityValue>2;
   WaittxtAdd = [' Estimation started at: ' StringTrim(datestr(now,14)) '. Using ' int2str(NumDraws) ' draws. This may take some time...'];
   ShowCancel = 1;
else;
   WaittxtAdd = [' Estimation started at: ' StringTrim(datestr(now,14)) '.'];
   ShowCancel = 0;
end;
txt = ['Please wait while YADA computes kernel density estimates of the marginal densities for the Monte Carlo filtering separation of parameters into the following cases: unique and convergent, indeterminate and no solution.' WaittxtAdd ' Current parameter: ' StringTrim(ParameterNames(1,:)) '.'];
WaitHandle = WaitDLG(txt,'information',['Kernel Density Estimation' LearningText],600,400,CurrINI,ShowCancel);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% treat the parameter draws like they can be from different distributions,
% i.e., we try to construct the x-axis and y-axis data using kernel density
% estimates
%
StopEstimation = 0;
for i=1:NumParam;
   %
   % update the wait dialog
   %
   txt = ['Please wait while YADA computes kernel density estimates of the marginal prior densities for the Monte Carlo filtering separation of parameters into the following cases: unique and convergent, indeterminate and no solution.' WaittxtAdd ' Current parameter: ' StringTrim(ParameterNames(i,:)) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   % parameters must be split into separate terms
   %
   thetaUnique = thetaDraws(i,OnePos)';
   thetaIndeterminacy = thetaDraws(i,TwoPos)';
   thetaNoSolution = thetaDraws(i,ZeroPos)';
   if DSGEModel.PosteriorDensityValue==1;
      if NumUniqueConv>=256;
         if exist('YADAksdensity')==2;
            [f1,x1] = YADAksdensity(thetaUnique,'npoints',256);
         else;
            if exist('ksdensity')==2;
               [f1,x1] = ksdensity(thetaUnique,'npoints',256);
            else;
               [x1,f1] = KernelDensityPlot(thetaUnique,'knorm');
            end;
         end;
      end;
      if NumIndeterminacy>=256;
         if exist('YADAksdensity')==2;
            [f2,x2] = YADAksdensity(thetaIndeterminacy,'npoints',256);
         else;
            if exist('ksdensity')==2;
               [f2,x2] = ksdensity(thetaIndeterminacy,'npoints',256);
            else;
               [x2,f2] = KernelDensityPlot(thetaIndeterminacy,'knorm');
            end;
         end;
      end;
      if NumNoSolution>=256;
         if exist('YADAksdensity')==2;
            [f0,x0] = YADAksdensity(thetaNoSolution,'npoints',256);
         else;
            if exist('ksdensity')==2;
               [f0,x0] = ksdensity(thetaNoSolution,'npoints',256);
            else;
               [x0,f0] = KernelDensityPlot(thetaNoSolution,'knorm');
            end;
         end;
      end;
   elseif DSGEModel.PosteriorDensityValue==2;
      if NumUniqueConv>=256;
         [x1,f1] = YADAMHDensEst(thetaUnique,2^8,0,'gaussian',[]);
      end;
      if NumIndeterminacy>=256;
         [x2,f2] = YADAMHDensEst(thetaIndeterminacy,2^8,0,'gaussian',[]);
      end;
      if NumNoSolution>=256;
         [x0,f0] = YADAMHDensEst(thetaNoSolution,2^8,0,'gaussian',[]);
      end;
   elseif DSGEModel.PosteriorDensityValue==3;
      if NumUniqueConv>=256;
         [x1,f1] = YADAMHDensEst(thetaUnique,2^8,-1,'gaussian',WaitControls.cancel);
      end;
      if NumIndeterminacy>=256;
         [x2,f2] = YADAMHDensEst(thetaIndeterminacy,2^8,-1,'gaussian',WaitControls.cancel);
      end;
      if NumNoSolution>=256;
         [x0,f0] = YADAMHDensEst(thetaNoSolution,2^8,-1,'gaussian',WaitControls.cancel);
      end;
   elseif DSGEModel.PosteriorDensityValue==4;
      if NumUniqueConv>=256;
         [x1,f1] = YADAMHDensEst(thetaUnique,2^8,-2,'gaussian',WaitControls.cancel);
      end;
      if NumIndeterminacy>=256;
         [x2,f2] = YADAMHDensEst(thetaIndeterminacy,2^8,-2,'gaussian',WaitControls.cancel);
      end;
      if NumNoSolution>=256;
         [x0,f0] = YADAMHDensEst(thetaNoSolution,2^8,-2,'gaussian',WaitControls.cancel);
      end;
   end;
   if (isempty(x1)==1)&(isempty(x2)==1)&(isempty(x0)==1);
      StopEstimation = 1;
      break;
   end;
   if i==1;
      if NumUniqueConv>=256;
         X1 = zeros(NumParam,length(x1));
         Y1 = zeros(NumParam,length(x1));
      else;
         X1 = [];
         Y1 = [];
      end;
      if NumIndeterminacy>=256;
         X2 = zeros(NumParam,length(x2));
         Y2 = zeros(NumParam,length(x2));
      else;
         X2 = [];
         Y2 = [];
      end;
      if NumNoSolution>=256;
         X0 = zeros(NumParam,length(x0));
         Y0 = zeros(NumParam,length(x0));
      else;
         X0 = [];
         Y0 = [];
      end;
   end;
   if isempty(X1)==0;
      if size(x1,1)>size(x1,2);
         X1(i,:) = x1';
      else;
         X1(i,:) = x1;
      end;
      if size(f1,1)>size(f1,2);
         Y1(i,:) = f1';
      else;
         Y1(i,:) = f1;
      end;
   end;
   if isempty(X2)==0;
      if size(x2,1)>size(x2,2);
         X2(i,:) = x2';
      else;
         X2(i,:) = x2;
      end;
      if size(f2,1)>size(f2,2);
         Y2(i,:) = f2';
      else;
         Y2(i,:) = f2;
      end;
   end;
   if isempty(X0)==0;
      if size(x0,1)>size(x0,2);
         X0(i,:) = x0';
      else;
         X0(i,:) = x0;
      end;
      if size(f0,1)>size(f0,2);
         Y0(i,:) = f0';
      else;
         Y0(i,:) = f0;
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
if StopEstimation==1;
   return;
end;
%
% Compute the number of figures (max 16 plots per graph)
%
NumFigs = ceil((NumParam)/16);
%
if NumFigs==1;
   [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumParam);
else;
   NumRows = 4;
   NumCols = 4;
end;
NumPlots = 0;
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
if DirExist(GraphDirectory)==0;
   stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
end;
%
% now we can start the loops
%
for i=1:NumFigs;
   if i>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>CurrINI.scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = CurrINI.scrsz(4)-700;
      end;
   end;
   FigHandles{i} = figure('Visible','off','Position',pos);
   if NumFigs==1;
      set(FigHandles{i},'Name',['Monte Carlo Filtering Densities' LearningText]);
   else;
      set(FigHandles{i},'Name',['Monte Carlo Filtering Densities' LearningText ' - Figure ' int2str(i)]);
   end;
   SetFigureProperties(FigHandles{i},CurrINI);
   k = 0;
   for r=1:NumRows;
      for c=1:NumCols;
         NumPlots = NumPlots+1;
         if NumPlots<=NumParam
            k = k+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
            subplot(NumRows,NumCols,k);
            %
            % start with the unique and convergent case
            %
            if isempty(X1)==0;
               UniqueHandle = plot(X1(NumPlots,:),Y1(NumPlots,:),'-','Color',[0 0 0.5],'LineWidth',2);
               hold('on');
            else;
               UniqueHandle = [];
            end;
            %
            % now the indeterminate
            %
            if isempty(X2)==0;
               IndeterminacyHandle = plot(X2(NumPlots,:),Y2(NumPlots,:),'-','Color',[0.5 0 0],'LineWidth',2);
               hold('on');
            else;
               IndeterminacyHandle = [];
            end;
            %
            % and finally the no solution
            %
            if isempty(X0)==0;
               NoSolutionHandle = plot(X0(NumPlots,:),Y0(NumPlots,:),'-','Color',[0 0.5 0],'LineWidth',2);
               hold('on');
            else;
               NoSolutionHandle = [];
            end;
            SetAxesFonts(CurrINI,'Title',StringTrim(ParameterNames(NumPlots,:)));
            axis('tight');
            %
            ylim = get(gca,'YLim');
            set(gca,'YLim',[0 1.1*ylim(2)]);
            hold('off');
         end;
      end;
   end;
   Handles = [];
   legText = '';
   if isempty(X1)==0;
      Handles = [Handles UniqueHandle];
      legText = strvcat(legText,'Unique & stable solution');
   end;
   if isempty(X2)==0;
      Handles = [Handles IndeterminacyHandle];
      legText = strvcat(legText,'Indeterminacy');
   end;
   if isempty(X0)==0;
      Handles = [Handles NoSolutionHandle];
      legText = strvcat(legText,'No stable solution');
   end;
   legHandle = legend(Handles,legText);
   %
   SetAxesFonts(CurrINI,legHandle);
   %
   % transform units temporarily into pixels
   %
   OldAxesUnits = get(gca,'Units');
   OldLegUnits = get(legHandle,'Units');
   set(gca,'Units','pixels');
   set(legHandle,'Units','pixels');
   %
   % check if we can move legend
   %
   LegPos = get(legHandle,'Position');
   AxPos = get(gca,'Position');
   if LegPos(1)<AxPos(1)+AxPos(3);
      if AxPos(1)+AxPos(3)+10+LegPos(3)<1000;
         LegPos(1) = AxPos(1)+AxPos(3)+10;
         set(legHandle,'Position',LegPos);
      end;
   end;
   set(gca,'Units',OldAxesUnits);
   set(legHandle,'Units',OldLegUnits);
   %
   % save it to disk before
   %
   if DirExist(GraphDirectory)==1;
      if IsAdaptiveLearning==1;
         if NumFigs>1;
            file = FixFilePath([GraphDirectory '\MonteCarloFilteringDensity-AdaptiveLearning-' int2str(NumDraws) '-' num2str(i,'%0.0f') '.fig']);
         else;
            file = FixFilePath([GraphDirectory '\MonteCarloFilteringDensity-AdaptiveLearning-' int2str(NumDraws) '.fig']);
         end;
      else;
         if NumFigs>1;
            file = FixFilePath([GraphDirectory '\MonteCarloFilteringDensity-' int2str(NumDraws) '-' num2str(i,'%0.0f') '.fig']);
         else;
            file = FixFilePath([GraphDirectory '\MonteCarloFilteringDensity-' int2str(NumDraws) '.fig']);
         end;
      end;
      saveas(FigHandles{i},file);
   end;
   %
   % Fix the toolbar cdata
   %
   SetToolbarImages(FigHandles{i},CurrINI.images);
   %
   % Display the current figure
   %
   set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;

%
% end of GraphMonteCarloFilteringDensity.m
%
