function GraphSMCAdaption(NumberAcceptedDraws,ScaleParameter,EffectiveSampleSize,ResamplingThreshold,pstar,lambda,NumPostDraws,CurrChain,NumChains,DSGEModel,CurrINI)
% GraphSMCAdaption: Produces plots of the percentage of accepted draws in the mutation step over all tempering stages
%                   of the SMC with likelihood tempering posterior sampler. It plots this series against the target
%                   acceptance rate. Furthermore, the function provides plots of the scale parameter adaption over
%                   all tempering stages, as well development of the effective sample size compared with the threshold.
%
% USAGE:
%
%       GraphSMCAdaption(NumberAcceptedDraws,ScaleParameter,EffectiveSampleSize,ResamplingThreshold,pstar,lambda,...
%                        NumPostDraws,CurrChain,NumChains,DSGEModel,CurrINI)
%
% REQUIRED INPUT: NumberAcceptedDraws (vector) of dimension Ntau x 1 with the number of accepted draws.
%
%                 ScaleParameter (vector) of dimension Ntau x 1 with the adapted values of the scale parameter
%                                cstar over the tempering stages.
%
%                 EffectiveSampleSize (vector) of dimension Ntau x 1 with the estimated effective sample size
%                                during each tempering stage.
%
%                 ResamplingThreshold (integer) with the value of the threshold when resampling occurred
%                                during the tempering stages.
%
%                 pstar (scalar) with the target acceptance rate.
%
%                 lambda (scalar) with the weight on the prior in the DSGE-VAR models. Is empty for DSGE models.
%
%                 NumPostDraws (integer) with the number of posterior draws (particles).
%
%                 CurrChain (integer) with the number of the current SMC chain.
%
%                 NumChains (integer) with the total number of available SMC chains.
%
%                 DSGEModel (structure) whose fields contain all required information about the model, such as
%                                input files, variable and sample data.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: September 15, 2017.
%                        This version: January 6, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-10-2017: Updated the documentation.
%
% * 30-04-2018: Updated the documentation.
%
% * 29-08-2018: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%
%
Ntau = length(NumberAcceptedDraws);
Positions = [1000 750];
DefaultPositions = CurrINI.scrsz;
if Positions(1)>DefaultPositions(3);
   Positions(1) = DefaultPositions(3);
end;
if Positions(2)>DefaultPositions(4);
   Positions(2) = DefaultPositions(4);
end;
XTemp = (1:1:Ntau);
%
% prepare the figure
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
if isempty(lambda)==1;
   NameStr = ['SMC Adaption - ' DSGEModel.NameOfModel ' Chain ' int2str(CurrChain) ' - ' int2str(NumChains)];
else;
   NameStr = ['DSGE-VAR(' num2str(lambda,'%0.4f') ') SMC Adaption - ' DSGEModel.NameOfModel ' Chain ' int2str(CurrChain) ' - ' int2str(NumChains)];
end;
%
FigHandle = figure('Visible','off', ...
                   'Position',[(CurrINI.scrsz(3)-Positions(1))/2 (CurrINI.scrsz(4)-Positions(2))/2 Positions], ...
                   'Tag','SMCAdaption', ...
                   'Name',NameStr);
%
SetFigureProperties(FigHandle,CurrINI);
%
% 1. Acceptance ratio plot across all tempering stages
%
subplot(3,1,1);
%
% plot the target acceptance rate
%
plot([1 Ntau],[pstar pstar],'LineStyle','-','Color',[0 0 0],'LineWidth',1);
hold('on');
%
% plot the acceptance rate across tempering stages
%
plot(XTemp,NumberAcceptedDraws'/NumPostDraws,'LineStyle','-','Color',[0.75 0 0],'LineWidth',2,'Marker','.');
%
if isempty(lambda)==1;
   SetAxesFonts(CurrINI,'Title',['Adaption of acceptance ratio for DSGE model (target ratio=' num2str(pstar,'%0.2f') ')'],'XLabel','SMC tempering stages (n)','YLabel','Ratio');
else;
   SetAxesFonts(CurrINI,'Title',['Adaption of acceptance ratio for DSGE-VAR(' num2str(lambda,'%0.4f') ') model'],'XLabel','SMC tempering stages (n)','YLabel','Ratio');
end;
set(gca,'XLim',[1 Ntau],'YLim',[0 1]);
%
% 2. Scale parameter
%
subplot(3,1,2);
%
% plot the scale parameter development
%
plot(XTemp,ScaleParameter','LineStyle','-','Color',[0.75 0 0],'LineWidth',2,'Marker','.');
%
SetAxesFonts(CurrINI,'Title','Adaption of scale parameter for proposal density','XLabel','SMC tempering stages (n)','YLabel','c_n');
set(gca,'XLim',[1 Ntau]);
%
% 3. effective sample size and the threshold
%
subplot(3,1,3);
%
% plot the threshold
%
plot([1 Ntau],[ResamplingThreshold ResamplingThreshold],'LineStyle','-','Color',[0 0 0],'LineWidth',1);
hold('on');
%
% plot the effective sample size
%
plot(XTemp,EffectiveSampleSize','LineStyle','-','Color',[0.75 0 0],'LineWidth',2,'Marker','.');
%
SetAxesFonts(CurrINI,'Title',['Adaption of effective sample size (resampling threshold=' int2str(ResamplingThreshold) ')'],'XLabel','SMC tempering stages (n)','YLabel','ESS_n');
%
set(gca,'XLim',[1 Ntau],'YLim',[floor(min(EffectiveSampleSize)/100)*100 NumPostDraws]);
%
% save the graph to disk
%
if DirExist(GraphDirectory);
   if isempty(lambda)==1;
      FigFile = FixFilePath([GraphDirectory '\SMCAdaption-' DSGEModel.NameOfModel '-' int2str(Ntau) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.' int2str(NumChains) '.fig']);
   else;
      FigFile = FixFilePath([GraphDirectory '\SMCAdaption-' DSGEModel.NameOfModel '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(Ntau) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.' int2str(NumChains) '.fig']);
   end;
   saveas(FigHandle,FigFile);
end;
%
% display the graph
%
SetToolbarImages(FigHandle,CurrINI.images);
set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
drawnow;

%
% end of GraphSMCAdaption.m
%
