function ObsWeightsDLG(selector,StateDecomp,VarStr,EstStr,DSGEModel,CurrINI)
% ObsWeightsDLG: Displays graphs of observation weights of estimated state variable and shocks.
%
% USAGE:
%
%       ObsWeightsDLG(selector,StateDecomp,VarStr,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "samplebegin", "sampleend", "showgraph",
%                          "weightperiod", and "done".
%
% OPTIONAL INPUT: StateDecomp (structure) whose fields depend on if state variables or shock have been decomposed.
%
%                 VarStr (string vector) that takes on the values "State Variables" and "Shocks".
%
%                 EstStr (string vector) that takes on the values "Initial Values" and "Posterior Mode".
%
%                 DSGEModel (structure) whose fields contain all required information about the model, such as
%                          input files, variable and sample data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: September 29, 2009.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 30-09-2009: Finished the function.
%
% * 05-10-2009: Made sure that the initial conditions are shown properly.
%
% * 09-10-2009: The initial conditions weighting cell array are now row vector arrays instead
%               of "diagonal" matrix arrays.
%
% * 06-01-2010: Updated the documentation.
%
% * 25-05-2010: Corrected a text issue on the dialog.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
%  * 23-03-2018: Fixed an issue to make legend compatible with matlab 2017a and later.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.VarStr = VarStr;
   DSGEModel.StateDecomp = StateDecomp;
   DSGEModel.EstStr = EstStr;
   WeightVariables = DSGEModel.VariableNames;
   for i=1:size(DSGEModel.StateVariableNames,1);
      WeightVariables = strvcat(WeightVariables,['Initial condition for ' DSGEModel.StateVariableNames(i,:)]);
   end;
   DSGEModel.WeightVariables = WeightVariables;
else;
   GraphGUI = findobj('Type','figure','Tag','ObsWeightsDLG');
   controls = get(GraphGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
% create the dialog
%
switch selector;
   case 'init';
      GraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ObsWeightsDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-520)/2 (CurrINI.scrsz(4)-205)/2 520 205], ...
         'Tag','ObsWeightsDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Observation Weights - ' VarStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','ObsWeightsDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(GraphGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 518 195],'Graphics Selections',45,[0.5 1],'on',CurrINI);
      %
      % sample selection controls
      %
      controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 155 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 160 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',StateDecomp.SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ObsWeightsDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[303 160 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',StateDecomp.SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ObsWeightsDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(StateDecomp.SubSampleStrMat,1));
      %
      % Select variable
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 120 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String',['Select ' lower(VarStr(1,1:length(VarStr)-1)) ':'], ...
         'HorizontalAlignment','left', ...
         'Tag','StateShockVarText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 125 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',StateDecomp.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','StateShockVarPopup',...
         'Value',1);
      %
      controls.weightvariabletext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 85 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select weights on:', ...
         'HorizontalAlignment','left', ...
         'Tag','WeightVariableText');
      %
      controls.weightvariable = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 90 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',DSGEModel.WeightVariables, ...
         'HorizontalAlignment','center', ...
         'Tag','WeightVariablePopup',...
         'Value',1);
      %
      controls.weightperiodtext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select weight period:', ...
         'HorizontalAlignment','left', ...
         'Tag','WeightPeriodText');
      %
      controls.weightperiod = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 55 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',StateDecomp.SubSampleStrMat, ...
         'CallBack','ObsWeightsDLG weightperiod', ...
         'HorizontalAlignment','center', ...
         'Tag','WeightPeriodPopup',...
         'Value',1);
      %
      controls.normalizeweights = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Normalize weights by st. dev. of observed variable/initial condition', ...
            'Value',0, ...
            'Enable','off', ...
            'Tag','NormalizeWeightsCheckBox');
      %
      if isempty(StateDecomp.StdDev)==0;
         set(controls.normalizeweights,'Enable','on','Value',1);
      end;
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[415 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ObsWeightsDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[415 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ObsWeightsDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(GraphGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
      weightperiod = get(controls.weightperiod,'Value');
      if weightperiod<samplebeg;
         txt = 'The weight period is currently an earlier date than the start period of the sample you have selected. A graph of the observation weights for the selected sample may therefore not include the more interesting data.';
         About(txt,'information','YADA - Observation Weights',200,500,CurrINI);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.samplebeg,'Value',sampleend);
      end;
      weightperiod = get(controls.weightperiod,'Value');
      if weightperiod<samplebeg;
         txt = 'The weight period is currently an earlier date than the start period of the sample you have selected. A graph of the observation weights for the selected sample may therefore not include the more interesting data.';
         About(txt,'information','YADA - Observation Weights',200,500,CurrINI);
      end;
   case 'weightperiod'
      weightperiod = get(controls.weightperiod,'Value');
      samplebeg = get(controls.samplebeg,'Value');
      if weightperiod<samplebeg;
         txt = 'The weight period is currently an earlier date than the start period of the sample you have selected. A graph of the observation weights for the selected sample may therefore not include the more interesting data.';
         About(txt,'information','YADA - Observation Weights',200,500,CurrINI);
      end;
   case 'showgraph'
      VariableValue = get(controls.variables,'Value');
      CurrVariable = StringTrim(DSGEModel.StateDecomp.VariableNames(VariableValue,:));
      WeightValue = get(controls.weightvariable,'Value');
      n = DSGEModel.n;
      r = size(DSGEModel.StateVariableNames,1);
      if WeightValue<n+r;
         set(controls.weightvariable,'Value',WeightValue+1);
      else;
         set(controls.weightvariable,'Value',1);
         if VariableValue<size(DSGEModel.StateDecomp.VariableNames,1);
            set(controls.variables,'Value',VariableValue+1);
         else;
            set(controls.variables,'Value',1);
         end;
      end;
      %
      % get the selected sample
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      %
      % period for weights on observed variables or initial conditions
      %
      weightperiod = get(controls.weightperiod,'Value');
      CurrWeightVariable = StringTrim(DSGEModel.WeightVariables(WeightValue,:));
      if (length(CurrWeightVariable)>7)&(strcmp(lower(CurrWeightVariable(1,1:8)),'initial ')==1);
         CurrWeightVariable = ['initial' CurrWeightVariable(1,8:length(CurrWeightVariable))];
      end;
      %
      % collect data
      %
      StateDecomp = DSGEModel.StateDecomp;
      DateVector = StateDecomp.DateVector;
      T = size(get(controls.weightperiod,'String'),1);
      WeightDate = StateDecomp.SubSampleStrMat(weightperiod,:);
      %
      if strcmp(lower(DSGEModel.VarStr),'state variables')==1;
         [ForecastData,UpdateData,SmoothData] = deal(NaN*ones(1,T));
         for t=1:T;
            if WeightValue<=n;
               FDataMat = StateDecomp.StateForecastWeights{t,weightperiod};
               UDataMat = StateDecomp.StateUpdateWeights{t,weightperiod};
               SDataMat = StateDecomp.StateSmoothWeights{t,weightperiod};
               if isempty(FDataMat)==0;
                  ForecastData(t) = FDataMat(VariableValue,WeightValue);
               end;
               if isempty(UDataMat)==0;
                  UpdateData(t) = UDataMat(VariableValue,WeightValue);
               end;
               SmoothData(t) = SDataMat(VariableValue,WeightValue);
            else;
               FDataMat = StateDecomp.StateForecastInitCondWeights{1,t};
               UDataMat = StateDecomp.StateUpdateInitCondWeights{1,t};
               SDataMat = StateDecomp.StateSmoothInitCondWeights{1,t};
               if isempty(FDataMat)==0;
                  ForecastData(t) = FDataMat(VariableValue,WeightValue-n);
               end;
               if isempty(UDataMat)==0;
                  UpdateData(t) = UDataMat(VariableValue,WeightValue-n);
               end;
               SmoothData(t) = SDataMat(VariableValue,WeightValue-n);
            end;
         end;
         %
         % reweight the weights by the standard deviation of the weighting variable if asked to
         %
         if get(controls.normalizeweights,'Value')==1;
            if WeightValue<=n;
               StdDev = StateDecomp.StdDev(WeightValue,weightperiod);
            else;
               StdDev = StateDecomp.StateStdDev(WeightValue-n,1);
            end;
            if StdDev<=0;
               StdDev = 1;
            end;
            ForecastData = (1/StdDev)*ForecastData;
            UpdateData = (1/StdDev)*UpdateData;
            SmoothData = (1/StdDev)*SmoothData;
         end;
      else;
         %
         % we have shocks
         %
         ForecastData = [];
         [UpdateData,SmoothData] = deal(NaN*ones(1,T));
         for t=1:T;
            if WeightValue<=n;
               UDataMat = StateDecomp.ShockUpdateWeights{t,weightperiod};
               SDataMat = StateDecomp.ShockSmoothWeights{t,weightperiod};
               if isempty(UDataMat)==0;
                  UpdateData(t) = UDataMat(VariableValue,WeightValue);
               end;
               SmoothData(t) = SDataMat(VariableValue,WeightValue);
            else;
               UDataMat = StateDecomp.ShockUpdateInitCondWeights{1,t};
               SDataMat = StateDecomp.ShockSmoothInitCondWeights{1,t};
               if isempty(UDataMat)==0;
                  UpdateData(t) = UDataMat(VariableValue,WeightValue-n);
               end;
               SmoothData(t) = SDataMat(VariableValue,WeightValue-n);
            end;
         end;
         %
         % reweight the weights by the standard deviation of the weighting variable if asked to
         %
         if get(controls.normalizeweights,'Value')==1;
            if WeightValue<=n;
               StdDev = StateDecomp.StdDev(WeightValue,weightperiod);
            else;
               StdDev = StateDecomp.StateStdDev(WeightValue-n,1);
            end;
            if StdDev<=0;
               StdDev = 1;
            end;
            UpdateData = (1/StdDev)*UpdateData;
            SmoothData = (1/StdDev)*SmoothData;
         end;
      end;
      %
      % display the data
      %
      FigHandle = figure('Visible','off', ...
                     'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                     'Tag',CurrVariable, ...
                     'Name',['Observation Weights - ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % the weights for the forecasts
      %
      if (isempty(ForecastData)==0)&((weightperiod<sampleend)|(WeightValue>n));
         [XData,YData] = stairs(DateVector,ForecastData);
         ForecastHandle = plot(XData,YData,'--','LineWidth',2,'Color',[0 0 0.75]);
         hold('on');
         LegStr = 'Forecast (t|t-1)';
      else;
         ForecastHandle = [];
         LegStr = '';
      end;
      %
      % now the updates
      %
      if (weightperiod<=sampleend)|(WeightValue>n);
         [XData,YData] = stairs(DateVector,UpdateData);
         UpdateHandle = plot(XData,YData,'-.','LineWidth',2,'Color',[0.5 0.5 0.5]);
         hold('on');
         LegStr = strvcat(LegStr,'Update (t|t)');
      else;
         UpdateHandle = [];
      end;
      %
      % finally the the smoothed weights
      %
      [XData,YData] = stairs(DateVector,SmoothData);
      SmoothHandle = plot(XData,YData,':','LineWidth',2,'Color',[1 0 0]);
      hold('on');
      LegStr = strvcat(LegStr,'Smooth (t|T)');
      %
      axis('tight');
      %
      ylim = get(gca,'YLim');
      AddOn = 0.1*(ylim(2)-ylim(1));
      ylim = [ylim(1)-AddOn ylim(2)+AddOn];
      if ylim(2)-ylim(1)<1.0e-08;
         ylim = [ylim(1)-0.1 ylim(2)+0.1];
      end;
      set(gca,'YLim',ylim);
      %
      if (ylim(1)<0)&(ylim(2)>0);
         kids = get(gca,'Children');
         if size(kids,2)>size(kids,1);
            kids = kids';
         end;
         ZeroHandle = plot(get(gca,'XLim'),[0 0],'-','Color',[0 0 0]);
         hold('on');
         set(gca,'Children',[kids;ZeroHandle]);
      end;
      %
      % fix the X-limit
      %
      set(gca,'XLim',[DateVector(samplebeg) DateVector(sampleend)]);
      %
      %
      if WeightValue<=n;
         SetAxesFonts(CurrINI,'Title',['Weights on ' CurrWeightVariable ' at ' WeightDate ' for projections of ' CurrVariable ' at ' lower(DSGEModel.EstStr)]);
      else;
         SetAxesFonts(CurrINI,'Title',['Weights on ' CurrWeightVariable ' for projections of ' CurrVariable ' at ' lower(DSGEModel.EstStr)]);
      end;
      %
      if MatlabNumber>=7;
         if (weightperiod>=(samplebeg+sampleend)/2)&(WeightValue<=n);
            LegPos = 'NorthWest';
         else;
            LegPos = 'NorthEast';
         end;
         legHandle = legend([ForecastHandle UpdateHandle SmoothHandle],LegStr,'Location',LegPos);
      else;
         if (weightperiod>=(samplebeg+sampleend)/2)&(WeightValue<=n);
            LegPos = 2;
         else;
            LegPos = 1;
         end;
         legHandle = legend([ForecastHandle UpdateHandle SmoothHandle],LegStr,LegPos);
      end;
      SetAxesFonts(CurrINI,legHandle);
      %
      SetXTickIntegers(gca);
      %
      % make the figure visible
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(controls.done,'UserData','done');
      delete(GraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(GraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(GraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of ObsWeightsDLG.m
%
