function PermanentShockGraphDLG(selector,CurrINI,DSGEModel,IRStructure,NameStr)
% PermanentShockGraphDLG: Displays the responses to the state and the observed variables from a permanent
%                         shock to a selected target variable in a selected target/policy equation.
%
% USAGE:
%
%       PermanentShockGraphDLG(selector,CurrINI,DSGEModel,IRStructure,NameStr)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 IRStructure (structure) with fields Ksi (rxh+1), AccKsi (rxh+1), Y (nxh+1) and AccY (nxh+1). The
%                           responses of the invidual variables are located in the rows, while the columns give
%                           the response horizon. If the "DSGEModel.ResponseDenominatorPosition" field takes a
%                           value between 1 and n, the function will attempt to divided the results in Y and AccY
%                           with the last column entry in the row in Y determined by this value. Note that these
%                           relative responses will only be computed for the observed variables  when the entry in
%                           row DSGEModel.ResponseDenominatorPosition and column h+1 of Y is nonzero. In this case
%                           the field "RelativeResponse" of IRStructure will be 1, otherwise it is 0.
%
%                 NameStr (string vector) taking on the values "Posterior Mode" or "Initial Values".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: March 12, 2021.
%                        This version: June 22, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-03-2021: Updated the documentation.
%
% * 28-04-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.IRStructure = IRStructure;
   DSGEModel.NameStr = NameStr;
else
   PermanentShockGraphGUI = findobj('Type','figure','Tag','PermanentShockGraphDLG');
   PSControl = get(PermanentShockGraphGUI,'UserData');
   CurrINI = get(PSControl.variables,'UserData');
   DSGEModel = get(PSControl.variablestext,'UserData');
end;
switch selector;
   case 'init';
      PermanentShockGraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','PermanentShockGraphDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-120)/2 500 140], ...
         'Tag','PermanentShockGraphDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Permanent Shock Responses - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','PermanentShockGraphDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(PermanentShockGraphGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 498 130],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      PSControl.sample = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 80 260 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String',['Permanent shock responses from year 0 to ' int2str(size(IRStructure.Y,2)-1)], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         set(PSControl.sample,'String',strrep(get(PSControl.sample,'String'),'year','quarter'));
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         set(PSControl.sample,'String',strrep(get(PSControl.sample,'String'),'year','month'));
      end;
      %
      % Select variable
      %
      PSControl.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 176 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');
      %
      PSControl.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 55 165 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',strvcat(DSGEModel.VariableNames,DSGEModel.StateVariableNames), ...
         'HorizontalAlignment','center', ...
         'Tag','VariablePopup',...
         'Value',1);
      %
      PSControl.accumulateY = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 20 341 20], ...
         'Style','checkbox', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String','Show accumulated responses for the selected variable', ...
         'HorizontalAlignment','center', ...
         'Tag','ShowAccumulatedCheckbox',...
         'Value',0);
      %
      % Create graph button
      %
      PSControl.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','PermanentShockGraphDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      PSControl.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','PermanentShockGraphDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(PermanentShockGraphGUI,'UserData',PSControl,...
         'HandleVisibility','callback');
   case 'showgraph'
      CurrValue = get(PSControl.variables,'Value');
      VarNames = get(PSControl.variables,'String');
      CurrName = StringTrim(VarNames(CurrValue,:));
      if CurrValue<size(VarNames,1);
         set(PSControl.variables,'Value',CurrValue+1);
      else;
         set(PSControl.variables,'Value',1);
      end;
      %
      % check if we picked an observed or a state variable
      %
      if CurrValue<=DSGEModel.n;
         if get(PSControl.accumulateY,'Value')==0;
            ResponseData = DSGEModel.IRStructure.Y(CurrValue,:);
            UseAccumulated = 0;
         else;
            ResponseData = DSGEModel.IRStructure.AccY(CurrValue,:);
            UseAccumulated = 1;
         end;
      else;
         if get(PSControl.accumulateY,'Value')==0;
            ResponseData = DSGEModel.IRStructure.Ksi(CurrValue-DSGEModel.n,:);
            UseAccumulated = 0;
         else;
            ResponseData = DSGEModel.IRStructure.AccKsi(CurrValue-DSGEModel.n,:);
            UseAccumulated = 1;
         end;
      end;
      %
      % the response horizon
      %
      h = size(DSGEModel.IRStructure.Y,2)-1;
      pos = [(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-440)/2 650 400];
      %
      FigHandle = figure('Visible','off',...
                         'Position',pos, ...
                         'Name',['Permanent Response to ' CurrName ' - Target Variable ' StringTrim(DSGEModel.StateVariableNames(DSGEModel.TargetVariablePosition,:))]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % plot the response
      %
      LineHandle = plot((0:h),ResponseData,'-','Color',[0.75 0 0],'LineWidth',2);
      hold('on');
      %
      % deal with the title
      %
      if (DSGEModel.IRStructure.RelativeResponse==0)|(CurrValue>DSGEModel.n);
         if UseAccumulated==0;
            SetAxesFonts(CurrINI,'Title',['Response of ' CurrName ' from a permanent shock of ' num2str(DSGEModel.TargetShockValue,'%0.4f') ' to target of ' StringTrim(DSGEModel.StateVariableNames(DSGEModel.TargetVariablePosition,:))]);
         else;
            SetAxesFonts(CurrINI,'Title',['Accumulated response of ' CurrName ' from a permanent shock of ' num2str(DSGEModel.TargetShockValue,'%0.4f') ' to target of ' StringTrim(DSGEModel.StateVariableNames(DSGEModel.TargetVariablePosition,:))]);
         end;
      else;
         if UseAccumulated==0;
            SetAxesFonts(CurrINI,'Title',['Relative response of ' CurrName ' wrt ' StringTrim(DSGEModel.VariableNames(DSGEModel.ResponseDenominatorPosition,:)) ' from a permanent shock of ' num2str(DSGEModel.TargetShockValue,'%0.4f') ' to target of ' StringTrim(DSGEModel.StateVariableNames(DSGEModel.TargetVariablePosition,:))]);
         else;
            SetAxesFonts(CurrINI,'Title',['Relative accumulated response of ' CurrName ' wrt ' StringTrim(DSGEModel.VariableNames(DSGEModel.ResponseDenominatorPosition,:)) ' from a permanent shock of ' num2str(DSGEModel.TargetShockValue,'%0.4f') ' to target of ' StringTrim(DSGEModel.StateVariableNames(DSGEModel.TargetVariablePosition,:))]);
         end;
      end;
      axis('tight');
      %
      % fix the yaxis
      %
      ylim = get(gca,'YLim');
      addY = 0.1*(ylim(2)-ylim(1));
      ylim = ylim+[-addY addY];
      set(gca,'YLim',ylim);
      %
      % add a zero line if necessary
      %
      if (ylim(2)>0)&(ylim(1)<0);
         hold('on');
         plot([0 h],[0 0],'-','Color',[0 0 0]);
         kids = get(gca,'Children');
         kids = [kids(2) kids(1)]';
         set(gca,'Children',kids);
      end;
      hold('off');
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(PSControl.done,'UserData','done');
      delete(PermanentShockGraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(PermanentShockGraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(PermanentShockGraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(PSControl.done);
   end;
   waitfor(PSControl.done,'UserData','done');
end;

%
% end of PermanentShockGraphDLG.m
%
