function [action,TargetVariablePosition,TargetEqPosition,TargetShockValue,ResponseDenominatorPosition] = PermanentShockSelectionDlg(selector,StateVariableNames,StateEquationNames,TargetVariablePosition,TargetEqPosition,TargetShockValue,DenominatorVariableNames,ResponseDenominatorPosition,CurrINI)
% PermanentShockSelectionDlg: Dialog for setting the permanent shock equation, target variable and permanent shock size.
%
% USAGE:
%
%       [action,TargetVariablePosition,TargetEqPosition,TargetShockValue,ResponseDenominatorPosition] = PermanentShockSelectionDlg(selector,StateVariableNames,...
%                                                                                        StateEquationNames,TargetVariablePosition,...
%                                                                                        TargetEqPosition,TargetShockValue,DenominatorVariableNames,ResponseDenominatorPosition,CurrINI)
%
% REQUIRED INPUT:  selector (string vector), with the possible values "init", "checkedit" and "quit".
%
% OPTIONAL INPUT:  StateVariableNames (string matrix) with the names of the state variables.
%
%                  StateEquationNames (string matrix) with the names of the state equations.
%
%                  TargetVariablePosition (integer) with the position of the target variable among
%                                         the state variables.
%
%                  TargetEqPosition (integer) with the position of the target (policy) equation among
%                                         the state equations.
%
%                  TargetShockValue (scalar) with the value of the permanent shock to the steady-state
%                                         of the target variable.
%
%                  DenominatorVariableNames (string matrix) with the names of the variables from which
%                                         the permanent responses can be measured in relation to. These
%                                         are the observed variables and None, being no variable.
%
%                  ResponseDenominatorPosition (integer) which determine the row in DenominatorVariableNames
%                                         which is the current default.
%
%                  CurrINI (structure) with initialization information.
%
% OPTIONAL OUTPUT: action (string vector) taking the value "OK" or "Cancel". Depends on which button was
%                                         clicked on.
%
%                  TargetVariablePosition (integer) with the position of the target variable among
%                                         the state variables.
%
%                  TargetEqPosition (integer) with the position of the target (policy) equation among
%                                         the state equations.
%
%                  TargetShockValue (scalar) with the value of the permanent shock to the steady-state
%                                         of the target variable.
%
%                  ResponseDenominatorPosition (integer) with the position in DenominatorVariableNames
%                                         that the permanent responses will be measured relative to, i.e.
%                                         the responses are divided by the response in the selected
%                                         variable.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: February 23, 2021.
%                        This version: June 22, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 03-03-2021: Updated the documentation.
%
% * 11-03-2021: Added the selection of a response denominator variable among the observed
%               and none.
%
% * 12-03-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 17-02-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize temporarily
%
if strcmp(selector,'init')==1;
   action = 'OK';
   SV.CurrINI = CurrINI;
   SV.TargetVariablePosition = TargetVariablePosition;
   SV.TargetEqPosition = TargetEqPosition;
   SV.TargetShockValue = TargetShockValue;
   SV.ResponseDenominatorPosition = ResponseDenominatorPosition;
   if isempty(TargetVariablePosition)==1;
      TargetVariablePosition = 1;
   end;
   if isempty(TargetEqPosition)==1;
      TargetEqPosition = 1;
   end;
   if isempty(ResponseDenominatorPosition)==1;
      ResponseDenominatorPosition = size(DenominatorVariableNames,1);
   end;
   scrsz = CurrINI.scrsz;
else;
   PermanentShockSelectionGUI = findobj('Type','figure','Tag','PermanentShockSelectionDlg');
   controls = get(PermanentShockSelectionGUI,'UserData');
   SV = get(controls.cancel,'UserData');
   CurrINI = SV.CurrINI;
end;
%
switch selector
   case 'init'
      PermanentShockSelectionGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','PermanentShockSelectionDlg.m', ...
         'MenuBar','none', ...
         'Units','pixels', ...
         'Position',[(scrsz(3)-455)/2 (scrsz(4)-200)/2 455 200], ...
         'Tag','PermanentShockSelectionDlg', ...
         'Resize','off', ...
         'Visible','off', ...
         'Name','Permanent Shock Data', ...
         'NumberTitle','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','PermanentShockSelectionDlg quit', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(PermanentShockSelectionGUI,'DockControl','off');
      end;
      %
      AxesBox([2 2 453 190],'Permanent Shock Selection',100,[0.5 1],'on',CurrINI);
      %
      controls.targetvariablepositiontext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 145 250 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select the model variable to target:', ...
         'HorizontalAlignment','left', ...
         'Tag','TargetVariablePositionText');
      %
      controls.targetvariablepositionpopup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[280 150 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',StateVariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','TargetVariablePositionPopup',...
         'Value',TargetVariablePosition);
      %
      controls.targeteqpositiontext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 115 250 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select the target (policy) equation name:', ...
         'HorizontalAlignment','left', ...
         'Tag','TargetEqPositionText');
      %
      controls.targeteqpositionpopup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[280 120 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',StateEquationNames, ...
         'HorizontalAlignment','center', ...
         'Tag','TargetEqPositionPopup',...
         'Value',TargetEqPosition);
      %
      controls.targetshockvaluetext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 85 250 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select permanent shock value to target:', ...
         'HorizontalAlignment','left', ...
         'Tag','TargetShockValueText');
      %
      controls.targetshockvalue = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[280 90 155 20], ...
         'Style','edit', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',num2str(TargetShockValue), ...
         'HorizontalAlignment','left', ...
         'Callback','PermanentShockSelectionDlg checkedit', ...
         'Tag','TargetShockValueEdit',...
         'Value',1);
      %
      controls.denominatorvariabletext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 55 250 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select response denominator variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','DenominatorVariableNamesText');
      %
      controls.denominatorvariablepopup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[280 60 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DenominatorVariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','DenominatorVariableNamesPopup',...
         'Value',ResponseDenominatorPosition);
      %
      controls.ok = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[280 20 75 25], ...
         'String','OK', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Callback','PermanentShockSelectionDlg quit', ...
         'UserData','waiting', ...
         'Tag','ButtonOK');
      %
      controls.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[360 20 75 25], ...
         'String','Cancel', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'UserData',SV, ...
         'Callback','PermanentShockSelectionDlg quit', ...
         'Tag','ButtonCancel');
      %
      set(PermanentShockSelectionGUI,'UserData',controls);
   case 'checkedit'
      TempValue = str2num(get(controls.targetshockvalue,'String'));
      if (isempty(TempValue)==1)|(isinf(TempValue)==1)|(isnan(TempValue)==1);
         set(controls.targetshockvalue,'String',num2str(SV.TargetShockValue));
      end;
   case 'quit'
      set(controls.ok,'UserData','done');
end;
%
if strcmp(selector,'init')==1;
   set(PermanentShockSelectionGUI,'Visible','on');
   drawnow;
   SetYADAIcon(PermanentShockSelectionGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.ok);
   end;
   %
   waitfor(controls.ok,'UserData','done');
   %
   % check here if the user really wants to make the selection
   %
   if (isempty(get(PermanentShockSelectionGUI,'CurrentObject'))==1)|(strcmp(get(get(PermanentShockSelectionGUI,'CurrentObject'),'Type'),'uicontrol')==0);
      action = 'Cancel';
   else;
      action = get(get(PermanentShockSelectionGUI,'CurrentObject'),'String');
      if (isempty(action)==1)|(strcmp(action,'OK')==0);
         action = 'Cancel';
      end;
   end;
   SV = get(controls.cancel,'UserData');
   if strcmp(lower(action),'ok')==1;
      TargetVariablePosition = get(controls.targetvariablepositionpopup,'Value');
      TargetEqPosition = get(controls.targeteqpositionpopup,'Value');
      TargetShockValue = str2num(get(controls.targetshockvalue,'String'));
      ResponseDenominatorPosition = get(controls.denominatorvariablepopup,'Value');
   else;
      TargetVariablePosition = SV.TargetVariablePosition;
      TargetEqPosition = SV.TargetEqPosition;
      TargetShockValue = SV.TargetShockValue;
      ResponseDenominatorPosition = SV.ResponseDenominatorPosition;
   end;
   %
   delete(PermanentShockSelectionGUI);
   drawnow;
   pause(0.02);
end;

%
% end of PermanentShockSelectionDlg.m
%

