function PlotAPrior(selector,DSGEModel,CurrINI)
% PlotAPrior:
%
% USAGE:
%
%       PlotAPrior(selector,DSGEModel,CurrINI)
%
% REQUIRED INPUT: selector (string vector) that can take on the values "init", "method",
%                          "priordensity", "checkvalue", "show", "done"
%
% OPTIONAL INPUT: DSGEModel (structure) with default settings for controls.
%
%                 CurrINI (structure) with initializattion information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 29, 2009.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 30-03-2009: Finished the function.
%
% * 31-03-2009: Draws greater than or less than 6 times the scale factor relative to
%               the location parameter are now discarded for the Student-t and Cauchy
%               distributions.
%
% * 01-04-2009: Made sure that the "mode" parameter b of the Pareto is called the
%               location parameter.
%
% * 02-04-2009: Took into account that the logistic distribution supports a shape
%               parameter. Added the Gumbel distribution.
%
% * 05-04-2009: Set "Interruptible" property of all edit controls to "off".
%
% * 09-11-2009: Took the additional values of "DSGEModel.GridWidth" and "DSGEModel.NumberOfGridPoints"
%               into account.
%
% * 06-01-2010: Updated the documentation.
%
% * 17-03-2010: Updated the documentation.
%
% * 18-03-2010: Took into account that "logInverseGammaPDF" has changed name to "logInvertedGammaPDF".
%
% * 15-04-2010: The function now determines the mode of the plotted distribution when it exists.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 25-07-2016: Changed gco to gcbo for the 'checkvalue' function.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 08-01-2018: Added plots for a log-normal distribution.
%
% * 18-01-2019: Updated the documentation.
%
% * 30-04-2019: The inverted Gamma and student-t are now allowed to have any positive value
%               for the degrees of freedom.
%
% * 07-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==0;
   PlotAPriorDLG = findobj('Type','figure','Tag','PlotAPriorDLG');
   PlotControls = get(PlotAPriorDLG,'UserData');
   DSGEModel = get(PlotControls.priordensitytext,'UserData');
   CurrINI = get(PlotControls.show,'UserData');
end;
%
switch selector;
   case 'init';
      scrsz = CurrINI.scrsz;
      PlotAPriorDLG = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','About.m', ...
         'MenuBar','none', ...
         'Units','pixels', ...
         'Position',[(scrsz(3)-600)/2 (scrsz(4)-300)/2 600 300], ...
         'Tag','PlotAPriorDLG', ...
         'Resize','off', ...
         'Visible','off', ...
         'Name','Graph-A-Prior', ...
         'NumberTitle','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','PlotAPrior done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(PlotAPriorDLG,'DockControl','off');
      end;
      %
      AxesBox([2 2 598 290],'Selections For Prior Density Plotting',200,[0.5 1],'on',CurrINI);
      %
      % select prior density
      %
      PlotControls.priordensitytext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[15 245 285 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'Enable','on', ...
         'UserData',DSGEModel, ...
         'HorizontalAlignment','left', ...
         'String','Select prior distribution:', ...
         'Tag','PriorDensityText');
      %
      PlotControls.priordensity = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[300 250 285 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','popup', ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'CallBack','PlotAPrior priordensity', ...
         'String','Gamma distribution|Beta distribution|Normal distribution|Log-normal distribution|Inverted gamma distribution|Left truncated normal distribution|Uniform distribution|Student-t distribution|Cauchy distribution|Logistic distribution|Gumbel distribution|Pareto distribution', ...
         'Value',3, ...
         'UserData',3, ...
         'Tag','PriorDensityPopup');
      %
      % prior parameters
      %
      PlotControls.parameter1text = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[15 210 135 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'String','Mean', ...
         'Tag','Parameter1Text');
      %
      PlotControls.parameter1 = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[15 190 135 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','edit', ...
         'Enable','on', ...
         'CallBack','PlotAPrior checkvalue', ...
         'HorizontalAlignment','left', ...
         'Interruptible','off', ...
         'UserData',0, ...
         'String','0', ...
         'Tag','Parameter1Edit');
      %
      PlotControls.parameter2text = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[160 210 135 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'String','Standard deviation', ...
         'Tag','Parameter2Text');
      %
      PlotControls.parameter2 = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[160 190 135 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','edit', ...
         'Enable','on', ...
         'CallBack','PlotAPrior checkvalue', ...
         'HorizontalAlignment','left', ...
         'Interruptible','off', ...
         'UserData',1, ...
         'String','1', ...
         'Tag','Parameter2Edit');
      %
      PlotControls.parameter3text = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[305 210 135 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'Enable','off', ...
         'HorizontalAlignment','left', ...
         'String','Not used', ...
         'Tag','Parameter3Text');
      %
      PlotControls.parameter3 = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[305 190 135 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','edit', ...
         'Enable','off', ...
         'CallBack','PlotAPrior checkvalue', ...
         'HorizontalAlignment','left', ...
         'Interruptible','off', ...
         'UserData',[], ...
         'String',' ', ...
         'Tag','Parameter3Edit');
      %
      PlotControls.parameter4text = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[450 210 135 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'Enable','off', ...
         'HorizontalAlignment','left', ...
         'String','Not used', ...
         'Tag','Parameter4Text');
      %
      PlotControls.parameter4 = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[450 190 135 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','edit', ...
         'Enable','off', ...
         'CallBack','PlotAPrior checkvalue', ...
         'HorizontalAlignment','left', ...
         'Interruptible','off', ...
         'UserData',[], ...
         'String',' ', ...
         'Tag','Parameter4Edit');
      %
      % select method for plotting the density
      %
      PlotControls.densitymethodtext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[15 140 285 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'String','Select density calculation method:', ...
         'Tag','DensityMethodText');
      %
      PlotControls.densitymethod = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[300 145 285 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','popup', ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'CallBack','PlotAPrior method', ...
         'String','Grid-based density estimates|Kernel density estimates', ...
         'Value',1+DSGEModel.PriorKernel, ...
         'Tag','DensityMethodPopup');
      %
      % select grid width
      %
      PlotControls.gridwidthtext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[15 110 285 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'String','Grid width:', ...
         'Tag','GridWidthText');
      %
      PlotControls.gridwidth = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[300 115 285 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','popup', ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'String','+/-1 standard deviation|+/-2 standard deviations|+/-3 standard deviations|+/-4 standard deviations|+/-5 standard deviations|+/-6 standard deviations|+/-7 standard deviations|+/-8 standard deviations|+/-9 standard deviations|+/-10 standard deviations', ...
         'Value',DSGEModel.GridWidth, ...
         'Tag','GridWidthPopup');
      %
      % select number of grid points
      %
      PlotControls.gridpointstext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[15 80 285 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'String','Number of points in grid:', ...
         'Tag','GridPointsText');
      %
      PlotControls.gridpoints = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[300 85 285 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','popup', ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'String','10|15|20|25|30|35|40|45|50|55|60|65|70|75|80|85|90|95|100|110|120|130|140|150|160|170|180|190|200', ...
         'Value',DSGEModel.NumberOfGridPoints, ...
         'Tag','GridPointsPopup');
      %
      % select kernel density estimator
      %
      PlotControls.kerneldensitytext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[15 50 285 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'Enable','off', ...
         'HorizontalAlignment','left', ...
         'String','Select kernel density estimator:', ...
         'Tag','KernelDensityText');
      %
      PlotControls.kerneldensity = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[300 55 285 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','popup', ...
         'Enable','off', ...
         'HorizontalAlignment','left', ...
         'String','Normal kernel|Epanechnikov kernel|Rectangular kernel|Triangular kernel|Bi-weight kernel|Tri-weight kernel|Laplace kernel|Logistic kernel', ...
         'Value',DSGEModel.KernelDensityValue, ...
         'UserData',strvcat('knorm','kepan','krect','ktria','kbiwe','ktriw','klapl','klogi'), ...
         'Tag','KernelDensityPopup');
      %
      % show and done buttons
      %
      PlotControls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[400 15 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','PlotAPrior show', ...
         'UserData',CurrINI, ...
         'Enable','on', ...
         'Tag','ShowButton');
      %
      PlotControls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 15 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','PlotAPrior done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','DoneButton');
      %
      % check which controls should be enabled.
      %
      if DSGEModel.PriorKernel==1;
         set(PlotControls.gridwidthtext,'Enable','off');
         set(PlotControls.gridwidth,'Enable','off');
         set(PlotControls.gridpointstext,'Enable','off');
         set(PlotControls.gridpoints,'Enable','off');
         set(PlotControls.kerneldensitytext,'Enable','on');
         set(PlotControls.kerneldensity,'Enable','on');
      end;
      set(PlotAPriorDLG,'UserData',PlotControls,...
         'HandleVisibility','callback');
   case 'show'
      SelectedDist = get(PlotControls.priordensity,'Value');
      DensityMethod = get(PlotControls.densitymethod,'Value');
      %
      % get the number of grid points and the grid width
      %
      GridWidth = get(PlotControls.gridwidth,'Value');
      GridPointsStr = get(PlotControls.gridpoints,'String');
      NumGridPoints = str2num(StringTrim(GridPointsStr(get(PlotControls.gridpoints,'Value'),:)));
      NumGridPointsHalf = NumGridPoints/2;
      MethodStr = 'Grid Density Estimate';
      if DensityMethod==2;
         %
         % get the kernel density estimator
         %
         KDEStr = get(PlotControls.kerneldensity,'UserData');
         KernelDensityEstimator = StringTrim(KDEStr(get(PlotControls.kerneldensity,'Value'),:));
         %
         % initialize random number generators
         %
         if DSGEModel.RandomNumberValue==1;
            rand('state',0);
            randn('state',0);
         else;
            rand('state',sum(100*clock));
            randn('state',sum(100*clock));
         end;
         MethodStr = 'Kernel Density Estimate';
      end;
      Parameter1 = get(PlotControls.parameter1,'UserData');
      Parameter2 = get(PlotControls.parameter2,'UserData');
      ErrorStr = '';
      try;
         if SelectedDist==1;
            %
            % gamma distribution
            %
            LowerBound = get(PlotControls.parameter3,'UserData');
            [a,b] = MomentToParamGammaPDF(Parameter1-LowerBound,Parameter2);
            %
            if a>1;
               ModeValue = (b*(a-1))+LowerBound;
            else;
               ModeValue = NaN;
            end;
            %
            % check which method we should use
            %
            if DensityMethod==1;
               %
               % grid values
               %
               lb = Parameter1-(GridWidth*Parameter2);
               dev = (Parameter1-lb)/NumGridPointsHalf;
               if lb<=LowerBound;
                  lb = 1.0e-004+LowerBound;
               end;
               XValues = (lb:dev:lb+(NumGridPoints*dev))';
               YValues = exp(logGammaPDF(XValues-LowerBound,a,b));
            else;
               %
               % random draws from the prior
               %
               z = LowerBound+GammaRndFcn(a,b,5000);
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               if sum(XValues>LowerBound)<length(XValues);
                  XValues = XValues(XValues>LowerBound);
                  YValues = YValues(length(YValues)-length(XValues)+1:length(YValues));
               end;
            end;
   %         TitleStr = strvcat('Gamma distribution',['(mean,standard deviation,lower bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(LowerBound) ')']);
            TitleStr = {'Gamma distribution' ['(mean,standard deviation,lower bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(LowerBound) ')']};
            NameStr = 'Gamma Distribution';
            Bounds = [LowerBound Inf];
         elseif SelectedDist==2;
            %
            % beta distribution
            %
            LowerBound = get(PlotControls.parameter3,'UserData');
            UpperBound = get(PlotControls.parameter4,'UserData');
            [a,b] = MomentToParamStdbetaPDF((Parameter1-LowerBound)/(UpperBound-LowerBound),Parameter2/(UpperBound-LowerBound));
            %
            if (a>1)&(b>1);
               ModeValue = LowerBound+(((UpperBound-LowerBound)*(a-1))/(a+b-2));
            else;
               ModeValue = NaN;
            end;
            %
            % check which method we should use
            %
            if DensityMethod==1;
               %
               % grid values
               %
               lb = Parameter1-(GridWidth*Parameter2);
               dev = (Parameter1-lb)/NumGridPointsHalf;
               if lb<=LowerBound;
                  lb = LowerBound+1.0e-004;
               end;
               XValues = (lb:dev:lb+(NumGridPoints*dev))';
               XValues(XValues>=UpperBound) = UpperBound-1.0e-004;
               YValues = exp(logBetaPDF(XValues,a,b,LowerBound,UpperBound));
            else;
               %
               % random draws from the prior
               %
               z = BetaRndFcn(a,b,LowerBound,UpperBound,5000);
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               if sum(XValues>LowerBound)<length(XValues);
                  XValues = XValues(XValues>LowerBound);
                  YValues = YValues(length(YValues)-length(XValues)+1:length(YValues));
               end;
               if sum(XValues<UpperBound)<length(XValues);
                  XValues = XValues(XValues<UpperBound);
                  YValues = YValues(1:length(XValues));
               end;
            end;
   %         TitleStr = strvcat('Beta distribution',['(mean,standard deviation,lower bound,upper bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(LowerBound) ',' num2str(UpperBound) ')']);
            TitleStr = {'Beta distribution' ['(mean,standard deviation,lower bound,upper bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(LowerBound) ',' num2str(UpperBound) ')']};
            NameStr = 'Beta Distribution';
            Bounds = [LowerBound UpperBound];
         elseif SelectedDist==3;
            %
            % normal distribution
            %
            lb = Parameter1-(GridWidth*Parameter2);
            dev = (Parameter1-lb)/NumGridPointsHalf;
            XValues = (lb:dev:lb+(NumGridPoints*dev))';
            logDensityValues = logNormalPDF(XValues,Parameter1,Parameter2);
            %
            ModeValue = Parameter1;
            %
            % check which method we should use
            %
            if DensityMethod==1;
               YValues = exp(logDensityValues);
            else;
               z = NormalRndFcn(Parameter1,Parameter2,5000);
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               YValues = (max(exp(logDensityValues))/max(YValues))*YValues;
            end;
   %         TitleStr = strvcat('Normal distribution',['(mean,standard deviation) = (' num2str(Parameter1) ',' num2str(Parameter2) ')']);
            TitleStr = {'Normal distribution' ['(mean,standard deviation) = (' num2str(Parameter1) ',' num2str(Parameter2) ')']};
            NameStr = 'Normal Distribution';
            Bounds = [-Inf Inf];
         elseif SelectedDist==4;
            %
            % log-normal distribution: convert to mu and sigma
            %
            sigma = sqrt(log(((Parameter2/Parameter1)^2)+1));
            mu = log(Parameter1)-((sigma^2)/2);
            %
            % create the grid for the logged values
            %
            lb = mu-(GridWidth*sigma);
            dev = (mu-lb)/NumGridPointsHalf;
            XValues = (lb:dev:lb+(NumGridPoints*dev))';
            %
            logDensityValues = logNormalPDF(XValues,mu,sigma)-XValues;
            %
            ModeValue = exp(mu-sigma^2);
            %
            % check which method we should use
            %
            if DensityMethod==1;
               YValues = exp(logDensityValues);
               XValues = exp(XValues);
            else;
               if exist('lognrnd')==2;
                  z = lognrnd(mu,sigma,1,5000);
               else;
                  z = exp(NormalRndFcn(mu,sigma,5000));
               end;
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               YValues = (max(exp(logDensityValues))/max(YValues))*YValues;
            end;
            TitleStr = {'Log-normal distribution' ['(mean,standard deviation,lower bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ',0)']};
            NameStr = 'Log-normal Distribution';
            Bounds = [0 Inf];
         elseif SelectedDist==5;
            %
            % inverted gamma distribution
            %
            LowerBound = get(PlotControls.parameter3,'UserData');
            %
            ModeValue = LowerBound+(Parameter1*sqrt(Parameter2/(Parameter2+1)));
            %
            % check which method we should use
            %
            if DensityMethod==1;
               %
               % grid values
               %
               if Parameter2>2;
                  InvGammaMean = gammaln((Parameter2-1)/2)-gammaln(Parameter2/2);
                  InvGammaMean = exp(InvGammaMean)*sqrt(Parameter2/2)*Parameter1;
                  InvGammaStd = sqrt(((Parameter2*Parameter1*Parameter1)/(Parameter2-2))-(InvGammaMean^2));
               else;
                  InvGammaMean = gammaln(1)-gammaln(3/2);
                  InvGammaMean = exp(InvGammaMean)*sqrt(3/2)*Parameter1;
                  InvGammaStd = sqrt(((3*Parameter1*Parameter1))-(InvGammaMean^2));
               end;
               lb = Parameter1-(GridWidth*InvGammaStd);
               dev = (Parameter1-lb)/NumGridPointsHalf;
               if lb<=LowerBound;
                  lb = 1.0e-004+LowerBound;
               end;
               XValues = (lb:dev:lb+(NumGridPoints*dev))';
               YValues = exp(logInvertedGammaPDF(XValues-LowerBound,Parameter1-LowerBound,Parameter2));
            else;
               %
               % random draws from the prior
               %
               z = LowerBound+InvGammaRndFcn(Parameter1-LowerBound,Parameter2,5000);
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               if sum(XValues>LowerBound)<length(XValues);
                  XValues = XValues(XValues>LowerBound);
                  YValues = YValues(length(YValues)-length(XValues)+1:length(YValues));
               end;
            end;
   %         TitleStr = strvcat('Inverted gamma distribution',['(location,degrees of freedom,lower bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(LowerBound) ')']);
            TitleStr = {'Inverted gamma distribution' ['(location,degrees of freedom,lower bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(LowerBound) ')']};
            NameStr = 'Inverted Gamma Distribution';
            Bounds = [LowerBound Inf];
         elseif SelectedDist==6;
            %
            % left truncated normal distribution
            %
            LowerBound = get(PlotControls.parameter3,'UserData');
            %
            if Parameter1>=LowerBound
               ModeValue = Parameter1;
            else;
               ModeValue = LowerBound;
            end;
            %
            % check which method we should use
            %
            if DensityMethod==1;
               %
               % grid values
               %
               lb = Parameter1-(GridWidth*Parameter2);
               dev = abs(Parameter1-lb)/NumGridPointsHalf;
               XValues = (max(lb,LowerBound+1.0e-008):dev:max(lb,LowerBound+1.0e-008)+(NumGridPoints*dev))';
               YValues = exp(logLTNormalPDF(XValues,Parameter1,Parameter2,LowerBound));
            else;
               %
               % random draws from the prior
               %
               z = LTNormalRndFcn(Parameter1,Parameter2,LowerBound,5000);
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               if sum(XValues>LowerBound)<length(XValues);
                  XValues = XValues(XValues>LowerBound);
                  YValues = YValues(length(YValues)-length(XValues)+1:length(YValues));
               end;
            end;
   %         TitleStr = strvcat('Left truncated normal distribution',['(location,scale,lower bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(LowerBound) ')']);
            TitleStr = {'Left truncated normal distribution' ['(location,scale,lower bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(LowerBound) ')']};
            NameStr = 'Left Truncated Normal Distribution';
            Bounds = [LowerBound Inf];
         elseif SelectedDist==7;
            %
            % uniform distribution
            %
            ModeValue = NaN;
            %
            if DensityMethod==1;
               %
               % grid values: we only need two
               %
               XValues = [Parameter1 Parameter2]';
               YValues = exp(logUniformPDF(Parameter1,Parameter2)*ones(2,1));
            else;
               %
               % random draws from the prior
               %
               z = UniformRndFcn(Parameter1,Parameter2,5000);
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               if sum(XValues>Parameter1)<length(XValues);
                  XValues = XValues(XValues>Parameter1);
                  YValues = YValues(length(YValues)-length(XValues)+1:length(YValues));
               end;
               if sum(XValues<Parameter2)<length(XValues);
                  XValues = XValues(XValues<Parameter2);
                  YValues = YValues(1:length(XValues));
               end;
            end;
   %         TitleStr = strvcat('Uniform distribution',['(lower bound,upper bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ')']);
            TitleStr = {'Uniform distribution' ['(lower bound,upper bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ')']};
            NameStr = 'Uniform Distribution';
            Bounds = [Parameter1 Parameter2];
         elseif SelectedDist==8;
            %
            % Student-t distribution
            %
            DegFree = get(PlotControls.parameter3,'UserData');
            %
            ModeValue = Parameter1;
            %
            % grid values
            %
            lb = Parameter1-(GridWidth*Parameter2);
            dev = (Parameter1-lb)/NumGridPointsHalf;
            XValues = (lb:dev:lb+(NumGridPoints*dev))';
            logDensityValues = logStudentTAltPDF(XValues,Parameter1,Parameter2,DegFree);
            %
            % check which method we should use
            %
            if DensityMethod==1;
               YValues = exp(logDensityValues);
            else;
               %
               % random draws from the prior
               %
               Numz = 0;
               Oldz = [];
               lb = Parameter1-(6*Parameter2);
               ub = Parameter1+(6*Parameter2);
               while Numz<5000;
                  z = StudentTAltRndFcn(Parameter1,Parameter2,DegFree,10000);
                  %
                  % remove value of z that are too big or too small
                  %
                  if sum(z>=lb)<length(z);
                     z = z(z>=lb);
                  end;
                  if sum(z<=ub)<length(z);
                     z = z(z<=ub);
                  end;
                  Oldz = [Oldz z];
                  z = Oldz;
                  if length(z)>5000;
                     z = z(1:5000);
                  end;
                  Numz = length(z);
               end;
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               YValues = (max(exp(logDensityValues))/max(YValues))*YValues;
            end;
   %         TitleStr = strvcat('Student-t distribution',['(location,scale,degrees of freedom) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(DegFree) ')']);
            TitleStr = {'Student-t distribution' ['(location,scale,degrees of freedom) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(DegFree) ')']};
            NameStr = 'Student-t Distribution';
            Bounds = [-Inf Inf];
         elseif SelectedDist==9;
            %
            % Cauchy distribution
            %
            ModeValue = Parameter1;
            %
            lb = Parameter1-(GridWidth*Parameter2);
            dev = (Parameter1-lb)/NumGridPointsHalf;
            XValues = (lb:dev:lb+(NumGridPoints*dev))';
            logDensityValues = logCauchyPDF(XValues,Parameter1,Parameter2);
            %
            % check which method we should use
            %
            if DensityMethod==1;
               YValues = exp(logDensityValues);
            else;
               %
               % random draws from the prior
               %
               Numz = 0;
               Oldz = [];
               lb = Parameter1-(6*Parameter2);
               ub = Parameter1+(6*Parameter2);
               while Numz<5000;
                  z = CauchyRndFcn(Parameter1,Parameter2,10000);
                  %
                  % remove value of z that are too big or too small
                  %
                  if sum(z>=lb)<length(z);
                     z = z(z>=lb);
                  end;
                  if sum(z<=ub)<length(z);
                     z = z(z<=ub);
                  end;
                  Oldz = [Oldz z];
                  z = Oldz;
                  if length(z)>5000;
                     z = z(1:5000);
                  end;
                  Numz = length(z);
               end;
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               YValues = (max(exp(logDensityValues))/max(YValues))*YValues;
            end;
   %         TitleStr = strvcat('Cauchy distribution',['(location,scale) = (' num2str(Parameter1) ',' num2str(Parameter2) ')']);
            TitleStr = {'Cauchy distribution' ['(location,scale) = (' num2str(Parameter1) ',' num2str(Parameter2) ')']};
            NameStr = 'Cauchy Distribution';
            Bounds = [-Inf Inf];
         elseif SelectedDist==10;
            %
            % logistic distribution
            %
            c = get(PlotControls.parameter3,'UserData');
            [a,b] = MomentToParamLogisticPDF(Parameter1,Parameter2,c);
            ModeValue = a+(b*log(c));
            %
            lb = (a+(b*log(c)))-(GridWidth*Parameter2);
            dev = (a+(b*log(c))-lb)/NumGridPointsHalf;
            XValues = (lb:dev:lb+(NumGridPoints*dev))';
            logDensityValues = logLogisticPDF(XValues,a,b,c);
            if DensityMethod==1;
               YValues = exp(logDensityValues);
            else;
               %
               % random draws from the prior
               %
               z = LogisticRndFcn(a,b,c,5000);
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               YValues = (max(exp(logDensityValues))/max(YValues))*YValues;
            end;
   %         TitleStr = strvcat('Logistic distribution',['(mean,standard deviation) = (' num2str(Parameter1) ',' num2str(Parameter2) ')']);
            TitleStr = {'Logistic distribution' ['(mean,standard deviation,shape) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(c) ')']};
            NameStr = 'Logistic Distribution';
            Bounds = [-Inf Inf];
         elseif SelectedDist==11;
            %
            % Gumbel distribution
            %
            [a,b] = MomentToParamGumbelPDF(Parameter1,Parameter2);
            ModeValue = a;
            lb = Parameter1-(GridWidth*Parameter2);
            dev = (Parameter1-lb)/NumGridPointsHalf;
            XValues = (lb:dev:lb+(NumGridPoints*dev))';
            logDensityValues = logGumbelPDF(XValues,a,b);
            if DensityMethod==1;
               YValues = exp(logDensityValues);
            else;
               %
               % random draws from the prior
               %
               z = GumbelRndFcn(a,b,5000);
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               YValues = (max(exp(logDensityValues))/max(YValues))*YValues;
            end;
            TitleStr = {'Gumbel distribution' ['(mean,standard deviation) = (' num2str(Parameter1) ',' num2str(Parameter2) ')']};
            NameStr = 'Gumbel Distribution';
            Bounds = [-Inf Inf];
         else;
            %
            % pareto distribution: select an appropriate upper bound, i.e. we let
            % 99.9 percent of the mass is less than or equal to the upper bound
            % using the cdf
            %
            c = get(PlotControls.parameter3,'UserData');
            ub = c+(Parameter2/((0.01)^(1/Parameter1)));
            if ub-Parameter2-c>20;
               ub = 20+Paremeter2+c;
            end;
            ModeValue = c+Parameter2;
            if DensityMethod==1;
               %
               % grid values
               %
               dev = (ub-Parameter2-c)/NumGridPoints;
               XValues = (Parameter2+c:dev:ub)';
               YValues = exp(logParetoPDF(XValues-c,Parameter1,Parameter2));
            else;
               %
               % random draws from the prior
               %
               z = c+ParetoRndFcn(Parameter1,Parameter2,5000);
               [XValues,YValues] = KernelDensityPlot(z,KernelDensityEstimator);
               %
               % compute the height of the pareto at Parameter2
               %
               maxValue = exp(logParetoPDF(Parameter2,Parameter1,Parameter2));
               YValues = (maxValue/max(YValues))*YValues;
               %
               % deal with the lower bound
               %
               if sum(XValues>Parameter2)<length(XValues);
                  XValues = XValues(XValues>Parameter2);
                  YValues = YValues(length(YValues)-length(XValues)+1:length(YValues));
               end;
               %
               % deal with the upper bound
               %
               if sum(XValues<=ub)<length(XValues);
                  XValues = XValues(XValues<=ub);
                  YValues = YValues(1:length(XValues));
               end;
            end;
   %         TitleStr = strvcat('Pareto distribution',['(shape,lower bound) = (' num2str(Parameter1) ',' num2str(Parameter2) ')']);
            TitleStr = {'Pareto distribution' ['(shape,location,origin) = (' num2str(Parameter1) ',' num2str(Parameter2) ',' num2str(c) ')']};
            NameStr = 'Pareto Distribution';
            Bounds = [Parameter2+c ub+0.1];
         end;
      catch;
         ErrorStr = ['YADA was unable to compute the data for the graph. The error message caught is: ' YADALastError];
         if SelectedDist==1;
            NameStr = 'Gamma Distribution';
         elseif SelectedDist==2;
            NameStr = 'Beta Distribution';
         elseif SelectedDist==3;
            NameStr = 'Normal Distribution';
         elseif SelectedDist==4;
            NameStr = 'Log-normal Distribution';
         elseif SelectedDist==5;
            NameStr = 'Inverted Gamma Distribution';
         elseif SelectedDist==6;
            NameStr = 'Left Truncated Normal Distribution';
         elseif SelectedDist==7;
            NameStr = 'Uniform Distribution';
         elseif SelectedDist==8;
            NameStr = 'Student-t Distribution';
         elseif SelectedDist==9;
            NameStr = 'Cauchy Distribution';
         elseif SelectedDist==10;
            NameStr = 'Logistic Distribution';
         elseif SelectedDist==11;
            NameStr = 'Gumbel Distribution';
         elseif SelectedDist==12;
            NameStr = 'Pareto Distribution';
         end;
      end;
      %
      if isempty(ErrorStr)==1;
         %
         % create the figure
         %
         FigHandle =figure('Visible','off', ...
                   'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                   'Tag','JustAnotherYADAFigure', ...
                   'Name',[NameStr ' - ' MethodStr]);
         %
         SetFigureProperties(FigHandle,CurrINI);
         %
         plot(XValues,YValues,'-','Color',[0.75 0 0],'LineWidth',2);
         hold('on');
         %
         % deal with x-axis limits
         %
         XLimit = get(gca,'XLim');
         if isinf(Bounds(1))==0;
            XLimit(1) = Bounds(1);
         end;
         if isinf(Bounds(2))==0;
            XLimit(2) = Bounds(2);
         end;
         if (isnan(XLimit(2))==1)|(XLimit(1)>XLimit(2));
            XLimit(2) = XLimit(1)+1;
         end;
         %
         % and y-axis
         %
         YLimit = get(gca,'YLim');
         YLimit(1) = 0;
         YLimit(2) = 1.1*max(YValues);
         set(gca,'XLim',XLimit,'YLim',YLimit);
         %
         % write the title
         %
         SetAxesFonts(CurrINI,'Title',TitleStr,'XLabel',['mode=' num2str(ModeValue)]);
         %
         % show the plot
         %
         hold('off');
         %
         % display the graph
         %
         SetToolbarImages(FigHandle,CurrINI.images);
         set(FigHandle,'Visible','on');
         drawnow;
         pause(0.02);
      else;
         %
         % show an error dialog
         %
         About(ErrorStr,'error',['Error: ' NameStr ' - ' MethodStr],500,600,CurrINI);
      end;
   case 'method'
      if get(PlotControls.densitymethod,'Value')==1;
         set(PlotControls.gridwidthtext,'Enable','on');
         set(PlotControls.gridwidth,'Enable','on');
         set(PlotControls.gridpointstext,'Enable','on');
         set(PlotControls.gridpoints,'Enable','on');
         set(PlotControls.kerneldensitytext,'Enable','off');
         set(PlotControls.kerneldensity,'Enable','off');
      else;
         set(PlotControls.gridwidthtext,'Enable','off');
         set(PlotControls.gridwidth,'Enable','off');
         set(PlotControls.gridpointstext,'Enable','off');
         set(PlotControls.gridpoints,'Enable','off');
         set(PlotControls.kerneldensitytext,'Enable','on');
         set(PlotControls.kerneldensity,'Enable','on');
      end;
   case 'priordensity'
      SelectedDist = get(PlotControls.priordensity,'Value');
      OldDist = get(PlotControls.priordensity,'UserData');
      if SelectedDist~=OldDist;
         set(PlotControls.priordensity,'UserData',SelectedDist);
         if SelectedDist==1;
            %
            % gamma distribution
            %
            set(PlotControls.parameter1text,'String','Mean');
            set(PlotControls.parameter1,'String','1','UserData',1);
            set(PlotControls.parameter2text,'String','Standard deviation');
            set(PlotControls.parameter2,'String','1','UserData',1);
            set(PlotControls.parameter3text,'String','Lower bound','Enable','on');
            set(PlotControls.parameter3,'String','0','UserData',0,'Enable','on');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         elseif SelectedDist==2;
            %
            % beta distribution
            %
            set(PlotControls.parameter1text,'String','Mean');
            set(PlotControls.parameter1,'String','0.5','UserData',0.5);
            set(PlotControls.parameter2text,'String','Standard deviation');
            set(PlotControls.parameter2,'String',num2str(sqrt(1/12),'%0.6f'),'UserData',sqrt(1/12));
            set(PlotControls.parameter3text,'String','Lower bound','Enable','on');
            set(PlotControls.parameter3,'String','0','UserData',0,'Enable','on');
            set(PlotControls.parameter4text,'String','Upper bound','Enable','on');
            set(PlotControls.parameter4,'String','1','UserData',1,'Enable','on');
         elseif SelectedDist==3;
            %
            % normal distribution
            %
            set(PlotControls.parameter1text,'String','Mean');
            set(PlotControls.parameter1,'String','0','UserData',0);
            set(PlotControls.parameter2text,'String','Standard deviation');
            set(PlotControls.parameter2,'String','1','UserData',1);
            set(PlotControls.parameter3text,'String','Not used','Enable','off');
            set(PlotControls.parameter3,'String',' ','UserData',[],'Enable','off');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         elseif SelectedDist==4;
            %
            % log-normal distribution
            %
            set(PlotControls.parameter1text,'String','Mean');
            set(PlotControls.parameter1,'String','1','UserData',1);
            set(PlotControls.parameter2text,'String','Standard deviation');
            set(PlotControls.parameter2,'String','1','UserData',1);
            set(PlotControls.parameter3text,'String','Not used','Enable','off');
            set(PlotControls.parameter3,'String',' ','UserData',[],'Enable','off');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         elseif SelectedDist==5;
            %
            % inverted gamma distribution
            %
            set(PlotControls.parameter1text,'String','Location');
            set(PlotControls.parameter1,'String','0.1','UserData',0.1);
            set(PlotControls.parameter2text,'String','Degrees of freedom');
            set(PlotControls.parameter2,'String','3','UserData',3);
            set(PlotControls.parameter3text,'String','Lower bound','Enable','on');
            set(PlotControls.parameter3,'String','0','UserData',0,'Enable','on');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         elseif SelectedDist==6;
            %
            % left truncated normal distribution
            %
            set(PlotControls.parameter1text,'String','Location');
            set(PlotControls.parameter1,'String','0','UserData',0);
            set(PlotControls.parameter2text,'String','Scale');
            set(PlotControls.parameter2,'String','1','UserData',1);
            set(PlotControls.parameter3text,'String','Lower bound','Enable','on');
            set(PlotControls.parameter3,'String','-1','UserData',-1,'Enable','on');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         elseif SelectedDist==7;
            %
            % uniform distribution
            %
            set(PlotControls.parameter1text,'String','Lower bound');
            set(PlotControls.parameter1,'String','0','UserData',0);
            set(PlotControls.parameter2text,'String','Upper bound');
            set(PlotControls.parameter2,'String','1','UserData',1);
            set(PlotControls.parameter3text,'String','Not used','Enable','off');
            set(PlotControls.parameter3,'String',' ','UserData',[],'Enable','off');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         elseif SelectedDist==8;
            %
            % Student-t distribution
            %
            set(PlotControls.parameter1text,'String','Location');
            set(PlotControls.parameter1,'String','0','UserData',0);
            set(PlotControls.parameter2text,'String','Scale');
            set(PlotControls.parameter2,'String','1','UserData',1);
            set(PlotControls.parameter3text,'String','Degrees of freedom','Enable','on');
            set(PlotControls.parameter3,'String','1','UserData',1,'Enable','on');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         elseif SelectedDist==9;
            %
            % Cauchy distribution
            %
            set(PlotControls.parameter1text,'String','Location');
            set(PlotControls.parameter1,'String','0','UserData',0);
            set(PlotControls.parameter2text,'String','Scale');
            set(PlotControls.parameter2,'String','1','UserData',1);
            set(PlotControls.parameter3text,'String','Not used','Enable','off');
            set(PlotControls.parameter3,'String',' ','UserData',[],'Enable','off');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         elseif SelectedDist==10;
            %
            % logistic distribution
            %
            set(PlotControls.parameter1text,'String','Mean');
            set(PlotControls.parameter1,'String','0','UserData',0);
            set(PlotControls.parameter2text,'String','Standard deviation');
            set(PlotControls.parameter2,'String','1','UserData',1);
            set(PlotControls.parameter3text,'String','shape','Enable','on');
            set(PlotControls.parameter3,'String','1','UserData',1,'Enable','on');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         elseif SelectedDist==11;
            %
            % Gumbel distribution
            %
            set(PlotControls.parameter1text,'String','Mean');
            set(PlotControls.parameter1,'String','0','UserData',0);
            set(PlotControls.parameter2text,'String','Standard deviation');
            set(PlotControls.parameter2,'String','1','UserData',1);
            set(PlotControls.parameter3text,'String','Not used','Enable','off');
            set(PlotControls.parameter3,'String',' ','UserData',[],'Enable','off');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         else;
            %
            % Pareto distribution
            %
            set(PlotControls.parameter1text,'String','Shape');
            set(PlotControls.parameter1,'String','3','UserData',3);
            set(PlotControls.parameter2text,'String','Location');
            set(PlotControls.parameter2,'String',num2str(2/3,'%0.6f'),'UserData',2/3);
            set(PlotControls.parameter3text,'String','Origin','Enable','on');
            set(PlotControls.parameter3,'String','0','UserData',0,'Enable','on');
            set(PlotControls.parameter4text,'String','Not used','Enable','off');
            set(PlotControls.parameter4,'String',' ','UserData',[],'Enable','off');
         end;
      end;
   case 'checkvalue'
      %
      % make sure that the value is numeric
      %
      Value = str2num(StringTrim(get(gcbo,'String')));
      if (isempty(Value)==1)|(isnumeric(Value)==0)|(isnan(Value)==1)|(isinf(Value)==1);
         set(gcbo,'String',num2str(get(gcbo,'UserData'),'%0.6f'));
         return;
      end;
      SelectedDist = get(PlotControls.priordensity,'Value');
      if gcbo==PlotControls.parameter1;
         %
         % check if value is allowed
         %
         if SelectedDist==1;
            %
            % gamma: parameter 1 must be greater than lower bound (parameter 3)
            %
            LowerBound = get(PlotControls.parameter3,'UserData');
            if Value>LowerBound;
               set(PlotControls.parameter1,'UserData',Value);
            else;
               set(PlotControls.parameter1,'String',num2str(get(PlotControls.parameter1,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the mean (' num2str(Value,'%0.6f') ') is less than or equal to the lower bound (' num2str(LowerBound,'%0.6f') '). The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Gamma Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==2;
            %
            % beta: parameter 1 must be greater than lower bound and less than upper bound
            %
            LowerBound = get(PlotControls.parameter3,'UserData');
            UpperBound = get(PlotControls.parameter4,'UserData');
            if (Value>LowerBound)&(Value<UpperBound);
               %
               % test that the implied a and b parameters are both positive
               %
               BetaVariance = get(PlotControls.parameter2,'UserData')^2;
               if (Value-LowerBound)*(UpperBound-Value)>BetaVariance;
                  set(PlotControls.parameter1,'UserData',Value);
               else;
                  set(PlotControls.parameter1,'String',num2str(get(PlotControls.parameter1,'UserData')));
                  %
                  % show a message box
                  %
                  txt = ['The beta distribution is based on shape parameters a and b that are positive. This implies that the mean is greater than the lower bound (lb) and less than the upper bound (ub). In addition, the product (mean-lb)(ub-mean) must be greater than the variance. The second condition is violated by your choice of parameters. The entry was therefore rejected.'];
                  About(txt,'information','Graph-A-Prior - Beta Distribution',200,500,CurrINI);
               end;
            else;
               set(PlotControls.parameter1,'String',num2str(get(PlotControls.parameter1,'UserData')));
               %
               % show a message box
               %
               if Value<=LowerBound;
                  txt = ['The value you selected for the mean (' num2str(Value,'%0.6f') ') is less than or equal to the lower bound (' num2str(LowerBound,'%0.6f') '). The entry was therefore rejected.'];
               else;
                  txt = ['The value you selected for the mean (' num2str(Value,'%0.6f') ') is greater than or equal to the upper bound (' num2str(UpperBound,'%0.6f') '). The entry was therefore rejected.'];
               end;
               About(txt,'information','Graph-A-Prior - Beta Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==4;
            %
            % log-normal: parameter 1 must be positive
            %
            if Value>0;
               set(PlotControls.parameter1,'UserData',Value);
            else;
               set(PlotControls.parameter1,'String',num2str(get(PlotControls.parameter1,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the mean (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Log-normal Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==5;
            %
            % inverted gamma: parameter 1 must be greater than lower bound (parameter 3)
            %
            LowerBound = get(PlotControls.parameter3,'UserData');
            if Value>LowerBound;
               set(PlotControls.parameter1,'UserData',Value);
            else;
               set(PlotControls.parameter1,'String',num2str(get(PlotControls.parameter1,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the location parameter (' num2str(Value,'%0.6f') ') is less than or equal to the lower bound (' num2str(LowerBound,'%0.6f') '). The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Inverted Gamma Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==7;
            %
            % uniform: parameter 1 must be less than the upper bound (parameter 2)
            %
            UpperBound = get(PlotControls.parameter2,'UserData');
            if Value<UpperBound;
               set(PlotControls.parameter1,'UserData',Value);
            else;
               set(PlotControls.parameter1,'String',num2str(get(PlotControls.parameter1,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the lower bound (' num2str(Value,'%0.6f') ') is greater than or equal to the upper bound (' num2str(UpperBound,'%0.6f') '). The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Uniform Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==12;
            %
            % pareto: parameter 1 must be positive
            %
            if Value>0;
               set(PlotControls.parameter1,'UserData',Value);
            else;
               set(PlotControls.parameter1,'String',num2str(get(PlotControls.parameter1,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the shape parameter (' num2str(Value,'%0.6f') ') is less than or equal to the zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Pareto Distribution',200,500,CurrINI);
            end;
         else;
            set(PlotControls.parameter1,'UserData',Value);
         end;
      elseif gcbo==PlotControls.parameter2;
         %
         % check if value is allowed
         %
         if SelectedDist==1;
            %
            % gamma: parameter 2 must be positive
            %
            if Value>0;
               set(PlotControls.parameter2,'UserData',Value);
            else;
               set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the standard deviation (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Gamma Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==2;
            %
            % beta: parameter 2 must be positive and be consistent with a and b being positive
            %
            if Value>0;
               LowerBound = get(PlotControls.parameter3,'UserData');
               UpperBound = get(PlotControls.parameter4,'UserData');
               BetaMean = get(PlotControls.parameter1,'UserData');
               if (BetaMean-LowerBound)*(UpperBound-BetaMean)>Value^2;
                  set(PlotControls.parameter2,'UserData',Value);
               else;
                  set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
                  %
                  % show a message box
                  %
                  txt = ['The beta distribution is based on shape parameters a and b that are positive. This implies that the mean is greater than the lower bound (lb) and less than the upper bound (ub). In addition, the product (mean-lb)(ub-mean) must be greater than the variance. The second condition is violated by your choice of parameters. The entry was therefore rejected.'];
                  About(txt,'information','Graph-A-Prior - Beta Distribution',200,500,CurrINI);
               end;
            else;
               set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the standard deviation (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - beta Distribution',200,500,CurrINI);
            end;
         elseif (SelectedDist==3)|(SelectedDist==4);
            %
            % normal and log-normal: parameter 2 must be positive
            %
            if Value>0;
               set(PlotControls.parameter2,'UserData',Value);
            else;
               set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the standard deviation (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Normal Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==5;
            %
            % inverted gamma: parameter 2 must be positive
            %
            if Value>0;
               set(PlotControls.parameter2,'UserData',Value);
            else;
               set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the number of degrees of freedom parameter (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Inverted Gamma Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==6;
            %
            % left truncated normal: parameter 2 must be positive
            %
            if Value>0;
               set(PlotControls.parameter2,'UserData',Value);
            else;
               set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the scale parameter (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Left Truncated Normal Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==7;
            %
            % uniform: parameter 2 must be greater then the lower bound (parameter 1)
            %
            LowerBound = get(PlotControls.parameter1,'UserData');
            if Value>LowerBound;
               set(PlotControls.parameter2,'UserData',Value);
            else;
               set(PlotControls.parameter1,'String',num2str(get(PlotControls.parameter1,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the upper bound (' num2str(Value,'%0.6f') ') is less than or equal to the upper bound (' num2str(LowerBound,'%0.6f') '). The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Uniform Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==8;
            %
            % student-t: parameter 2 must be positive
            %
            if Value>0;
               set(PlotControls.parameter2,'UserData',Value);
            else;
               set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the scale parameter (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Student-t Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==9;
            %
            % Cauchy: parameter 2 must be positive
            %
            if Value>0;
               set(PlotControls.parameter2,'UserData',Value);
            else;
               set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the scale parameter (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Cauchy Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==10;
            %
            % Logistic: parameter 2 must be positive
            %
            if Value>0;
               set(PlotControls.parameter2,'UserData',Value);
            else;
               set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the standard deviation (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Logistic Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==11;
            %
            % Gumbel: parameter 2 must be positive
            %
            if Value>0;
               set(PlotControls.parameter2,'UserData',Value);
            else;
               set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the standard deviation (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Gumbel Distribution',200,500,CurrINI);
            end;
         else;
            %
            % Pareto: parameter 2 must be positive
            %
            if Value>0;
               set(PlotControls.parameter2,'UserData',Value);
            else;
               set(PlotControls.parameter2,'String',num2str(get(PlotControls.parameter2,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the location parameter (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Pareto Distribution',200,500,CurrINI);
            end;
         end;
      elseif gcbo==PlotControls.parameter3;
         %
         % check if value is allowed
         %
         if SelectedDist==1;
            %
            % gamma: parameter 3 must be less than the mean (parameter 1)
            %
            GammaMean = get(PlotControls.parameter1,'UserData');
            if Value<GammaMean;
               set(PlotControls.parameter3,'UserData',Value);
            else;
               set(PlotControls.parameter3,'String',num2str(get(PlotControls.parameter3,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the lower bound (' num2str(Value,'%0.6f') ') is greater than or equal to the mean (' num2str(GammaMean,'%0.6f') '). The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Gamma Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==2;
            %
            % beta: parameter 3 must be less than the mean
            %
            BetaMean = get(PlotControls.parameter1,'UserData');
            UpperBound = get(PlotControls.parameter4,'UserData');
            if BetaMean>Value;
               %
               % test that the implied a and b parameters are both positive
               %
               BetaVariance = get(PlotControls.parameter2,'UserData')^2;
               if (BetaMean-Value)*(UpperBound-BetaMean)>BetaVariance;
                  set(PlotControls.parameter3,'UserData',Value);
               else;
                  set(PlotControls.parameter3,'String',num2str(get(PlotControls.parameter3,'UserData')));
                  %
                  % show a message box
                  %
                  txt = ['The beta distribution is based on shape parameters a and b that are positive. This implies that the mean is greater than the lower bound (lb) and less than the upper bound (ub). In addition, the product (mean-lb)(ub-mean) must be greater than the variance. The second condition is violated by your choice of parameters. The entry was therefore rejected.'];
                  About(txt,'information','Graph-A-Prior - Beta Distribution',200,500,CurrINI);
               end;
            else;
               set(PlotControls.parameter3,'String',num2str(get(PlotControls.parameter3,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the lower bound (' num2str(Value,'%0.6f') ') is greater than or equal to the mean (' num2str(BetaMean,'%0.6f') '). The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Beta Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==5;
            %
            % inverted gamma: parameter 3 must be less than the location parameter
            %
            InvGammaLocation = get(PlotControls.parameter1,'UserData');
            if Value<InvGammaLocation;
               set(PlotControls.parameter3,'UserData',Value);
            else;
               set(PlotControls.parameter3,'String',num2str(get(PlotControls.parameter3,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the lower bound (' num2str(Value,'%0.6f') ') is greater than or equal to the location parameter (' num2str(InvGammaLocation,'%0.6f') '). The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Inverted Gamma Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==6;
            %
            % left truncated normal: parameter 3 must be real
            %
            set(PlotControls.parameter3,'UserData',Value);
         elseif SelectedDist==8;
            %
            % Student-t: parameter 3 must be a positive
            %
            if Value>0;
               set(PlotControls.parameter3,'UserData',Value);
            else;
               set(PlotControls.parameter3,'String',num2str(get(PlotControls.parameter3,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the number of degrees of freedom parameter (' num2str(Value,'%0.6f') ') is less than or equal to the zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Student-t Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==10;
            %
            % logistic: paremeter 3 must be positive
            %
            if Value>0;
               set(PlotControls.parameter3,'UserData',Value);
            else;
               set(PlotControls.parameter3,'String',num2str(get(PlotControls.parameter3,'UserData')));
               %
               % show a message box
               %
               txt = ['The value you selected for the shape parameter (' num2str(Value,'%0.6f') ') is less than or equal to zero. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Logistic Distribution',200,500,CurrINI);
            end;
         elseif SelectedDist==12;
            %
            % Pareto: parameter 3 must be a real number only
            %
            set(PlotControls.parameter3,'UserData',Value);
         end;
      elseif gcbo==PlotControls.parameter4;;
         %
         % parameter 4 is only used by the beta distribution
         %
         BetaMean = get(PlotControls.parameter1,'UserData');
         if BetaMean<Value;
            BetaVariance = get(PlotControls.parameter2,'UserData')^2;
            LowerBound = get(PlotControls.parameter3,'UserData');
            if (BetaMean-LowerBound)*(Value-BetaMean)>BetaVariance;
               set(PlotControls.parameter4,'UserData',Value);
            else;
               set(PlotControls.parameter4,'String',num2str(get(PlotControls.parameter4,'UserData')));
               %
               % show a message box
               %
               txt = ['The beta distribution is based on shape parameters a and b that are positive. This implies that the mean is greater than the lower bound (lb) and less than the upper bound (ub). In addition, the product (mean-lb)(ub-mean) must be greater than the variance. The second condition is violated by your choice of parameters. The entry was therefore rejected.'];
               About(txt,'information','Graph-A-Prior - Beta Distribution',200,500,CurrINI);
            end;
         else;
            set(PlotControls.parameter4,'String',num2str(get(PlotControls.parameter4,'UserData')));
            %
            % show a message box
            %
            txt = ['The value you selected for the upper bound (' num2str(Value,'%0.6f') ') is less than or equal to the mean (' num2str(BetaMean,'%0.6f') '). The entry was therefore rejected.'];
            About(txt,'information','Graph-A-Prior - Beta Distribution',200,500,CurrINI);
         end;
      end;
   case 'done'
      set(PlotControls.done,'UserData','done');
      delete(PlotAPriorDLG);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(PlotAPriorDLG,'Visible','on');
   drawnow;
   SetYADAIcon(PlotAPriorDLG,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      %
      % set focus on OK button
      %
      uicontrol(PlotControls.done);
   end;
   waitfor(PlotControls.done,'UserData','done');
end;

%
% end of PlotAPrior.m
%
 