function LogMargs = PlotMargLikeChibJeliazkov(PostSample,LogPost,NumBurnin,CurrChain,DSGEModel,CurrINI)
% PlotMargLikeChibJeliazkov: Calculates sequentially the Chib and Jeliazkov marginal likelihood
%                            identity based estimator of the log marginal likelihood.
%
% USAGE:
%
%       LogMargs = PlotMargLikeChibJeliazkov(PostSample,LogPost,NumBurnin,CurrChain,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  PostSample (matrix) of dimension NumIterxNumParam with the posterior sample draws
%                             of the phi parameters.
%
%                  LogPost (vector) of dimension (NumIterx1) with all the log posterior values for the
%                             non-discarded posterior draws. These posterior values are calculated
%                             using the "logPosteriorPhiDSGE" function.
%
%                  NumBurnin (integer), with the number of burn-in draws.
%
%                  CurrChain (integer) with the current MCMC chain number.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% OPTIONAL OUTPUT: LogMargs (matrix) with the sequential estimates of the log marginal likelihood
%                             in the second column and the numerical standard errors in the
%                             third column. The first column holds the number of parameter draws
%                             that were used for the estimate of the marginal likelihood.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 13, 2006.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-03-2007: Updated the documentation.
%
% * 10-04-2007: Took the name change RWMH to RWM into account.
%
% * 12-04-2007: Made sure that the MargLikeChibJeliazkov function received a unit value for SequentialML.
%
% * 17-04-2007: Removed a SetLogText command.
%
% * 09-11-2007: Updated the documentation.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 11-03-2008: Added code for writing output to file.
%
% * 12-03-2008: Random number generators are now initialized properly.
%
% * 23-05-2008: Updated the documentation.
%
% * 28-05-2008: Changed name text on figure window.
%
% * 03-07-2008: Made sure that the measurement equation receives only the active state variable names.
%
% * 19-12-2008: Updated the documentation.
%
% * 09-02-2009: Changed the input "length(DSGEModel.StateVariablePositions)" to "size(F,1)" when calling the
%               measurement equation function. This required that the function called additional functions
%               so that the dimension of F could be retrieved.
%
% * 13-02-2009: The mcode values 7 and 8 are now properly mentioned when encountered.
%
% * 09-03-2009: Added quotation marks around paths for the dos copy commands.
%
% * 07-04-2009: Changed the maximum height of the "Error - " message box.
%
% * 25-06-2009: Added the optional output variables "LogMargs".
%
% * 06-01-2010: Updated the documentation.
%
% * 02-03-2010: Made sure that the inverse Hessian at the mode is the same as the one used
%               by the posterior sampler.
%
% * 24-03-2010: Removed usage of sample information from the posterior mode file. Current YADA settings are
%               used instead.
%
% * 16-12-2010: Took into account that "logPosteriorPhiDSGE" and "MargLikeChibJeliazkov" take the input
%               variable "OrderQZ" from the CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 13-01-2011: Added additional support for unix and mac paths.
%
% * 20-06-2011: Corrected the spelling error "covergent".
%
% * 24-08-2011: Made sure "F" and "B0" are included in the "ModelParameters" structure via the "YADA" field.
%
% * 04-01-2012: Updated the documentation.
%
% * 27-02-2012: Updated the documentation.
%
% * 28-02-2012: Made sure that the cancel button is not displayed on the wait dialog when the function provides
%               output rather than plots the results.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" inpur vector.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 06-04-2017: The filename of the saved graph now takes the posterior sampler into account.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 12-06-2017: The function now supports the system prior.
%
% * 29-08-2017: Took into account that "logPosteriorPhiDSGE" and "MargLikeChibJeliazkov" require the input
%               variable "NumWorkers".
%
% * 08-01-2018: Updated the documentation.
%
% * 22-03-2018: Adapted the function to sequential estimation.
%
% * 01-08-2018: The function supports user written model solver function.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 21-01-2019: Updated the documentation.
%
% * 10-12-2019: Took into account that the DSGEModel.ModelSolverFile now requires the three additional
%               input variables "DSGEModel.StateVariableNames", "DSGEModel.StateEquationNames" and 
%               "DSGEModel.StateShockNames".
%
% * 07-01-2020: Updated the documentation.
%
% * 29-05-2020: Moved the code for estimating the Hessian.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed additional eval calls to feval.
%
% * 31-03-2022: Made sure that "DSGEModel.InitializeParameterFile" exists before trying to run it.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
% * 09-01-2024: Took into account that "YADAHessianMatrix" now takes the input variable "NumWorkers".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% check if we should initialize the output.
%
if nargout==1;
   LogMargs = [];
end;
%
% locate the controls structure
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
%
% Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% Scale factor for the random walk Metropolis algorithm, i.e., c in "YADA Manual - Computational Details".
%
ScaleStr = get(controls.posterior.scalefactor,'String');
c = str2num(StringTrim(ScaleStr(DSGEModel.MHScaleFactor,:)));
%
% sequential marginal likelihood start value and step length
%
SeqStartStr = get(controls.dsge.sequentialstart,'String');
DSGEModel.SequentialStartIterationValue = str2num(strrep(StringTrim(SeqStartStr(DSGEModel.SequentialStartIteration,:)),',',''));
SeqStepStr = get(controls.dsge.sequentialstep,'String');
DSGEModel.SequentialStepLengthValue = str2num(strrep(StringTrim(SeqStepStr(DSGEModel.SequentialStepLength,:)),',',''));
%
% Try to load the posterior mode estimates
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   end;
   return;
end;
%
phiMode = ModeData.phiMode;
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load additional information about the model.
%
thetaIndex = ModeData.thetaIndex;
UniformBounds = ModeData.UniformBounds;
LowerBound = ModeData.LowerBound;
thetaPositions = ModeData.thetaPositions;
thetaDist = ModeData.thetaDist;
PriorDist = ModeData.PriorDist;
ModelParameters = ModeData.ModelParameters;
ParameterNames = ModeData.ParameterNames;
logPostMode = ModeData.LogPostDensity;
%
% create sample dates
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% state vector initialization data
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
ErrorFile = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
end;
%
%
% Test if there are any problems
%
firstUpdatePos = 0;
lastUpdatePos = 0;
firstInitializePos = 0;
lastInitializePos = 0;
if isempty(ErrorStr)==1;
   if DSGEModel.UseOwnInitialState==1;
      if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
         DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
      else;
         DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
      end;
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
   %
   if (DSGEModel.RunInitializeFirst==1)&(FileExist(DSGEModel.InitializeParameterFile)==1);
      %
      % check if the file with additional parameters that only needs to be updated is OK
      % we need to run the file DSGEModel.InitializeParameterFile first
      %
      if MatlabNumber>=8.0;
         try;
            firstInitializePos = length(fieldnames(ModelParameters))+1;
            ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
            lastInitializePos = length(fieldnames(ModelParameters));
         catch ME;
            ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError(ME)];
            ErrorFile = DSGEModel.InitializeParameterFile;
         end;
      else;
         try;
            firstInitializePos = length(fieldnames(ModelParameters))+1;
            ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
            lastInitializePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError];
            ErrorFile = DSGEModel.InitializeParameterFile;
         end;
      end;
      if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ErrorStr)==1);
         if MatlabNumber>=8.0;
            try;
               firstUpdatePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
               lastUpdatePos = length(fieldnames(ModelParameters));
            catch ME;
               ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError(ME)];
               ErrorFile = DSGEModel.UpdateParameterFile;
            end;
         else;
            try;
               firstUpdatePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
               lastUpdatePos = length(fieldnames(ModelParameters));
            catch;
               ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError];
               ErrorFile = DSGEModel.UpdateParameterFile;
            end;
         end;
      end;
   else;
      if FileExist(DSGEModel.UpdateParameterFile)==1;
         if MatlabNumber>=8.0;
            try;
               firstUpdatePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
               lastUpdatePos = length(fieldnames(ModelParameters));
            catch ME;
               ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError(ME)];
               ErrorFile = DSGEModel.UpdateParameterFile;
            end;
         else;
            try;
               firstUpdatePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
               lastUpdatePos = length(fieldnames(ModelParameters));
            catch;
               ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError];
               ErrorFile = DSGEModel.UpdateParameterFile;
            end;
         end;
      end;
      if (FileExist(DSGEModel.InitializeParameterFile)==1)&(isempty(ErrorStr)==1);
         if MatlabNumber>=8.0;
            try;
               firstInitializePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
               lastInitializePos = length(fieldnames(ModelParameters));
            catch ME;
               ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError(ME)];
               ErrorFile = DSGEModel.InitializeParameterFile;
            end;
         else;
            try;
               firstInitializePos = length(fieldnames(ModelParameters))+1;
               ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
               lastInitializePos = length(fieldnames(ModelParameters));
            catch;
               ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError];
               ErrorFile = DSGEModel.InitializeParameterFile;
            end;
         end;
      end;
   end;
end;
%
% check if the model can be solved
%
if isempty(ErrorStr)==1;
   if MatlabNumber>=8.0;
      try;
         if DSGEModel.ModelSolver==1;
            SolverStr = 'AiM';
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            SolverStr = 'Klein';
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            SolverStr = 'Gensys';
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch ME;
         ErrorStr = ['Error found in "compute_aim_matrices.m". Message caught is: ' YADALastError(ME)];
         ErrorFile = FixFilePath([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m']);
      end;
   else;
      try;
         if DSGEModel.ModelSolver==1;
            SolverStr = 'AiM';
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            SolverStr = 'Klein';
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            SolverStr = 'Gensys';
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch;
         ErrorStr = ['Error found in "compute_aim_matrices.m". Message caught is: ' YADALastError];
         ErrorFile = FixFilePath([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m']);
      end;
   end;
end;
%
if (isempty(ErrorStr)==1)&(mcode==1)&(DSGEModel.ModelSolver==1);
   if MatlabNumber>=8.0;
      try;
         [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
      catch ME;
         ErrorStr = ['Error found when using the output from "compute_aim_matrices.m". Message caught is: ' YADALastError(ME)];
         ErrorFile = FixFilePath([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m']);
      end;
   else;
      try;
         [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
      catch;
         ErrorStr = ['Error found when using the output from "compute_aim_matrices.m". Message caught is: ' YADALastError];
         ErrorFile = FixFilePath([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m']);
      end;
   end;
elseif (isempty(ErrorStr)==1)&(mcode~=1);
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   ErrorStr = ['The DSGE model doesn''t have a unique convergent solution. ' SolverStr ' solver returns the status: ' mcodeStr];
   ErrorFile = FixFilePath([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m']);
end;
%
% check the measurement equation file for internal validity
%
if isempty(ErrorStr)==1;
   %
   % add the values of F and B0 to the ModelParameters.YADA field
   %
   [B0Alt,KeepVar] = RemoveRedundantColumns(B0);
   ModelParameters.YADA.F = F;
   ModelParameters.YADA.B0 = B0Alt;
   if MatlabNumber>=8.0;
      try;
         [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
      catch ME;
         ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
         ErrorFile = DSGEModel.MeasurementEquationFile;
      end;
   else;
      try;
         [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
      catch;
         ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
         ErrorFile = DSGEModel.MeasurementEquationFile;
      end;
   end;
end;
%
% Check if we should exit
%
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
% make sure that we use the correct SigmaMode matrix, i.e., the same matrix as for posterior
% sampling
%
if DSGEModel.InverseHessianEstimator==1;
   SigmaMode = ModeData.InverseHessian;
elseif DSGEModel.InverseHessianEstimator==2;
   if sum(isnan(ModeData.phiFittedStd))==0;
      SigmaStd = sqrt(abs(diag(ModeData.InverseHessian)));
      SigmaCorr = ModeData.InverseHessian ./ (SigmaStd * SigmaStd');
      phiFittedStd = ModeData.phiFittedStd;
      oldphiFittedStd = phiFittedStd;
      if DSGEModel.ModifiedHessian==1;
         FullSet = (1:length(phiFittedStd));
         for i=1:length(phiFittedStd);
            iExcluded = setdiff(FullSet,i);
%            phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*inv(SigmaCorr(iExcluded,iExcluded))*SigmaCorr(iExcluded,i)));
            phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*(SigmaCorr(iExcluded,iExcluded)\SigmaCorr(iExcluded,i))));
         end;
      end;
      SigmaMode = SigmaCorr .* (phiFittedStd * phiFittedStd');
   else;
      if isempty(CurrINI.Sequence)==1;
         txt = 'YADA has not fitted a quadratic to the evaluated log posterior. Please run the "Check posterior mode" function on the View menu first. Alternatively, you can use the estimator of the inverse Hessian from the optimization routine.';
         About(txt,'information','YADA - Modified Inverse Hessian',120,500,CurrINI);
         drawnow;
      end;
      return;
   end;
elseif DSGEModel.InverseHessianEstimator==3;
   if sum(sum(isnan(ModeData.phiInverseHessianFinDiff)))==0;
      SigmaMode = ModeData.phiInverseHessianFinDiff;
   else;
      if isempty(CurrINI.Sequence)==1;
         txt = 'YADA hasn''t estimated the inverse Hessian using finite difference during the posterior mode estimation. Would you like to begin the finite difference estimation now?';
         answer = Query(txt,'question',140,'YADA - Finite Difference Inverse Hessian',500,'no',CurrINI);
         drawnow;
      else;
         answer = CurrINI.Sequence.QueryAnswers{1};
      end;
      if strcmp(lower(answer),'yes')==1;
         SigmaMode = [];
      else;
         return;
      end;
   end;
elseif DSGEModel.InverseHessianEstimator==4;
   %
   % try to load the file
   %
   CovMatData = load(DSGEModel.ParameterCovMatrix);
   SigmaMode = CovMatData.ParameterCovarianceMatrix;
   if size(SigmaMode,1)~=NumParam;
      if isempty(CurrINI.Sequence)==1;
         txt = ['The dimension of the parameter covariance matrix you have selected from the file "' GetName(DSGEModel.ParameterCovMatrix) '" is ' int2str(size(SigmaMode,1)) ', while the number of parameters to estimate is ' int2str(NumParam) '. YADA is therefore unable to compute the log marginal likelihood.'];
         About(txt,'information','YADA - Own Parameter Covariance Matrix',200,500,CurrINI);
         drawnow;
      end;
      return;
   end;
end;
if isempty(SigmaMode)==1;
   tic;
   VechHessian = YADAHessianMatrix('logPosteriorPhiDSGE',phiMode,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
   FinDiffTime = toc;
   if sum(isnan(VechHessian))==0;
      HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
      phiInverseHessianFinDiff = HessianMat\eye(NumParam);
      %
      % save to post mode file
      %
      save(ModeFile,'phiInverseHessianFinDiff','-APPEND');
      SigmaMode = phiInverseHessianFinDiff;
   else;
      %
      % bad Hessian encountered
      %
      if isempty(CurrINI.Sequence)==1;
         txt = ['The inverse Hessian estimator using finite differences with a step length of ' num2str(StepLength) ' has at least one NaN entry. Please change the step length or select a different estimator of the inverse Hessian.'];
         About(txt,'information','YADA - Inverse Hessian Estimation Problem',150,500,CurrINI);
         drawnow;
      end;
      return;
   end;
end;
%
% check if we should manipulate the correlations for the proposal density
%
if DSGEModel.MaxCorrelationValue>1;
   MaxCorrStr = get(controls.posterior.maxcorrelation,'String');
   MaxCorr = str2num(StringTrim(MaxCorrStr(DSGEModel.MaxCorrelationValue,:)));
   %
   % create the correlation matrix
   %
   StdVec = sqrt(diag(SigmaMode));
   CorrMat = SigmaMode ./ (StdVec*StdVec');
   for i=1:length(StdVec);
      CorrMat(i,i) = 0;
   end;
   ScaleFactor = max(max(abs(CorrMat)));
   ScaleMatrix = min(1,MaxCorr/ScaleFactor)*ones(length(StdVec),length(StdVec));
   for i=1:length(StdVec);
      ScaleMatrix(i,i) = 1;
   end;
   SigmaMode = SigmaMode .* ScaleMatrix;
else;
   MaxCorr = 1;
end;
%
% test if SigmaMode is positive definite
%
if IsPositiveDefinite(SigmaMode)==0;
   EigValues = eig(SigmaMode);
   EigValues = EigValues(EigValues<=0);
   if isempty(CurrINI.Sequence)==1;
      txt = ['The inverse Hessian is not positive definite. YADA has found that ' int2str(length(EigValues)) ' eigenvalues of the estimated inverse Hessian are less than or equal to zero. This means that the inverse Hessian estimator can''t be used for when computing the log marginal likelihood. Do you want YADA to try to create a positive definite matrix using your matrix as input?'];
      answer = Query(txt,'question',300,'YADA - Inverse Hessian Estimation Problem',500,'no',CurrINI);
   else;
      answer = CurrINI.Sequence.QueryAnswers{1};
   end;
   if strcmp(lower(answer),'yes')==1;
      [NewSigmaMode,pdstatus] = CreatePositiveDefinite(SigmaMode);
      if pdstatus==1;
         SigmaMode = NewSigmaMode;
      else;
         if isempty(CurrINI.Sequence)==1;
            txt = 'YADA wasn''t able to "make" the inverse Hessian positive definite. The calculation of the log marginal likelihood has therefore been aborted.';
            About(txt,'information','YADA - Inverse Hessian Estimation Problem',200,500,CurrINI);
            drawnow;
         end;
         return;
      end;
   else;
      return;
   end;
end;
%
% initialize the random number generators
%
%
if DSGEModel.RandomNumberValue==1;
   rand('state',0);
   randn('state',0);
else;
   rand('state',sum(100*clock));
   randn('state',sum(100*clock));
end;
%
% Open up a wait dialog.
%
WaitTitle = 'Sequential Marginal Likelihood - Chib & Jeliazkov';
ShowCancel = 1;
if (nargout==1)&(DSGEModel.SequentialML==0);
   WaitTitle = 'Marginal Likelihood - Chib & Jeliazkov';
   ShowCancel = 0;
end;
txt = ['Please wait while YADA computes Chib and Jeliazkov''s marginal likelihood identity based estimator of the log marginal likelihood. Estimation started at: ' StringTrim(datestr(now,14)) '. This will take some time!'];
WaitHandle = WaitDLG(txt,'information',WaitTitle,500,150,CurrINI,ShowCancel);
WaitControls = get(WaitHandle,'UserData');
if isfield(WaitControls,'cancel')==0;
   WaitControls.cancel = [];
end;
%
% run the Chib and Jeliazkov estimator
%
if nargout==0;
   LogMargsCJ = MargLikeChibJeliazkov(PostSample,LogPost,phiMode,SigmaMode,c,logPostMode,NumBurnin,1,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers,WaitControls.cancel);
else;
   LogMargsCJ = MargLikeChibJeliazkov(PostSample,LogPost,phiMode,SigmaMode,c,logPostMode,NumBurnin,DSGEModel.SequentialML,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers,WaitControls.cancel);
end;
%
% shut down the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
if isempty(LogMargsCJ)==0;
   %
   % ask if we write to file
   %
   if isempty(CurrINI.Sequence)==1;
      txt = 'Would you like to save the sequential estimation data to file?';
      answer = Query(txt,'question',200,'Save - Chib & Jeliazkov',500,'yes',CurrINI);
   else;
      answer = CurrINI.Sequence.QueryAnswers{2};
   end;
   if strcmp(lower(answer),'yes')==1;
      Xvalues = LogMargsCJ(:,1)+NumBurnin;
      LogMargLikeCJ = LogMargsCJ(:,2:3);
      %
      file = FixFilePath([DSGEModel.OutputDirectory '\SeqChibJeliazkov-' DSGEModel.NameOfModel '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.mat']);
      save(file,'Xvalues','LogMargLikeCJ');
      %
      mfile = FixFilePath([DSGEModel.OutputDirectory '\SeqCJ' strrep(DSGEModel.NameOfModel,'-','') int2str(NumPostDraws) int2str(CurrChain) '.m']);
      fid = fopen(mfile,'wt');
      fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
      fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
      fprintf(fid,'%%\n%% Number of posterior draws used for sequential estimation (N x 1) vector\n%%\n');
      fprintf(fid,'Xvalues = DataStructure.Xvalues;\n');
      fprintf(fid,'%%\n%% N x 2 matrix with log marginal likelihood values and numerical standard\n%% error using the Chib and Jeliazkov estimator\n%%\n');
      fprintf(fid,'LogMargLikeCJ = DataStructure.LogMargLikeCJ;\n');
      fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
      %
      % end the script file
      %
      fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
      fclose(fid);
      %
      if isempty(CurrINI.Sequence)==1;
         txt = ['The data on sequential estimation of the log marginal likelihood has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: Xvalues (Nx1 vector with the number of posteror draws used for the sequential estimation), and LogMargLikeCJ (Nx2 matrix with estimates of the log marginal likelihood and the numerical standard errors with the Chib and Jeliazkov approach). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
         About(txt,'information',WaitTitle,200,500,CurrINI);
      end;
   end;
   %
   if nargout==1;
      LogMargs = LogMargsCJ;
      return;
   end;
   %
   GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
   %
   FigHandle =figure('Visible','off', ...
          'Position',[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500], ...
          'Tag','MargLikeCJ', ...
          'Name','Sequential Marginal Likelihood - Chib and Jeliazkov');
   %
   SetFigureProperties(FigHandle,CurrINI);
   %
   Xvalues = LogMargsCJ(:,1)+NumBurnin;
   Yvalues = LogMargsCJ(:,2);
   %
   plot(Xvalues,Yvalues,'LineStyle','-','Color',[0.75 0 0],'LineWidth',2,'Marker','.');
   hold('off');
   SetAxesFonts(CurrINI,'Title','Chib and Jeliazkov Estimates','XLabel',['RWM draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Log marginal likelihood');
   axis('tight');
   %
   % check the YLim
   %
   ylim = get(gca,'YLim');
   add_on = 0.1*(ylim(2)-ylim(1));
   ylim = [ylim(1)-add_on ylim(2)+add_on];
   set(gca,'YLim',ylim);
   %
   % save the grasph
   %
   if DirExist(GraphDirectory);
      if DSGEModel.PosteriorSampler==1;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ChibJeliazkov-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==3;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ChibJeliazkov-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==4;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ChibJeliazkov-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==5;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ChibJeliazkov-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==6;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ChibJeliazkov-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==7;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-ChibJeliazkov-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      end;
      saveas(FigHandle,file);
   end;
   %
   % display the graph
   %
   SetToolbarImages(FigHandle,CurrINI.images);
   set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;

%
% end of PlotMargLikeChibJeliazkov.m
%
