function PointForecastObsWeightDecompDLG(selector,PredMean,Yss,YTh,PredDateVector,DataStr,EstStr,DSGEModel,CurrINI)
% PointForecastObsWeightDecompDLG: Displays graphs of the observation weight decompositions
%                                  of the point forecasts over the forecast horizon.
%
% USAGE:
%
%       PointForecastObsWeightDecompDLG(selector,PredMean,Yss,YTh,PredDateVector,DataStr,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "samplebegin", "sampleend", "showgraph",
%                          and "done".
%
% OPTIONAL INPUT: PredMean (matrix) of dimension nxh with the point forecasts of the observed
%                          variables.
%
%                 Yss (matrix) of dimension nxh with the steady-state values of the observed
%                          variables over the forecast sample.
%
%                 YTh (matrix) of dimension nx(n+1)xh with the decomposition of the point
%                          forecasts of the observed variables into their historical data
%                          and initial value of the state variables.
%
%                 PredDateVector (vector) of dimension h with the dates for the predicted
%                          variables.
%
%                 DataStr (string vector) that reflects if the original data is predicted
%                          or the annual data.
%
%                 EstStr (string vector) that takes on the values "Initial Values" and "Posterior Mode".
%
%                 DSGEModel (structure) whose fields contain all required information about the model, such as
%                          input files, variable and sample data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: July 27, 2023.
%                        This version: July 28, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-07-2023: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.DataStr = DataStr;
   DSGEModel.PredMean = PredMean;
   DSGEModel.Yss = Yss;
   DSGEModel.YTh = YTh;
   DSGEModel.PredDateVector = PredDateVector;
   DSGEModel.EstStr = EstStr;
else;
   GraphGUI = findobj('Type','figure','Tag','PointForecastObsWeightDecompDLG');
   controls = get(GraphGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
% create the dialog
%
switch selector;
   case 'init';
      GraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','PointForecastObsWeightDecompDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-520)/2 (CurrINI.scrsz(4)-140)/2 520 140], ...
         'Tag','PointForecastObsWeightDecompDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Point Forecast Observation Weight Decompositions', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','PointForecastObsWeightDecompDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(GraphGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 518 130],'Graphics Selections',45,[0.5 1],'on',CurrINI);
      %
      % Select variable
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 85 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVarText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 90 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVarPopup',...
         'Value',1);
      %
      controls.blackedgebars = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 40 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Draw bars with black edges', ...
            'Value',1, ...
            'Enable','on', ...
            'Tag','BlackEdgeBarsCheckBox');
      %
      controls.useobsvargroups = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Use observed variable groups', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','UseObsVarGorupsCheckBox');
      %
      if max(DSGEModel.ObsVarGroups)<DSGEModel.n;
         set(controls.useobsvargroups,'Value',1);
      end;
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[415 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','PointForecastObsWeightDecompDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[415 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','PointForecastObsWeightDecompDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(GraphGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'showgraph'
      VariableValue = get(controls.variables,'Value');
      CurrVariable = StringTrim(DSGEModel.VariableNames(VariableValue,:));
      if VariableValue<size(DSGEModel.VariableNames,1);
         set(controls.variables,'Value',VariableValue+1);
      else;
         set(controls.variables,'Value',1);
      end;
      %
      % check if the bars should have black edges or not
      %
      BlackEdgeBars = get(controls.blackedgebars,'Value');
      %
      % check if we should use observed variable groups or not
      %
      UseObsVarGroups = get(controls.useobsvargroups,'Value');
      %
      % collect data
      %
      YTh = DSGEModel.YTh;
      PredMean = DSGEModel.PredMean;
      Yss = DSGEModel.Yss;
      PredDateVector = DSGEModel.PredDateVector;
      h = length(PredDateVector);
      n = DSGEModel.n;
      if UseObsVarGroups==1;
         PointForecastDecomp = zeros(max(DSGEModel.ObsVarGroups)+2,h);
         if (isempty(DSGEModel.ObsVarGroupColors)==0)&(size(DSGEModel.ObsVarGroupColors,1)==size(DSGEModel.ObsVarGroupNames,1));
            GroupColors = DSGEModel.ObsVarGroupColors;
         else;
            GroupColors = CreateDefaultColors(n);
         end;
      else;
         PointForecastDecomp =zeros(n+2,h);
         GroupColors = CreateDefaultColors(n);
      end;
      %
      % make sure we have colors for the steady-state predictions and
      % the initial values of the state variables
      %
      GroupColors = [1 1 1;GroupColors;0.3 0.3 0.3];
      %
      % initialize
      %
      if UseObsVarGroups==1;
         Groups = DSGEModel.ObsVarGroups;
         GroupNames = '';
         for i=1:size(DSGEModel.ObsVarGroupNames,1);
            GroupNames = strvcat(GroupNames,StringTrim(strrep(DSGEModel.ObsVarGroupNames(i,:),'_',' ')));
         end;
      else;
         Groups = (1:n);
         GroupNames = DSGEModel.VariableNames;
      end;
      %
      GroupNames = strvcat('Steady-state',GroupNames,'Initial State');
      %
      % collect the steady-state and the initial values effects
      % on the decomposition
      %
      PointForecastDecomp(1,:) = Yss(VariableValue,:);
      PointForecastDecomp(max(DSGEModel.ObsVarGroups)+2,:) = squeeze(YTh(VariableValue,n+1,:))';
      %
      % collect the contributions of the individual groups/variables
      %
      for i=1:n;
         PointForecastDecomp(Groups(i)+1,:) = PointForecastDecomp(Groups(i)+1,:)+squeeze(YTh(VariableValue,i,:))';
      end;
      y = PredMean(VariableValue,:);
      %
      % plot the predicted variable
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                         'Tag',CurrVariable, ...
                         'Name',['Observation Weight Decomposition - Forecasted ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % Create the bars and collect the handles
      %
      if MatlabNumber>8.3;
         BarHandles = bar(PredDateVector',PointForecastDecomp','stacked');
      else;
         if MatlabNumber>=7.0;
            %
            % make sure the bars are written as patches
            %
            BarHandles = bar('v6',PredDateVector',PointForecastDecomp','stacked');
         else;
            BarHandles = bar(PredDateVector',PointForecastDecomp','stacked');
         end;
      end;
      %
      % remake the bar patches such that all bars are displayed properly.
      %
      BarHandles = RemakeBars(BarHandles,GroupColors,BlackEdgeBars);
      %
      % plot the data
      %
      LineHandle = plot(PredDateVector,y,'.-','Color',[0 0 0],'LineWidth',2);
      %
      SetAxesFonts(CurrINI,'Title',['Point forecast of ' lower(DSGEModel.DataStr) ' ' CurrVariable ' at ' lower(DSGEModel.EstStr)]);
      %
      axis('tight');
      ylim = get(gca,'YLim');
      AddOn = 0.1*(ylim(2)-ylim(1));
      ylim = [ylim(1)-AddOn ylim(2)+AddOn];
      set(gca,'YLim',ylim);
      if (ylim(1)<0)&(ylim(2)>0);
         hold('on');
         plot([PredDateVector(1) PredDateVector(h)],[0 0],'-','Color',[0 0 0]);
         %
         % make sure the zero line is displayed below the observed variable
         %
         kids = get(gca,'Children');
         if size(kids,1)>size(kids,2);
            kids = kids';
         end;
         kids = [kids(2:length(kids)) kids(1)]';
         set(gca,'Children',kids);
      end;
      %
      if MatlabNumber>=7;
         legHandle = legend([BarHandles LineHandle],strvcat(GroupNames,CurrVariable),'Location','SouthEast');
      else;
         legHandle = legend([BarHandles LineHandle],strvcat(GroupNames,CurrVariable),4);
      end;
      SetAxesFonts(CurrINI,legHandle);
      hold('off');
      %
      % check if we should fix the X label
      %
      SetXTickIntegers(gca);
      %
      % display the graphs
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(controls.done,'UserData','done');
      delete(GraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(GraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(GraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of PointForecastObsWeightDecompDLG.m
%
