function RawPosteriorPlots(ParamSample,NumBurnin,TotalDraws,CurrChain,TypeStr,IsAdaptiveLearning,DSGEModel,CurrINI)
% RawPosteriorPlots: Provides plots of the posterior draws over all the draws
%
% USAGE:
%
%       RawPosteriorPlots(ParamSample,NumBurnin,TotalDraws,CurrChain,TypeStr,IsAdaptiveLearning,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ParamSample (matrix) of dimension NumIterxNumParam with all the posterior
%                             draws of the original parameters.
%
%                 NumBurnin (integer) with the number of burn-in draws.
%
%                 TotalDraws (integer) with the total number of available posterior draws
%                             that could have been plotted.
%
%                 CurrChain (integer) with the number for the used sampling chain.
%
%                 TypeStr (string vector) that takes the values "Original" or "Transformed".
%
%                 IsAdaptiveLearning (boolean) which takes the value 1 if the adaptive learning mechanism is used for the
%                             expectations formation, and 0 otherwise.
%
%                 DSGEModel (structure)  whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 13, 2006.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-12-2006: Added the Chain number to figure name string.
%
% * 18-12-2006: Added an additional drawnow command for refreshing all figures.
%
% * 09-01-2007: The plots are now automatically stored in the graphs directory.
%
% * 08-02-2007: Added a second "drawnow" call at the end of the figure loop.
%
% * 28-02-2007: Added the "TypeStr" string input. Thereby we can plots posterior draws of either
%               the original parameters or of the transformed parameters.
%
% * 09-11-2007: Updated the documentation.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 10-03-2008: Made sure that the "CurrChain" variable affects the filename for the figure.
%
% * 23-05-2008: Updated the documentation.
%
% * 19-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 06-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 04-01-2012: Updated the documentation.
%
% * 07-05-2012: Added the input variable "TotalDraws" to deal with having a subsample of the posterior
%               draws to plot.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
% * 10-08-2017: Made the function work with raw prior draws.
%
% * 08-01-2018: Updated the documentation.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 21-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 28-05-2020: Added the input variable "IsAdaptiveLearning".
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[NumIter,NumParam] = size(ParamSample);
%
% check if we have prior or posterior draws
%
if isempty(CurrChain)==0;
   IsPosterior = 1;
else;
   IsPosterior = 0;
end;
%
if IsAdaptiveLearning==0;
   if IsPosterior==1;
      %
      % Try to load the posterior mode estimates
      %
      ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
   else;
      ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
   end;
else;
   if IsPosterior==1;
      %
      % Try to load the posterior mode estimates
      %
      ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
   else;
      ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
   end;
end;
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
if strcmp(lower(TypeStr),'original')==1;
   thetaMode = ModeData.thetaMode;
else;
   if IsPosterior==1;
      thetaMode = ModeData.phiMode;
   else;
      thetaMode = ModeData.phi;
   end;
end;
ParameterNames = ModeData.ParameterNames;
%
% Directory where the plots are stored
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
% Compute the number of figures (max 16 plots per graph)
%
NumFigs = ceil(length(thetaMode)/16);
%
if NumFigs==1;
   [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumParam);
else;
   NumRows = 4;
   NumCols = 4;
end;
NumPlots = 0;
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
Xvalues = (1:NumIter)';
%
% now we can start the loops
%
for i=1:NumFigs;
   if i>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>CurrINI.scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = CurrINI.scrsz(4)-700;
      end;
   end;
   FigHandles{i} = figure('Visible','off','Position',pos);
   if IsPosterior==1;
      if NumFigs==1;
         set(FigHandles{i},'Name',['Raw Posterior Draws of ' TypeStr ' Parameters - Chain ' num2str(CurrChain,'%0.0f')]);
      else;
         set(FigHandles{i},'Name',['Raw Posterior Draws of ' TypeStr ' Parameters - Figure ' num2str(i,'%0.0f') ' Chain ' num2str(CurrChain,'%0.0f')]);
      end;
   else;
      if NumFigs==1;
         set(FigHandles{i},'Name',['Raw Prior Draws of ' TypeStr ' Parameters']);
      else;
         set(FigHandles{i},'Name',['Raw Prior Draws of ' TypeStr ' Parameters - Figure ' int2str(i)]);
      end;
   end;
   SetFigureProperties(FigHandles{i},CurrINI);
   %
   k = 0;
   for r=1:NumRows;
      for c=1:NumCols;
         NumPlots = NumPlots+1;
         if NumPlots<=NumParam;
            k = k+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
            subplot(NumRows,NumCols,k);
            %
            % plot the posterior draws
            %
            dh = plot(Xvalues,ParamSample(:,NumPlots),'-','Color',[0.75 0 0],'LineWidth',1);
            hold('on');
            SetAxesFonts(CurrINI,'Title',StringTrim(ParameterNames.estimated(NumPlots,:)));
            axis('tight');
            ylim = get(gca,'YLim');
            add_on = 0.1*(ylim(2)-ylim(1));
            ylim = [ylim(1)-add_on ylim(2)+add_on];
            %
            set(gca,'YLim',ylim,'XLim',[0 NumIter]);
            %
            % plot the BurnIn value
            %
            if TotalDraws==NumIter;
               bi = plot(NumBurnin*ones(1,2),ylim,'-','Color',[0 0 0]);
               hold('on');
            end;
            %
            % plot the posterior mode estimate
            %
            pme = plot([0 NumIter],thetaMode(NumPlots)*ones(1,2),'-','Color',[0 0 0]);
            hold('off');
            drawnow;
         end;
      end;
   end;
   %
   % save it to disk before
   %
   if DirExist(GraphDirectory)==1;
      if IsAdaptiveLearning==0;
         if IsPosterior==1;
            if NumFigs>1;
               file = FixFilePath([GraphDirectory '\RawPosteriorDraws-' TypeStr '-' int2str(NumIter) '.' int2str(TotalDraws) '-' int2str(CurrChain) '-' int2str(i) '.fig']);
            else;
               file = FixFilePath([GraphDirectory '\RawPosteriorDraws-' TypeStr '-' int2str(NumIter) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.fig']);
            end;
         else;
            if NumFigs>1;
               file = FixFilePath([GraphDirectory '\RawPriorDraws-' TypeStr '-' int2str(NumIter) '-' int2str(i) '.fig']);
            else;
               file = FixFilePath([GraphDirectory '\RawPriorDraws-' TypeStr '-' int2str(NumIter) '.fig']);
            end;
         end;
      else;
         if IsPosterior==1;
            if NumFigs>1;
               file = FixFilePath([GraphDirectory '\RawPosteriorDraws-AdaptiveLearning-' TypeStr '-' int2str(NumIter) '.' int2str(TotalDraws) '-' int2str(CurrChain) '-' int2str(i) '.fig']);
            else;
               file = FixFilePath([GraphDirectory '\RawPosteriorDraws-AdaptiveLearning-' TypeStr '-' int2str(NumIter) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.fig']);
            end;
         else;
            if NumFigs>1;
               file = FixFilePath([GraphDirectory '\RawPriorDraws-AdaptiveLearning-' TypeStr '-' int2str(NumIter) '-' int2str(i) '.fig']);
            else;
               file = FixFilePath([GraphDirectory '\RawPriorDraws-AdaptiveLearning-' TypeStr '-' int2str(NumIter) '.fig']);
            end;
         end;
      end;
      saveas(FigHandles{i},file);
   end;
   %
   % Fix the toolbar cdata
   %
   SetToolbarImages(FigHandles{i},CurrINI.images);
   %
   % Display the current figure
   %
   set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;
%
clear('ParamSample');

%
% end of RawPosteriorPlots.m
%
