function ScatterPlotMonteCarloFiltering(selector,SolutionCode,thetaDraws,ParameterNames,DSGEModel,CurrINI)
% ScatterPlotMonteCarloFiltering:  Displays a dialog for selecting 2 parameters to use for computing scatter
%                                  plots of these parameters based on the model solution property.
%
% USAGE:
%
%       ScatterPlotMonteCarloFiltering(selector,SolutionCode,thetaDraws,ParameterNames,DSGEModel,CurrINI)
%
% REQUIRED INPUT: selector (string vector) that takes on the values  "init", "showgraph",
%                          "checkparams", and "done".
%
% OPTIONAL INPUT: SolutionCode (vector) of dimension NumDraws with integer 1 for unique, convergent solution,
%                              2 for indeterminacy, and 0 for no stable solution.
%
%                 thetaDraws (matrix) of dimension NumParamxNumDraws with values of the parameters of the
%                              DSGE model.
%
%                 ParameterNames (string matrix) with the names of the estimated parameters.
%
%                 DSGEModel (structure) whose fields contain all required information about the model,
%                              such as input files, variable and sample data.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 19, 2011.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20/12/2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 10-09-2015: The function is now ready for Matlab 2014b and later.
%
% * 14-09-2015: Fixed the units for the scatter plot function with Matlab 2014b and later.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 30-10-2023: Made sure that the order of the handles is switched when the number of unique and
%               convergent solutions is lower than the number of unstable and non-determinate.
%
% * 02-11-2023: Corrected a bug when setting up the parameter data to plot.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.SolutionCode = SolutionCode;
   DSGEModel.thetaDraws = thetaDraws;
else;
   ScatterMCFilteringGUI = findobj('Type','figure','Tag','ScatterPlotMonteCarloFiltering');
   ScatterControls = get(ScatterMCFilteringGUI,'UserData');
   CurrINI = get(ScatterControls.parameter2popup,'UserData');
   DSGEModel = get(ScatterControls.parameter2text,'UserData');
end;
%
switch selector;
   case 'init';
      ScatterMCFilteringGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ScatterPlotMonteCarloFiltering.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-110)/2 500 110], ...
         'Tag','ScatterPlotMonteCarloFiltering', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Scatter-Plot Parameters - Monte Carlo Filtering', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','ScatterPlotMonteCarloFiltering done', ...
         'ToolBar','none');

      if MatlabNumber>=7.0;
         set(ScatterMCFilteringGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 498 100],'Model Solution Properties',45,[0.5 1],'on',CurrINI);
      %
      % select parameter 1
      %
      ScatterControls.parameter1text = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 45 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select parameter 1:', ...
         'HorizontalAlignment','left', ...
         'Tag','Parameter1Text');
      %
      ScatterControls.parameter1popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 50 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',ParameterNames, ...
         'Callback','ScatterPlotMonteCarloFiltering checkparams', ...
         'HorizontalAlignment','center', ...
         'Tag','Parameter1Popup',...
         'Value',1);
      %
      % Select parameter 2
      %
      ScatterControls.parameter2text = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 15 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select parameter 2:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVarText');
      %
      ScatterControls.parameter2popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 20 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',ParameterNames, ...
         'Callback','ScatterPlotMonteCarloFiltering checkparams', ...
         'HorizontalAlignment','center', ...
         'Tag','Parameter2Popup',...
         'Value',2);
      %
      % Create graph button
      %
      ScatterControls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ScatterPlotMonteCarloFiltering showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      ScatterControls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ScatterPlotMonteCarloFiltering done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(ScatterMCFilteringGUI,'UserData',ScatterControls,...
         'HandleVisibility','callback');
   case 'checkparams'
      Param1Val = get(ScatterControls.parameter1popup,'Value');
      Param2Val = get(ScatterControls.parameter2popup,'Value');
      if Param1Val==Param2Val;
         %
         % check which controls should be adjusted
         %
         NumParams = size(get(ScatterControls.parameter1popup,'String'),1);
         if gco==ScatterControls.parameter1popup;
            %
            % parameter 1 has changed
            %
            if Param2Val<NumParams;
               set(ScatterControls.parameter2popup,'Value',Param2Val+1);
            else;
               set(ScatterControls.parameter2popup,'Value',1);
            end;
         else;
            %
            % parameter 2 has changed
            %
            if Param2Val<NumParams;
               set(ScatterControls.parameter1popup,'Value',Param1Val+1);
            else;
               set(ScatterControls.parameter1popup,'Value',1);
            end;
         end;
      end;
   case 'showgraph'
      Param1Value = get(ScatterControls.parameter1popup,'Value');
      Param1String = get(ScatterControls.parameter1popup,'String');
      Param1 = StringTrim(Param1String(Param1Value,:));
      %
      Param2Value = get(ScatterControls.parameter2popup,'Value');
      Param2String = get(ScatterControls.parameter2popup,'String');
      Param2 = StringTrim(Param2String(Param2Value,:));
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                         'Tag',['ScatterPlot-MCFilter-' Param1 '-' Param2], ...
                         'Name',['Scatter-Plot of ' Param1 ' and ' Param2]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % get the data
      %
      x = DSGEModel.thetaDraws(Param1Value,:);
      y = DSGEModel.thetaDraws(Param2Value,:);
      %
      if MatlabNumber>8.3;
         %
         % initialize
         %
         Handles = [];
         LegendStr = '';
         %
         % number of elements
         %
         sUnique = DSGEModel.SolutionCode(DSGEModel.SolutionCode==1);
         sIndeterminacy = DSGEModel.SolutionCode(DSGEModel.SolutionCode==2);
         sNoSolution = DSGEModel.SolutionCode(DSGEModel.SolutionCode==0);
         %
         % the parameter data to plot
         %
         xUnique = x(DSGEModel.SolutionCode==1);
         yUnique = y(DSGEModel.SolutionCode==1);
         xIndeterminacy = x(DSGEModel.SolutionCode==2);
         yIndeterminacy = y(DSGEModel.SolutionCode==2);
         xNoSolution = x(DSGEModel.SolutionCode==0);
         yNoSolution = y(DSGEModel.SolutionCode==0);
         %
         % we here make three scatter-plots
         %
         if length(sUnique)>=(length(sIndeterminacy)+length(sNoSolution));
            %
            % we have as least as many stable solutions as problem solutions
            %
            if isempty(sUnique)==0;
               UniqueHandle = scatter(xUnique,yUnique,25,[0 0 0.5],'filled');
               hold('on');
               Handles = [Handles UniqueHandle];
               LegendStr = strvcat(LegendStr,'Unique & stable solution');
            end;
            %
            if isempty(sIndeterminacy)==0;
               InDeterHandle = scatter(xIndeterminacy,yIndeterminacy,25,[0.5 0 0],'filled');
               hold('on');
               Handles = [Handles InDeterHandle];
               LegendStr = strvcat(LegendStr,'Indeterminacy');
            end;
            %
            if isempty(sNoSolution)==0;
               NoSolutionHandle = scatter(xNoSolution,yNoSolution,25,[0 0.5 0],'filled');
               hold('on');
               Handles = [Handles NoSolutionHandle];
               LegendStr = strvcat(LegendStr,'No stable solution');
            end;
         else;
            %
            % there are more problem solutions than stable and covergent ones
            %
            if isempty(sIndeterminacy)==0;
               InDeterHandle = scatter(xIndeterminacy,yIndeterminacy,25,[0.5 0 0],'filled');
               hold('on');
               Handles = [Handles InDeterHandle];
               LegendStr = strvcat(LegendStr,'Indeterminacy');
            end;
            %
            if isempty(sNoSolution)==0;
               NoSolutionHandle = scatter(xNoSolution,yNoSolution,25,[0 0.5 0],'filled');
               hold('on');
               Handles = [Handles NoSolutionHandle];
               LegendStr = strvcat(LegendStr,'No stable solution');
            end;
            %
            if isempty(sUnique)==0;
               UniqueHandle = scatter(xUnique,yUnique,25,[0 0 0.5],'filled');
               hold('on');
               Handles = [Handles UniqueHandle];
               LegendStr = strvcat(LegendStr,'Unique & stable solution');
            end;
         end;
      else;
         if MatlabNumber>=7;
            ScatHandle = scatter('v6',x,y,4,DSGEModel.SolutionCode,'filled');
         else;
            ScatHandle = scatter(x,y,4,DSGEModel.SolutionCode,'filled');
         end;
         hold('on');
         %
         % now grab a handle for each case which is present
         %
         Handles = [];
         LegendStr = '';
         UniqueHandle = ScatHandle(DSGEModel.SolutionCode==1);
         if isempty(UniqueHandle)==0;
            Handles = [Handles UniqueHandle(1)];
            LegendStr = strvcat(LegendStr,'Unique & stable solution');
         end;
         InDeterHandle = ScatHandle(DSGEModel.SolutionCode==2);
         if isempty(InDeterHandle)==0;
            Handles = [Handles InDeterHandle(1)];
            LegendStr = strvcat(LegendStr,'Indeterminacy');
         end;
         NoSolutionHandle = ScatHandle(DSGEModel.SolutionCode==0);
         if isempty(NoSolutionHandle)==0;
            Handles = [Handles NoSolutionHandle(1)];
            LegendStr = strvcat(LegendStr,'No stable solution');
         end;
         %
         if length(UniqueHandle)<(length(InDeterHandle)+length(NoSolutionHandle));
            Kids = get(gca,'Children');
            set(gca,'Children',flipud(fliplr(Kids)));
         end;
      end;
      axis('tight');
      %
      % plot the legend
      %
      LegHandle = legend(Handles,LegendStr);
      SetAxesFonts(CurrINI,LegHandle);
      %
      % Show title and labels
      %
      SetAxesFonts(CurrINI,'Title',[Param1 ' and ' Param2 ' (' num2str(length(x),'%0.0f') ' draws from prior)'],'XLabel',Param1,'Ylabel',Param2);
      if strcmp(get(gca,'YGrid'),'on')==1;
         set(gca,'XGrid','on');
      end;
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(ScatterControls.done,'UserData','done');
      delete(ScatterMCFilteringGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(ScatterMCFilteringGUI,'Visible','on');
   drawnow;
   SetYADAIcon(ScatterMCFilteringGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(ScatterControls.done);
   end;
   waitfor(ScatterControls.done,'UserData','done');
end;

%
% end of ScatterPlotMonteCarloFiltering.m
%
