function SetLogText(TextStrMat,maingui)
% SetLogText: Writes text to the estimation log window.
%
% USAGE:
%
%       SetLogText(TextStrMat,maingui)
%
% REQUIRED INPUT: TextStrMat (string matrix) with text to add to the log window.
%
% OPTIONAL INPUT: maingui (handle) for the figure where the log window is located.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 8, 2006.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-05-2008: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 19-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 06-08-2009: The estimation log window in now 377 pixels in height.
%
% * 07-08-2009: Changed the determination of "ListTopValue" when it is initially 1
%               and the height of the estimation log window is assumed the be less
%               than the height of the text. The text heigt determination was also
%               changed such that each line is assumed to have the height 1.1 times
%               the font size.
%
% * 06-01-2010: Updated the documentation.
%
% * 26-01-2010: Added the textwidth check that adds 1 line to the "ListboxTop" property
%               if the width is at least 75 characters.
%
% * 27-05-2010: Changed the condition for "LogTextWidth" from 75 to 74 when "AddLine" is
%               set to unity.
%
% * 25-08-2010: The function now attempts to compensate for handle losses to the main dialog window
%               of YADA.
%
% * 14-12-2010: Took into account that the "estimation_log" controls now has height 407 pixels.
%
% * 04-01-2011: Updated the documentation.
%
% * 14-01-2011: Made sure that 26 is subtracted from "ListBoxTop" when it is first set to a number
%               greater than 1.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 18-09-2013: Adjusted the code to take into account that the log windows height has decreased from
%               407 pixels to 379 pixels. The number of rows have therefore been lowered from 29 to
%               25 rows.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the groot function into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 08-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-02-2021: The function now takes into account that the a screen size in
%               logical pixels can differ from the physical pixels such as when
%               the display is set to a scaling other than 100%, such as 125%.
%
% * 04-01-2022: Updated the documentation.
%
% * 07-01-2022: Added a workaround for issues with setting the ListboxTop value.
%
% * 13-01-2022: Took DSGEModel.ShowEstimationLog into account.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% retrieve old log text
%
if nargin==1;
   maingui = findobj('Type','figure','Tag','YADA');
   if isempty(maingui)==1;
      maingui = 1;
      if IsHandle(maingui)==0;
         txt = strvcat('Matlab has lost the handle to the YADA main dialog. YADA is therefore unable to write to the estimation log window.',' ','NOTE: This problem is due to the environment you run YADA in. Unless you correct or change the environment, the problem is likely to persist.');
         About(txt,'information','GUI Handle Problem',300,500,InitializeINIFile);
         return;
      end;
   end;
end;
controls = get(maingui,'UserData');
%
% check if we should skip setting the log text
%
DSGEModel = get(controls.open,'UserData');
if DSGEModel.ShowEstimationLog==0;
   return;
end;
%
OldLogText = get(controls.estimation_log,'String');
ListTopValue = get(controls.estimation_log,'UserData');
%
% create new log text
%
NewLogText = strvcat(OldLogText,TextStrMat);
LogTextWidth = size(NewLogText,2);
%
% if the text is too wide, a horizontal scroller appears
% we therefore need to add one line to see the last line.
%
if LogTextWidth<74;
   AddLine = 0;
else;
   AddLine = 1;
end;
%
% Determine if ListboxTop property should be changed
%
if ListTopValue>1;
   ListTopValue = ListTopValue+size(TextStrMat,1);
else;
   if exist('groot')==0;
      GraphicsRoot = 0;
   else;
      GraphicsRoot = groot;
   end;
   FontSizeValue = get(controls.estimation_log,'FontSize');
   PointsPerPixel = 72/get(GraphicsRoot,'ScreenPixelsPerInch');
   LogHeight = floor(379*PointsPerPixel);
   LogTextHeight = 1.1*FontSizeValue*size(NewLogText,1);
   if LogTextHeight>LogHeight;
      ScreenScaling = GetScreenScaling(GraphicsRoot);
      ListTopValue = max(size(NewLogText,1)-round(24/ScreenScaling),1);
   end;
end;
%
% Insert new properties into log control
%
set(controls.estimation_log,'String',NewLogText);
%
% setting a ListboxTop value doesn't work properly on some computers.
% a workaround is to run the get function before the set
%
drawnow;
ListBoxTopValue = get(controls.estimation_log,'ListboxTop');
set(controls.estimation_log,'ListboxTop',ListTopValue+AddLine,'UserData',ListTopValue);
drawnow;
pause(0.1);
if get(controls.estimation_log,'ListboxTop')~=ListTopValue+AddLine;
   set(controls.estimation_log,'ListboxTop',ListTopValue+AddLine,'UserData',ListTopValue);
   drawnow;
   pause(0.1);
end;

%
% end of SetLogText.m
%
