function TextGUIPrint
% TextGUIPrint: Sends a text file to the printer. The full path and filename of the file
%               must be stored in a control ".print" on the current figure.
%
% USAGE:
%
%       TextGUIPrint
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 2, 2008.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 23-05-2008: Updated the documentation.
%
% * 16-09-2008: Made sure that "TextFile" is sent to notepad on the command line.
%
% * 08-12-2008: Added quotes around the call to exe-files.
%
% * 02-05-2009: Updated the documentation.
%
% * 06-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 25-01-2011: Added unix print command code.
%
% * 31-01-2011: Changed the print commands to use "printopt" information.
%
% * 09-02-2011: Changed the "ErrorStr" variable under unix.
%
% * 16-02-2011: Added try-catch code around txtprint command.
%
% * 28-03-2011: Removed the second output argument from the dos calls. It messed up
%               the stat value when running txtprint.
%
% * 24-06-2011: The txtprint dos status value should be 1 when it goes well. This has therefore been
%               fixed.
%
% * 25-11-2011: Switched to checking "mess" when running the txtprint command. It it is an empty
%               string then YADA assumes printing with txtprint went well.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the groot function into account.
%
% * 09-06-2015: The code now takes gcf into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 08-01-2018: Updated the documentation.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 09-12-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 26-10-2023: Added stat not equal to zero as a condition for txtprint.exe not working.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

TextControls = get(gcf,'UserData');
TextFile = get(TextControls.print,'UserData');
if FileExist(TextFile)==1;
   if ispc==1;
      if FileExist([pwd '\bin\txtprint.exe'])==1;
         cmd = ['"' pwd '\bin\txtprint.exe" -p "' TextFile '"'];
         try;
            [stat,mess] = dos(cmd);
         catch;
            stat = 1;
            mess = '';
         end;
         if (stat~=0)|(isempty(mess)==0);
            %
            % try the printopt based command
            %
            cmd = sprintf(strrep(printopt,'\','\\'),['"' TextFile '"']);
            [stat,mess] = dos(cmd);
         end;
      else;
         %
         % we use the printopt data
         %
         cmd = sprintf(strrep(printopt,'\','\\'),['"' TextFile '"']);
         [stat,mess] = dos(cmd);
      end;
   else;
      %
      % maybe add some unix print command later
      %
      ErrorStr = '';
      cmd = sprintf(strrep(printopt,'\','\\'),TextFile);
      if MatlabNumber>=8.0;
         try;
            [stat,mess] = unix(cmd);
         catch ME;
            ErrorStr = ['There was an error running the UNIX print command (' printopt ') on ' FixFilePath(TextFile) '. Message caught is: ' YADALastError(ME)];
            stat = 0;
         end;
      else;
         try;
            [stat,mess] = unix(cmd);
         catch;
            ErrorStr = ['There was an error running the UNIX print command (' printopt ') on ' FixFilePath(TextFile) '. Message caught is: ' YADALastError];
            stat = 0;
         end;
      end;
      if (exist('stat')==1)&(stat~=0);
         if (isempty(mess)==0)&(size(mess,1)==1);
            ErrorStr = strvcat('The was an error running the unix command:',' ',cmd,' ',['The matlab unix call returned status: ' int2str(stat) '. YADA also caught the message: ' mess]);
         else;
            ErrorStr = strvcat('The was an error running the unix command:',' ',cmd,' ',['The matlab unix call returned status: ' int2str(stat) '.']);
         end;
      end;
      if isempty(ErrorStr)==0;
         About(ErrorStr,'error','YADA - Print Error',500,600,InitializeINIFile);
      end;
   end;
   pause(1);
end;

%
% end of TextGUIPrint.m
%
