function [XValues,YValues,ZValues] = KernelDensityPlot2D(X,Y,InterPolateOption,GridSize)
% KernelDensityPlot2D: Creates x, y, and z vectors of values for plotting a density function using a bivariate Kernel
%                      Density estimator.
%
% USAGE:
%
%       [XValues,YValues] = KernelDensityPlot2D(X,Y,InterPolateOption,GridSize)
%
% REQUIRED INPUT:  X (vector) of values for a random variable.
%
%                  InterPolateOption (string vector), that takes on one of the following values:
%
%                          knorm:   normal kernel
%                          kepan:   Epanechnikov
%                          krect:   rectangular
%                          ktria:   triangular
%                          kbiwe:   bi-weight
%                          ktriw:   tri-weight
%
% OPTIONAL INPUT:  GridSize (integer) that is a power of 2; such as 2, 4, 8, 16,
%                           32, 64, 128, 256, etc.
%
% REQUIRED OUTPUT: XValues (vector) with 128 values for the X axis.
%
%                  YValues (vector) with 128 values for the Y axis.
%
%                  ZValues (matrix) with 128x128 values for the Z axis.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 16, 2013.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-08-2013: Changed the number of bins from 128 to 256.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 08-01-2018: Updated the documentation.
%
% * 11-10-2018: Made sure that hy is not displayed in the command window. Added the optional
%               GridSize input variable.
%
% * 21-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargin==3;
   GridSize = 256;
end;
if strcmp(InterPolateOption,'knorm')==1;
   hx = 0;
   hy = 0;
else;
   hx = hns(X,InterPolateOption);
   hy = hns(Y,InterPolateOption);
end;
[XValues,YValues,ZValues] = kdfft2([X Y],InterPolateOption,GridSize,[hx hy],0);
MaxCumZValues = max(max(cdf2d(XValues,YValues,ZValues)));
ZValues = (1/MaxCumZValues)*ZValues;

%
% end of KernelDensityPlot2D.m
%
