function AdaptiveLearningDSGEISMitISEMPosteriorSampler(DSGEModel,CurrINI,controls)
% AdaptiveLearningDSGEISMitISEMPosteriorSampler: Runs the IS based on MitISEM algorithm to sample from the posterior
%                                                distribution of the parameters of the DSGE model with adapative
%                                                learning instead of rational expectations.
%
% USAGE:
%
%       AdaptiveLearningDSGEISMitISEMPosteriorSampler(DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT: DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                 CurrINI (structure), with initialization information.
%
%                 controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 27, 2020.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-05-2020: Updated the documentation.
%
% * 02-07-2020: Removed the field "NonCentralGLSMatrix" from DSGEModel and the input variable "mu" for
%               "AdaptiveLearningInitialValuesDSGE".
%
% * 01-10-2020: Fixed a bug with j instead of : for X matrix.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 22-01-2021: Updated the documentation.
%
% * 28-01-2021: The function supports a user defined model solver under adaptive learning.
%
% * 03-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed additional eval calls to feval.
%
% * 22-02-2022: Took into account that the system prior file should receive true (=1) for IsAdaptiveLearning.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
% * 09-01-2024: Took into account that "YADAHessianMatrix" now takes the input variable "NumWorkers".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
SetLogText(strvcat(' ','Initializing posterior sampling using importance sampling','based on the MitISEM algorithm...',[' * Attempting to load posterior mode data file...']));
%
% First we load the posterior mode estimates
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==1;
   SetLogText(strvcat('   STATUS: OK',' * Attempting to load DSGE model data file...'));
else;
   SetLogText(strvcat(' ',['Unable to load the file "' GetFilename(ModeFile) '".']));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
phiMode = ModeData.phiMode;
NumParam = length(phiMode);
if DSGEModel.InverseHessianEstimator==1;
   SigmaMode = ModeData.InverseHessian;
   InvHessianTxt = 'Optimization routine';
elseif DSGEModel.InverseHessianEstimator==2;
   if sum(isnan(ModeData.phiFittedStd))==0;
      SigmaStd = sqrt(abs(diag(ModeData.InverseHessian)));
      SigmaCorr = ModeData.InverseHessian ./ (SigmaStd * SigmaStd');
      phiFittedStd = ModeData.phiFittedStd;
      oldphiFittedStd = phiFittedStd;
      if DSGEModel.ModifiedHessian==1;
         FullSet = (1:length(phiFittedStd));
         for i=1:length(phiFittedStd);
            iExcluded = setdiff(FullSet,i);
%            phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*inv(SigmaCorr(iExcluded,iExcluded))*SigmaCorr(iExcluded,i)));
            phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*(SigmaCorr(iExcluded,iExcluded)\SigmaCorr(iExcluded,i))));
         end;
      end;
      SigmaMode = SigmaCorr .* (phiFittedStd * phiFittedStd');
   else;
      SetLogText(strvcat(' ','Quadratic fit to log posterior unavailable.'));
      if isempty(CurrINI.Sequence)==1;
         txt = 'YADA has not fitted a quadratic to the evaluated log posterior. Please run the "Check posterior mode" function on the View menu first. Alternatively, you can use the estimator of the inverse Hessian from the optimization routine.';
         About(txt,'information','YADA - Modified Inverse Hessian',120,500,CurrINI);
         drawnow;
      end;
      return;
   end;
   InvHessianTxt = 'Quadratic modification';
elseif DSGEModel.InverseHessianEstimator==3;
   if sum(sum(isnan(ModeData.phiInverseHessianFinDiff)))==0;
      SigmaMode = ModeData.phiInverseHessianFinDiff;
   else;
      if isempty(CurrINI.Sequence)==1;
         txt = 'YADA has not estimated the inverse Hessian using finite difference during the posterior mode estimation. Would you like to begin the finite difference estimation now?';
         answer = Query(txt,'question',140,'YADA - Finite Difference Inverse Hessian',500,'no',CurrINI);
         drawnow;
      else;
         answer = CurrINI.Sequence.QueryAnswers{1};
      end;
      if strcmp(lower(answer),'yes')==1;
         SigmaMode = [];
      else;
         SetLogText(strvcat(' ','Finite difference inverse Hessian unavailable.'));
         return;
      end;
   end;
   InvHessianTxt = 'Finite difference';
elseif DSGEModel.InverseHessianEstimator==4;
   %
   % try to load the file
   %
   CovMatData = load(DSGEModel.ParameterCovMatrix);
   SigmaMode = CovMatData.ParameterCovarianceMatrix;
   if size(SigmaMode,1)==NumParam;
      InvHessianTxt = 'Own matrix';
   else;
      SetLogText(strvcat(' ',['The dimension of your parameter covariance matrix (' int2str(size(SigmaMode,1)) ') is not'],['compatible with the number of parameters to estimate (' int2str(NumParam) ').']));
      if isempty(CurrINI.Sequence)==1;
         txt = ['The dimension of the parameter covariance matrix you have selected from the file "' GetName(DSGEModel.ParameterCovMatrix) '" is ' int2str(size(SigmaMode,1)) ', while the number of parameters to estimate is ' int2str(NumParam) '. YADA is therefore unable to continue with the posterior sampling.'];
         About(txt,'information','YADA - Own Parameter Covariance Matrix',200,500,CurrINI);
         drawnow;
      end;
      return;
   end;
end;
%
% load additional information about the model.
%
thetaIndex = ModeData.thetaIndex;
UniformBounds = ModeData.UniformBounds;
LowerBound = ModeData.LowerBound;
thetaPositions = ModeData.thetaPositions;
thetaDist = ModeData.thetaDist;
PriorDist = ModeData.PriorDist;
ModelParameters = ModeData.ModelParameters;
ParameterNames = ModeData.ParameterNames;
thetaMode = ModeData.thetaMode;
thetaInit = ModeData.theta;
if isfield(ModeData,'LogMargLikeSystemLaplace')==1;
   LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
else;
   LogMargLikeSystemLaplace = 0;
end;
%
% Set up the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ParameterNames.additional)==0);
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   %
   % ask if posterior distribition of parameters that are updated should be computed
   %
   if isempty(CurrINI.Sequence)==1;
      txt = ['The model "' DSGEModel.NameOfModel '" has parameters that are a function of the estimated parameters. Should YADA compute the posterior distribution for these additional parameters?'];
      answer = Query(txt,'question',200,'Posterior Sampling - Additional Parameters',500,'no',CurrINI);
      drawnow;
   else;
      answer = CurrINI.Sequence.QueryAnswers{2};
   end;
   if strcmp(lower(answer),'yes')==1;
      CalcUpdatePosterior = 1;
   else;
      CalcUpdatePosterior = 0;
   end;
else;
   CalcUpdatePosterior = 0;
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   IsSystemPrior = 1;
else;
   IsSystemPrior = 0;
end;
%
% check if we should also copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
   end;
end;
%
ErrorFile = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
end;
%
% Test if there are any problems
%
firstUpdatePos = 0;
lastUpdatePos = 0;
firstInitializePos = 0;
lastInitializePos = 0;
if isempty(ErrorStr)==1;
   if DSGEModel.UseOwnInitialState==1;
      if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
         DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
      else;
         DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
      end;
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
      if MatlabNumber>=8.0;
         try;
            firstUpdatePos = length(fieldnames(ModelParameters))+1;
            ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
            lastUpdatePos = length(fieldnames(ModelParameters));
         catch ME;
            ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError(ME)];
            ErrorFile = DSGEModel.UpdateParameterFile;
         end;
      else;
         try;
            firstUpdatePos = length(fieldnames(ModelParameters))+1;
            ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
            lastUpdatePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError];
            ErrorFile = DSGEModel.UpdateParameterFile;
         end;
      end;
   end;
end;
%
% Check if we should exit
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
% Try to solve the DSGE model at the posterior mode
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE model has a unique convergent','   solution for posterior mode parameter values...'));
if DSGEModel.ModelSolver==1;
   SolverStr = 'AiM';
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein';
   [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Gensys';
   [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==4;
   SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
if mcode~=1;
   %
   % There doesn't exist a unique convergent solution for the initial parameter values.
   %
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" The posterior mode values are no longer valid.'];
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% do it again to measure the time
%
tic;
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==4;
   [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
SolveTime = toc;
if isempty(DSGEModel.UnitRootStates)==1;
   F1 = F;
else;
   KeepPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
   F1 = F(KeepPos,KeepPos);
end;
if (((max(abs(eig(F1)))>=1)|(IsAlmostEqual(max(abs(eig(F1))),1)==1))&(DSGEModel.AllowUnitRoot==0))|((isempty(DSGEModel.UnitRootStates)==1)&(DSGEModel.AllowUnitRoot==1)&(max(abs(eig(F1)))>1));
   SetLogText(strvcat('   STATUS: Non-stationary state equation',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The largest eigenvalue of the state equation is ' num2str(max(abs(eig(F1)))) '. The state equation is therefore non-stationary. The posterior mode values are no longer valid.'];
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check the measurement equation file for internal validity
%
if isempty(ErrorStr)==1;
   %
   % add the values of F and B0 to the ModelParameters.YADA field
   %
   [B0Alt,KeepVar] = RemoveRedundantColumns(B0);
   ModelParameters.YADA.F = F;
   ModelParameters.YADA.B0 = B0Alt;
   SetLogText(strvcat('   STATUS: OK',['   DSGE model solver:      ' SolverStr],['   Maximum absolute error: ' num2str(MaxAbsError)],['   Solution time:          ' num2str(SolveTime) ' seconds'],[' * Checking ' GetName(DSGEModel.MeasurementEquationFile) '...']));
   %
   if MatlabNumber>=8.0;
      try;
         [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
      catch ME;
         ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
         ErrorFile = DSGEModel.MeasurementEquationFile;
      end;
   else;
      try;
         [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
      catch;
         ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
         ErrorFile = DSGEModel.MeasurementEquationFile;
      end;
   end;
end;
%
SetLogText(strvcat('   STATUS: OK',' * Checking the adaptive learning solution...'));
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Determine the belief system matrices
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
m = BeliefSys.m;
G = BeliefSys.G;
k = DSGEModel.k;
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
Stilde = [SMatrix Sperp];
%
% call the function: 
%
[tstatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
if tstatus==0;
   SetLogText(strvcat('   STATUS: Unable to substitute for expected non-forward looking variables',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem.';
      About(txt,'information','YADA - DSGE Model Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
ConstPos = 0;
if k>0
   x = DSGEModel.X(:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      X = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
      T = size(X,2);
      for j=1:k;
         ConstTest = (DSGEModel.X(j,DSGEModel.FirstPeriod:DSGEModel.LastPeriod)==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   x = [];
   xbar = [];
end;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
%
% solve the DSGE model under adaptive learning based on the initial values
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% make sure that DSGEModel has the required fields to AdaptiveLearning:
%
DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
DSGEModel.AdaptiveLearning.r = r;
DSGEModel.AdaptiveLearning.f = f;
DSGEModel.AdaptiveLearning.Sperp = Sperp;
DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
DSGEModel.AdaptiveLearning.Kgf = Kgf;
%
% check Ft1: if largest eigenvalue is greater than 1, give a warning
%
condn = 10^(-DSGEModel.AIMTolerance-2);
if max(abs(eig(Ft1)))>1+condn;
   SetLogText(strvcat('  WARNING: Adaptive learning solution is unstable at posterior mode values.','           Continuing'));
end;
%
% checking the system prior file if it exists
%
if (IsSystemPrior==1)&(isempty(ErrorStr)==1);
   SetLogText(strvcat('   STATUS: OK',' * Checking if the system prior can be calculated...'));
   if MatlabNumber>=8.0;
      try;
         lnSP = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
      catch ME;
         ErrorStr  = ['Unable to calculate the log of the system prior. Message caught is: ' YADALastError(ME)];
         ErrorFile = DSGEModel.SystemPriorFile;
         SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
         if isempty(CurrINI.Sequence)==1;
            About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
            drawnow;
         end;
         return;
      end;
   else;
      try;
         lnSP = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
      catch;
         ErrorStr  = ['Unable to calculate the log of the system prior. Message caught is: ' YADALastError];
         ErrorFile = DSGEModel.SystemPriorFile;
         SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
         if isempty(CurrINI.Sequence)==1;
            About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
            drawnow;
         end;
         return;
      end;
   end;
   SetLogText(strvcat('   STATUS: OK',' * Checking if the system prior mode has been calculated...'));
   SystemPriorModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
   if FileExist(SystemPriorModeFile)==1;
      SystemPriorModeData = load(SystemPriorModeFile);
      %
      % we only need the Laplace approximation of the log marginal likelihood of the system prior
      %
      LogMargLikeSystemLaplace = SystemPriorModeData.LogMargLikeSystemLaplace;
      SetLogText(strvcat('   STATUS: OK',' * Checking if prior draws are available from the system prior...'));
   else;
      %
      % we need to first compute the system prior mode
      %
      [SystemPriorStatus,LogMargLikeSystemLaplace] = ComputeSystemPriorMode(ModeData.theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,[firstUpdatePos lastUpdatePos firstInitializePos lastInitializePos],1,AIMData,controls,DSGEModel,CurrINI);
      if SystemPriorStatus==0;
         SetLogText('Prior mode estimation has been aborted.');
         return;
      end;
   end;
   %
   % checking if the system prior draws are available
   %
   PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
   if FileExist(PriorDrawsFile)==1;
      PDData = load(PriorDrawsFile);
      if isfield(PDData,'LogMargsMH')==1;
         SPLogMargsMH = PDData.LogMargsMH;
         SPNWStdErrMH = PDData.NWStdErrMH;
         SPCovProb = PDData.CovProb;
         if LogMargLikeSystemLaplace==0;
            LogMargLikeSystemLaplace = PDData.LogMargLikeSystemLaplace;
         end;
         SetLogText('   STATUS: Data available');
      else;
         SPLogMargsMH = [];
         SPNWStdErrMH = [];
         SPCovProb = [];
         SetLogText('   STATUS: Data unavailable');
      end;
   else;
      SPLogMargsMH = [];
      SPNWStdErrMH = [];
      SPCovProb = [];
      SetLogText('   STATUS: Data unavailable');
   end;
else;
   LogMargLikeSystemLaplace = 0;
   SPLogMargsMH = [];
   SPNWStdErrMH = [];
   SPCovProb = [];
end;
%
% Check if we should exit
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
% Try to calculate the the log posterior at the posterior mode
%
if DSGEModel.UseDoublingAlgorithm==1;
   InitialAlgorithmText = '   Initialization method:  Analytic';
elseif DSGEModel.UseDoublingAlgorithm==2;
   InitialAlgorithmText = '   Initialization method:  Numeric (doubling algorithm)';
else;
   InitialAlgorithmText = '   Initialization method:  Constant times identity';
end;
SetLogText(strvcat('   STATUS: OK',' * Checking if the log posterior can be calculated...'));
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      logPostMode = logPosteriorAdaptiveLearningPhiDSGE(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
   catch ME;
      ErrorStr = ['Unable to calculate the log posterior. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      logPostMode = logPosteriorAdaptiveLearningPhiDSGE(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
   catch;
      ErrorStr = ['Unable to calculate the log posterior. Message caught is: ' YADALastError];
   end;
end;
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
if ((exist('logPostMode')==0)|(logPostMode==1000000))&(isempty(ErrorStr)==1);
   SetLogText(strvcat('   STATUS: Unable to compute log posterior at posterior mode',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = 'Unable to compute the log posterior at the posterior mode values. You probably need to re-estimate the posterior mode.';
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% there doesn't seem to be any problems with the model
%
tic;
logPostMode = -logPosteriorAdaptiveLearningPhiDSGE(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
LogPostTime = toc;
SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(LogPostTime) ' seconds'],'   Kalman algorithm:       Adaptive learning filter',InitialAlgorithmText));
%
% check if we should calculate a finite difference Hessian
%
StepLengthStr = get(controls.optimize.steplength,'String');
StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
if isempty(SigmaMode)==1;
   SetLogText(' * Estimating inverse Hessian with finite differences...');
   tic;
   VechHessian = YADAHessianMatrix('logPosteriorAdaptiveLearningPhiDSGE',phiMode,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
   FinDiffTime = toc;
   if sum(isnan(VechHessian))==0;
      SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(FinDiffTime) ' seconds']));
      HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
      phiInverseHessianFinDiff = HessianMat\eye(NumParam);
      %
      % save to post mode file
      %
      save(ModeFile,'phiInverseHessianFinDiff','-APPEND');
      SigmaMode = phiInverseHessianFinDiff;
   else;
      %
      % bad Hessian encountered
      %
      SetLogText(strvcat('   STATUS: Bad estimator of inverse Hessian using','           finite differences',' '));
      if isempty(CurrINI.Sequence)==1;
         txt = ['The inverse Hessian estimator using finite differences with a step length of ' num2str(StepLength) ' has at least one NaN entry. Please change the step length or select a different estimator of the inverse Hessian.'];
         About(txt,'information','YADA - Inverse Hessian Estimation Problem',150,500,CurrINI);
         drawnow;
      end;
      return;
   end;
end;
%
% check if we should manipulate the correlations for the proposal density
%
if DSGEModel.MaxCorrelationValue>1;
   MaxCorrStr = get(controls.posterior.maxcorrelation,'String');
   MaxCorr = str2num(StringTrim(MaxCorrStr(DSGEModel.MaxCorrelationValue,:)));
   %
   % create the correlation matrix
   %
   StdVec = sqrt(diag(SigmaMode));
   CorrMat = SigmaMode ./ (StdVec*StdVec');
   for i=1:length(StdVec);
      CorrMat(i,i) = 0;
   end;
   ScaleFactor = max(max(abs(CorrMat)));
   ScaleMatrix = min(1,MaxCorr/ScaleFactor)*ones(length(StdVec),length(StdVec));
   for i=1:length(StdVec);
      ScaleMatrix(i,i) = 1;
   end;
   SigmaMode = SigmaMode .* ScaleMatrix;
else;
   MaxCorr = 1;
end;
%
% test if SigmaMode is positive definite
%
SetLogText(strvcat(' * Checking if the covariance matrix for the proposal density is','   positive definite...'));
if IsPositiveDefinite(SigmaMode)==0;
   SetLogText('   STATUS: Covariance matrix is not positive definite');
   EigValues = eig(SigmaMode);
   EigValues = EigValues(EigValues<=0);
   if isempty(CurrINI.Sequence)==1;
      txt = ['The inverse Hessian is not positive definite. YADA has found that ' int2str(length(EigValues)) ' eigenvalues of the estimated inverse Hessian are less than or equal to zero. This means that the inverse Hessian estimator can''t be used for the proposal density when running the random-walk Metropolis algorithm. Do you want YADA to try to create a positive definite matrix using your matrix as input?'];
      answer = Query(txt,'question',300,'YADA - Inverse Hessian Estimation Problem',500,'no',CurrINI);
      drawnow;
   else;
      answer = CurrINI.Sequence.QueryAnswers{3};
   end;
   if strcmp(lower(answer),'yes')==1;
      SetLogText(strvcat(' * Attempting the create positive definite covariance matrix...'));
      [NewSigmaMode,pdstatus] = CreatePositiveDefinite(SigmaMode);
      if pdstatus==1;
         SigmaMode = NewSigmaMode;
         SetLogText('   STATUS: OK');
      else;
         SetLogText(strvcat('   STATUS: Unsuccessful.',' ','Posterior sampling aborted.'));
         return;
      end;
   else;
      SetLogText(strvcat(' ','Posterior sampling aborted.'));
      return;
   end;
else;
   SetLogText('   STATUS: OK');
end;
%
% Get the base values number of draws, etc.
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% test sufficient memory
%
SetLogText(' * Testing if memory is sufficient...');
tic;
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      SampleMatrix = zeros(NumPostDraws,NumParam);
   catch ME;
      ErrorStr = ['There was an error creating the matrix to store the posterior draws. YADA caught the following error message: ' YADALastError(ME)];
   end;
else;
   try;
      SampleMatrix = zeros(NumPostDraws,NumParam);
   catch;
      ErrorStr = ['There was an error creating the matrix to store the posterior draws. YADA caught the following error message: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   SampleMatrixTime = toc;
   SetLogText(strvcat(' ','Too many total draws: memory problem','Posterior sampling was aborted.'));
   if isempty(CurrINI.Sequence)==1;
      txt = 'Too many total draws for your computer memory.';
      About(txt,'information','YADA - Memory Problem',200,500,CurrINI);
      drawnow;
   end;
   return;
end;
SampleMatrixTime = toc;
SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(SampleMatrixTime) ' seconds']));
clear('SampleMatrix');
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 3. maximum number of mixture components
%
Hstar = DSGEModel.ISMaxMixCompValue+1;
%
% 4. Number of degrees of freedom when initializing the first Student-t
%
d1 = DSGEModel.ISInitialDF;
%
% 5. Shares of draws used to estimate location and scale parameters for
%    component number H+1 when iterating on the number of components
%
%    NOTE: This can be a vector of values between 0 and 1
%
WeightShareStr = get(controls.posterior.isweightshare,'String');
alpha = str2num(StringTrim(strrep(strrep(strrep(WeightShareStr(DSGEModel.ISWeightShareValue,:),',',' '),'(',''),')','')));
alphaStr = StringTrim(WeightShareStr(DSGEModel.ISWeightShareValue,:));
%
% 6. Initial weight on component H+1
%
AddedMixWeightStr = get(controls.posterior.isaddedmixweight,'String');
piH1 = str2num(StringTrim(AddedMixWeightStr(DSGEModel.ISAddedMixWeightValue,:)));
%
% 7. Number of degrees of freedom for component H+1 when initialized
%
AddedDFStr = get(controls.posterior.isaddeddf,'String');
dfH1 = str2num(StringTrim(AddedDFStr(DSGEModel.ISAddedDF,:)));
%
% 8. Tolerance level of MitISEM for relative CoV change
%
CoVToleranceStr = get(controls.posterior.covtolerance,'String');
CoVTolerance = str2num(StringTrim(CoVToleranceStr(DSGEModel.CoVToleranceValue,:)));
%
% 9. Tolerance level for relative objective function change
%
ToleranceStr = get(controls.optimize.tolerance,'String');
ToleranceValue = str2num(StringTrim(ToleranceStr(DSGEModel.OptToleranceValue,:)));
%
% make sure the output directory is exists
%
stat = MakeDir(DSGEModel.OutputDirectory,'is');
if stat~=1;
   if ispc==1;
      SetLogText(strvcat(' ','Couldn''t create the directory:',['  ' DSGEModel.OutputDirectory '\is'],'Posterior sampling was aborted.'));
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\is". Posterior sampling has therefore been aborted.'];
   else;
      SetLogText(strvcat(' ','Couldn''t create the directory:',['  ' DSGEModel.OutputDirectory '/is'],'Posterior sampling was aborted.'));
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/is". Posterior sampling has therefore been aborted.'];
   end;
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check if we should add the objective function criterion for
%
if isempty(CurrINI.Sequence)==1;
   txt = strvcat(['YADA can add the relative change of the normalized objective function as an additional convergence criterion (to the relative change of the coefficient of variation) for convergence when estimating the parameters of the candidate density. The tolerance level is then given by ' num2str(ToleranceValue,'%0.10f') '. This criterion considers a negative change of the objective function to imply convergence at the previous value for H, the number of Student-t mixture components.'],' ','Would you like to add this convergence criterion?');
   answer = Query(txt,'question',250,'YADA - Additional Estimation Convergence Criterion',600,'yes',CurrINI);
   drawnow;
else;
   answer = CurrINI.Sequence.QueryAnswers{4};
end;
if strcmp(lower(answer),'yes')==1;
   UseObjFcnCriterion = 1;
else;
   UseObjFcnCriterion = 0;
end;
%
SetLogText(strvcat(' ',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Draws per sample:                   ' num2str(NumPostDraws,'%0.0f')],[' * Number of parallel samples:         ' num2str(NumChains,'%0.0f')],[' * Inverse Hessian estimator:          ' InvHessianTxt],[' * Maximum absolute correlation:       ' num2str(MaxCorr)],[' * Maximum number of t-densities:      ' int2str(Hstar)],[' * Initial df value for t-density:     ' int2str(DSGEModel.ISInitialDF)],[' * Draws & weight shares for H+1 comp: ' alphaStr],[' * Initial weight on H+1 comp:         ' num2str(piH1,'%0.2f')],[' * Initial df value for H+1 comp:      ' int2str(dfH1)],[' * Number of parameters to estimate:   ' num2str(NumParam)],[' * Total number of parameters:         ' num2str(size(ParameterNames.all,1),'%0.0f')],[' * Selected sample:                    ' DSGEModel.SubBeginYear ':' DSGEModel.SubBeginPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Initializing Kalman filter:         ' num2str(DSGEModel.KalmanFirstObservation-1,'%0.0f') ' observations'],[' * Effective sample size:              ' num2str(DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1),'%0.0f')],[' * Estimation sample:                  ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:       ' num2str(DSGEModel.n,'%0.0f')],[' * Number of exogenous variables:      ' num2str(DSGEModel.k,'%0.0f')],[' * Number of state variables:          ' num2str(length(DSGEModel.StateVariablePositions),'%0.0f')],[' * Number of state shocks:             ' num2str(size(DSGEModel.StateShockNames,1),'%0.0f')],[' * Number of measurement errors:       ' num2str(rank(R),'%0.0f')],[' * Number of forward variables:        ' int2str(f)],' '));
%
% deal with the random number generator
%
if DSGEModel.RandomNumberValue==1;
   rand('state',0);
   randn('state',0);
else;
   %
   % we set the state based on the current time
   %
   rand('state',sum(100*clock));
   randn('state',sum(100*clock));
end;
%
% initialize the PostSampleTime variable
%
tic;
PostSampleTime = toc;
abort = '';
%
for NChains=1:NumChains;
   tic;
   %
   txt = ['Please wait while YADA runs the importance sampler based on the MitISEM algorithm for ' int2str(NChains) ' of ' int2str(NumChains) ' posterior samples. The total number of posterior draws per sample is ' int2str(NumPostDraws) '. The current sample was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
   WaitHandle = WaitDLG(txt,'information',['Importance Sampling  - Sample ' num2str(NChains,'%0.0f') ' of ' num2str(NumChains,'%0.0f')],500,200,CurrINI,1);
   WaitControls = get(WaitHandle,'UserData');
   %
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\is\IS-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(NChains) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   if (DSGEModel.OverwriteDraws==1)|(FileExist(PostSampleFile)==0);
      %
      % check if we have already run the initialization phase
      %
      PosteriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\is\ISDraws-Initialize-AL-IS-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(DSGEModel.ISInitialDF) '-' int2str(NumPostDraws) '-' int2str(NChains) '.' int2str(NumChains) '.mat']);
      %
      if (DSGEModel.OverwriteDraws==0)&(FileExist(PosteriorDrawsFile)==1);
         SetLogText(strvcat('Initializing draws and weights... data found','   Loading draws and weights from naive density','   Initialization file:',' ',['      ' GetFilename(PosteriorDrawsFile)],' '));
         InitialDrawsData = load(PosteriorDrawsFile);
         %
         % loading the candidate draws, weights and two other matrices
         % due to the naive density and used by MitISEM
         %
         phiS = InitialDrawsData.phiS;
         ISWeights = InitialDrawsData.ISWeights;
         UnISWeights = InitialDrawsData.UnISWeights;
         tHeights = InitialDrawsData.tHeights;
         rhoMatrix = InitialDrawsData.rhoMatrix;
         mu = InitialDrawsData.mu;
         Sigma = InitialDrawsData.Sigma;
         clear('InitialDrawsData');
         %
         % deal with random number generators
         %
         if DSGEModel.RandomNumberValue==1;
            %
            % for drawing from the Student-t
            %
            ATemp = randn(2*NumParam,NumPostDraws);
            BTemp = rand(4,NumPostDraws);
            clear('ATemp','BTemp');
         end;
      else;
         %
         % Initialization step 0:
         %
         % a. simulate N candidate draws phi from Student-t; (N=NumPostDraws)
         %
         SetLogText(strvcat('Initializing draws and weights... computing data','   Simulating draws from 1 student-t component'));
         CholeskiSigma = chol(SigmaMode)';
         phiS = MultiStudentTRndFcn(phiMode,CholeskiSigma,d1,NumPostDraws);
         drawnow;
         SetLogText('   Computing IS weights');
         [ISWeights,UnISWeights,LogPost,tHeights,rhoMatrix,phiS] = AdaptiveLearningDSGEMitISEMWeights(phiS,phiMode,SigmaMode,d1,1,phiMode,1,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
         drawnow;
         %
         % check if user aborted
         %
         if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
            if IsHandle(WaitHandle)==1;
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               abort = 'cancel';
               drawnow;
               pause(0.02);
            end;
            SetLogText(strvcat(' ','Posterior sampling aborted.'));
            break;
         end;
         %
         % check how many draws have log posterior kernel equal to -1000000, i.e. not a unique and convergent solution
         %
         ShareBadDraws = (1/NumPostDraws)*sum((LogPost==-1000000));
         %
         % b. update location and scale with MitISEM keeping d1 fixed
         %
         SetLogText(strvcat('   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDraws)],'   Estimating location and scale with MitISEM'));
         [mu,Sigma,piweights,d1] = MitISEMAlgorithm(phiS,ISWeights,tHeights,rhoMatrix,d1,1,0,CurrINI.NumWorkers);
         %
         % now we simulate draws and weights from the located naive density with location mu, scale Sigma, and d1 degrees of freedom
         %
         SetLogText('   Simulating draws from naive candidate');
         phiS = MultiStudentTRndFcn(mu,CholeskiSigma,d1,NumPostDraws);
         drawnow;
         SetLogText('   Computing IS weights from naive density');
         [ISWeights,UnISWeights,LogPost,tHeights,rhoMatrix,phiS] = AdaptiveLearningDSGEMitISEMWeights(phiS,mu,Sigma,d1,1,phiMode,1,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
         drawnow;
         %
         % check if user aborted
         %
         if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
            if IsHandle(WaitHandle)==1;
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               abort = 'cancel';
               drawnow;
               pause(0.02);
            end;
            SetLogText(strvcat(' ','Posterior sampling aborted.'));
            break;
         end;
         %
         % check how many draws have log posterior kernel equal to -1000000, i.e. not a unique and convergent solution
         %
         ShareBadDraws = (1/NumPostDraws)*sum((LogPost==-1000000));
         %
         % save the results to file
         %
         SetLogText(strvcat('   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDraws)],'   Saving draws and weights from naive density','   Initialization file:',' ',['      ' GetFilename(PosteriorDrawsFile)],['      ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],' '));
         save(PosteriorDrawsFile,'phiS','ISWeights','UnISWeights','tHeights','rhoMatrix','mu','Sigma','ShareBadDraws');
         %
         SetLogText(strvcat('   Initialization completed',' '));
      end;
      %
      % check if we have already run the adaption step
      %
      PosteriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\is\ISDraws-Adapt-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(DSGEModel.ISInitialDF) '-' int2str(NumPostDraws) '-' int2str(NChains) '.' int2str(NumChains) '.mat']);
      %
      if (DSGEModel.OverwriteDraws==0)&(FileExist(PosteriorDrawsFile)==1);
         SetLogText('Adapting draws and weights from naive density... data found');
         AdaptiveDrawsData = load(PosteriorDrawsFile);
         %
         % loading the candidate draws, weights and two other matrices
         % due to the adaptive density and used by MitISEM
         %
         phiS = AdaptiveDrawsData.phiS;
         ISWeights = AdaptiveDrawsData.ISWeights;
         UnISWeights = AdaptiveDrawsData.UnISWeights;
         if isfield(AdaptiveDrawsData,'NormWeights')==1;
            NormWeights = AdaptiveDrawsData.NormWeights;
         else;
            NormWeights = 1/sum(UnISWeights);
         end;
         tHeights = AdaptiveDrawsData.tHeights;
         rhoMatrix = AdaptiveDrawsData.rhoMatrix;
         mu = AdaptiveDrawsData.mu;
         Sigma = AdaptiveDrawsData.Sigma;
         df = AdaptiveDrawsData.df;
         piweights = AdaptiveDrawsData.piweights;
         CoV = AdaptiveDrawsData.CoV;
         LogPost = AdaptiveDrawsData.LogPost;
         ShareBadDraws = AdaptiveDrawsData.ShareBadDraws;
         %
         % compute the value of the objective function based on weights normalized with NormWeights
         %
         OFWeights = NormWeights*UnISWeights;
         logObjFcn = sum(OFWeights.*log(tHeights));
         %
         clear('AdaptiveDrawsData');
         SetLogText(strvcat('   Loaded draws and weights from optimized candidate density H=1','   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDraws)],['   Coefficient of Variation:                 ' num2str(CoV)],['   Value of the scaled objective function:   ' num2str(logObjFcn)],'   Adaption file:',' ',['      ' GetFilename(PosteriorDrawsFile)],' '));
         %
         % deal with random number generators
         %
         if DSGEModel.RandomNumberValue==1;
            %
            % for drawing from the Student-t
            %
            ATemp = randn(2*NumParam,NumPostDraws);
            BTemp = rand(4,NumPostDraws);
            clear('ATemp','BTemp');
         end;
      else;
         %
         % Adaption step 1:
         %
         % a. estimate the target distribution's mean and covariance matrix using
         %    the draws and weights from the naive density
         %
         SetLogText(strvcat('Adapting draws and weights from naive density... computing data','   Computing IS estimates of mean and covariance of target distribution'));
         %
         phiMean = phiS*ISWeights/sum(ISWeights);
         ExpandWeights = ones(NumParam,1)*ISWeights';
         phiDev = phiS-(phiMean*ones(1,NumPostDraws));
         SigmaMean = (phiDev.*ExpandWeights)*phiDev'/sum(ISWeights);
         %
         % simulating draws and weight from the adaptive candidate density
         %
         SetLogText('   Simulating draws from adaptive candidate');
         CholeskiSigma = chol(SigmaMean)';
         phiS = MultiStudentTRndFcn(phiMean,CholeskiSigma,d1,NumPostDraws);
         drawnow;
         SetLogText('   Computing IS weights from adaptive density');
         [ISWeights,UnISWeights,LogPost,tHeights,rhoMatrix,phiS] = AdaptiveLearningDSGEMitISEMWeights(phiS,phiMean,SigmaMean,d1,1,phiMode,1,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
         drawnow;
         %
         % check if user aborted
         %
         if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
            if IsHandle(WaitHandle)==1;
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               abort = 'cancel';
               drawnow;
               pause(0.02);
            end;
            SetLogText(strvcat(' ','Posterior sampling aborted.'));
            break;
         end;
         %
         % check how many draws have log posterior kernel equal to -1000000, i.e. not a unique and convergent solution
         %
         ShareBadDraws = (1/NumPostDraws)*sum((LogPost==-1000000));
         %
         % b. apply the IS-weighted EM algorithm using the draws and weights from the adaptive candidate density
         %
         SetLogText(strvcat('   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDraws)],'   Estimating location, scale and deg. freedom with MitISEM'));
         [mu,Sigma,piweights,df] = MitISEMAlgorithm(phiS,ISWeights,tHeights,rhoMatrix,d1,1,1,CurrINI.NumWorkers);
         %
         % simulate new draws and weights with this first real candidate density
         %
         SetLogText(strvcat(['   Degrees of freedom estimated at ' num2str(df,'%0.4f')],'   Simulating draws from optimized candidate density H=1'));
         CholeskiSigma = chol(Sigma)';
         phiS = MultiStudentTRndFcn(mu,CholeskiSigma,df,NumPostDraws);
         drawnow;
         SetLogText('   Computing IS weights from optimized candidate density H=1');
         [ISWeights,UnISWeights,LogPost,tHeights,rhoMatrix,phiS] = AdaptiveLearningDSGEMitISEMWeights(phiS,mu,Sigma,df,1,phiMode,1,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
         drawnow;
         %
         % check if user aborted
         %
         if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
            if IsHandle(WaitHandle)==1;
               set(WaitControls.text,'UserData','done');
               delete(WaitHandle);
               abort = 'cancel';
               drawnow;
               pause(0.02);
            end;
            SetLogText(strvcat(' ','Posterior sampling aborted.'));
            break;
         end;
         %
         % check how many draws have log posterior kernel equal to -1000000, i.e. not a unique and convergent solution
         %
         ShareBadDraws = (1/NumPostDraws)*sum((LogPost==-1000000));
         %
         % compute the coefficient of variation of the non-normalized weights.
         %
         MeanISWeights = (1/NumPostDraws)*sum(ISWeights);
         StdISWeights = sqrt((1/NumPostDraws)*(ISWeights-MeanISWeights)'*(ISWeights-MeanISWeights));
         CoV = StdISWeights/MeanISWeights;
         NormWeights = 1/sum(UnISWeights);
         %
         % compute the value of the objective function based on weights normalized with NormWeights
         %
         OFWeights = NormWeights*UnISWeights;
         logObjFcn = sum(OFWeights.*log(tHeights));
         %
         % additional info we need to store in case H=1 is selected
         %
         hRepeat = 0;
         DlogObjFcn = -0.00000001;
         alphastar = NaN;
         RelCoV = NaN;
         PostSample = phiS';
         %
         % saving the results to file
         %
         SetLogText(strvcat('   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDraws)],['   Coefficient of Variation:                 ' num2str(CoV)],['   Value of the scaled objective function:   ' num2str(logObjFcn)],'   Saving draws and weights from optimized density','   Adaption file:',' ',['      ' GetFilename(PosteriorDrawsFile)],['      ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],' '));
         save(PosteriorDrawsFile,'phiS','PostSample','ISWeights','UnISWeights','LogPost','tHeights','rhoMatrix','mu','Sigma','df','piweights','CoV','ShareBadDraws','NormWeights','hRepeat','alphastar','RelCoV','logObjFcn','DlogObjFcn');
         %
         SetLogText(strvcat('   Adaption completed',' '));
      end;
      %
      % we are now ready for the iterations
      %
      h = 1;
      hRepeat = 0;
      ConvValue = 0;
      StopEstimation = 0;
      LastPosteriorDrawsFile = PosteriorDrawsFile;
      while (h<Hstar)&(ConvValue==0)&(hRepeat<max(Hstar,5))&(StopEstimation==0);
         %
         % check if we have saved additional repeat files for current h before we increase it
         %
         PDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\is\ISDraws-MitISEM-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(h) '.' int2str(hRepeat+1) '-' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(NumPostDraws) '-' int2str(NChains) '.' int2str(NumChains) '.mat']);
         if FileExist(PDrawsFile)==0;
            %
            % intialize old CoV value and the old log of the objective function
            %
            OldCoV = CoV;
            OldlogObjFcn = logObjFcn;
            h = h+1;
         else;
            hRepeat = hRepeat+1;
         end;
         PosteriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\is\ISDraws-MitISEM-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(h) '.' int2str(hRepeat) '-' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(NumPostDraws) '-' int2str(NChains) '.' int2str(NumChains) '.mat']);
         %
         if (DSGEModel.OverwriteDraws==0)&(FileExist(PosteriorDrawsFile)==1);
            LastPosteriorDrawsFile = PosteriorDrawsFile;
            %
            % load the data from file
            %
            SetLogText(['Iterating on number of mixture components... H=' int2str(h) ' data found']);
            ComponentDrawsData = load(PosteriorDrawsFile);
            %
            % loading the candidate draws, weights and two other matrices
            % due to the adaptive density and used by MitISEM
            %
            phiS = ComponentDrawsData.phiS;
            ISWeights = ComponentDrawsData.ISWeights;
            UnISWeights = ComponentDrawsData.UnISWeights;
            tHeights = ComponentDrawsData.tHeights;
            rhoMatrix = ComponentDrawsData.rhoMatrix;
            mu = ComponentDrawsData.mu;
            Sigma = ComponentDrawsData.Sigma;
            df = ComponentDrawsData.df;
            piweights = ComponentDrawsData.piweights;
            CoV = ComponentDrawsData.CoV;
            RelCoV = abs((CoV-OldCoV)/OldCoV);
            ConvValue = ComponentDrawsData.ConvValue;
            alphastar = ComponentDrawsData.alphastar;
            LogPost = ComponentDrawsData.LogPost;
            ShareBadDraws = ComponentDrawsData.ShareBadDraws;
            NumStudTimes = ComponentDrawsData.NumStudTimes;
            %
            % compute the value of the objective function based on weights normalized with NormWeights
            % taken from the weights obtained from the first real candidate density
            %
            OFWeights = NormWeights*UnISWeights;
            logObjFcn = sum(OFWeights.*log(tHeights*piweights));
            DlogObjFcn = (logObjFcn-OldlogObjFcn)/abs(OldlogObjFcn);
            %
            RepeatStr = '';
            if hRepeat>0;
               RepeatStr = ['   Number of repeats: ' int2str(hRepeat)];
            end;
            clear('ComponentDrawsData');
            SetLogText(strvcat(['   Loaded draws and weights from optimized candidate density H=' int2str(h)],RepeatStr,'   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDraws)],['   Coefficient of Variation:                 ' num2str(CoV)],['   Relative change coefficient of variation: ' num2str(RelCoV)],['   Tolerance level:                          ' num2str(CoVTolerance)],['   Best CoV value obtained for share alpha:  ' num2str(alphastar)],['   Value of the scaled objective function:   ' num2str(logObjFcn)],['   Relative change of scaled obj. function:  ' num2str(DlogObjFcn,'%0.8f')],['   Component file for H=' int2str(h) ':'],' ',['      ' GetFilename(PosteriorDrawsFile)],' '));
            %
            % deal with random number generators
            %
            if (DSGEModel.RandomNumberValue==1)&(NumStudTimes>0);
               %
               % for drawing from the Student-t
               %
               ATemp = randn(NumStudTimes*h*NumParam,NumPostDraws);
               BTemp = rand(NumStudTimes*h*2,NumPostDraws);
               clear('ATemp','BTemp');
            end;
         else;
            %
            % initialize
            %
            RepeatStr = '';
            if hRepeat>0;
               RepeatStr = ['   Number of repeats: ' int2str(hRepeat)];
            end;
            CoVstar = [];
            dfstar = [];
            NumStudTimes = 0;
            SetLogText(strvcat(['Iterating on number of mixture components... H=' int2str(h)],RepeatStr));
            %
            % loop over the number of alpha share values
            %
            for alphai=alpha;
               %
               % initialize location and scale
               %
               SetLogText(['   Initializing candidate density for share alpha=' num2str(alphai)]);
               %
               % initialize piweights and df
               %
               piweightsalpha = [(1-piH1)*piweights;piH1];
               dfalpha = [df;dfH1];
               %
               % get the positions of the sorted IS weights
               %
               if isequal(alphai,alpha(1))==1;
                  [SortedISWeights,ISPos] = sort(ISWeights,'descend');
               end;
               NumDrawsalpha = ceil(alphai*NumPostDraws);
               %
               % get the weights
               %
               Posalpha = ISPos(1:NumDrawsalpha);
               ISWeightsalpha = ISWeights(Posalpha);
               phiSalpha = phiS(:,Posalpha);
               %
               % compute the IS weighted mean and covariance
               %
               phiMean = phiSalpha*ISWeightsalpha/sum(ISWeightsalpha);
               ExpandWeightsalpha = ones(NumParam,1)*ISWeightsalpha';
               phiDev = phiSalpha-(phiMean*ones(1,NumDrawsalpha));
               SigmaMean = (phiDev.*ExpandWeightsalpha)*phiDev'/sum(ISWeightsalpha);
               if IsPositiveDefinite(SigmaMean)==1;
                  SetLogText(strvcat('   Initial values OK','   Computing t-density heights of new component for candidate draws'));
                  %
                  % need to compute tHeights and rhoMatrix for component h
                  %
                  LogDetSigmah = LogDet(SigmaMean);
                  InvSigmah = eye(NumParam)/SigmaMean;
                  LogComph = zeros(NumPostDraws,1);
                  rhoVech = LogComph;
                  for i=1:NumPostDraws;
                     [lnS,rhoValue] = logMultiStudentTPDF(phiS(:,i),phiMean,LogDetSigmah,InvSigmah,dfH1);
                     LogComph(i) = lnS;
                     rhoVech(i) = rhoValue;
                     drawnow;
                  end;
                  %
                  % compute the exp of LogComp
                  %
                  MeanLogComp = (1/NumPostDraws)*sum(LogComph);
                  LogComph = LogComph-MeanLogComp;
                  tHeightsh = exp(LogComph)*exp(MeanLogComp);
                  %
                  tHeightsalpha = [tHeights tHeightsh];
                  rhoMatrixalpha = [rhoMatrix rhoVech];
                  %
                  SetLogText('   Estimating location, scale, weights and deg. freedom with MitISEM');
                  [mualpha,Sigmaalpha,piweightsalpha,dfalpha] = MitISEMAlgorithm(phiS,ISWeights,tHeightsalpha,rhoMatrixalpha,dfalpha,piweightsalpha,1,CurrINI.NumWorkers);
                  %
                  % check if all piweights are sufficiently large and if all Sigma matrices are positive definite
                  %
                  BadComponent = zeros(h,1);
                  for i=1:h;
                     if IsPositiveDefinite(Sigmaalpha(:,:,i))==0;
                        BadComponent(i) = 1;
                     end;
                     if piweightsalpha(i)<0.001;
                        BadComponent(i) = 1;
                     end;
                  end;
                  TestBadComponent = sum(BadComponent);
                  if TestBadComponent>1;
                     SetLogText(strvcat(['   WARNING: ' int2str(TestBadComponent) ' components have too little relevance.'],['            Skipping case (H,alpha)=(' int2str(h) ',' num2str(alphai) ').']));
                  elseif TestBadComponent==1;
                     %
                     % remove the bad component
                     %
                     SetLogText(strvcat('   NOTE: One component has too little relevance.',['         Removing it for alpha=' num2str(alphai)]));
                     %
                     % store results in temporary matrices
                     %
                     muTemp = mualpha;
                     SigmaTemp = Sigmaalpha;
                     piweightsTemp = piweightsalpha;
                     dfTemp = dfalpha;
                     %
                     % fresh values
                     %
                     mualpha = zeros(NumParam,h-1);
                     Sigmaalpha = zeros(NumParam,NumParam,h-1);
                     piweightsalpha = zeros(h-1,1);
                     dfalpha = zeros(h-1,1);
                     %
                     % fill in the values we keep
                     %
                     Col = 0;
                     piaddon = 0;
                     for i=1:h;
                        if BadComponent(i)==0;
                           Col = Col+1;
                           mualpha(:,Col) = muTemp(:,i);
                           Sigmaalpha(:,:,Col) = SigmaTemp(:,:,i);
                           piweightsalpha(Col) = piweightsTemp(i);
                           dfalpha(Col) = dfTemp(i);
                        else;
                           piaddon = piweightsTemp(i);
                        end;
                     end;
                     %
                     % need to fix piweights
                     %
                     [MaxVal,MaxPos] = max(piweightsalpha);
                     piweightsalpha(MaxPos) = piweightsalpha(MaxPos)+piaddon;
                  end;
                  %
                  % check if we can continue with present selection of components and alpha share
                  %
                  if TestBadComponent<=1;
                     %
                     % simulate draws from an h-component mixture of Student-t distributions
                     %
                     SetLogText('   Simulating draws from optimized candidate density');
                     phiSalpha = MixedMultiStudentTRndFcn(mualpha,Sigmaalpha,piweightsalpha,dfalpha,NumPostDraws);
                     drawnow;
                     NumStudTimes = NumStudTimes+1;
                     %
                     SetLogText('   Computing IS weights from optimized candidate density');
                     [ISWeightsalpha,UnISWeightsalpha,LogPostalpha,tHeightsalpha,rhoMatrixalpha,phiSalpha] = AdaptiveLearningDSGEMitISEMWeights(phiSalpha,mualpha,Sigmaalpha,dfalpha,piweightsalpha,phiMode,1,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
                     drawnow;
                     if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
                        SetLogText(strvcat(' ','Posterior sampling aborted.'));
                        abort = 'cancel';
                        StopEstimation = 1;
                        break;
                     end;
                     %
                     % check how many draws have log posterior kernel equal to -1000000, i.e. not a unique and convergent solution
                     %
                     ShareBadDrawsalpha = (1/NumPostDraws)*sum((LogPostalpha==-1000000));
                     %
                     % compute the value of the objective function based on weights normalized with NormWeights
                     % taken from the weights obtained from the first real candidate density
                     %
                     OFWeights = NormWeights*UnISWeightsalpha;
                     logObjFcnalpha = sum(OFWeights.*log(tHeightsalpha*piweightsalpha));
                     DlogObjFcnalpha = (logObjFcnalpha-OldlogObjFcn)/abs(OldlogObjFcn);
                     %
                     % compute the coefficient of variation of the non-normalized weights.
                     %
                     MeanISWeights = (1/NumPostDraws)*sum(ISWeightsalpha);
                     StdISWeights = sqrt((1/NumPostDraws)*(ISWeightsalpha-MeanISWeights)'*(ISWeightsalpha-MeanISWeights));
                     CoValpha = StdISWeights/MeanISWeights;
                     %
                     if isempty(CoVstar)==1;
                        CoVstar = CoValpha;
                        mustar = mualpha;
                        Sigmastar = Sigmaalpha;
                        piweightsstar = piweightsalpha;
                        dfstar = dfalpha;
                        alphastar = alphai;
                        phiSstar = phiSalpha;
                        ISWeightsstar = ISWeightsalpha;
                        UnISWeightsstar = UnISWeightsalpha;
                        tHeightsstar = tHeightsalpha;
                        rhoMatrixstar = rhoMatrixalpha;
                        LogPoststar = LogPostalpha;
                        ShareBadDrawsstar = ShareBadDrawsalpha;
                        logObjFcnstar = logObjFcnalpha;
                        DlogObjFcnstar = DlogObjFcnalpha;
                        SetLogText(strvcat('   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDrawsalpha)],['   Coefficient of Variation:                 ' num2str(CoValpha)],['   Relative change coefficient of variation: ' num2str(abs((CoValpha-OldCoV)/OldCoV))],['   Tolerance level:                          ' num2str(CoVTolerance)],['   Value of the scaled objective function:   ' num2str(logObjFcnalpha)],['   Relative change of scaled obj. function:  ' num2str(DlogObjFcnalpha,'%0.8f')],['   Current share alpha=' num2str(alphai) ' is finished and best valid selection'],' '));
                     else;
                        %
                        SetLogText(strvcat('   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDrawsalpha)],['   Coefficient of Variation:                 ' num2str(CoValpha)],['   Relative change coefficient of variation: ' num2str(abs((CoValpha-OldCoV)/OldCoV))],['   Tolerance level:                          ' num2str(CoVTolerance)],['   Value of the scaled objective function:   ' num2str(logObjFcnalpha)],['   Relative change of scaled obj. function:  ' num2str(DlogObjFcnalpha,'%0.8f')]));
                        if ((CoValpha<CoVstar)&(length(dfalpha)==h))|(length(dfstar)<h);
                           AddOnStr = '';
                           if length(dfstar)<h;
                              AddOnStr = ['   It uses all components and is therefore better than alpha=' num2str(alphastar)];
                           end;
                           CoVstar = CoValpha;
                           mustar = mualpha;
                           Sigmastar = Sigmaalpha;
                           piweightsstar = piweightsalpha;
                           dfstar = dfalpha;
                           alphastar = alphai;
                           phiSstar = phiSalpha;
                           ISWeightsstar = ISWeightsalpha;
                           UnISWeightsstar = UnISWeightsalpha;
                           tHeightsstar = tHeightsalpha;
                           rhoMatrixstar = rhoMatrixalpha;
                           LogPoststar = LogPostalpha;
                           ShareBadDrawsstar = ShareBadDrawsalpha;
                           logObjFcnstar = logObjFcnalpha;
                           DlogObjFcnstar = DlogObjFcnalpha;
                           SetLogText(strvcat(['   Current share alpha=' num2str(alphai) ' is finished and best valid selection'],AddOnStr,' '));
                        else;
                           SetLogText(strvcat(['   Current share alpha=' num2str(alphai) ' is finished, valid, but not best selection'],' '));
                        end;
                     end;
                  end;
               else;
                  SetLogText(strvcat('   Unable to use selected share. Covariance matrix is NOT','   positive definite.',' '));
               end;
            end;
            %
            if StopEstimation==1;
               if IsHandle(WaitHandle)==1;
                  set(WaitControls.text,'UserData','done');
                  delete(WaitHandle);
                  drawnow;
                  pause(0.02);
               end;
               break;
            end;
            %
            % check if we have useful results
            %
            if isempty(CoVstar)==0;
               if (DlogObjFcnstar>0)|(UseObjFcnCriterion==0);
                  %
                  % current mixture is valid and we should store results to disk
                  %
                  if length(dfstar)==h;
                     PosteriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\is\ISDraws-MitISEM-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(h) '.' int2str(hRepeat) '-' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(NumPostDraws) '-' int2str(NChains) '.' int2str(NumChains) '.mat']);
                     hRepeat = 0;
                  else;
                     h = h-1;
                     hRepeat = hRepeat+1;
                     PosteriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\is\ISDraws-MitISEM-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(h) '.' int2str(hRepeat) '-' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(NumPostDraws) '-' int2str(NChains) '.' int2str(NumChains) '.mat']);
                  end;
                  mu = mustar;
                  Sigma = Sigmastar;
                  piweights = piweightsstar;
                  df = dfstar;
                  phiS = phiSstar;
                  ISWeights = ISWeightsstar;
                  UnISWeights = UnISWeightsstar;
                  tHeights = tHeightsstar;
                  rhoMatrix = rhoMatrixstar;
                  CoV = CoVstar;
                  RelCoV = abs((CoV-OldCoV)/OldCoV);
                  LogPost = LogPoststar;
                  ShareBadDraws = ShareBadDrawsstar;
                  logObjFcn = logObjFcnstar;
                  DlogObjFcn = DlogObjFcnstar;
                  %
                  % what is the tolerance value in the optimization. check what is used by Hoogerheide & co
                  %
                  ConvValue = (RelCoV<CoVTolerance);
                  %
                  SetLogText(strvcat(['   Best CoV value obtained for share alpha:  ' num2str(alphastar)],['   Coefficient of Variation:                 ' num2str(CoV)],['   Relative change coefficient of variation: ' num2str(RelCoV)],['   Value of the scaled objective function:   ' num2str(logObjFcn)],['   Relative change of scaled obj. function:  ' num2str(DlogObjFcn,'%0.8f')],'   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDraws)],'   Saving draws and weights from optimized density',['   Component file for H=' int2str(h) ':'],' ',['      ' GetFilename(PosteriorDrawsFile)],['      ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],' '));
                  save(PosteriorDrawsFile,'phiS','LogPost','ISWeights','UnISWeights','tHeights','rhoMatrix','mu','Sigma','df','piweights','CoV','ConvValue','RelCoV','logObjFcn','DlogObjFcn','hRepeat','alphastar','ShareBadDraws','NumStudTimes');
                  LastPosteriorDrawsFile = PosteriorDrawsFile;
                  %
                  % check if we are done
                  %
                  if (UseObjFcnCriterion==1)&(DlogObjFcn<=ToleranceValue);
                     ConvValue = 1;
                  end;
               else;
                  %
                  % the change of the objective function was negative
                  %
                  h = h-1;
                  ConvValue = 1;
                  SetLogText(strvcat(['   Best CoV value obtained for share alpha:  ' num2str(alphastar)],['   Value of the scaled objective function:   ' num2str(logObjFcnstar)],['   Relative change of scaled obj. function:  ' num2str(DlogObjFcnstar,'%0.8f')],['   => The candidate density for H=' int2str(h) ' is preferred to H=' int2str(h+1)],' '));
               end;
            else;
               %
               % current mixture did not give any useful results
               %
               h = h-1;
               ConvValue = 1;
               SetLogText(strvcat(['   Unable to compute proper candidate density for H=' int2str(h+1)],' '));
            end;
         end;
         drawnow;
         pause(0.01);
      end;
   else;
      ConvValue = 2;
   end;
   %
   % we're done so record the total computational time
   %
   if ConvValue==1;
      SetLogText(strvcat(['Algorithm has converged for H=' int2str(h) '!'],' '));
      %
      % now we can compute all the posterior draws and weights with the selected mixture of Student-t distributions
      %
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\is\IS-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(NChains) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
      if (DSGEModel.OverwriteDraws==1)|(FileExist(PostSampleFile)==0);
         %
         % load parameters
         %
         PosteriorDrawsData = load(LastPosteriorDrawsFile);
         %
         % loading the candidate draws, weights and two other matrices
         % due to the adaptive density and used by MitISEM
         %
         PostSample = PosteriorDrawsData.phiS';
         ISWeights = PosteriorDrawsData.ISWeights;
         UnISWeights = PosteriorDrawsData.UnISWeights;
         Weights = ISWeights;
         LogPost = PosteriorDrawsData.LogPost;
         tHeights = PosteriorDrawsData.tHeights;
         rhoMatrix = PosteriorDrawsData.rhoMatrix;
         mu = PosteriorDrawsData.mu;
         Sigma = PosteriorDrawsData.Sigma;
         df = PosteriorDrawsData.df;
         hRepeat = PosteriorDrawsData.hRepeat;
         piweights = PosteriorDrawsData.piweights;
         alphastar = PosteriorDrawsData.alphastar;
         ShareBadDraws = PosteriorDrawsData.ShareBadDraws;
         h = length(df);
         CoV = PosteriorDrawsData.CoV;
         RelCoV = PosteriorDrawsData.RelCoV;
         logObjFcn = PosteriorDrawsData.logObjFcn;
         DlogObjFcn = PosteriorDrawsData.DlogObjFcn;
         %
         % need to transform all PostSample draws into thetaPostSample
         %
         SetLogText('   Calculating original parameters from transformed');
         thetaPostSample = zeros(NumPostDraws,NumParam);
         for i=1:NumPostDraws;
            thetaPostSample(i,:) = PhiToTheta(PostSample(i,:)',thetaIndex,UniformBounds,LowerBound)';
            drawnow;
         end;
         %
         BatchPostSample = PostSample;
         BatchthetaPostSample = thetaPostSample;
         BatchLogPost = LogPost;
         BatchAcceptedDraws = (1:NumPostDraws)';
         %
         % compute the marginal likelihood value
         %
         wbar = (1/NumPostDraws)*sum(UnISWeights);
         LogMargLikeIS = log(wbar);
         %
         % and now the asymptotic variance
         %
         wadj = UnISWeights-wbar;
         taubarN = (1/NumPostDraws)*wadj'*wadj;
         AsVarLogMargLikeIS = taubarN/(wbar^2);
         %
         % AcceptedDraws vector should also be saved with AcceptedDraws=(1:NumPostDraws)';
         %
         SetLogText(strvcat('   Saving draws and weights from optimized density',['   Component file for H=' int2str(h) ':'],' ',['      ' GetFilename(PostSampleFile)],['      ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],' '));
         save(PostSampleFile,'PostSample','thetaPostSample','LogPost','Weights','ISWeights','UnISWeights','LogMargLikeIS','AsVarLogMargLikeIS','tHeights','rhoMatrix','mu','Sigma','df','piweights','CoV','RelCoV','ConvValue','logObjFcn','DlogObjFcn','hRepeat','alphastar','ShareBadDraws','BatchPostSample','BatchthetaPostSample','BatchLogPost','BatchAcceptedDraws');
      else;
         %
         % load the PostSampleFile to extract a few parameters for display
         %
         PosteriorDrawsData = load(PostSampleFile);
         df = PosteriorDrawsData.df;
         alphastar = PosteriorDrawsData.alphastar;
         ShareBadDraws = PosteriorDrawsData.ShareBadDraws;
         h = length(df);
         CoV = PosteriorDrawsData.CoV;
         RelCoV = PosteriorDrawsData.RelCoV;
         logObjFcn = PosteriorDrawsData.logObjFcn;
         DlogObjFcn = PosteriorDrawsData.DlogObjFcn;
         %
         SetLogText(strvcat(['   Posterior draws already exists with optimal H=' int2str(h)],['   Best CoV value obtained for share alpha:  ' num2str(alphastar)],'   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDraws)],['   Coefficient of Variation:                 ' num2str(CoV)],['   Relative change coefficient of variation: ' num2str(RelCoV)],['   Value of the scaled objective function:   ' num2str(logObjFcn)],['   Relative change of scaled obj. function:  ' num2str(DlogObjFcn,'%0.8f')],' '));
      end;
   elseif ConvValue==2;
      %
      % load the PostSampleFile to extract a few parameters for display
      %
      PosteriorDrawsData = load(PostSampleFile);
      df = PosteriorDrawsData.df;
      alphastar = PosteriorDrawsData.alphastar;
      ShareBadDraws = PosteriorDrawsData.ShareBadDraws;
      h = length(df);
      CoV = PosteriorDrawsData.CoV;
      RelCoV = PosteriorDrawsData.RelCoV;
      logObjFcn = PosteriorDrawsData.logObjFcn;
      DlogObjFcn = PosteriorDrawsData.DlogObjFcn;
      %
      SetLogText(strvcat(['   Posterior draws already exists with optimal H=' int2str(h)],['   Best CoV value obtained for share alpha:  ' num2str(alphastar)],'   Share of draws with a non-unique or a',['                non-convergent solution:     ' num2str(ShareBadDraws)],['   Coefficient of Variation:                 ' num2str(CoV)],['   Relative change coefficient of variation: ' num2str(RelCoV)],['   Value of the scaled objective function:   ' num2str(logObjFcn)],['   Relative change of scaled obj. function:  ' num2str(DlogObjFcn,'%0.8f')],' '));
   else;
      if StopEstimation==0;
         SetLogText(strvcat('Algorithm has not converged.',' '));
      end;
   end;
   PostSampleTime = toc;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
   if ConvValue>0;
      if NumChains==1;
         SetLogText(strvcat('Posterior sampling with importance sampling based on MitISEM completed!',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Posterior sampling time:            ' SecondsToHoursStr(PostSampleTime) ' (hrs:min:sec)']));
      elseif NChains<NumChains;
         SetLogText(strvcat(' ',['Importance sampling draws sample no. ' int2str(NChains) ' completed.'],[' * Time:                                  ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Posterior sampling time:               ' SecondsToHoursStr(PostSampleTime) ' (hrs:min:sec)'],' '));
      else;
         SetLogText(strvcat(' ',['Importance sampling draws sample no. ' int2str(NChains) ' completed.'],[' * Time:                                  ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Posterior sampling time:               ' SecondsToHoursStr(PostSampleTime) ' (hrs:min:sec)']));
      end;
   else;
      abort = 'cancel';
      break;
   end;
end;
%
% clean up the tmp directory for AiM related error data.
%
if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
   delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
end;
%
if strcmp(abort,'cancel')==1;
   return;
end;
%
% check if we have just run 1 chain. if so display certain information.
%
for CurrChain=1:NumChains;
   %
   % make sure that we load the correct results
   %
   if NumChains>1;
      %
      % load the data
      %
      SetLogText(strvcat(' ','Reloading posterior draws data...'));
      if exist('PostSample')==1;
         %
         % make sure the workspace is cleared from the last chain draws before loading the draws
         % from the first sample
         %
         clear('PostSample','thetaPostSample','LogPost','AcceptedDraws','Weights');
      end;
      [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
      clear('AcceptedDraws');
      %
      % temporarily remove sequential estimation
      %
      DSGEModel.SequentialML = 0;
      SetLogText(strvcat('Computing summary information about the parameters...',[' * Sample number:                      ' num2str(CurrChain,'%0.0f')],[' * Time:                               ' StringTrim(datestr(now,14))]));
   else;
      NumBurnin = 0;
      PosteriorDrawsData = load(PostSampleFile);
      PostSample = PosteriorDrawsData.PostSample;
      thetaPostSample = PosteriorDrawsData.thetaPostSample;
      LogPost = PosteriorDrawsData.LogPost;
      Weights = PosteriorDrawsData.Weights;
      clear('PosteriorDrawsData');
      SetLogText(strvcat(' ','Computing summary information about the parameters...',[' * Time:                               ' StringTrim(datestr(now,14))]));
   end;
   NumAcceptedDraws = NumPostDraws;
   %
   % compute summary information about the parameter draws for the original parameters
   %
   tic;
   ParData = ParameterSummary(thetaPostSample,Weights);
   ParTime = toc;
   SetLogText(strvcat('Computation completed!',[' * Estimation time:                    ' SecondsToHoursStr(ParTime) ' (hrs:min:sec)']));
   %
   % check if we have parameters to update
   %
   if (FileExist(DSGEModel.UpdateParameterFile)==1)&(CalcUpdatePosterior==1);
      tic;
      SetLogText(strvcat(' ','Preparing posterior values for parameters to update...',[' * Time:                               ' StringTrim(datestr(now,14))]));
      [UpdatePostSample,UpdateMode,UpdateNames] = AdaptiveLearningGetUpdatePosteriorDraws(thetaPostSample,NumBurnin,CurrChain,NumChains,0,'Marginal',DSGEModel,CurrINI);
      SetLogText(strvcat('Computing summary information...',[' * Time:                               ' StringTrim(datestr(now,14))]));
      UpdateParData = ParameterSummary(UpdatePostSample,Weights);
      UpParTime = toc;
      SetLogText(strvcat('Computation completed!',[' * Total preparation time:             ' SecondsToHoursStr(UpParTime) ' (hrs:min:sec)']));
   else;
      UpdatePostSample = [];
      UpdateParData = [];
      UpdateMode = [];
      UpdateNames = '';
   end;
   %
   % try to estimate the posterior mode of the marginal distribution.
   %
   if DSGEModel.PosteriorDensityValue==1;
      ModeAlgorithm = ' * Algorithm:                          Gaussian';
   elseif DSGEModel.PosteriorDensityValue==2;
      ModeAlgorithm = strvcat(' * Algorithm:                          Silverman-type and','                                       Skld-Roberts correction');
   elseif DSGEModel.PosteriorDensityValue==3;
      ModeAlgorithm = ' * Algorithm:                          Sheather-Jones bandwidth';
   elseif DSGEModel.PosteriorDensityValue==4;
      ModeAlgorithm = ' * Algorithm:                          Bump killing bandwidth';
   end;
   SetLogText(strvcat(' ','Computing the posterior mode of the marginal distribution...',ModeAlgorithm,[' * Time:                               ' StringTrim(datestr(now,14))]));
   tic;
   [thetaMargMode,UpdateMargMode] = AdaptiveLearningGraphPosteriorDensity(thetaPostSample,NumBurnin,CurrChain,UpdatePostSample,UpdateMode,UpdateNames,[],[],[],DSGEModel,CurrINI,controls);
   ModeTime = toc;
   if isempty(thetaMargMode)==1;
      thetaMargMode = thetaMode;
      UpdateMargMode = UpdateMode;
      SetLogText(strvcat('Estimation aborted.',' * Using the posterior mode of the joint distribution.'));
   else;
      SetLogText(strvcat('Estimation completed!',[' * Estimation time:                    ' SecondsToHoursStr(ModeTime) ' (hrs:min:sec)']));
   end;
   %
   % clear original parameter draws from memory
   %
   clear('thetaPostSample','UpdatePostSample','PostSample','LogPost');
   %
   % save parameter data to file
   %
   MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\is\MargLikeData-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   save(MargLikeFile,'ParData','thetaMargMode','UpdateParData','UpdateMargMode');
   %
   % load marginal likelihood estimates
   %
   PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\is\IS-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   PosteriorDrawsData = load(PostSampleFile);
   ParData.LogMargLikeIS = PosteriorDrawsData.LogMargLikeIS;
   ParData.StErrLogMargLikeIS = sqrt(PosteriorDrawsData.AsVarLogMargLikeIS/NumPostDraws);
   ParData.CoV = PosteriorDrawsData.CoV;
   ParData.alphastar = PosteriorDrawsData.alphastar;
   ParData.RelCoV = PosteriorDrawsData.RelCoV;
   ParData.CoVTolerance = CoVTolerance;
   ParData.logObjFcn = PosteriorDrawsData.logObjFcn;
   ParData.DlogObjFcn = PosteriorDrawsData.DlogObjFcn;
   ParData.ConvValue = PosteriorDrawsData.ConvValue;
   ParData.ShareBadDraws = PosteriorDrawsData.ShareBadDraws;
   ParData.df = PosteriorDrawsData.df;
   ParData.piweights = PosteriorDrawsData.piweights;
   ParData.mu = PosteriorDrawsData.mu;
   ParData.Sigma = PosteriorDrawsData.Sigma;
   ParData.Hstar = Hstar;
   ParData.ISInitialDF = DSGEModel.ISInitialDF;
   ParData.alphaStr = alphaStr;
   ParData.piH1 = piH1;
   ParData.dfH1 = dfH1;
   ParData.UseObjFcnCriterion = UseObjFcnCriterion;
   ParData.ToleranceValue = ToleranceValue;
   %
   % write sample statistics to file
   %
   DSGEModel.BeliefSys = BeliefSys;
   PostSampleTextFile = FixFilePath([DSGEModel.OutputDirectory '\ISPosteriorSamplingSummary-AdaptiveLearning-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
   SetLogText(strvcat(' ','Writing posterior sampling summary results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: ' GetFilename(PostSampleTextFile)]));
   PrintPosteriorSamplingSummary(PostSampleTextFile,DSGEModel,CurrINI,ParameterNames,PriorDist,thetaDist,ParData,thetaMargMode,UpdateParData,UpdateMargMode,UpdateNames,NumPostDraws,NumBurnin,rank(R),[],[],PostSampleTime,NumAcceptedDraws,[],[],[],[],[],[],[],[],[],StepLength,CurrChain,NumChains,thetaInit,[],MaxCorr,IsSystemPrior,LogMargLikeSystemLaplace,SPLogMargsMH,SPCovProb,SPNWStdErrMH,1);
   %
   % display the results
   %
   if (isempty(CurrINI.Sequence)==1)&(CurrChain==NumChains);
      TextGUI(PostSampleTextFile,['Posterior Sampling - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
   elseif (CurrChain==NumChains);
      SetLogText(' ');
   end;
end;

%
% end of AdaptiveLearningDSGEISMitISEMPosteriorSampler.m
%
