function [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL)
% AdaptiveLearningExtendPriorFields: Extends variables related to the prior distribution of the DSGE model with additional
%                                    data obtained from the adaptive learning version of the model.
%
% USAGE:
%
%       [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] ...
%                         = AdaptiveLearningExtendPriorFields(theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,...
%                                                             PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,...
%                                                             LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,...
%                                                             ParameterNamesAL,UniformBoundsAL)
%
% REQUIRED INPUT:  theta (vector) with all parameters to be estimated
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, Gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  ModelParameters (structure) with fields having the names of the inidividual parameters as they are
%                             specified under the "model parameter" header in the file with prior distribution data,
%                             and having values as specified under the header "initial value" in the same file
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                             of the model parameter in position i of theta.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                             "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each
%                             field returns a string matrix with the parameter names.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                             for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%                  thetaAL (vector) with all parameters to be estimated from the adaptive learning system.
%
%                  thetaIndexAL (vector) with integers where 1=gamma, invgamma; 2=beta, 3=uniform. This is used
%                             to determine the type of transformation that may need to be performed for the
%                             theta parameters.
%
%                  thetaDistAL (vector) with integers 1 if gamma, 2 if beta, 4 if invgamma, and 6 if uniform.
%
%                  LowerBoundAL (vector) with lower bound values for the theta vector.
%
%                  ModelParametersAL (structure) with fields having the names of the inidividual parameters as they are
%                             specified under the "model parameter" header in the file with prior distribution data,
%                             and having values as specified under the header "initial value" in the same file
%
%                  thetaPositionsAL (structure) where thetaPositions(i).parameter is a string vector giving the name
%                             of the model parameter in position i of theta.
%
%                  PriorDistAL (structure) with fields "beta", "gamma", "invgamma", and "uniform",, where the values are
%                             matrices with the parameter values for the distributions. In the case of the beta and gamma
%                             these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the
%                             uniform the parameter values are the lower and the upper bound. 
%
%                  ParameterNamesAL (structure) with fields "calibrated", "beta", "gamma", "invgamma", "uniform", "all" and
%                             "estimated". Each field returns a string matrix with the parameter names. Notice that exactly
%                             three parameter names may be used "rho-al", "sigmar-al" and "sigmae-al", corresponding to the
%                             parameters used in the Kalman filter equations of the adaptive learning system.
%
%                  UniformBoundsAL (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                             for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
% REQUIRED OUTPUT: theta (vector) with all parameters to be estimated
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, Gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  ModelParameters (structure) with fields having the names of the inidividual parameters as they are
%                             specified under the "model parameter" header in the file with prior distribution data,
%                             and having values as specified under the header "initial value" in the same file
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                             of the model parameter in position i of theta.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                             "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each
%                             field returns a string matrix with the parameter names.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                             for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 11, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-03-2020: Updated the documentation.
%
% * 17-04-2020: Added 11 missing fields to PriorDist including the "_ab" parts.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% start with the theta, thetaIndex thetaDist and LowerBound vectors
%
p = length(theta);
pAL = length(thetaAL);
theta = [theta;thetaAL];
thetaIndex = [thetaIndex;thetaIndexAL];
thetaDist = [thetaDist;thetaDistAL];
LowerBound = [LowerBound;LowerBoundAL];
%
% now the UniformBounds matrix
%
UniformBounds = [UniformBounds;UniformBoundsAL];
%
% update the ModelParameters structure
%
ALNames = fieldnames(ModelParametersAL);
for j=1:length(ALNames);
   ModelParameters = setfield(ModelParameters,ALNames{j},getfield(ModelParametersAL,ALNames{j}));
end;
%
% deal with PriorDist
%
PriorDist.beta = [PriorDist.beta;PriorDistAL.beta];
PriorDist.gamma = [PriorDist.gamma;PriorDistAL.gamma];
PriorDist.invgamma = [PriorDist.invgamma;PriorDistAL.invgamma];
PriorDist.uniform = [PriorDist.uniform;PriorDistAL.uniform];
%PriorDist.normal = [PriorDist.normal;PriorDistAL.normal];
%PriorDist.truncnormal = [PriorDist.truncnormal;PriorDistAL.truncnormal];
%PriorDist.student = [PriorDist.student;PriorDistAL.student];
%PriorDist.cauchy = [PriorDist.cauchy;PriorDistAL.cauchy];
%PriorDist.logistic = [PriorDist.logistic;PriorDistAL.logistic];
%PriorDist.gumbel = [PriorDist.gumbel;PriorDistAL.gumbel];
%PriorDist.pareto = [PriorDist.pareto;PriorDistAL.pareto];
PriorDist.beta_ab = [PriorDist.beta_ab;PriorDistAL.beta_ab];
PriorDist.gamma_ab = [PriorDist.gamma_ab;PriorDistAL.gamma_ab];
%PriorDist.logistic_ab = [PriorDist.logistic_ab;PriorDistAL.logistic_ab];
%PriorDist.gumbel_ab = [PriorDist.gumbel_ab;PriorDistAL.gumbel_ab];
%
% update the thetaPositions structure
%
for j=1:pAL;
   thetaPositions(p+j).parameter = StringTrim(ParameterNamesAL.estimated(j,:));
end;
%
% finally, deal with all fields of ParameterNames
%
ParameterNames.calibrated = strvcat(ParameterNames.calibrated,ParameterNamesAL.calibrated);
ParameterNames.all = strvcat(ParameterNames.all,ParameterNamesAL.all);
ParameterNames.beta = strvcat(ParameterNames.beta,ParameterNamesAL.beta);
ParameterNames.gamma = strvcat(ParameterNames.gamma,ParameterNamesAL.gamma);
ParameterNames.uniform = strvcat(ParameterNames.uniform,ParameterNamesAL.uniform);
ParameterNames.invgamma = strvcat(ParameterNames.invgamma,ParameterNamesAL.invgamma);
ParameterNames.estimated = strvcat(ParameterNames.estimated,ParameterNamesAL.estimated);

%
% end of AdaptiveLearningExtendPriorFields.m
%
