function AdaptiveLearningIRGraphDLG(selector,CurrINI,DSGEModel,IRStructure,NameStr,UseLevels)
% AdaptiveLearningIRGraphDLG: Displays impulse response functions based on the adaptive learning solutions
%                             and the rational expectations solution.
%
% USAGE:
%
%       AdaptiveLearningIRGraphDLG(selector,CurrINI,DSGEModel,IRStructure,NameStr,UseLevels)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 IRStructure (structure) with fields RE and AL of dimensions 1 and T respectively
%                           with matrices Ksi (rxqxh+1) and Y (nxqxh+1). The responses of the
%                           invidual variables are located in the rows, while the columns give the shocks.
%                           The third dimension is the response horizon.
%
%                 NameStr (string vector) taking on the values "Initial Values", "DSGE Posterior Mode"
%                           and "Posterior Mode".
%
%                 UseLevels (integer) which indicates if variables in first differences should be
%                           accumulated (1), annualized (2), or left alone (0).
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 12, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-05-2020: Finished the function.
%
% * 14-05-2020: Updated the documentation.
%
% * 15-04-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 16-03-2022: The function now also plots the steady-state IRFs under adaptive learning.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.IRStructure = IRStructure;
   DSGEModel.NameStr = NameStr;
   DSGEModel.UseLevels = UseLevels;
else
   IRGraphGUI = findobj('Type','figure','Tag','AdaptiveLearningIRGraphDLG');
   IRcontrol = get(IRGraphGUI,'UserData');
   CurrINI = get(IRcontrol.shock,'UserData');
   DSGEModel = get(IRcontrol.shocktext,'UserData');
end;
%
switch selector;
   case 'init';
      IRGraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','IRGraphDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-600)/2 (CurrINI.scrsz(4)-150)/2 600 170], ...
         'Tag','AdaptiveLearningIRGraphDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Impulse Responses - Adaptive Learning - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','AdaptiveLearningIRGraphDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(IRGraphGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 598 160],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      IRcontrol.sample = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 110 260 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String',['Impulse responses from year 0 to ' int2str(size(IRStructure.RE.Y,3)-1)], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         set(IRcontrol.sample,'String',strrep(get(IRcontrol.sample,'String'),'year','quarter'));
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         set(IRcontrol.sample,'String',strrep(get(IRcontrol.sample,'String'),'year','month'));
      end;
      %
      % select variable
      %
      IRcontrol.variabletext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 80 176 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');
      %
      IRcontrol.variable = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 85 265 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',strvcat(DSGEModel.VariableNames,DSGEModel.StateVariableNames), ...
         'HorizontalAlignment','center', ...
         'Tag','VariablePopup',...
         'Value',1);
      %
      % Select shock
      %
      IRcontrol.shocktext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 176 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select shock:', ...
         'HorizontalAlignment','left', ...
         'Tag','ShockText');
      %
      ShockAliases = DSGEModel.ShockAliases;
      for j=1:size(ShockAliases,1);
         ShockAliases(j,:) = strrep(ShockAliases(j,:),'_',' ');
      end;
      %
      IRcontrol.shock = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 55 265 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',ShockAliases, ...
         'HorizontalAlignment','center', ...
         'Tag','ShocksPopup',...
         'Value',1);
      %
      IRcontrol.surfplot = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 20 300 20], ...
         'Style','checkbox', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String','Use surface plot instead of 3D lines', ...
         'HorizontalAlignment','center', ...
         'Tag','SurfacePlotCheckbox',...
         'Value',0);
      %
      % Create graph button
      %
      IRcontrol.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','AdaptiveLearningIRGraphDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      IRcontrol.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','AdaptiveLearningIRGraphDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(IRGraphGUI,'UserData',IRcontrol,...
         'HandleVisibility','callback');
   case 'showgraph'
      ShockValue = get(IRcontrol.shock,'Value');
      ShockNames = get(IRcontrol.shock,'String');
      CurrShock = StringTrim(ShockNames(ShockValue,:));
      VariableValue = get(IRcontrol.variable,'Value');
      VariableNames = get(IRcontrol.variable,'String');
      CurrVariable = StringTrim(VariableNames(VariableValue,:));
      if ShockValue<size(ShockNames,1);
         set(IRcontrol.shock,'Value',ShockValue+1);
      else;
         set(IRcontrol.shock,'Value',1);
         if VariableValue<size(VariableNames,1);
            set(IRcontrol.variable,'Value',VariableValue+1);
         else;
            set(IRcontrol.variable,'Value',1);
         end;
      end;
      %
      IRStructure = DSGEModel.IRStructure;
      n = DSGEModel.n;
      h = size(IRStructure.RE.Y,3)-1;
      T = length(IRStructure.AL);
      %
      % vector with the individual response horizons
      %
      Horizon = (0:1:h)';
      %
      % create a vector with dates
      %
      DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
      DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
      yAL = zeros(h+1,T);
      if VariableValue<=n;
         %
         % yRE is h+1 x 1
         %
         yRE = squeeze(IRStructure.RE.Y(VariableValue,ShockValue,:));
         yALSS = squeeze(IRStructure.ALSS.Y(VariableValue,ShockValue,:));
         for t=1:T;
            yAL(:,t) = squeeze(IRStructure.AL(t).Y(VariableValue,ShockValue,:));
         end;
      else;
         yRE = squeeze(IRStructure.RE.Ksi(VariableValue-n,ShockValue,:));
         yALSS = squeeze(IRStructure.ALSS.Ksi(VariableValue-n,ShockValue,:));
         for t=1:T;
            yAL(:,t) = squeeze(IRStructure.AL(t).Ksi(VariableValue-n,ShockValue,:));
         end;
      end;
      HorizonMat = Horizon*ones(1,T);
      DateMat = ones(h+1,1)*DateVector;
      %
      % Setting up the figure
      %
      FigHandle = figure('Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'Tag',[CurrVariable '-' CurrShock], ...
         'Name',['Impulse Responses For ' CurrVariable ' From Shock To ' CurrShock ' - ' DSGEModel.NameStr]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      if get(IRcontrol.surfplot,'Value')==1;
         %
         % we use the jet colormap
         %
         colormap(jet);
         %
         SurfHandle = surf(HorizonMat,DateMat,yAL,'EdgeColor',0.5*[1 1 1],'LineWidth',0.1);
      else;
         %
         % we use the default colormap
         %
         SurfHandle = plot3(HorizonMat,DateMat,yAL,'-','LineWidth',0.1);
      end;
      %
      hold('on');
      %
      % fix the axes properties
      %
      set(gca,'XGrid','on','YGrid','on','ZGrid','on','YDir','reverse');
      %
      % plot the steady-state adaptive learning IRFs
      %
      LineHandleSS = plot3(Horizon,(DateVector(T)+0.25)*ones(h+1,1),yALSS,'--','LineWidth',1.5,'Color',[0.5 0 0]);
      %
      % plot the rational expectations
      %
      LineHandle = plot3(Horizon,(DateVector(T)+0.25)*ones(h+1,1),yRE,'-','LineWidth',1.5,'Color',[0 0 0]);
      %
      if DSGEModel.UseLevels==0;
         SetAxesFonts(CurrINI,'Title',['Original Resp(' CurrVariable ') from shock ' CurrShock],'XLabel','Horizon','Ylabel','Dates','ZLabel','');
      elseif DSGEModel.UseLevels==1;
         SetAxesFonts(CurrINI,'Title',['Levels Resp(' CurrVariable ') from shock ' CurrShock],'XLabel','Horizon','Ylabel','Dates','ZLabel','');
      elseif DSGEModel.UseLevels==2;
         SetAxesFonts(CurrINI,'Title',['Annualized Resp(' CurrVariable ') from shock ' CurrShock],'XLabel','Horizon','Ylabel','Dates','ZLabel','');
      end;
      %
      % add a legend
      %
      if MatlabNumber>=7;
         LegHandle = legend([SurfHandle(1,1) LineHandleSS LineHandle],strvcat('Adaptive learning','AL (steady-state)','Rational expectations'),'Location','best');
      else;
         LegHandle = legend([SurfHandle(1,1) LineHandleSS LineHandle],strvcat('Adaptive learning','AL (steady-state)','Rational expectations'),0);
      end;
      %
      % Fix the toolbar
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(IRcontrol.done,'UserData','done');
      delete(IRGraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(IRGraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(IRGraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(IRcontrol.done);
   end;
   waitfor(IRcontrol.done,'UserData','done');
end;

%
% end of AdaptiveLearningIRGraphDLG.m
%
