function [PredEventData,PredictionEvent,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = LoadAdaptiveLearningDSGEPredictionUncertainty(OutputDirectory,CondStr,ShockMethod,NameOfModel,FixedBeliefsForProj,MaxPH,NumDraws,NumPredPaths,AnnualizeData,TransData,CurrChain,IsPosterior)
% LoadAdaptiveLearningDSGEPredictionUncertainty: Loads the results for the prediction uncertainty decomposition for a DSGE model
%                                                based on adaptive learning from disk. It also loads any data it finds about
%                                                prediction events.
%
% USAGE:
%
%       [PredEventData,PredictionEvent,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = ...
%                      LoadAdaptiveLearningDSGEPredictionUncertainty(OutputDirectory,CondStr,ShockMethod, ...
%                              NameOfModel,FixedBeliefsForProj,MaxPH,NumDraws,NumPredPaths,AnnualizeData, ...
%                              TransData,CurrChain,IsPosterior)
%
% REQUIRED INPUT:  OutputDirectory (string vector) with the full path to the base output directory.
%
%                  CondStr (string vector) that is either "Cond" or "".
%
%                  ShockMethod (string vector) that is either "Direct-", "WaggonerZha-", or "".
%
%                  NameOfModel (string vector) with the name of the model.
%
%                  FixedBeliefsForProj (boolean) that takes the value 1 if fixed beliefs for the projections are used
%                                      and 0 otherwise.
%
%                  MaxPH (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPredPaths (integer) with tht total number of prediction paths per parameter draw.
%
%                  AnnualizeData (boolean) that take the value 1 if annualized results should be checked and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number of the currently used Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                      if draws from the prior are supplied.
%
% REQUIRED OUTPUT: PredEventData (n x 8) matrix where the first column holds the number of times
%                                      the PredPaths data satisfies the event, the second holds the number
%                                      of times the PredPaths data is greater than the event, the third
%                                      the number of time the data is above the upper bound, while the
%                                      fourth column holds the number of times the event has been tested.
%                                      The fifth and sixth column measure the average deviation from the
%                                      lower bound when below the lower bound and the square of this, while
%                                      the seventh and eighth columns hold similar data for the upper bound
%                                      of the event when the variables are above the upper bound.
%
%                  PredictionEvent (n x 3) matrix with the prediction event information. The first
%                                      column holds the upper bound, the second the lower bound for the event,
%                                      while the third holds the event length in periods.
%
%                  ShockMean (q x MaxPH) matrix with the mean of the shocks under conditioning.
%
%                  StateCov (n x n x MaxPH) matrix with the estimated state uncertainty.
%
%                  ShockNames (q x qstr) string matrix with the names of the economic shocks.
%
%                  ShockCov (n x n x MaxPH) matrix with the estimated shock uncertainty.
%
%                  MeasureCov (n x n) matrix the the estimated measurement error uncertainty.
%
%                  ParameterCov (n x n x MaxPH) matrix with the estimated parameter uncertainty.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 25, 2021.
%                        This version: March 13, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-05-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 13-03-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
PredEventData = [];
PredictionEvent = [];
StateCov = [];
ShockCov = [];
MeasureCov = [];
ParameterCov = [];
ShockMean = [];
ShockNames = '';
if IsPosterior==1;
   if AnnualizeData==0;
      if TransData==0;
         file = FixFilePath([OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(FixedBeliefsForProj) '-' CondStr 'Prediction-Original-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumDraws,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
      else;
         file = FixFilePath([OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(FixedBeliefsForProj) '-' CondStr 'Prediction-Transformed-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumDraws,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
      end;
   else;
      file = FixFilePath([OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(FixedBeliefsForProj) '-' CondStr 'Prediction-Annual-' ShockMethod NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumDraws,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
   end;
else;
   if AnnualizeData==0;
      if TransData==0;
         file = FixFilePath([OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(FixedBeliefsForProj) '-' CondStr 'Prediction-Original-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(NumDraws,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
      else;
         file = FixFilePath([OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(FixedBeliefsForProj) '-' CondStr 'Prediction-Transformed-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(NumDraws,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
      end;
   else;
      file = FixFilePath([OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(FixedBeliefsForProj) '-' CondStr 'Prediction-Annual-Prior-' ShockMethod NameOfModel '-' num2str(MaxPH,'%0.0f') '-' num2str(NumDraws,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
   end;
end;
if FileExist(file)==1;
   PredData = load(file);
   if isfield(PredData,'StateCov')==1;
      StateCov = PredData.StateCov;
      ShockCov = PredData.ShockCov;
      MeasureCov = PredData.MeasureCov;
      ParameterCov = PredData.ParameterCov;
   end;
   if isfield(PredData,'PredEventData')==1;
      if size(PredData.PredEventData,2)==8;
         PredEventData = PredData.PredEventData;
      end;
   end;
   if isfield(PredData,'PredictionEvent')==1;
      PredictionEvent = PredData.PredictionEvent;
   end;
   if isfield(PredData,'ShockMean')==1;
      ShockMean = PredData.ShockMean;
   end;
   if isfield(PredData,'ShockNames')==1;
      ShockNames = PredData.ShockNames;
   end;
end;

%
% end of LoadAdaptiveLearningDSGEPredictionUncertainty.m
%
