function status = VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrentChain,controls)
% VerifyAdaptiveLearningDSGEPosteriorDraws: checks if all files exists for loading the posterior draws data for a given sampling chain
%                                           of a DSGE model based on adaptive learning instead of rational expectations.
%
% USAGE:
%
%       status = VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrentChain,controls)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrentChain (integer) the value of the parallel chain to load data from.
%
% OPTIONAL INPUT:  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% REQUIRED OUTPUT: status (boolean), which is 1 is all files exists and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 27, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-05-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 1;
%
% locate the controls structure
%
if nargin<3;
   maingui = findobj('Type','figure','Tag','YADA');
   controls = get(maingui,'UserData');
end;
%
% Get the base values number of draws, etc.
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of posterior sample batches to save per chain
%
if DSGEModel.PosteriorSampler<=7;
   BatchStr = get(controls.posterior.batch,'String');
   NumBatches = str2num(StringTrim(strrep(BatchStr(DSGEModel.SampleBatchValue,:),',','')));
else;
   NumBatches = 1;
end;
if NumBatches>NumPostDraws;
   NumBatches = NumPostDraws;
end;
if IsInteger(NumPostDraws/NumBatches)==0;
   NumBatches = 1;
end;
%
% 3. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% number of tempering stages
%
TempStagesStr = get(controls.posterior.temperingstages,'String');
TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
%
for NBatch=1:NumBatches;
   if DSGEModel.PosteriorSampler==1;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==2;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\slice\Slice-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==3;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-t-AL-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==4;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-AL-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==5;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-AL-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==6;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-AL-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==7;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-AL-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '.' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==8;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-AL-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==9;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==10;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\is\IS-AL-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   end;
   if FileExist(PostSampleFile)==0;
      status = 0;
      break;
   end;
end;

%
% end of VerifyAdaptiveLearningDSGEPosteriorDraws.m
%
