function BeliefSys = YADADefaultBeliefSystem(X,XVariableNames,StateVariableNames,ForwardVariableNames)
% YADADefaultBeliefSystem: Sets up a default belief system where each forward looking variable
%                          is explained by two own lags and a constant if such a variable is
%                          available.
%
% USAGE:
%
%       BeliefSys = YADADefaultBeliefSystem(X,XVariableNames,StateVariableNames,ForwardVariableNames)
%
% REQUIRED INPUT:  X (matrix) of dimension k x T with the deterministic variables (if any).
%
%                  XVariableNames (string matrix) of dimension k x k1 with the names of the deterministic variables.
%
%                  StateVariableNames (string matrix) of dimension r x r1 with the names of the state variables.
%
%                  ForwardVariableNames (string matrix) of dimension f x f1 with the names of the forward
%                                       looking variables.
%
% REQUIRED OUTPUT: BeliefSys (structure) with fields "m" an integer denoting the maximum number of lags of the state
%                                       variables in the belief equations; "Name" a string with the  name of the
%                                       belief system; and G a cell array with f matrices, one for each forward
%                                       looking variable of dimension (k+r*m) x g_j where g_j is the number of
%                                       explanatory variables in the belief equation for forward looking variable j.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 12, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-03-2020; Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 02-02-2021: Switched from using the LocateStr to loc function to ensure an exact
%               match.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
m = 2;
BeliefSys.m = m;
BeliefSys.Name = 'Default';
BeliefSys.G = [];
%
% first we check if we have a constant
%
[k,T] = size(X);
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      for j=1:k;
         ConstTest = (X(j,:)==ones(1,T));
         if sum(ConstTest)==k2;
            ConstPos = j;
            break;
         end;
      end;
   end;
end;
r = size(StateVariableNames,1);
f = size(ForwardVariableNames,1);
G = cell(f,1);
Ghelp = zeros(k+(r*m),3);
if ConstPos>0;
   Ghelp(ConstPos,1) = 1;
end;
for j=1:f;
   Gj = Ghelp;
   CurrForward = StringTrim(ForwardVariableNames(j,:));
   Gj(MatchStrRow(StateVariableNames,CurrForward)+k,2) = 1;
   Gj(MatchStrRow(StateVariableNames,CurrForward)+k+r,3) = 1;
   G{j,1} = Gj;
end;
%
% complete the output structure
%
BeliefSys.G = G;

%
% end of YADADefaultBeliefSystem.m
%
