function G = ZLBBeliefSystem(G,m,k,r,RtildePosition,ZLBRuleType)
% ZLBBeliefSystem: Rewrites the belief equations defined through G such that the coefficients on the
%                  unrestricted policy rate in the original equations appear last among the lagged
%                  variables for each time period, while the restricted policy rate is left out from
%                  the belief equations.
%
% USAGE:
%
%       G = ZLBBeliefSystem(G,m,k,r,RtildePosition)
%
% REQUIRED INPUT:  G (cell array) with f matrices selecting the "explanatory" variables for each one
%                    of the f forward looking variables.
%
%                  m (integer) with the number of lags in the belief equations.
%
%                  k (integer) with the number of possible deterministic variables in the belief
%                    equations.
%
%                  r (integer) with the number of original state/model variables.
%
%                  RtildePosition (integer) with the position of the monetary policy rate among the state variables.
%
%                  ZLBRuleType (integer) which is 1 (default) if the policy rule can include the lagged
%                    unrestricted policy rate and 2 if it can include the restricted policy rate instead.
%                    When rewriting the Gj matrices, YADA allows the lagged unrestricted policy rate to
%                    enter the PLM when the variable is 1, and the lagged restricted policy rate when the
%                    variable is 2.
%
% REQUIRED OUTPUT: G (cell array) with f matrices selecting the "explanatory" variables for each one
%                    of the f forward looking variables. Each matrix takes the increased number of state/model
%                    variables into account.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 28, 2024.
%                        This version: February 29, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-02-2024: Added the "ZLBRuleType" input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

f = size(G,1);
e_pr = zeros(r,1);
e_pr(RtildePosition) = 1;
Ip = eye(r);
Kpr = Ip(:,setdiff((1:r),RtildePosition));
GNew = cell(f,1);
if ZLBRuleType==1;
   for j=1:f;
      Gj = G{j};
      gj = size(Gj,2);
      GNewj = zeros(k+((r+1)*m),gj);
      if k>0;
         GNewj(1:k,:) = Gj(1:k,:);
      end;
      Gj1 = Gj(k+1:k+r,:);
      GNewj(k+1:k+r+1,:) = [Gj1'*Kpr*Kpr' Gj1'*e_pr]';
      if m==2;
         Gj2 = Gj(k+r+1:k+(2*r),:);
         GNewj(k+r+2:k+(2*(r+1)),:) = [Gj2'*Kpr*Kpr' Gj2'*e_pr]';
      end;
      GNew{j} = GNewj;
   end;
else;
   for j=1:f;
      Gj = G{j};
      gj = size(Gj,2);
      GNewj = zeros(k+((r+1)*m),gj);
      if k>0;
         GNewj(1:k,:) = Gj(1:k,:);
      end;
      Gj1 = Gj(k+1:k+r,:);
      GNewj(k+1:k+r+1,:) = [Gj1' zeros(gj,1)]';
      if m==2;
         Gj2 = Gj(k+r+1:k+(2*r),:);
         GNewj(k+r+2:k+(2*(r+1)),:) = [Gj2' zeros(gj,1)]';
      end;
      GNew{j} = GNewj;
   end;
end;
G = GNew;

%
% end of ZLBBeliefSystem.m
%
