function [DoneCalc,PredEventData,StateCov,ShockCov,MeasureCov,ParameterCov] = AdaptiveLearningDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,h,NumPaths,AnnualizeData,TransData,CurrChain,IsPosterior,Weights)
% AdaptiveLearningDSGEPredictionPaths: Calculates a certain number of possible paths that the endogenous variables
%                                      can take out-of-sample for a given sample of draws from the posterior of the
%                                      parameters in the DSGE model under adaptive learning.
%
% USAGE:
%
%       [DoneCalc,PredEventData,StateCov,ShockCov,MeasureCov,ParameterCov] = ...
%                     AdaptiveLearningDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,...
%                                           CurrINI,X,h,NumPaths,AnnualizeData,TransData,CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                                  distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) with the posterior mode or initial value of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxMaxPH with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  h (integer) with the prediction horizon.
%
%                  NumPaths (integer) the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                  likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                  the PredPaths data satisfies the event, the second hold the number
%                                  of times the PredPaths data is greater than the event, the third
%                                  the number of time the data is above the upper bound, while the
%                                  fourth column holds the number of times the event has been tested.
%                                  The fifth and sixth column measure the average deviation from the
%                                  lower bound when below the lower bound and the square of this, while
%                                  the seventh and eighth columns hold similar data for the upper bound
%                                  of the event when the variables are above the upper bound.
%
%                  StateCov (n x n x h) matrix with the estimated state uncertainty.
%
%                  ShockCov (n x n x h) matrix with the estimated shock uncertainty.
%
%                  MeasureCov (n x n) matrix the the estimated measurement error uncertainty.
%
%                  ParameterCov (n x n x h) matrix with the estimated parameter uncertainty.
%
%                  
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 20, 2021.
%                        This version: March 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <https://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <https://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-05-2021: Finished the function from the beginning of the path computations.
%
% * 25-05-2021: Took filename change into account.
%
% * 03-01-2022: Updated the documentation.
%
% * 16-02-2022: Began work on parallel computing code.
%
% * 17-02-2022: Finished the parallel computations code.
%
% * 21-02-2022: Changed eval call to feval.
%
% * 06-01-2023: Updated the documentation.
%
% * 24-10-2023: The function now supports the fixed belief Kalman filter.
%
% * 28-02-2024: Updated the documentation.
%
% * 05-03-2024: Fixed an issue with updating the progress dialog. Added smoothed measurement errors
%               to YHistPred.
%
% * 11-03-2024: Added mean calculations at the mode value and checks to make sure the predictions are
%               reasonable.
%
% * 12-03-2024: Made sure that uncertainty is calculated using the correct parameters.
%
% * 13-03-2024: The logical function "CheckAdaptiveLearningFixedBeliefs" is now only called once.
%
% * 19-03-2024: Made sure that LastSolution is reset when needed.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
[NumDraws,NumParam] = size(thetaPostSample);
k = DSGEModel.k;
n = DSGEModel.n;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
DoneCalc = 0;
PredEventData = [];
StateCov = zeros(n,n,h);
ShockCov = StateCov;
MeasureCov = zeros(n,n);
ParameterCov = StateCov;
MeanPredictY = zeros(n,h);
if (AnnualizeData==1)|(TransData==1);
   if TransData==0;
      TypeStr = 'Annualized';
   else;
      TypeStr = 'Transformed';
   end;
else;
   TypeStr = 'Original';
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of predictive distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of predictive distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the unconditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
FirstPeriod = DSGEModel.FirstPeriod;
LastPeriod = DSGEModel.LastPeriod;
%
% Determine the belief system matrices
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
m = BeliefSys.m;
q = size(B0,2);
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
T = LastPeriod-(FirstPeriod+KalmanFirstObservation-1)+1;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      Xhist = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(Xhist,2);
      for j=1:k;
         ConstTest = (Xhist==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% call the transformation function
%
[TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% check if the adaptive learning parameters allows for updating of the beliefs
%
IsALUpdate = (CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0);
%
% run the Kalman filter
%
if IsALUpdate==1;
   if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
      [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
   else;
      [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
end;
%
% determine initial conditions for the forecasts
%
KsiTT = Ksitt(:,T);
PTT = Ptt(:,:,T);
PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
%
% the last estimates
%
MT1 = Mtilde(:,:,T);
FT1 = Ftilde(:,:,T);
BT1 = Btilde(:,:,T);
PredictYmode = zeros(n,h);
%
% initialize solution matrices for possible later use
%
MTh = zeros(r,k,h);
FTh = zeros(r,r,h);
BTh = zeros(r,q,h);
%
% deal with deterministic variables
%
if k>0;
   Ax = A'*X;
else;
   Ax = zeros(n,h);
   X = zeros(r,h)
end;
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % forecast with fixed beliefs
      %
      KsiTiT = KsiTT;
      for i=1:h;
         KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
         PredictYmode(:,i) = (Ax(:,i))+(H'*KsiTiT);
      end;
   else;
      %
      % forecast with updated beliefs
      %
      KsiTiT = KsiTT;
      MTiT = MT1;
      FTiT = FT1;
      BTiT = BT1;
      for i=1:h;
         KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
         PredictYmode(:,i) = (Ax(:,i))+(H'*KsiTiT);
         %
         % store the used solution matrices
         %
         MTh(:,:,i) = MTiT;
         FTh(:,:,i) = FTiT;
         BTh(:,:,i) = BTiT;
         %
         % update beliefs
         %
         if i==1;
            MTiT = LastSolution.MTT;
            FTiT = LastSolution.FTT;
            BTiT = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
         else;
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         end;
      end;
   end;
else;
   % 
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % forecast with fixed beliefs
      %
      KsiTiT = KsiTT;
      for i=1:h;
         KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
         PredictYmode(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
      end;
   else;
      %
      % forecast with updated beliefs
      %
      KsiTiT = KsiTT;
      MTiT = MT1;
      FTiT = FT1;
      BTiT = BT1;
      for i=1:h;
         KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
         PredictYmode(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
         %
         % store the used solution matrices
         %
         MTh(:,:,i) = MTiT;
         FTh(:,:,i) = FTiT;
         BTh(:,:,i) = BTiT;
         %
         % update beliefs
         %
         if i==1;
            MTiT = LastSolution.MTT;
            FTiT = LastSolution.FTT;
            BTiT = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
         else;
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         end;
      end;
   end;
end;
%
% store mode results for use if required
%
Ksitt1mode = Ksitt1;
Ptt1mode = Ptt1;
Yerrormode = Yerror;
InvMSEYmode = InvMSEY;
Hmode = H;
Rmode = R;
Amode = A;
Ftildemode = Ftilde;
Btildemode = Btilde;
KsiTTmode = KsiTT;
LastSolutionmode = LastSolution;
%
% use the Schur decomposition for the reduced rank matrix PTT instead of
% the Choleski decomposition
%
[Smode,Lmode] = SchurDecomposition(PTT);
BT1mode = BT1;
MT1mode = MT1;
FT1mode = FT1;
BThmode = BTh;
MThmode = MTh;
FThmode = FTh;
%
% set up the data matrices
%
%Y = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   Xdata = [];
end;
if strcmp(DSGEModel.DataFrequency,'q')==1;
   Yinit = DSGEModel.Y(:,LastPeriod-2:LastPeriod);
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   Yinit = DSGEModel.Y(:,LastPeriod-10:LastPeriod);
else;
   Yinit = [];
end;
maxYinit = max(max(isnan(Yinit)));
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(n,8);
end;
%
if TransData==1;
   Yinitial = DSGEModel.Y(:,1:DSGEModel.LastPeriod);
   %
   % try to covert: h
   %
   YTmp = [Yinitial DSGEModel.Y(:,DSGEModel.LastPeriod)*ones(1,h)];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   maxYinitial = max(max(isnan(Yinitial)));
else;
   maxYinitial = [];
   Tt = [];
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should use parallel computations or not
%
if CurrINI.NumWorkers<2;
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' num2str(NumDraws*NumPaths,'%0.0f') ' Prediction Paths'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Path Number:';
      %
      ProgressStructure.name = ['Predictive Distributions' PriorHeader];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   else;
      txt = ['Please wait while YADA computes predictive distributions under adaptive learning for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',['Predictive Distributions ' PriorHeader ' - ' TypeStr ' Data'],600,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % begin the loop
   %
   abort = '';
   StopEstimation = 0;
   if length(size(H))==2;
      for ParamIndex=1:NumDraws;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation function
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         end;
         %
         % determine initial conditions for the forecasts
         %
         KsiTT = Ksitt(:,T);
         PTT = Ptt(:,:,T);
         PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
         %
         % use the Schur decomposition for the reduced rank matrix PTT instead of
         % the Choleski decomposition
         %
         [S,L] = SchurDecomposition(PTT);
         if isreal(sqrt(L))==0;
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
            [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
            [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
            [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
            else;
               [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            if IsALUpdate==1;
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
               else;
                  [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            end;
            KsiTT = Ksitt(:,T);
            PTT = Ptt(:,:,T);
            PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
            [S,L] = SchurDecomposition(PTT);
         end;
         %
         % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
         %
         if rank(R)>0;
            [Rm,Lm] = SchurDecomposition(R);
            Rmeasure = Rm*sqrt(Lm);
            NumMeasure = size(Lm,1);
         else;
            Rmeasure = [];
            wTh = 0;
            NumMeasure = 0;
         end;
         %
         % now we can start with the moments
         %
         MeasureCov = MeasureCov+((1/NumDraws)*R*Weights(ParamIndex));
         PredictY = zeros(n,h);
         Fi = eye(r);
         FjSum = zeros(r,r);
         MT1 = Mtilde(:,:,T);
         FT1 = Ftilde(:,:,T);
         BT1 = Btilde(:,:,T);
         %
         % deal with deterministic variables
         %
         if k>0;
            Ax = A'*X;
         else;
            Ax = zeros(n,h);
            X = zeros(r,h)
         end;
         %
         % check beliefs and compute prediction
         %
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % forecast with fixed beliefs
            %
%            Q = BT1*BT1';
            KsiTiT = KsiTT;
            for i=1:h;
%               FjSum = FjSum+(Fi*Q*Fi');
%               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*FjSum*H);
               %
               % update Fi
               %
%               Fi = FT1*Fi;
%               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fi*PTT*Fi'*H);
               KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
               PredictY(:,i) = (Ax(:,i))+(H'*KsiTiT);
%               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
            end;
         else;
            %
            % forecast with updated beliefs
            %
            KsiTiT = KsiTT;
            MTiT = MT1;
            FTiT = FT1;
            BTiT = BT1;
            %
            % initialize solution matrices to be used later
            %
%            MTh = zeros(r,k,h);
%            FTh = zeros(r,r,h);
%            BTh = zeros(r,q,h);
            for i=1:h;
%               FjSum = FjSum+(Fi*BTiT*BTiT'*Fi');
%               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*FjSum*H);
               %
               % update Fi
               %
%               Fi = FTiT*Fi;
%               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fi*PTT*Fi'*H);
               KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
               PredictY(:,i) = (Ax(:,i))+(H'*KsiTiT);
%               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
               %
               % store the used solution matrices
               %
%               MTh(:,:,i) = MTiT;
%               FTh(:,:,i) = FTiT;
%               BTh(:,:,i) = BTiT;
               %
               % update beliefs
               %
               if i==1;
                  MTiT = LastSolution.MTT;
                  FTiT = LastSolution.FTT;
                  BTiT = LastSolution.BTT;
                  betaTiT = LastSolution.betaTT;
               elseif i<h;
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
               end;
            end;
         end;
         TestValue = max(max(abs((PredictY-PredictYmode)./PredictYmode)));
         if TestValue<100;
            MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredictY);
         else;
            %
            % we need to use the appropriate entries for the mode values
            %
            PredictY = PredictYmode;
            Ksitt1 = Ksitt1mode;
            Ptt1 = Ptt1mode;
            Yerror = Yerrormode;
            InvMSEY = InvMSEYmode;
            H = Hmode;
            R = Rmode;
            Ftilde = Ftildemode;
            Btilde = Btildemode;
            A = Amode;
            KsiTT = KsiTTmode;
            S = Smode;
            L = Lmode;
            BT1 = BT1mode;
            MT1 = MT1mode;
            FT1 = FT1mode;
            BTh = BThmode;
            MTh = MThmode;
            FTh = FThmode;
            LastSolution = LastSolutionmode;
            MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredictY);
         end;
         %
         % check beliefs and compute uncertainty
         %
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % forecast with fixed beliefs
            %
            Q = BT1*BT1';
%            KsiTiT = KsiTT;
            for i=1:h;
               FjSum = FjSum+(Fi*Q*Fi');
               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*FjSum*H);
               %
               % update Fi
               %
               Fi = FT1*Fi;
               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fi*PTT*Fi'*H);
%               KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
%               PredictY(:,i) = (Ax(:,i))+(H'*KsiTiT);
               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
            end;
         else;
            %
            % forecast with updated beliefs
            %
%            KsiTiT = KsiTT;
            MTiT = MT1;
            FTiT = FT1;
            BTiT = BT1;
            %
            % initialize solution matrices to be used later
            %
            MTh = zeros(r,k,h);
            FTh = zeros(r,r,h);
            BTh = zeros(r,q,h);
            for i=1:h;
               FjSum = FjSum+(Fi*BTiT*BTiT'*Fi');
               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*FjSum*H);
               %
               % update Fi
               %
               Fi = FTiT*Fi;
               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fi*PTT*Fi'*H);
%               KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
%               PredictY(:,i) = (Ax(:,i))+(H'*KsiTiT);
               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
               %
               % store the used solution matrices
               %
               MTh(:,:,i) = MTiT;
               FTh(:,:,i) = FTiT;
               BTh(:,:,i) = BTiT;
               %
               % update beliefs
               %
               if i==1;
                  MTiT = LastSolution.MTT;
                  FTiT = LastSolution.FTT;
                  BTiT = LastSolution.BTT;
                  betaTiT = LastSolution.betaTT;
               elseif i<h;
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
               end;
            end;
         end;
         %
         if AnnualizeData==1;
            if maxYinit==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect annualization
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               if k>0;
                  YHistPred = (A'*Xdata)+(H'*KsitT);
               else;
                  YHistPred = H'*KsitT;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
               else;
                  Yinit = [];
               end;
            end;
         elseif TransData==1;
            if maxYinitial==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect general transformations
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               if k>0;
                  YHistPred = (A'*Xdata)+(H'*KsitT);
               else;
                  YHistPred = H'*KsitT;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               Yinitial = YSmooth(:,1:LastPeriod);
            end;
         end;
         if DSGEModel.AdjustPredictionPaths==1;
            %
            % compute the mean prediction at the given parameter values
            %
            PredMean = PredictY;
            %
            % annualize the mean predictions when needed
            %
            if AnnualizeData==1;
               PredMean = Annualize([Yinit PredMean],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               YTmp = [Yinitial PredMean];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredMean = YData(Tt-h+1:Tt,:)';
            end;
         else;
            PredMean = [];
         end;
         %
         % we are now ready to compute possible paths at theta
         %
         PredPaths = zeros(n,h,NumPaths);
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % we have fixed beliefs
            %
            for it=1:NumPaths;
               CurrPath = it+(NumPaths*(ParamIndex-1));
               %
               % check if we should update the progress dialog
               %
               if DSGEModel.ShowProgress==1;
                  abort = get(WaitHandle,'UserData');
                  if strcmp(abort,'cancel')==1;
                     StopEstimation = 1;
                     break;
                  else;
                     ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
                  end;
               end;
               %
               % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % Draw a sequence of state shocks
               %
               vTh = BT1*randn(q,h);
               %
               % calculate a path for the states
               %
               KsiTh = zeros(r,h);
               KsiTh(:,1) = (MT1*X(:,1))+(FT1*KsiT)+vTh(:,1);
               for i=2:h;
                  KsiTh(:,i) = (MT1*X(:,i))+(FT1*KsiTh(:,i-1))+vTh(:,i);
               end;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
               end;
               %
               % add everything
               %
               if AnnualizeData==0;
                  PredPaths(:,:,it) = Ax+(H'*KsiTh)+wTh;
                  if TransData==1;
                     YTmp = [Yinitial PredPaths(:,:,it)];
                     [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                     PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
                  end;
               else;
                  PredPaths(:,:,it) = Annualize([Yinit Ax+(H'*KsiTh)+wTh],DSGEModel.annual,DSGEModel.annualscale);
               end;
               drawnow;
            end;      
         else;
            %
            % we have updated beliefs
            %
            for it=1:NumPaths;
               CurrPath = it+(NumPaths*(ParamIndex-1));
               %
               % check if we should update the progress dialog
               %
               if DSGEModel.ShowProgress==1;
                  abort = get(WaitHandle,'UserData');
                  if strcmp(abort,'cancel')==1;
                     StopEstimation = 1;
                     break;
                  else;
                     ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
                  end;
               end;
               %
               % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % Draw state shocks at T+1
               %
               vTh = BTh(:,:,1)*randn(q,1);
               %
               % calculate a path for the states
               %
               KsiTh = zeros(r,h);
               KsiTh(:,1) = (MTh(:,:,1)*X(:,1))+(FTh(:,:,1)*KsiT)+vTh;
               for i=2:h;
                  vTh = BTh(:,:,i)*randn(q,1);
                  KsiTh(:,i) = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*KsiTh(:,i-1))+vTh;
               end;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
               end;
               %
               % add everything
               %
               if AnnualizeData==0;
                  PredPaths(:,:,it) = Ax+(H'*KsiTh)+wTh;
                  if TransData==1;
                     YTmp = [Yinitial PredPaths(:,:,it)];
                     [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                     PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
                  end;
               else;
                  PredPaths(:,:,it) = Annualize([Yinit Ax+(H'*KsiTh)+wTh],DSGEModel.annual,DSGEModel.annualscale);
               end;
               drawnow;
            end;
         end;
         if StopEstimation==1;
            break;
         end;
         %
         if DSGEModel.AdjustPredictionPaths==1;
            %
            % we need to fix the paths such that the sample mean is equal to the population mean.
            %
            MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
            AdjustMean = PredMean-MeanPrediction;
            for i=1:NumPaths;
               PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
            end;
         end;
         %
         % save the calculations to disk
         %
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         end;
         save(file,'PredPaths');
         if DSGEModel.RunPredictionEvent==1;
            %
            % compute the prediction events results
            %
            PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
         end;
      end;
   else;
      %
      % time-varying H matrix
      %
      for ParamIndex=1:NumDraws;
%         if DSGEModel.ShowProgress==1;
%            abort = get(WaitHandle,'UserData');
%            if strcmp(abort,'cancel')==1;
%               StopEstimation = 1;
%               break;
%            else;
%               ProgressDLG([ParamIndex/NumDraws ParamIndex]);
%            end;
%         end;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation fucntion
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         end;
         %
         % determine initial conditions for the forecasts
         %
         KsiTT = Ksitt(:,T);
         PTT = Ptt(:,:,T);
         PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
         %
         % use the Schur decomposition for the reduced rank matrix PTT instead of
         % the Choleski decomposition
         %
         [S,L] = SchurDecomposition(PTT);
         if isreal(sqrt(L))==0;
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
            [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
            [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
            [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
            else;
               [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            if IsALUpdate==1;
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
               else;
                  [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            end;
            KsiTT = Ksitt(:,T);
            PTT = Ptt(:,:,T);
            PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
            [S,L] = SchurDecomposition(PTT);
         end;
         %
         % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
         %
         if rank(R)>0;
            [Rm,Lm] = SchurDecomposition(R);
            Rmeasure = Rm*sqrt(Lm);
            NumMeasure = size(Lm,1);
         else;
            Rmeasure = [];
            wTh = 0;
            NumMeasure = 0;
         end;
         %
         % now we can start with the moments
         %
         MeasureCov = MeasureCov+((1/NumDraws)*R*Weights(ParamIndex));
         PredictY = zeros(n,h);
         Fi = eye(r);
         FjSum = zeros(r,r);
         MT1 = Mtilde(:,:,T);
         FT1 = Ftilde(:,:,T);
         BT1 = Btilde(:,:,T);
         %
         % deal with deterministic variables
         %
         if k>0;
            Ax = A'*X;
         else;
            Ax = zeros(n,h);
            X = zeros(r,h)
         end;
         %
         % check beliefs and compute prediction
         %
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % forecast with fixed beliefs
            %
%            Q = BT1*BT1';
            KsiTiT = KsiTT;
            for i=1:h;
%               FjSum = FjSum+(Fi*Q*Fi');
%               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*FjSum*H(:,:,LastPeriod+i));
               %
               % update Fi
               %
%               Fi = FT1*Fi;
%               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,LastPeriod+i));
               KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
               PredictY(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
%               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
            end;
         else;
            %
            % forecast with updated beliefs
            %
            KsiTiT = KsiTT;
            MTiT = MT1;
            FTiT = FT1;
            BTiT = BT1;
            %
            % initialize solution matrices to be used later
            %
%            MTh = zeros(r,k,h);
%            FTh = zeros(r,r,h);
%            BTh = zeros(r,q,h);
            for i=1:h;
%               FjSum = FjSum+(Fi*BTiT*BTiT'*Fi');
%               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*FjSum*H(:,:,LastPeriod+i));
               %
               % update Fi
               %
%               Fi = FTiT*Fi;
%               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,LastPeriod+i));
               KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
               PredictY(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
%               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
               %
               % store the used solution matrices
               %
%               MTh(:,:,i) = MTiT;
%               FTh(:,:,i) = FTiT;
%               BTh(:,:,i) = BTiT;
               %
               % update beliefs
               %
               if i==1;
                  MTiT = LastSolution.MTT;
                  FTiT = LastSolution.FTT;
                  BTiT = LastSolution.BTT;
                  betaTiT = LastSolution.betaTT;
               elseif i<h;
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
               end;
            end;
         end;
         TestValue = max(max(abs((PredictY-PredictYmode)./PredictYmode)));
         if TestValue<100;
            MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredictY);
         else;
            %
            % we need to use the appropriate entries for the mode values
            %
            PredictY = PredictYmode;
            Ksitt1 = Ksitt1mode;
            Ptt1 = Ptt1mode;
            Yerror = Yerrormode;
            InvMSEY = InvMSEYmode;
            H = Hmode;
            R = Rmode;
            Ftilde = Ftildemode;
            Btilde = Btildemode;
            A = Amode;
            KsiTT = KsiTTmode;
            S = Smode;
            L = Lmode;
            BT1 = BT1mode;
            MT1 = MT1mode;
            FT1 = FT1mode;
            BTh = BThmode;
            MTh = MThmode;
            FTh = FThmode;
            LastSolution = LastSolutionmode;
            MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredictY);
         end;
         %
         % check beliefs and compute uncertainty
         %
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % forecast with fixed beliefs
            %
            Q = BT1*BT1';
%            KsiTiT = KsiTT;
            for i=1:h;
               FjSum = FjSum+(Fi*Q*Fi');
               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*FjSum*H(:,:,LastPeriod+i));
               %
               % update Fi
               %
               Fi = FT1*Fi;
               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,LastPeriod+i));
%               KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
%               PredictY(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
            end;
         else;
            %
            % forecast with updated beliefs
            %
%            KsiTiT = KsiTT;
            MTiT = MT1;
            FTiT = FT1;
            BTiT = BT1;
            %
            % initialize solution matrices to be used later
            %
            MTh = zeros(r,k,h);
            FTh = zeros(r,r,h);
            BTh = zeros(r,q,h);
            for i=1:h;
               FjSum = FjSum+(Fi*BTiT*BTiT'*Fi');
               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*FjSum*H(:,:,LastPeriod+i));
               %
               % update Fi
               %
               Fi = FTiT*Fi;
               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,LastPeriod+i));
%               KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
%               PredictY(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
               %
               % store the used solution matrices
               %
               MTh(:,:,i) = MTiT;
               FTh(:,:,i) = FTiT;
               BTh(:,:,i) = BTiT;
               %
               % update beliefs
               %
               if i==1;
                  MTiT = LastSolution.MTT;
                  FTiT = LastSolution.FTT;
                  BTiT = LastSolution.BTT;
                  betaTiT = LastSolution.betaTT;
               elseif i<h;
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
               end;
            end;
         end;
         if AnnualizeData==1;
            if maxYinit==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect annualization
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               YHistPred = zeros(n,T);
               if k>0;
                  for t=1:T;
                     YHistPred = (A'*Xdata(:,t))+(H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t));
                  end;
               else;
                  for t=1:T;
                     YHistPred = H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t);
                  end;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
               else;
                  Yinit = [];
               end;
            end;
         elseif TransData==1;
            if maxYinitial==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect general transformations
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               YHistPred = zeros(n,T);
               if k>0;
                  for t=1:T;
                     YHistPred = (A'*Xdata(:,t))+(H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t));
                  end;
               else;
                  for t=1:T;
                     YHistPred = H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t);
                  end;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               Yinitial = YSmooth(:,1:LastPeriod);
            end;
         end;
         if DSGEModel.AdjustPredictionPaths==1;
            %
            % compute the mean prediction at the given parameter values
            %
            PredMean = PredictY;
            %
            % annualize the mean predictions when needed
            %
            if AnnualizeData==1;
               PredMean = Annualize([Yinit PredMean],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               YTmp = [Yinitial PredMean];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredMean = YData(Tt-h+1:Tt,:)';
            end;
         else;
            PredMean = [];
         end;
         %
         % we are now ready to compute possible paths at theta
         %
         PredPaths = zeros(n,h,NumPaths);
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % we have fixed beliefs
            %
            for it=1:NumPaths;
               CurrPath = it+(NumPaths*(ParamIndex-1));
               %
               % check if we should update the progress dialog
               %
               if DSGEModel.ShowProgress==1;
                  abort = get(WaitHandle,'UserData');
                  if strcmp(abort,'cancel')==1;
                     StopEstimation = 1;
                     break;
                  else;
                     ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
                  end;
               end;
               %
               % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % Draw a sequence of state shocks
               %
               vTh = BT1*randn(q,h);
               %
               % calculate a path for the states
               %
               KsiTh = zeros(r,h);
               KsiTh(:,1) = (MT1*X(:,1))+(FT1*KsiT)+vTh(:,1);
               for i=2:h;
                  KsiTh(:,i) = (MT1*X(:,i))+(FT1*KsiTh(:,i-1))+vTh(:,i);
               end;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
               end;
               %
               % add everything
               %
               if AnnualizeData==0;
                  for i=1:h;
                     PredPaths(:,i,it) = Ax(:,i)+(H(:,:,LastPeriod+i)'*KsiTh(:,i))+wTh(:,i);
                  end;
                  if TransData==1;
                     YTmp = [Yinitial PredPaths(:,:,it)];
                     [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                     PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
                  end;
               else;
                  PredPaths(:,:,it) = Annualize([Yinit Ax+(H'*KsiTh)+wTh],DSGEModel.annual,DSGEModel.annualscale);
               end;
               drawnow;
            end;      
         else;
            %
            % we have updated beliefs
            %
            for it=1:NumPaths;
               CurrPath = it+(NumPaths*(ParamIndex-1));
               %
               % check if we should update the progress dialog
               %
               if DSGEModel.ShowProgress==1;
                  abort = get(WaitHandle,'UserData');
                  if strcmp(abort,'cancel')==1;
                     StopEstimation = 1;
                     break;
                  else;
                     ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
                  end;
               end;
               %
               % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % Draw state shocks at T+1
               %
               vTh = BTh(:,:,1)*randn(q,1);
               %
               % calculate a path for the states
               %
               KsiTh = zeros(r,h);
               KsiTh(:,1) = (MTh(:,:,1)*X(:,1))+(FTh(:,:,1)*KsiT)+vTh;
               for i=2:h;
                  vTh = BTh(:,:,i)*randn(q,1);
                  KsiTh(:,i) = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*KsiTh(:,i-1))+vTh;
               end;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
               end;
               %
               % add everything
               %
               if AnnualizeData==0;
                  for i=1:h;
                     PredPaths(:,i,it) = Ax(:,i)+(H(:,:,LastPeriod+i)'*KsiTh(:,i))+wTh(:,i);
                  end;
                  if TransData==1;
                     YTmp = [Yinitial PredPaths(:,:,it)];
                     [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                     PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
                  end;
               else;
                  PredPaths(:,:,it) = Annualize([Yinit Ax+(H'*KsiTh)+wTh],DSGEModel.annual,DSGEModel.annualscale);
               end;
               drawnow;
            end;
         end;
         if StopEstimation==1;
            break;
         end;
         %
         if DSGEModel.AdjustPredictionPaths==1;
            %
            % we need to fix the paths such that the sample mean is equal to the population mean.
            %
            MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
            AdjustMean = PredMean-MeanPrediction;
            for i=1:NumPaths;
               PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
            end;
         end;
         %
         % save the calculations to disk
         %
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         end;
         save(file,'PredPaths');
         if DSGEModel.RunPredictionEvent==1;
            %
            % compute the prediction events results
            %
            PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
         end;
      end;
   end;
   %
   % close the wait dialog
   %
   if DSGEModel.ShowProgress==1;
      if IsHandle(WaitHandle)==1;
         set(WaitHandle,'UserData','done');
         close(WaitHandle);
         drawnow;
         pause(0.02);
      end;
   else;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   %
   % we can do parallel computations over the parameter draws
   %
   txt = ['Please wait while YADA computes predictive distributions under adaptive learning for the ' lower(TypeStr) ' data using parallel computations with ' int2str(CurrINI.NumWorkers) ' workers. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Predictive Distributions ' PriorHeader ' - ' TypeStr ' Data'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   % begin the loop
   %
   abort = '';
   StopEstimation = 0;
   ErrMsg = '';
   status = 1;
   MP = ModelParameters;
   GT = Gtilde;
   Kg = Kgf;
   Xhelp = X;
   if length(size(H))==2;
      parfor ParamIndex=1:NumDraws;
         %
         % initialization
         %
         ModelParameters = MP;
         Gtilde = GT;
         Kgf = Kg;
         X = Xhelp;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation fucntion
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         end;
         %
         % determine initial conditions for the forecasts
         %
         KsiTT = Ksitt(:,T);
         PTT = Ptt(:,:,T);
         PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
         %
         % use the Schur decomposition for the reduced rank matrix PTT instead of
         % the Choleski decomposition
         %
         [S,L] = SchurDecomposition(PTT);
         if isreal(sqrt(L))==0;
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
            [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
            [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
            [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
            else;
               [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            if IsALUpdate==1;
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
               else;
                  [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            end;
            KsiTT = Ksitt(:,T);
            PTT = Ptt(:,:,T);
            PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
            [S,L] = SchurDecomposition(PTT);
         end;
         %
         % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
         %
         if rank(R)>0;
            [Rm,Lm] = SchurDecomposition(R);
            Rmeasure = Rm*sqrt(Lm);
            NumMeasure = size(Lm,1);
         else;
            Rmeasure = [];
            wTh = 0;
            NumMeasure = 0;
         end;
         %
         % now we can start with the moments
         %
         MeasureCov = (1/NumDraws)*R*Weights(ParamIndex);
         PredictY = zeros(n,h);
         Fi = eye(r);
         FjSum = zeros(r,r);
         MT1 = Mtilde(:,:,T);
         FT1 = Ftilde(:,:,T);
         BT1 = Btilde(:,:,T);
         PredictY = zeros(n,h);
         StateCov = zeros(n,n,h);
         ShockCov = StateCov;
         ParameterCov = StateCov;
         MeanPredictY = zeros(n,h);
         %
         % deal with deterministic variables
         %
         if k>0;
            Ax = A'*X;
         else;
            Ax = zeros(n,h);
            X = zeros(r,h)
         end;
         %
         % check beliefs and compute prediction
         %
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % forecast with fixed beliefs
            %
%            Q = BT1*BT1';
            KsiTiT = KsiTT;
            for i=1:h;
%               FjSum = FjSum+(Fi*Q*Fi');
%               ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H'*FjSum*H;
               %
               % update Fi
               %
%               Fi = FT1*Fi;
%               StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H'*Fi*PTT*Fi'*H;
               KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
               PredictY(:,i) = (Ax(:,i))+(H'*KsiTiT);
%               ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)';
            end;
         else;
            %
            % forecast with updated beliefs
            %
            KsiTiT = KsiTT;
            MTiT = MT1;
            FTiT = FT1;
            BTiT = BT1;
            %
            % initialize solution matrices to be used later
            %
%            MTh = zeros(r,k,h);
%            FTh = zeros(r,r,h);
%            BTh = zeros(r,q,h);
            for i=1:h;
%               FjSum = FjSum+(Fi*BTiT*BTiT'*Fi');
%               ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H'*FjSum*H;
               %
               % update Fi
               %
%               Fi = FTiT*Fi;
%               StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H'*Fi*PTT*Fi'*H;
               KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
               PredictY(:,i) = (Ax(:,i))+(H'*KsiTiT);
%               ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)';
               %
               % store the used solution matrices
               %
%               MTh(:,:,i) = MTiT;
%               FTh(:,:,i) = FTiT;
%               BTh(:,:,i) = BTiT;
               %
               % update beliefs
               %
               if i==1;
                  MTiT = LastSolution.MTT;
                  FTiT = LastSolution.FTT;
                  BTiT = LastSolution.BTT;
                  betaTiT = LastSolution.betaTT;
               elseif i<h;
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
               end;
            end;
         end;
         TestValue = max(max(abs((PredictY-PredictYmode)./PredictYmode)));
         if TestValue<100;
            MeanPredictY = (Weights(ParamIndex)/NumDraws)*PredictY;
         else;
            %
            % we need to use the appropriate entries for the mode values
            %
            PredictY = PredictYmode;
            Ksitt1 = Ksitt1mode;
            Ptt1 = Ptt1mode;
            Yerror = Yerrormode;
            InvMSEY = InvMSEYmode;
            H = Hmode;
            R = Rmode;
            Ftilde = Ftildemode;
            Btilde = Btildemode;
            A = Amode;
            KsiTT = KsiTTmode;
            S = Smode;
            L = Lmode;
            BT1 = BT1mode;
            MT1 = MT1mode;
            FT1 = FT1mode;
            BTh = BThmode;
            MTh = MThmode;
            FTh = FThmode;
            LastSolution = LastSolutionmode;
            MeanPredictY = (Weights(ParamIndex)/NumDraws)*PredictY;
         end;
         %
         % check beliefs and compute uncertainty
         %
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % forecast with fixed beliefs
            %
            Q = BT1*BT1';
%            KsiTiT = KsiTT;
            for i=1:h;
               FjSum = FjSum+(Fi*Q*Fi');
               ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H'*FjSum*H;
               %
               % update Fi
               %
               Fi = FT1*Fi;
               StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H'*Fi*PTT*Fi'*H;
%               KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
%               PredictY(:,i) = (Ax(:,i))+(H'*KsiTiT);
               ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)';
            end;
         else;
            %
            % forecast with updated beliefs
            %
%            KsiTiT = KsiTT;
            MTiT = MT1;
            FTiT = FT1;
            BTiT = BT1;
            %
            % initialize solution matrices to be used later
            %
            MTh = zeros(r,k,h);
            FTh = zeros(r,r,h);
            BTh = zeros(r,q,h);
            for i=1:h;
               FjSum = FjSum+(Fi*BTiT*BTiT'*Fi');
               ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H'*FjSum*H;
               %
               % update Fi
               %
               Fi = FTiT*Fi;
               StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H'*Fi*PTT*Fi'*H;
%               KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
%               PredictY(:,i) = (Ax(:,i))+(H'*KsiTiT);
               ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)';
               %
               % store the used solution matrices
               %
               MTh(:,:,i) = MTiT;
               FTh(:,:,i) = FTiT;
               BTh(:,:,i) = BTiT;
               %
               % update beliefs
               %
               if i==1;
                  MTiT = LastSolution.MTT;
                  FTiT = LastSolution.FTT;
                  BTiT = LastSolution.BTT;
                  betaTiT = LastSolution.betaTT;
               elseif i<h;
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
               end;
            end;
         end;
         %
         if AnnualizeData==1;
            if maxYinit==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect annualization
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               if k>0;
                  YHistPred = (A'*Xdata)+(H'*KsitT);
               else;
                  YHistPred = H'*KsitT;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
               else;
                  Yinit = [];
               end;
            end;
         elseif TransData==1;
            if maxYinitial==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect general transformations
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               if k>0;
                  YHistPred = (A'*Xdata)+(H'*KsitT);
               else;
                  YHistPred = H'*KsitT;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               Yinitial = YSmooth(:,1:LastPeriod);
            end;
         end;
         %
         if DSGEModel.AdjustPredictionPaths==1;
            %
            % compute the mean prediction at the given parameter values
            %
            PredMean = PredictY;
            %
            % annualize the mean predictions when needed
            %
            if AnnualizeData==1;
               PredMean = Annualize([Yinit PredMean],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               YTmp = [Yinitial PredMean];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredMean = YData(Tt-h+1:Tt,:)';
            end;
         else;
            PredMean = [];
         end;
         %
         % we are now ready to compute possible paths at theta
         %
         PredPaths = zeros(n,h,NumPaths);
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % we have fixed beliefs
            %
            for it=1:NumPaths;
               %
               % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % Draw a sequence of state shocks
               %
               vTh = BT1*randn(q,h);
               %
               % calculate a path for the states
               %
               KsiTh = zeros(r,h);
               KsiTh(:,1) = (MT1*X(:,1))+(FT1*KsiT)+vTh(:,1);
               for i=2:h;
                  KsiTh(:,i) = (MT1*X(:,i))+(FT1*KsiTh(:,i-1))+vTh(:,i);
               end;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
               end;
               %
               % add everything
               %
               if AnnualizeData==0;
                  PredPaths(:,:,it) = Ax+(H'*KsiTh)+wTh;
                  if TransData==1;
                     YTmp = [Yinitial PredPaths(:,:,it)];
                     [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                     PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
                  end;
               else;
                  PredPaths(:,:,it) = Annualize([Yinit Ax+(H'*KsiTh)+wTh],DSGEModel.annual,DSGEModel.annualscale);
               end;
            end;      
         else;
            %
            % we have updated beliefs
            %
            for it=1:NumPaths;
               %
               % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % Draw state shocks at T+1
               %
               vTh = BTh(:,:,1)*randn(q,1);
               %
               % calculate a path for the states
               %
               KsiTh = zeros(r,h);
               KsiTh(:,1) = (MTh(:,:,1)*X(:,1))+(FTh(:,:,1)*KsiT)+vTh;
               for i=2:h;
                  vTh = BTh(:,:,i)*randn(q,1);
                  KsiTh(:,i) = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*KsiTh(:,i-1))+vTh;
               end;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
               end;
               %
               % add everything
               %
               if AnnualizeData==0;
                  PredPaths(:,:,it) = Ax+(H'*KsiTh)+wTh;
                  if TransData==1;
                     YTmp = [Yinitial PredPaths(:,:,it)];
                     [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                     PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
                  end;
               else;
                  PredPaths(:,:,it) = Annualize([Yinit Ax+(H'*KsiTh)+wTh],DSGEModel.annual,DSGEModel.annualscale);
               end;
            end;
         end;
         %
         if DSGEModel.AdjustPredictionPaths==1;
            %
            % we need to fix the paths such that the sample mean is equal to the population mean.
            %
            MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
            AdjustMean = PredMean-MeanPrediction;
            for i=1:NumPaths;
               PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
            end;
         end;
         %
         % save the calculations to disk
         %
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         end;
         if (AnnualizeData==0)&(TransData==0);
            savefileparfor(file,PredPaths,MeasureCov,StateCov,ShockCov,ParameterCov,MeanPredictY);
         else;
            savefileparfor(file,PredPaths);
         end;
         if DSGEModel.RunPredictionEvent==1;
            %
            % compute the prediction events results
            %
            PredEventData = CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
            savefileparfor(file,[],[],[],[],[],[],PredEventData);
         end;
      end;
      %
      % end of parfor loop
      %
      % run additional calculations to collect estimates if required to do so
      %
      if (AnnualizeData==0)&(TransData==0);
         %
         % initialize
         %
         if DSGEModel.RunPredictionEvent==1;
            PredEventData = zeros(n,8);
         end;
         StateCov = zeros(n,n,h);
         ShockCov = StateCov;
         MeasureCov = zeros(n,n);
         ParameterCov = StateCov;
         MeanPredictY = zeros(n,h);
         for ParamIndex=1:NumDraws;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
            %
            % load the file
            %
            ParamData = load(file);
            StateCov = StateCov+ParamData.StateCov;
            ShockCov = ShockCov+ParamData.ShockCov;
            MeasureCov = MeasureCov+ParamData.MeasureCov;
            ParameterCov = ParameterCov+ParamData.ParameterCov;
            MeanPredictY = MeanPredictY+ParamData.MeanPredictY;
            if DSGEModel.RunPredictionEvent==1;
               PredEventData = PredEventData+ParamData.PredEventData;
            end;
         end;
      else;
         if DSGEModel.RunPredictionEvent==1;
            for ParamIndex=1:NumDraws
               if IsPosterior==1;
                  if TransData==0;
                     file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
                  else;
                     file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
                  end;
               else;
                  if TransData==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
                  else;
                     file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
                  end;
               end;
               %
               % load the file
               %
               ParamData = load(file);
               PredEventData = PredEventData+ParamData.PredEventData;
            end;
         end;
      end;
   else;
      %
      % we have a time-varying H-matrix
      %
      parfor ParamIndex=1:NumDraws;
         %
         % initialization
         %
         ModelParameters = MP;
         Gtilde = GT;
         Kgf = Kg;
         X = Xhelp;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation fucntion
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         end;
         %
         % determine initial conditions for the forecasts
         %
         KsiTT = Ksitt(:,T);
         PTT = Ptt(:,:,T);
         PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
         %
         % use the Schur decomposition for the reduced rank matrix PTT instead of
         % the Choleski decomposition
         %
         [S,L] = SchurDecomposition(PTT);
         if isreal(sqrt(L))==0;
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
            [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
            [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
            [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
            else;
               [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            if IsALUpdate==1;
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
               else;
                  [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            end;
            KsiTT = Ksitt(:,T);
            PTT = Ptt(:,:,T);
            PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
            [S,L] = SchurDecomposition(PTT);
         end;
         %
         % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
         %
         if rank(R)>0;
            [Rm,Lm] = SchurDecomposition(R);
            Rmeasure = Rm*sqrt(Lm);
            NumMeasure = size(Lm,1);
         else;
            Rmeasure = [];
            wTh = 0;
            NumMeasure = 0;
         end;
         %
         % now we can start with the moments
         %
         MeasureCov = (1/NumDraws)*R*Weights(ParamIndex);
         PredictY = zeros(n,h);
         Fi = eye(r);
         FjSum = zeros(r,r);
         MT1 = Mtilde(:,:,T);
         FT1 = Ftilde(:,:,T);
         BT1 = Btilde(:,:,T);
         PredictY = zeros(n,h);
         StateCov = zeros(n,n,h);
         ShockCov = StateCov;
         ParameterCov = StateCov;
         MeanPredictY = zeros(n,h);
         %
         % deal with deterministic variables
         %
         if k>0;
            Ax = A'*X;
         else;
            Ax = zeros(n,h);
            X = zeros(r,h)
         end;
         %
         % check beliefs and compute prediction
         %
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % forecast with fixed beliefs
            %
%            Q = BT1*BT1';
            KsiTiT = KsiTT;
            for i=1:h;
%               FjSum = FjSum+(Fi*Q*Fi');
%               ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*FjSum*H(:,:,LastPeriod+i);
               %
               % update Fi
               %
%               Fi = FT1*Fi;
%               StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,LastPeriod+i);
               KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
               PredictY(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
%               ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)';
            end;
         else;
            %
            % forecast with updated beliefs
            %
            KsiTiT = KsiTT;
            MTiT = MT1;
            FTiT = FT1;
            BTiT = BT1;
            %
            % initialize solution matrices to be used later
            %
%            MTh = zeros(r,k,h);
%            FTh = zeros(r,r,h);
%            BTh = zeros(r,q,h);
            for i=1:h;
%               FjSum = FjSum+(Fi*BTiT*BTiT'*Fi');
%               ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*FjSum*H(:,:,LastPeriod+i);
               %
               % update Fi
               %
%               Fi = FTiT*Fi;
%               StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,LastPeriod+i);
               KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
               PredictY(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
%               ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)';
               %
               % store the used solution matrices
               %
%               MTh(:,:,i) = MTiT;
%               FTh(:,:,i) = FTiT;
%               BTh(:,:,i) = BTiT;
               %
               % update beliefs
               %
               if i==1;
                  MTiT = LastSolution.MTT;
                  FTiT = LastSolution.FTT;
                  BTiT = LastSolution.BTT;
                  betaTiT = LastSolution.betaTT;
               elseif i<h;
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
               end;
            end;
         end;
         TestValue = max(max(abs((PredictY-PredictYmode)./PredictYmode)));
         if TestValue<100;
            MeanPredictY = (Weights(ParamIndex)/NumDraws)*PredictY;
         else;
            %
            % we need to use the appropriate entries for the mode values
            %
            PredictY = PredictYmode;
            Ksitt1 = Ksitt1mode;
            Ptt1 = Ptt1mode;
            Yerror = Yerrormode;
            InvMSEY = InvMSEYmode;
            H = Hmode;
            R = Rmode;
            Ftilde = Ftildemode;
            Btilde = Btildemode;
            A = Amode;
            KsiTT = KsiTTmode;
            S = Smode;
            L = Lmode;
            BT1 = BT1mode;
            MT1 = MT1mode;
            FT1 = FT1mode;
            BTh = BThmode;
            MTh = MThmode;
            FTh = FThmode;
            LastSolution = LastSolutionmode;
            MeanPredictY = (Weights(ParamIndex)/NumDraws)*PredictY;
         end;
         %
         % check beliefs and compute uncertainty
         %
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % forecast with fixed beliefs
            %
            Q = BT1*BT1';
%            KsiTiT = KsiTT;
            for i=1:h;
               FjSum = FjSum+(Fi*Q*Fi');
               ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*FjSum*H(:,:,LastPeriod+i);
               %
               % update Fi
               %
               Fi = FT1*Fi;
               StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,LastPeriod+i);
%               KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
%               PredictY(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
               ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)';
            end;
         else;
            %
            % forecast with updated beliefs
            %
%            KsiTiT = KsiTT;
            MTiT = MT1;
            FTiT = FT1;
            BTiT = BT1;
            %
            % initialize solution matrices to be used later
            %
            MTh = zeros(r,k,h);
            FTh = zeros(r,r,h);
            BTh = zeros(r,q,h);
            for i=1:h;
               FjSum = FjSum+(Fi*BTiT*BTiT'*Fi');
               ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*FjSum*H(:,:,LastPeriod+i);
               %
               % update Fi
               %
               Fi = FTiT*Fi;
               StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,LastPeriod+i);
%               KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
%               PredictY(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
               ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)';
               %
               % store the used solution matrices
               %
               MTh(:,:,i) = MTiT;
               FTh(:,:,i) = FTiT;
               BTh(:,:,i) = BTiT;
               %
               % update beliefs
               %
               if i==1;
                  MTiT = LastSolution.MTT;
                  FTiT = LastSolution.FTT;
                  BTiT = LastSolution.BTT;
                  betaTiT = LastSolution.betaTT;
               elseif i<h;
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
               end;
            end;
         end;
         %
         if AnnualizeData==1;
            if maxYinit==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect annualization
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               if k>0;
                  for t=1:T;
                     YHistPred = (A'*Xdata(:,t))+(H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t));
                  end;
               else;
                  for t=1:T;
                     YHistPred = H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t);
                  end;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
               else;
                  Yinit = [];
               end;
            end;
         elseif TransData==1;
            if maxYinitial==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect general transformations
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               if k>0;
                  for t=1:T;
                     YHistPred = (A'*Xdata(:,t))+(H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t));
                  end;
               else;
                  for t=1:T;
                     YHistPred = H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t);
                  end;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               Yinitial = YSmooth(:,1:LastPeriod);
            end;
         end;
         %
         if DSGEModel.AdjustPredictionPaths==1;
            %
            % compute the mean prediction at the given parameter values
            %
            PredMean = PredictY;
            %
            % annualize the mean predictions when needed
            %
            if AnnualizeData==1;
               PredMean = Annualize([Yinit PredMean],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               YTmp = [Yinitial PredMean];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredMean = YData(Tt-h+1:Tt,:)';
            end;
         else;
            PredMean = [];
         end;
         %
         % we are now ready to compute possible paths at theta
         %
         PredPaths = zeros(n,h,NumPaths);
         if DSGEModel.FixedBeliefsForProj==1;
            %
            % we have fixed beliefs
            %
            for it=1:NumPaths;
               %
               % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % Draw a sequence of state shocks
               %
               vTh = BT1*randn(q,h);
               %
               % calculate a path for the states
               %
               KsiTh = zeros(r,h);
               KsiTh(:,1) = (MT1*X(:,1))+(FT1*KsiT)+vTh(:,1);
               for i=2:h;
                  KsiTh(:,i) = (MT1*X(:,i))+(FT1*KsiTh(:,i-1))+vTh(:,i);
               end;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
               end;
               %
               % add everything
               %
               if AnnualizeData==0;
                  PredPaths(:,:,it) = Ax+(H(:,:,LastPeriod+i)'*KsiTh)+wTh;
                  if TransData==1;
                     YTmp = [Yinitial PredPaths(:,:,it)];
                     [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                     PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
                  end;
               else;
                  for i=1:h;
                     PredPaths(:,i,it) = Ax(:,i)+(H(:,:,LastPeriod+i)'*KsiTh(:,i))+wTh(:,i);
                  end;
                  PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
               end;
            end;      
         else;
            %
            % we have updated beliefs
            %
            for it=1:NumPaths;
               %
               % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % Draw state shocks at T+1
               %
               vTh = BTh(:,:,1)*randn(q,1);
               %
               % calculate a path for the states
               %
               KsiTh = zeros(r,h);
               KsiTh(:,1) = (MTh(:,:,1)*X(:,1))+(FTh(:,:,1)*KsiT)+vTh;
               for i=2:h;
                  vTh = BTh(:,:,i)*randn(q,1);
                  KsiTh(:,i) = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*KsiTh(:,i-1))+vTh;
               end;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
               end;
               %
               % add everything
               %
               if AnnualizeData==0;
                  for i=1:h;
                     PredPaths(:,i,it) = Ax(:,i)+(H(:,:,LastPeriod+i)'*KsiTh(:,i))+wTh(:,i);
                  end;
                  if TransData==1;
                     YTmp = [Yinitial PredPaths(:,:,it)];
                     [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                     PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
                  end;
               else;
                  for i=1:h;
                     PredPaths(:,i,it) = Ax(:,i)+(H(:,:,LastPeriod+i)'*KsiTh(:,i))+wTh(:,i);
                  end;
                  PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
               end;
            end;
         end;
         %
         if DSGEModel.AdjustPredictionPaths==1;
            %
            % we need to fix the paths such that the sample mean is equal to the population mean.
            %
            MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
            AdjustMean = PredMean-MeanPrediction;
            for i=1:NumPaths;
               PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
            end;
         end;
         %
         % save the calculations to disk
         %
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         end;
         if (AnnualizeData==0)&(TransData==0);
            savefileparfor(file,PredPaths,MeasureCov,StateCov,ShockCov,ParameterCov,MeanPredictY);
         else;
            savefileparfor(file,PredPaths);
         end;
         if DSGEModel.RunPredictionEvent==1;
            %
            % compute the prediction events results
            %
            PredEventData = CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
            savefileparfor(file,[],[],[],[],[],[],PredEventData);
         end;
      end;
      %
      % end of parfor loop
      %
      % run additional calculations to collect estimates if required to do so
      %
      if (AnnualizeData==0)&(TransData==0);
         %
         % initialize
         %
         if DSGEModel.RunPredictionEvent==1;
            PredEventData = zeros(n,8);
         end;
         StateCov = zeros(n,n,h);
         ShockCov = StateCov;
         MeasureCov = zeros(n,n);
         ParameterCov = StateCov;
         MeanPredictY = zeros(n,h);
         for ParamIndex=1:NumDraws;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Original-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
            %
            % load the file
            %
            ParamData = load(file);
            StateCov = StateCov+ParamData.StateCov;
            ShockCov = ShockCov+ParamData.ShockCov;
            MeasureCov = MeasureCov+ParamData.MeasureCov;
            ParameterCov = ParameterCov+ParamData.ParameterCov;
            MeanPredictY = MeanPredictY+ParamData.MeanPredictY;
            if DSGEModel.RunPredictionEvent==1;
               PredEventData = PredEventData+ParamData.PredEventData;
            end;
         end;
      else;
         if DSGEModel.RunPredictionEvent==1;
            for ParamIndex=1:NumDraws
               if IsPosterior==1;
                  if TransData==0;
                     file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
                  else;
                     file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
                  end;
               else;
                  if TransData==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Transformed-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
                  else;
                     file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-Prediction-Annual-Prior-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
                  end;
               end;
               %
               % load the file
               %
               ParamData = load(file);
               PredEventData = PredEventData+ParamData.PredEventData;
            end;
         end;
      end;
   end;
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
if StopEstimation==1;
   return;
end;
%
% make sure the parameter uncertainty is corrected for the mean
%
DoneCalc = 1;
if isfield(DSGEModel,'PredictionEvent')==1;
   PredictionEvent = DSGEModel.PredictionEvent;
else;
   PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
end;
if (AnnualizeData==0)&(TransData==0);
   for i=1:h;
      ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
   end;
   save(file,'PredEventData','PredictionEvent','StateCov','ShockCov','MeasureCov','ParameterCov','-APPEND');
else;
   StateCov = [];
   ShockCov = [];
   MeasureCov = [];
   ParameterCov = [];
   save(file,'PredEventData','PredictionEvent','-APPEND');
end;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% a local function for saving to matfile from a parfor loop
%

function savefileparfor(file,PredPaths,MeasureCov,StateCov,ShockCov,ParameterCov,MeanPredictY,PredEventData)

if nargin==7;
   save(file,'PredPaths','MeasureCov','StateCov','ShockCov','ParameterCov','MeanPredictY');
elseif nargin==8;
   save(file,'PredEventData','-APPEND');
elseif nargin==2;
   save(file,'PredPaths');
end;

%
% end of AdaptiveLearningDSGEPredictionPaths.m
%
