function FileMenuFunctions(selector,maingui)
% FileMenuFunctions: switch function for running the function on the file menu.
%
% USAGE:
%
%       FileMenuFunctions(selector)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
% OPTIONAL INPUT: maingui (handle) to the main YADA GUI.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 13, 2010.
%                        This version: March 18, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-02-2010: Updated the documentation.
%
% * 16-02-2010: Changed the order of the input variables for "YADAEnableControls".
%
% * 03-03-2010: The "quit" case was moved to "YADAGUI".
%
% * 07-03-2010: Changed the cases from lower case to mixed upper and lower case.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 13-01-2011: Made sure that the path provided by uiputfile ends with '/' on unix and mac.
%
% * 31-01-2011: Added the "PrintSetup" case.
%
% * 01-02-2011: Removed if statement from the "PrintSetup" case.
%
% * 13-12-2011: DSGEModel.YNaNs is now determined from the selected sample.
%
% * 02-01-2012: Updated the documentation.
%
% * 06-01-2012: Fixed a bug with missing observations.
%
% * 03-01-2013: Updated the documentation.
%
% * 17-07-2013: The function now fully supports the Waggoner-Zha approach to conditional forecasting.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-08-2014: Added try-catch code when attempting to read a DSGE model file.
%
% * 24-10-2014: Began work on the ModelSequence function. Took into account that the path can be extended by a particular model.
%               To correct for this, YADA resets the path each time a model is opened or closed.
%
% * 28-10-2014: Began work on the model sequence determination tool.
%
% * 30-10-2014: Finished the Select model sequence tool. Added functions for opening and closing a parallel pool.
%
% * 31-10-2014: Made sure the the field "NumWorkers" for the CurrINI structure is updated when a parallel pool is
%               opened and closed.
%
% * 02-12-2014: Started work on the run model sequence tool.
%
% * 03-12-2104: Finished adding model sequence run for posterior mode estimation of a DSGE model.
%
% * 09-12-2014: Began work on the predictive likelihood model sequence toolo for a DSGE model.
%
% * 10-12-2014: Finished the sequential predictive likelihood code for the DSGE model.
%
% * 11-12-2014: Improved the SetLogText code when loading a new model with the sequential run code.
%
% * 14-01-2015: Updated the documentation.
%
% * 15-01-2015: Added code for estimating the posterior mode of DSGE-VARs for a sequence of models.
%
% * 16-01-2015: Added code for posterior sampling of DSGE model parameters with a sequence of DSGE-VAR models.
%               as well as for posterior sampling of the VAR parameters for a sequence of DSGE-VARs.
%
% * 20-01-2015: Began work of sequential predictive likelihood tool for DSGE-VARs.
%
% * 21-01-2015: Finished the sequential predictive likelihood tool for DSGE-VARs.
%
% * 27-01-2015: Made sure that the input variables "DSGEModel", "CurrINI", and "controls" appear in the
%               correct order when calling "SequentialPredictiveLikelihoodEstimation".
%
% * 04-02-2015: Began work on sequential estimation of predictive moments for DSGE models.
%
% * 05-02-2015: Finished working on sequential estimation of predictive moments.
%
% * 18-02-2015: Added calls to "YADADeleteTmpFiles" to ensure that previous copied model related files
%               are deleted from the tmp directory before loading a valid new model.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 31-03-2015: Added code to take Status=-1 into account for the sequential tools.
%
% * 12-06-2015: Took into account that "SequentialPredictiveLikelihoodEstimation" requires the input and
%               output variable "SelectedYData".
%
% * 15-06-2015: Took into account that "SequentialPredictiveLikelihoodDSGEVAR" requires the input and
%               output variable "SelectedYData".
%
% * 24-08-2015: The sequential estimation of DSGE-VAR model posterior modes now sets the query if old
%               results should be reused to 'yes', thereby potentially speeding up the estimation.
%
% * 27-08-2015: Took into account that a Student-t proposal density is covered when using RWM posterior
%               sampling with a DSGE model.
%
% * 28-08-2015: Fixed a bug with a missing "DSGEModel.PosteriorSampler" condition for an elseif statement.
%
% * 02-09-2015: Made sure that the optimization routine (csminwel and newrat) obtain an answer to the
%               query under sequential estimation.
%
% * 01-10-2015: Added try-catch code around opening a parallel session.
%
% * 27-10-2015: Added another query answer when running the DSGE model posterior mode estimation.
%
% * 27-01-2016: Remember Shoah.
%
% * 17-03-2017: The run model sequence case is adapted to handle back- and nowcasts.
%
% * 02-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 06-04-2017: The sequential posterior mode estimation for DSGE and DSGE-VAR models now includes the setting
%               for computing the finite difference Hessian and use this for the Laplace approximation
%               of the marginal likelihood.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 19-04-2017: Updated the documentation.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 30-05-2017: Added the open text file function.
%
% * 14-08-2017: Added some initial code for using the SMC with likelihood tempering posterior sampler.
%
% * 15-08-2017: Took into account that SMC with likelihood tempering for the DSGE model provides the
%               output variable "SMCNumMHSteps".
%
% * 17-08-2017: Took into account that SMC with likelihood tempering has an optional input variable
%               "SMCNumMHSteps" which needed to be initialized as empty.
%
% * 28-08-2017: Took into account that SMC with likelihood tempering no longer has an optional input
%               variable "SMCNumMHSteps" and the it no longer has an output variable with the same
%               name.
%
% * 30-08-2017: Added the home command when opening or closing a parallel pool.
%
% * 08-09-2017: Added a call to "DSGEVARSMCLikelihoodTemperingPosteriorSampler".
%
% * 11-09-2017: Fixed an error when calling "DSGEVARSMCLikelihoodTemperingPosteriorSampler".
%
% * 15-09-2017: Updated the documentation.
%
% * 08-01-2018: Updated the documentation.
%
% * 21-03-2018: Added sequential estimation of the log marginal likelihood for DSGE models.
%
% * 22-03-2018: Updated the documentation.
%
% * 23-03-2018: Made sure that the correct control is checked when attempting to run the sequential
%               estimation of the marginal likelihoods for DSGE models.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 16-10-2018: The function mainly supports the SMC with data tempering posterior sampler.
%
% * 04-12-2018: The function partly supports IS based on MitISEM.
%
% * 10-01-2019: Added code for turning off warning when using the parallel computing toolbox.
%
% * 16-01-2019: The function now fully supports IS based on MitISEM.
%
% * 14-02-2019: Added support for sequential estimation of the marginal likelihood for DSGE-VARs.
%
% * 10-01-2020: Updated the documentation.
%
% * 22-10-2020: Began work on sequential estimation of probability integral transforms for DSGE models.
%
% * 23-10-2020: Finished work on the sequential estimation of PITs for DSGE models.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-03-2021: Check a bug.
%
% * 28-04-2021: Updated the documentation.
%
% * 14-12-2021: Began work on sequential estimation of predictive likelihoods for adaptive learning
%               DSGE models.
%
% * 03-01-2022: Updated the documentation.
%
% * 13-01-2022: Took DSGEModel.ShowEstimationLog into account.
%
% * 18-01-2022: Added sequential estimation of PITs for adaptive learning DSGE models.
%
% * 19-01-2022: Added sequential posterior mode estimation, posterior sampling, predictive moments
%               and marginal likelihood estimation for adaptive learning DSGE models.
%
% * 20-01-2022: Added missing code for running the predictive moments for adaptive learning DSGE models.
%
% * 27-01-2022: Added sequential Laplace approximation estimation for DSGE models under rational
%               expectations, adaptive learning and for DSGE-VARs.
%
% * 07-02-2022: Added sequential estimation of CRPS and the ES for DSGE models.
%
% * 08-02-2022: Added sequential estimation of CRPS and the ES for adaptive learning DSGE models.
%
% * 11-02-2022: Added check to the sequential CRPS calculations if the user has selected a large
%               number of paths.
%
% * 17-05-2022: Added Share for the sequential CRPS calculations for the SMC with likelihood tempering
%               and MitISEM algorithms.
%
% * 19-05-2022: Added Share for the sequential PIT and marginal predictive moment calcualtions for
%               the DSGE models under rational expectations and adaptive learning.
%
% * 05-01-2023: Updated the documentation.
%
% * 23-01-2023: Added the "DriverSelection" function and began work on "RunDriverFunctions" function.
%
% * 24-01-2023: Finished the "RunDriverFunctions" function.
%
% * 26-01-2023: Added the "Run Matlab Script File" function.
%
% * 03-02-2023: Calls to parpool, etc. are now captured by evalc to avoid command window output.
%
% * 09-02-2023: Added running drivers functions to the run model sequence function.
%
% * 17-02-2023: Updated the "Run Matlab Script File" function to take the DSGEModel field "RunScriptFile"
%               into account and that it supports driver functionality.
%
% * 21-02-2023: Added the "maingui" input variable. The "RunDriverFunctions" can now be run from the
%               command line.
%
% * 22-02-2023: The "RunModelSequence" using the RunDriverFunctions case can now be run from the
%               command line.
%
% * 30-03-2023: Added the save inverse Hessian function.
%
% * 06-04-2023: Fixed an error when clicking on cancel on uigetfile in run matlab script.
%
% * 24-04-2023: Added a pause command before displaying About dialog when opening a parallel pool.
%
% * 26-04-2023: Added a call to pctRunOnAll with warning off.
%
% * 19-06-2023: Added the case "ShowYADAIcon".
%
% * 26-06-2023: The ShowYADAIcon function can now also revert the icon back to the original matlab icon.
%
% * 27-07-2023: Fixed a display issue when running certain driver functions.
%
% * 09-01-2024: Took into account that "YADAHessianMatrix" now takes the input variable "NumWorkers".
%
% * 12-01-2024: The IdleTimeout property of a parallel pool is set to Inf when computing the inverse Hessian
%               with finite differences and when asked to do so.
%
% * 16-01-2024: The IdleTimeout property is set to Inf when opening a parallel pool with parpool.
%
% * 17-01-2024: Made some cosmetic changes to the code from the 16th of January.
%
% * 15-03-2024: Changed the lower vertical position of the TextGUI function from 32 pixels to 50.
%
% * 18-03-2024: Took into account that "CompareDSGEModel" provides an optional output variable "DiffField".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
if nargin==1;
   maingui = findobj('Type','figure','Tag','YADA');
end;
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
OrigDSGEModel = get(controls.openmenu,'UserData');
if MatlabNumber<7;
   lasterr = '';
end;
%
switch selector
   case 'Open'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if there's already a model loaded and if it has been changed.
         %
         if isempty(DSGEModel.NameOfModel)==0;
            [status,DiffField] = CompareDSGEModel(DSGEModel,OrigDSGEModel);
            if status==0;
               txt = strvcat(['Your DSGE model settings have changed. The field "' DiffField '" differs. Are you sure you want to open a new model without saving the current model "' DSGEModel.NameOfModel '" first?'],' ','NOTE: The field names are explained in the file "ExtendingYADA.pdf" in the "doc" directory.');
               answer = Query(txt,'question',200,'Open DSGE Model File',500,'no',CurrINI);
            else;
               answer = 'Yes';
            end;
         else;
            answer = 'Yes';
         end;
         %
         % Open a valid DSGE Model File
         %
         if strcmp(lower(answer),'yes')==1;
            [status,ModelFile] = FileBrowse(CurrINI.CurrentModelFile,'Open DSGE Model File',[],'dsge',CurrINI);
            if status==1;
               ValidFile = 1;
               try;
                  NewDSGEModel = ReadDSGEModel(ModelFile);
               catch;
                  ValidFile = 0;
               end;
               if ValidFile==1;
                  OrigPath = path;
                  path(CurrINI.YADApath);
                  [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
                  [ModelStatus,NewDSGEModel] = VerifyDSGEModel(NewDSGEModel,ModelFile,DSGEModel.DataConstructionFile,CurrINI);
                  %
                  % if ModelStatus=1, then we can set DSGEModel to NewDSGEModel and adjust the GUI controls!
                  %
                  if ModelStatus==1;
                     [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
                     if (isempty(NewDSGEModel.StateShockPositions)==0)&(isempty(NewDSGEModel.DSGEVARShocks)==1);
                        NewDSGEModel.DSGEVARShocks = (1:min(length(NewDSGEModel.StateShockPositions),NewDSGEModel.n));
                     end;
                     %
                     % clean up the tmp directory
                     %
                     YADADeleteTmpFiles(DSGEModel);
                     DSGEModel = NewDSGEModel;
                     if isempty(DSGEModel.Y)==0;
                        [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                        DSGEModel.YNaNs = max(max(isnan(DSGEModel.Y(:,FirstPeriod:LastPeriod))));
                     else;
                        DSGEModel.YNaNs = 0;
                     end;
                     %
                     StateCStr = get(controls.dsge.statecovariance,'String');
                     DSGEModel.StateCovConstValue = str2num(strrep(StringTrim(StateCStr(DSGEModel.StateCovConst,:)),',',''));
                     set(controls.open,'UserData',DSGEModel);
                     OrigDSGEModel = DSGEModel;
                     set(controls.openmenu,'UserData',OrigDSGEModel);
                     SetYADAUIControls(maingui,DSGEModel,ModelFile,CurrINI.images);
                     %
                     % reset the CurrentModelFile field of CurrINI
                     %
                     CurrINI.CurrentModelFile = ModelFile;
                     CurrINI.ModelFiles = ReOrderModelFiles(CurrINI.ModelFiles,CurrINI.CurrentModelFile);
                     %
                     % Update the menus
                     %
                     if isempty(CurrINI.ModelFiles)==0;
                        controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
                        set(maingui,'UserData',controls);
                     end;
                  else;
                     txt = ['The DSGE model file you selected, "' GetFilename(ModelFile) '", isn''t fit to be loaded into YADA. Please check it in a text editor to make sure it doesn''t point to invalid paths or files.'];
                     About(txt,'information','Bad DSGE Model File',400,600,CurrINI);
                     path(OrigPath);
                  end;
               else;
                  txt = ['The DSGE model file you selected, "' GetFilename(ModelFile) '", couldn''t be loaded into YADA. The file has possibly been corrupted.'];
                  About(txt,'information','Bad DSGE Model File',400,600,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SaveModel'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if we should save using a new name
         %
         if FileExist(CurrINI.CurrentModelFile)==1;
            txt = ['The current file for your DSGE model "' DSGEModel.NameOfModel '" is "' GetFilename(CurrINI.CurrentModelFile) '". Would you like to save it into a new file?'];;
            answer = Query(txt,'question',200,'Save DSGE Model File',450,'no',CurrINI);
         else;
            answer = 'Yes';
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            if FileExist(CurrINI.CurrentModelFile)==0;
               if (strcmp(DSGEModel.NameOfModel,'')==1)|(isempty(DSGEModel.NameOfModel)==1);
                  filename = '*.dsge';
               else;
                  filename = [DSGEModel.NameOfModel '.dsge'];
               end;
               filename = FixFilePath([DSGEModel.OutputDirectory '\' filename]);
            else;
               if strcmp(lower(FixFilePath([DSGEModel.OutputDirectory '\'])),lower(GetPath(CurrINI.CurrentModelFile)))==1;
                  filename = CurrINI.CurrentModelFile;
               else;
                  %
                  % we select a new directory for the DSGE model file based on the OutputDirectory selection
                  %
                  filename = strrep(CurrINI.CurrentModelFile,GetPath(CurrINI.CurrentModelFile),[DSGEModel.OutputDirectory '\']);
               end;
            end;
            if MatlabNumber>=6;
               fileflt = {'*.dsge', 'DSGE Model Files (*.dsge)'};
               [FileName,FilePath] = uiputfile(fileflt,'Save DSGE Model File',filename);
            else;
               [FileName,FilePath] = uiputfile(filename,'Save DSGE Model File');
            end;
            drawnow;
            pause(0.02);
         else;
            FileName = GetFilename(CurrINI.CurrentModelFile);
            FilePath = GetPath(CurrINI.CurrentModelFile);
         end;
         if (isnumeric(FileName)==0)&(isunix==1);
            if strcmp(FilePath(1,length(FilePath)),'/')==0;
               FilePath = [FilePath '/'];
            end;
         end;
         if ischar(FileName)==1;
            NewModelFile = [FilePath FileName];
            %
            % make sure it has the file extension .dsge
            %
            if strcmp(lower(GetExt(NewModelFile)),'.dsge')==0;
               NewModelFile = [FilePath GetName(NewModelFile) '.dsge'];
            end;
            %
            % save the current model data to text file
            %
            SaveDSGEModel(NewModelFile,DSGEModel);
            %
            % update the ModelFiles structure
            %
            CurrINI.CurrentModelFile = NewModelFile;
            CurrINI.ModelFiles = ReOrderModelFiles(CurrINI.ModelFiles,CurrINI.CurrentModelFile);
            %
            % Update the menus
            %
            if isempty(CurrINI.ModelFiles)==0;
               controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
               set(maingui,'UserData',controls);
            end;
            %
            % update the maingui name
            %
            set(maingui,'Name',['YADA - ' DSGEModel.NameOfModel ' - [' GetFilename(CurrINI.CurrentModelFile) ']']);
            %
            % Update the OrigDSGEModel structure
            %
            OrigDSGEModel = DSGEModel;
            set(controls.openmenu,'UserData',OrigDSGEModel);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ReloadModel'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if FileExist(CurrINI.CurrentModelFile)==1;
            ValidFile = 1;
            try;
               NewDSGEModel = ReadDSGEModel(CurrINI.CurrentModelFile);
            catch;
               ValidFile = 0;
            end;
            if ValidFile==1;
               [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
               [ModelStatus,NewDSGEModel] = VerifyDSGEModel(NewDSGEModel,CurrINI.CurrentModelFile,DSGEModel.DataConstructionFile,CurrINI);
               if ModelStatus==1;
                  [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
                  if (isempty(NewDSGEModel.StateShockPositions)==0)&(isempty(NewDSGEModel.DSGEVARShocks)==1);
                     NewDSGEModel.DSGEVARShocks = (1:min(length(NewDSGEModel.StateShockPositions),NewDSGEModel.n));
                  end;
                  %
                  % delete any files that were stored in the tmp directory
                  %
                  YADADeleteTmpFiles(DSGEModel);
                  DSGEModel = NewDSGEModel;
                  if isempty(DSGEModel.Y)==0;
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.YNaNs = max(max(isnan(DSGEModel.Y(:,FirstPeriod:LastPeriod))));
                  else;
                     DSGEModel.YNaNs = 0;
                  end;
                  %
                  StateCStr = get(controls.dsge.statecovariance,'String');
                  DSGEModel.StateCovConstValue = str2num(strrep(StringTrim(StateCStr(DSGEModel.StateCovConst,:)),',',''));
                  set(controls.open,'UserData',DSGEModel);
                  OrigDSGEModel = DSGEModel;
                  set(controls.openmenu,'UserData',OrigDSGEModel);
                  SetYADAUIControls(maingui,DSGEModel,CurrINI.CurrentModelFile,CurrINI.images);
               end;
            else;
               txt = ['The DSGE model file you have tried to reload, "' GetFilename(ModelFile) '", couldn''t be loaded into YADA. The file has possibly been corrupted.'];
               About(txt,'information','Bad DSGE Model File',200,600,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'OpenModel'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if there's already a model loaded and if it has been changed.
         %
         if isempty(DSGEModel.NameOfModel)==0;
            [status,DiffField] = CompareDSGEModel(DSGEModel,OrigDSGEModel);
            if status==0;
               txt = strvcat(['Your DSGE model settings have changed. The field "' DiffField '" differs. Are you sure you want to open a new model without saving the current model "' DSGEModel.NameOfModel '" first?'],' ','NOTE: The field names are explained in the file "ExtendingYADA.pdf" in the "doc" directory.');
               answer = Query(txt,'question',200,'Reopen Model',500,'no',CurrINI);
            else;
               answer = 'Yes';
            end;
         else;
            answer = 'Yes';
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % which item was clicked on
            %
            ModelFile = get(gcbo,'UserData');
            if FileExist(ModelFile)==1;
               ValidFile = 1;
               try;
                  NewDSGEModel = ReadDSGEModel(ModelFile);
               catch;
                  ValidFile = 0;
               end;
               if ValidFile==1;
                  OrigPath = path;
                  path(CurrINI.YADApath);
                  [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
                  [ModelStatus,NewDSGEModel] = VerifyDSGEModel(NewDSGEModel,ModelFile,DSGEModel.DataConstructionFile,CurrINI);
                  %
                  % if ModelStatus=1, then we can set DSGEModel to NewDSGEModel and adjust the GUI controls!
                  %
                  if ModelStatus==1;
                     [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
                     if (isempty(NewDSGEModel.StateShockPositions)==0)&(isempty(NewDSGEModel.DSGEVARShocks)==1);
                        NewDSGEModel.DSGEVARShocks = (1:min(length(NewDSGEModel.StateShockPositions),NewDSGEModel.n));
                     end;
                     %
                     % clean up the tmp directory
                     %
                     YADADeleteTmpFiles(DSGEModel);
                     DSGEModel = NewDSGEModel;
                     %
                     if isempty(DSGEModel.Y)==0;
                        [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                        DSGEModel.YNaNs = max(max(isnan(DSGEModel.Y(:,FirstPeriod:LastPeriod))));
                     else;
                        DSGEModel.YNaNs = 0;
                     end;
                     %
                     StateCStr = get(controls.dsge.statecovariance,'String');
                     DSGEModel.StateCovConstValue = str2num(strrep(StringTrim(StateCStr(DSGEModel.StateCovConst,:)),',',''));
                     set(controls.open,'UserData',DSGEModel);
                     OrigDSGEModel = DSGEModel;
                     set(controls.openmenu,'UserData',OrigDSGEModel);
                     SetYADAUIControls(maingui,DSGEModel,ModelFile,CurrINI.images);
                     %
                     % reset the CurrentModelFile field of CurrINI
                     %
                     CurrINI.CurrentModelFile = ModelFile;
                     CurrINI.ModelFiles = ReOrderModelFiles(CurrINI.ModelFiles,CurrINI.CurrentModelFile);
                     %
                     % Update the menus
                     %
                     if isempty(CurrINI.ModelFiles)==0;
                        controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
                        set(maingui,'UserData',controls);
                     end;
                  else;
                     %
                     % make sure that CurrINI.CurrentModelFile is removed from CurrINI.ModelFiles
                     %
                     CurrINI.ModelFiles = RemoveFromModelFiles(CurrINI.ModelFiles,ModelFile);
                     delete(gcbo);
                     if isempty(CurrINI.ModelFiles)==0;
                        controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
                        set(maingui,'UserData',controls);
                     end;
                     txt = ['The DSGE model file you selected, "' GetFilename(ModelFile) '", isn''t fit to be loaded into YADA. Please check it in a text editor to make sure it doesn''t point to invalid paths or files.'];
                     About(txt,'information','Bad DSGE Model File',400,600,CurrINI);
                     path(OrigPath);
                  end;
               else;
                  txt = ['The DSGE model file you selected, "' GetFilename(ModelFile) '", couldn''t be loaded into YADA. The file has possibly been corrupted.'];
                  About(txt,'information','Bad DSGE Model File',400,600,CurrINI);
               end;
            else;
               %
               % the file must very recently have been deleted.
               %
               CurrINI.ModelFiles = RemoveFromModelFiles(CurrINI.ModelFiles,ModelFile);
               delete(gcbo);
               if isempty(CurrINI.ModelFiles)==0;
                  controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
                  set(maingui,'UserData',controls);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ModelSequence'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [action,ModelSequence] = ModelSequenceDLG('init',CurrINI);
         if strcmp(lower(action),'ok')==1;
            CurrINI.ModelSequence = ModelSequence;
            %
            % save the ini-file
            %
            SaveINIFile(CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'RunModelSequence'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if isempty(DSGEModel.NameOfModel)==0;
            status = CompareDSGEModel(DSGEModel,OrigDSGEModel);
            if status==0;
               answer = Query(['Your DSGE model settings have changed. Are you sure you want to run the model sequence without saving the current model "' DSGEModel.NameOfModel '" first?'],'question',200,'Run Model Sequence',500,'no',CurrINI);
            else;
               answer = 'Yes';
            end;
         else;
            answer = 'Yes';
         end;
         %
         RunSequentialDriver = DSGEModel.RunSequentialDriver;
         ShowEstimationLog = DSGEModel.ShowEstimationLog;
         %
         if strcmp(lower(answer),'yes')==1;
            if RunSequentialDriver==0;
               ModelSequenceOptions = strvcat('1. Posterior mode estimation with DSGE models','2. Posterior sampling with DSGE models','3. Predictive likelihood using posteriors for DSGE models','4. Marginal predictive moments using posteriors for DSGE models','5. Marginal likelihood for DSGE models','6. Probability integral transforms (PITs) using posteriors for DSGE models','7. Laplace approximation for DSGE models','8. CRPS and Energy Score for DSGE models','9. Posterior mode estimation with DSGE-VARs','10. Posterior sampling of DSGE model parameters with DSGE-VARs','11. Posterior sampling of VAR parameters with DSGE-VARs','12. Predictive likelihood using posteriors for DSGE-VARs','13. Marginal predictive moments using posteriors for DSGE-VARs','14. Marginal likelihood for DSGE-VARs','15. Laplace approximation for DSGE-VARs','16. Posterior mode estimation for adaptive learning DSGE models','17. Posterior sampling for adaptive learning DSGE models','18. Predictive likelihood using posteriors for adaptive learning DSGE models','19. Marginal predictive moments using posteriors for adaptive learning DSGE models','20. Marginal likelihood for adaptive learning DSGE models','21. PITs using posteriors for adaptive learning DSGE models','22. Laplace approximation for adaptive learning DSGE models','23. CRPS and Energy Score for adaptive learning DSGE models','24. Run driver functions');
               [action,RMSChoice] = SelectionDlg(ModelSequenceOptions,1,'Select the tool to run:','Model Sequence Tool Selection','Run Model Sequence','','',CurrINI);
            else;
               action = 'ok';
               RMSChoice = 24;
            end;
            if strcmp(lower(action),'ok')==1;
               CurrentModelFile = CurrINI.CurrentModelFile;
               %
               % set autotmatic answers to queries
               %
               if RMSChoice==1;
                  CurrINI.Sequence.QueryAnswers = {'no' 'no' 'no' 'yes' 'no'};
                  CurrINI.Sequence.OptimizeAnswer = {'yes'};
               elseif RMSChoice==2;
                  CurrINI.Sequence.QueryAnswers = {'no' 'no' 'no' 'yes' 'yes'};
               elseif RMSChoice==3;
                  CurrINI.Sequence.QueryAnswers = {''};
                  SelectedYData = [];
                  IsOriginal = [];
                  IsMarginal = [];
                  CurrChain = [];
                  IsSubSample = [];
                  PredictionVariables = [];
                  RecomputePL = [];
               elseif RMSChoice==4;
                  CurrChain = [];
                  IsSubSample = [];
                  RecomputeMoments = [];
                  Share = [];
               elseif RMSChoice==5;
                  CurrChain = [];
                  IsSubSample = [];
                  RecomputePosteriorKernel = [];
                  CurrINI.Sequence.QueryAnswers = {'yes' 'no'};
               elseif RMSChoice==6;
                  CurrChain = [];
                  IsSubSample = [];
                  PredictionVariables = [];
                  RecomputePITs = [];
                  ComputePredPaths = [];
                  SelectedYData = [];
                  Share = [];
               elseif RMSChoice==7;
                  RecalculateInverseHessian = [];
               elseif RMSChoice==8;
                  CurrINI.Sequence.QueryAnswers = {''};
                  SelectedYData = [];
                  IsOriginal = [];
                  CurrChain = [];
                  IsSubSample = [];
                  PredictionVariables = [];
                  RecomputeCRPS = [];
                  UsePaths = [];
                  Share = [];
               elseif RMSChoice==9;
                  CurrINI.Sequence.QueryAnswers = {'yes' 'yes' 'no' 'yes' 'yes'};
                  CurrINI.Sequence.OptimizeAnswer = {'yes'};
                  IsMarginal = [];
               elseif RMSChoice==10;
                  CurrINI.Sequence.QueryAnswers = {'no' 'no' 'no' 'yes' 'yes'};
                  CurrINI.Sequence.OptimizeAnswer = {'yes'};
                  SelectedMode = [];
                  lambdai = [];
               elseif RMSChoice==11;
                  CurrINI.Sequence.QueryAnswers = {'yes'};
                  value = [];
                  CurrChain = [];
                  PME = [];
                  Lambda = [];
               elseif RMSChoice==12;
                  CurrINI.Sequence.QueryAnswers = {''};
                  SelectedYData = [];
                  value = [];
                  CurrChain = [];
                  PME = [];
                  Lambda = [];
                  IsOriginal= [];
                  IsMarginal = [];
                  PredictionVariables = [];
                  RecomputePL = [];
               elseif RMSChoice==13;
                  value = [];
                  CurrChain = [];
                  PME = [];
                  Lambda = [];
                  RecomputeMoments = [];
               elseif RMSChoice==14;
                  value = [];
                  CurrChain = [];
                  PME = [];
                  Lambda = [];
                  IsSubSample = [];
                  RecomputePosteriorKernel = [];
                  CurrINI.Sequence.QueryAnswers = {'yes' 'no'};
               elseif RMSChoice==15;
                  RecomputeLML = [];
                  PME = [];
                  Lambda = [];
                  CurrModes = zeros(1,3);
               elseif RMSChoice==16;
                  CurrINI.Sequence.QueryAnswers = {'no' 'no' 'no' 'yes' 'no'};
                  CurrINI.Sequence.OptimizeAnswer = {'yes'};
               elseif RMSChoice==17;
                  CurrINI.Sequence.QueryAnswers = {'no' 'no' 'no' 'yes' 'yes'};
               elseif RMSChoice==18;
                  CurrINI.Sequence.QueryAnswers = {''};
                  SelectedYData = [];
                  IsOriginal = [];
                  IsMarginal = [];
                  CurrChain = [];
                  IsSubSample = [];
                  PredictionVariables = [];
                  RecomputePL = [];
               elseif RMSChoice==19;
                  CurrChain = [];
                  IsSubSample = [];
                  RecomputeMoments = [];
                  Share = [];
               elseif RMSChoice==20;
                  CurrChain = [];
                  IsSubSample = [];
                  RecomputePosteriorKernel = [];
                  CurrINI.Sequence.QueryAnswers = {'yes' 'no'};
               elseif RMSChoice==21;
                  CurrChain = [];
                  IsSubSample = [];
                  PredictionVariables = [];
                  RecomputePITs = [];
                  ComputePredPaths = [];
                  SelectedYData = [];
                  Share = [];
               elseif RMSChoice==22;
                  RecalculateInverseHessian = [];
               elseif RMSChoice==23;
                  CurrINI.Sequence.QueryAnswers = {''};
                  SelectedYData = [];
                  IsOriginal = [];
                  CurrChain = [];
                  IsSubSample = [];
                  PredictionVariables = [];
                  RecomputeCRPS = [];
                  UsePaths = [];
                  Share = [];
               end;
               set(controls.filemenu,'UserData',CurrINI);
               %
               % setup the sequence of modes
               %
               NumFiles = size(CurrINI.ModelSequence,1);
               oldtab = [];
               i = 1;
               for i=1:NumFiles;
                  if i==1;
                     if ShowEstimationLog==1;
                        set(controls.about,'UserData',0);
                        oldtab = get(controls.tab1,'UserData');
                        set(maingui,'CurrentObject',controls.tab6);
                        YADAGUI('tab');
                        if MatlabNumber>=7;
                           uicontrol(controls.tab6);
                        end;
                        set(controls.about,'UserData',1);
                        drawnow;
                     end;
                     %
                     % check if we should add some diving line to the log-text control
                     %
                     LogText = get(controls.estimation_log,'String');
                     if isempty(LogText)==0;
                        SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************',' '));
                     end;
                  end;
                  %
                  % try to load each model
                  %
                  CurrINI.CurrentModelFile = StringTrim(CurrINI.ModelSequence(i,:));
                  set(controls.about,'UserData',0);
                  set(controls.filemenu,'UserData',CurrINI);
                  LoadTxt = sprintf('Loading model number %1.0f: %s',i,GetName(CurrINI.CurrentModelFile));
                  SetLogText(strvcat(LoadTxt,MultiCharStr('=',length(LoadTxt))));
                  if RunSequentialDriver==0;
                     FileMenuFunctions('ReloadModel');
                  else;
                     if i==1;
                        disp(LoadTxt);
                     else;
                        disp(strvcat(' ',LoadTxt));
                     end;
                     FileMenuFunctions('ReloadModel',maingui);
                  end;
                  controls = get(maingui,'UserData');
                  DSGEModel = get(controls.open,'UserData');
                  if RunSequentialDriver==1;
                     DSGEModel.RunSequentialDriver = 1;
                     DSGEModel.ShowEstimationLog = 0;
                     set(controls.open,'UserData',DSGEModel);
                     set(controls.openmenu,'UserData',DSGEModel);
                  end;
                  set(controls.about,'UserData',1);
                  %
                  % now run the tool
                  %
                  if RMSChoice==1;
                     %
                     % run DSGE posterior mode estimation for the sequence
                     %
                     if strcmp(get(controls.postmodemenu,'Enable'),'on')==1;
                        PosteriorModeEstimation(DSGEModel,CurrINI,controls);
                     else;
                        SetLogText('Posterior mode estimation is not enabled.');
                     end;
                  elseif RMSChoice==2;
                     %
                     % run DSGE posterior sampling for the sequence
                     %
                     if strcmp(get(controls.postsamplingmenu,'Enable'),'on')==1;
                        if DSGEModel.PosteriorSampler==1;
                           %
                           % random walk Metropolis with normal proposal
                           %
                           DSGERWMPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==2;
                           %
                           % slice sampler
                           %
                           DSGESlicePosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==3;
                           %
                           % random walk Metropolis with Student-t proposal
                           %
                           DSGERWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==4;
                           %
                           % fixed blocking RWM with normal proposal
                           %
                           DSGEFixedBlockingRWMPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==5;
                           %
                           % fixed blocking RWM with a Student-t proposal
                           %
                           DSGEFixedBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==6;
                           %
                           % random blocking RWM with a normal proposal
                           %
                           DSGERandomBlockingRWMPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==7;
                           %
                           % random blocking RWM with a Student-t proposal
                           %
                           DSGERandomBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==8;
                           %
                           % SMC with likelihood tempering
                           %
                           DSGESMCLikelihoodTemperingPosteriorSampler(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==9;
                           %
                           % SMC with data tempering
                           %
                           DSGESMCDataTemperingPosteriorSampler(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==10;
                           %
                           % IS based on MitISEM
                           %
                           DSGEISMitISEMPosteriorSampler(DSGEModel,CurrINI,controls);
                        end;
                     else;
                        SetLogText('Posterior sampling is not enabled.');
                     end;
                  elseif RMSChoice==3;
                     %
                     % run the DSGE predictive likelihood
                     %
                     if strcmp(get(controls.predict.unconditional.original.height.posterior,'Enable'),'on')==1;
                        %
                        % check if the predictive likelihoods should be recalculated when old values exist.
                        %
                        if isempty(RecomputePL)==1;
                           txt = 'If YADA locates posterior calculations of the log predictive likelihood on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Predictive Likelihood - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputePL = 1;
                           else;
                              RecomputePL = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,IsOriginal,IsMarginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData] = SequentialPredictiveLikelihoodEstimation(DSGEModel,CurrINI,controls,IsOriginal,IsMarginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData,RecomputePL);
                        if Status==-1;
                           break;
                        end;
                     else;
                        SetLogText('Predictive likelihood calculation is not enabled.');
                     end;
                  elseif RMSChoice==4;
                     %
                     % check if we can compute predictive moments
                     %
                     if strcmp(get(controls.predict.unconditional.original.moments.posterior,'Enable'),'on')==1;
                        %
                        % check if unconditional predictive moments should be recalculated when old values exist
                        %
                        if isempty(RecomputeMoments)==1;
                           txt = 'If YADA locates posterior calculations of the marginal predictive moments on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Marginal Predictive Moments - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputeMoments = 1;
                           else;
                              RecomputeMoments = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,CurrChain,IsSubSample,Share] = SequentialPredictiveMoments(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,RecomputeMoments,Share);
                     else;
                        SetLogText('Unconditional marginal predictive moment estimation is not enabled.');
                     end;
                  elseif RMSChoice==5;
                     %
                     % check if we can compute the marginal likelihood for DSGE models
                     %
                     if strcmp(get(controls.postsample.rawplots_logpost,'Enable'),'on')==1;
                        if isempty(RecomputePosteriorKernel)==1;
                           if DSGEModel.PosteriorSampler<=7;
                              txt = 'If YADA locates posterior calculations of the marginal likelihood on disk, would you like to use these results?';
                              answer = Query(txt,'question',200,'Marginal Likelihood Estimation',500,'no',CurrINI);
                              if strcmp(lower(answer),'no')==1;
                                 RecomputeML = 1;
                              else;
                                 RecomputeML = 0;
                              end;
                           else;
                              RecomputeML = 0;
                           end;
                        end;
                        %
                        [Status,CurrChain,IsSubSample,RecomputePosteriorKernel] = SequentialMarginalLikelihood(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,RecomputePosteriorKernel,RecomputeML);
                     else;
                        SetLogText('Unable to compute the marginal likelihood. Posterior draws are not available.');
                     end;
                  elseif RMSChoice==6;
                     %
                     % check if we can compute predictive moments for PITs
                     %
                     if strcmp(get(controls.predict.unconditional.original.pit.posterior,'Enable'),'on')==1;
                        %
                        % check if unconditional predictive moments along with the PITs should be recalculated when old values exist
                        %
                        if isempty(RecomputePITs)==1;
                           txt = 'If YADA locates posterior draws based estimates of the moments on disk that can be used for the probability integral transform (PIT), would you like to use these results?';
                           answer = Query(txt,'question',200,'Probability Integral Transform - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputePITs = 1;
                           else;
                              RecomputePITs = 0;
                           end;
                        end;
                        %
                        % check if path based PITs should also be computed
                        %
                        if isempty(ComputePredPaths)==1;
                           txt = strvcat('YADA can also estimate probability integral transform (PIT) values based on simulated paths from the predictive distribution. Would you like to run these calculations as well?',' ','NOTE: If you answer YES, then YADA will recompute the prediction paths even if they already exist on disk.');
                           answer = Query(txt,'question',200,'Probability Integral Transform - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              ComputePredPaths = 1;
                           else;
                              ComputePredPaths = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,CurrChain,IsSubSample,RecomputePITs,ComputePredPaths,PredictionVariables,SelectedYData,Share] = SequentialPITsEstimation(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,RecomputePITs,ComputePredPaths,PredictionVariables,SelectedYData,Share);
                     else;
                        SetLogText('Posterior estimation of the probability integral transforms (PITs) is not enabled.');
                     end;
                  elseif RMSChoice==7;
                     %
                     % Laplace approximation of marginal likelihood
                     %
                     % check if posterior mode results exist
                     %
                     if strcmp(get(controls.postsample.laplacepostmode,'Enable'),'on')==1;
                        if isempty(RecalculateInverseHessian)==1;
                           txt = 'Do you want YADA to recalculate the inverse Hessian at the posterior mode for the Laplace approximation of the marginal likelihood?';
                           answer = Query(strvcat(txt,' ','NOTE: The recalculation will use finite differences for the inverse Hessian. YADA will otherwise use the inverse Hessian from the posterior mode estimation output for the transformed parameters and if the finite difference estimator already exists it will be selected.'),'question',400,'Laplace Approximation - Posterior Mode',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecalculateInverseHessian = 0;
                           else;
                              RecalculateInverseHessian = 1;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        SequentialLaplaceMarginalLikelihood(DSGEModel,CurrINI,controls,RecalculateInverseHessian);
                     else;
                        SetLogText('Laplace approximation estimation is not enabled.');
                     end;
                  elseif RMSChoice==8;
                     %
                     % run the DSGE continuous ranked probability score (CRPS) and energy scores (ES)
                     %
                     if strcmp(get(controls.predict.unconditional.original.crps.posterior,'Enable'),'on')==1;
                        %
                        % check if the predictive likelihoods should be recalculated when old values exist.
                        %
                        if isempty(RecomputeCRPS)==1;
                           txt = 'If YADA locates posterior calculations of the continuous ranked probability score (CRPS) and the energy score (ES) on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Predictive Distributions (CRPS) - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputeCRPS = 1;
                           else;
                              RecomputeCRPS = 0;
                           end;
                        end;
                        %
                        % try to run the estimations
                        %
                        [Status,IsOriginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData,UsePaths,Share] = SequentialCRPS(DSGEModel,CurrINI,controls,IsOriginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData,RecomputeCRPS,UsePaths,Share);
                        if Status==-1;
                           break;
                        end;
                     else;
                        SetLogText('CRPS estimation is not enabled.');
                     end;
                  elseif RMSChoice==9;
                     %
                     % check if marginal or joint posterior mode estimation should be performed
                     %
                     if isempty(IsMarginal)==1;
                        txt = 'Please select if YADA should estimate DSGE-VAR models at the marginal or the joint posterior mode.';
                        answer = Query(txt,'question',200,'DSGE-VAR Posterior Mode Estimation',550,'yes',CurrINI,{'Marginal' 'Joint'});
                        if strcmp(lower(answer),'marginal')==1;
                           IsMarginal = 1;
                           MarginalLambda = [];
                        else;
                           IsMarginal = 0;
                           JointLambda = [];
                        end;
                     end;
                     %
                     % run the DSGE-VAR posterior mode estimation.
                     %
                     if strcmp(get(controls.dsgevarmargmodemenu,'Enable'),'on')==1;
                        if IsMarginal==1;
                           MarginalLambda = DSGEVARMargPosteriorModeEstimation(DSGEModel,CurrINI,controls,MarginalLambda);
                        else;
                           JointLambda = DSGEVARPosteriorModeEstimation(DSGEModel,CurrINI,controls,JointLambda);
                        end;
                     else;
                        SetLogText('Posterior mode estimation for DSGE-VARs is not enabled.');
                     end;
                  elseif RMSChoice==10;
                     %
                     % run posterior sampling of DSGE model parameters with DSGE-VARs for the sequence
                     %
                     if strcmp(get(controls.dsgevarpostsamplingmenu,'Enable'),'on')==1;
                        if DSGEModel.PosteriorSampler==1;
                           %
                           % random walk Metropolis with normal proposal density
                           %
                           [SelectedMode,lambdai] = DSGEVARRWMPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==2;
                           %
                           % slice sampler
                           %
                           [SelectedMode,lambdai] = DSGEVARSlicePosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==3;
                           %
                           % random walk Metropolis with Student-t proposal density
                           %
                           [SelectedMode,lambdai] = DSGEVARRWMStudentPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==4;
                           %
                           % fixed blocking RWM with normal proposal
                           %
                           [SelectedMode,lambdai] = DSGEVARFixedBlockingRWMPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==5;
                           %
                           % fixed blocking RWM with Student-t proposal
                           %
                           [SelectedMode,lambdai] = DSGEVARFixedBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==6;
                           %
                           % random blocking RWM with normal proposal
                           %
                           [SelectedMode,lambdai] = DSGEVARRandomBlockingRWMPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==7;
                           %
                           % random blocking RWM with Student-t proposal
                           %
                           [SelectedMode,lambdai] = DSGEVARRandomBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==8;
                           %
                           % SMC with likelihood tempering
                           %
                           [SelectedMode,lambdai] = DSGEVARSMCLikelihoodTemperingPosteriorSampler(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==9;
                           %
                           % SMC with data tempering
                           %
                           [SelectedMode,lambdai] = DSGEVARSMCDataTemperingPosteriorSampler(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==10;
                           %
                           % IS based on MitISEM
                           %
                           [SelectedMode,lambdai] = DSGEVARISMitISEMPosteriorSampler(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        end;
                     else;
                        SetLogText('Posterior sampling of DSGE model parameters for DSGE-VAR is not enabled.');
                     end;
                  elseif RMSChoice==11;
                     %
                     % run posterior sampling of VAR parameters with DSGE-VARs for the sequence
                     %
                     if strcmp(get(controls.dsgevarvarpostsamplingmenu,'Enable'),'on')==1;
                        [value,CurrChain,PME,Lambda] = DSGEVARPosteriorSampling(DSGEModel,CurrINI,maingui,controls,value,CurrChain,PME,Lambda);
                     else;
                        SetLogText('Posterior sampling of VAR parameters for DSGE-VAR is not enabled.');
                     end;
                  elseif RMSChoice==12;
                     %
                     % run the DSGE predictive likelihood
                     %
                     if strcmp(get(controls.dsgevartools.forecast.unconditional.original.height.posterior,'Enable'),'on')==1;
                        %
                        % check if the predictive likelihoods should be recalculated when old values exist.
                        %
                        if isempty(RecomputePL)==1;
                           txt = 'If YADA locates posterior calculations of the log predictive likelihood for DSGE-VARs on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Predictive Likelihood - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputePL = 1;
                           else;
                              RecomputePL = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,value,CurrChain,PME,Lambda,IsOriginal,IsMarginal,PredictionVariables,SelectedYData] = SequentialPredictiveLikelihoodDSGEVAR(DSGEModel,CurrINI,controls,value,CurrChain,PME,Lambda,IsOriginal,IsMarginal,PredictionVariables,SelectedYData,RecomputePL);
                        if Status==-1;
                           break;
                        end;
                     else;
                        SetLogText('Predictive likelihood calculation is not enabled.');
                     end;
                  elseif RMSChoice==13;
                     %
                     % check if we can compute predictive moments for DSGE-VARs
                     %
                     if strcmp(get(controls.dsgevartools.forecast.unconditional.original.posterior,'Enable'),'on')==1;
                        %
                        % check if unconditional predictive moments should be recalculated when old values exist
                        %
                        if isempty(RecomputeMoments)==1;
                           txt = 'If YADA locates posterior calculations of the marginal predictive moments for DSGE-VARs on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Marginal Predictive Moments - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputeMoments = 1;
                           else;
                              RecomputeMoments = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,value,CurrChain,PME,Lambda] = SequentialPredictiveMomentsDSGEVAR(DSGEModel,CurrINI,controls,value,CurrChain,PME,Lambda,RecomputeMoments);
                     else;
                        SetLogText('Unconditional marginal predictive moment estimation is not enabled.');
                     end;
                  elseif RMSChoice==14;
                     %
                     % check if we posterior draws of the DSGE parameters
                     %
                     if strcmp(get(controls.dsgevarvarpostsamplingmenu,'Enable'),'on')==1;
                        if isempty(RecomputePosteriorKernel)==1;
                           if DSGEModel.PosteriorSampler<=7;
                              txt = 'If YADA locates posterior calculations of the marginal likelihood on disk, would you like to use these results?';
                              answer = Query(txt,'question',200,'Marginal Likelihood Estimation',500,'no',CurrINI);
                              if strcmp(lower(answer),'no')==1;
                                 RecomputeML = 1;
                              else;
                                 RecomputeML = 0;
                              end;
                           else;
                              RecomputeML = 0;
                           end;
                        end;
                        %
                        [Status,value,CurrChain,PME,Lambda,IsSubSample,RecomputePosteriorKernel] = SequentialMarginalLikelihoodDSGEVAR(DSGEModel,CurrINI,controls,value,CurrChain,PME,Lambda,IsSubSample,RecomputePosteriorKernel,RecomputeML);
                     else;
                        SetLogText('Unable to compute the marginal likelihood. Posterior draws are not available.');
                     end;
                  elseif RMSChoice==15;
                     %
                     % check if the user wants to recompute the log marginal likelihood
                     %
                     if isempty(RecomputeLML)==1;
                        txt = 'Do you want YADA to recompute the Laplace approximation of the marginal likelihood?';
                        answer = Query(strvcat(txt,' ','NOTE: The recomputation will use finite differences for the inverse Hessian. YADA will otherwise use the inverse Hessian from the posterior mode estimation output for the transformed parameters and if the finite difference estimator already exists it will be selected.'),'question',400,'Laplace Approximation - Posterior Mode',500,'no',CurrINI);
                        if strcmp(lower(answer),'no')==1;
                           RecomputeLML = 0;
                        else;
                           RecomputeLML = 1;
                        end;
                     end;
                     %
                     [PME,Lambda] = SequentialDSGEVARLaplaceMarginalLikelihood(DSGEModel,CurrINI,controls,PME,Lambda,RecomputeLML);
                     %
                  elseif RMSChoice==16;
                     %
                     % run DSGE posterior mode estimation under adaptive learning for the sequence
                     %
                     if strcmp(get(controls.learning.postmodemenu,'Enable'),'on')==1;
                        AdaptiveLearningPosteriorModeEstimation(DSGEModel,CurrINI,controls);
                     else;
                        SetLogText('Posterior mode estimation is not enabled.');
                     end;
                  elseif RMSChoice==17;
                     %
                     % run DSGE posterior sampling under adaptive learning for the sequence
                     %
                     if strcmp(get(controls.learning.posteriorsamplingmenu,'Enable'),'on')==1;
                        if DSGEModel.PosteriorSampler==1;
                           %
                           % random walk Metropolis with normal proposal
                           %
                           AdaptiveLearningDSGERWMPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==2;
                           %
                           % slice sampler
                           %
                           AdaptiveLearningDSGESlicePosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==3;
                           %
                           % random walk Metropolis with Student-t proposal
                           %
                           AdaptiveLearningDSGERWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==4;
                           %
                           % fixed blocking RWM with normal proposal
                           %
                           AdaptiveLearningDSGEFixedBlockingRWMPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==5;
                           %
                           % fixed blocking RWM with a Student-t proposal
                           %
                           AdaptiveLearningDSGEFixedBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==6;
                           %
                           % random blocking RWM with a normal proposal
                           %
                           AdaptiveLearningDSGERandomBlockingRWMPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==7;
                           %
                           % random blocking RWM with a Student-t proposal
                           %
                           AdaptiveLearningDSGERandomBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==8;
                           %
                           % SMC with likelihood tempering
                           %
                           AdaptiveLearningDSGESMCLikelihoodTemperingPosteriorSampler(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==9;
                           %
                           % SMC with data tempering
                           %
                           AdaptiveLearningDSGESMCDataTemperingPosteriorSampler(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==10;
                           %
                           % IS based on MitISEM
                           %
                           AdaptiveLearningDSGEISMitISEMPosteriorSampler(DSGEModel,CurrINI,controls);
                        end;
                     else;
                        SetLogText('Posterior sampling is not enabled.');
                     end;
                  elseif RMSChoice==18;
                     %
                     % run the DSGE predictive likelihood under adaptive learning
                     %
                     if strcmp(get(controls.learningtools.predictivedist.unconditional.original.height.posterior,'Enable'),'on')==1;
                        %
                        % check if the predictive likelihoods should be recalculated when old values exist.
                        %
                        if isempty(RecomputePL)==1;
                           txt = 'If YADA locates posterior calculations of the log predictive likelihood under adaptive learning on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Predictive Likelihood - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputePL = 1;
                           else;
                              RecomputePL = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,IsOriginal,IsMarginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData] = SequentialAdaptiveLearningPredictiveLikelihoodEstimation(DSGEModel,CurrINI,controls,IsOriginal,IsMarginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData,RecomputePL);
                        if Status==-1;
                           break;
                        end;
                     else;
                        SetLogText('Predictive likelihood calculation under adaptive learning is not enabled.');
                     end;
                  elseif RMSChoice==19;
                     %
                     % check if we can compute predictive moments
                     %
                     if strcmp(get(controls.learningtools.predictivedist.unconditional.original.moments.posterior,'Enable'),'on')==1;
                        %
                        % check if unconditional predictive moments should be recalculated when old values exist
                        %
                        if isempty(RecomputeMoments)==1;
                           txt = 'If YADA locates posterior calculations of the marginal predictive moments on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Marginal Predictive Moments - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputeMoments = 1;
                           else;
                              RecomputeMoments = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,CurrChain,IsSubSample,Share] = SequentialAdaptiveLearningPredictiveMoments(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,RecomputeMoments,Share);
                     else;
                        SetLogText('Unconditional marginal predictive moment estimation is not enabled.');
                     end;
                  elseif RMSChoice==20;
                     %
                     % check if we can compute the marginal likelihood for DSGE models
                     %
                     if strcmp(get(controls.learning.viewmenu.postsample.rawplots.logpost,'Enable'),'on')==1;
                        if isempty(RecomputePosteriorKernel)==1;
                           if DSGEModel.PosteriorSampler<=7;
                              txt = 'If YADA locates posterior calculations of the marginal likelihood on disk, would you like to use these results?';
                              answer = Query(txt,'question',200,'Marginal Likelihood Estimation',500,'no',CurrINI);
                              if strcmp(lower(answer),'no')==1;
                                 RecomputeML = 1;
                              else;
                                 RecomputeML = 0;
                              end;
                           else;
                              RecomputeML = 0;
                           end;
                        end;
                        %
                        [Status,CurrChain,IsSubSample,RecomputePosteriorKernel] = SequentialAdaptiveLearningMarginalLikelihood(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,RecomputePosteriorKernel,RecomputeML);
                     else;
                        SetLogText('Unable to compute the marginal likelihood. Posterior draws are not available.');
                     end;
                  elseif RMSChoice==21;
                     %
                     % check if we can compute predictive moments for PITs
                     %
                     if strcmp(get(controls.learningtools.predictivedist.unconditional.original.pit.posterior,'Enable'),'on')==1;
                        %
                        % check if unconditional predictive moments along with the PITs should be recalculated when old values exist
                        %
                        if isempty(RecomputePITs)==1;
                           txt = 'If YADA locates posterior draws based estimates of the moments on disk that can be used for the probability integral transform (PIT), would you like to use these results?';
                           answer = Query(txt,'question',200,'Probability Integral Transform - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputePITs = 1;
                           else;
                              RecomputePITs = 0;
                           end;
                        end;
                        %
                        % check if path based PITs should also be computed
                        %
                        if isempty(ComputePredPaths)==1;
                           txt = strvcat('YADA can also estimate probability integral transform (PIT) values based on simulated paths from the predictive distribution. Would you like to run these calculations as well?',' ','NOTE: If you answer YES, then YADA will recompute the prediction paths even if they already exist on disk.');
                           answer = Query(txt,'question',200,'Probability Integral Transform - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              ComputePredPaths = 1;
                           else;
                              ComputePredPaths = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,CurrChain,IsSubSample,RecomputePITs,ComputePredPaths,PredictionVariables,SelectedYData,Share] = SequentialAdaptiveLearningPITsEstimation(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,RecomputePITs,ComputePredPaths,PredictionVariables,SelectedYData,Share);
                     else;
                        SetLogText('Posterior estimation of the probability integral transforms (PITs) is not enabled.');
                     end;
                  elseif RMSChoice==22;
                     %
                     % Laplace approximation of marginal likelihood
                     %
                     % check if posterior mode results exist
                     %
                     if strcmp(get(controls.learning.viewmenu.postsample.laplace.postmode,'Enable'),'on')==1;
                        if isempty(RecalculateInverseHessian)==1;
                           txt = 'Do you want YADA to recalculate the inverse Hessian at the posterior mode for the Laplace approximation of the marginal likelihood?';
                           answer = Query(strvcat(txt,' ','NOTE: The recalculation will use finite differences for the inverse Hessian. YADA will otherwise use the inverse Hessian from the posterior mode estimation output for the transformed parameters and if the finite difference estimator already exists it will be selected.'),'question',400,'Laplace Approximation - Posterior Mode',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecalculateInverseHessian = 0;
                           else;
                              RecalculateInverseHessian = 1;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        SequentialAdaptiveLearningLaplaceMarginalLikelihood(DSGEModel,CurrINI,controls,RecalculateInverseHessian);
                     else;
                        SetLogText('Laplace approximation estimation is not enabled.');
                     end;
                  elseif RMSChoice==23;
                     %
                     % run the DSGE continuous ranked probability score (CRPS) and energy scores (ES)
                     %
                     if strcmp(get(controls.learningtools.predictivedist.unconditional.original.crps.posterior,'Enable'),'on')==1;
                        %
                        % check if the predictive likelihoods should be recalculated when old values exist.
                        %
                        if isempty(RecomputeCRPS)==1;
                           txt = 'If YADA locates posterior calculations of the continuous ranked probability score (CRPS) and the energy score (ES) on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Predictive Distributions (CRPS) - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputeCRPS = 1;
                           else;
                              RecomputeCRPS = 0;
                           end;
                        end;
                        %
                        % try to run the estimations
                        %
                        [Status,IsOriginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData,UsePaths,Share] = SequentialAdaptiveLearningCRPS(DSGEModel,CurrINI,controls,IsOriginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData,RecomputeCRPS,UsePaths,Share);
                        if Status==-1;
                           break;
                        end;
                     else;
                        SetLogText('CRPS estimation is not enabled.');
                     end;
                  else;
                     %
                     % trying to run the driver functionality
                     %
                     DriverFunctions = DSGEModel.DriverFunctions;
                     OldDSGEModel = DSGEModel;
                     %
                     % setup the sequence of functions
                     %
                     NumFiles = size(DriverFunctions,1);
                     if NumFiles>=1;
                        for i=1:NumFiles;
                           %
                           % try to run each function
                           %
                           MatFile = FixFilePath([pwd '\driver\' StringTrim(DriverFunctions(i,:))]);
                           %
                           % make sure that set(controls.about,'UserData',0); is set before
                           % and set(controls.about,'UserData',1); after running the function
                           %
                           DriverData = load(MatFile);
                           cmd = DriverData.cmd;
                           drivertxt = DriverData.drivertxt;
                           handletxt = DriverData.handle;
                           %
                           % check if the handle is enabled
                           %
                           eval(['CurrHandle = ' handletxt ';']);
                           if IsHandle(CurrHandle)==1;
                              if strcmp(get(CurrHandle,'Enable'),'on')==1;
                                 DSGEModel.RunDriver = 1;
                                 DSGEModel.answers = DriverData.answers;
                                 DSGEModel.ShowProgress = 0;
                                 set(controls.open,'UserData',DSGEModel);
                                 SetLogText(['* Running: ' drivertxt]);
                                 set(controls.about,'UserData',0);
                                 %
                                 % now we try to run the function
                                 %
                                 if RunSequentialDriver==0;
                                    eval([cmd ';']);
                                 else;
                                    disp(['Running: ' drivertxt]);
                                    cmdMatrix = Str2Matrix([cmd ' '],' ');
                                    cmdstr = [StringTrim(cmdMatrix(1,:)) '(''' StringTrim(cmdMatrix(2,:)) ''',maingui)'];
                                    eval([cmdstr ';']);
                                 end;
                                 set(controls.about,'UserData',1);
                              else;
                                 SetLogText(strvcat(['* The driver function "' drivertxt '" is not'],'   enabled on the relevant menu item. Unable to run function.'));
                              end;
                           else;
                              SetLogText(strvcat(['* The driver function "' drivertxt '" is not'],'   supported, i.e., the related handle on the menu item for the','   function is not proper.'));
                           end;
                           %
                           % reset everything
                           %
                           DSGEModel = OldDSGEModel;
                           set(controls.open,'UserData',DSGEModel);
                        end;
                     else;
                        if RunSequentialDriver==0;
                           SetLogText('No driver functions have been selected.');
                        else;
                           disp('No driver functions have been selected.');
                        end;
                     end;
                     %
                  end;
               end;
               %
               % reset everything
               %
               CurrINI.Sequence = '';
               %
               % reload the original file: CurrINI.CurrentModelFile
               %
               CurrINI.CurrentModelFile = CurrentModelFile;
               set(controls.filemenu,'UserData',CurrINI);
               %
               % make sure we can run this function
               %
               set(controls.about,'UserData',0);
               if RunSequentialDriver==0;
                  FileMenuFunctions('ReloadModel');
               else;
                  FileMenuFunctions('ReloadModel',maingui);
               end;
               set(controls.about,'UserData',1);
               DSGEModel = get(controls.open,'UserData');
               controls = get(maingui,'UserData');
               if isempty(oldtab)==0;
                  SetLogText('Done!');
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'DriverSelection'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [action,DriverFunctions] = DriverSelectionDLG('init',DSGEModel,CurrINI);
         if strcmp(lower(action),'ok')==1;
            DSGEModel.DriverFunctions = DriverFunctions;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'RunDriverFunctions'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if there are any functions listed in the driver.
         %
         if isempty(DSGEModel.DriverFunctions)==0;
            if isempty(DSGEModel.NameOfModel)==0;
               [status,DiffField] = CompareDSGEModel(DSGEModel,OrigDSGEModel);
               if status==0;
                  txt = strvcat(['Your DSGE model settings have changed. The field "' DiffField '" differs. Are you sure you want to open a new model without saving the current model "' DSGEModel.NameOfModel '" first?'],' ','NOTE: The field names are explained in the file "ExtendingYADA.pdf" in the "doc" directory.');
                  answer = Query(txt,'question',200,'Run Driver Functions',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
            else;
               answer = 'Yes';
            end;
            %
            if strcmp(lower(answer),'yes')==1;
               %
               % setup the sequential run
               %
               DriverFunctions = DSGEModel.DriverFunctions;
               OldDSGEModel = DSGEModel;
               %
               % setup the sequence of modes
               %
               NumFiles = size(DriverFunctions,1);
               oldtab = [];
               for i=1:NumFiles;
                  if i==1;
                     if DSGEModel.ShowEstimationLog==1;
                        set(controls.about,'UserData',0);
                        oldtab = get(controls.tab1,'UserData');
                        set(maingui,'CurrentObject',controls.tab6);
                        YADAGUI('tab');
                        if MatlabNumber>=7;
                           uicontrol(controls.tab6);
                        end;
                        set(controls.about,'UserData',1);
                        drawnow;
                     end;
                     %
                     % check if we should add some diving line to the log-text control
                     %
                     LogText = get(controls.estimation_log,'String');
                     if isempty(LogText)==0;
                        SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************',' '));
                     end;
                     SetLogText(strvcat('Running YADA''s driver function...',' '));
                  end;
                  %
                  % try to run each function
                  %
                  MatFile = FixFilePath([pwd '\driver\' StringTrim(DriverFunctions(i,:))]);
                  %
                  % make sure that set(controls.about,'UserData',0); is set before
                  % and set(controls.about,'UserData',1); after running the function
                  %
                  DriverData = load(MatFile);
                  cmd = DriverData.cmd;
                  drivertxt = DriverData.drivertxt;
                  handletxt = DriverData.handle;
                  %
                  % check if the handle is enabled
                  %
                  eval(['CurrHandle = ' handletxt ';']);
                  if IsHandle(CurrHandle)==1;
                     if strcmp(get(CurrHandle,'Enable'),'on')==1;
                        DSGEModel.RunDriver = 1;
                        DSGEModel.answers = DriverData.answers;
                        DSGEModel.ShowProgress = 0;
                        set(controls.open,'UserData',DSGEModel);
                        if isempty(oldtab)==0;
                           SetLogText(['* Running: ' drivertxt]);
                        end;
                        set(controls.about,'UserData',0);
                        %
                        % now we try to run the function
                        %
                        if nargin==1;
                           eval([cmd ';']);
                        else;
                           disp(['Running: ' drivertxt]);
                           cmdMatrix = Str2Matrix([cmd ' '],' ');
                           cmdstr = [StringTrim(cmdMatrix(1,:)) '(''' StringTrim(cmdMatrix(2,:)) ''',maingui)'];
                           eval([cmdstr ';']);
                        end;
                        %
                        % create the necessary command to include maingui
                        %
                        set(controls.about,'UserData',1);
                     else;
                        SetLogText(strvcat(['* The driver function "' drivertxt '" is not'],'   enabled on the relevant menu item. Unable to run function.'));
                     end;
                  else;
                     SetLogText(strvcat(['* The driver function "' drivertxt '" is not'],'   supported, i.e., the related handle on the menu item for the','   function is not proper.'));
                  end;
               end;
               %
               % reset everything
               %
               DSGEModel = OldDSGEModel;
               if isempty(oldtab)==0;
                  SetLogText('Done!');
               end;
            end;
         else;
            txt = 'Your driver selection is empty. Please select the functions from the "Select Driver Functions" dialog before you try to run them.';
            About(txt,'information','Run Driver Functions',300,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'RunMatlabScript'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % select a script function
         %
         ScriptFile = DSGEModel.RunScriptFile;
         if DSGEModel.RunDriver==0;
            if FileExist(ScriptFile)==0;
               ScriptFile = FixFilePath([ScriptFile '\*.m']);
            end;
            if MatlabNumber>=6;
               FileType = {'*.m', 'Matlab Script Files (*.m)'};
               [FileName,FilePath] = uigetfile(FileType,'Matlab Script File - Select File',ScriptFile);
            else;
               [FileName,FilePath] = uigetfile(ScriptFile,'Matlab Script File - Select File');
            end;
            %
            % fix the path if necessary
            %
            if (isnumeric(FileName)==0)&(isunix==1);
               if strcmp(FilePath(1,length(FilePath)),'/')==0;
                  FilePath = [FilePath '/'];
               end;
            end;
         else;
            if FileExist(ScriptFile)==1;
               [FilePath,FileName,FileExt] = fileparts(ScriptFile);
               FileName = [FileName FileExt];
               if ispc==1;
                  FilePath = [FilePath '\'];
               else;
                  if strcmp(FilePath(1,length(FilePath)),'/')==0;
                     FilePath = [FilePath '/'];
                  end;
               end;
            else;
               FileName = '';
               FilePath = '';
               if (isempty(GetFilename(ScriptFile))==0)&(strcmp(lower(ScriptFile),lower(pwd))==0);
                  SetLogText(strvcat(['* The scriptfile "' GetFilename(ScriptFile) '" doesn''t exist.'],' '));
               else;
                  SetLogText(strvcat('* You haven''t selected a script file.',' '));
               end;
            end;
         end;
         %
         % check if a file was selected
         %
         if (isnumeric(FileName)==0)&(FileExist([FilePath FileName])==1)&(strcmp(lower(GetExt(FileName)),'.m')==1);
            %
            % try to run it from its directory
            %
            ErrorStr = '';
            Message = '';
            CurrPath = pwd;
            cd(FilePath);
            Name = GetName(FileName);
            if MatlabNumber>=8.0;
               try;
                  Message = evalc(GetName(FileName));
               catch ME;
                  ErrorStr = ['Unable to run the file "' FileName '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  Message = evalc(GetName(FileName));
               catch;
                  ErrorStr = ['Unable to run the file "' FileName '". Message caught is: ' YADALastError];
               end;
            end;
            cd(CurrPath);
            if isempty(ErrorStr)==1;
               DSGEModel.RunScriptFile = [FilePath FileName];
               if isempty(Message)==0;
                  if DSGEModel.RunDriver==0;
                     txt = ['The matkab script "' FileName '" was successfully executed. YADA captured a message from the file. Would you like to view it?'];
                     answer = Query(txt,'question',200,['Run Matlab Script File - ' FileName],600,'yes',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        About(deblank(Message),'information',['Run Matlab Script File - ' FileName],CurrINI.scrsz(4)-50,600,CurrINI)
                     end;
                  else;
                     %
                     % trim empty rows from Message
                     %
                     SetLogText(strvcat('Script output message:',' ',deblank(Message),' ',['The matkab script "' FileName '" was successfully executed.'],' '));
                  end;
               else;
                  if DSGEModel.RunDriver==0;
                     txt = ['The matkab script "' FileName '" was successfully executed.'];
                     About(txt,'information',['Run Matlab Script File - ' FileName],200,600,CurrINI);
                  else;
                     SetLogText(strvcat(['* Finished script "' FileName '" successfully.'],' '));
                  end;
               end;
            else;
               if DSGEModel.RunDriver==0;
                  About(ErrorStr,'error',['Error - ' FileName],CurrINI.scrsz(4)-50,600,CurrINI);
               else;
                  SetLogText(strvcat(ErrorStr,' '));
               end;
            end;
         elseif (isnumeric(FileName)==0)&(strcmp(lower(GetExt(FileName)),'.m')==0);
            if DSGEModel.RunDriver==0;
               txt = ['The file you selected "' FileName '" is not a matlab script file. YADA is therefore unwilling to run it.'];
               About(txt,'information',['Bad File Extension - ' FileName],300,600,CurrINI);
            else;
               if isempty(FileName)==0;
                  SetLogText(strvcat(['The file you selected "' FileName '" is not a matlab script file.'],' '));
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SaveInverseHessian'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % We need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI,0);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         if InitStatus==1;
            %
            % check if we have parameters to estimate
            %
            if isempty(ParameterNames.estimated)==0;
               %
               % select parameter values from a mat-file
               %
               if MatlabNumber>=6;
                  ImportValuesFile = FixFilePath([DSGEModel.OutputDirectory '\']);
               else;
                  ImportValuesFile = FixFilePath([DSGEModel.OutputDirectory '\*.mat']);
               end;
               if MatlabNumber>=6;
                  FileType = {'*.mat', 'Matlab mat Files (*.mat)'};
                  [FileName,FilePath] = uigetfile(FileType,'Parameter Values File - Select File',ImportValuesFile);
               else;
                  [FileName,FilePath] = uigetfile(ImportValuesFile,'Parameter Values File - Select File');
               end;
               drawnow;
               pause(0.02);
               %
               if (isnumeric(FileName)==0)&(isunix==1);
                  if strcmp(FilePath(1,length(FilePath)),'/')==0;
                     FilePath = [FilePath '/'];
                  end;
               end;
               %
               % check if a file was selected
               %
               if (isnumeric(FileName)==0)&(FileExist([FilePath FileName])==1);
                  ImportValuesFile = [FilePath FileName];
                  %
                  % try to load the file
                  %
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ImportValuesData = load(ImportValuesFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ImportValuesFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ImportValuesData = load(ImportValuesFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ImportValuesFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  %
                  if isempty(ErrorStr)==1;
                     %
                     % check which type of file we have
                     %
                     SelectValues = 1;
                     if (length(FileName)>=17)&(strcmp(FileName(1,1:13),'MargLikeData-')==1);
                        %
                        % we have a file based on posterior sampling
                        %
                        ParameterValuesChoices = strvcat('thetaMargMode','ParData.SampleMean','ParData.SampleMedian');
                        DisplayChoices = strvcat('Marginal posterior theta mode','Posterior theta mean','Posterior theta median');
                     elseif (length(FileName)>=18)&(strcmp(FileName(1,1:14),'PosteriorMode-')==1);
                        %
                        % we have a posterior mode file
                        %
                        ParameterValuesChoices = strvcat('theta','thetaMode');
                        DisplayChoices = strvcat('Initial theta values','Joint posterior theta mode');
                     else;
                        %
                        % we have some other file
                        %
                        if length(fieldnames(ImportValuesData))>0;
                           DisplayChoices = char(fieldnames(ImportValuesData));
                           ParameterValuesChoices = DisplayChoices;
                        else;
                           SelectValues = 0;
                           txt = ['The file you selected "' GetFilename(ImportValuesFile) '" is empty. Hence, there are no parameters for which an inverse Hessian matrix can be computed.'];
                           About(txt,'information',['Save Inverse Hessian - ' GetFilename(ImportValuesFile)],300,600,CurrINI);
                        end;
                     end;
                     %
                     % select which variable is the appropriate choice
                     %
                     if SelectValues==1;
                        if size(DisplayChoices,1)>1;
                           [action,SelectedOption] = SelectionDlg(DisplayChoices,1,'Select variable for theta values:','Theta Values Selection','Save Inverse Hessian','','',CurrINI);
                        else;
                           action = 'ok';
                           SelectedOption = 1;
                        end;
                        if strcmp(lower(action),'ok')==1;
                           thetaMode = vec(eval(['ImportValuesData.' StringTrim(ParameterValuesChoices(SelectedOption,:))]));
                           %
                           % check if the length of thetaMode is consistent with theta
                           %
                           NumParam = length(theta);
                           if length(thetaMode)==NumParam;
                              %
                              % covert to phi
                              %
                              phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
                              %
                              % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
                              %
                              [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
                              [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
                              if FileExist(DSGEModel.UpdateParameterFile)==1;
                                 [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
                              end;
                              if FileExist(DSGEModel.InitializeParameterFile)==1;
                                 [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
                              end;
                              [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
                              %
                              % check if we should copy model solver file
                              %
                              if DSGEModel.ModelSolver==4;
                                 if FileExist(DSGEModel.ModelSolverFile)==1;
                                    [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
                                 end;
                              end;
                              try;
                                 AIMData = load(DSGEModel.AIMDataFile);
                              catch;
                                 AIMData = '';
                              end;
                              if isempty(AIMData)==0;
                                 ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
                                 %
                                 % collect variables required for the calculations
                                 %
                                 StepLengthStr = get(controls.optimize.steplength,'String');
                                 StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                                 %
                                 [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                                 %
                                 if mcode==1;
                                    %
                                    % get the estimation sample
                                    %
                                    [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
                                    %
                                    % set the InitialStateVector value
                                    %
                                    if DSGEModel.UseOwnInitialState==1;
                                       if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
                                          DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
                                       else;
                                          DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                                       end;
                                    else;
                                       DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                                    end;
                                    %
                                    % try to compute the log posterior kernel
                                    %
                                    ErrorStr = '';
                                    if MatlabNumber>=8.0;
                                       try;
                                          logPost = logPosteriorPhiDSGE(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                       catch;
                                          ErrorStr = ['Unable to calculate the log posterior at the selected theta value. Message caught is: ' YADALastError(ME)];
                                       end;
                                    else;
                                       try;
                                          logPost = logPosteriorPhiDSGE(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                       catch;
                                          ErrorStr = ['Unable to calculate the log posterior at the selected theta value. Message caught is: ' YADALastError];
                                       end;
                                    end;
                                    if isempty(ErrorStr)==1;
                                       if (exist('matlabpool')==2)|(exist('parpool')==2);
                                          if CurrINI.NumWorkers==0;
                                             if isempty(CurrINI.Sequence)==1;
                                                txt = 'Would you like to use parallel computations when computing the inverse Hessian with finite differences?';
                                                answer = Query(txt,'question',500,'Computing Inverse Hessian',600,'yes',CurrINI);
                                             else;
                                                answer = 'Yes';
                                             end;
                                             if strcmp(lower(answer),'yes')==1;
                                                %
                                                % try to open a parallel pool
                                                %
                                                set(controls.about,'UserData',0);
                                                maingui = findobj('Type','figure','Tag','YADA');
                                                FileMenuFunctions('OpenPool',maingui);
                                                %
                                                CurrINI = get(controls.filemenu,'UserData');
                                                set(controls.about,'UserData',1);
                                             end;
                                          end;
                                          if (CurrINI.NumWorkers>1)&(exist('parpool')==2);
                                             poolobj = gcp('nocreate');
                                             if isempty(poolobj)==0;
                                                poolobj.IdleTimeout = Inf;
                                             end;
                                          end;
                                       end;
                                       %
                                       % compute the inverse Hessian
                                       %
                                       txt = ['Please wait while YADA attempts to compute the inverse Hessian at the selected parameter values using finite differences. Computations started at: ' StringTrim(datestr(now,14))];
                                       WaitHandle = WaitDLG(txt,'information','Computing Inverse Hessian',600,150,CurrINI,0);
                                       WaitControls = get(WaitHandle,'UserData');
                                       drawnow;
                                       tic;
                                       %
                                       VechHessian = YADAHessianMatrix('logPosteriorPhiDSGE',phiMode,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                       CompTime = toc;
                                       %
                                       set(WaitControls.text,'UserData','done');
                                       delete(WaitHandle);
                                       drawnow;
                                       pause(0.02);
                                       %
                                       HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
                                       %
                                       % check if the matrix is positive definite
                                       %
                                       if IsPositiveDefinite(HessianMat)==1;
                                          InverseHessian = HessianMat\eye(NumParam);
                                          %
                                          % create the directory hessian and save the results there
                                          %
                                          OutputDir = FixFilePath([DSGEModel.OutputDirectory '\hessian']);
                                          stat = MakeDir(DSGEModel.OutputDirectory,'hessian');
                                          if stat~=1;
                                             OutputDir = DSGEModel.OutputDirectory;
                                          end;
                                          %
                                          % save to file
                                          %
                                          SaveNo = 1;
                                          DoSave = 0;
                                          while DoSave==0;
                                             MatFile = FixFilePath([OutputDir '\InverseHessian-' DSGEModel.NameOfModel '-' int2str(SelectedOption) '.' int2str(SaveNo) '.mat']);
                                             if FileExist(MatFile)==0;
                                                %
                                                % save to disk
                                                %
                                                save(MatFile,'thetaMode','phiMode','InverseHessian','ParameterNames','DSGEModel');
                                                DoSave = 1;
                                                txt = ['YADA has saved the inverse Hessian at the selected theta value and the additional variables to the file "' GetFilename(MatFile) '", which is located in the directory "' OutputDir '". Computation time: ' SecondsToHoursStr(CompTime) ' (hrs:min:sec)'];
                                                About(txt,'information','Save Inverse Hessian',300,600,CurrINI);
                                             else;
                                                SaveNo = SaveNo+1;
                                             end;
                                          end;
                                       else;
                                          txt = 'The Hessian matrix computed from your selected theta vector is not positive definite. YADA is therefore unwilling to store the results.'
                                          About(txt,'information','YADA - DSGE Model Problem',500,600,CurrINI);
                                       end;
                                    else;
                                       About(ErrorStr,'error','Error - Log Posterior',500,600,CurrINI);
                                    end;
                                 else;
                                    if isempty(ErrMsg)==1;
                                       ErrMsg = 'YADA was unable to solve the model under rational expectations for the selected parameter values.';
                                    end;
                                    About(ErrMsg,'information','YADA - DSGE Model Problem',500,600,CurrINI);
                                 end;
                              else;
                                 if MatlabNumber>=8.0;
                                    txt = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
                                 else;
                                    txt = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
                                 end;
                                 About(txt,'information','YADA - Bad MAT File',500,600,CurrINI);
                              end;
                           else;
                              txt = ['The number of elements of the variable you selected "' StringTrim(DisplayChoices(SelectedOption,:)) '" is ' int2str(length(thetaMode)) ' and it doesn''t match the number of parameters in theta ' int2str(NumParam) '. YADA is therefore unable to compute and save a suitable inverse Hessian.'];
                              About(txt,'information','Save Inverse Hessian',300,600,CurrINI);
                           end;
                        end;
                     end;
                  else;
                     About(ErrorStr,'error',['Error - ' GetFilename(ImportValuesFile)],CurrINI.scrsz(4)-50,600,CurrINI);
                  end;
               end;
            else;
               txt = 'All parameters of your DSGE model have status calibrated. Hence, there are no parameters for which an inverse Hessian matrix can be computed.';
               About(txt,'information','Save Inverse Hessian - Calibrated DSGE Model',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'OpenPool'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % we could possibly extend the code here to allow for user-defined profiles.
         % these are determined via parallel.clusterProfiles in matlab 2012b
         %
         Message = '';
         IdleTimeoutStr = '';
         if exist('parpool')==2;
            try;
               %parpool;
               Message = evalc('parpool;');
               poolobj = gcp('nocreate');
               if isempty(poolobj)==0;
                  CurrINI.NumWorkers = poolobj.NumWorkers;
                  poolobj.IdleTimeout = Inf;
                  IdleTimeoutStr = 'IdleTimeout has been set to Inf for this session.';
               else;
                  CurrINI.NumWorkers = 0;
               end;
            catch;
               CurrINI.NumWorkers = 0;
            end;
         else;
            try;
               %matlabpool;
               Message = evalc('matlabpool;');
               CurrINI.NumWorkers = matlabpool('size');
            catch;
               CurrINI.NumWorkers = 0;
            end;
         end;
%         if exist('home')>0;
%            home;
%         end;
         if CurrINI.NumWorkers>1;
            %
            % try to turn off warnings on all workers
            %
            if exist('pctRunOnAll')~=0;
               eval('pctRunOnAll warning(''off'');')
            else;
               if exist('parfevalOnAll')~=0;
                  eval('parfevalOnAll(gcp(),@warning,0,''off'',''all'');');
               else;
                  if exist('spmd')~=0;
                     eval('spmd; warning(''off''); end;');
                  end;
               end;
            end;
         end;
         %
         % check if we should diplay a message
         %
         if isempty(Message)==0;
            pos = strfind(Message,'ans =');
            if isempty(pos)==0;
               Message = Message(1,1:max(pos(1)-1,1));
            end;
            pause(2);
            About(strvcat(Message,' ',IdleTimeoutStr),'information','Parallel Computing Toolbox',500,500,CurrINI);
            drawnow;
         end;
         %
         set(controls.parallel.closepool,'Enable','on');
         set(controls.parallel.openpool,'Enable','off');
         %
         set(controls.about,'UserData',0);
      end;
   case 'ClosePool'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         Message = '';
         if exist('parpool')==2;
            poolobj = gcp('nocreate');
            if isempty(poolobj)==0;
               %delete(poolobj);
               Message = evalc('delete(poolobj);');
            end;
            set(controls.parallel.closepool,'Enable','off');
            set(controls.parallel.openpool,'Enable','on');
         else;
            %matlabpool('close');
            Message = evalc('matlabpool(''close'')')
            set(controls.parallel.closepool,'Enable','off');
            set(controls.parallel.openpool,'Enable','on');
         end;
         CurrINI.NumWorkers = 0;
         %
         % check if we should display a message
         %
         if isempty(Message)==0;
            About(Message,'information','Parallel Computing Toolbox',300,500,CurrINI);
         end;
%         if exist('home')>0;
%            home;
%         end;
         set(controls.about,'UserData',0);
      end;
   case 'ImportSettings'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [status,ModelFile] = FileBrowse(CurrINI.CurrentModelFile,'Import Settings From DSGE Model File',[],'dsge',CurrINI);
         if (status==1)&(strcmp(lower(CurrINI.CurrentModelFile),lower(ModelFile))==0);
            ValidFile = 1;
            try;
               NewDSGEModel = ReadDSGEModel(ModelFile);
            catch;
               ValidFile = 0;
            end;
            if ValidFile==1;
               %
               % Change only certain settings on the DSGEModel structure
               %
               DSGEModel = ImportYADASettings(DSGEModel,NewDSGEModel,controls);
               %
               % update the dialog
               %
               SetYADAUIControls(maingui,DSGEModel,CurrINI.CurrentModelFile,CurrINI.images);
            else;
               txt = ['The DSGE model file you selected, "' GetFilename(ModelFile) '", has possibly been corrupted. YADA is therefore unable to import settings from it.'];
               About(txt,'information','Bad DSGE Model File',200,600,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'OpenTextFile'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [status,TextFile] = FileBrowse(FixFilePath([GetPath(CurrINI.CurrentModelFile) '*.txt']),'Open Text File',[],'txt',CurrINI);
         if status==1;
            if FileExist(TextFile)==1;
               TextGUI(TextFile,['Open Text File - ' TextFile],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Information',50,CurrINI,1,1);
               drawnow;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShowYADAIcon'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine if user wants to show or hide the icon
         %
         if CurrINI.ShowYADAIcon==1;
            CurrINI.ShowYADAIcon = 0;
            %
            try;
               f = figure('Visible','off');
               jframe = get(f,'javaframe');
               figIcon = get(jframe,'FigureIcon');
               delete(f);
               jframe = get(maingui,'javaframe');
               jframe.setFigureIcon(figIcon);
               txt = 'The alarm clock YADA icon is no longer shown on the main GUI or other dialogs that can display the YADA icon if your matlab version supports it.';
            catch;
               % do nothing
               txt = 'The alarm clock YADA icon will not be shown the next time you open YADA on the main GUI or other dialogs that can display the YADA icon.';
            end;
         else;
            CurrINI.ShowYADAIcon = 1;
            txt = 'The alarm clock YADA icon will now be shown if your matlab version supports it.';
            SetYADAIcon(maingui,CurrINI.ShowYADAIcon);
         end;
         %
         About(txt,'information','YADA Icon',200,600,CurrINI);
         %
         set(controls.about,'UserData',0);
      end;
   case 'PrintSetup'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % show the Print Setup dialog.
         %
         filemenufcn(maingui,'FilePrintSetup');
         %
         set(controls.about,'UserData',0);
      end;
   case 'CloseModel'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if DSGEModel and OrigDSGEModel structures are equal. If not suggest that DSGE model should be saved
         %
         if isempty(DSGEModel.NameOfModel)==0;
            [status,DiffField] = CompareDSGEModel(DSGEModel,OrigDSGEModel);
            if status==0;
               txt = strvcat(['Your DSGE model settings have changed. The field "' DiffField '" differs. Are you sure you want to open a new model without saving the current model "' DSGEModel.NameOfModel '" first?'],' ','NOTE: The field names are explained in the file "ExtendingYADA.pdf" in the "doc" directory.');
               answer = Query(txt,'question',200,'Close Model',500,'no',CurrINI);
            else;
               answer = 'Yes';
            end;
         else;
            answer = 'Yes';
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % clean up the tmp directory
            %
            YADADeleteTmpFiles(DSGEModel);
            %
            % Uncheck any item on the reload model menu
            %
            for i=1:size(CurrINI.ModelFiles,1);
               if (isfield(controls,['openmodel' num2str(i,'%0.0f')])==1)&(IsHandle(getfield(controls,['openmodel' num2str(i,'%0.0f')]))==1);
                  set(getfield(controls,['openmodel' num2str(i,'%0.0f')]),'Checked','off');
               else;
                  break;
               end;
            end;
            %
            % reset all controls and initialize the DSGEModel structure
            %
            DSGEModel = InitializeDSGEModel;
            [DSGEModel.SubBeginYearValue,DSGEModel.SubBeginPeriodValue,DSGEModel.SubEndYearValue,DSGEModel.SubEndPeriodValue,DSGEModel.YearStrMatrix,DSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(DSGEModel);
            StateCStr = get(controls.dsge.statecovariance,'String');
            DSGEModel.StateCovConstValue = str2num(strrep(StringTrim(StateCStr(DSGEModel.StateCovConst,:)),',',''));
            OrigDSGEModel = DSGEModel;
            set(controls.openmenu,'UserData',OrigDSGEModel);
            CurrINI.CurrentModelFile = '';
            SetYADAUIControls(maingui,DSGEModel,CurrINI.CurrentModelFile,CurrINI.images);
            %
            % reset the path
            %
            path(CurrINI.YADApath);
         end;
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);
%if strcmp(lower(get(controls.leave,'UserData')),'driver')==1;
%   set(controls.leave,'UserData','done');
%end;

%
% end of FileMenuFunctions.m
%
