function LogMarg = MargLikeLaplace(LogPost,Hessian,IsInverse)
% MargLikeLaplace: Calculates the Laplace approximation of the log marginal likelihood.
%
% USAGE:
%
%       LogMarg = MargLikeLaplace(LogPost,Hessian,IsInverse)
%
% where the Laplace approximation is based on a multivariate normal approximation, evluated at the posterior
% mode. This means that for a parameter vector of dimension k:
%
%          LogMarg = LogPost + (k/2)*log(2*pi) + (1/2)*log(det(InvHessian))
%
% where InvHessian is the inverse of the Hessian at the posterior mode.
%
%
% REQUIRED INPUT:  LogPost (scalar) with the value of the log posterior at the posterior mode.
%
%                  Hessian (pxp matrix) with the Hessian or the inverse Hessian.
%
%                  IsInverse (boolean) takes the value 1 if Hessian is the inverse Hessian
%                            and 0 otherwise.
%
% REQUIRED OUTPUT: LogMarg (scalar) with the value of the log marginal likelihood under the Laplace
%                            approximation.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 20, 2006.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 19-12-2008: Updated the documentation.
%
% * 03-05-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 28-03-2012: The function can now compute the Laplace approximation using the Hessian or the
%               inverse Hessian.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 30-04-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
% * 27-02-2024: The function now supports an empty Hessian matrix.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if isempty(Hessian)==1;
   LogMarg = LogPost;
   return;
end;
%
if IsInverse==0;
   LogMarg = LogPost+((length(Hessian)/2)*log(2*pi))-((1/2)*LogDet(Hessian));
else;
   LogMarg = LogPost+((length(Hessian)/2)*log(2*pi))+((1/2)*LogDet(Hessian));
end;

%
% end of MargLikeLaplace.m
%
