function PrintMonteCarloFiltering(ResFile,SolutionCode,thetaDraws,ParameterNames,DSGEModel,CurrINI)
% PrintMonteCarloFiltering: Computes Kolmogorov-Smirnov tests from the Monte Carlo filtering results
%                           and writes them to a text-file.
%
% USAGE:
%
%       PrintMonteCarloFiltering(ResFile,SolutionCode,thetaDraws,ParameterNames,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string) with the name of the text file to write results to.
%
%                 SolutionCode (vector) of dimension NumDraws with integer 1 for unique, convergent solution,
%                              2 for indeterminacy, and 0 for no stable solution.
%
%                 thetaDraws (matrix) of dimension NumParamxNumDraws with values of the parameters of the
%                              DSGE model.
%
%                 ParameterNames (string matrix) with the names of the estimated parameters.
%
%                 DSGEModel (structure) whose fields contain all required information about the model,
%                              such as input files, variable and sample data.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 16, 2011.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-12-2011: Finished the function.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 09-06-2015: Attempted to use histcounts instead of histc when it exists.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 14-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[NumParam,NumDraws] = size(thetaDraws);
%
% locate unique and convergent solutions
%
OnePos = (SolutionCode==1);
TwoPos = (SolutionCode==2);
ZeroPos = (SolutionCode==0);
NumUniqueConv = sum(OnePos);
NumIndeterminacy = sum(TwoPos);
NumNoSolution = sum(ZeroPos);
NumOther = NumDraws-NumUniqueConv;
%
KolmogorovSmirnovTest = zeros(NumParam,2);
N = (NumUniqueConv*NumOther)/(NumUniqueConv+NumOther);
%NNBar = sqrt(N)+0.12+0.11/sqrt(N);
NNBar = sqrt(N);
KSTestExist = exist('kstest2');
%
% determine the empirical distributions
%
if KSTestExist==2;
   for i=1:NumParam;
      [H,pVal,KStest] = kstest2(thetaDraws(i,OnePos),thetaDraws(i,~OnePos));
      KolmogorovSmirnovTest(i,1) = NNBar*KStest;
      %
      % compute the p-value using own approximation function
      %
      KolmogorovSmirnovTest(i,2) = KolmogorovDist(KolmogorovSmirnovTest(i,1),1000);
   end;
else;
   for i=1:NumParam;
      Bins = [-Inf sort(thetaDraws(i,:)) Inf];
      if exist('histcounts')==0;
         PDFUniqueConv = histc(thetaDraws(i,OnePos),Bins,2);
         PDFOther = histc(thetaDraws(i,~OnePos),Bins,2);
      else;
         PDFUniqueConv = histcounts(thetaDraws(i,OnePos),Bins);
         PDFOther = histcounts(thetaDraws(i,~OnePos),Bins);
      end;
      CDFUniqueConv = cumsum(PDFUniqueConv)/NumUniqueConv;
      CDFOther = cumsum(PDFOther)/NumOther;
      KolmogorovSmirnovTest(i,1) = NNBar*max(abs(CDFUniqueConv-CDFOther));
      %
      % compute the p-value
      %
      KolmogorovSmirnovTest(i,2) = KolmogorovDist(KolmogorovSmirnovTest(i,1),1000);
   end;
end;
%
% we can now write the results to ResFile
%
fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*               K O L M O G O R O V - S M I R N O V   T E S T S                *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% some information about the prediction sample
%
fprintf(fid,'Total number of parameter draws:                     %0.0f\n',NumDraws);
fprintf(fid,'Number of draws with unique and convergent solution: %0.0f\n',NumUniqueConv);
fprintf(fid,'Number of draws with indeterminant solution:         %0.0f\n',NumIndeterminacy);
fprintf(fid,'Number of draws with no stable solution              %0.0f\n\n',NumNoSolution);
%
% write the test results to file
%
ns = size(ParameterNames,2);
if ns<20;
   AddStr = SpaceStr(20-ns);
else;
   AddStr = '';
end;
PrtStr = '%s%s   %11.4f   %0.4f \n';
fprintf(fid,'============================================\n');
fprintf(fid,'           parameter       KS stat  p-value \n');
fprintf(fid,'--------------------------------------------\n');
for i=1:NumParam;
   fprintf(fid,PrtStr,AddStr,ParameterNames(i,:),KolmogorovSmirnovTest(i,:)');
end;
fprintf(fid,'============================================\n\n');
%
% write some information concerning the tests
%
fprintf(fid,'NOTE: The null hypothesis is that the distributions of a parameter when\n');
fprintf(fid,'      the model has a unique and convergent solution and when there is\n');
fprintf(fid,'      either indeterminacy or no stable solution are equal.\n\n');
%
fprintf(fid,'********************************************************************************\n\n');
%
% Some information about the filename and its location
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintMonteCarloFiltering.m
%
