function TextFile = PrintPriorInformation(ParameterNames,theta,thetaDist,PriorDist,AdditionalSystemPriorInfo,IsAdaptiveLearning,DSGEModel,CurrINI)
% PrintPriorInformation: Prints information about parameters to be estimated using the data in the prior file.
%
% USAGE:
%
%       PrintPriorInformation(ParameterNames,theta,thetaDist,PriorDist,AdditionalSystemPriorInfo,IsAdaptiveLearning,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                                 "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated".
%                                 Each field returns a string matrix with the parameter names.
%
%                  theta (vector) with initial values for the parameters to be estimated.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                                 truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                                 and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                                 "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                                 parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                                 and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                                 it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                                 truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                                 values are the lower and the upper bound. For the student-t they are the location (mode),
%                                 the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                                 and scale parameters are needed. For the Pareto the parameter values are for the mode
%                                 and the shape parameter.
%
%                  AdditionalSystemPriorInfo (string matrix) with additional information about the system prior.
%
%                  IsAdaptiveLearning (boolean) that takes the value 1 if the prior data for adaptive learning version of
%                                 the DSGE model is to be displayed and 0 otherwise.
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as input
%                                 files, variable and sample data. Actual observations are not stored in this structure.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: TextFile (string vector) with the name of the output file.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: July 24, 2008.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-01-2009: Updated the documentation.
%
% * 13-03-2009: Added support for the Student-t, the Cauchy, the logistic, and the Pareto
%               prior distributions.
%
% * 31-03-2009: Moved the text string "deg. freedom" for the Student-t prior to line up
%               with the 3rd data column. Made sure that the origin parameter of the
%               Pareto distribution is taken into account.
%
% * 02-04-2009: Took into account that the logistic supports a shape parameter. Added
%               support for the Gumbel prior distribution.
%
% * 20-04-2009: Made sure that the fields in PriorDist have the correct length.
%
% * 27-08-2009: Fixed a bug where the initial value and prior parameter 1 parameters were printed
%               in the wrong order for the beta and uniform distributions. Tobias Blattner discovered
%               this bug!
%
% * 05-01-2010: Updated the documentation.
%
% * 17-03-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 16-05-2017: Took into account that a system prior may also be used.
%
% * 01-06-2017: Added further information when a system prior is used.
%
% * 13-12-2017: Added code for printing possible extra information from a system prior.
%
% * 14-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 08-02-2021: Added the input variable "IsAdaptiveLearning".
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if IsAdaptiveLearning==0;
   if isempty(DSGEModel.NameOfModel)==0;
      TextFile = FixFilePath([DSGEModel.OutputDirectory '\PriorInformation-' DSGEModel.NameOfModel '.txt']);
   else;
      TextFile = FixFilePath([DSGEModel.OutputDirectory '\PriorInformation.txt']);
   end;
else;
   if isempty(DSGEModel.NameOfModel)==0;
      TextFile = FixFilePath([DSGEModel.OutputDirectory '\PriorInformation-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']);
   else;
      TextFile = FixFilePath([DSGEModel.OutputDirectory '\PriorInformation-AdaptiveLaerning.txt']);
   end;
end;
%
fid = fopen(TextFile,'wt');
%
% the header information
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*         P R I O R   D I S T R I B U T I O N   I N F O R M A T I O N          *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
fprintf(fid,'Prior distribution data file:     %s\n',GetFilename(DSGEModel.PriorFile));
fprintf(fid,'Sheet name:                       %s\n',DSGEModel.PriorFileSheet);
if strcmp(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
   fprintf(fid,'System prior density file:        %s\n',GetFilename(DSGEModel.SystemPriorFile));
   %
   % check if we have more info to write to file
   %
   if isempty(AdditionalSystemPriorInfo)==0;
      fprintf(fid,'\nSystem Prior Information\n========================\n');
      for j=1:size(AdditionalSystemPriorInfo,1);
         fprintf(fid,'%s\n',AdditionalSystemPriorInfo(j,:));
      end;
   end;
end;
fprintf(fid,'\n');
%
prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
%
% check the length of parameter names string
%
ns = size(ParameterNames.estimated,2);
if ns<20;
   add_str = SpaceStr(20-ns);
else;
   add_str = '';
end;
%
if (isempty(PriorDist.beta)==0)|(isempty(PriorDist.uniform)==0);
   fprintf(fid,'====================================================================================================================\n');
   fprintf(fid,'                                                Original Parameters                                                 \n');
   fprintf(fid,'      parameter          prior dist   initial value     parameter 1     parameter 2     lower bound     upper bound\n');
   SepStr = '--------------------------------------------------------------------------------------------------------------------';
   EndStr = '====================================================================================================================';
else;
   if (isempty(PriorDist.gamma)==0)|(isempty(PriorDist.invgamma)==0)|(isempty(PriorDist.truncnormal)==0)|(isempty(PriorDist.student)==0)|(isempty(PriorDist.pareto)==1);
      fprintf(fid,'====================================================================================================\n');
      fprintf(fid,'                                        Original Parameters                                         \n');
      fprintf(fid,'      parameter          prior dist   initial value     parameter 1     parameter 2     lower bound\n');
      SepStr = '----------------------------------------------------------------------------------------------------';
      EndStr = '====================================================================================================';
   else;
      fprintf(fid,'====================================================================================\n');
      fprintf(fid,'                                        Original Parameters                         \n');
      fprintf(fid,'      parameter          prior dist   initial value     parameter 1     parameter 2\n');
      SepStr = '------------------------------------------------------------------------------------';
      EndStr = '====================================================================================';
   end;
end;
%
% initialize parameters
%
n_gamma = 0;
n_beta = 0;
n_normal = 0;
n_invgamma = 0;
n_truncnormal = 0;
n_uniform = 0;
n_student = 0;
n_cauchy = 0;
n_logistic = 0;
n_gumbel = 0;
n_pareto = 0;
%
% initialize print string
%
PrtStr = ['%s%s    %s%s    ' prt_val '    ' prt_val '    ' prt_val];
for i=1:length(theta);
   if thetaDist(i)==1;
      dist_str = 'gamma';
      dist_add = SpaceStr(6);
      n_gamma = n_gamma+1;
      prior_val = PriorDist.gamma(n_gamma,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0];
      end;
   elseif thetaDist(i)==2;
      dist_str = 'beta';
      dist_add = SpaceStr(7);
      n_beta = n_beta+1;
      prior_val = PriorDist.beta(n_beta,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0 1];
      end;
   elseif thetaDist(i)==3;
      dist_str = 'normal';
      dist_add = SpaceStr(5);
      n_normal = n_normal+1;
      prior_val = PriorDist.normal(n_normal,:);
   elseif thetaDist(i)==4;
      dist_str = 'invgamma';
      dist_add = SpaceStr(3);
      n_invgamma = n_invgamma+1;
      prior_val = PriorDist.invgamma(n_invgamma,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0];
      end;
   elseif thetaDist(i)==5;
      dist_str = 'truncnormal';
      dist_add = '';
      n_truncnormal = n_truncnormal+1;
      prior_val = PriorDist.truncnormal(n_truncnormal,:);
   elseif thetaDist(i)==6;
      dist_str = 'uniform';
      dist_add = SpaceStr(4);
      n_uniform = n_uniform+1;
      prior_val = [(PriorDist.uniform(n_uniform,1)+PriorDist.uniform(n_uniform,2))/2 sqrt(((PriorDist.uniform(n_uniform,2)-PriorDist.uniform(n_uniform,1))^2)/12) PriorDist.uniform(n_uniform,:)];
   elseif thetaDist(i)==7;
      dist_str = 'student';
      dist_add = SpaceStr(4);
      n_student = n_student+1;
      prior_val = PriorDist.student(n_student,:);
   elseif thetaDist(i)==8;
      dist_str = 'cauchy';
      dist_add = SpaceStr(5);
      n_cauchy = n_cauchy+1;
      prior_val = PriorDist.cauchy(n_cauchy,:);
   elseif thetaDist(i)==9;
      dist_str = 'logistic';
      dist_add = SpaceStr(3);
      n_logistic = n_logistic+1;
      prior_val = PriorDist.logistic(n_logistic,:);
   elseif thetaDist(i)==10;
      dist_str = 'gumbel';
      dist_add = SpaceStr(5);
      n_gumbel = n_gumbel+1;
      prior_val = PriorDist.gumbel(n_gumbel,:);
   elseif thetaDist(i)==11;
      dist_str = 'pareto';
      dist_add = SpaceStr(5);
      n_pareto = n_pareto+1;
      prior_val = PriorDist.pareto(n_pareto,:);
   end;
   fprintf(fid,'%s\n',SepStr);
   if (strcmp(dist_str,'beta')==1)|(strcmp(dist_str,'uniform')==1);
      %
      % we have a beta or uniform prior
      %
      fprintf(fid,[PrtStr '    ' prt_val '    ' prt_val '\n'],add_str,ParameterNames.estimated(i,:),dist_add,dist_str,theta(i),prior_val(1,1),prior_val(1,2),prior_val(1,3),prior_val(1,4));
      fprintf(fid,'                                                               mean        st. dev.                                \n');
   elseif (strcmp(dist_str,'normal')==1)|(strcmp(dist_str,'gumbel')==1);
      %
      % we have a normal or gumbel prior
      %
      fprintf(fid,[PrtStr '\n'],add_str,ParameterNames.estimated(i,:),dist_add,dist_str,theta(i),prior_val(1,1),prior_val(1,2));
      fprintf(fid,'                                                               mean        st. dev.                                \n');
   elseif strcmp(dist_str,'logistic')==1;
      %
      % we have a logistic prior
      %
      fprintf(fid,[PrtStr '    ' prt_val '\n'],add_str,ParameterNames.estimated(i,:),dist_add,dist_str,theta(i),prior_val(1,1),prior_val(1,2),prior_val(1,3));
      fprintf(fid,'                                                               mean        st. dev.           shape                \n');
   elseif strcmp(dist_str,'cauchy')==1;
      %
      % we have a cauchy prior
      %
      fprintf(fid,[PrtStr '\n'],add_str,ParameterNames.estimated(i,:),dist_add,dist_str,theta(i),prior_val(1,1),prior_val(1,2));
      fprintf(fid,'                                                           location           scale                                \n');
   elseif strcmp(dist_str,'pareto')==1;
      %
      % we have a Pareto prior
      %
      fprintf(fid,[PrtStr '    ' prt_val '\n'],add_str,ParameterNames.estimated(i,:),dist_add,dist_str,theta(i),prior_val(1,1),prior_val(1,2),prior_val(1,3));
      fprintf(fid,'                                                              shape            mode          origin                \n');
   elseif strcmp(dist_str,'student')==1;
      %
      % we have a student-t prior
      %
      fprintf(fid,[PrtStr '    ' prt_val '\n'],add_str,ParameterNames.estimated(i,:),dist_add,dist_str,theta(i),prior_val(1,1),prior_val(1,2),prior_val(1,3));
      fprintf(fid,'                                                           location           scale    deg. freedom\n');
   else;
      %
      % we have a gamma, inverted gamma, or left truncated normal prior
      %
      fprintf(fid,[PrtStr '    ' prt_val '\n'],add_str,ParameterNames.estimated(i,:),dist_add,dist_str,theta(i),prior_val(1,1),prior_val(1,2),prior_val(1,3));
      if strcmp(dist_str,'gamma')==1;
         fprintf(fid,'                                                               mean        st. dev.                                \n');
      elseif strcmp(dist_str,'invgamma')==1;;
         fprintf(fid,'                                                           location    deg. freedom                                \n');
      else;
         fprintf(fid,'                                                           location           scale                                \n');
      end;
   end;
end;
fprintf(fid,'%s\n\n',EndStr);
%
if strcmp(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
   fprintf(fid,'NOTE: The above properties of the prior distributions concern the marginal distribu-\n      tions for each parameter and they do NOT take the system prior into account.\n\n');
end;
%
% Some information about the filename and its location
%
fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintPriorInformation.m
%
