function PrintSmallEigenValues(TextFile,SmallEigenValues,SmallEigenVectors,ParameterNames,WriteString,CurrINI)
% PrintSmallEigenValues: Prints the smallest eigenvalues and the associated eigenvectors for the
%                        information matrix to file.
%
% USAGE:
%
%       PrintSmallEigenValues(TextFile,SmallEigenValues,SmallEigenVectors,ParameterNames,WriteString,CurrINI)
%
% REQUIRED INPUT: TextFile (string) with the full path and name of the output file.
%
%                 SmallEigenValues (vector) of dimension s with the smallest eigenvalues of the
%                                  information matrix.
%
%                 SmallEigenVectors (matrix) of dimension pxs with the eigenvectors that are
%                                  associated with the small eigenvalues.
%
%                 ParameterNames (string matrix) with the p parameter names.
%
%                 WriteString (string) with write data for the fopen command.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 5, 2010.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-01-2010: Updated the documentation.
%
% * 06-05-2010: Changed the header information in the output to take into account that
%               some eigenvalues need not be "small".
%
% * 05-01-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 14-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

fid = fopen(TextFile,WriteString);
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*       I N F O R M A T I O N   M A T R I X :   E I G E N V E C T O R S        *\n');
fprintf(fid,'*        A S S O C I A T E D   W I T H   T H E   E I G E N V A L U E S         *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
%
[p,s] = size(SmallEigenVectors);
ns = size(ParameterNames,2);
if ns<20;
   AddStr = SpaceStr(20-ns);
else;
   AddStr = '';
end;
%
PrtStr = ['   %s%s   %' num2str(4+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
for i=1:s;
   fprintf(fid,'======================================================\n');
   fprintf(fid,'   Parameter             eigenvector       eigenvalue \n');
   fprintf(fid,'------------------------------------------------------\n');
   for j=1:p;
      if j>1;
         fprintf(fid,[PrtStr ' \n'],ParameterNames(j,:),AddStr,SmallEigenVectors(j,i));
      else;
         fprintf(fid,[PrtStr '   %14.10f \n'],ParameterNames(1,:),AddStr,SmallEigenVectors(j,i),SmallEigenValues(i));
      end;
   end;
   fprintf(fid,'======================================================\n\n');
end;
fclose(fid);

%
% end of PrintSmallEigenValues.m
%
