function PrintStationaryPredictiveVariance(ResFile,MeasureCov,ShockCov,StateCov,ParameterCov,TotalCov,lambda,NumDraws,TotalDraws,NumBurnin,IsPosterior,CompTime,DSGEModel,CurrINI)
% PrintStationaryPredictiveVariance: prints marginal predictive variances and decompositions into state, shock,
%                                    measurement error and parameter uncertainty for prior or posterior draws.
%
% USAGE:
%
%       PrintStationaryPredictiveVariance(ResFile,MeasureCov,ShockCov,StateCov,ParameterCov,TotalCov,lambda,...
%                                         NumDraws,TotalDraws,NumBurnin,IsPosterior,CompTime,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string) with the full path and name of the text-file where the results
%                         are written.
%
%                 MeasureCov (matrix) of dimension nxn with the stationary measurement error covariances.
%
%                 ShockCov (matrix) of dimension nxnxh with the stationary shock covariances up to horizon h.
%
%                 StateCov (matrix) of dimension nxnxh with the stationary initial state covariances impact
%                         up to horizon h.
%
%                 ParameterCov (matrix) of dimension nxnxh with the stationary parameter uncertainty impact
%                         up to horizon h.
%
%                 TotalCov (matrix) of dimension nxnxh with the sum of the previous four variables.
%
%                 lambda (scalar) with the lambda hyperparameter of the DSGE-VAR.
%
%                 NumDraws (integer) with the number of used parameter draws.
%
%                 TotalDraws (integer) with the total number of available parameter draws.
%
%                 NumBurnin (integer) with the number of burn-in draws.
%
%                 IsPosterior (boolean) with is 1 if posterior draws have been used and 0 if the predictive
%                         moments have been computed using prior draws.
%
%                 CompTime (time) with the computational time.
%
%                 DSGEModel (structure) whose fields contain all required information
%                         about the model, such as input files, variable and sample data.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 25, 2019.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-03-2019: Updated the documentation.
%
% * 28-03-2019: Corrected a print string for DSGE-VAR models.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*        S T A T I O N A R Y   P R E D I C T I V E   V A R I A N C E S         *\n');
if isempty(lambda)==1;
   fprintf(fid,'*           A N D   D E C O M P O S I T I O N S   I N T O   F O U R            *\n');
else;
   fprintf(fid,'*            A N D   D E C O M P O S I T I O N S   I N T O   T W O             *\n');
end;
fprintf(fid,'*                 S O U R C E S   O F   U N C E R T A I N T Y                  *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
if IsPosterior==1;
   DistStr = 'Posterior';
else;
   DistStr = 'Prior';
end;
%
PrtVal = ['%' int2str(6+CurrINI.decimals) '.' int2str(CurrINI.decimals) 'f'];
PrtVal2 = ['%' int2str(4+CurrINI.decimals) '.' int2str(CurrINI.decimals) 'f'];
%
fprintf(fid,'Parameter draws:                                   %s distribution \n',DistStr);
if IsPosterior==1;
   fprintf(fid,'Number of available posterior draws:               %0.0f\n',TotalDraws);
   fprintf(fid,'Number of used draws:                              %0.0f\n',NumDraws);
   fprintf(fid,'Number of burn-in draws:                           %0.0f\n',NumBurnin);
else;
   fprintf(fid,'Number of available prior draws:                   %0.0f\n',TotalDraws);
end;
if isempty(lambda)==0;
   fprintf(fid,'Number of lags in DSGE-VAR:                        %0.0f \n',DSGEModel.BVARLags);
   fprintf(fid,'Lambda:                                            %0.4f \n',lambda);
end;
fprintf(fid,'Computation time:                                  %s (hrs:min:sec)\n\n',SecondsToHoursStr(CompTime));
%
[n,nv] = size(DSGEModel.VariableNames);
h = size(TotalCov,3);
if nv<15;
   AddStr = SpaceStr(15-nv);
else;
   AddStr = '';
end;
if isempty(lambda)==1;
   FirstStr = MultiCharStr('=',93);
   DivStr = MultiCharStr('-',93);
   PrtStr = ['%s%s     ' PrtVal '   ' PrtVal '   ' PrtVal '   ' PrtVal '   ' PrtVal '\n'];
   PrtStr2 = ['Shares of variance:                  ' PrtVal2 '     ' PrtVal2 '     ' PrtVal2 '     ' PrtVal2 '\n'];
   for i=1:h;
      fprintf(fid,'%s\n',FirstStr);
      fprintf(fid,'Forecast horizon: %2.0f                                             measurement \n',i);
      fprintf(fid,'  Variable            variance        state          shock          error        parameter \n');
      fprintf(fid,'%s\n',FirstStr);
      for j=1:n;
         fprintf(fid,PrtStr,DSGEModel.VariableNames(j,:),AddStr,TotalCov(j,j,i),StateCov(j,j,i),ShockCov(j,j,i),MeasureCov(j,j),ParameterCov(j,j,i));
         fprintf(fid,PrtStr2,StateCov(j,j,i)/TotalCov(j,j,i),ShockCov(j,j,i)/TotalCov(j,j,i),MeasureCov(j,j)/TotalCov(j,j,i),ParameterCov(j,j,i)/TotalCov(j,j,i));
         if j<n;
            fprintf(fid,'%s\n',DivStr);
         else;
            fprintf(fid,'%s\n\n',FirstStr);
         end;
      end;
   end;
else;
   FirstStr = MultiCharStr('=',63);
   DivStr = MultiCharStr('-',63);
   PrtStr = ['%s%s     ' PrtVal '   ' PrtVal '   ' PrtVal '\n'];
   PrtStr2 = ['Shares of variance:                  ' PrtVal2 '     ' PrtVal2 '\n'];
   for i=1:h;
      fprintf(fid,'%s\n',FirstStr);
      fprintf(fid,'Forecast horizon: %2.0f \n',i);
      fprintf(fid,'  Variable            variance        shock        parameter \n');
      fprintf(fid,'%s\n',FirstStr);
      for j=1:n;
         fprintf(fid,PrtStr,DSGEModel.VariableNames(j,:),AddStr,TotalCov(j,j,i),ShockCov(j,j,i),ParameterCov(j,j,i));
         fprintf(fid,PrtStr2,ShockCov(j,j,i)/TotalCov(j,j,i),ParameterCov(j,j,i)/TotalCov(j,j,i));
         if j<n;
            fprintf(fid,'%s\n',DivStr);
         else;
            fprintf(fid,'%s\n\n',FirstStr);
         end;
      end;
   end;
end;
%
% bottom file data
%
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintStationaryPredictiveVariance.m
%
