function PrintSystemPriorCorrelations(ResFile,ParameterNames,ParameterCorrParam,IsPosterior,NumDraws,UsedDraws,NumBurnin,CurrChain,IsAdaptiveLearning,DSGEModel,CurrINI)
% PrintSystemPriorCorrelations: print the correlations between the estimated and possibly the updated parameters
%                               based on the system prior or the posterior draws.
%
% USAGE:
%
%       PrintSystemPriorCorrelations(ResFile,ParameterNames,ParameterCorrParam,IsPosterior,NumDraws,...
%                                    UsedDraws,NumBurnin,CurrChain,IsAdaptiveLearning,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string) with the full path and name of the output text file.
%
%                 ParameterNames (string matrix) of dimension NumParam x NP1 with the names of the parameters
%                                for which the correlations have been computed.
%
%                 ParameterCorrParam (matrix) of dimension NumParam x NumParam with the correlations in the
%                                off-diagonal elements.
%
%                 IsPosterior (boolean) that takes the value 1 if we have posterior draws, and 0 if we have
%                                prior draws.
%
%                 NumDraws (integer) with the number of parameter draws.
%
%                 UsedDraws (integer) with the number of used draws for the correlation computations.
%
%                 NumBurnin (integer) with the number of discarded burnin draws.
%
%                 CurrChain (integer) with the number of the current posterior sampling chain.
%
%                 IsAdaptiveLearning (boolean) which takes the value 1 if the adaptive learning mechanism is used for the
%                                expectations formation, and 0 otherwise.
%
%                 DSGEModel (structure) whose fields contain all required information about the model, such
%                                as input files, variable and sample data.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 31, 2018.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-11-2018: Updated the documentation.
%
% * 05-11-2018: Added the input variables IsPosterior, UsedDraws, NumBurnin and CurrChain while
%               NumPriorDraws has been renamed NumDraws.
%
% * 06-11-2018: Updated the documentation.
%
% * 04-12-2018: The function should now support IS based on MitISEM.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 28-05-2020: Added the input variable "IsAdaptiveLearning".
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
fid = fopen(ResFile,'wt');
if IsPosterior==1;
   fprintf(fid,'********************************************************************************\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'*       P O S T E R I O R   P A R A M E T E R   C O R R E L A T I O N S        *\n');
   if IsAdaptiveLearning==1;
      fprintf(fid,'*                      A D A P T I V E   L E A R N I N G                       *\n');
   end;
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'********************************************************************************\n\n');
   %
   if DSGEModel.PosteriorSampler<=7;
      fprintf(fid,'Number of posterior draws:              %0.0f\n',NumDraws+NumBurnin);
      fprintf(fid,'Number of post burn-in posterior draws: %0.0f\n',NumDraws);
      fprintf(fid,'Number of burn-in draws:                %0.0f\n',NumBurnin);
      fprintf(fid,'Number of used posterior draws:         %0.0f\n',UsedDraws);
      fprintf(fid,'MCMC chain number:                      %0.0f\n\n',CurrChain);
   elseif DSGEModel.PosteriorSampler==10;
      fprintf(fid,'Number of posterior draws:              %0.0f\n',NumDraws);
      fprintf(fid,'IS sample number:                       %0.0f\n\n',CurrChain);
   else;
      fprintf(fid,'Number of posterior draws:              %0.0f\n',NumDraws);
      fprintf(fid,'SMC sample number:                      %0.0f\n\n',CurrChain);
   end;
else;
   fprintf(fid,'********************************************************************************\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'*              S Y S T E M   P R I O R   C O R R E L A T I O N S               *\n');
   fprintf(fid,'*                                                                              *\n');
   fprintf(fid,'********************************************************************************\n\n');
   %
   fprintf(fid,'Number of prior draws:   %0.0f\n\n',NumDraws);
end;
% initializing the formatting of output numbers. we usually apply the 12.6f format.
%
PrtValue = ['%' int2str(3+CurrINI.decimals) '.' int2str(CurrINI.decimals) 'f'];
[NumParam,ParamWidth] = size(ParameterNames);
if ParamWidth<20;
   AddNameStr = SpaceStr(20-ParamWidth);
else;
   AddNameStr = '';
end;
PrtStr = ['%s%s,%s%s     ' PrtValue '%s \n'];
%
fprintf(fid,'==========================================================\n');
fprintf(fid,'              parameter pair                 correlation \n');
fprintf(fid,'----------------------------------------------------------\n');
AstExist = 0;
for i=1:NumParam-1;
   for j=i+1:NumParam;
      Ast = '';
      if abs(ParameterCorrParam(i,j))>=0.5;
         Ast = '*';
         AstExist = 1;
      end;
      fprintf(fid,PrtStr,AddNameStr,ParameterNames(i,:),ParameterNames(j,:),AddNameStr,ParameterCorrParam(i,j),Ast);
   end;
end;
%
fprintf(fid,'==========================================================\n\n');
if AstExist==1;
   fprintf(fid,'NOTE: Absolute correlations greater than or equal to 0.5\n      are indicated with a star (*).\n\n');
end;
%
% bottom file data
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintSystemPriorCorrelations.m
%

