function PrintZLBPredictiveLikelihood(ResFile,ModelStr,PredictionVariables,PredictionSampleStrMat,YDataStr,LogPredLike,AnnualizeData,IsPosterior,IsConditional,IsAdaptiveLearning,IsZLB,NumDraws,NumBurnin,TotalDraws,NumPredPaths,lambda,CompTime,DSGEModel,CurrINI)
% PrintZLBPredictiveLikelihood: Prints results from estimating the log predictive likelihood from prediction paths that
%                               are subject to the ZLB.
%
% USAGE:
%
%       PrintZLBPredictiveLikelihood(ResFile,ModelStr,PredictionVariables,PredictionSampleStrMat,YDataStr,LogPredLike,AnnualizeData,...
%                                    IsPosterior,IsConditional,IsAdaptiveLearning,IsZLB,NumDraws,NumBurnin,TotalDraws,NumPredPaths,...
%                                    lambda,CompTime,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string) with the full path and name of the text file to write to.
%
%                 ModelStr (string) which takes the value "DSGE", "DSGE with learning" or "DSGE-VAR".
%
%                 PredictionVariables (vector) with integer values between 1 and n with the variables used
%                                     for the joint PIT calculations.
%
%                 PredictionSampleStrMat (string matrix) with the prediction sample dates.
%
%                 YDataStr (string vector) with a description of the actuals data used for the computation.
%
%                 LogPredLike (matrix) of dimension (n+1) x h with the predictive likelihood values for the n
%                                     observed variables and the joint predictive likelihood for the specified
%                                     combination in PredictionVariables over the forecast horizon.
%
%                 AnnualizeData (boolean) which takes the value 1 if annualized data have been predicted and
%                                     0 otherwise.
%
%                 IsPosterior (boolean) that takes the value 1 if the parameter draws are taken from the posterior
%                                     distribution and 0 otherwise.
%
%                 IsConditional (boolean) that takes the value 1 if conditional forecasts have been used and 0
%                                     otherwise.
%
%                 IsAdaptiveLearning (boolean) that takes the value 1 if the adaptive learning mechanism is used for the
%                                     expectations formation, and 0 otherwise.
%
%                 IsZLB (boolean) that takes the value 1 if the forecasts have been computed subject to the ZLB
%                                     and 0 otherwise.
%
%                 NumDraws (integer) with the total number of parameters draws used to compute the PITs.
%
%                 NumBurnin (integer) with the number of burn-in draws.
%
%                 TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                 NumPredPaths (integer) with the number of prediction paths per parameter draw.
%
%                 lambda (scalar) hyperparameter value that is used to measure degree of misspecification
%                                     for DSGE-VAR models.
%
%                 CompTime (time) with the computational time for the prediction paths and the individual
%                                     and joint predictive likelihoods.
%
%                 DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure), with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 31, 2024.
%                        This version: February 2, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-02-2024: Updated the documentation.
%
% * 08-02-2024: Added the "IsZLB" input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[n1,h] = size(LogPredLike);
n = n1-1;
fid = fopen(ResFile,'wt');
%
% the header information
%
fprintf(fid,'*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*                           L O G   P R E D I C T I V E   L I K E L I H O O D                           *\n');
if IsAdaptiveLearning==1;
   fprintf(fid,'*                                   A D A P T I V E   L E A R N I N G                                   *\n');
end;
if IsZLB==1;
   fprintf(fid,'*                     S U B J E C T   T O   T H E   Z E R O   L O W E R   B O U N D                     *\n');
end;
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
if IsPosterior==1;
   DistStr = 'Posterior';
else;
   DistStr = 'Prior';
end;
if AnnualizeData==0;
   TypeStr = 'Original';
else;
   TypeStr = 'Annualized';
end;
%
fprintf(fid,'Model used:                                        %s \n',ModelStr);
fprintf(fid,'Parameter draws:                                   %s distribution \n',DistStr);
if IsPosterior==1;
   fprintf(fid,'Number of available posterior draws:               %0.0f\n',TotalDraws);
   fprintf(fid,'Number of used draws:                              %0.0f\n',NumDraws);
   fprintf(fid,'Number of burn-in draws:                           %0.0f\n',NumBurnin);
   fprintf(fid,'Number of prediction paths per draw:               %0.0f\n',NumPredPaths);
   fprintf(fid,'Total number of paths:                             %0.0f\n',NumDraws*NumPredPaths);
else;
   fprintf(fid,'Number of available prior draws:                   %0.0f\n',TotalDraws);
   fprintf(fid,'Number of paths per draw:                          %0.0f\n',NumPredPaths);
   fprintf(fid,'Total number of paths:                             %0.0f\n',NumDraws*NumPredPaths);
end;
if isempty(lambda)==0;
   fprintf(fid,'Number of lags in DSGE-VAR:                        %0.0f \n',DSGEModel.BVARLags);
   fprintf(fid,'Lambda:                                            %0.4f \n',lambda);
end;
if IsAdaptiveLearning==1;
   fprintf(fid,'Number of forward variables:                       %0.0f\n\n',length(DSGEModel.ForwardVariablePositions));
   fprintf(fid,'Name of belief system:                             %s\n',DSGEModel.BeliefSys.Name);
   if DSGEModel.BeliefSystem==1;
      fprintf(fid,'Belief system file:                                YADA''s default (two own lags)\n');
   else;
      fprintf(fid,'Belief system file:                                %s\n',GetFilename(DSGEModel.BeliefSystemFile));
      fprintf(fid,'Maximum lag order:                                 %0.0f\n', DSGEModel.BeliefSys.m);
   end;
end;
if IsConditional==1;
   VarNames = StrWrap(vecstr(DSGEModel.ZVariableNames(DSGEModel.ConditionalVariables,:),' '),40);
   fprintf(fid,'Names of conditional variables:                    %s\n',VarNames(1,:));
   for i=2:size(VarNames,1);
      if strcmp(StringTrim(VarNames(i,:)),'')==0;
         fprintf(fid,'                                                   %s\n',VarNames(i,:));
      end;
   end;
   if DSGEModel.ShockControlMethod==1;
      ShockMethod = 'Values for shocks';
   elseif DSGEModel.ShockControlMethod==2;
      ShockMethod = 'Distribution of shocks (Waggoner-Zha)';
   else;
      ShockMethod = 'Distribution of subset of shocks';
   end;
   fprintf(fid,'Conditioning method:                               %s\n',ShockMethod);
end;
fprintf(fid,'Set of actual data for the observed variables:     %s \n',YDataStr);
fprintf(fid,'Selected data type:                                %s data\n',TypeStr);
fprintf(fid,'Computation time:                                  %s (hrs:min:sec)\n\n',SecondsToHoursStr(CompTime));
%
PrtVal = ['%' num2str(5+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
%
FirstStr = MultiCharStr('=',56);
DivStr = MultiCharStr('-',56);
PrtStr1 = ['%s%s     %2.0f     %s      ' PrtVal '\n'];
PrtStr2 = ['                         %2.0f     %s      ' PrtVal '\n'];
%
nv = size(DSGEModel.VariableNames,2);
if nv<20;
   AddStr = SpaceStr(20-nv);
else;
   AddStr = '';
end;
VarStr = vecstr(DSGEModel.VariableNames(PredictionVariables,:),', ');
VarStr = VarStr(1:length(VarStr)-1);
fprintf(fid,'%s\n',FirstStr);
fprintf(fid,'    Variable          Horizon   Period      Pred. Like. \n');
fprintf(fid,'%s\n',DivStr);
for i=1:n;
   fprintf(fid,PrtStr1,DSGEModel.VariableNames(i,:),AddStr,1,PredictionSampleStrMat(1,:),LogPredLike(i,1));
   for j=2:h;
      fprintf(fid,PrtStr2,j,PredictionSampleStrMat(j,:),LogPredLike(i,j));
   end;
   if i<n;
      fprintf(fid,'%s\n',DivStr);
   else;
      fprintf(fid,'%s\n',FirstStr);
      fprintf(fid,'Pred. likelihood for selected prediction variables:\n%s \n',VarStr);
      fprintf(fid,'%s\n',DivStr);
      for j=1:h;
         fprintf(fid,PrtStr2,j,PredictionSampleStrMat(j,:),LogPredLike(n+1,j));
      end;
      fprintf(fid,'%s\n\n',FirstStr);
   end;
end;
%
% bottom file data
%
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintZLBPredictiveLikelihood.m
%
