function PrintSpatialDistortions(DSGEModel,CurrINI,SpatDist,SelectedParameters,NumDraws,TotalDraws,CurrChain,IsPosterior)
% PrintSpatialDistortions: writes the results on spatial distortions to a text file
%
% USAGE:
%
%       PrintSpatialDistortions(DSGEModel,CurrINI,SpatDist,SelectedParameters,NumDraws,TotalDraws,CurrChain,IsPosterior)
%
% REQUIRED INPUT: DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure), with initialization information.
%
%                 SpatDist (matrix or structure) with the results on the spatial distortions.
%
%                 SelectedParameters (vector) of dimension NumParam which indicates which parameters can vary from
%                                    one draw to the next (1) and which parameters are fixed at posterior mode (0).
%
%                 NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                                    parameters.
%
%                 TotalDraws (integer) with the number of draws from the posterior distribution of the parameters
%                                    that could have been used.
%
%                 CurrChain (integer) with the number for the current Markov chain.
%
%                 IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                    if draws from the prior are supplied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 12, 2009.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-06-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 14-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% determine the file name
%
if NumDraws==1;
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\SpatDist-' DSGEModel.NameOfModel '-PosteriorMode.txt']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\SpatDist-' DSGEModel.NameOfModel '-InitialValues.txt']);
   end;
else;
   SelParamStr = strrep(num2str(SelectedParameters),' ','');
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\SpatDist-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' SelParamStr '-' int2str(NumDraws) '.' int2str(TotalDraws) '.txt']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\SpatDistPrior-' DSGEModel.NameOfModel '-' SelParamStr '-' int2str(NumDraws) '.txt']);
   end;
end;
%
% open the output file
%
fid = fopen(file,'wt');
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*                    S P A T I A L   D I S T O R T I O N S                     *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% initializing the formatting of output numbers.
% we usually apply the 12.6f format.
%
prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' ...
                   num2str(CurrINI.decimals,'%0.0f') 'f'];
%
[n,vn] = size(DSGEModel.VariableNames);
if vn<15;
   AddNameStr = SpaceStr(15-vn);
else;
   AddNameStr = '';
end;
%
% write the output in SpatDist
%





%
% bottom file data
%
fprintf(fid,'Directory for file: %s\n',GetPath(file));
fprintf(fid,'Name of file:       %s\n',GetFilename(file));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintSpatialDistortions.m
%
