function z = MixedMultiStudentTRndFcn(mu,Sigma,piweights,df,NumDraws)
% MixedMultiStudentTRndFcn: Draws a matrix of random data from a mixture of multivariate Student-t distributions
%                           with positive weights adding to unity.
%
% USAGE:
%
%       z = MixedMultiStudentTRndFcn(mu,Sigma,piweights,df,NumDraws)
%
% REQUIRED INPUT:  mu (matrix) of dimension n x h with location parameters of mixture of multivariate Student-t distributions.
%
%                  Sigma (matrix) of dimension n x n x h with the scale parameters for the h mixture components.
%
%                  piweights (vector) of dimension h x 1 with the weights on the h mixture components.
%
%                  df (vector) of dimension h x 1 with the degrees of freedom parameters for the h mixture components.
%
%                  NumDraws (integer) with the number of draws.
%
% REQUIRED OUTPUT: z (matrix) of dimension nxNumDraws with draws from the multivarate Student-t
%                    distribution.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 12, 2018.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-12-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% loop over all components
%
z = 0;
h = length(df);
for i=1:h;
   CholSigma = chol(Sigma(:,:,i))';
   z = z+(piweights(i)*MultiStudentTRndFcn(mu(:,i),CholSigma,df(i),NumDraws));
end;

%
% end of MixedMultiStudentTRndFcn.m
%
