function LogPredLike = ComputePredLikeDSGECondPaths(YTrueValues,PredictionVariables,NumDraws,NumPredPaths,MaxPH,AnnualizeData,CurrChain,IsPosterior,ShockMethod,DSGEModel,CurrINI)
% ComputePredLikeDSGECondPaths: Computes the log predictive likelihood based on paths from
%                               the predictive distribution of the DSGE model.
%
% USAGE:
%
%       LogPredLike = ComputePredLikeDSGECondPaths(YTrueValues,PredictionVariables,NumDraws,NumPredPaths,MaxPH,...
%                                               AnnualizeData,CurrChain,IsPosterior,ShockMethod,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  YTrueValues (matrix) of dimension n x h with the true values of the
%                              observed variables over the forecast horizon.
%
%                  PredictionVariables (vector) with the integer values of the observed
%                              variables for which joint PITs should be computed.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPredPaths (integer) with the total number of prediction paths per parameter draw.
%
%                  MaxPH (integer) with the maximum prediction horizon used for the paths (can be
%                              shorter than the horizon h for which true values are available.
%
%                  AnnualizeData (boolean) that takes the value 1 if annualized data have been predicted
%                              and 0 if the original data has been predicted.
%
%                  CurrChain (integer) with the number of the currently used Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                              if draws from the prior are supplied.
%
%                  ShockMethod (string vector) that is either "Direct-", "WaggonerZha-", "Subset-", or "".
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
% REQUIRED OUTPUT: LogPredLike (matrix) of dimension n+1 x h with the CRPS for the individual variables in the
%                              top n rows and for the joint set of variables determined by PredictionVariables
%                              in the last row.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 8, 2024.
%                        This version: February 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%


pv = length(PredictionVariables);
UseYADA = 0;
if exist('YADAmvksdensity')==2;
   UseYADA = 1;
end;
%
% initialize
%
[n,h] = size(YTrueValues);
LogPredLike = zeros(n+1,h);
%
% set parameters for the bandwidth
%
BW = zeros(n,h);
%
if AnnualizeData==1;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
      DSGEModel.Frequency = 4;
   else;
      Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
      DSGEModel.Frequency = 12;
   end;
   DSGEModel.Yinit = Yinit;
   %
   % make sure that we take NaN's over the historical sample into account appropriately
   %
   YTrueValues = Annualize([Yinit YTrueValues],DSGEModel.annual,ones(n,1));
   TypeStr = 'Annual-';
else;
   TypeStr = 'Original-';
end;
%
% set up a wait dialog
%
HeadStr = 'Predictive Distributions (Predictive Likelihood)';
if IsPosterior==0;
   HeadStr = [HeadStr ' - Prior'];
end;
txt = 'Please wait while YADA computes the log predictive likelihood values for the individual observed variables and the log predictive likelihood values for the joint predicted variables. Current forecast horizon:  1.';
WaitHandle = WaitDLG(txt,'information',HeadStr,600,250,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% compute the predictive likelihood for each horizon at a time
%
TotalPaths = NumPredPaths*NumDraws;
for j=1:h;
   txt = ['Please wait while YADA computes the log predictive likelihood values for the individual observed variables and the log predictive likelihood values for the joint predicted variables. Current forecast horizon: ' int2str(j) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   % initialize a temporary variable
   %
   TempVar = zeros(TotalPaths,n);
   for ParamIndex=1:NumDraws;
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-' TypeStr ShockMethod DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-' TypeStr 'Prior-' ShockMethod DSGEModel.NameOfModel '-' int2str(MaxPH) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
      end;
      %
      % load the data
      %
      PredData = load(file);
      PredPaths = PredData.PredPaths;
      for p=1:NumPredPaths;
         CurrPath = p+(NumPredPaths*(ParamIndex-1));
         TempVar(CurrPath,:) = PredPaths(:,j,p)';
      end;
      clear('PredData');
      drawnow;
   end;
   %
   % compute the predictive likelihood for horizon j and variable i based on the paths
   %
   for i=1:n;
      %
      % bandwidth computation
      %
      if DSGEModel.PosteriorDensityValue<=2;
         %
         % compute Silverman's rule of thumb for bandwidth
         %
         sigmai = std(TempVar(:,i));
         BW(i,j) = sigmai*((4/((pv+2)*TotalPaths))^(1/(pv+4)));
         %
         % and now the log predictive likelihood
         %
         LogPredLike(i,j) = log(ksdensity(TempVar(:,i),YTrueValues(i,j),'Bandwidth',sigmai*((4/(3*TotalPaths))^(1/5))));
      else;
         %
         % compute optimal bandwidth from Zdravko Botev's kde function instead
         % the much simpler Silverman's rule
         %
         %  Reference:
         % Kernel density estimation via diffusion
         % Z. I. Botev, J. F. Grotowski, and D. P. Kroese (2010)
         % Annals of Statistics, Volume 38, Number 5, pages 2916-2957.
         %
         BW(i,j) = YADAkde(TempVar(:,i));
         %
         % and now the log predictive likelihood
         %
         LogPredLike(i,j) = log(ksdensity(TempVar(:,i),YTrueValues(i,j),'Bandwidth',BW(i,j)));
      end;
   end;
   %
   % and now the possibly multivariate
   %
   if pv<=2;
      LogPredLike(n+1,j) = log(ksdensity(TempVar(:,PredictionVariables),YTrueValues(PredictionVariables,j)','Bandwidth',BW(PredictionVariables,j)));
   else;
      if UseYADA==1;
         LogPredLike(n+1,j) = log(YADAmvksdensity(TempVar(:,PredictionVariables),YTrueValues(PredictionVariables,j)','Bandwidth',BW(PredictionVariables,j)'));
      else;
         LogPredLike(n+1,j) = log(mvksdensity(TempVar(:,PredictionVariables),YTrueValues(PredictionVariables,j)','Bandwidth',BW(PredictionVariables,j)'));
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of ComputePredLikeDSGECondPaths.m
%
