function [CorrDec,mcode] = DSGECorrelationDecompTheta(theta,thetaPositions,ModelParameters,VarStr,DSGEModel,CurrINI)
% DSGECorrelationDecompTheta: Computes the correlation decompositions for the observed variables and the state variables.
%
% USAGE:
%
%       [CorrDec,mcode] = DSGECorrelationDecompTheta(theta,thetaPositions,ModelParameters,VarStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values for the original parameters of the DSGE model.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  VarStr (string vector) that takes on the values "Observed Variables" and "State Variables".
%
%                  DSGEModel (structure) whose fields contain all required information
%                                 about the model, such as input files, variable and sample
%                                 data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: CorrDec (structure) with fields "Y", "Xi", "R", "AutoCovHorizon", "ShockNames", "ShockGroups",
%                                 "ShockGroupNames", "ShockColors", "ShockAliases", and "ShockGroupColors".
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 17, 2011.
%                        This version: January 25 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-01-2011: Added wait dialog code.
%
% * 05-04-2011: Added support for the "ShockColors" and "ShockAliases" fields of "CorrDec".
%
% * 18-04-2011: "ShockColors" can now also be initialized from the color of the "ShockGroups" the different
%               shocks belong to.
%
% * 05-01-2012: Updated the documentation.
%
% * 06-01-2012: Made sure that ShockGroupColors is only used when it has as many rows as there are shock groups.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 05-01-2018: Made sure that AutoCovHorizon is nonnegative.
%
% * 02-08-2018: The function supports user written model solver function.
%
% * 17-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
CorrDec = [];
if nargout==1;
   %
   % copy files to the tmp directory for posterior mode values
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
else;
   status = 1;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>1;
      status = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% try to solve the model
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0
   About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
   drawnow;
   return;
end;
[B0,KeepVar] = RemoveRedundantColumns(B0);
[r,q] = size(B0);
if q==0;
   txt = 'The DSGE model doesn''t have any structural shocks. YADA is therefore unable to compute any correlation decompostions.';
   About(txt,'information','YADA - No Shocks',200,500,CurrINI);
   return;
end;
n = DSGEModel.n;
%
% check if the solution is unique and convergent
%
if nargout>1;
   status = mcode;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      elseif DSGEModel.ModelSolver==4;
         SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' int2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
      return;
   end;
end;
%
% check the sample
%
[FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
T = LastPeriod-FirstPeriod-DSGEModel.KalmanFirstObservation+2;
%
% check if measurement equations are properly defined for all time periods
%
if length(size(H))==3;
   if size(H,3)<DSGEModel.T;
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is less than the number of observations (T = ' int2str(DSGEModel.T) '). YADA has therefore aborted from the spectral decompositions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
   %
   % compute the average H-matrix
   %
   H = H(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
   H = (1/size(H,3))*sum(H,3);
end;
%
% compute the population covariances
%
[SigmaXi,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
SigmaY = (H'*SigmaXi*H)+R;
StdXi = sqrt(diag(SigmaXi));
StdY = sqrt(diag(SigmaY));
%
% compute proper standard deviations matrices, i.e., that can be divided by
%
StdXi(abs(StdXi)<=1.0e-12) = 1;
XiStd = StdXi*StdXi';
YStd = StdY*StdY';
%
% Determine the autocovariance horizon
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
AutoCovHorizon = max(min([SampleFreq*3 T-20 18]),0);
CorrDec.AutoCovHorizon = AutoCovHorizon;
CorrDec.ShockNames = DSGEModel.StateShockNames(KeepVar,:);
CorrDec.ShockAliases = DSGEModel.ShockAliases(KeepVar,:);
if isempty(DSGEModel.ShockColors)==0;
   CorrDec.ShockColors = DSGEModel.ShockColors(KeepVar,:);
else;
   if max(DSGEModel.ShockGroups)==length(DSGEModel.StateShockPositions);
      CorrDec.ShockColors = CreateDefaultColors(length(KeepVar));
   else;
      CorrDec.ShockColors = CreateShockColors(DSGEModel.ShockGroups(KeepVar),DSGEModel.ShockGroupColors);
   end;
end;
CorrDec.ShockGroups = DSGEModel.ShockGroups(KeepVar);
CorrDec.ShockGroupNames = DSGEModel.ShockGroupNames(unique(CorrDec.ShockGroups),:);
if (isempty(DSGEModel.ShockGroupColors)==0)&(size(DSGEModel.ShockGroupColors,1)==length(unique(CorrDec.ShockGroups)));
   CorrDec.ShockGroupColors = DSGEModel.ShockGroupColors(unique(CorrDec.ShockGroups),:);
else;
   CorrDec.ShockGroupColors = CreateDefaultColors(length(unique(CorrDec.ShockGroups)));
end;
CorrDec.R = R;
CorrDec.Y = zeros(n*(n+1)/2,(2*AutoCovHorizon)+1,q+1);
CorrDec.Xi = zeros(r*(r+1)/2,(2*AutoCovHorizon)+1,q);
%
% check if we should display a wait dialog
%
if (n>=7)|(r>=20);
   %
   % initialize the wait dialog
   %
   if nargout>1;
      EstType = 'Initial Values';
   else;
      EstType = 'Posterior Mode';
   end;
   txt = ['Please wait while YADA computes correlation decompositions of the ' lower(VarStr) '.'];
   WaitHandle = WaitDLG(txt,'information',['Correlation Decompositions - ' EstType],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% we can now compute the correlation decomposition for the individual shocks
%
for j=1:q;
   [SigmaXij,stat] = DoublingAlgorithmLyapunov(F,B0(:,j)*B0(:,j)',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
   SigmaYj = H'*SigmaXij*H;
   CorrXi = SigmaXij./XiStd;
   CorrDec.Xi(:,AutoCovHorizon+1,j) = vech(CorrXi);
   CorrY = SigmaYj./YStd;
   CorrDec.Y(:,AutoCovHorizon+1,j) = vech(CorrY);
   FPower = F;
   for i=1:AutoCovHorizon;
      CorrXi = (FPower*SigmaXij)./XiStd;
      CorrDec.Xi(:,AutoCovHorizon+1+i,j)= vech(CorrXi');
      CorrDec.Xi(:,AutoCovHorizon+1-i,j)= vech(CorrXi);
      CorrY = (H'*FPower*SigmaXij*H)./YStd;
      CorrDec.Y(:,AutoCovHorizon+1+i,j)= vech(CorrY');
      CorrDec.Y(:,AutoCovHorizon+1-i,j)= vech(CorrY);
      FPower = FPower*F;
   end;
   drawnow;
end;
%
% and now the measurement error term
%
if rank(R)>0;
   CorrDec.Y(:,AutoCovHorizon+1,q+1) = vech(R./YStd);
end;
%
% close the wait dialog
%
if (n>=7)|(r>=20);
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;

%
% end of DSGECorrelationDecompTheta.m
%
