function [SVEs,status] = DSGECorrelationSimulationTheta(theta,thetaPositions,ModelParameters,NumPaths,EstType,DSGEModel,CurrINI)
% DSGECorrelationSimulationTheta: Estimates observed variable correlations for a DSGE model for a given parameter value.
%                                 The function relies on simulated data and, hence, provides simulation based estimates
%                                 of the sample moments rather than of the population equivalents.
%
% USAGE:
%
%       [SVEs,status] = DSGECorrelationSimulationTheta(theta,thetaPositions,ModelParameters,NumPaths,EstType,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with the posterior mode estimates of the parameters.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                                 of the model parameter in position i of theta. The field "distribution" has a string
%                                 vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  NumPaths (integer) with the number of data paths to simulate per parameter value.
%
%                  EstType (string vector) which determines the type of parameter values that are used. This is
%                                 either "Posterior Mode" or "Initial Values".
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
% REQUIRED OUTPUT: SVEs (structure) with sample means and quantiles of the mean, covariance, autocorrelations,
%                                 and skewness.
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 29, 2008.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 03-03-2008: Added the code for computing the mean and the quantiles.
%
% * 01-04-2008: Change the AutoCovHorizon value to "min([SampleFreq*3 T-20 18])".
%
% * 16-04-2008: The code now checks if the model has a unique convergent solution also at the posterior mode
%               estimates.
%
% * 23-05-2008: Updated the documentation.
%
% * 29-05-2008: Added a test to check if the model can be solved at the mode.
%
% * 09-07-2008: Added code to allow for a time-varying measurement matrix H.
%
% * 09-01-2009: Updated the documentation.
%
% * 13-02-2009: The mcode values 7 and 8 are now properly mentioned when encountered.
%
% * 09-03-2009: Added quotation marks around paths for the dos copy commands.
%
% * 07-04-2009: Changed the maximum height of the "YADA - Error Message" message boxes.
%
% * 05-01-2010: Updated the documentation.
%
% * 16-06-2010: Data estimates of the autocorrelations now use a fixed sample size.
%
% * 24-06-2010: Changed "eye(r,r)" to "eye(r)".
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 19-12-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 17-03-2011: The correlations in the data as well as for the simulated data now take missing
%               observations into account.
%
% * 25-11-2011: The function now also computes the sample skewness and provides it as output.
%
% * 02-12-2011: The function now calculates sample kurtosis.
%
% * 21-12-2011: Made sure that the data is corrected for the mean correctly when computing the mean.
%
% * 05-01-2012: Updated the documentation.
%
% * 27-08-2012: Added support for higest probability density confidence bands.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 05-01-2018: Made sure that AutoCovHorizon is nonnegative.
%
% * 16-05-2018: Fixed an error when computing the autocorrelations for the observed
%               and the simulated data.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 17-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
SVEs = [];
%
% set up the ModelParameters structure based on theta and thetaPositions
%
if nargout==1;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout==2;
      status = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% now we try to solve the model
%
if nargout==1;
   ErrorStr = '';
   if MatlabNumber>=8.0;
      try;
         [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if isempty(ErrMsg)==0;
            ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
         end;
      catch ME;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
      end;
   else;
      try;
         [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if isempty(ErrMsg)==0;
            ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
         end;
      catch;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
      end;
   end;
   if isempty(ErrorStr)==0;
      About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
      return;
   end;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      elseif DSGEModel.ModelSolver==4;
         SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
      return;
   end;
else;
   ErrorStr = '';
   if MatlabNumber>=8.0;
      try;
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      catch ME;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
      end;
   else;
      try;
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      catch;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
      end;
   end;
   if isempty(ErrorStr)==0;
      About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
      return;
   end;
   if status~=1;
      return;
   end;
end;
%
% make sure that only non-zero shocks are kept
%
[B0,KeepShocks] = RemoveRedundantColumns(B0);
n = DSGEModel.n;
k = DSGEModel.k;
[r,q] = size(B0);
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% initial values for the mean and covariance matrix of the distribution of the state variables
%
if isempty(DSGEModel.UnitRootStates)==1;
   [P0,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
else;
   StationaryPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
   P0 = DSGEModel.StateCovConstValue*eye(r);
   F1 = F(StationaryPos,StationaryPos);
   Q1 = B0(StationaryPos,:)*B0(StationaryPos,:)';
   [Pstat,stat] = DoublingAlgorithmLyapunov(F1,Q1,DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
   P0(StationaryPos,StationaryPos) = Pstat;
end;
Ksi0 = DSGEModel.InitialStateVector;
[S,L] = SchurDecomposition(P0);
%
% deal with measurement errors
%
if rank(R)>0;
   [Rm,Lm] = SchurDecomposition(R);
   Rmeasure = Rm*sqrt(Lm);
   NumMeasure = size(Lm,1);
else;
   Rmeasure = [];
   wSim = 0;
   NumMeasure = 0;
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(DSGEModel.LastPeriod) '). YADA has therefore aborted from the observed variable correlations.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% set up Y and X
%
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   X = DSGEModel.X(:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
   XSim = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
else;
   X = [];
   XSim = [];
end;
YTmp = Y;
XTmp = X;
%
% compute the autocorrelations for the actual data
%
T = size(Y,2);
if DSGEModel.YNaNs==0;
   if min(size(X))>0;
      SVEs.Data.Mean = (Y*X')/(X*X');
      %
      % de-mean Y using estimate of the mean from the data rather than A'.
      %
      Y = Y-(SVEs.Data.Mean*X);
   else;
      SVEs.Data.Mean = (1/T)*sum(Y,2);
      Y = Y-(SVEs.Data.Mean*ones(1,T));
   end;
else;
   pos = (~isnan(Y));
   NaNY = isnan(Y);
   Y(NaNY) = 0;
   Tvec = T./sum(pos,2);
   if isempty(DSGEModel.X)==0;
      SVEs.Data.Mean = diag(Tvec)*((Y*X')/(X*X'));
      %
      % de-mean Y using estimate of the mean from the data rather than A'.
      %
      Y = Y-(SVEs.Data.Mean*X);
   else;
      SVEs.Data.Mean = Tvec.*((1/T)*sum(Y,2));
      Y = Y-(SVEs.Data.Mean*ones(1,T));
   end;
   Y(NaNY) = 0;
end;
%
% set up the autocovariance horizon
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
AutoCovHorizon = max(min([SampleFreq*3 T-20 18]),0);
SVEs.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
%
if DSGEModel.YNaNs==0;
   AutoCov = (1/T)*Y*Y';
   CorrVec = sqrt(diag(AutoCov));
   SVEs.Data.Std = CorrVec;
   SVEs.Data.Cov = AutoCov;
   SVEs.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   SVEs.Data.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(CorrVec*CorrVec'));
   SVEs.Data.Skew = ((1/T)*sum(Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec);
   SVEs.Data.Kurt = ((1/T)*sum(Y.*Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec.*CorrVec);
   %
   TFixed = T;
   for i=1:AutoCovHorizon;
      Y = YTmp(:,1+i:size(YTmp,2));
      YL = YTmp(:,1:size(YTmp,2)-i);
      T = size(Y,2);
      if min(size(X))>0;
         X = XTmp(:,1+i:size(YTmp,2));
         XL = XTmp(:,1:size(YTmp,2)-i);
         NewMeanY = (Y*X')/(X*X');
         Y = Y-(NewMeanY*X);
         YL = YL-(NewMeanY*XL);
      else;
         Y = Y-(SVEs.Data.Mean*ones(1,T));
         YL = YL-(SVEs.Data.Mean*ones(1,T));
      end;
      %
      AutoCov = (1/TFixed)*Y*YL';
      Corr = AutoCov./(SVEs.Data.Std*SVEs.Data.Std');
      SVEs.Data.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
      SVEs.Data.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
   end;
else;
   TMat = ones(n,n)./TMatrix(sum(pos,2));
   AutoCov = TMat.*(Y*Y');
   CorrVec = sqrt(diag(AutoCov));
   SVEs.Data.Std = CorrVec;
   SVEs.Data.Cov = AutoCov;
   SVEs.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   SVEs.Data.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(CorrVec*CorrVec'));
   SVEs.Data.Skew = ((Tvec/T).*sum(Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec);
   SVEs.Data.Kurt = ((Tvec/T).*sum(Y.*Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec.*CorrVec);
   %
   TFixed = T;
   for i=1:AutoCovHorizon;
      Y = YTmp(:,1+i:size(YTmp,2));
      YL = YTmp(:,1:size(YTmp,2)-i);
      T = size(Y,2);
      %
      pos = (~isnan(Y));
      NaNY = isnan(Y);
      NaNYL = isnan(YL);
      Y(NaNY) = 0;
      YL(NaNYL) = 0;
      Tvec = T./sum(pos,2);
      %
      if isempty(DSGEModel.X)==0;
         X = XTmp(:,1+i:size(YTmp,2));
         XL = XTmp(:,1:size(YTmp,2)-i);
         NewMeanY = diag(Tvec)*((Y*X')/(X*X'));
         Y = Y-(NewMeanY*X);
         YL = YL-(NewMeanY*XL);
      else;
         Y = Y-(SVEs.Data.Mean*ones(1,T));
         YL = YL-(SVEs.Data.Mean*ones(1,T));
      end;
      Y(NaNY) = 0;
      YL(NaNYL) = 0;
      %
      AutoCov = TMat.*(Y*YL');
      Corr = AutoCov./(SVEs.Data.Std*SVEs.Data.Std');;
      SVEs.Data.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
      SVEs.Data.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
   end;
end;
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
%
% initialize the random number generator
%
if DSGEModel.RandomNumberValue==1;
   randn('state',0);
   rand('state',0);
else;
   randn('state',sum(100*clock));
   rand('state',sum(100*clock));
end;
%
% reset number of observations
%
T = DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1;
Tsum = T-DSGEModel.KalmanFirstObservation+1;
%
% begin the loop
%
abort = '';
MeanEstimationTime = 0;
SVE = [];
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumPaths,'%0.0f') ' simulation paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean estimation time:';
   %
   ProgressStructure.name = ['Autocorrelation Distribution - ' EstType];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA simulates ' int2str(NumPaths) ' samples with data from the model to compute sample means, autocorrelations, and skewness of the observed variables from the simulated data.'];
   WaitHandle = WaitDLG(txt,'information',['Autocorrelation Distribution - ' EstType],600,400,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% start the simulation of data
%
if length(size(H))==2;
   for p=1:NumPaths;
      tic;
      %
      % fix the progress dialog is necessary
      %
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopSimulation = 1;
            break;
         else;
            ProgressDLG([p/NumPaths MeanEstimationTime]);
         end;
      end;
      %
      % draw the economic shocks
      %
      etaSim = randn(q,T);
      %
      % draw the measurement errors
      %
      if isempty(Rmeasure)==0;
         wSim = Rmeasure*randn(NumMeasure,T);
      end;
      %
      % draw an initial state
      %
      StateSim = zeros(r,T+1);
      StateSim(:,1) = Ksi0+(S*sqrt(L)*randn(size(L,2),1));
      %
      % generate the state matrix
      %
      for t=1:T;
         StateSim(:,t+1) = (F*StateSim(:,t))+(B0*etaSim(:,t));
      end;
      drawnow;
      %
      % remove the initial value of the state vector
      %
      StateSim = StateSim(:,2:T+1);
      %
      % create the observed variables
      %
      if k>0;
         YSim = (A'*XSim)+(H'*StateSim)+wSim;
      else;
         YSim = (H'*StateSim)+wSim;
      end;
      %
      % compute the sample moments based on the simulated data
      %
      YTmp = YSim(:,DSGEModel.KalmanFirstObservation:T);
      if DSGEModel.YNaNs==0;
         if k>0;
            XTmp = XSim(:,DSGEModel.KalmanFirstObservation:T);
            MeanY = (YTmp*XTmp')/(XTmp*XTmp');
            YTmp = YTmp-(MeanY*XTmp);
         else;
            MeanY = (1/Tsum)*sum(YTmp,2);
            YTmp = YTmp-MeanY;
         end;
         SVE.CorrData(p).MeanY = MeanY;
         SVE.CorrData(p).StdY = sqrt(diag((1/Tsum)*YTmp*YTmp'));
         %
         % compute simulated skewness
         %
         SVE.CorrData(p).Skew = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and kurtosis
         %
         SVE.CorrData(p).Kurt = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and now for the autocovariances
         %
         SVE.CorrData(p).CorrY = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
         SVE.CorrData(p).CorrY(:,AutoCovHorizon+1) = vech(((1/Tsum)*YTmp*YTmp')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY'));
         for i=1:AutoCovHorizon;
            Yt = YTmp(:,1+i:Tsum);
            YtL = YTmp(:,1:Tsum-i);
            if k>0;
               Xt = XTmp(:,1+i:Tsum);
               XtL = XTmp(:,1:Tsum-i);
               NewMeanYt = (Yt*Xt')/(Xt*Xt');
               Yt = Yt-(NewMeanYt*Xt);
               YtL = YtL-(NewMeanYt*XtL);
            else;
               Yt = Yt-MeanY;
               YtL = YtL-MeanY;
            end;
            Corr = ((1/Tsum)*Yt*YtL')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1+i) = vech(Corr');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1-i) = vech(Corr);         
         end;
      else;
         pos = (~isnan(Y));
         NaNY = isnan(Y);
         YTmp(NaNY) = 0;
         Tvec = T./sum(pos,2);
         if k>0;
            XTmp = XSim(:,DSGEModel.KalmanFirstObservation:T);
            MeanY = diag(Tvec)*(YTmp*XTmp')/(XTmp*XTmp');
            YTmp = YTmp-(MeanY*XTmp);
         else;
            MeanY = Tvec.*((1/Tsum)*sum(YTmp,2));
            YTmp = YTmp-MeanY;
         end;
         YTmp(NaNY) = 0;
         SVE.CorrData(p).MeanY = MeanY;
         SVE.CorrData(p).StdY = sqrt(diag(TMat.*(YTmp*YTmp')));
         %
         % compute simulated skewness
         %
         SVE.CorrData(p).Skew = ((Tvec/Tsum).*sum(YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and kurtosis
         %
         SVE.CorrData(p).Kurt = ((Tvec/Tsum).*sum(YTmp.*YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and now for the autocovariances
         %
         SVE.CorrData(p).CorrY = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
         SVE.CorrData(p).CorrY(:,AutoCovHorizon+1) = vech(((1/Tsum)*YTmp*YTmp')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY'));
         for i=1:AutoCovHorizon;
            Yt = YTmp(:,1+i:Tsum);
            YtL = YTmp(:,1:Tsum-i);
            NaNYt = isnan(Y(:,1+i:Tsum));
            NaNYtL = isnan(Y(:,1:Tsum-i));
            if k>0;
               Xt = XTmp(:,1+i:Tsum);
               XtL = XTmp(:,1:Tsum-i);
               NewMeanYt = diag(Tvec)*((Yt*Xt')/(Xt*Xt'));
               Yt = Yt-(NewMeanYt*Xt);
               YtL = YtL-(NewMeanYt*XtL);
            else;
               Yt = Yt-MeanY;
               YtL = YtL-MeanY;
            end;
            Yt(NaNYt) = 0;
            YtL(NaNYtL) = 0;
            Corr = (TMat.*(Yt*YtL'))./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1+i) = vech(Corr');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1-i) = vech(Corr);         
         end;
      end;
      MeanEstimationTime = (((p-1)/p)*MeanEstimationTime)+((1/p)*toc);
      drawnow;
   end;
else;
   for p=1:NumPaths;
      tic;
      %
      % fix the progress dialog is necessary
      %
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopSimulation = 1;
            break;
         else;
            ProgressDLG([p/NumPaths MeanEstimationTime]);
         end;
      end;
      %
      % draw the economic shocks
      %
      etaSim = randn(q,T);
      %
      % draw the measurement errors
      %
      if isempty(Rmeasure)==0;
         wSim = Rmeasure*randn(NumMeasure,T);
      else;
         wSim = zeros(n,T);
      end;
      %
      % draw an initial state
      %
      StateSim = zeros(r,T+1);
      StateSim(:,1) = Ksi0+(S*sqrt(L)*randn(size(L,2),1));
      %
      % generate the state matrix
      %
      for t=1:T;
         StateSim(:,t+1) = (F*StateSim(:,t))+(B0*etaSim(:,t));
      end;
      drawnow;
      %
      % remove the initial value of the state vector
      %
      StateSim = StateSim(:,2:T+1);
      %
      % create the observed variables
      %
      if k>0;
         for t=DSGEModel.FirstPeriod:DSGEModel.LastPeriod;
            YSim(:,t-DSGEModel.FirstPeriod+1) = (A'*XSim(:,t-DSGEModel.FirstPeriod+1))+(H(:,:,t)'*StateSim(:,t-DSGEModel.FirstPeriod+1))+wSim(:,t-DSGEModel.FirstPeriod+1);
         end;
      else;
         for t=DSGEModel.FirstPeriod:DSGEModel.LastPeriod;
            YSim(:,t-DSGEModel.FirstPeriod+1) = (H(:,:,t)'*StateSim(:,t-DSGEModel.FirstPeriod+1))+wSim(:,t-DSGEModel.FirstPeriod+1);
         end;
      end;
      %
      % compute the sample moments based on the simulated data
      %
      YTmp = YSim(:,DSGEModel.KalmanFirstObservation:T);
      if DSGEModel.YNaNs==0;
         if k>0;
            XTmp = XSim(:,DSGEModel.KalmanFirstObservation:T);
            MeanY = (YTmp*XTmp')/(XTmp*XTmp');
            YTmp = YTmp-(MeanY*XTmp);
         else;
            MeanY = (1/Tsum)*sum(YTmp,2);
            YTmp = YTmp-MeanY;
         end;
         SVE.CorrData(p).MeanY = MeanY;
         SVE.CorrData(p).StdY = sqrt(diag(TMat.*(YTmp*YTmp')));
         %
         % compute simulated skewness
         %
         SVE.CorrData(p).Skew = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and kurtosis
         %
         SVE.CorrData(p).Kurt = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and now for the autocovariances
         %
         SVE.CorrData(p).CorrY = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
         SVE.CorrData(p).CorrY(:,AutoCovHorizon+1) = vech(((1/Tsum)*YTmp*YTmp')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY'));
         for i=1:AutoCovHorizon;
            Yt = YTmp(:,1+i:Tsum);
            YtL = YTmp(:,1:Tsum-i);
            if k>0;
               Xt = XTmp(:,1+i:Tsum);
               Yt = Yt-(((Yt*Xt')/(Xt*Xt'))*Xt);
               XtL = XTmp(:,1:Tsum-i);
               YtL = YtL-(((Yt*Xt')/(Xt*Xt'))*XtL);
            else;
               Yt = Yt-MeanY;
               YtL = YtL-MeanY;
            end;
            Corr = (TMat.*(Yt*YtL'))./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1+i) = vech(Corr');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1-i) = vech(Corr);         
         end;
      else;
         pos = (~isnan(Y));
         NaNY = isnan(Y);
         YTmp(NaNY) = 0;
         Tvec = T./sum(pos,2);
         if k>0;
            XTmp = XSim(:,DSGEModel.KalmanFirstObservation:T);
            MeanY = diag(Tvec)*(YTmp*XTmp')/(XTmp*XTmp');
            YTmp = YTmp-(MeanY*XTmp);
         else;
            MeanY = Tvec.*((1/Tsum)*sum(YTmp,2));
            YTmp = YTmp-MeanY;
         end;
         YTmp(NaNY) = 0;
         SVE.CorrData(p).MeanY = MeanY;
         SVE.CorrData(p).StdY = sqrt(diag((1/Tsum)*YTmp*YTmp'));
         %
         % compute simulated skewness
         %
         SVE.CorrData(p).Skew = ((Tvec/Tsum).*sum(YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and kurtosis
         %
         SVE.CorrData(p).Kurt = ((Tvec/Tsum).*sum(YTmp.*YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and now for the autocovariances
         %
         SVE.CorrData(p).CorrY = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
         SVE.CorrData(p).CorrY(:,AutoCovHorizon+1) = vech(((1/Tsum)*YTmp*YTmp')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY'));
         for i=1:AutoCovHorizon;
            Yt = YTmp(:,1+i:Tsum);
            YtL = YTmp(:,1:Tsum-i);
            NaNYt = isnan(Y(:,1+i:Tsum));
            NaNYtL = isnan(Y(:,1:Tsum-i));
            if k>0;
               Xt = XTmp(:,1+i:Tsum);
               Yt = Yt-(diag(Tvec)*((Yt*Xt')/(Xt*Xt'))*Xt);
               XtL = XTmp(:,1:Tsum-i);
               YtL = YtL-(diag(Tvec)*((Yt*Xt')/(Xt*Xt'))*XtL);
            else;
               Yt = Yt-MeanY;
               YtL = YtL-MeanY;
            end;
            Yt(NaNYt) = 0;
            YtL(NaNYtL) = 0;
            Corr = ((1/Tsum)*Yt*YtL')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1+i) = vech(Corr');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1-i) = vech(Corr);         
         end;
      end;
      MeanEstimationTime = (((p-1)/p)*MeanEstimationTime)+((1/p)*toc);
      drawnow;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;
%
% now we need to set up the distribution results
%
if strcmp(lower(abort),'cancel')==0;
   NumQuants = length(DSGEModel.Percentiles);
   %
   % initialize model based mean, standard deviation, and autocorrelations
   %
   if k>0;
      SVEs.Mean = zeros(n,k);
   else;
      SVEs.Mean = zeros(n,1);
   end;
   SVEs.Std = zeros(n,1);
   SVEs.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   SVEs.Skew = zeros(n,1);
   SVEs.Kurt = zeros(n,1);
   for i=1:NumQuants;
      if k>0;
         SVEs.Quantiles(i).Mean = zeros(n,k);
      else;
         SVEs.Quantiles(i).Mean = zeros(n,1);
      end;
      SVEs.Quantiles(i).Std = zeros(n,1);
      SVEs.Quantiles(i).Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
      SVEs.Quantiles(i).Skew = zeros(n,1);
      SVEs.Quantiles(i).Kurt = zeros(n,1);
      SVEs.Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % setup a wait dialog
   %
   txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. Current variable: ' StringTrim(DSGEModel.VariableNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',['Autocorrelation Distribution - ' EstType],600,400,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   i = 1;
   for i=1:n;
      %
      % setup a wait dialog
      %
      txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. Current variable: ' StringTrim(DSGEModel.VariableNames(i,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      % set up a matrix for the data
      %
      if k>0;
         TempMean = zeros(NumPaths,k);
      else;
         TempMean = zeros(NumPaths,1);
      end;
      TempStd = zeros(NumPaths,1);
      TempSkew = zeros(NumPaths,1);
      TempKurt = zeros(NumPaths,1);
      for p=1:NumPaths;
         TempMean(p,:) = SVE.CorrData(p).MeanY(i,:);
         TempStd(p,1) = SVE.CorrData(p).StdY(i);
         TempSkew(p,1) = SVE.CorrData(p).Skew(i);
         TempKurt(p,1) = SVE.CorrData(p).Kurt(i);
      end;
      drawnow;
      %
      % we can now calculate the mean
      %
      SVEs.Mean(i,:) = (1/NumPaths)*sum(TempMean,1);
      SVEs.Std(i,1) = (1/NumPaths)*sum(TempStd,1);
      SVEs.Skew(i,1) = (1/NumPaths)*sum(TempSkew,1);
      SVEs.Kurt(i,1) = (1/NumPaths)*sum(TempKurt,1);
      %
      % now we can sort these and create the quantiles from the sorted
      %
      for j=1:size(TempMean,2)
         TempMeanj = sort(TempMean(:,j));
         if j==1;
            TempStd = sort(TempStd);
            TempSkew = sort(TempSkew);
            TempKurt = sort(TempKurt);
         end;
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumPaths);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumPaths);
               end;
               SVEs.Quantiles(l).Mean(i,j) = TempMeanj(QuantVal,1);
               if j==1;
                  SVEs.Quantiles(l).Std(i,1) = TempStd(QuantVal,1);
                  SVEs.Quantiles(l).Skew(i,1) = TempSkew(QuantVal,1);
                  SVEs.Quantiles(l).Kurt(i,1) = TempKurt(QuantVal,1);
               end;
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumPaths/100);
               if j==1;
                  BoundsStd = HighestProbabilityDensityBand(TempStd,ConfDraws);
                  SVEs.Quantiles(l).Std(i,1) = BoundsStd(1);
                  SVEs.Quantiles(NumQuants-l+1).Std(i,1) = BoundsStd(2);
                  BoundsSkew = HighestProbabilityDensityBand(TempSkew,ConfDraws);
                  SVEs.Quantiles(l).Skew(i,1) = BoundsSkew(1);
                  SVEs.Quantiles(NumQuants-l+1).Skew(i,1) = BoundsSkew(2);
                  BoundsKurt = HighestProbabilityDensityBand(TempKurt,ConfDraws);
                  SVEs.Quantiles(l).Kurt(i,1) = BoundsKurt(1);
                  SVEs.Quantiles(NumQuants-l+1).Kurt(i,1) = BoundsKurt(2);
               end;
               Bounds = HighestProbabilityDensityBand(TempMeanj,ConfDraws);
               SVEs.Quantiles(l).Mean(i,j) = Bounds(1);
               SVEs.Quantiles(NumQuants-l+1).Mean(i,j) = Bounds(2);
            end;
         end;
         drawnow;
      end;
   end;
   %
   % now we need to deal with the autocorrelations
   %
   SVEs.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
   for j=1:(2*AutoCovHorizon)+1;
      %
      % setup a wait dialog
      %
      txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. Current autocorrelation horizon: ' int2str(j-(AutoCovHorizon+1)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      TempCorr = zeros(NumPaths,(n*(n+1))/2);
      for p=1:NumPaths;
         TempCorr(p,:) = SVE.CorrData(p).CorrY(:,j)';
      end;
      drawnow;
      %
      % we can now calculate the mean
      %
      SVEs.Corr(:,j) = (1/NumPaths)*sum(TempCorr,1)';
      %
      % sort the data and determine the quantile values
      %
      TempCorr = sort(TempCorr);
      if DSGEModel.ConfidenceRegionMethod==1;
         for l=1:NumQuants;
            if DSGEModel.Percentiles(l)<50;
               QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumPaths);
            else;
               QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumPaths);
            end;
            SVEs.Quantiles(l).Corr(:,j) = TempCorr(QuantVal,:)';
         end;
      else;
         for l=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumPaths/100);
            for t=1:n*(n+1)/2;
               Bounds = HighestProbabilityDensityBand(TempCorr(:,t),ConfDraws);
               SVEs.Quantiles(l).Corr(t,j) = Bounds(1);
               SVEs.Quantiles(NumQuants-l+1).Corr(t,j) = Bounds(2);
            end;
         end;
      end;
      drawnow;
   end;
   %
   % close the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
else;
   SVEs = [];
end;

%
% end of DSGECorrelationSimulationTheta.m
%
