function [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix)
% DSGEIdentificationPatterns: Computes identification patterns for the information matrix using Michal
%                             Andrle's ideas via subset selection/strong rank revealing qr-algorithm.
%
% USAGE:
%
%       [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix)
%
% REQUIRED INPUT:  InformationMatrix (matrix) of dimension pxp with the chosen scaling of the estimated identification
%                                    matrix.
%
% REQUIRED OUTPUT: EigenVectorMatrix (matrix) with the eigenvectors linked with the eigenvalues of the information
%                                    matrix in its columns.
%
%                  EigenValues (vector) with the eigenvalues ordered from the largest to the smallest.
%
%                  ParameterOrdering (matrix) of dimension px2 with the selected orderings of the parameters using
%                                    the svd-qr approach (first column) and the qr-approach (second column).
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 23, 2009.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-12-2009: Continued work on the function.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 04-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
p = size(InformationMatrix,2);
ParameterOrdering = zeros(p,2);
%
% first we consider the svd-qr approach
%
k = p;
InfMat = InformationMatrix;
ParamVector = (1:p);
while k>1;
   [U,S,V] = svd(InfMat);
   if k==p;
      EigenVectorMatrix = V;
      EigenValues = diag(S);
   end;
   [Q,R,P] = qr(V');
   SelectParamPos = ParamVector*P(:,k);
   ParameterOrdering(k,1) = SelectParamPos;
   ParamVector = setdiff(ParamVector,SelectParamPos);
   KeepElements = sort((1:k)*P(:,1:k-1));
   InfMat = InfMat(KeepElements,KeepElements);
   k = k-1;
end;
ParameterOrdering(1,1) = ParamVector(1);
%
% second we consider the qr-approach
%
k = p;
InfMat = InformationMatrix;
ParamVector = (1:p);
while k>1;
   [Q,R,P] = qr(InfMat);
   SelectParamPos = ParamVector*P(:,k);
   ParameterOrdering(k,2) = SelectParamPos;
   ParamVector = setdiff(ParamVector,SelectParamPos);
   KeepElements = sort((1:k)*P(:,1:k-1));
   InfMat = InfMat(KeepElements,KeepElements);
   k = k-1;
end;
ParameterOrdering(1,2) = ParamVector(1);

%
% end of DSGEIdentificationPatterns.m
%
