function DoneCalc = DSGEMomentsResponse(thetaDraws,theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,OutputType,CurrChain,IsPosterior,TotalDraws,DSGEModel,CurrINI)
% DSGEMomentsResponse: Computes the distribution of the numerical partial derivatives of the means,
%                      standard deviations, and correlations with respect to a change in one structural
%                      parameters using either prior or posterior draws.
%
% USAGE:
%
%       DoneCalc = DSGEMomentsResponse(thetaDraws,theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,...
%                                      OutputType,CurrChain,IsPosterior,TotalDraws,DSGEModel,CurrINI)

% REQUIRED INPUT:  thetaDraws (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                             distribution of the original DSGE model parameters.
%
%                  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  ParameterNames (string matrix) with the names of the parameters.
%
%                  ParamNumber (integer) with the position of the parameter in theta to change.
%
%                  OutputType (boolean) that takes the value 1 if responses to the moments should be computed
%                             for the observed variables, and 0 if the responses should be computed for the
%                             state variables.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                             if draws from the prior are supplied.
%
%                  TotalDraws (integer) with the total number of draws that could have been used.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 24, 2014.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-03-2014: Finished the function.
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 17-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 12-09-2018: Added code related to potential memory issues.
%
% * 24-01-2019: Updated the documentation.
%
% * 16-04-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize output
%
DoneCalc = 0;
[NumDraws,NumParam] = size(thetaDraws);
ParamStr = StringTrim(ParameterNames(ParamNumber,:));
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'paramchange');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\paramchange". The computation distributions for the numerical partial derivatives of the change in the parameter "' ParamStr '" to the means, standard deviations, and correlations has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/paramchange". The computation distributions for the numerical partial derivatives of the change in the parameter "' ParamStr '" to the means, standard deviations, and correlations has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% solve the model for the posterior mode estimate
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [Amode,Hmode,Rmode,Fmode,B0mode,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [Amode,Hmode,Rmode,Fmode,B0mode,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(Hmode))==3;
   [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
   if size(Hmode,3)<LastPeriod;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(Hmode,3)) ') is lower than the number of observations (T = ' int2str(LastPeriod) '). YADA has therefore aborted from computing the effects on population means, standard deviations and correlations from a change in the parameter "' ParamStr '".'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
k = DSGEModel.k;
n = DSGEModel.n;
[r,q] = size(B0mode);
IdentMat = eye(length(theta));
thetaMode = theta;
%
if (n>20)|(r>100);
   YADApack;
end;
%
% initialize index for saving
%
SaveAfterDraws = min(NumDraws,100);
NumFiles = ceil(NumDraws/SaveAfterDraws);
%
% begin the loop
%
abort = '';
MeanEstimationTime = 0;
LastDraw = 0;
ThisSave = 0;
PD = [];
mcodevec = zeros(NumDraws,1);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(NumDraws) ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean estimation time:';
   %
   ProgressStructure.name = ['Parameter Change To "' ParamStr '"' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the distribution of the partial derivatives of the means, standard deviations and correlations from a change in the parameter "' ParamStr '". Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Parameter Change To "' ParamStr '"' PriorHeader],500,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% start the iterations
%
for it=1:NumDraws;
   LastDraw = LastDraw+1;
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if strcmp(abort,'cancel')==1;
         break;
      else;
         ProgressDLG([it/NumDraws MeanEstimationTime]);
      end;
   end;
   %
   % Solve the model for the current parameter vector
   %
   theta = thetaDraws(it,:)';
   tic;
   [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if status~=1;
      %
      % use the posterior mode or initial values
      %
      A = Amode;
      H = Hmode;
      R = Rmode;
      F = Fmode;
      B0 = B0mode;
      theta = thetaMode;
   end;
   %
   % compute the moments at theta
   %
   if OutputType==1;
      if k>0;
         MeanMat = A';
      end;
      [CovMat,stat]  = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      if stat==1;
         CovMat = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r)
      end;
      if length(size(H))==2;
         CovMat = (H'*CovMat*H)+R;
      else;
         [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
         H = H(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
         H = (1/size(H,3))*sum(H,3);
         CovMat = (H'*CovMat*H)+R;
      end;
   else;
      [CovMat,stat]  = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      if stat==1;
         CovMat = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r)
      end;
   end;
   StdVec = real(sqrt(diag(CovMat)));
   if n>1;
      CorrVec = vechl(CovMat./(StdVec*StdVec'));
   else;
      CorrResp = [];
   end;
   %
   % now make the parameter change
   %
   thetaChange = 0.001*abs(theta);
   thetaAltChange = 0.0001*abs(theta);
   ParamDiff = thetaChange(ParamNumber);
   thetaNew = theta+(ParamDiff*IdentMat(:,ParamNumber));
   %
   % try to solve the model at the new value
   %
   [A,H,R,F,B0,mcode] = SolveDSGEModel(DSGEModel,thetaNew,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if mcode~=1;
      ParamDiff = thetaAltChange(ParamNumber);
      thetaNew = theta+(ParamDiff*IdentMat(:,ParamNumber));
      [A,H,R,F,B0,mcode] = SolveDSGEModel(DSGEModel,thetaNew,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if mcode~=1;
         %
         % make sure we use the mode estimate of theta
         %
         A = Amode;
         H = Hmode;
         R = Rmode;
         F = Fmode;
         B0 = B0mode;
         theta = thetaMode;
         %
         % recompute the moments
         %
         if OutputType==1;
            if k>0;
               MeanMat = A';
            end;
            [CovMat,stat]  = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
            if stat==1;
               CovMat = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r)
            end;
            if length(size(H))==2;
               CovMat = (H'*CovMat*H)+R;
            else;
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               H = H(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
               H = (1/size(H,3))*sum(H,3);
               CovMat = (H'*CovMat*H)+R;
            end;
         else;
            [CovMat,stat]  = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
            if stat==1;
               CovMat = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r)
            end;
         end;
         StdVec = real(sqrt(diag(CovMat)));
         if n>1;
            CorrVec = vechl(CovMat./(StdVec*StdVec'));
         end;
         %
         % impose the parameter change
         %
         thetaNew = theta+(ParamDiff*IdentMat(:,ParamNumber));
         %
         % try one last time to solve. if not possible we skip this draw
         %
         [A,H,R,F,B0,mcode] = SolveDSGEModel(DSGEModel,thetaNew,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
   end;
   mcodevec(it) = (mcode==1);
   if mcode==1;
      %
      % compute the moments at the new values
      %
      if OutputType==1;
         if k>0;
            MeanResp = (A'-MeanMat)./ParamDiff;
         else;
            MeanResp = zeros(n,1);
         end;
         [CovMat,stat]  = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
         if stat==1;
            CovMat = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r)
         end;
         if length(size(H))==2;
            CovMat = (H'*CovMat*H)+R;
         else;
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            H = H(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
            H = (1/size(H,3))*sum(H,3);
            CovMat = (H'*CovMat*H)+R;
         end;
         PD(LastDraw).MeanResp = MeanResp;
      else;
         MeanResp = zeros(r,1);
         [CovMat,stat]  = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
         if stat==1;
            CovMat = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r);
         end;
      end;
      %
      % compute the remaining numerical partials
      %
      StdVecNew = real(sqrt(diag(CovMat)));
      StdResp = (StdVecNew-StdVec)./ParamDiff;
      if n>1;
         CorrVecNew = vechl(CovMat./(StdVecNew*StdVecNew'));
         CorrResp = (CorrVecNew-CorrVec)./ParamDiff;
      end;
      PD(LastDraw).StdResp = StdResp;
      PD(LastDraw).CorrResp = CorrResp;
   end;
   MeanEstimationTime = (((it-1)/it)*MeanEstimationTime)+((1/it)*toc);
   %
   % check if we should save to disk
   %
   if (LastDraw==SaveAfterDraws)|(it==NumDraws);
      ThisSave = ThisSave+1;
      LastDraw = 0;
      %
      % set up the file name and then save
      %
      if OutputType==1;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-ObsVar-ParamChange-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-ObsVar-ParamChange-Prior-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
         end;
      else;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-StateVar-ParamChange-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-StateVar-ParamChange-Prior-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
         end;
      end;
      save(file,'PD');
      %
      % restore the data structure
      %
      PD = [];
   end;
   drawnow;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if we have computed all output
%
if strcmp(abort,'cancel')==0;
   if OutputType==1;
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-ObsVar-ParamChange-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-1-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-ObsVar-ParamChange-Prior-' ParamStr '-' DSGEModel.NameOfModel '-1-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
      end;
   else;
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-StateVar-ParamChange-' ParamStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-1-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-StateVar-ParamChange-Prior-' ParamStr '-' DSGEModel.NameOfModel '-1-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
      end;
   end;
   save(file,'mcodevec','-APPEND');
   DoneCalc = 1;
end;

%
% end of DSGEMomentsResponse.m
%
