function [StateVarStructure,TotalPaths,status,kalmanstatus] = DSGESimulationSmootherTheta(theta,thetaPositions,ModelParameters,VarType,EstStr,DSGEModel,controls,CurrINI)
% DSGESimulationSmootherTheta: Computes the distribution of the state variables, shocks and measurement errors conditional on
%                              the data for fixed parameter values. Antithetic variables for balancing the location and scale
%                              and for speeding up the simulations are used.
%
% USAGE:
%
%       [StateVarStructure,TotalPaths,status,kalmanstatus] = DSGESimulationSmootherTheta(theta,thetaPositions,ModelParameters,VarType,...
%                                                                             EstStr,DSGEModel,controls,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  VarType (integer) which determines the type of unobserved variable that should be
%                                 computed. 1 is state variable, 2 is economic shock, and 3 is measurement
%                                 error.
%
%                  EstStr (string) that takes the values "Initial Values" or "Posterior Mode".
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  controls (structure) with handles to all controls on the YADA main GUI.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: StateVarStructure (structure) with fields "KsitT", "PtT" and "StateVariableNames" if "VarType" is
%                                 1; "etatT" and "ShockNames" if "VarType" is 2; and "wtT", "VariableNames" and
%                                 "ErrorIndicator" if "VarType" is 3 and the model has measurement errors. In addition,
%                                 the output fields are "Quantiles" and "percent", with subfield names that
%                                 depend on "VarType".
%
%                  TotalPaths (integer) with the total number of paths used for the simulation smoother.
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%                  kalmanstatus (boolean) indicating the covariance stationarity of the state variables. If all the
%                                 eigenvalues of F are inside the unit circle, then kalmanstatus=0 and the log-likelihood function
%                                 is calculated. If not, then status=1 and initP is set to 0. In addition, kalmanstatus can be
%                                 -1 if ths forecast covariance matrix is not positive definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 7, 2008.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-10-2010: Continued work on the function.
%
% * 11-10-2010: To the sounds of Spock's Beard, began the actual simulations.
%
% * 12-10-2010: This seems to be dragging out a bit. Finished the simulation part. Sorting remains.
%
% * 13-10-2010: Started working on the sorting.
%
% * 14-10-2010: Finished the function.
%
% * 15-10-2010: Updated the documentation.
%
% * 17-12-2010: The function now supports the Klein and Sims DSGE model solvers.
%
% * 19-12-2010: Updated the documentation.
%
% * 03-01-2011: The function now supports the square root filter.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 11-03-2011: Switched to the "RunKalmanFilter" function.
%
% * 15-03-2011: Switched to the "RunKalmanSmoother" function.
%
% * 16-03-2011: The function now supports missing observations, including simulated values of the
%               observed variables.
%
% * 08-04-2011: The function now uses "ShockAliases" in "ShockNames" instead of "StateShockNames".
%
% * 24-08-2011: Made sure "F" and "B0" are included in the "ModelParameters" structure via the "YADA" field.
%
% * 25-10-2011: Took into account that "RunKalmanSmoother" now requires the input variable "DiffuseInit".
%
% * 06-01-2012: Updated the documentation.
%
% * 29-02-2012: Took into account that the number of simulated paths per parameter value can be unity.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 28-08-2012: Added support for higest probability density confidence bands.
%
% * 04-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 17-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-12-2019: Took into account that the DSGEModel.ModelSolverFile now requires the three additional
%               input variables "DSGEModel.StateVariableNames", "DSGEModel.StateEquationNames" and 
%               "DSGEModel.StateShockNames".
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed additional eval calls to feval.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
% * 26-02-2024: Changed the way the smooth structural shocks are computed.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors for "RunKalmanSmoother".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
StateVarStructure = [];
TotalPaths = [];
%
% set up the ModelParameters structure based on theta and thetaPositions
%
if nargout==2;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
else;
   status = 1;
   kalmanstatus = 0;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>2;
      status = 0;
      kalmanstatus = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% See if we can solve the model
%
if nargout==2;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   if MatlabNumber>=8.0;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch ME;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError(ME)];
         end;
         kalmanstatus = 0;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   else;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError];
         end;
         kalmanstatus = 0;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   end;
end;
if nargout>2;
   status = mcode;
   if status~=1;
      return;
   end;
else;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      elseif DSGEModel.ModelSolver==4;
         SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
      return;
   end;
end;
%
% Convert to the state equation matrices
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% obtain the measurement equation matrices
%
if MatlabNumber>=8.0;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch ME;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
else;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
if length(size(H))==3;
   if size(H,3)<size(DSGEModel.Y,2);
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(size(DSGEModel.Y,2)) '). YADA has therefore aborted from the calculations.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
if (VarType==3)&(rank(R)==0);
   txt = ['The DSGE model "' DSGEModel.NameOfModel '" subject to the state variable constraints doesn''t have any measurement errors.'];
   About(txt,'information','Measurement Errors',500,600,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% We can now try to run the Kalman filter
%
if MatlabNumber>=8.0;
   try;
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
   catch ME;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      return;
   end;
else;
   try;
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
   catch;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      return;
   end;
end;
%
% we should adjust everything based on DSGEModel.KalmanFirstObservation
%
if nargout>2;
   kalmanstatus = stat;
   if kalmanstatus~=0;
      return;
   end;
end;
%
% Run the state smoother
%
if MatlabNumber>=8.0;
   try;
      [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   catch ME;
      Ksitt = [];
      KsitT = [];
      NtMat = [];
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
else;
   try;
      [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   catch;
      Ksitt = [];
      KsitT = [];
      NtMat = [];
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
%
% now we can calculate the shocks
%
[B0,KeepShocks] = RemoveRedundantColumns(B0);
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
T = size(Y,2);
n = DSGEModel.n;
k = DSGEModel.k;
[r,q] = size(B0);
%
% check if all
%
if VarType==2;
   CondCMat = eye(q)-(B0'*NtMat(:,:,T)*B0);
   if max(max(abs(CondCMat)))<1.0e-010;
      txt = 'YADA has concluded that all state shocks can be observed from the full sample. There is therefore no reason to run the simulation smoother.';
      About(txt,'error',['State Shock Simulation Smoother - ' EstStr],200,500,CurrINI);
      return;
   end;
end;
%
if isempty(DSGEModel.X)==0;
   X = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
   Ax = A'*X;
else;
   Ax = zeros(n,T);
   X = [];
end;
%
% check if we should compute something else
%
if VarType==1
   StateVarStructure.KsitT = KsitT(:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.PtT = PtT(:,:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.StateVariableNames = DSGEModel.StateVariableNames;
   VarStr = 'State Variable';
elseif VarType==2;
   etatT = B0'*rTMat;
   StateVarStructure.etatT = etatT(:,DSGEModel.KalmanFirstObservation:T);
   ShockAliases = DSGEModel.ShockAliases(KeepShocks,:);
   for j=1:size(ShockAliases,1);
      ShockAliases(j,:) = strrep(ShockAliases(j,:),'_',' ');
   end;
   StateVarStructure.ShockNames = ShockAliases;
   VarStr = 'State Shock';
elseif VarType==3;
   if length(size(H))==2;
      wtT = Y-Ax-(H'*KsitT);
   else;
      wtT = zeros(size(Y));
      for t=1:size(wtT,2);
         wtT(:,t) = Y(:,t)-Ax(:,t)-(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
      end;
   end;
   StateVarStructure.wtT = wtT(:,DSGEModel.KalmanFirstObservation:T);;
   StateVarStructure.ErrorIndicator = zeros(n,1);
   StateVarStructure.VariableNames = DSGEModel.VariableNames;
   for i=1:n;
      if R(i,i)>0;
         StateVarStructure.ErrorIndicator(i) = 1;
      end;
   end;
   VarStr = 'Measurement Error';
end;
%
% initial values for the mean and covariance matrix of the distribution of the state variables
%
if isempty(DSGEModel.UnitRootStates)==1;
   [P0,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
else;
   StationaryPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
   P0 = DSGEModel.StateCovConstValue*eye(r);
   F1 = F(StationaryPos,StationaryPos);
   Q1 = B0(StationaryPos,:)*B0(StationaryPos,:)';
   [Pstat,stat] = DoublingAlgorithmLyapunov(F1,Q1,DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
   P0(StationaryPos,StationaryPos) = Pstat;
end;
Ksi0 = DSGEModel.InitialStateVector;
[PS,PL] = SchurDecomposition(P0);
%
% deal with measurement errors
%
if rank(R)>0;
   [Rm,Lm] = SchurDecomposition(R);
   Rmeasure = Rm*sqrt(Lm);
   NumMeasure = size(Lm,1);
else;
   Rmeasure = [];
   wSim = zeros(n,T);
   NumMeasure = 0;
end;
DF = (T-DSGEModel.KalmanFirstObservation+1)*(q+NumMeasure);
%
% check how critical values from the chi-square should be computed
%
if exist('YADAchi2inv')==2;
   UseYADAVersion = 2;
else;
   if exist('chi2inv')==2;
      UseYADAVersion = 1;
   else;
      UseYADAVersion = 0;
   end;
end;
%
% initialize the random number generator
%
if DSGEModel.RandomNumberValue==1;
   randn('state',0);
   rand('state',0);
else;
   randn('state',sum(100*clock));
   rand('state',sum(100*clock));
end;
%
% determine how many paths to compute
%
NumPathsStr = get(controls.forecast.predpaths,'String');
TotalPaths = max(str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',','')),4);
NumPaths = TotalPaths/4;
%
% check if we should add a progress dialog
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(TotalPaths,'%0.0f') ' ' lower(VarStr) ' draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean estimation time:';
   %
   ProgressStructure.name = [VarStr ' Simulation Smoother - ' EstStr];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the simulation smoother for the ' lower(VarStr) 's at ' lower(EstStr) '. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',[VarStr ' Simulation Smoother - ' EstStr],500,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% initialize the simulations
%
abort = '';
LastDraw = 0;
MeanEstimationTime = 0;
SSResults = [];
%
if length(size(H))==2;
   for LastDraw=1:NumPaths;
      tic;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            break;
         else;
            ProgressDLG([(4*LastDraw)/TotalPaths MeanEstimationTime]);
         end;
      end;
      %
      % draw the economic shocks
      %
      etaSim = randn(q,T);
      %
      % draw the initial state
      %
      StateSim = zeros(r,T+1);
      StateSim(:,1) = Ksi0+(PS*sqrt(PL)*randn(size(PL,2),1));
      %
      % draw the measurement errors
      %
      if isempty(Rmeasure)==0;
         wSim = Rmeasure*randn(NumMeasure,T);
      end;
      %
      % generate the state matrix
      %
      for t=1:T;
         StateSim(:,t+1) = (F*StateSim(:,t))+(B0*etaSim(:,t));
      end;
      drawnow;
      %
      % remove the initial value of the state vector
      %
      StateSim = StateSim(:,2:T+1);
      %
      % generate the data
      %
      YSim = Ax+(H'*StateSim)+wSim;
      if DSGEModel.YNaNs==1;
         %
         % make sure that YSim has NaNs in the same place as Y
         %
         YSim(isnan(Y)) = NaN;
      end;
      %
      % take the training sample into account
      %
      if DSGEModel.KalmanFirstObservation>1;
         etaSim = etaSim(:,DSGEModel.KalmanFirstObservation:T);
         if isempty(Rmeasure)==0;
            wSim = wSim(:,DSGEModel.KalmanFirstObservation:T);
         end;
         StateSim = StateSim(:,DSGEModel.KalmanFirstObservation:T);
      end;
      %
      % compute the smooth estimates for the simulated data
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(YSim,X,1,size(YSim,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      [Ksitt,Ptt,SimKsitT,PtT,rttMat,SimrTMat,NtMat] = RunKalmanSmoother(YSim,1,size(YSim,2),Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      %
      % collect the results
      %
      if VarType==1;
         SSResults(LastDraw,1).KsitT = StateVarStructure.KsitT+StateSim-SimKsitT(:,DSGEModel.KalmanFirstObservation:T);
         %
         % antithetic for balancing the location
         %
         SSResults(LastDraw,2).KsitT = StateVarStructure.KsitT-StateSim+SimKsitT(:,DSGEModel.KalmanFirstObservation:T);
      elseif VarType==2;
         SimetatT = B0'*SimrTMat;
         SSResults(LastDraw,1).etatT = StateVarStructure.etatT+etaSim-SimetatT(:,DSGEModel.KalmanFirstObservation:T);
         %
         % antithetic for balancing the location
         %
         SSResults(LastDraw,2).etatT = StateVarStructure.etatT-etaSim+SimetatT(:,DSGEModel.KalmanFirstObservation:T);
      else;
         SimwtT = YSim-Ax-(H'*SimKsitT);
         SSResults(LastDraw,1).wtT = StateVarStructure.wtT+wSim-SimwtT(:,DSGEModel.KalmanFirstObservation:T);
         %
         % antithetic for balancing the location
         %
         SSResults(LastDraw,2).wtT = StateVarStructure.wtT-wSim+SimwtT(:,DSGEModel.KalmanFirstObservation:T);
      end;
      %
      % compute c
      %
      etavec = vec(etaSim);
      wvec = vec(wSim);
      c = (etavec'*etavec)+(wvec'*wvec);
      %
      % compute p and d
      %
      if UseYADAVersion==2;
         p = YADAchi2cdf(c,DF);
         d = YADAchi2inv(1-p,DF);
      elseif UseYADAVersion==1;
         p = chi2cdf(c,DF);
         d = chi2inv(1-p,DF);
      else;
         p = YADACheapchi2cdf(c,DF);
         d = YADACheapGammaInv(1-p,(DF/2))*2;
      end;
      %
      % antithetics for balancing for scale
      %
      AdjFactor = sqrt(d/c);
      if VarType==1;
         SSResults(LastDraw,3).KsitT = StateVarStructure.KsitT+(AdjFactor*(SSResults(LastDraw,1).KsitT-StateVarStructure.KsitT));
         SSResults(LastDraw,4).KsitT = StateVarStructure.KsitT+(AdjFactor*(SSResults(LastDraw,2).KsitT-StateVarStructure.KsitT));
      elseif VarType==2;
         SSResults(LastDraw,3).etatT = StateVarStructure.etatT+(AdjFactor*(SSResults(LastDraw,1).etatT-StateVarStructure.etatT));
         SSResults(LastDraw,4).etatT = StateVarStructure.etatT+(AdjFactor*(SSResults(LastDraw,2).etatT-StateVarStructure.etatT));
      else;
         SSResults(LastDraw,3).wtT = StateVarStructure.wtT+(AdjFactor*(SSResults(LastDraw,1).wtT-StateVarStructure.wtT));
         SSResults(LastDraw,4).wtT = StateVarStructure.wtT+(AdjFactor*(SSResults(LastDraw,2).wtT-StateVarStructure.wtT));
      end;
      MeanEstimationTime = (((LastDraw-1)/LastDraw)*MeanEstimationTime)+((1/LastDraw)*toc);
      drawnow;
      pause(0.02);
   end;
else;
   for LastDraw=1:NumPaths;
      tic;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            break;
         else;
            ProgressDLG([(4*LastDraw)/TotalPaths MeanEstimationTime]);
         end;
      end;
      %
      % draw the economic shocks
      %
      etaSim = randn(q,T);
      %
      % draw the initial state
      %
      StateSim = zeros(r,T+1);
      StateSim(:,1) = Ksi0+(PS*sqrt(PL)*randn(size(PL,2),1));
      %
      % draw the measurement errors
      %
      if isempty(Rmeasure)==0;
         wSim = Rmeasure*randn(NumMeasure,T);
      end;
      %
      % generate the state matrix
      %
      for t=1:T;
         StateSim(:,t+1) = (F*StateSim(:,t))+(B0*etaSim(:,t));
      end;
      drawnow;
      %
      % remove the initial value of the state vector
      %
      StateSim = StateSim(:,2:T+1);
      %
      % generate the data
      %
      YSim = zeros(n,T);
      for t=1:T;
         YSim(:,t) = Ax(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*StateSim(:,t))+wSim(:,t);
      end;
      %
      if DSGEModel.YNaNs==1;
         %
         % make sure that YSim has NaNs in the same place as Y
         %
         YSim(isnan(Y)) = NaN;
      end;
      %
      % take the training sample into account
      %
      if DSGEModel.KalmanFirstObservation>1;
         etaSim = etaSim(:,DSGEModel.KalmanFirstObservation:T);
         if isempty(Rmeasure)==0;
            wSim = wSim(:,DSGEModel.KalmanFirstObservation:T);
         end;
         StateSim = StateSim(:,DSGEModel.KalmanFirstObservation:T);
      end;
      %
      % compute the smooth estimates for the simulated data
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(YSim,X,1,size(YSim,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      [Ksitt,Ptt,SimKsitT,PtT,rttMat,SimrTMat,NtMat] = RunKalmanSmoother(YSim,1,size(YSim,2),Yhat,Ksitt1,Ptt1,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      %
      % collect the results
      %
      if VarType==1;
         SSResults(LastDraw,1).KsitT = StateVarStructure.KsitT+StateSim-SimKsitT(:,DSGEModel.KalmanFirstObservation:T);
         %
         % antithetic for balancing the location
         %
         SSResults(LastDraw,2).KsitT = StateVarStructure.KsitT-StateSim+SimKsitT(:,DSGEModel.KalmanFirstObservation:T);
      elseif VarType==2;
         SimetatT = B0'*SimrTMat;
         SSResults(LastDraw,1).etatT = StateVarStructure.etatT+etaSim-SimetatT(:,DSGEModel.KalmanFirstObservation:T);
         %
         % antithetic for balancing the location
         %
         SSResults(LastDraw,2).etatT = StateVarStructure.etatT-etaSim+SimetatT(:,DSGEModel.KalmanFirstObservation:T);
      else;
         SimwtT = zeros(n,T);
         for t=1:T;
            SimwtT(:,t) = YSim(:,t)-Ax(:,t)-(H(:,:,DSGEModel.FirstPeriod+t-1)'*SimKsitT(:,t));
         end;
         SSResults(LastDraw,1).wtT = StateVarStructure.wtT+wSim-SimwtT(:,DSGEModel.KalmanFirstObservation:T);
         %
         % antithetic for balancing the location
         %
         SSResults(LastDraw,2).wtT = StateVarStructure.wtT-wSim+SimwtT(:,DSGEModel.KalmanFirstObservation:T);
      end;
      %
      % compute c
      %
      etavec = vec(etaSim);
      wvec = vec(wSim);
      c = (etavec'*etavec)+(wvec'*wvec);
      %
      % compute p and d
      %
      if UseYADAVersion==2;
         p = YADAchi2cdf(c,DF);
         d = YADAchi2inv(1-p,DF);
      elseif UseYADAVersion==1;
         p = chi2cdf(c,DF);
         d = chi2inv(1-p,DF);
      else;
         p = YADACheapchi2cdf(c,DF);
         d = YADACheapGammaInv(1-p,(DF/2))*2;
      end;
      %
      % antithetics for balancing for scale
      %
      AdjFactor = sqrt(d/c);
      if VarType==1;
         SSResults(LastDraw,3).KsitT = StateVarStructure.KsitT+(AdjFactor*(SSResults(LastDraw,1).KsitT-StateVarStructure.KsitT));
         SSResults(LastDraw,4).KsitT = StateVarStructure.KsitT+(AdjFactor*(SSResults(LastDraw,2).KsitT-StateVarStructure.KsitT));
      elseif VarType==2;
         SSResults(LastDraw,3).etatT = StateVarStructure.etatT+(AdjFactor*(SSResults(LastDraw,1).etatT-StateVarStructure.etatT));
         SSResults(LastDraw,4).etatT = StateVarStructure.etatT+(AdjFactor*(SSResults(LastDraw,2).etatT-StateVarStructure.etatT));
      else;
         SSResults(LastDraw,3).wtT = StateVarStructure.wtT+(AdjFactor*(SSResults(LastDraw,1).wtT-StateVarStructure.wtT));
         SSResults(LastDraw,4).wtT = StateVarStructure.wtT+(AdjFactor*(SSResults(LastDraw,2).wtT-StateVarStructure.wtT));
      end;
      MeanEstimationTime = (((LastDraw-1)/LastDraw)*MeanEstimationTime)+((1/LastDraw)*toc);
      drawnow;
      pause(0.02);
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if the user cancelled
%
if strcmp(lower(abort),'cancel')==1;
   clear('SSResults');
   StateVarStructure = [];
   return;
end;
%
% now we can attempt to compute distributional aspects of the simulation smoother
%
NumQuants = length(DSGEModel.Percentiles);
T = T-DSGEModel.KalmanFirstObservation+1;
if VarType==1;
   %
   % sort the simulated distribution of the state variables
   %
   for i=1:NumQuants;
      Quantiles(i).KsitT = zeros(r,T);
      Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % open a wait dialog
   %
   txt = ['Please wait while YADA computes the percentile values for the state variable paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(StateVarStructure.StateVariableNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',[VarStr ' Simulation Smoother - ' EstStr],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   %
   %
   for i=1:r;
      txt = ['Please wait while YADA computes the percentile values for the state variable paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(StateVarStructure.StateVariableNames(i,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      % set up matrices for storing the data
      %
      TempT = zeros(TotalPaths,T);
      for j=1:NumPaths;
         TempT(4*(j-1)+1,:) = SSResults(j,1).KsitT(i,:);
         TempT(4*(j-1)+2,:) = SSResults(j,2).KsitT(i,:);
         TempT(4*(j-1)+3,:) = SSResults(j,3).KsitT(i,:);
         TempT(4*(j-1)+4,:) = SSResults(j,4).KsitT(i,:);
         drawnow;
      end;
      %
      % sort each column of TempT
      %
      TempT = sort(TempT);
      if DSGEModel.ConfidenceRegionMethod==1;
         for l=1:NumQuants;
            if DSGEModel.Percentiles(l)<50;
               QuantVal = ceil((DSGEModel.Percentiles(l)/100)*TotalPaths);
            else;
               QuantVal = floor((DSGEModel.Percentiles(l)/100)*TotalPaths);
            end;
            Quantiles(l).KsitT(i,:) = TempT(QuantVal,:);
         end;
      else;
         for l=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*TotalPaths/100);
            for t=1:T;
               Bounds = HighestProbabilityDensityBand(TempT(:,t),ConfDraws);
               Quantiles(l).KsitT(i,t) = Bounds(1);
               Quantiles(NumQuants-l+1).KsitT(i,t) = Bounds(2);
            end;
         end;
      end;
      drawnow;
   end;
elseif VarType==2;
   %
   % sort the simulated distribution of the state shocks
   %
   for i=1:NumQuants;
      Quantiles(i).etatT = zeros(q,T);
      Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % open a wait dialog
   %
   txt = ['Please wait while YADA computes the percentile values for the state shock paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(StateVarStructure.ShockNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',[VarStr ' Simulation Smoother - ' EstStr],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   %
   for i=1:q;
      txt = ['Please wait while YADA computes the percentile values for the state shock paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(StateVarStructure.ShockNames(i,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      % set up matrices for storing the data
      %
      TempT = zeros(TotalPaths,T);
      for j=1:NumPaths;
         TempT(4*(j-1)+1,:) = SSResults(j,1).etatT(i,:);
         TempT(4*(j-1)+2,:) = SSResults(j,2).etatT(i,:);
         TempT(4*(j-1)+3,:) = SSResults(j,3).etatT(i,:);
         TempT(4*(j-1)+4,:) = SSResults(j,4).etatT(i,:);
         drawnow;
      end;
      %
      % sort each column of TempT
      %
      TempT = sort(TempT);
      if DSGEModel.ConfidenceRegionMethod==1;
         for l=1:NumQuants;
            if DSGEModel.Percentiles(l)<50;
               QuantVal = ceil((DSGEModel.Percentiles(l)/100)*TotalPaths);
            else;
               QuantVal = floor((DSGEModel.Percentiles(l)/100)*TotalPaths);
            end;
            Quantiles(l).etatT(i,:) = TempT(QuantVal,:);
         end;
      else;
         for l=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*TotalPaths/100);
            for t=1:T;
               Bounds = HighestProbabilityDensityBand(TempT(:,t),ConfDraws);
               Quantiles(l).etatT(i,t) = Bounds(1);
               Quantiles(NumQuants-l+1).etatT(i,t) = Bounds(2);
            end;
         end;
      end;
      drawnow;
   end;
else;
   %
   % sort the simulated distribution of the measurement errors
   %
   for i=1:NumQuants;
      Quantiles(i).wtT = zeros(n,T);
      Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % open a wait dialog
   %
   txt = ['Please wait while YADA computes the percentile values for the measurement error paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Measurement error for variable: ' StringTrim(DSGEModel.VariableNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',[VarStr ' Simulation Smoother - ' EstStr],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   %
   for i=1:n;
      if StateVarStructure.ErrorIndicator(i)==1;
         txt = ['Please wait while YADA computes the percentile values for the measurement error paths from the simulation smoother. The data are loaded sequentially from disk and it may therefore take some time. Measurement error for variable: ' StringTrim(DSGEModel.VariableNames(i,:)) '.'];
         set(WaitControls.text,'String',txt);
         drawnow;
         %
         % set up matrices for storing the data
         %
         TempT = zeros(TotalPaths,T);
         for j=1:NumPaths;
            TempT(4*(j-1)+1,:) = SSResults(j,1).wtT(i,:);
            TempT(4*(j-1)+2,:) = SSResults(j,2).wtT(i,:);
            TempT(4*(j-1)+3,:) = SSResults(j,3).wtT(i,:);
            TempT(4*(j-1)+4,:) = SSResults(j,4).wtT(i,:);
            drawnow;
         end;
         %
         % sort each column of TempT
         %
         TempT = sort(TempT);
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*TotalPaths);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*TotalPaths);
               end;
               Quantiles(l).wtT(i,:) = TempT(QuantVal,:);
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*TotalPaths/100);
               for t=1:T;
                  Bounds = HighestProbabilityDensityBand(TempT(:,t),ConfDraws);
                  Quantiles(l).wtT(i,t) = Bounds(1);
                  Quantiles(NumQuants-l+1).wtT(i,t) = Bounds(2);
               end;
            end;
         end;
         drawnow;
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
StateVarStructure.Quantiles = Quantiles;
DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
StateVarStructure.DateVector = DateVector(DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);

%
% end of DSGESimulationSmootherTheta.m
%
