function VDs = DSGEStateVarDecDistribution(DSGEModel,CurrINI,UseGroups,NumDraws,TotalDraws,CurrChain,IsPosterior,Weights)
% DSGEStateVarDecDistribution: Computes the mean value of variance decompositions for the state variables
%                              as well as quantiles or confidence bands by loading data from disk.
%
% USAGE:
%
%       VDs = DSGEStateVarDecDistribution(DSGEModel,CurrINI,UseGroups,NumDraws,TotalDraws,...
%                                    CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
%                  UseGroups (boolean) that is 1 if shock groups should be used and 0 otherwise.
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                             parameters.
%
%                  TotalDraws (integer) with the total number of draws from the posterior distribution of the
%                             parameters that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                             if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                             likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: VDs (structure) with fields "FEVD", "LRVD", "Econ", "EconLR", and "Quantiles". The "FEVD" field holds a
%                             3D matrix of dimension n x (2+q) x h, where n is the number of observed variables, q
%                             the number of shocks, and h the impulse response horizon. The values are the posterior mean
%                             forecast error variance decomposition. The "LRVD" field holds a matrix of dimension nx(1+q)
%                             with the posterior mean long-run forecast error variance decomposition. The "Econ" field holds
%                             an nxh matrix with the posterior mean of forecast error variance decomposition for all
%                             economic (structural) shocks. Similarly, "EconLR" is an nx1 vector with the posterior mean
%                             long-run forecast error variance decomposition of all economic shocks. The field "Quantiles"
%                             is a vector structure with fields "FEVD", "LRVD", "Econ", "EconLR", and "percent", where "FEVD"
%                             holds a 3D matrix of dimension n x (n+q+1) x h with values from the percent quantile of the
%                             marginal distribution of the forecast error variance decomposition, "LRVD" holds a matrix of
%                             dimension nx(n+q) with values from the percent quantile of the marginal distribution of the
%                             long-run forecast error variance decomposition, "Econ" is a matrix of dimension nxh with values
%                             from the percent quantile of the marginal distribution of the forecast error variance
%                             decomposition of all the economci shocks, "EconLR" is a matrix of dimension nx1 with values
%                             from the percent quantile of the marginal distribution of the long-run forecast error variance
%                             decomposition of all the economic shocks, while "percent" holds the percentile value in percent.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 21, 2017.
%                        This version: January 26, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 10-04-2018: Memory issue code for large models.
%
% * 30-04-2018: Took into account that the number of stored files increased when n>20.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 27-01-2020: Remember Shoah.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 26-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
VDs = [];
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
n = DSGEModel.n;
if n<=20;
   NumFiles = ceil(NumDraws/min(NumDraws,1000));
else;
   NumFiles = ceil(NumDraws/min(NumDraws,100));
end;
%
% Determine the forecast horizon
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
h = SampleFreq*DSGEModel.IRHorizon;
%
ShockNames ='all state variables';
if UseGroups==0;
   q = length(DSGEModel.StateShockPositions);
   ShockNames = strvcat(ShockNames,DSGEModel.StateShockNames);
else;
   IMatrix = SubIdentityMatrix(DSGEModel.ShockGroups);
   IMatrixSR = [1 zeros(1,size(IMatrix,2));zeros(size(IMatrix,1),1) IMatrix];
   IMatrixLR = IMatrix;
   q = max(DSGEModel.ShockGroups);
   ShockNames = char(zeros(size(DSGEModel.ShockGroupNames)));
   for i=1:q;
      ShockNames(i,:) = lower(strrep(DSGEModel.ShockGroupNames(i,:),'_',' '));
   end;
   ShockNames = strvcat('all state variables',ShockNames);
end;
r = length(DSGEModel.StateVariablePositions);
NumQuants = length(DSGEModel.Percentiles);
%
if n>20;
   YADApack;
end;
%
% initialize output
%
VDs.FEVD = zeros(r,1+q,h);
VDs.Econ = zeros(r,h);
VDs.LRVD = zeros(r,q);
VDs.EconLR = zeros(r,1);
for i=1:NumQuants;
   VDs.Quantiles(i).FEVD = zeros(r,1+q,h);
   VDs.Quantiles(i).Econ = zeros(r,h);
   VDs.Quantiles(i).LRVD = zeros(r,q);
   VDs.Quantiles(i).EconLR = zeros(r,1);
   VDs.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
% set up a wait dialog
%
if IsPosterior==1;
   PriorHeader = '';
   txt = ['Please wait while YADA computes the posterior mean forecast error variance decompositions as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,source): (' StringTrim(DSGEModel.StateVariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ').'];
else;
   PriorHeader = ' For Prior';
   txt = ['Please wait while YADA computes the prior mean forecast error variance decompositions as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,source): (' StringTrim(DSGEModel.StateVariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ').'];
end;
WaitHandle = WaitDLG(txt,'information',['Variance Decompositions' PriorHeader ' - State Variables'],500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% now we can start loading the data
%
for i=1:r;
   for j=1:q+1;
      %
      % update the wait dialog
      %
      if IsPosterior==1;
         txt = ['Please wait while YADA computes the posterior mean forecast error variance decompositions as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,source): (' StringTrim(DSGEModel.StateVariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ').'];
      else;
         txt = ['Please wait while YADA computes the prior mean forecast error variance decompositions as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,source): (' StringTrim(DSGEModel.StateVariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ').'];
      end;
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         TempVar = zeros(NumDraws,h);
         if j<1+q;
            TempLR = zeros(NumDraws,1);
         else;
            TempLR = [];
         end;
         if j==1;
            TempEcon = zeros(NumDraws,h);
            TempEconLR = zeros(NumDraws,1);
         end;
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\vardec\DSGE-StateVD-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\vardec\DSGE-StateVD-Prior-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
            end;
            VDData = load(file);
            VD = VDData.VarDec;
            clear('VDData');
            NumSaves = length(VD);
            %
            % load data from CurrSaves to NumSaves+1-CurrSaves
            %
            for s=1:NumSaves;
               %
               % check if we should study data for shock groups or not.
               %
               if UseGroups==0
                  FEVD = VD(s).FEVD;
                  LRVD = VD(s).LRVD;
               else;
                  FEVD = zeros(r,1+q,h);
                  for k=1:h;
                     FEVD(:,:,k) = VD(s).FEVD(:,:,k)*IMatrixSR;
                  end;
                  LRVD = VD(s).LRVD*IMatrixLR;
               end;
               TempVar(CurrSaves+s,:) = FEVD(i,j,:);
               if j<1+q;
                  TempLR(CurrSaves+s,1) = LRVD(i,j);
               end;
               if j==1;
                  for j1=1:h;
                     TempEcon(CurrSaves+s,j1) = sum(FEVD(i,2:1+q,j1),2);
                  end;
                  TempEconLR(CurrSaves+s,1) = sum(LRVD(i,:),2);
               end;
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % now we sort and compute the mean the long-run decompositions
         %
         if j<1+q;
            if max(abs(TempLR))>1.0e-10;
               VDs.LRVD(i,j) = mean(TempLR.*Weights);
            end;
            TempLR = sort(TempLR);
            if j==1;
               if max(abs(TempEconLR))>1.0e-10;
                  VDs.EconLR(i,1) = mean(TempEconLR.*Weights);
               end;
               TempEconLR = sort(TempEconLR);
            end;
            %
            % deal with quantiles
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
                  end;
                  if abs(TempLR(QuantVal,1))>1.0e-10;
                     VDs.Quantiles(l).LRVD(i,j) = TempLR(QuantVal,1);
                  end;
                  if j==1;
                     if abs(TempEconLR(QuantVal,1))>1.0e-10;
                        VDs.Quantiles(l).EconLR(i) = TempEconLR(QuantVal,1);
                     end;
                  end;
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempLR,ConfDraws);
                  if abs(Bounds(1))>1.0e-10;
                     VDs.Quantiles(l).LRVD(i,j) = Bounds(1);
                  end;
                  if abs(Bounds(2))>1.0e-10;
                     VDs.Quantiles(NumQuants-l+1).LRVD(i,j) = Bounds(2);
                  end;
                  if j==1;
                     Bounds = HighestProbabilityDensityBand(TempEconLR,ConfDraws);
                     if abs(Bounds(1))>1.0e-10;
                        VDs.Quantiles(l).EconLR(i) = Bounds(1);
                     end;
                     if abs(Bounds(2))>1.0e-10;
                        VDs.Quantiles(NumQuants-l+1).EconLR(i) = Bounds(2);
                     end;
                  end;
               end;
            end;
         end;
         %
         % now we deal with the individual forecast horizons
         %
         for k=1:h;
            if max(abs(TempVar(:,k)))>1.0e-10;
               VDs.FEVD(i,j,k) = mean(TempVar(:,k).*Weights);
            end;
            TempVar(:,k) = sort(TempVar(:,k));
            if j==1;
               if max(abs(TempEcon(:,k)))>1.0e-10;
                  VDs.Econ(i,k) = mean(TempEcon(:,k).*Weights);
               end;
               TempEcon(:,k) = sort(TempEcon(:,k));
            end;
            %
            % now we put the quantile information in its correct place
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
                  end;
                  if abs(TempVar(QuantVal,k))>1.0e-10;
                     VDs.Quantiles(l).FEVD(i,j,k) = TempVar(QuantVal,k);
                  end;
                  if j==1;
                     if abs(TempEcon(QuantVal,k))>1.0e-10;
                        VDs.Quantiles(l).Econ(i,k) = TempEcon(QuantVal,k);
                     end;
                  end;
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempVar(:,k),ConfDraws);
                  if abs(Bounds(1))>1.0e-10;
                     VDs.Quantiles(l).FEVD(i,j,k) = Bounds(1);
                  end;
                  if abs(Bounds(2))>1.0e-10;
                     VDs.Quantiles(NumQuants-l+1).FEVD(i,j,k) = Bounds(2);
                  end;
                  if j==1;
                     Bounds = HighestProbabilityDensityBand(TempEcon(:,k),ConfDraws);
                     if abs(Bounds(1))>1.0e-10;
                        VDs.Quantiles(l).Econ(i,k) = Bounds(1);
                     end;
                     if abs(Bounds(2))>1.0e-10;
                        VDs.Quantiles(NumQuants-l+1).Econ(i,k) = Bounds(2);
                     end;
                  end;
               end;
            end;
            drawnow;
         end;
      catch;
         for k1=1:h;
            TempVar = zeros(NumDraws,1);
            if j<1+q;
               TempLR = zeros(NumDraws,1);
            else;
               TempLR = [];
            end;
            if j==1;
               TempEcon = zeros(NumDraws,1);
               TempEconLR = zeros(NumDraws,1);
            end;
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\vardec\DSGE-StateVD-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\vardec\DSGE-StateVD-Prior-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
               end;
               VDData = load(file);
               VD = VDData.VarDec;
               clear('VDData');
               NumSaves = length(VD);
               %
               % load data from CurrSaves to NumSaves+1-CurrSaves
               %
               for s=1:NumSaves;
                  %
                  % check if we should study data for shock groups or not.
                  %
                  if UseGroups==0
                     FEVD = VD(s).FEVD;
                     LRVD = VD(s).LRVD;
                  else;
                     FEVD = zeros(r,1+q,h);
                     for k=1:h;
                        FEVD(:,:,k) = VD(s).FEVD(:,:,k)*IMatrixSR;
                     end;
                     LRVD = VD(s).LRVD*IMatrixLR;
                  end;
                  TempVar(CurrSaves+s,1) = FEVD(i,j,k1);
                  if j<1+q;
                     TempLR(CurrSaves+s,1) = LRVD(i,j);
                  end;
                  if j==1;
                     TempEcon(CurrSaves+s,k1) = sum(FEVD(i,2:1+q,k1),2);
                     TempEconLR(CurrSaves+s,1) = sum(LRVD(i,:),2);
                  end;
               end;
               CurrSaves = CurrSaves+NumSaves;
               drawnow;
            end;
            %
            % now we sort and compute the mean the long-run decompositions
            %
            if j<1+q;
               if max(abs(TempLR))>1.0e-10;
                  VDs.LRVD(i,j) = mean(TempLR.*Weights);
               end;
               TempLR = sort(TempLR);
               if j==1;
                  if max(abs(TempEconLR))>1.0e-10;
                     VDs.EconLR(i,1) = mean(TempEconLR.*Weights);
                  end;
                  TempEconLR = sort(TempEconLR);
               end;
               %
               % deal with quantiles
               %
               if DSGEModel.ConfidenceRegionMethod==1;
                  for l=1:NumQuants;
                     if DSGEModel.Percentiles(l)<50;
                        QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
                     else;
                        QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
                     end;
                     if abs(TempLR(QuantVal,1))>1.0e-10;
                        VDs.Quantiles(l).LRVD(i,j) = TempLR(QuantVal,1);
                     end;
                     if j==1;
                        if abs(TempEconLR(QuantVal,1))>1.0e-10;
                           VDs.Quantiles(l).EconLR(i) = TempEconLR(QuantVal,1);
                        end;
                     end;
                  end;
               else;
                  for l=1:floor(NumQuants/2);
                     ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
                     Bounds = HighestProbabilityDensityBand(TempLR,ConfDraws);
                     if abs(Bounds(1))>1.0e-10;
                        VDs.Quantiles(l).LRVD(i,j) = Bounds(1);
                     end;
                     if abs(Bounds(2))>1.0e-10;
                        VDs.Quantiles(NumQuants-l+1).LRVD(i,j) = Bounds(2);
                     end;
                     if j==1;
                        Bounds = HighestProbabilityDensityBand(TempEconLR,ConfDraws);
                        if abs(Bounds(1))>1.0e-10;
                           VDs.Quantiles(l).EconLR(i) = Bounds(1);
                        end;
                        if abs(Bounds(2))>1.0e-10;
                           VDs.Quantiles(NumQuants-l+1).EconLR(i) = Bounds(2);
                        end;
                     end;
                  end;
               end;
            end;
            %
            %
            if max(abs(TempVar))>1.0e-10;
               VDs.FEVD(i,j,k1) = mean(TempVar.*Weights);
            end;
            TempVar = sort(TempVar);
            if j==1;
               if max(abs(TempEcon))>1.0e-10;
                  VDs.Econ(i,k1) = mean(TempEcon.*Weights);
               end;
               TempEcon = sort(TempEcon);
            end;
            %
            % now we put the quantile information in its correct place
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
                  end;
                  if abs(TempVar(QuantVal,1))>1.0e-10;
                     VDs.Quantiles(l).FEVD(i,j,k1) = TempVar(QuantVal,1);
                  end;
                  if j==1;
                     if abs(TempEcon(QuantVal,1))>1.0e-10;
                        VDs.Quantiles(l).Econ(i,k1) = TempEcon(QuantVal,1);
                     end;
                  end;
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
                  if abs(Bounds(1))>1.0e-10;
                     VDs.Quantiles(l).FEVD(i,j,k1) = Bounds(1);
                  end;
                  if abs(Bounds(2))>1.0e-10;
                     VDs.Quantiles(NumQuants-l+1).FEVD(i,j,k1) = Bounds(2);
                  end;
                  if j==1;
                     Bounds = HighestProbabilityDensityBand(TempEcon,ConfDraws);
                     if abs(Bounds(1))>1.0e-10;
                        VDs.Quantiles(l).Econ(i,k1) = Bounds(1);
                     end;
                     if abs(Bounds(2))>1.0e-10;
                        VDs.Quantiles(NumQuants-l+1).Econ(i,k1) = Bounds(2);
                     end;
                  end;
               end;
            end;
            drawnow;
         end;
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEStateVarDecDistribution.m
%
