function [FEVDs,LRVD,status,RiccatiResults,UniquenessCheck] = DSGEStateVarianceDecompFcn(H,F,B0,R,h,DAMaxIter,DAConvValue,RicMaxIter,RicConvValue)
% DSGEStateVarianceDecompFcn: Calculates the forecast error variance decomposition for state variables given
%                             that the matlab function dare exists.
%
% USAGE:
%
%       [FEVDs,LRVD,status,RiccatiResults,UniquenessCheck] = DSGEStateVarianceDecompFcn(H,F,B0,R,h,DAMaxIter,...
%                                                                         DAConvValue,RicMaxIter,RicConvValue)
%
% REQUIRED INPUT:  H (rxn) matrix where n is the number of observed variables and r the number of
%                          state variables. This matrix is part of the measurement equation of the
%                          state-space representation for the Kalman filter.
%
%                  F (rxr) matrix from the state equation.
%
%                  B0 (rxq) matrix, where B0*B0' is the covariance matrix for the shocks to the
%                          state equation.
%
%                  R (nxn) matrix with the covariance for the measurement errors. May also be the scalar
%                          0.
%
%                  h (positive integer) giving the forecast horizon.
%
%                  DAMaxIter (integer) which determines the maximum number of iterations that
%                          the doubling algorithm for the Lyapunov aquations can use.
%
%                  DAConvValue (integer) which determined the tolerance level for the convergence of the
%                          doubling algorithms for the Lyapunov equations.
%
%                  RicMaxIter (integer) which determined the maximum number of iterations that the
%                          Riccati equation solver can use.
%
%                  RicConvValue (integer) which determines the tolerance level for the convergence of
%                          the Riccati equations solver.
%
% REQUIRED OUTPUT: FEVDs (rx(1+q)xh) matrix with the forecast error variance decompositions. For each h, the rx(q+1) matrix
%                          is ordered such tthat the initial state is ordered first and the q economic shocks in the
%                          following columns. The state variables are are determined through the row numbers.
%
%                  LRVD (rxq) matrix with the long run forecast error variance decomposition.
%
%                  status (integer) takes the value 0 if the solution method converged and 1 otherwise. It takes
%                          on the value 2 in the special case that infinite or NaN's are found in P1 during the
%                          iterations.
%
%                  RiccatiResults (1x2 vector), with the number of iterations and the TestValue for the Riccati
%                          solver in the columns. The TestValue refers to the degree of convergence.
%
%                  UniquenessCheck (scalar), with largest eigenvalues of (F-K*H'), where K is the asymptote of the
%                          Kalman gain matrix, i.e., K = F*P1*H*inv(H'*P1*H+R) for the full forecast error
%                          variance.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 19, 2017.
%                        This version: January 26, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-12-2017: Checked the function for small errors.
%
% * 16-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 27-01-2020: Remember Shoah.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 26-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
FEVDs = [];
LRVD = [];
status = 0;
RiccatiResults = [1 0];
UniquenessCheck = 1;
%
[r,n] = size(H);
q = size(B0,2);
Q = B0*B0';
P0 = Q - (Q*(H/((H'*Q*H)+R))*H'*Q);
%
if max(abs(eig(F*P0*F')))>1.0e-10;
   [P_init,stat] = DoublingAlgorithmLyapunov(F,Q,DAMaxIter*100,10^(-(DAConvValue+1)));
   [P1,status,NumIter,TestValue] = RiccatiSolver(F,H,Q,R,P_init,RicMaxIter*1000,10^(-(RicConvValue+1)));
   RiccatiResults = [NumIter TestValue];
else;
   P1 = Q;
end;
if status~=0;
   return;
end;
%
% continue with the calculations
%
K = F*P1*(H/((H'*P1*H)+R));
UniquenessCheck = max(abs(eig(F-K*H')));
%
% initialize F*P0*F'
%
FP0 = P1-(B0*B0');
%
% initialize the matrix with forecast error variance decompositions.
%
FEVDs = zeros(r,1+q,h);
LRVD = zeros(r,q);
%
DenomOld = zeros(r,r);
NumerOld = zeros(r,1+q);
Fi = eye(r);
for i=1:h;
   Ri = Fi*B0;
   DenomOld = DenomOld+((FP0+(Ri*Ri')) .* eye(r));
   NumerOld = NumerOld+[diag(FP0) (Ri .* Ri)];
   FEVDs(:,:,i) = DenomOld\NumerOld;
   Fi = Fi*F;
   FP0 = F*FP0*F';
end;
%
% and now the long-run
%
[LRV,stat] = DoublingAlgorithmLyapunov(F,Q,DAMaxIter*100,10^(-(DAConvValue+1)));
LRV = diag(LRV);
for j=1:q;
   Qj = B0(:,j)*B0(:,j)';
   [LRVj,stat] = DoublingAlgorithmLyapunov(F,Qj,DAMaxIter*100,10^(-(DAConvValue+1)));
   LRVD(:,j) = diag(LRVj)./LRV;
end;

%
% end of DSGEStateVarianceDecompFcn.m
%
