function [Hlag,H,Hexp,D] = DSGEStructuralForm(ModelParameters,nlead,StateVariablePositions,StateEquationPositions,StateShockPositions)
% DSGEStructuralForm: Sets up the structural form of the DSGE model for fixed parameter values.
%
% USAGE:
%
%       [Hlag,H,Hexp,D] = DSGEStructuralForm(ModelParameters,nlead,StateVariablePositions,StateEquationPositions,StateShockPositions)
%
% REQUIRED INPUT:  ModelParameters (structure) with fields having the names of the inidividual parameters as they are
%                                  specified under the "model parameter" header in the file with prior distribution data,
%                                  and having values as specified under the header "initial value" in the same file.
%
%                  nlead (integer) with the number of forward looking time periods.
%
%                  StateVariablePositions (vector) with the positions of the state variables among all variables in
%                                  the AiM specification.
%
%                  StateEquationPositions (vector) with the positions of the equations that determine the state variables.
%
%                  StateShockPositions (vector) with the positions of the structural shocks among all variables in
%                                  the AiM specification.
%
% REQUIRED OUTPUT: Hlag (matrix) of dimension r x r with the coefficient values on the lagged state variables.
%
%                  H0 (matrix) of dimension r x r with the coefficient values on the contemporaneosu state variables.
%
%                  Hexp (matrix) of dimension r x r*nlead with the coefficient values on the expectated future values
%                                  of the state variables.
%
%                  D (matrix) of dimension r x q with the coefficient values on the structural shocks.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 12, 2020.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-03-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% get the structural form matrices
%
[GMatrix,HMatrix] = compute_aim_matrices(ModelParameters);
[rh,ch] = size(HMatrix);
[rg,cg] = size(GMatrix);
H = zeros(rh,ch);
H(1:rg,1:cg) = GMatrix;
H = H+HMatrix;
Hlag = H(:,1:rh);
Hexp = H(:,(2*rh)+1:(2+nlead)*rh);
H = H(:,rh+1:2*rh);
%
% now we select the variables
%
StateVarPos = StateVariablePositions;
if nlead>1;
   for j=2:nlead;
      StateVarPos = [StateVarPos StateVariablePositions+(rh*(j-1))];
   end;
end;
Hlag = Hlag(StateEquationPositions,StateVariablePositions);
Hexp = Hexp(StateEquationPositions,StateVariablePositions);
D = -H(StateEquationPositions,StateShockPositions);
H = H(StateEquationPositions,StateVariablePositions);

%
% end of DSGEStructuralForm.m
%
