function [FEVDs,LRVD,status,RiccatiResults,UniquenessCheck] = DSGEVarianceDecompFcn(H,F,B0,R,h,DAMaxIter,DAConvValue,RicMaxIter,RicConvValue)
% DSGEVarianceDecompFcn: Calculates the forecast error variance decomposition given that the matlab function dare
%                        exists.
%
% USAGE:
%
%       [FEVDs,LRVD,status,RiccatiResults,UniquenessCheck] = DSGEVarianceDecompFcn(H,F,B0,R,h,DAMaxIter,DAConvValue, ...
%                                                                                  RicMaxIter,RicConvValue)
%
% REQUIRED INPUT:  H (rxn) matrix where n is the number of observed variables and r the number of
%                          state variables. This matrix is part of the measurement equation of the
%                          state-space representation for the Kalman filter.
%
%                  F (rxr) matrix from the state equation.
%
%                  B0 (rxq) matrix, where B0*B0' is the covariance matrix for the shocks to the
%                          state equation.
%
%                  R (nxn) matrix with the covariance for the measurement errors. May also be the scalar
%                          0.
%
%                  h (positive integer) giving the forecast horizon.
%
%                  DAMaxIter (integer) which determines the maximum number of iterations that
%                          the doubling algorithm for the Lyapunov aquations can use.
%
%                  DAConvValue (integer) which determined the tolerance level for the convergence of the
%                          doubling algorithms for the Lyapunov equations.
%
%                  RicMaxIter (integer) which determined the maximum number of iterations that the
%                          Riccati equation solver can use.
%
%                  RicConvValue (integer) which determines the tolerance level for the convergence of
%                          the Riccati equations solver.
%
% REQUIRED OUTPUT: FEVDs (nx(2+q)xh) matrix with the forecast error variance decompositions. For each h, the nx(n+q+1) matrix
%                          is ordered such the the measurement errors are located in the first n columns, and the q economic
%                          shocks in the following columns. The remaining column captures possible signal extraction error
%                          variances. The observed variables are determined through the row numbers.
%
%                  LRVD (matrix) of dimension nx(q+1) with the long-run forecast error variance decomposition.
%
%                  status (integer) takes the value 0 if the solution method converged and 1 otherwise. It takes
%                          on the value 2 in the special case that infinite or NaN's are found in P1 during the
%                          iterations.
%
%                  RiccatiResults (1x2 vector), with the number of iterations and the TestValue for the Riccati
%                          solver in the columns. The TestValue refers to the degree of convergence.
%
%                  UniquenessCheck (scalar), with largest eigenvalues of (F-K*H'), where K is the asymptote of the
%                          Kalman gain matrix, i.e., K = F*P1*H*inv(H'*P1*H+R) for the full forecast error
%                          variance.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 23, 2006.
%                        This version: January 26, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-10-2006: Changes the convergence value for the Riccati equation solver from 1.0e-8 to 1.0e-5.
%
% * 31-10-2006: Made sure that the value of DSGEModel.DAToleranceValue is sent in the appropriate form to the
%               "DoublingAlgorithmLyapunov" function. Furthermore, the function "RiccatiSolver" now takes the
%               model selected number of iterations and tolerance level into account. Long run variance
%               decompositions were also added and provided as output. The input CurrINI has been removed in
%               the call to the function. Furthermore, the DSGEModel structure is no longer given as input.
%               Instead, the function takes the 4 field values of this structure that it previously required
%               as the 4 variables "DAMaxIter", "DAConvValue", "RicMaxIter", and
%               "RicConvValue".
%
% * 20-11-2006: The status from the Riccati solver function is given as output.
%
% * 21-11-2006: The function also checks for the uniqueness of P1. "UniquenessCheck" variable therefore added
%               to the output list.
%
% * 22-11-2006: Added one column to the FEVDs output matrix and one row to RiccatiResults.
%
% * 24-11-2006: Made sure the last run of the Riccati solver is only considered if all previous runs were
%               successful. Moreover, if this run is unsuccessful, then FEVDs and LRVD are set to empty
%               matrices.
%
% * 13-11-2007: Updated the documentation.
%
% * 21-04-2008: Updated the documentation.
%
% * 22-04-2008: Using B*B' = P1 when the measurement errors are set to zero. UniquenessCheck is now a scalar, while
%               RiccatiResults is a vector.
%
% * 23-04-2008: If P0 with P1 set to Q is sufficiently close to zero, then P1 = Q is used.
%
% * 23-05-2008: Updated the documentation.
%
% * 23-01-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 06-01-2010: The function now uses "pinv" when computing P0 for the individual shocks. The R-matrix
%               is now also sent to the Riccati solver for the individual shocks.
%
% * 11-02-2010: Removed a disp-command.
%
% * 05-01-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 04-01-2013: Updated the documentation.
%
% * 30-08-2013: Changed the calculations of the FEVDs such that the state variable uncertainty is not decomposed
%               into shocks, i.e., the calculation has been changed from a nonlinear decomposition to a linear.
%
% * 13-09-2013: The Riccati solver is now only used when F*P0*F' is close to 0.
%
% * 16-09-2013: Added the "LRVD" output variable.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 19-12-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 27-01-2020: Remember Shoah.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 26-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

FEVDs = [];
LRVD = [];
status = 0;
RiccatiResults = [1 0];
UniquenessCheck = 1;
%
[r,n] = size(H);
q = size(B0,2);
Q = B0*B0';
P0 = Q - (Q*(H/((H'*Q*H)+R))*H'*Q);
%
if max(abs(eig(F*P0*F')))>1.0e-10;
   [P_init,stat] = DoublingAlgorithmLyapunov(F,Q,DAMaxIter*100,10^(-(DAConvValue+1)));
   [P1,status,NumIter,TestValue] = RiccatiSolver(F,H,Q,R,P_init,RicMaxIter*1000,10^(-(RicConvValue+1)));
   RiccatiResults = [NumIter TestValue];
else;
   P1 = Q;
end;
%
if status==0;
   K = F*P1*(H/((H'*P1*H)+R));
   UniquenessCheck = max(abs(eig(F-K*H')));
   %
   % initialize F*P0*F'
   %
   FP0 = P1-(B0*B0');
   %
   % initialize the matrix with forecast error variance decompositions.
   %
   FEVDs = zeros(n,2+q,h);
   LRVD = zeros(n,q+1);
   %
   DenomOld = zeros(n,n);
   NumerOld = zeros(n,2+q);
   Fi = eye(r);
   for i=1:h;
      Ri = H'*Fi*B0;
      HFP0 = H'*FP0*H;
      DenomOld = DenomOld+((HFP0+(Ri*Ri')+R) .* eye(n));
      NumerOld = NumerOld+[diag(HFP0) diag(R) (Ri .* Ri)];
      FEVDs(:,:,i) = DenomOld\NumerOld;
      Fi = Fi*F;
      FP0 = F*FP0*F';
   end;
   %
   % and now the long-run
   %
   [PLR,stat] = DoublingAlgorithmLyapunov(F,Q,DAMaxIter*100,10^(-(DAConvValue+1)));
   LRV = diag((H'*PLR*H)+R);
   LRVD(:,1) = diag(R)./LRV;
   for j=1:q;
      Qj = B0(:,j)*B0(:,j)';
      [PLRj,stat] = DoublingAlgorithmLyapunov(F,Qj,DAMaxIter*100,10^(-(DAConvValue+1)));
      LRVD(:,1+j) = diag(H'*PLRj*H)./LRV;
   end;
end;

%
% end of DSGEVarianceDecompFcn.m
%
