function KSResults = KolmogorovSmirnovTwoSample(Draws,AltDraws,KSTestExist)
% KolmogorovSmirnovTwoSample: Computes two-sample based Kolmogorov-Smirnov test and p-value.
%
% USAGE:
%
%       KSResults = KolmogorovSmirnovTwoSample(Draw,AltDraw,KSTestExist)
%
% REQUIRED INPUT:  Draws (vector) of dimension N with draws from an unknown distribution.
%
%                  AltDraws (vector) of dimension N with other draws from an unknown distribution.
%
% OPTIONAL INPUT:  KSTestExist (integer) is equal to 2 if the matlab function "kstest2" exists
%                              and 0 otherwise.
%
% REQUIRED OUTPUT: KSResults (vector) of dimension 1x2 with Kolmogorov-Smirnov test value in
%                              its first element and the p-value of that Draws and AltDraws
%                              are from the same distribution in its second element.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: September 16, 2014.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-09-2014: Updated the documentation.
%
% * 23-09-2014: Made sure that the values that are equal in Draws and AltDraws are collected in D and taken from Draws,
%               and those that are different are collected in A and taken from Draws. These latter draws are assumed
%               to have been discarded by the analysis that makes use of Draws and AltDraws.
%
% * 23-10-2014: Made sure that the test is only conducted when Draws and AltDraws are different.
%
% * 14-01-2015: Updated the documentation.
%
% * 09-06-2015: Attempted to use histcounts instead of histc when it exists.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize output
%
if nargin<3;
   KSTestExist = exist('kstest2');
end;
KSResults = zeros(1,2);
D = Draws(Draws==AltDraws);
A = Draws(Draws~=AltDraws);
N = length(D);
M = length(A);
NNBar = sqrt(N*M/(N+M));
%
if (N>0)&(M>0);
   if KSTestExist==2;
      [H,pVal,KStest] = kstest2(D,A);
      KSResults(1) = NNBar*KStest;
      KSResults(2) = KolmogorovDist(KSResults(1),1000);
   else;
      Bins = [-Inf sort([vec(D)' vec(A)']) Inf];
      if exist('histcounts')==0;
         PDFD = histc(vec(D)',Bins,2);
         PDFA = histc(vec(A)',Bins,2);
      else;
         PDFD = histcounts(vec(D)',Bins);
         PDFA = histcounts(vec(A)',Bins);
      end;
      CDFD = cumsum(PDFD)/N;
      CDFA = cumsum(PDFA)/M;
      KSResults(1) = NNBar*max(abs(CDFD-CDFA));
      KSResults(2) = KolmogorovDist(KSResults(1),1000);
   end;
else;
   KSResults = [NaN NaN];
end;

%
% end of KolmogorovSmirnovTwoSample.m
%
