function [Status,IsOriginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData,UsePaths,Share] = SequentialCRPS(DSGEModel,CurrINI,controls,IsOriginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData,RecomputeCRPS,UsePaths,Share)
% SequentialCRPS: Can be used to estimate the CRPS and the ES sequentially for DSGE models.
%
% USAGE:
%
%       [Status,IsOriginal,CurrChain,IsSubSample,PredictionVariables,...
%                         SelectedYData,UsePaths,Share] = SequentialCRPS(DSGEModel,CurrINI,controls,...
%                                                                  IsOriginal,CurrChain,IsSubSample,...
%                                                                  PredictionVariables,SelectedYData,...
%                                                                  RecomputeCRPS,UsePaths,Share)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information 
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                  IsOriginal (boolean) that takes the value 1 if the predictive likelihood
%                            should be computed for the original data and 0 if for annualized
%                            data. When empty, the function checks which alternative should
%                            be considered.
%
%                  CurrChain (integer) with the value of the MCMC chain to use.
%
%                  IsSubSample (boolean) that takes the value 1 if a subsample of the posterior
%                            draws should be used and 0 if all post-burnin draws should be used.
%                            When empty, the function checks which alternative should be considered.
%
%                  PredictionVariables (vector) with the positions of the observed variables
%                            to compute the predictive likelihood for. When empty, the function
%                            checks which alternative should be considered.
%
%                  SelectedYData (integer) with the selected set of actuals for the observed
%                            variables.
%
%                  RecomputeCRPS (boolean) which takes the value 1 if the CRPS and the ES should be
%                            recomputed and 0 if old calculations should be used. When empty, the
%                            function checks which alternative should be considered.
%
%                  UsePaths (boolean) which is unity if calculations should be performed and zero
%                            otherwise.
%
%                  Share (scalar) with the share of the posterior draws to use. Only relevant for
%                            the SMC with likelihood tempering and MitISEM posterior samplers.
%
% REQUIRED OUTPUT: Status (boolean) that takes the value 1 if the computations could be successfully
%                            carries out, 0 if not, and -1 if they were aborted by the user.
%
%                  IsOriginal (boolean) that takes the value 1 if the predictive likelihood
%                            should be computed for the original data and 0 if for annualized
%                            data.
%
%                  CurrChain (integer) with the value of the MCMC chain to use.
%
%                  IsSubSample (boolean) that takes the value 1 if a subsample of the posterior
%                            draws should be used and 0 if all post-burnin draws should be used.
%
%                  PredictionVariables (vector) with the positions of the observed variables
%                            to compute the predictive likelihood for.
%
%                  SelectedYData (integer) with the selected set of actuals for the observed
%                            variables.
%
%                  UsePaths (boolean) which is unity if calculations should be performed and zero
%                            otherwise.
%
%                  Share (scalar) with the share of the posterior draws to use. Only relevant for
%                            the SMC with likelihood tempering and MitISEM posterior samplers.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 7, 2022.
%                        This version: January 29, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-02-2022: Updated the documentation.
%
% * 11-02-2022: Added check if sample is too large. Took into account that PrintCRPS requires
%               the input variable NumPredPaths.
%
% * 14-02-2022: Fixed a bug where break was used instead of return.
%
% * 15-02-2022: Made sure that the number of exogenous variables matches the prediction horizon
%               rather than the forecast horizon (forecast horizon <= prediction horizon).
%
% * 17-05-2022: Added sub-sample checking code for the SMC and MitISEM samplers when combining paths with posterior
%               draws.
%
% * 24-05-2022: The variable "DSGEModel.NameOfModel" now affects the names of saved output files.
%
% * 06-01-2023: Updated the documentation.
%
% * 02-01-2024: Took into account that "PrintCRPS" now takes the IsConditional input variable.
%
% * 29-01-2024: Took into account that "PrintCRPS" now takes the IsZLB input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Status = 0;
OriginalAnnual = [(strcmp(get(controls.predict.unconditional.original.crps.posterior,'Enable'),'on')==1) (strcmp(get(controls.predict.unconditional.annual.crps.posterior,'Enable'),'on')==1)];
if isempty(IsOriginal)==1;
   if OriginalAnnual(2)==1;
      txt = 'Would you like to estimate the continuous ranked probability score (CRPS) and the energy score (ES) for the original data or the annualized data of the DSGE models?';
      answer = Query(txt,'question',200,'Predictive Distributions (CRPS) - Posterior Distribution',550,'yes',CurrINI,{'Original' 'Annualized'});
      if strcmp(lower(answer),'original')==1;
         IsOriginal = 1;
      else;
         IsOriginal = 0;
      end;
   else;
      IsOriginal = 1;
   end;
end;
if IsOriginal==1;
   DataStr = 'Original';
else;
   DataStr = 'Annualized';
end;
AnnualizeData = (IsOriginal==0);
%
% first attempt to load some posterior mode data
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
ErrorStr = '';
HeaderStr = 'YADA - Bad MAT File';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
%
% check if file loading was successful
%
if isempty(ErrorStr)==0;
   SetLogText('Unable to load posterior mode data. Aborting calculation.');
   return;
end;
%
% if there are multiple chains, check which one to use if necessary
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
if (isempty(CurrChain)==1)&(NumChains>1);
   %
   % ask which chain to load
   %
   ChainsStr = '';
   for i=1:NumChains;
      ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
   end;
   [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Predictive Distributions (CRPS) - Posterior Distribution','','',CurrINI);
elseif (isempty(CurrChain)==0)&(CurrChain>NumChains);
   action = 'cancel';
   SetLogText(['The selected chain number (' int2str(CurrChain) ') is greater than the available number of MCMC chains.']);
else;
   CurrChain = 1;
   action = 'ok';
end;
if strcmp(lower(action),'cancel')==1;
   SetLogText('Aborting calculation.');
   Status = -1;
   return;
end;
%
% check if the data actually exists
%
if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==0;
   SetLogText('Unable to locate the posterior parameter draws. Aborting calculation.');
   return;
end;
%
% check if a sub-sample should be used
%
if isempty(IsSubSample)==1;
   txt = 'Would you like to use a sub-sample of the posterior draws when estimating the continuous ranked probability score (CRPS) and the energy score (ES)?';
   answer = Query(txt,'question',400,'Predictive Distributions (CRPS) - Posterior Distribution',500,'no',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      IsSubSample = 1;
   else;
      IsSubSample = 0;
   end;
end;
%
% check the PredictionVariables
%
if isempty(PredictionVariables)==1;
   PredictionVariables = ones(1,DSGEModel.n);
   PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
   [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
   if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
      SetLogText('No prediction variables selected. Aborting calculation.');
      Status = -1;
      return;
   elseif strcmp(lower(Action),'ok')==0;
      SetLogText('Aborting calculation.');
      Status = -1;
      return;
   end;
end;
%
% check if the selected variables to predict can indeed be used.
%
if max(PredictionVariables)>DSGEModel.n;
   SetLogText('Too many prediction variables. Aborting calculation.');
   return;
end;
DSGEModel.PredictionVariables = PredictionVariables;
%
% load the data
%
[PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
clear('LogPost','AcceptedDraws','PostSample');
[NumPostDraws,NumParam] = size(thetaPostSample);
thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
NumDraws = NumPostDraws-NumBurnin;
if IsSubSample==1;
   if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
      UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
      thetaPostSample = thetaPostSample(UseDraws,:);
      NumDraws = length(UseDraws);
   else;
      %
      % check which draws we should use
      %
      UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'CRPS - Posterior Distribution',Weights,Share);
      if isempty(UseDraws)==1;
         Status = -1;
         return;
      end;
      thetaPostSample = thetaPostSample(UseDraws,:);
      Weights = Weights(UseDraws,1);
      Weights = Weights*length(Weights)/sum(Weights);
      Share = length(UseDraws)/NumDraws;
      NumDraws = length(UseDraws);
   end;
end;
%
% determine how many prediction paths to use
%
NumPredPathsStr = get(controls.forecast.predpaths,'String');
NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
%
% check the number of prediction paths
%
if isempty(UsePaths)==1;
   UsePaths = 1;
   if NumPredPaths*NumDraws>100000;
      txt = ['The total number of paths that the CRPS calculations will use is equal ' int2str(NumPredPaths*NumDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
      answer = Query(txt,'question',400,'CRPS - Posterior Distribution',500,'no',CurrINI);
      if strcmp(lower(answer),'no')==1;
         UsePaths = 0;
         SetLogText('Too many prediction paths. Aborting calculations.');
         Status = -1;
         return;
      else;
         SetLogText(['Total number of prediction paths: ' int2str(NumPredPaths*NumDraws)]);
      end;
   else;
      SetLogText(['Total number of prediction paths: ' int2str(NumPredPaths*NumDraws)]);
   end;
else;
   SetLogText(['Total number of prediction paths: ' int2str(NumPredPaths*NumDraws)]);
end;
%
% collect some required data
%
theta = ModeData.thetaMode;
thetaPositions = ModeData.thetaPositions;
thetaIndex = ModeData.thetaIndex;
thetaDist = ModeData.thetaDist;
PriorDist = ModeData.PriorDist;
LowerBound = ModeData.LowerBound;
UniformBounds = ModeData.UniformBounds;
ModelParameters = ModeData.ModelParameters;
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
%
% check how many time periods to use for predictive distributions
%
MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
% check the maximum when we take NaNs into account
%
YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
%
% check which actuals should be used
%
if isempty(SelectedYData)==1;
   if isempty(DSGEModel.Actuals)==1;
      SelectedYData = 1;
      DataAction = 'ok';
      YDataStr = 'Actuals from original sample';
   else;
      YDataStr = 'Actuals from original sample';
      m = length(DSGEModel.Actuals);
      for i=1:m;
         YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
      end;
      [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','CRPS - Posterior Distribution','','',CurrINI);
      if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
         YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
         BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
      end;
   end;
else;
   DataAction = 'ok';
   YDataStr = 'Actuals from original sample';
   if isempty(DSGEModel.Actuals)==0;
      m = length(DSGEModel.Actuals);
      for i=1:m;
         YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
      end;
   end;
   if (SelectedYData>1)&(length(DSGEModel.Actuals)>=SelectedYData-1);
      YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
      BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
   end;
end;
%
% test if we should continue
%
if strcmp(lower(DataAction),'ok')==0;
   SetLogText('Aborting calculation.');
   Status = -1;
   return;
end;
%
MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
[MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
MaxFH = min(MaxFH,MaxPH);
YData = YData(:,1:MaxFH);
%
% matrix with exogenous variables
%
X = [];
if DSGEModel.k>0;
   if NumPredX<MaxPH;
      X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
   end;
   if NumPredX>0;
      X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
   end;
end;
%
% Determine the prediction sample
%
SampleStrMat = CreateFullSample(DSGEModel);
PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
%
% check if older calculations exists given that such may be used
%
DSGEModel.RunPredictionEvent = 0;
tic;
if (RecomputeCRPS==0)&(PredictionsExist(DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,MaxPH,NumDraws,NumPredPaths,AnnualizeData,0,CurrChain,1)==1);
   DoneCalc = 1;
else;
   [DoneCalc,PredEventData,StateCov,ShockCov,MeasureCov,ParameterCov] = DSGEPredictionPaths(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,0,CurrChain,1,Weights);
end;
clear('thetaPostSample');
if DoneCalc==0;
   SetLogText('Aborting calculation.');
   toc;
   return;
end;
%
% we can now compute the continuous ranked probability scores
%
CRPSMatrix = ComputeCRPSDSGEPaths(YData,PredictionVariables,NumDraws,NumPredPaths,MaxPH,AnnualizeData,CurrChain,1,Weights,'','',DSGEModel,CurrINI);
CompTime = toc;
%
% write the results to a text file
%
ResFile = FixFilePath([DSGEModel.OutputDirectory '\CRPS-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
PrintCRPS(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),CRPSMatrix,AnnualizeData,1,0,0,0,NumDraws,NumBurnin,NumPostDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
%
% save output also to a mat-file
%
[PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
MatFile = FixFilePath([DSGEModel.OutputDirectory '\CRPS-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
YNames = DSGEModel.VariableNames;
save(MatFile,'YNames','PredictionVariables','CRPSMatrix','PredDateVector');
%
% create the m-file as well
%
mfile = FixFilePath([DSGEModel.OutputDirectory '\CRPSPosterior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' int2str(AnnualizeData) '_' int2str(CurrChain) '_' int2str(MaxPH) '_' int2str(NumDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
fid = fopen(mfile,'wt');
fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
fprintf(fid,'YNames = DataStructure.YNames;\n');
fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable CRPS and the joint energy score\n%%\n for each forecast horizon.\n%%\n');
fprintf(fid,'CRPSMatrix = DataStructure.CRPSMatrix;\n');
fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
%
% end the script file
%
fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
fclose(fid);
%
VarStr = vecstr(DSGEModel.VariableNames(PredictionVariables,:),', ');
VarStr = VarStr(1:length(VarStr)-1);
%
% write the screen unless ordered not to
%
if FileExist(ResFile)==1;
   PrtVal = '%9.4f';
   PrtStr1 = ['%s%s     %2.0f     %s      ' PrtVal];
   PrtStr2 = ['                         %2.0f     %s      ' PrtVal];
   nv = size(DSGEModel.VariableNames,2);
   n = DSGEModel.n;
   if nv<20;
      AddStr = SpaceStr(20-nv);
   else;
      AddStr = '';
   end;
   [n1,h] = size(CRPSMatrix);
   %
   for i=PredictionVariables;
      ResultsStr = '';
      ResultsStr = strvcat(ResultsStr,sprintf(PrtStr1,DSGEModel.VariableNames(i,:),AddStr,1,PredictionSampleStrMat(1,:),CRPSMatrix(i,1)));
      for j=2:h;
         ResultsStr = strvcat(ResultsStr,sprintf(PrtStr2,j,PredictionSampleStrMat(j,:),CRPSMatrix(i,j)));
      end;
      if i==PredictionVariables(1);
         %
         DivStr = MultiCharStr('-',size(ResultsStr,2)+1);
         FirstStr = MultiCharStr('=',size(ResultsStr,2)+1);
         SetLogText(strvcat(['Computation time: ' SecondsToHoursStr(CompTime) ' (hrs:min:sec)'],' ',FirstStr,'    Variable          Horizon   Period         CRPS',DivStr));
         pause(0.02);
      end;
      if i<max(PredictionVariables);
         ResultsStr = strvcat(ResultsStr,DivStr);
      else;
         ResultsStr = strvcat(ResultsStr,FirstStr,sprintf('Energy scores for: %s',VarStr),DivStr);
         for j=1:h;
            ResultsStr = strvcat(ResultsStr,sprintf(PrtStr2,j,PredictionSampleStrMat(j,:),CRPSMatrix(n+1,j)));
         end;
         ResultsStr = strvcat(ResultsStr,FirstStr);
      end;
      %
      SetLogText(ResultsStr);
   end;
   SetLogText(strvcat(' ','Output written to:',['Text file: ' GetFilename(ResFile)],['Mat file:  ' GetFilename(MatFile)],['Directory: ' GetPath(ResFile)],' '));
   Status = 1;
end;

%
% end of SequentialCRPS.m
%
