function SequentialLaplaceMarginalLikelihood(DSGEModel,CurrINI,controls,RecalculateInverseHessian)
% SequentialLaplaceMarginalLikelihood: Computes or collects an already estimated marginal likelihood
%                                      with the Laplace approximation for a DSGE model.
%
% USAGE:
%
%       SequentialLaplaceMarginalLikelihood(DSGEModel,CurrINI,controls,RecalculateInverseHessian)
%
% REQUIRED INPUT: DSGEModel (structure) whose fields contain all required information 
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%                 controls (structure) whose fields provide handles to all the UI
%                           controls in YADA's main dialog.
%
%                 RecalculateInverseHessian (boolean) which takes the value 1 if the
%                           inverse Hessian should be recalculated with finite
%                           differences and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 27, 2022.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-01-2022: Updated the documentation.
%
% * 31-01-2022: Made sure that the posterior mode file is updated with a finite difference
%               based inverse Hessian when it has been recalculated and its previous value
%               indicates that it is missing.
%
% * 01-02-2022: Added the LogLikeDensity and LogPostDensity values to the output mat-file.
%
% * 22-02-2022: Changed eval call to feval.
%
% * 06-01-2023: Updated the documentation.
%
% * 05-07-2023: The function now tries to load the AIMData file directly.
%
% * 09-01-2024: Took into account that "YADAHessianMatrix" now takes the input variable "NumWorkers".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% first attempt to load some posterior mode data
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
ErrorStr = '';
HeaderStr = 'YADA - Bad MAT File';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
%
% check if file loading was successful
%
if isempty(ErrorStr)==0;
   SetLogText('Unable to load posterior mode data. Aborting calculation.');
   return;
end;
theta = ModeData.thetaMode;
thetaPositions = ModeData.thetaPositions;
ModelParameters = ModeData.ModelParameters;
%
% try to load the AIMData, other wise use ModeData
%
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      AIMData = ModeData.AIMData;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      AIMData = ModeData.AIMData;
   end;
end;
%
% we can now calculate the log-likelihood
%
if DSGEModel.KalmanAlgorithm~=2;
   StateVarStructure = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
else;
   [StateVarStructure.lnLt,StateVarStructure.nonsingular,StateVarStructure.KalmanTime] = CalculateDSGESquareRootFilter(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI);
end;
if (isempty(StateVarStructure)==1)|(isfield(StateVarStructure,'lnLt')==0)|(isempty(StateVarStructure.lnLt)==1)|(sum(isinf(StateVarStructure.lnLt))==1)|(StateVarStructure.nonsingular==0);
   SetLogText('Unable to compute the log-likelihood.');
   return;
end;
%
% compute the value of the prior
%
LogJacobianMode = logJacobian(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
LogPriorMode = logPriorDSGE(ModeData.thetaMode,ModeData.thetaDist,ModeData.PriorDist,ModeData.LowerBound);
IsSystemPrior = 0;
if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
   LogSPMode = 0;
   LogMargLikeSystemLaplace = 0;
else;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      %
      % Test if we can calculate
      %
      try;
         LogSPMode = feval(GetName(DSGEModel.SystemPriorFile),ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);
         LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
         IsSystemPrior = 1;
      catch;
         LogSPMode = 0;
         LogMargLikeSystemLaplace = 0;
      end;
   else;
      LogSPMode = 0;
      LogMargLikeSystemLaplace = 0;
   end;
end;
%
% check if we should recalculate the inverse Hessian
%
if RecalculateInverseHessian==1;
   %
   % display a wait dialog
   %
   txt = ['Please wait while YADA computes the inverse Hessian with finite differences. Calculation started at: ' StringTrim(datestr(now,14)) '.'];
   WaitHandle = WaitDLG(txt,'information','Laplace Approximation Of Marginal Likelihood',500,300,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   %
   NumParam = length(theta);
   StepLengthStr = get(controls.optimize.steplength,'String');
   StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
   %
   % set the InitialStateVector value
   %
   if DSGEModel.UseOwnInitialState==1;
      if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
         DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
      else;
         DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
      end;
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
   [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
   %
   VechHessian = YADAHessianMatrix('logPosteriorPhiDSGE',ModeData.phiMode,StepLength,CurrINI.NumWorkers,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   %
   % delete the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
else;
   if isnan(ModeData.phiInverseHessianFinDiff)==0;
      InverseHessian = ModeData.phiInverseHessianFinDiff;
   else;
      InverseHessian = ModeData.InverseHessian;
   end;
end;
LogLikeDensity = sum(StateVarStructure.lnLt);
LogPostDensity = LogLikeDensity+LogJacobianMode+LogPriorMode+LogSPMode;
%
if RecalculateInverseHessian==0;
   LogMargLike = MargLikeLaplace(LogPostDensity,InverseHessian,1)-LogMargLikeSystemLaplace;
else;
   LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0)-LogMargLikeSystemLaplace;
   if isnan(ModeData.phiInverseHessianFinDiff)==1;
      phiInverseHessianFinDiff = HessianMat\eye(NumParam);
      save(ModeFile,'phiInverseHessianFinDiff','-APPEND');
   end;
end;
%
% print to file
%
PrtVal = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
%
TextFile = FixFilePath([DSGEModel.OutputDirectory '\LaplaceMarginalLikelihood-' DSGEModel.NameOfModel '-PostMode.txt']);
fid = fopen(TextFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*           L A P L A C E   A P P R O X I M A T I O N   O F   T H E            *\n');
fprintf(fid,'*                    M A R G I N A L   L I K E L I H O O D                     *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% sample information
%
fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
% now the marginal likelihood
%
fprintf(fid,['Log marginal likelihood:           ' PrtVal ' \n\n'],LogMargLike);
fprintf(fid,'where \n\n');
fprintf(fid,['   Log posterior density of phi:   ' PrtVal ' \n'],LogPostDensity);
fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity+LogMargLikeSystemLaplace);
if IsSystemPrior==1;
   fprintf(fid,['   Correction for the log marginal\n   likelihood of the system prior: ' PrtVal ' \n\n'],LogMargLikeSystemLaplace);
end;
fprintf(fid,'================================================================================\n\n');
%
% Some information about the filename and its location
%
fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);
%
% save results to mat-file
%
MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-LaplaceMarginalLikelihood-' DSGEModel.NameOfModel '-PostMode.mat']);
save(MatFile,'LogMargLike','LogMargLikeSystemLaplace','LogLikeDensity','LogPostDensity');
%
% write to log text
%
if LogMargLikeSystemLaplace==0;
   SetLogText(strvcat(' ',['Log ML (Laplace):              ' num2str(LogMargLike,'%0.4f')],' ',['Log posterior density of phi:  ' num2str(LogPostDensity,'%0.4f')],['Inverse Hessian term:          ' num2str(LogMargLike-LogPostDensity+LogMargLikeSystemLaplace,'%0.4f')],'Output written to:',['Mat file:  ' GetFilename(MatFile)],['Directory: ' GetPath(MatFile)],' '));
else;
   SetLogText(strvcat(' ',['Log ML (Laplace):              ' num2str(LogMargLike,'%0.4f')],' ',['Log posterior density of phi:  ' num2str(LogPostDensity,'%0.4f')],['Inverse Hessian term:          ' num2str(LogMargLike-LogPostDensity+LogMargLikeSystemLaplace,'%0.4f')],['Log system prior ML (Laplace): ' num2str(LogMargLikeSystemLaplace,'%0.4f')],' ','Output written to:',['Mat file:  ' GetFilename(MatFile)],['Directory: ' GetPath(MatFile)],' '));
end;

%
% end of SequentialLaplaceMarginalLikelihood.m
%
