function [Status,CurrChain,IsSubSample,RecomputePITs,ComputePredPaths,PredictionVariables,SelectedYData,Share] = SequentialPITsEstimation(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,RecomputePITs,ComputePredPaths,PredictionVariables,SelectedYData,Share)
% SequentialPITsEstimation: Can be used for computing the probability integral transform sequentially.
%
% USAGE:
%
%       [Status,CurrChain,IsSubSample,RecomputePITs,ComputePredPaths,PredictionVariables,...
%                         SelectedYData,Share] = SequentialPITsEstimation(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,...
%                                                                   RecomputePITs,ComputePredPaths,PredictionVariables,...
%                                                                   SelectedYData,Share)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information 
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                  CurrChain (integer) with the value of the MCMC chain to use.
%
%                  IsSubSample (boolean) that takes the value 1 if a subsample of the posterior
%                            draws should be used and 0 if all post-burnin draws should be used.
%                            When empty, the function checks which alternative should be considered.
%
%                  RecomputePITs (boolean) which takes the value 1 if the PITs should be recomputed
%                            and 0 otherwise.
%
%                  ComputePredPaths (boolean) which takes the value 1 if PITs based on prediction
%                            paths should also be computed and 0 otherwise.
%
%                  PredictionVariables (vector) with the positions of the observed variables
%                            to compute the predictive likelihood for. When empty, the function
%                            checks which alternative should be considered.
%
%                  SelectedYData (integer) with the selected set of actuals for the observed
%                            variables.
%
%                  Share (scalar) with the share of the posterior draws to use. Only relevant for
%                            the SMC with likelihood tempering and MitISEM posterior samplers.
%
% REQUIRED OUTPUT: Status (boolean) that takes the value 1 if the computations could be successfully
%                            carries out, 0 if not, and -1 if they were aborted by the user.
%
%                  CurrChain (integer) with the value of the MCMC chain to use.
%
%                  IsSubSample (boolean) that takes the value 1 if a subsample of the posterior
%                            draws should be used and 0 if all post-burnin draws should be used.
%                            When empty, the function checks which alternative should be considered.
%
%                  RecomputePITs (boolean) which takes the value 1 if the PITs should be recomputed
%                            and 0 otherwise.
%
%                  ComputePredPaths (boolean) which takes the value 1 if PITs based on prediction
%                            paths should also be computed and 0 otherwise.
%
%                  PredictionVariables (vector) with the positions of the observed variables
%                            to compute the predictive likelihood for. When empty, the function
%                            checks which alternative should be considered.
%
%                  SelectedYData (integer) with the selected set of actuals for the observed
%                            variables.
%
%                  Share (scalar) with the share of the posterior draws to use. Only relevant for
%                            the SMC with likelihood tempering and MitISEM posterior samplers.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 22, 2020.
%                        This version: March 18, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 23-10-2020: Finished the function.
%
% * 27-01-2021: Remember Shoah.
%
% * 18-05-2021: Took into account that "PrintPredictiveMoments" now requires the input variable
%               "IsAdaptiveLearning".
%
% * 19-05-2021: Took into account that "PrintPITs" now requires the input variable "IsAdaptiveLearning".
%
% * 06-01-2022: Updated the documentation.
%
% * 08-02-2022: Updated the documentation.
%
% * 15-02-2022: Took into account that ComputePITsDSGEPaths take the input variable MaxPH and that the prediction
%               horizon for simulated paths can exceed the horizon for which true values over this horizon are
%               available.
%
% * 19-05-2022: Added sub-sample checking code for the SMC and MitISEM samplers when combining paths with posterior
%               draws.
%
% * 24-05-2022: The Variable "DSGEModel.NameOfModel" now affacts the names of the output files.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-06-2023: Fixed print bugs with missing comment signs.
%
% * 25-01-2024: Updated the documentation.
%
% * 18-03-2024: Made sure that the PredictionEvent field is not changed.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Status = 0;
%
% first attempt to load some posterior mode data
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
ErrorStr = '';
HeaderStr = 'YADA - Bad MAT File';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
%
% check if file loading was successful
%
if isempty(ErrorStr)==0;
   SetLogText('Unable to load posterior mode data. Aborting calculation.');
   return;
end;
%
% if there are multiple chains, check which one to use if necessary
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
if (isempty(CurrChain)==1)&(NumChains>1);
   %
   % ask which chain to load
   %
   ChainsStr = '';
   for i=1:NumChains;
      ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
   end;
   [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Probability Integral Transform - Posterior Distribution','','',CurrINI);
elseif (isempty(CurrChain)==0)&(CurrChain>NumChains);
   action = 'cancel';
   SetLogText(['The selected chain number (' int2str(CurrChain) ') is greater than the available number of MCMC chains.']);
else;
   CurrChain = 1;
   action = 'ok';
end;
if strcmp(lower(action),'cancel')==1;
   SetLogText('Aborting calculation.');
   Status = -1;
   return;
end;
%
% check if the data actually exists
%
if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==0;
   SetLogText('Unable to locate the posterior parameter draws. Aborting calculation.');
   return;
end;
%
% check the PredictionVariables
%
if isempty(PredictionVariables)==1;
   PredictionVariables = ones(1,DSGEModel.n);
   PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
   [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
   if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
      SetLogText('No prediction variables selected. Aborting calculation.');
      Status = -1;
      return;
   elseif strcmp(lower(Action),'ok')==0;
      SetLogText('Aborting calculation.');
      Status = -1;
      return;
   end;
end;
%
% check if the selected variables to predict can indeed be used.
%
if max(PredictionVariables)>DSGEModel.n;
   SetLogText('Too many prediction variables. Aborting calculation.');
   return;
end;
DSGEModel.PredictionVariables = PredictionVariables;
%
% check if sub-sample should be
%
if isempty(IsSubSample)==1;
   txt = 'Would you like to use a sub-sample of the posterior draws when estimating the probability integral transforms via the marginal predictive moments?';
   answer = Query(txt,'question',400,'Probability Integral Transform - Posterior Distribution',500,'no',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      IsSubSample = 1;
   else;
      IsSubSample = 0;
   end;
end;
%
% load the data
%
[PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
clear('LogPost','AcceptedDraws','PostSample');
[NumPostDraws,NumParam] = size(thetaPostSample);
thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
NumDraws = NumPostDraws-NumBurnin;
if IsSubSample==1;
   if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
      UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
      thetaPostSample = thetaPostSample(UseDraws,:);
      NumDraws = length(UseDraws);
   else;
      %
      % check which draws we should use
      %
      UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'Probability Integral Transform - Posterior Distribution',Weights,Share);
      if isempty(UseDraws)==1;
         Status = -1;
         return;
      end;
      thetaPostSample = thetaPostSample(UseDraws,:);
      Weights = Weights(UseDraws,1);
      Weights = Weights*length(Weights)/sum(Weights);
      Share = length(UseDraws)/NumDraws;
      NumDraws = length(UseDraws);
   end;
end;
%
% collect some required data
%
theta = ModeData.thetaMode;
thetaPositions = ModeData.thetaPositions;
thetaIndex = ModeData.thetaIndex;
thetaDist = ModeData.thetaDist;
PriorDist = ModeData.PriorDist;
LowerBound = ModeData.LowerBound;
UniformBounds = ModeData.UniformBounds;
ModelParameters = ModeData.ModelParameters;
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% check how many time periods to use for predictive distributions
%
MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
% check the maximum when we take NaNs into account
%
YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
%
% check which actuals should be used
%
if isempty(SelectedYData)==1;
   if isempty(DSGEModel.Actuals)==1;
      SelectedYData = 1;
      DataAction = 'ok';
      YDataStr = 'Actuals from original sample';
   else;
      YDataStr = 'Actuals from original sample';
      m = length(DSGEModel.Actuals);
      for i=1:m;
         YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
      end;
      [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Probability Integral Transform - Posterior Distribution','','',CurrINI);
      if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
         YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
         BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
      end;
   end;
else;
   DataAction = 'ok';
   YDataStr = 'Actuals from original sample';
   if isempty(DSGEModel.Actuals)==0;
      m = length(DSGEModel.Actuals);
      for i=1:m;
         YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
      end;
   end;
   if (SelectedYData>1)&(length(DSGEModel.Actuals)>=SelectedYData-1);
      YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
      BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
   end;
end;
%
% test if we should continue
%
if strcmp(lower(DataAction),'ok')==0;
   SetLogText('Aborting calculation.');
   Status = -1;
   return;
end;
%
MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
[MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
MaxFH = min(MaxFH,MaxPH);
YData = YData(:,1:MaxFH);
%
% matrix with exogenous variables
%
X = [];
XP = [];
if DSGEModel.k>0;
   if NumPredX<MaxFH;
      X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
   end;
   if NumPredX>0;
      X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
   end;
   if NumPredX<MaxPH;
      XP = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
   end;
   if NumPredX>0;
      XP = [XP DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
   end;
end;
%
% Determine the prediction sample
%
SampleStrMat = CreateFullSample(DSGEModel);
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
BNSampleStrMat = SampleStrMat(LastPeriod-1:LastPeriod,:);
SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
%
% check if older calculations exists given that such may be used
%
tic;
MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-PredictiveMoments-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(CurrChain) '-' int2str(MaxFH) '.' int2str(NumDraws) '-' int2str(NumPostDraws) '.mat']);
PITMomentExist = 0;
if (RecomputePITs==0)&(FileExist(MatFile)==1);
   CheckData = load(MatFile);
   if isfield(CheckData,'PITMoments')==1;
      if isempty(CheckData.PITMoments)==0;
         PITMomentExist = 1;
      end;
   end;
   clear('CheckData');
end;
if PITMomentExist==1;
   DoneCalc = 1;
else;
   %
   % compute the predictive moments along with the PITs
   %
   SetLogText('Computing PITs based on predictive moments');
   DoneCalc = DSGEPredictiveMoments(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxFH,NumPostDraws,CurrChain,1,Weights,SmpStr,1);
end;
if DoneCalc==0;
   clear('thetaPostSample');
   SetLogText('Aborting calculation.');
   CompTime = toc;
   return;
end;
%
% we can now do the PIT calculations based on the moments
%
[PITMatrix,BNPITMatrix] = ComputePITs(YData,DSGEModel.Y(:,LastPeriod-1:LastPeriod),BackYData,PredictionVariables,MatFile,Weights,1,(exist('mvncdf')~=0),CurrINI);
CompTimeStepOne = toc;
SetLogText(['Computation time: ' SecondsToHoursStr(CompTimeStepOne) ' (hrs:min:sec)']);
%
% check if the user also wants to study PITs based on simulated paths
%
if ComputePredPaths==1;
   %
   % setup the necessary variables for computing or loading simulated paths from the predictive distribution
   %
   % determine how many prediction paths to use
   %
   NumPredPathsStr = get(controls.forecast.predpaths,'String');
   NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
   %
   SetLogText('Computing PITs using the prediction paths');
   %
   TransData = 0;
   AnnualizeData = 0;
%   if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
%      DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
%   end;
   DSGEModel.RunPredictionEvent = 0;
   tic;
   [DoneCalc2,PredEventData,StateCov,ShockCov,MeasureCov,ParameterCov] = DSGEPredictionPaths(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,XP,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
   if DoneCalc2==1;
      %
      % compute PITs based on the paths from the predictive distribution
      %
      PathPITMatrix = ComputePITsDSGEPaths(YData,NumDraws,NumPredPaths,MaxPH,CurrChain,1,Weights,DSGEModel,CurrINI);
      CompTimeStepTwo = toc;
      SetLogText(['Computation time: ' SecondsToHoursStr(CompTimeStepTwo) ' (hrs:min:sec)']);
   else;
      %
      % make sure that path based estimates are empty
      %
      PathPITMatrix = [];
      CompTimeStepTwo = [];
   end;
else;
   %
   % make sure that path based estimates are empty
   %
   PathPITMatrix = [];
   CompTimeStepTwo = [];
   NumPredPaths = 0;
end;
clear('thetaPostSample');
%
% store results on disk
%
SetLogText('Storing results to disk.');
[PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
PITFile = FixFilePath([DSGEModel.OutputDirectory '\PITs-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
YNames = DSGEModel.VariableNames;
save(PITFile,'YNames','PredictionVariables','PITMatrix','PredDateVector');
if isempty(BNPITMatrix)==0;
   save(PITFile,'BNPITMatrix','-APPEND')
end;
if isempty(PathPITMatrix)==0;
   save(PITFile,'PathPITMatrix','-APPEND');
end;
mfile = FixFilePath([DSGEModel.OutputDirectory '\PITsPosterior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' int2str(CurrChain) '_' int2str(MaxPH) '_' int2str(NumDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
fid = fopen(mfile,'wt');
fprintf(fid,['%%\n%% load the data in ' GetFilename(PITFile) '\n%%\n']);
fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(PITFile));
fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
fprintf(fid,'YNames = DataStructure.YNames;\n');
fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable uniform PITs and the joint uniform PIT\n%%\n%% for each forecast horizon. Use norminv to transform to normal.\n%%\n');
fprintf(fid,'PITMatrix = DataStructure.PITMatrix;\n');
if isempty(BNPITMatrix)==0;
   fprintf(fid,'%%\n%% n+1 x 2 matrix with the individual variable uniform PITs and the joint uniform PIT\n%%\n%% for the backcast and the nowcast. Use norminv to transform to normal.\n%%\n');
   fprintf(fid,'BNPITMatrix = DataStructure.BNPITMatrix;\n');
end;
if isempty(PathPITMatrix)==0;
   fprintf(fid,'%%\n%% n x h matrix with the individual variable uniform PITs using the prediction\n%%\n%% paths for each forecast horizon. Use norminv to transform to normal.\n%%\n');
   fprintf(fid,'PathPITMatrix = DataStructure.PathPITMatrix;\n');
end;
fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
%
% end the script file
%
fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
fclose(fid);
%
% write the results to a text file
%
ResFile = FixFilePath([DSGEModel.OutputDirectory '\PITs-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
PrintPITs(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,BNSampleStrMat,YDataStr(SelectedYData,:),PITMatrix,BNPITMatrix,PathPITMatrix,1,0,NumDraws,NumBurnin,NumPostDraws,[],CompTimeStepOne,CompTimeStepTwo,DSGEModel,CurrINI);
SetLogText(strvcat('Results on PITs written to:',['Text file: ' GetFilename(ResFile)],['Mat file:  ' GetFilename(PITFile)],['Directory: ' GetPath(ResFile)]));
%
% load the results on predictive moments
%
MPM = load(MatFile);
PredY = MPM.MeanPredictY;
n = DSGEModel.n;
CovPred = zeros(n,n,MaxFH);
for i=1:MaxFH;
   CovPred(:,:,i) = MPM.MeasureCov+MPM.ShockCov(:,:,i)+MPM.StateCov(:,:,i)+MPM.ParameterCov(:,:,i);
end;
BNCov = [];
BNPredictY = [];
if isfield(MPM,'BNPredictY')==1;
   BNPredictY = MPM.BNPredictY;
   if isempty(MPM.BNPredictY)==0;
      if isfield(MPM,'BNCov')==1;
         BNCov = MPM.BNCov;
      else;
         BNCov = MPM.BNObsCov+MPM.BNParamCov;
         save(MatFile,'BNCov','-APPEND');
      end;
      IsBackCast = 1;
   else;
      IsBackCast = 0;
   end;
else;
   IsBackCast = 0;
end;
if isfield(MPM,'BNSampleStrMat')==0;
   save(MatFile,'BNSampleStrMat','-APPEND');
end;
%
if isfield(MPM,'CovPred')==0;
   save(MatFile,'CovPred','PredictionSampleStrMat','-APPEND');
end;
if isfield(MPM,'YData')==0;
   save(MatFile,'YData','-APPEND');
end;
%
% write the results to log text and to file
%
ResFile = strrep(MatFile,'.mat','.txt');
PrintPredictiveMoments(ResFile,PredY,CovPred,PredictionSampleStrMat,BNPredictY,BNCov,BNSampleStrMat,[],NumDraws,NumPostDraws,NumBurnin,1,0,CompTimeStepOne+CompTimeStepTwo,DSGEModel,CurrINI);
%
% write results to screen
%
if FileExist(ResFile)==1;
   %PrtStr = ['%s%s   %10.4f   %10.4f'];
   PrtStr1 = '%s%s   %2.0f      %s   %7.4f   %7.4f';
   PrtStr2 = '                  %2.0f      %s   %7.4f   %7.4f';
   VarStr = vecstr(DSGEModel.VariableNames(PredictionVariables,:),', ');
   VarStr = VarStr(1:length(VarStr)-1);
   VarLength = size(DSGEModel.VariableNames,2);
   if VarLength<15;
      AddStr = SpaceStr(15-VarLength);
   else;
      AddStr = '';
   end;
   FirstStr = MultiCharStr('=',53);
   DivStr = MultiCharStr('-',53);
   ResultsStr = strvcat(FirstStr,'Variable        Horizon   Period   uniform    normal',DivStr);
   for i=1:n;
      ResultsStr = strvcat(ResultsStr,sprintf(PrtStr1,DSGEModel.VariableNames(i,:),AddStr,1,PredictionSampleStrMat(1,:),PITMatrix(i,1),norminv(PITMatrix(i,1))));
      for j=2:MaxFH;
         ResultsStr = strvcat(ResultsStr,sprintf(PrtStr2,j,PredictionSampleStrMat(j,:),PITMatrix(i,j),norminv(PITMatrix(i,j))));
      end;
      if i<n;
         ResultsStr = strvcat(ResultsStr,DivStr);
      else;
         ResultsStr = strvcat(ResultsStr,FirstStr);
         ResultsStr = strvcat(ResultsStr,['Selected prediction variables: ' StringTrim(VarStr)]);
         ResultsStr = strvcat(ResultsStr,DivStr);
         for j=1:MaxFH;
            ResultsStr = strvcat(ResultsStr,sprintf(PrtStr2,j,PredictionSampleStrMat(j,:),PITMatrix(n+1,j),norminv(PITMatrix(n+1,j))));
         end;
         ResultsStr = strvcat(ResultsStr,FirstStr);
      end;
   end;
   if isempty(BNPITMatrix)==0;
      ResultsStr = strvcat(ResultsStr,' ',FirstStr);
      ResultsStr = strvcat(ResultsStr,'Variable        Horizon   Period   uniform    normal',DivStr);
      for i=1:n;
         ResultsStr = strvcat(ResultsStr,sprintf(PrtStr1,DSGEModel.VariableNames(i,:),AddStr,-1,BNSampleStrMat(1,:),BNPITMatrix(i,1),norminv(BNPITMatrix(i,1))));
         ResultsStr = strvcat(ResultsStr,sprintf(PrtStr2,0,BNSampleStrMat(2,:),BNPITMatrix(i,2),norminv(BNPITMatrix(i,2))));
         if i<n;
            ResultsStr = strvcat(ResultsStr,DivStr);
         else;
            ResultsStr = strvcat(ResultsStr,FirstStr);
            ResultsStr = strvcat(ResultsStr,['Selected prediction variables: ' StringTrim(VarStr)]);
            ResultsStr = strvcat(ResultsStr,DivStr);
            ResultsStr = strvcat(ResultsStr,sprintf(PrtStr2,-1,BNSampleStrMat(1,:),BNPITMatrix(n+1,1),norminv(BNPITMatrix(n+1,1))));
            ResultsStr = strvcat(ResultsStr,sprintf(PrtStr2,0,BNSampleStrMat(2,:),BNPITMatrix(n+1,2),norminv(BNPITMatrix(n+1,2))));
            ResultsStr = strvcat(ResultsStr,FirstStr);
         end;
      end;
   end;
   SetLogText(strvcat(' ','Moment based PIT estimates:','---------------------------',' ',ResultsStr,'Predictive moment output written to:',['Text file: ' GetFilename(ResFile)],['Mat file:  ' GetFilename(MatFile)],['Directory: ' GetPath(ResFile)],' '));
   Status = 1;
end;

%
% end of SequentialPITsEstimation.m
%
