function [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY,AverageZLBShocks,NumZLBTimes] = ZLBDSGEPredictionDistribution(n,OutputDirectory,ModelStr,CondStr,ShockMethod,NameOfModel,ZLBSampleT,RtildePosition,REqPosition,ZLBRuleType,h,NumDraws,NumPredPaths,AnnualizeData,TransData,CurrChain,IsPosterior,IsCentered,YDataStr,Weights,QuantileVector,VariableNames,ConfidenceRegionMethod,CurrINI)
% ZLBDSGEPredictionDistribution: Computes the mean value of conditional and unconditional prediction paths when subject
%                                to the zero lower bound as well as quantiles by loading data from disk.
%
% USAGE:
%
%       [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY,...
%                                   AverageZLBShocks,NumZLBTimes] = ZLBDSGEPredictionDistribution(n,OutputDirectory,ModelStr,CondStr,...
%                                                                                  ShockMethod,NameOfModel,ZLBSampleT,RtildePosition,...
%                                                                                  REqPosition,ZLBRuleType,h,NumDraws,NumPredPaths,...
%                                                                                  AnnualizeData,TransData,CurrChain,IsPosterior,...
%                                                                                  IsCentered,YDataStr,Weights,QuantileVector,...
%                                                                                  VariableNames,ConfidenceRegionMethod,CurrINI)
%
% REQUIRED INPUT:  n (integer) with the number of observed variables.
%
%                  OutputDirectory (string vector) with the full path to the base output directory.
%
%                  ModelStr (string vector) that is either "DSGE" or "AdaptiveLearningDSGE-x".
%
%                  CondStr (string vector) that is either "Cond" or "".
%
%                  ShockMethod (string vector) that is either "Direct-", "WaggonerZha-", "Subset-" or "".
%
%                  NameOfModel (string vector) with the name of the model.
%
%                  ZLBSampleT (integer)  with the length of the sample over which the zero lower
%                             bound may be binding.
%
%                  RtildePosition (integer) with the position of the monetary policy interest rate.
%
%                  REqPosition (integer) with the position of the monetary policy rule.
%
%                  ZLBRuleType (integer) which is either 1 for lagged unrestricted policy rate (the
%                             default) or 2 for lagged restricted policy rate.
%
%                  h (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPredPaths (integer) with tht total number of prediction paths per parameter draw.
%
%                  AnnualizeData (boolean) that take the value 1 if annualized results should be checked and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                             if draws from the prior are supplied.
%
%                  IsCentered (boolean) that takes the value 1 if the distributions not subject to the lower
%                             bound have been centered on the observed data and 0 otherwise.
%
%                  YDataStr (string) reflecting which data for the observed variables has been used. Is an
%                             empty string when IsCentered is 0.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                             likelihood tempering posterior sampler.
%
%                  QuantileVector (vector) of dimension 1xNumQuants with percent values for the quantiles,
%                             e.g. [5 10 90 95].
%
%                  VariableNames (string matrix) with the names of the observed variables.
%
%                  ConfidenceRegionMethod (integer) that takes the value 1 for the equal tails method, and
%                             2 for the highest probability density method.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: MeanPrediction (matrix) of dimension nxh with the mean predictions.
%
%                  CovPrediction (matrix) of dimension nxnxMaxPH with the sample covariance matrix for the predictions.
%
%                  Quantiles (structure) of dimension NumQuants where each has a field called "Quantile" which
%                             is an nxh matrix with the quantiles of the prediction paths. The second field
%                             is "percent" that gives the percentile value.
%
%                  KernelX (3D matrix) of dimension n x 2^8 x h with the x-axis values.
%
%                  KernelY (3D matrix) of dimension n x 2^8 x h with the y-axis values.
%
%                  AverageZLBShocks (qxh) matrix with the average shock values when the ZLB is binding in a
%                             given forecast sample period.
%
%                  NumZLBTimes (1xh) vector with the number of times the ZLB is binding
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: September 1, 2022.
%                        This version: March 5, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-09-2022: Continued work on the function after the initialization part.
%
% * 08-09-2022: Added the input variable "YDataStr".
%
% * 06-01-2023: Updated the documentation.
%
% * 21-11-2023: Added the output variables "AverageZLBShocks" and "NumZLBTimes".
%
% * 23-11-2023: Added the input variable "ZLBRuleType".
%
% * 26-01-2024: Updated the documentation.
%
% * 05-03-2024: Added the ModelStr input variable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
MeanPrediction = zeros(n,h);
CovPrediction = zeros(n,n,h);
NumQuants = length(QuantileVector);
KernelX = zeros(n,2^8,h);
KernelY = KernelX;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
AverageZLBShocks = [];
NumZLBTimes = [];
%
% Initialize quantiles output
%
for i=1:NumQuants;
   Quantiles(i).Quantile = zeros(n,h);
   Quantiles(i).percent = QuantileVector(i);
end;
%
if isempty(CondStr)==1;
   HeadStr = 'ZLB Predictive Distributions';
   txtStr = '';
else;
   HeadStr = 'ZLB Conditional Predictive Distributions';
   txtStr = 'conditional ';
end;
if IsPosterior==0;
   HeadStr = [HeadStr ' - Prior'];
end;
%
% set up a wait dialog
%
txt = ['Please wait while YADA computes the mean ' txtStr 'predictions for the observed variables. The data are loaded sequentially from disk and it may therefore take some time.'];
WaitHandle = WaitDLG(txt,'information',HeadStr,500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% compute the mean prediction
%
CheckZLBData = 1;
NumZLBTimes = zeros(1,h);
TotalPaths = NumPredPaths*NumDraws;
for ParamIndex=1:NumDraws;
   if IsPosterior==1;
      if AnnualizeData==0;
         if TransData==0;
            file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
         else;
            file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Transformed-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
         end;
      else;
         file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Annual-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
      end;
   else;
      if AnnualizeData==0;
         if TransData==0;
            file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
         else;
            file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Transformed-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
         end;
      else;
         file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Annual-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
      end;
   end;
   %
   % load the data
   %
   PredData = load(file);
   PredPaths = PredData.PredPaths;
   MeanPrediction = MeanPrediction+((1/TotalPaths)*sum(PredPaths*Weights(ParamIndex),3));
   for j=1:h;
      PP = shiftdim(PredPaths(:,j,:),2)';
      CovPrediction(:,:,j) = CovPrediction(:,:,j)+((Weights(ParamIndex)/TotalPaths)*PP*PP');
   end;
   %
   % check if the input files contain "NumZLBTimes".
   %
   if (CheckZLBData==1)&(isfield(PredData,'NumZLBTimes')==1)&(isfield(PredData,'ShockMatrix')==1);
      NumZLBTimes = NumZLBTimes+PredData.NumZLBTimes;
      if ParamIndex==1;
         q = size(PredData.ShockMatrix,1);
         AverageZLBShocks = zeros(size(PredData.ShockMatrix));
      end;
      AverageZLBShocks = AverageZLBShocks+((Weights(ParamIndex)/TotalPaths)*PredData.ShockMatrix);
   else;
      NumZLBTimes = [];
      AverageZLBShocks = [];
      CheckZLBData = 0;
   end;
   %
   clear('PredData');
   drawnow;
end;
%
if CheckZLBData==1;
   for i=1:h;
      if NumZLBTimes(i)>0;
         AverageZLBShocks(:,i) = (TotalPaths/NumZLBTimes(i))*AverageZLBShocks(:,i);
      end;
   end;
end;
%
% compute the central second moment from the non-central
%
for j=1:h;
   CovPrediction(:,:,j) = CovPrediction(:,:,j)-(MeanPrediction(:,j)*MeanPrediction(:,j)');
   CovPrediction(:,:,j) = diag(abs(diag(CovPrediction(:,:,j))))+tril(CovPrediction(:,:,j),-1)+tril(CovPrediction(:,:,j),-1)';
end;
%
% compute all the quantile values
%
for j=1:h;
   if ConfidenceRegionMethod==1;
      txt = ['Please wait while YADA computes quantile values and kernel density estimates for the ' txtStr 'predictions of the observed variables. Current forecast horizon ' int2str(j) '.'];
   else;
      txt = ['Please wait while YADA computes confidence/credible regions and kernel density estimates for the ' txtStr 'predictions of the observed variables. Current forecast horizon ' int2str(j) '.'];
   end;
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   % initialize a temporary variable
   %
   try;
      TempVar = zeros(TotalPaths,n);
      for ParamIndex=1:NumDraws;
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
               else;
                  file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Transformed-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
               end;
            else;
               file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Annual-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
               else;
                  file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Transformed-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
               end;
            else;
               file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Annual-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
            end;
         end;
         %
         % load the data
         %
         PredData = load(file);
         PredPaths = PredData.PredPaths;
         for p=1:NumPredPaths;
            CurrPath = p+(NumPredPaths*(ParamIndex-1));
            TempVar(CurrPath,:) = PredPaths(:,j,p)';
         end;
         clear('PredData');
         drawnow;
      end;
      %
      % TempVar is now fully loaded and may
      %
      TempVar = sort(TempVar);
      if ConfidenceRegionMethod==1;
         for k=1:NumQuants;
            if QuantileVector(k)<50;
               QuantVal = ceil((QuantileVector(k)/100)*TotalPaths);
            else;
               QuantVal = floor((QuantileVector(k)/100)*TotalPaths);
            end;
            Quantiles(k).Quantile(:,j) = TempVar(QuantVal,:)';
         end;
      else;
         for k=1:floor(NumQuants/2);
            ConfDraws = floor((QuantileVector(NumQuants-k+1)-QuantileVector(k))*TotalPaths/100);
            for t=1:n;
               Bounds = HighestProbabilityDensityBand(TempVar(:,t),ConfDraws);
               Quantiles(k).Quantile(t,j) = Bounds(1);
               Quantiles(NumQuants-k+1).Quantile(t,j) = Bounds(2);
            end;
         end;
      end;
   catch;
      for t=1:n;
         TempVar = zeros(TotalPaths,1);
         for ParamIndex=1:NumDraws;
            if IsPosterior==1;
               if AnnualizeData==0;
                  if TransData==0;
                     file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
                  else;
                     file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Transformed-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
                  end;
               else;
                  file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Annual-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
               end;
            else;
               if AnnualizeData==0;
                  if TransData==0;
                     file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
                  else;
                     file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Transformed-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
                  end;
               else;
                  file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Annual-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
               end;
            end;
            %
            % load the data
            %
            PredData = load(file);
            PredPaths = PredData.PredPaths;
            for p=1:NumPredPaths;
               CurrPath = p+(NumPredPaths*(ParamIndex-1));
               TempVar(CurrPath,1) = PredPaths(t,j,p)';
            end;
            clear('PredData');
            drawnow;
         end;
         %
         % TempVar is now fully loaded and may
         %
         TempVar = sort(TempVar);
         if ConfidenceRegionMethod==1;
            for k=1:NumQuants;
               if QuantileVector(k)<50;
                  QuantVal = ceil((QuantileVector(k)/100)*TotalPaths);
               else;
                  QuantVal = floor((QuantileVector(k)/100)*TotalPaths);
               end;
               Quantiles(k).Quantile(t,j) = TempVar(QuantVal,1);
            end;
         else;
            for k=1:floor(NumQuants/2);
               ConfDraws = floor((QuantileVector(NumQuants-k+1)-QuantileVector(k))*TotalPaths/100);
               Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
               Quantiles(k).Quantile(t,j) = Bounds(1);
               Quantiles(NumQuants-k+1).Quantile(t,j) = Bounds(2);
            end;
         end;
      end;
   end;
   drawnow;
   %
   for i=1:n;
      if TempVar(1,i)+1.0e-6<TempVar(TotalPaths,i);
         [x,f] = YADAMHDensEst(TempVar(:,i),2^8,0,'gaussian',[]);
         KernelX(i,:,j) = x';
         KernelY(i,:,j) = f';
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of ZLBDSGEPredictionDistribution.m
%
