function [Hlage,H0e,Hexpe,De,DAe,M] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,T_ZLB,RtildePosition,REqPosition,ZLBRuleType)
% ZLBDSGEStructuralForm: Reshuffles the structural form of the DSGE model such that the restricted policy rate
%                        is separated from the unrestricted and anticipated shocks are allowed for.
%
% USAGE:
%
%       [Hlage,H0e,Hexpe,De,DAe] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,T_ZLB,RtildePosition,REqPosition,ZLBRuleType)
%
% REQUIRED INPUT:  Hlag (matrix) of dimension r x r with the coefficient values on the lagged state variables.
%
%                  H0 (matrix) of dimension r x r with the coefficient values on the contemporaneosu state variables.
%
%                  Hexp (matrix) of dimension r x r with the coefficient values on the expectated future values
%                                  of the state variables.
%
%                  D (matrix) of dimension r x q with the coefficient values on the structural shocks.
%
%                  T_ZLB (integer) with the length of the sample over which the zero lower bound may be
%                                         binding.
%
%                  RtildePosition (integer) with the position of the monetary policy rate among the state variables.
%
%                  REqPosition (integer) with the position of the monetary policy rule among the state
%                                         equations.
%
%                  ZLBRuleType (integer) which is 1 (default) if the policy rule can include the lagged
%                                        unrestricted policy rate and 2 if it can include the restricted
%                                        policy rate instead.
%
% REQUIRED OUTPUT: Hlage (matrix) of dimension (r+1) x (r+1) with the coefficient values on the lagged state variables.
%
%                  H0e (matrix) of dimension (r+1) x r+1 with the coefficient values on the contemporaneous state variables.
%
%                  Hexpe (matrix) of dimension (r+1) x (r+1) with the coefficient values on the expectated future values
%                                  of the state variables.
%
%                  De (matrix) of dimension (r+1) x q with the coefficient values on the structural shocks.
%
%                  DAe (matrix) of dimension (r+1) x (T_ZLB+1) with the coefficients on the anticipated structural shocks.
%
%                  M (matrix) of dimension (T_ZLB+1) x (T_ZLB+1) which links current to lagged anticipated shocks.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 23, 2024.
%                        This version: February 29, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-02-2024: Updated the documentation.
%
% * 29-02-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DAe = [];
De = [];
[k,q] = size(D);
Hlage = [];
H0e = [];
Hexpe = [];
%
% transform the system by adding a separate equation for the unrestricted policy rate and separating it from
% the restricted policy rate
%
e_T1 = zeros(T_ZLB+1,1);
e_T1(1) = 1;
e_ps = zeros(k,1);
e_ps(RtildePosition) = 1;
e_pm = zeros(k,1);
e_pm(REqPosition) = 1;
Ik = eye(k);
Kps = Ik(:,setdiff((1:k),RtildePosition));
Kpm = Ik(:,setdiff((1:k),REqPosition));
%
% new code starts here
%
T = T_ZLB;
if T>1;
   M = [zeros(T,1) eye(T);zeros(1,T+1)];
else;
   M = 0;
end;
%
% define matrices from the non-monetary policy equations
%
Hmlag = Kpm'*Hlag;
Hm = Kpm'*H0;
hmlag = e_pm'*Hlag*Kps*Kps';
hm = e_pm'*H0*Kps*Kps';
hslag = e_pm'*Hlag*e_ps;
hs = e_pm'*H0*e_ps;
Dm = Kpm'*D;
dm = e_pm'*D;
if ZLBRuleType==2;
   %
   % replace the lagged unrestricted policy rate with the lagged
   % restricted policy rate
   %
   hmlag = hmlag+(hslag*e_pm');
   hslag = 0;
end;
Hmexp = Kpm'*Hexp;
hmexp = e_pm'*Hexp*Kps*Kps';
hsexp = e_pm'*Hexp*e_ps;
%
% now create the output matrix
%
Hlage = [Hmlag zeros(k-1,1);hmlag hslag;zeros(1,k+1)];
H0e = [Hm zeros(k-1,1);hm hs;e_ps' -1];
Hexpe = [Hmexp zeros(k-1,1);hmexp hsexp;zeros(1,k+1)];
De = [Dm;dm;zeros(1,q)];
DAe = [zeros(k,T+1);e_T1'];

%
% end of ZLBDSGEStructuralForm.m
%
