function [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = DSGECondPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,h,NumPaths,AnnualizeData,TransData,CurrChain,IsPosterior,Weights)
% DSGECondPredictionPaths: Calculates a certain number of possible paths that the endogenous variables
%                          can take out-of-sample for a given sample of draws from the posterior of the
%                          parameters in the DSGE model and a given path for some conditioning variables.
%
% USAGE:
%
%       [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = ...
%                 DSGECondPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,...
%                             CurrINI,X,Z,U,h,NumPaths,AnnualizeData,TransData,CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or the
%                                  prior distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) of dimension NumParam with the posterior mode estimates or the initial
%                                  values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxh with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                                  predictions on.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                                  data and the conditioning data.
%
%                  h (integer) with the prediction horizon.
%
%                  NumPaths (integer) the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                  likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                  the PredPaths data satisfies the event, the second hold the number
%                                  of times the PredPaths data is greater than the event, the third
%                                  the number of time the data is above the upper bound, while the
%                                  fourth column holds the number of times the event has been tested.
%                                  The fifth and sixth column measure the average deviation from the
%                                  lower bound when below the lower bound and the square of this, while
%                                  the seventh and eighth columns hold similar data for the upper bound
%                                  of the event when the variables are above the upper bound.
%
%                  ShockMean (q x h) matrix with the average of the shocks under conditioning when
%                                  the measurement error is set to zero and the state vector at T
%                                  is set to KsiTT.
%
%                  ShockNames (q x qstr) string matrix with the names of the economic shocks.
%
%                  StateCov (n x n x h) matrix with the estimated state uncertainty.
%
%                  ShockCov (n x n x h) matrix with the estimated shock uncertainty.
%
%                  MeasureCov (n x n) matrix the the estimated measurement error uncertainty.
%
%                  ParameterCov (n x n x h) matrix with the estimated parameter uncertainty.
%
%                  
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 9, 2007.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-03-2007: Finished the function.
%
% * 14-03-2007: Made sure that the K1 and K2 matrices took the field "ConditionalVariables" of the structure "DSGEModel"
%               into account.
%
% * 26-03-2007: Updated the documentation. The modesty statistics now use KsiT = KsiTT so that the state is treated as
%               known for the conditional and unconditional forecasts for these statistics.
%
% * 27-03-2007: Made sure the Leeper-Zha statistic uses the correct state vector.
%
% * 04-05-2007: Added the prediction event calculations.
%
% * 15-05-2007: Added an extra "drawnow" after shutting down the progress and the wait dialog.
%
% * 06-06-2007: Changed "predictive paths" to "prediction paths" in the title for the progress dialog.
%
% * 13-07-2007: Fixed a bug when computing "MeanPredictY" where shocks for i=g+1,...,h were not taklen into
%               account.
%
% * 25-10-2007: The code now takes the boolean variable "AdjustPredictionPaths" into account. Hence, for each
%               selected parameter vector, the 3D matrix with prediction paths is adjusted such that its
%               sample mean equals its population mean.
%
% * 13-11-2007: Updated the documentation.
%
% * 03-12-2007: Made sure the Kalman filter function includes the correct X matrix.
%
% * 17-12-2007: Added the input "h". This makes it possible for X to be an empty matrix.
%
% * 18-12-2007: Added the outputs "ShockMean" for the mean value of the shocks under conditioning, and "ShockNames"
%               with the names for the shocks.
%
% * 20-12-2007: Fixed an annualization bug when the theoretical mean is used for correcting the sampled
%               mean. The non-annualized mean was erroneously used.
%
% * 21-12-2007: Added the code for dealing with unit-root state variables when running the Kalman filter.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 24-01-2008: The input U is now a matrix. Smooth estimation of the state and state covariance using
%               the conditioning assumptions has been included.
%
% * 05-05-2008: Added the input variable "TransData".
%
% * 23-05-2008: Updated the documentation.
%
% * 02-07-2008: Added the variable "DSGEModel.AllowUnitRoot" for the Kalman filters.
%
% * 07-07-2008: Added code to allow for the case when the measurement matrix H is time-varying.
%
% * 22-01-2009: Updated the documentation.
%
% * 13-02-2009: The mcode values 7 and 8 are now properly mentioned when encountered.
%
% * 09-03-2009: Added quotation marks around paths for the dos copy commands.
%
% * 26-03-2009: Added the status output to the "SolveDSGEModel" code. The function also checks
%               if the PTT matrix is positive semidefinite. When it is not, the matrix is recomputed
%               using the initial parameter values of the posterior mode values depending on which
%               were provided as input. Added the input variable "IsPosterior".
%
% * 27-03-2009: Added a missing "status" output variable from "SolveDSGEModel".
%
% * 07-04-2009: Changed the maximum height of the "YADA - Error Message" message box.
%
% * 15-07-2009: Fixed a bug where "etaCTmp" was not properly initialized for the shocks
%               that are manipulated.
%
% * 20-07-2009: Made sure that "KsiCTmp" is computed using a lag of "KsiCTmp" rather than of "KsiT".
%
% * 27-08-2009: Added extra input to "StateSmoother" and "StateSmootherHt" function.
%
% * 04-12-2009: Made sure that smooth estimation of the states using the conditioning assumption only occurs
%               when the forecast horizon is not longer than the length of the historical sample.
%
% * 05-01-2010: Updated the documentation.
%
% * 07-06-2010: Changed the computation of "MeanPrediction".
%
% * 18-11-2010: Took into account that "PredEventData" now has 7 columns.
%
% * 19-11-2010: And now 8.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 02-01-2011: The function now supports the square root filter.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 25-01-2011: Took into account that "CalcTransformData" takes the input variable "TransMatrix".
%
% * 15-02-2011: Started work on computing the covariance matrix of the predictive distribution.
%
% * 28-02-2011: Deleted unused code.
%
% * 11-03-2011: Switched to the "RunKalmanFilter" function.
%
% * 15-03-2011: Switched to the "RunKalmanSmoother" function.
%
% * 18-05-2011: Added test for calling and call to "CondPredictionSmoother" and "CondPredictionSmootherHt".
%
% * 25-10-2011: Took into account that "RunKalmanSmoother" now requires the input variable "DiffuseInit".
%
% * 09-12-2011: Fixes for dealing with missing observations in historical sample.
%
% * 22-12-2011: Made sure that "KalmanGain" and so on are properly handled under diffuse initialization.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-02-2012: Added support for missing observations in the conditioning assumptions and for lag-varying
%               K2 matrices.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 25-08-2017: Added the "Weights" input variable.
%
% * 16-05-2018: Updated the documentation.
%
% * 02-08-2018: The function supports user written model solver function.
%
% * 17-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors for "RunKalmanSmoother".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
[NumDraws,NumParam] = size(thetaPostSample);
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
k = DSGEModel.k;
n = DSGEModel.n;
g = size(Z,2);
DoneCalc = 0;
PredEventData = [];
ShockMean = [];
ShockNames = DSGEModel.StateShockNames;
if (AnnualizeData==1)|(TransData==1);
   StateCov = [];
   ShockCov = [];
   MeasureCov = [];
   ParameterCov = [];
   if TransData==0;
      TypeStr = 'Annualized';
   else;
      TypeStr = 'Transformed';
   end;
else;
   StateCov = zeros(n,n,h);
   ShockCov = StateCov;
   MeasureCov = StateCov;
   ParameterCov = StateCov;
   MeanPredictY = zeros(n,h);
   TypeStr = 'Original';
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of predictive distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of predictive distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the conditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% set up the data matrices
%
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
else;
   Xdata = [];
end;
if strcmp(DSGEModel.DataFrequency,'q')==1;
   Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
else;
   Yinit = [];
end;
maxYinit = max(max(isnan(Yinit)));
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(n,8);
end;
%
if TransData==1;
   Yinitial = DSGEModel.Y(:,1:DSGEModel.LastPeriod);
   %
   % try to covert
   %
   YTmp = [Yinitial DSGEModel.Y(:,DSGEModel.LastPeriod)*ones(1,h)];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   maxYinitial = max(max(isnan(Yinitial)));
end;
%
q = length(DSGEModel.StateShockPositions);
%
% create the matrices for selecting shocks
%
N = eye(q);
M = N(:,DSGEModel.ConditionalShocks);
M_orth = N(:,setdiff((1:q),DSGEModel.ConditionalShocks));
m = size(Z,1);
ShockMean = zeros(q,h);
%
% create the mapping matrices for conditioning variables
%
K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
SCond = (isequal(U,zeros(size(U)))&isequal(K2,zeros(size(K2))));
if (DSGEModel.KsiUseCondData==1)&(SCond==1);
   if (k>0);
      XPred = X(:,1:g);
   else;
      XPred = [];
   end;
end;
%
K = zeros(n*g,m*g);
for i=1:g;
   Ind = (1+((g-i)*n):(g-i+1)*n);
   Ind2 = (1+((g-i)*m):(g-i+1)*m);
   K(Ind,Ind2) = K1;
   for j=1:i-1;
      Ind3 = Ind+(j*n);
      if K2length==2;
         K(Ind3,Ind2) = K2;
      else;
         if j<=size(K2,3);
            K(Ind3,Ind2) = K2(:,:,j);
         end;
      end;
   end;
end;
%
% create a vector with positions for the conditioning assumptions that should be used
%
ZTg = vec(fliplr(Z));
PosVec = (1:m*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
mg = length(PosVec);
%
% adjust for missing restrictions
%
K = K(:,PosVec);
%
IgM = kron(eye(g),M);
% new
IgM = IgM(:,PosVec);
%
%IgMMorth = kron(eye(g),M_orth*M_orth');
% new
IgMMorth = null(IgM','r');
IgMMorth = IgMMorth*IgMMorth';
%
% test if we can invert the model for the standardized shocks at posterior mode
%
if length(size(H))==2;
   etaMat = K1'*H'*B0*M;
   if rank(etaMat)<m;
      txt = ['The shocks you have selected for conditioning cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution has therfore been aborted.'];
      About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
      PredPaths = [];
      return;
   end;
else;
   BadEtaMat = 0;
   for i=1:g;
      etaMat = K1'*H(:,:,DSGEModel.LastPeriod+i)'*B0*M;
      if rank(etaMat)<m;
         txt = ['The shocks you have selected for conditioning cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution has therfore been aborted.'];
         About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
         PredPaths = [];
         BadEtaMat = 1;
         break;
      end;
   end;
   if BadEtaMat==1;
      return;
   end;
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumDraws*NumPaths,'%0.0f') ' Conditional Prediction Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = ['Conditional Predictive Distributions' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes conditional predictive distributions for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Conditional Predictive Distributions' PriorHeader ' - ' TypeStr ' Data'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
ZPos = ~isnan(Z);
%NumDraws = 1;
%NumPaths = 1;
if length(size(H))==2;
   for ParamIndex=1:NumDraws;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % solve the mode for initial parameter values or for the posterior mode values
         %
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
      %
      % check if we can use these values.
      %
      etaMat = K1'*H'*B0*M;
      if rank(etaMat)==m;
         %
         % We can now run the Kalman filter
         %
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
         %
         % Now we need to calculate smooth values for the states for g periods
         %
         if ParamIndex==1;
            [r,T] = size(Ksitt1);
            q = size(B0,2);
         end;
         %
         % calculate measurement errors for the sample T-g+1 until T
         %
         if (DSGEModel.KsiUseCondData==1)&(SCond==1);
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T);
               SigmaSqRoot = SigmaSqRoot(:,:,T);
               InvMSEY = InvMSEY(:,:,T);
               KalmanGain = KalmanGain(:,:,T);
            end;
            [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            [KsiTTz,PTTz] = CondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H,R,F,B0,DSGEModel.KalmanAlgorithm);
            KsiTT = KsitT(:,1);
         elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T-g+1:T);
               SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
               InvMSEY = InvMSEY(:,:,T-g+1:T);
               KalmanGain = KalmanGain(:,:,T-g+1:T);
            end;
            [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            if rank(R)>0;
               if k>0;
                  wtT = Y(:,T-g+1:T)-(A'*Xdata(:,T-g+1:T))-(H'*KsitT);
               else;
                  wtT = Y(:,T-g+1:T)-(H'*KsitT);
               end;
            else;
               wtT = zeros(n,g);
            end;
            %
            % Smooth estimation of the states at time T using the conditioning assumptions.
            %
            if k>0;
               if g>1;
                  Xtra = Xdata(:,T-g+2:T);
               else;
                  Xtra = [];
               end;
               [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,F,B0);
            else;
               [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,F,B0);
            end;
            KsiTT = KsitT(:,g);
         else;
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T);
               SigmaSqRoot = SigmaSqRoot(:,:,T);
               InvMSEY = InvMSEY(:,:,T);
               KalmanGain = KalmanGain(:,:,T);
            end;
            [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            KsiTTz = KsitT(:,1);
            PTTz = PtT(:,:,1);
            KsiTT = KsitT(:,1);
         end;
         PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
         %
         % use the Schur decomposition for the reduced rank matrix PTT instead of
         % the Choleski decomposition
         %
         [S,L] = SchurDecomposition(PTTz);
         %
         % test to make sure that PTTz is positive semidefinite
         %
         if isreal(sqrt(L))==0;
            %
            % recompute everything for the posterior mode or initial parameter values
            %
            [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
            etaMat = K1'*H'*B0*M;
            [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
            if ParamIndex==1;
               [r,T] = size(Ksitt1);
               q = size(B0,2);
            end;
            if (DSGEModel.KsiUseCondData==1)&(SCond==1);
               if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                  Yerror = Yerror(:,T);
                  SigmaSqRoot = SigmaSqRoot(:,:,T);
                  InvMSEY = InvMSEY(:,:,T);
                  KalmanGain = KalmanGain(:,:,T);
               end;
               [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
               [KsiTTz,PTTz] = CondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H,R,F,B0,DSGEModel.KalmanAlgorithm);
               KsiTT = KsitT(:,1);
            elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
               if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                  Yerror = Yerror(:,T-g+1:T);
                  SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
                  InvMSEY = InvMSEY(:,:,T-g+1:T);
                  KalmanGain = KalmanGain(:,:,T-g+1:T);
               end;
               [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
               if rank(R)>0;
                  if k>0;
                     wtT = Y(:,T-g+1:T)-(A'*Xdata(:,T-g+1:T))-(H'*KsitT);
                  else;
                     wtT = Y(:,T-g+1:T)-(H'*KsitT);
                  end;
               else;
                  wtT = zeros(n,g);
               end;
               if k>0;
                  if g>1;
                     Xtra = Xdata(:,T-g+2:T);
                  else;
                     Xtra = [];
                  end;
                  [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,F,B0);
               else;
                  [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,F,B0);
               end;
               KsiTT = KsitT(:,g);
            else;
               if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                  Yerror = Yerror(:,T);
                  SigmaSqRoot = SigmaSqRoot(:,:,T);
                  InvMSEY = InvMSEY(:,:,T);
                  KalmanGain = KalmanGain(:,:,T);
               end;
               [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
               KsiTTz = KsitT(:,1);
               PTTz = PtT(:,:,1);
               KsiTT = KsitT(:,1);
            end;
            PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
            [S,L] = SchurDecomposition(PTTz);
         end;
         %
         % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
         %
         if rank(R)>0;
            [Rm,Lm] = SchurDecomposition(R);
            Rmeasure = Rm*sqrt(Lm);
            NumMeasure = size(Lm,1);
         else;
            Rmeasure = [];
            wTh = zeros(n,h);
            NumMeasure = 0;
         end;
         if AnnualizeData==0;
            if TransData==0;
               G = zeros(n*g,r);
               D = zeros(n*g,q*g);
               Fexp = F;
               Fg = eye(r);
               for i=1:g;
                  Ind = (1+((g-i)*n):(g-i+1)*n);
                  Ind1 = (1+((g-i)*q):(g-i+1)*q);
                  G(Ind,:) = H'*Fexp;
                  D(Ind,Ind1) = H'*B0;
                  Fi = F;
                  for j=1:i-1;
                     Ind1 = Ind1+q;
                     D(Ind,Ind1) = H'*Fi*B0;
                     Fi = F*Fi;
                  end;
                  Fexp = F*Fexp;
               end;
               %
               % compute matrices for the predictive covariance matrix
               %
               InvKDIgM = eye(mg)/(K'*D*IgM);
               Dtilde = eye(n*g)-(D*IgM*InvKDIgM*K');
               CovShock = Dtilde*D*IgMMorth*D'*Dtilde';
               CovState = Dtilde*G*PTTz*G'*Dtilde';
               if rank(R)>0;
                  CovME = Dtilde*kron(eye(g),R)*Dtilde';
               end;
               Ntilde = eye(q*g)-(IgM*InvKDIgM*K'*D);
               SigmaTg = Ntilde*IgMMorth*Ntilde';
               Bbar = zeros(r,q*g);
            end;
            %
            % initialize output
            %
            PredictY = zeros(n,h);
            PredictYC = PredictY;
            Fi = eye(r);
            Q = B0*B0';
            %
            % initialize the state uncertainty for modesty statistics
            %
            PTTg = zeros(r,r);
            PTTgLZ = PTTg;
            for i=1:g;
               %
               % update Fi
               %
               Fi = F*Fi;
               if k>0;
                  PredictY(:,i) = (A'*X(:,i))+(H'*Fi*KsiTT);
                  PredictYC(:,i) = (A'*X(:,i))+(H'*Fi*KsiTTz);
               else;
                  PredictY(:,i) = H'*Fi*KsiTT;
                  PredictYC(:,i) = H'*Fi*KsiTTz;
               end;
               PTTg = (F*PTTg*F')+Q;
               ZPosi = ZPos(:,i);
               PTTgLZ = (F*PTTgLZ*F')+(B0*M(:,ZPosi)*M(:,ZPosi)'*B0');
            end;
         end;
         %
         % compute the mean prediction at the given parameter values
         %
         PredMean = zeros(n,h);
         KsiTm = KsiTTz;
         etaShocksm = zeros(q,h);
         K2summ = zeros(m,1);
         for i=1:g;
            %
            % compute the shock values that need to be manipulated
            %
            ZPosi = ZPos(:,i);
            m1 = sum(ZPosi);
            if m1>0;
               if k>0;
                  etaShocksm(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*A'*X(:,i)) - (K1(:,ZPosi)'*H'*F*KsiTm) - K2summ(ZPosi) -U(ZPosi,g));
               else;
                  etaShocksm(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*H'*F*KsiTm) - K2summ(ZPosi) -U(ZPosi,g));
               end;
            end;
            %
            % update the states
            %
            KsiTm = (F*KsiTm)+(B0*etaShocksm(:,i));
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H'*KsiTm);
            else;
               PredMean(:,i) = H'*KsiTm;
            end;
            %
            % update the K2sum measure
            %
            if K2length==2;
               K2summ = K2summ+(K2'*PredMean(:,i));
            else;
               K2summ = zeros(m,1);
               for j=1:i;
                  if j<=size(K2,3);
                     K2summ = K2summ+(K2(:,:,j)'*PredMean(:,i+1-j));
                  end;
               end;
            end;
            if (AnnualizeData==0)&(TransData==0);
               Ind1 = (1+((g-i)*n):(g-i+1)*n);
               Ind2 = (1+(i-1)*q):(i*q);
               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)'));
               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
               if rank(R)>0;
                  MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)'));
               end;
               Bbar(:,Ind2) = Fg*B0;
               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
               Fg = Fg*F;
            end;
         end;
         for i=g+1:h;
            %
            % update the states
            %
            KsiTm = (F*KsiTm);
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H'*KsiTm);
            else;
               PredMean(:,i) = H'*KsiTm;
            end;
            if (AnnualizeData==0)&(TransData==0);
               if i==g+1;
                  Ktilde = Bbar*IgM*InvKDIgM*K';
                  Gtilde = Fg-(Ktilde*G);
                  Fig = eye(r);
                  FjSum = zeros(r,r);
               end;
               FjSum = FjSum+(Fig*Q*Fig');
               Fig = Fig*F;
               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fig*Gtilde*PTTz*Gtilde'*Fig'*H);
               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H);
               if rank(R)>0;
                  MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*((H'*Fig*Ktilde*kron(eye(g),R)*Ktilde'*Fig'*H)+R));
               end;
               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            end;
         end;
         if (AnnualizeData==0)&(TransData==0);
            MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredMean);
         end;
         ShockMean = ShockMean+((Weights(ParamIndex)/NumDraws)*etaShocksm);
         %
         % we are now ready to compute possible paths at theta
         %
         PredPaths = zeros(n,h,NumPaths);
         LZPredPaths = zeros(n,g);
         if AnnualizeData==0;
            MultiModestyStat = zeros(NumPaths,2);
            UniModestyStat = zeros(NumPaths,n);
            UniModestyStatLZ = zeros(1,n);
            if TransData==1;
               %
               % check if we need to fix Yinitial
               %
               if maxYinitial==1;
                  YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
                  nanpos = isnan(YSmooth);
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  if DSGEModel.k>0;
                     YHistPred = (A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))+(H'*KsitT);
                  else;
                     YHistPred = H'*KsitT;
                  end;
                  YSmooth(nanpos) = YHistPred(nanpos);
                  if DSGEModel.FirstPeriod>1;
                     YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod) YSmooth];
                  end;
                  Yinitial = YSmooth;
               end;
               YTmp = [Yinitial PredMean];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredMean = YData(Tt-h+1:Tt,:)';
            end;
         else;
            %
            % check if we need to fix Yinit
            %
            if maxYinit==1;
               YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
               nanpos = isnan(YSmooth);
               [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
               if DSGEModel.k>0;
                  YHistPred = (A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))+(H'*KsitT);
               else;
                  YHistPred = H'*KsitT;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if DSGEModel.FirstPeriod>1;
                  YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod) YSmooth];
               end;
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  Yinit = YSmooth(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  Yinit = YSmooth(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
               else;
                  Yinit = [];
               end;
            end;
            PredMean = Annualize([Yinit PredMean],DSGEModel.annual,DSGEModel.annualscale);
            MultiModestyStat = [];
            UniModestyStat = [];
            UniModestyStatLZ = [];
         end;
         %
         % start the loop over the paths
         %
         for it=1:NumPaths;
            CurrPath = it+(NumPaths*(ParamIndex-1));
            %
            % check if we should update the progress dialog
            %
            if DSGEModel.ShowProgress==1;
               abort = get(WaitHandle,'UserData');
               if strcmp(abort,'cancel')==1;
                  StopEstimation = 1;
                  break;
               else;
                  ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
               end;
            end;
            if AnnualizeData==0;
               PredictYShocks = zeros(r,h);
            end;
            %
            % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
            %
            KsiTinit = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
            %
            % Draw a sequence of economic shocks and set the conditioning shocks to zero
            %
            etaTh = randn(q,h);
            etaTmp = etaTh(:,1:g);
            %etaTh(DSGEModel.ConditionalShocks,1:g) = zeros(length(DSGEModel.ConditionalShocks),g);
            etaCTmp = etaTh(:,1:g);
            etaLZ = zeros(q,h);
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
            end;
            %
            % initialize the K2 part
            %
            K2sum = zeros(m,1);
            K2sumLZ = zeros(m,1);
            K2sumCTmp = zeros(m,1);
            KsiT = KsiTinit;
   %         KsiTLZ = KsiTinit;
            KsiTLZ = KsiTT;
            KsiCTmp = KsiTT;
   %         KsiTmp = KsiTinit;
            KsiTmp = KsiTT;
            PredCTmp = zeros(n,g);
            %
            % start the loop that calculates shocks, states, and, observed variables
            %
            for i=1:g;
               ZPosi = ZPos(:,i);
               m1 = sum(ZPosi);
               if m1>0;
                  etaTh(DSGEModel.ConditionalShocks(ZPosi),i) = 0;
                  etaCTmp(DSGEModel.ConditionalShocks(ZPosi),i) = 0;
                  if k>0;
                     etaTh(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*A'*X(:,i)) - (K1(:,ZPosi)'*wTh(:,i)) - (K1(:,ZPosi)'*H'*F*KsiT) - (K1(:,ZPosi)'*H'*B0*etaTh(:,i)) - K2sum(ZPosi) -U(ZPosi,g));
                     etaCTmp(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*A'*X(:,i)) - (K1(:,ZPosi)'*wTh(:,i)) - (K1(:,ZPosi)'*H'*F*KsiCTmp) - (K1(:,ZPosi)'*H'*B0*etaCTmp(:,i)) - K2sumCTmp(ZPosi) -U(ZPosi,g));
                  else;
                     etaTh(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*wTh(:,i)) - (K1(:,ZPosi)'*H'*F*KsiT) - (K1(:,ZPosi)'*H'*B0*etaTh(:,i)) - K2sum(ZPosi) -U(ZPosi,g));
                     etaCTmp(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*wTh(:,i)) - (K1(:,ZPosi)'*H'*F*KsiCTmp) - (K1(:,ZPosi)'*H'*B0*etaCTmp(:,i)) - K2sumCTmp(ZPosi) -U(ZPosi,g));
                  end;
               end;
               %
               % recompute the state
               %
               KsiT = (F*KsiT)+(B0*etaTh(:,i));
               KsiCTmp = (F*KsiT)+(B0*etaCTmp(:,i));
               KsiTmp = (F*KsiTmp)+(B0*etaTmp(:,i));
               %
               % compute the observed variables
               %
               if k>0;
                  PredPaths(:,i,it) = (A'*X(:,i))+(H'*KsiT)+wTh(:,i);
                  PredCTmp(:,i) = (A'*X(:,i))+(H'*KsiCTmp)+wTh(:,i);
               else;
                  PredPaths(:,i,it) = (H'*KsiT)+wTh(:,i);
                  PredCTmp(:,i) = (H'*KsiCTmp)+wTh(:,i);
               end;
               %
               % update the sum of K2 times lagged values of the observed variables
               %
               if K2length==2;
                  K2sum = K2sum+(K2'*PredPaths(:,i,it));
                  if k>0;
                     K2sumCTmp = K2sumCTmp+(K2'*((A'*X(:,i))+(H'*KsiCTmp)+wTh(:,i)));
                  else;
                     K2sumCTmp = K2sumCTmp+(K2'*((H'*KsiCTmp)+wTh(:,i)));
                  end;
               else;
                  K2sum = zeros(m,1);
                  K2sumCTmp = K2sum;
                  for j=1:i;
                     if j<=size(K2,3);
                        K2sum = K2sum+(K2(:,:,j)'*PredPaths(:,i+1-j,it));
                        K2sumCTmp = K2sumCTmp+(K2(:,:,j)'*PredCTmp(:,i+1-j));
                     end;
                  end;
               end;
               %
               % deal with Leeper-Zha matters
               %
               if it==1;
                  if m1>0;
                     if k>0;
                        etaLZ(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*A'*X(:,i)) - (K1(:,ZPosi)'*H'*F*KsiTLZ) - K2sumLZ(ZPosi) -U(ZPosi,g));
                     else;
                        etaLZ(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*H'*F*KsiTLZ) - K2sumLZ(ZPosi) -U(ZPosi,g));
                     end;
                  end;
                  KsiTLZ = (F*KsiTLZ)+(B0*etaLZ(:,i));
                  if k>0;
                     LZPredPaths(:,i) = (A'*X(:,i))+(H'*KsiTLZ);
                  else;
                     LZPredPaths(:,i) = H'*KsiTLZ;
                  end;
                  if K2length==2;
                     K2sumLZ = K2sumLZ+(K2'*LZPredPaths(:,i));
                  else;
                     K2sumLZ = zeros(m,1);
                     for j=1:i;
                        if j<=size(K2,3);
                           K2sumLZ = K2sumLZ+(K2(:,:,j)'*LZPredPaths(:,i+1-j));
                        end;
                     end;
                  end;
               end;
               %
               % compute the average of the shocks that we shall add to PredictY(:,i)
               %
               if AnnualizeData==0;
                  if i==1;
                     PredictYShocks(:,i) = PredictYShocks(:,i)+(B0*M*etaTh(DSGEModel.ConditionalShocks,i));
                  else;
                     PredictYShocks(:,i) = PredictYShocks(:,i)+(F*PredictYShocks(:,i-1))+((B0*M*etaTh(DSGEModel.ConditionalShocks,i)));
                  end;
                  %
                  % compute modesty statistics
                  %
                  if i==g;
                     if k>0;
                        PredPathCTmp = (A'*X(:,g))+(H'*KsiCTmp)+wTh(:,g);
                     else;
                        PredPathCTmp = (H'*KsiCTmp)+wTh(:,g);
                     end;
                     PredDiff = PredPathCTmp-PredictY(:,g);
                     PredCov = (H'*PTTg*H)+R;
                     MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
                     UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
                     %
                     % comparison-path for multivariate statistic
                     %
                     if k>0;
                        PredPathTmp = (A'*X(:,g))+(H'*KsiTmp)+wTh(:,g);
                     else;
                        PredPathTmp = (H'*KsiTmp)+wTh(:,g);
                     end;
                     PredDiffTmp = PredPathTmp-PredictY(:,g);
                     MultiModestyStat(it,2) = PredDiffTmp'*(PredCov\PredDiffTmp);
                     %
                     % Leeper-Zha
                     %
                     PredDiff = LZPredPaths(:,g)-PredictY(:,g);
                     if it==1;
                        UniModestyStatLZ(1,:) = (PredDiff ./ sqrt(diag(H'*PTTgLZ*H)))';
                     end;
                  end;
               end;
            end;
            %
            % if h>g we need to run the loop
            %
            for i=g+1:h;
               KsiT = (F*KsiT)+(B0*etaTh(:,i));
               if k>0;
                  PredPaths(:,i,it) = (A'*X(:,i))+(H'*KsiT)+wTh(:,i);
               else;
                  PredPaths(:,i,it) = (H'*KsiT)+wTh(:,i);
               end;
               if AnnualizeData==0;
                  PredictYShocks(:,i) = F*PredictYShocks(:,i-1);
               end;
            end;
            %
            if AnnualizeData==0;
               if TransData==1;
                  YTmp = [Yinitial PredPaths(:,:,it)];
                  [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                  PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
               end;
            else;
               %
               % we should annualize the conditional prediction path
               %
               PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
            end;
            drawnow;
         end;
         if StopEstimation==1;
            break;
         end;
         %
         if DSGEModel.AdjustPredictionPaths==1;
            %
            % we need to fix the paths such that the sample mean is equal to the population mean
            %
            MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
            AdjustMean = PredMean-MeanPrediction;
            for i=1:NumPaths;
               PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
            end;
         end;
         %
         % save the calculations to disk
         %
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Original-Direct-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Transformed-Direct-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Annual-Direct-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Original-Prior-Direct-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Transformed-Prior-Direct-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Annual-Prior-Direct-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         end;
         save(file,'PredPaths','ShockMean','ShockNames','MultiModestyStat','UniModestyStat','UniModestyStatLZ');
         %
         if DSGEModel.RunPredictionEvent==1;
            %
            % compute the prediction events results
            %
            PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
         end;
      end;
   end;
else;
   for ParamIndex=1:NumDraws;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % solve the mode for initial parameter values or for the posterior mode values
         %
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
      %
      % check if we can use these values.
      %
      BadRank = 0;
      for i=1:g;
         etaMat = K1'*H(:,:,DSGEModel.LastPeriod+i)'*B0*M;
         if rank(etaMat)<m;
            BadRank = 1;
            break;
         end;
      end;
      if BadRank==0;
         %
         % We can now run the Kalman filter
         %
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
         %
         % Now we need to calculate smooth values for the states for g periods
         %
         if ParamIndex==1;
            [r,T] = size(Ksitt1);
            q = size(B0,2);
         end;
         %
         % calculate measurement errors for the sample T-g+1 until T
         %
         if (DSGEModel.KsiUseCondData==1)&(SCond==1);
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T);
               SigmaSqRoot = SigmaSqRoot(:,:,T);
               InvMSEY = InvMSEY(:,:,T);
               KalmanGain = KalmanGain(:,:,T);
            end;
            [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            [KsiTTz,PTTz] = CondPredictionSmootherHt(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H(:,:,DSGEModel.LastPeriod:DSGEModel.LastPeriod+g),R,F,B0,DSGEModel.KalmanAlgorithm);
            KsiTT = KsitT(:,1);
         elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T-g+1:T);
               SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
               InvMSEY = InvMSEY(:,:,T-g+1:T);
               KalmanGain = KalmanGain(:,:,T-g+1:T);
            end;
            [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H(:,:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            wtT = zeros(n,g);
            if rank(R)>0;
               if k>0;
                  for i=1:g;
                     wtT(:,i) = Y(:,i)-(A'*Xdata(:,i))-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
                  end;
               else;
                  for i=1:g;
                     wtT(:,i) = Y(:,i)-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
                  end;
               end;
            end;
            %
            % Smooth estimation of the states at time T using the conditioning assumptions.
            %
            if k>0;
               if g>1;
                  Xtra = Xdata(:,T-g+2:T);
               else;
                  Xtra = [];
               end;
               [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
            else;
               [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
            end;
            KsiTT = KsitT(:,g);
         else;
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T);
               SigmaSqRoot = SigmaSqRoot(:,:,T);
               InvMSEY = InvMSEY(:,:,T);
               KalmanGain = KalmanGain(:,:,T);
            end;
            [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            KsiTTz = KsitT(:,1);
            PTTz = PtT(:,:,1);
            KsiTT = KsitT(:,1);
         end;
         PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
         %
         % use the Schur decomposition for the reduced rank matrix PTT instead of
         % the Choleski decomposition
         %
         [S,L] = SchurDecomposition(PTTz);
         if isreal(sqrt(L))==0;
            %
            % redo the calculations for posterior mode or initial values
            %
            [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
            [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
            if ParamIndex==1;
               [r,T] = size(Ksitt1);
               q = size(B0,2);
            end;
            if (DSGEModel.KsiUseCondData==1)&(SCond==1);
               if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                  Yerror = Yerror(:,T);
                  SigmaSqRoot = SigmaSqRoot(:,:,T);
                  InvMSEY = InvMSEY(:,:,T);
                  KalmanGain = KalmanGain(:,:,T);
               end;
               [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
               [KsiTTz,PTTz] = CondPredictionSmootherHt(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H(:,:,DSGEModel.LastPeriod:DSGEModel.LastPeriod+g),R,F,B0,DSGEModel.KalmanAlgorithm);
               KsiTT = KsitT(:,1);
            elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
               if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                  Yerror = Yerror(:,T-g+1:T);
                  SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
                  InvMSEY = InvMSEY(:,:,T-g+1:T);
                  KalmanGain = KalmanGain(:,:,T-g+1:T);
               end;
               [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H(:,:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
               wtT = zeros(n,g);
               if rank(R)>0;
                  if k>0;
                     for i=1:g;
                        wtT(:,i) = Y(:,i)-(A'*Xdata(:,i))-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
                     end;
                  else;
                     for i=1:g;
                        wtT(:,i) = Y(:,i)-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
                     end;
                  end;
               end;
               if k>0;
                  if g>1;
                     Xtra = Xdata(:,T-g+2:T);
                  else;
                     Xtra = [];
                  end;
                  [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
               else;
                  [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
               end;
               KsiTT = KsitT(:,g);
            else;
               if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                  Yerror = Yerror(:,T);
                  SigmaSqRoot = SigmaSqRoot(:,:,T);
                  InvMSEY = InvMSEY(:,:,T);
                  KalmanGain = KalmanGain(:,:,T);
               end;
               [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
               KsiTTz = KsitT(:,1);
               PTTz = PtT(:,:,1);
               KsiTT = KsitT(:,1);
            end;
            PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
            [S,L] = SchurDecomposition(PTTz);
         end;
         %
         % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
         %
         if rank(R)>0;
            [Rm,Lm] = SchurDecomposition(R);
            Rmeasure = Rm*sqrt(Lm);
            NumMeasure = size(Lm,1);
         else;
            Rmeasure = [];
            wTh = zeros(n,h);
            NumMeasure = 0;
         end;
         if AnnualizeData==0;
            if TransData==0;
               G = zeros(n*g,r);
               D = zeros(n*g,q*g);
               Fexp = F;
               Fg = eye(r);
               for i=1:g;
                  Ind = (1+((g-i)*n):(g-i+1)*n);
                  Ind1 = (1+((g-i)*q):(g-i+1)*q);
                  G(Ind,:) = H(:,:,DSGEModel.LastPeriod+i)'*Fexp;
                  D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*B0;
                  Fi = F;
                  for j=1:i-1;
                     Ind1 = Ind1+q;
                     D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*Fi*B0;
                     Fi = F*Fi;
                  end;
                  Fexp = F*Fexp;
               end;
               %
               % compute matrices for the predictive covariance matrix
               %
               InvKDIgM = eye(mg)/(K'*D*IgM);
               Dtilde = eye(n*g)-(D*IgM*InvKDIgM*K');
               CovShock = Dtilde*D*IgMMorth*D'*Dtilde';
               CovState = Dtilde*G*PTTz*G'*Dtilde';
               if rank(R)>0;
                  CovME = Dtilde*kron(eye(g),R)*Dtilde';
               end;
               Ntilde = eye(q*g)-(IgM*InvKDIgM*K'*D);
               SigmaTg = Ntilde*IgMMorth*Ntilde';
               Bbar = zeros(r,q*g);
            end;
            %
            % initialize output
            %
            PredictY = zeros(n,h);
            PredictYC = PredictY;
            Fi = eye(r);
            Q = B0*B0';
            %
            % initialize the state uncertainty for modesty statistics
            %
            PTTg = zeros(r,r);
            PTTgLZ = PTTg;
            for i=1:g;
               %
               % update Fi
               %
               Fi = F*Fi;
               if k>0;
                  PredictY(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*Fi*KsiTT);
                  PredictYC(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*Fi*KsiTTz);
               else;
                  PredictY(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*Fi*KsiTT;
                  PredictYC(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*Fi*KsiTTz;
               end;
               PTTg = (F*PTTg*F')+Q;
               ZPosi = ZPos(:,i);
               PTTgLZ = (F*PTTgLZ*F')+(B0*M(:,ZPosi)*M(:,ZPosi)'*B0');
            end;
         end;
         %
         % compute the mean prediction at the given parameter values
         %
         PredMean = zeros(n,h);
         KsiTm = KsiTTz;
         etaShocksm = zeros(q,h);
         K2summ = zeros(m,1);
         for i=1:g;
            %
            % compute the shock values that need to be manipulated
            %
            ZPosi = ZPos(:,i);
            m1 = sum(ZPosi);
            if m1>0;
               if k>0;
                  etaShocksm(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*A'*X(:,i)) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*F*KsiTm) - K2summ(ZPosi) -U(ZPosi,g));
               else;
                  etaShocksm(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*F*KsiTm) - K2summ(ZPosi) -U(ZPosi,g));
               end;
            end;
            %
            % update the states
            %
            KsiTm = (F*KsiTm)+(B0*etaShocksm(:,i));
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiTm);
            else;
               PredMean(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiTm;
            end;
            %
            % update the K2sum measure
            %
            if K2length==2;
               K2summ = K2summ+(K2'*PredMean(:,i));
            else;
               K2summ = zeros(m,1);
               for j=1:i;
                  if j<=size(K2,3);
                     K2summ = K2summ+(K2(:,:,j)'*PredMean(:,i+1-j));
                  end;
               end;
            end;
            if (AnnualizeData==0)&(TransData==0);
               Ind1 = (1+((g-i)*n):(g-i+1)*n);
               Ind2 = (1+(i-1)*q):(i*q);
               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)'));
               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
               if rank(R)>0;
                  MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)'));
               end;
               Bbar(:,Ind2) = Fg*B0;
               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
               Fg = Fg*F;
            end;
         end;
         for i=g+1:h;
            %
            % update the states
            %
            KsiTm = (F*KsiTm);
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiTm);
            else;
               PredMean(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiTm;
            end;
            if (AnnualizeData==0)&(TransData==0);
               if i==g+1;
                  Ktilde = Bbar*IgM*InvKDIgM*K';
                  Gtilde = Fg-(Ktilde*G);
                  Fig = eye(r);
                  FjSum = zeros(r,r);
               end;
               FjSum = FjSum+(Fig*Q*Fig');
               Fig = Fig*F;
               StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,DSGEModel.LastPeriod+i)'*Fig*Gtilde*PTTz*Gtilde'*Fig'*H(:,:,DSGEModel.LastPeriod+i));
               ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,DSGEModel.LastPeriod+i)'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H(:,:,DSGEModel.LastPeriod+i));
               if rank(R)>0;
                  MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*((H(:,:,DSGEModel.LastPeriod+i)'*Fig*Ktilde*kron(eye(g),R)*Ktilde'*Fig'*H(:,:,DSGEModel.LastPeriod+i))+R));
               end;
               ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            end;
         end;
         if (AnnualizeData==0)&(TransData==0);
            MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredMean);
         end;
         ShockMean = ShockMean+((Weights(ParamIndex)/NumDraws)*etaShocksm);
         %
         % we are now ready to compute possible paths at theta
         %
         PredPaths = zeros(n,h,NumPaths);
         LZPredPaths = zeros(n,g);
         if AnnualizeData==0;
            MultiModestyStat = zeros(NumPaths,2);
            UniModestyStat = zeros(NumPaths,n);
            UniModestyStatLZ = zeros(1,n);
            if TransData==1;
               %
               % check if we need to fix Yinitial
               %
               if maxYinitial==1;
                  YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
                  nanpos = isnan(YSmooth);
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  if DSGEModel.k>0;
                     YHistPred = A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
                     for t=1:size(KsitT,2);
                        YHistPred(:,t) = YHistPred(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
                     end;
                  else;
                     YHistPred = zeros(n,size(KsitT,2));
                     for t=1:size(KsitT,2);
                        YHistPred(:,t) = H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t);
                     end;
                  end;
                  YSmooth(nanpos) = YHistPred(nanpos);
                  if DSGEModel.FirstPeriod>1;
                     YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod) YSmooth];
                  end;
                  Yinitial = YSmooth;
               end;
               YTmp = [Yinitial PredMean];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredMean = YData(Tt-h+1:Tt,:)';
            end;
         else;
            %
            % check if we need to fix Yinit
            %
            if maxYinit==1;
               YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
               nanpos = isnan(YSmooth);
               [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
               if DSGEModel.k>0;
                  YHistPred = A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
                  for t=1:size(KsitT,2);
                     YHistPred(:,t) = YHistPred(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
                  end;
               else;
                  YHistPred = zeros(n,size(KsitT,2));
                  for t=1:size(KsitT,2);
                     YHistPred(:,t) = H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t);
                  end;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if DSGEModel.FirstPeriod>1;
                  YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod) YSmooth];
               end;
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  Yinit = YSmooth(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  Yinit = YSmooth(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
               else;
                  Yinit = [];
               end;
            end;
            PredMean = Annualize([Yinit PredMean],DSGEModel.annual,DSGEModel.annualscale);
            MultiModestyStat = [];
            UniModestyStat = [];
            UniModestyStatLZ = [];
         end;
         %
         % start the loop over the paths
         %
         for it=1:NumPaths;
            CurrPath = it+(NumPaths*(ParamIndex-1));
            %
            % check if we should update the progress dialog
            %
            if DSGEModel.ShowProgress==1;
               abort = get(WaitHandle,'UserData');
               if strcmp(abort,'cancel')==1;
                  StopEstimation = 1;
                  break;
               else;
                  ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
               end;
            end;
            if AnnualizeData==0;
               PredictYShocks = zeros(r,h);
            end;
            %
            % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
            %
            KsiTinit = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
            %
            % Draw a sequence of economic shocks and set the conditioning shocks to zero
            %
            etaTh = randn(q,h);
            etaTmp = etaTh(:,1:g);
            %etaTh(DSGEModel.ConditionalShocks,1:g) = zeros(length(DSGEModel.ConditionalShocks),g);
            etaCTmp = etaTh(:,1:g);
            etaLZ = zeros(q,h);
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
            end;
            %
            % initialize the K2 part
            %
            K2sum = zeros(m,1);
            K2sumLZ = zeros(m,1);
            K2sumCTmp = zeros(m,1);
            KsiT = KsiTinit;
   %         KsiTLZ = KsiTinit;
            KsiTLZ = KsiTT;
            KsiCTmp = KsiTT;
   %         KsiTmp = KsiTinit;
            KsiTmp = KsiTT;
            PredCTmp = zeros(n,g);
            %
            % start the loop that calculates shocks, states, and, observed variables
            %
            for i=1:g;
               etaMat = K1'*H(:,:,DSGEModel.LastPeriod+i)'*B0*M;
               ZPosi = ZPos(:,i);
               m1 = sum(ZPosi);
               if m1>0;
                  etaTh(DSGEModel.ConditionalShocks(ZPosi),i) = 0;
                  etaCTmp(DSGEModel.ConditionalShocks(ZPosi),i) = 0;
                  if k>0;
                     etaTh(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*A'*X(:,i)) - (K1(:,ZPosi)'*wTh(:,i)) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*F*KsiT) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*B0*etaTh(:,i)) - K2sum(ZPosi) -U(ZPosi,g));
                     etaCTmp(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*A'*X(:,i)) - (K1(:,ZPosi)'*wTh(:,i)) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*F*KsiCTmp) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*B0*etaCTmp(:,i)) - K2sumCTmp(ZPosi) -U(ZPosi,g));
                  else;
                     etaTh(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*wTh(:,i)) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*F*KsiT) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*B0*etaTh(:,i)) - K2sum(ZPosi) -U(ZPosi,g));
                     etaCTmp(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*wTh(:,i)) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*F*KsiCTmp) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*B0*etaCTmp(:,i)) - K2sumCTmp(ZPosi) -U(ZPosi,g));
                  end;
               end;
               %
               % recompute the state
               %
               KsiT = (F*KsiT)+(B0*etaTh(:,i));
               KsiCTmp = (F*KsiCTmp)+(B0*etaCTmp(:,i));
               KsiTmp = (F*KsiTmp)+(B0*etaTmp(:,i));
               %
               % compute the observed variables
               %
               if k>0;
                  PredPaths(:,i,it) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
                  PredCTmp(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiCTmp)+wTh(:,i);
               else;
                  PredPaths(:,i,it) = (H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
                  PredCTmp(:,i) = (H(:,:,DSGEModel.LastPeriod+i)'*KsiCTmp)+wTh(:,i);
               end;
               %
               % update the sum of K2 times lagged values of the observed variables
               %
               if K2length==2;
                  K2sum = K2sum+(K2'*PredPaths(:,i,it));
                  if k>0;
                     K2sumCTmp = K2sumCTmp+(K2'*((A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiCTmp)+wTh(:,i)));
                  else;
                     K2sumCTmp = K2sumCTmp+(K2'*((H(:,:,DSGEModel.LastPeriod+i)'*KsiCTmp)+wTh(:,i)));
                  end;
               else;
                  K2sum = zeros(m,1);
                  K2sumCTmp = K2sum;
                  for j=1:i;
                     if j<=size(K2,3);
                        K2sum = K2sum+(K2(:,:,j)'*PredPaths(:,i+1-j,it));
                        K2sumCTmp = K2sumCTmp+(K2(:,:,j)'*PredCTmp(:,i+1-j));
                     end;
                  end;
               end;
               %
               % deal with Leeper-Zha matters
               %
               if it==1;
                  if m1>0;
                     if k>0;
                        etaLZ(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*A'*X(:,i)) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*F*KsiTLZ) - K2sumLZ(ZPosi) -U(ZPosi,g));
                     else;
                        etaLZ(DSGEModel.ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*F*KsiTLZ) - K2sumLZ(ZPosi) -U(ZPosi,g));
                     end;
                  end;
                  KsiTLZ = (F*KsiTLZ)+(B0*etaLZ(:,i));
                  if k>0;
                     LZPredPaths(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiTLZ);
                  else;
                     LZPredPaths(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiTLZ;
                  end;
                  if K2length==2;
                     K2sumLZ = K2sumLZ+(K2'*LZPredPaths(:,i));
                  else;
                     K2sumLZ = zeros(m,1);
                     for j=1:i;
                        if j<=size(K2,3);
                           K2sumLZ = K2sumLZ+(K2(:,:,j)'*LZPredPaths(:,i+1-j));
                        end;
                     end;
                  end;
               end;
               %
               % compute the average of the shocks that we shall add to PredictY(:,i)
               %
               if AnnualizeData==0;
                  if i==1;
                     PredictYShocks(:,i) = PredictYShocks(:,i)+(B0*M*etaTh(DSGEModel.ConditionalShocks,i));
                  else;
                     PredictYShocks(:,i) = PredictYShocks(:,i)+(F*PredictYShocks(:,i-1))+((B0*M*etaTh(DSGEModel.ConditionalShocks,i)));
                  end;
                  %
                  % compute modesty statistics
                  %
                  if i==g;
                     if k>0;
                        PredPathCTmp = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*KsiCTmp)+wTh(:,g);
                     else;
                        PredPathCTmp = (H(:,:,DSGEModel.LastPeriod+g)'*KsiCTmp)+wTh(:,g);
                     end;
                     PredDiff = PredPathCTmp-PredictY(:,g);
                     PredCov = (H(:,:,DSGEModel.LastPeriod+g)'*PTTg*H(:,:,DSGEModel.LastPeriod+g))+R;
                     MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
                     UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
                     %
                     % comparison-path for multivariate statistic
                     %
                     if k>0;
                        PredPathTmp = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*KsiTmp)+wTh(:,g);
                     else;
                        PredPathTmp = (H(:,:,DSGEModel.LastPeriod+g)'*KsiTmp)+wTh(:,g);
                     end;
                     PredDiffTmp = PredPathTmp-PredictY(:,g);
                     MultiModestyStat(it,2) = PredDiffTmp'*(PredCov\PredDiffTmp);
                     %
                     % Leeper-Zha
                     %
                     PredDiff = LZPredPaths(:,g)-PredictY(:,g);
                     if it==1;
                        UniModestyStatLZ(1,:) = (PredDiff ./ sqrt(diag(H(:,:,DSGEModel.LastPeriod+g)'*PTTgLZ*H(:,:,DSGEModel.LastPeriod+g))))';
                     end;
                  end;
               end;
            end;
            %
            % if h>g we need to run the loop
            %
            for i=g+1:h;
               KsiT = (F*KsiT)+(B0*etaTh(:,i));
               if k>0;
                  PredPaths(:,i,it) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
               else;
                  PredPaths(:,i,it) = (H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
               end;
               if AnnualizeData==0;
                  PredictYShocks(:,i) = F*PredictYShocks(:,i-1);
               end;
            end;
            %
            if AnnualizeData==0;
               if TransData==1;
                  YTmp = [Yinitial PredPaths(:,:,it)];
                  [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                  PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
               end;
            else;
               %
               % we should annualize the conditional prediction path
               %
               PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
            end;
            drawnow;
         end;
         if StopEstimation==1;
            break;
         end;
         %
         if DSGEModel.AdjustPredictionPaths==1;
            %
            % we need to fix the paths such that the sample mean is equal to the population mean
            %
            MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
            AdjustMean = PredMean-MeanPrediction;
            for i=1:NumPaths;
               PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
            end;
         end;
         %
         % save the calculations to disk
         %
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Original-Direct-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Transformed-Direct-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Annual-Direct-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Original-Prior-Direct-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Transformed-Prior-Direct-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Annual-Prior-Direct-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         end;
         save(file,'PredPaths','ShockMean','ShockNames','MultiModestyStat','UniModestyStat','UniModestyStatLZ');
         %
         if DSGEModel.RunPredictionEvent==1;
            %
            % compute the prediction events results
            %
            PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
         end;
      end;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
%
% make sure the parameter uncertainty is corrected for the mean
%
DoneCalc = 1;
if isfield(DSGEModel,'PredictionEvent')==1;
   PredictionEvent = DSGEModel.PredictionEvent;
else;
   PredictionEvent = [Inf*ones(n,1) zeros(n,1) ones(n,1)];
end;
if (AnnualizeData==0)&(TransData==0);
   for i=1:h;
      ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
      ParameterCov(:,:,i) = diag(abs(diag(ParameterCov(:,:,i))))+tril(ParameterCov(:,:,i),-1)+tril(ParameterCov(:,:,i),-1)';
   end;
   save(file,'PredEventData','PredictionEvent','StateCov','ShockCov','MeasureCov','ParameterCov','-APPEND');
else;
   save(file,'PredEventData','PredictionEvent','-APPEND');
end;

%
% end of DSGECondPredictionPaths.m
%
