function ES = MultiCRPS(Xmatrix,xvec,NumWorkers)
% MultiCRPS: Computes the energy score/multivariate continuous ranked probability score (CRPS)
%            for vector paths and a vector of values (reference point); based on beta=1 in
%            Gneiting and Raftery (2007) and the negative of the score, as in Gneiting,
%            Stanberry, Grimit, Held and Johnson (2008).
%
% USAGE:
%
%       ES = MultiCRPS(Xmatrix,xvec,NumWorkers)
%
% REQUIRED INPUT:  Xmatrix (matrix) of dimension n x T with T paths for the n-dimensional
%                          vector.
%
%                  xvec (vector) of dimension n x 1 with the reference values for the energy score.
%
%                  NumWorkers (integer) with the number of available cores for parallel
%                         computations.
%
% REQUIRED OUTPUT: ES (scalar) with the energy score (multivariate CRPS).
%
%
% References: See Gneiting, T. and A.E. Raftery (2007), "Strictly Proper Scoring Rules,
%                 Prediction, and Estimation", JASA, 102(407), 359-378.
%
%                 Gneiting, T., Stanberry, L.I., Grimit, E.P., Held, L. and N.A. Johnson
%                 (2008), "Assessing Probabilistic Forecasts of Multivariate Quantities, with
%                 and Application to Ensemble Predictions of Surface Winds", TEST, 17(2), 211-235.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: July 22, 2020.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 23-07-2020: Updated the documentation.
%
% * 30-07-2020: Added support for matlab version without the vecnorm function.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 11-02-2022: Added support for parallel computations.
%
% * 25-02-2022: Changed the definition of ES to match Gneiting & Raftery (JASA, 2007).
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[n,T] = size(Xmatrix);
if exist('vecnorm')~=0;
   ES1 = (1/T)*sum(vecnorm(Xmatrix-repmat(xvec,1,T),2,1));
   n2 = 2*(T*T);
   if NumWorkers<2;
      ES2 = 0;
      for j=1:T;
         ES2 = ES2+((1/n2)*sum(vecnorm(repmat(Xmatrix(:,j),1,T)-Xmatrix,2,1)));
      end;
   else;
      ESVector = zeros(1,T);
      parfor j=1:T;
         ESVector(j) = sum(vecnorm(repmat(Xmatrix(:,j),1,T)-Xmatrix,2,1));
      end;
      ES2 = (1/n2)*sum(ESVector);
   end;
else;
   ES1 = (1/T)*sum(VectorEuclideanNorm(Xmatrix-repmat(xvec,1,T)));
   n2 = 2*(T*T);
   if NumWorkers<2;
      ES2 = 0;
      for j=1:T;
         ES2 = ES2+((1/n2)*sum(VectorEuclideanNorm(repmat(Xmatrix(:,j),1,T)-Xmatrix)));
      end;
   else;
      ESVector = zeros(1,T);
      parfor j=1:T;
         ESVector(j) = sum(VectorEuclideanNorm(repmat(Xmatrix(:,j),1,T)-Xmatrix));
      end;
      ES2 = (1/n2)*sum(ESVector);
   end;
end;
%ES = ES1-ES2;
ES = ES2-ES1;

%
% end of MultiCRPS.m
%
