function LearningMenuFunctions(selector,maingui)
% LearningMenuFunctions: switch function for running the functions on the learning menu.
%
% USAGE:
%
%       LearningMenuFunctions(selector,maingui)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
% OPTIONAL INPUT: maingui (handle) to the YADA GUI.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 5, 2020.
%                        This version: March 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-03-2020: Added code for the case "SetBeliefSystem".
%
% * 09-03-2020: Finished the "SetForwardLookingVariables" and "SetBeliefSystem" case functions.
%
% * 10-03-2020: Finished the "SetBeliefParametersPriorFile" case function.
%
% * 11-03-2020: Began work on the log-likelihood function at posterior mode values for the DSGE model.
%
% * 12-03-2020: Continued work on the log-likelihood function at DSGE posterior mode.
%
% * 17-03-2020: Added code for displaying information when BeliefStatus is 0 for the
%               log-likelihood function calculation.
%
% * 18-03-2020: Added rank checking of H1 when selecting the forward looking variables.
%
% * 19-03-2020: Added a transformation status check to the structural form (TransStatus) which is examined
%               by the function.
%
% * 30-03-2020: Finished the log-likelihood function for the DSGE posterior mode case and completed the
%               log-likelihood for the initial values case. Finished the code for the call to the
%               function which estimates the posterior mode of the adaptive learning DSGE model.
%
% * 03-04-2020: Added log-likelihood function for the posterior mode case as well as functions for
%               plotting estimates of the state variables and the state shocks at posterior mode.
%
% * 06-04-2020: Added function for plotting the belief ciefficients at posterior mode.
%
% * 07-04-2020: Fixed some bugs setting up the betaPersistence and betaGrowth variables.
%
% * 08-04-2020: Dealing with issues involving the need to reorder equations when transforming the
%               structural form.
%
% * 09-04-2020: It is now possible to select a greater number of forward looking variables than the
%               rank of the matrix on the expectations term of the structural form.
%
% * 16-04-2020: Added functions for viewing the log prior and the log Jacobian at initial values,
%               DSGE posterior mode and posterior mode. Started code for displaying the belief
%               coefficients at the initial values and at the DSGE posterior mode values.
%
% * 17-04-2020: Finished the code for the belief coefficients. Added functions for displaying the
%               state shocks and the state variables at the initial values and at the DSGE posterior
%               mode values.
%
% * 21-04-2020: Finished code for displaying the observed variable decomposition at posterior mode.
%               Began code for the initial values and DSGE posterior mode cases of observed variable
%               decompositions.
%
% * 22-04-2020: Finished coding the initial values and DSGE posterior mode cases for the observed
%               variable decompositions. Added function for displaying state variable decompositions
%               at initial values, DSGE posterior mode and posterior mode.
%
% * 04-05-2020: Took into account before plotting that the estimated belief coefficients are lagged
%               one period.
%
% * 06-05-2020: Added function cases for the impulse responses based on original data.
%
% * 08-05-2020: Added the function "NonCentralGLSMatrix".
%
% * 11-05-2020: Added the function "NoSmoothedStatesForBeliefs".
%
% * 12-05-2020: Continued work on the impulse responses.
%
% * 13-05-2020: Finished impulse responses at initial values and at the DSGE posterior mode values.
%
% * 14-05-2020: Query of running function at initial values is no longer asked when the posterior mode
%               file does not exist. Added functions for displaying the laws of motion of the forward
%               looking variables.
%
% * 15-05-2020: Finished the laws of motion functions for the initial values and the DSGE posterior
%               mode values.
%
% * 18-05-2020: Added the posterior sampling function.
%
% * 20-05-2020: Added calls to the slice sampler and the random walk Metropolis with a student-t
%               proposal density.
%
% * 22-05-2020: Added calls to the fixed block RWM with a normal and a Student-t proposal.
%
% * 25-05-2020: Added calls to the random blocking RWM with a normal and a Student-t proposal, as well
%               as to the SMC with likelihood tempering posterior sampler.
%
% * 26-05-2020: Added a call to the SMC with data tempering posterior sampler.
%
% * 27-05-2020: Added a call to the importance sampler MitISEM function. Added the prior sampling
%               function, and a plotting function for the raw posterior draws.
%
% * 28-05-2020: Finished the raw plots function. Added function to view the marginal and bivariate
%               posterior densities as well as the posterior correlations. Added the modified
%               harmonic mean estimation function.
%
% * 29-05-2020: Finished the Sims, Waggoner and Zha function.
%
% * 02-06-2020: Tracking a bug with the Laplace approximation code under posterior mode values.
%
% * 02-07-2020: Removed the field "NonCentralGLSMatrix" from DSGEModel.
%
% * 03-07-2020: Began work on unconditional predictive distribution for the posterior mode values.
%
% * 07-07-2020: Changed the case name for "UseFixedBeliefsForProjections" to "FixedBeliefsForProjections".
%               Finished the unconditional predictive distribution at posterior mode.
%
% * 08-07-2020: Added the unconditional predictive distribution at the DSGE posterior mode and at the
%               initial values.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 26-01-2021: Updated the documentation.
%
% * 28-01-2021: Added the function ModelSolver.
%
% * 28-01-2021: The function supports a user defined model solver under adaptive learning.
%
% * 08-02-2021: Added the check posterior mode, iterated parameter estimates and prior distribution
%               information functions.
%
% * 09-02-2021: Added the Laplace posterior densities and the posterior mode surface functions.
%
% * 06-04-2021: Made sure that "logSystemPriorPhiDSGE" receives the input variable "IsAdaptiveLearning".
%
% * 21-04-2021: Fixed a memory issue when computing posterior correlations.
%
% * 22-04-2021: Bug checking!
%
% * 28-04-2021: Updated the documentation.
%
% * 03-05-2021: Began work on estimating the predictive likelihood for posterior draws.
%
% * 04-05-2021: Continued work on the predictive likelihood function.
%
% * 05-05-2021: Today as well.
%
% * 06-05-2021: Added more code to the predictive likelihood function for the posterior
%               draws.
%
% * 12-05-2021: Finished the predictive likelihood function for posterior draws.
%
% * 14-05-2021: Began work on the predictive moment and PIT functions for the posterior
%               draws.
%
% * 18-05-2021: Finished the predictive moments function for the posterior draws. Continued
%               work on the PIT function.
%
% * 19-05-2021: Finished the main part of the PIT function (prediction path part remains).
%               Continued work on the CRPS function and started work on the unconditional
%               predictive distribution for posterior draws.
%
% * 20-05-2021: Continued work on the unconditional predictive distribution for the
%               posterior draws.
%
% * 25-05-2021: Finished the unconditional predictive distributions for the posterior draws.
%               The CRPS and PIT functions now support path generated predictive distributions.
%
% * 26-05-2021: The CRPS function for the posterior draws now allows for annualized data.
%
% * 04-08-2021: Added the import prior function.
%
% * 05-08-2021: Finished the import prior function.
%
% * 06-01-2022: Updated the documentation.
%
% * 11-01-2022: Corrected the log prior values when viewing the log prior, the log Jacobian and the log
%               system prior, subject to the existence of the system prior.
%
% * 13-01-2022: Took DSGEModel.ShowEstimationLog into account.
%
% * 27-01-2022: Updated the documentation.
%
% * 01-02-2022: The Laplace approximation functions now also save the results to mat-file.
%
% * 02-02-2022: Fixed a bug due to not copying the user defined belief system file to the tmp
%               directory.
%
% * 11-02-2022: Added check to the CRPS calculations if the user has selected a large number of paths.
%               Took into account that PrintCRPS requires the input variable NumPredPaths.
%
% * 15-02-2022: Took into account that the prediction horizon when computing prediction paths can
%               be greater than the horizon for which true values of the predicted variables exist.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed eval calls to feval.
%
% * 22-02-2022: Changed last eval calls to feval.
%
% * 16-03-2022: Took into account that AdaptiveLearningIRFtheta now takes the steady-state solution
%               matrices Fss and Bss as input.
%
% * 31-03-2022: Made sure that "DSGEModel.InitializeParameterFile" exists before trying to run it.
%
% * 11-05-2022: The BeliefSys structure is not necessarily sent to "CalculateAdaptiveLearningStateVariables"
%               for the posterior mode cases.
%
% * 17-05-2022: Added sub-sample selection code for the SMC and MitISEM samplers when combining paths with posterior
%               draws for the CRPS and ES scores.
%
% * 18-05-2022: Added sub-sample selection for the SMC and MitISEM samplers when combining paths with
%               posterior draws for unconditional predictive distributions.
%
% * 19-05-2022: Added sub-sample selection for the SMC and MitISEM samplers when combining paths with posterior
%               draws for the PIT estimation and for the marginal predictive moments.
%
% * 24-05-2022: The variable "DSGEModel.NameOfModel" now affects the names of saved files under CRPS and PIT
%               calculations.
%
% * 05-01-2023: Updated the documentation.
%
% * 11-01-2023: Began work on the persistence function for the initial values, the DSGE posterior mode and
%               the posterior mode values.
%
% * 12-01-2023: Finished the three persistence functions.
%
% * 13-02-2023: Added the display parameter values functions and the time domain based information matrix
%               functions for the initial values, the DSGE posterior mode and the posterior mode estimates.
%
% * 14-02-2023: Added the frequency domain based information matrix functions and finished the posterior
%               mode case.
%
% * 15-02-2023: Finished the frequency domain based information matrix functions for the DSGE posterior mode
%               and initial values cases.
%
% * 22-02-2023: Added the optional input variable "maingui".
%
% * 29-03-2023: Added the "Save inverse Hessian" function.
%
% * 30-03-2023: Updated the documentation.
%
% * 19-06-2023: Fixed print bugs with missing comment signs.
%
% * 05-07-2023: The function now tries to load the AIMData file directly.
%
% * 14-08-2023: The function now supports an additional gradient function.
%
% * 15-08-2023: The "PriorInformation" function now supports system priors fully.
%
% * 05-10-2023: Added the Monte Carlo filtering function.
%
% * 06-10-2023: Made sure that the output directory exists for storing prior draws.
%
% * 25-10-2023: Added the DSGE model solution egenvalues functions.
%
% * 27-10-2023: Added an additional test to the user defined belief system file which
%               checks if each Gj matrix has rank equal to its number of columns.
%
% * 02-11-2023: Added a function for plotting the Monte Carlo filtering densities.
%
% * 02-01-2024: Took into account that "PrintCRPS" now takes the IsConditional input variable.
%
% * 09-01-2024: Took into account that "YADAHessianMatrix" now takes the input variable "NumWorkers".
%
% * 29-01-2024: Took into account that "PrintCRPS" now takes the IsZLB input variable.
%
% * 30-01-2024: Added estimation time for the Laplace approximation computations.
%
% * 11-03-2024: Added the function case for the unconditional predictive distribution using the
%               prior draws.
%
% * 12-03-2024: Added the function case for the predictive likelihood using the prior draws.
%
% * 13-03-2024: Made sure that the predictive likelihood computations also check for the annual
%               control.
%
% * 14-03-2024: Added the function case for the CRPS and ES using the prior draws.
%
% * 15-03-2024: Changed the lower vertical position of the TextGUI function from 32 pixels to 50.
%
% * 18-03-2024: Made sure that the PredictionEvent field is not changed unless user selects it.
%
% * 19-03-2024: Added the function cases for the marginal predictive moments and the PITs using the prior draws.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
if nargin==1;
   maingui = findobj('Type','figure','Tag','YADA');
end;
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
OrigDSGEModel = get(controls.openmenu,'UserData');
if MatlabNumber<7;
   lasterr = '';
end;
%
switch selector
   case 'SetForwardLookingVariables'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % now we need to examine the structural form of the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI,0);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         if InitStatus==1;
            %
            % load the AIMData file
            %
            ErrorStr = '';
            AIMData.nlead = [];
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
            if (isempty(ErrorStr)==1)&(AIMData.nlead==1);
               %
               % check if we can solve the model
               %
               if MatlabNumber>=8.0;
                  try;
                     [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                     if isempty(ErrMsg)==0;
                        ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                     end;
                  catch ME;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                     if isempty(ErrMsg)==0;
                        ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                     end;
                  catch;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  ForwardVariablePositions = [];
                  ForwardVariableNames = '';
                  %
                  % get the structural form matrices
                  %
                  [GMatrix,HMatrix] = compute_aim_matrices(ModelParameters);
                  [rh,ch] = size(HMatrix);
                  if (((2+AIMData.nlead)*rh)==ch)&(mcode==1);
                     [rg,cg] = size(GMatrix);
                     H = zeros(rh,ch);
                     H(1:rg,1:cg) = GMatrix;
                     H = H+HMatrix;
                     Hexp = H(:,(2*rh)+1:(2+AIMData.nlead)*rh);
                     Hexp = Hexp(DSGEModel.StateEquationPositions,DSGEModel.StateVariablePositions);
                     r = length(DSGEModel.StateVariablePositions);
                     %
                     % now we check which columns of Hexp are non-zero. The corresponding state variables
                     % are candidates for forward looking variables
                     %
                     f = rank(Hexp);
                     if isempty(DSGEModel.ForwardVariablePositions)==1;
                        for j=1:r;
                           %if (sum(abs(Hexp(:,j)))>1.0e-7)&(length(ForwardVariablePositions)<f);
                           if sum(abs(Hexp(:,j)))>1.0e-7;
                              ForwardVariablePositions = [ForwardVariablePositions j];
                              ForwardVariableNames = strvcat(ForwardVariableNames,DSGEModel.StateVariableNames(j,:));
                           end;
                        end;
                     else;
                        ForwardVariableNames = DSGEModel.ForwardVariableNames;
                        ForwardVariablePositions = DSGEModel.ForwardVariablePositions;
                     end;
                  elseif (((2+AIMData.nlead)*rh)==ch)&(mcode~=1);
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     elseif DSGEModel.ModelSolver==4;
                        SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                     end;
                     SolverMessage = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" YADA is therefore unable to create suggestion of forward looking variables.'];
                     About(SolverMessage,'information','YADA - Error Message',300,600,CurrINI);
                  else;
                     txt = 'The numbers of leads (1) and lags (1) don''t match with the dimension of the structural form matrix supplied through AiM. YADA is therefore unable to create suggestion of forward looking variables.';
                     About(txt,'information','YADA - Error Message',300,600,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
               end;
            elseif (isempty(ErrorStr)==1)&(AIMData.nlead~=1);
               txt = ['The adaptive learning approach in YADA requires that all expectation terms in the structural model have a lead of 1. Your model has a lead of ' int2str(AIMData.nlead) '. You may consider rewriting your model using new variables such that the maximum lead is 1.'];
               About(txt,'information','YADA - Error Message',300,600,CurrINI);
            else;
               About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
            end;
         end;
         %
         % test if we have some suggested forward looking variables
         %
         if isempty(ForwardVariablePositions)==0;
            %
            % now we can start with the selection of forward looking variables
            %
            [action,NewForwardVariableNames,NewForwardVariablePositions] = SetStateVarsShocksDLG('init','Forward Looking Variables',[' - Minimum = ' int2str(f)],DSGEModel.StateVariableNames,ForwardVariableNames,ForwardVariablePositions,CurrINI);
            if (strcmp(action,'OK')==1)&(isempty(NewForwardVariablePositions)==0);
               %
               % now test if the selected variables are valid
               %
               %if length(NewForwardVariablePositions)==f;
               if length(NewForwardVariablePositions)>=f;
                  %SMatrix = eye(r);
                  %Sperp = SMatrix(:,setdiff((1:r),NewForwardVariablePositions));
                  %SMatrix = SMatrix(:,NewForwardVariablePositions);
                  %
                  % get the re-ordering of the equations such that their ordering is equivalent
                  % to the order of the variables which appear in expectation terms
                  %
                  [C,ForwardEquationPositions] = GetForwardLookingVariableOrder(Hexp,NewForwardVariablePositions);
                  %
                  %if rank(SMatrix'*C*Hexp*SMatrix)==f;
                  if rank(Hexp)==rank(Hexp(ForwardEquationPositions,NewForwardVariablePositions));
                  %if (rank(SMatrix'*C*Hexp*SMatrix)==f)|(max(max(abs(Hexp*Sperp)))<1.0e-8);
                     DSGEModel.ForwardVariableNames = NewForwardVariableNames;
                     DSGEModel.ForwardVariablePositions = NewForwardVariablePositions;
                  else;
                     txt = ['The selected variables are inconsistent with the structural form as the rank of the matrix with coefficients on the expected future values is then equal to ' int2str(rank(Hexp(ForwardEquationPositions,NewForwardVariablePositions))) ' instead of the expected value ' int2str(f) '. Please change your selection.'];
                     About(txt,'information','Learning - Set Forward Variables',300,600,CurrINI);
                  end;
               else;
                  %txt = ['The number of forward looking variables you selected (' int2str(length(NewForwardVariablePositions)) ') is not equal to the rank of the matrix with coefficients on the expected future values (' int2str(f) '). Please change your selection.'];
                  txt = ['The number of forward looking variables you selected (' int2str(length(NewForwardVariablePositions)) ') is less than the rank of the matrix with coefficients on the expected future values (' int2str(f) '). Please change your selection.'];
                  About(txt,'information','Learning - Set Forward Variables',300,600,CurrINI);
               end;
            elseif (strcmp(action,'OK')==1)&(isempty(NewForwardVariablePositions)==1);
               txt = 'You didn''t select any forward looking variables. YADA is therefore unable to apply adaptive learning for the expectations formation.';
               About(txt,'information','Learning - Set Forward Variables',200,600,CurrINI);
               DSGEModel.ForwardVariableNames = '';
               DSGEModel.ForwardVariablePositions = [];
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SetBeliefSystem'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check mark the correct selection between the defeaul naive system and a user file
         %
         if gcbo==controls.learning.belief.default;
            if isfield(DSGEModel,'BeliefSystemFile')==1;
               BSF = FixFilePath([pwd '\tmp\' GetFilename(DSGEModel.BeliefSystemFile)]);
               if FileExist(BSF)==1;
                  delete(BSF);
               end;
            end;
            DSGEModel.BeliefSystem = 1;
            DSGEModel.BeliefSystemFile = pwd;
         else;
            %
            % first the user needs to select an m-file with a belief system
            %
            if isempty(DSGEModel.ForwardVariablePositions)==0;
               %
               % the user now has to select an m-file
               %
               if FileExist(DSGEModel.BeliefSystemFile)==1;
                  BeliefSystemFile = DSGEModel.BeliefSystemFile;
               else;
                  if MatlabNumber>=6;
                     BeliefSystemFile = '';
                  else;
                     BeliefSystemFile = [DSGEModel.BeliefSystemFile '\*.m;*.p'];
                  end;
               end;
               if MatlabNumber>=6;
                  FileType = {'*.m', 'Matlab M-Files (*.m)'; ...
                              '*.p', 'Matlab P-Files (*.p)'};
                  [FileName,FilePath] = uigetfile(FileType,'Belief System File - Select File',BeliefSystemFile);
               else;
                  [FileName,FilePath] = uigetfile(BeliefSystemFile,'Belief System File - Select File');
               end;
               drawnow;
               pause(0.02);
               if (isnumeric(FileName)==0)&(isunix==1);
                  if strcmp(FilePath(1,length(FilePath)),'/')==0;
                     FilePath = [FilePath '/'];
                  end;
               end;
               if (isnumeric(FileName)==0)&(FileExist([FilePath FileName])==1);
                  BeliefSystemFile = [FilePath FileName];
                  if FileExist(BeliefSystemFile)==1;
                     %
                     % check if the belief system file has changed
                     %
                     if isequal(DSGEModel.BeliefSystemFile,BeliefSystemFile)==0;
                        %
                        % copy the belief system file to the tmp folder
                        %
                        [stat,msg] = CopyFile(BeliefSystemFile,[pwd '\tmp']);
                        %
                        %
                        txt = ['YADA will now test if "' GetFilename(BeliefSystemFile) '" uses proper matlab syntax and if it provides the required output.'];
                        About(txt,'information',['Test of "' GetFilename(BeliefSystemFile) '"'],300,600,CurrINI);
                        %
                        % test the belief system file
                        %
                        ErrorStr = '';
                        if MatlabNumber>=8.0;
                           try;
                              BeliefSys = feval(GetName(BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           catch ME;
                              ErrorStr = ['YADA caught an error when running the file "' GetName(BeliefSystemFile) '". Message caught is: ' YADALastError(ME)];
                           end;
                        else;
                           try;
                              BeliefSys = feval(GetName(BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           catch;
                              ErrorStr = ['YADA caught an error when running the file "' GetName(BeliefSystemFile) '". Message caught is: ' YADALastError];
                           end;
                        end;
                        if isempty(ErrorStr)==1;
                           %
                           % the file executes properly. now check if the output is OK
                           %
                           if (isfield(BeliefSys,'m')==1)&(isfield(BeliefSys,'G')==1)&(isfield(BeliefSys,'Name')==1);
                              %
                              % test the number of lags
                              %
                              if (BeliefSys.m>=0)&(BeliefSys.m<=2);
                                 %
                                 % now we check the G matrices
                                 %
                                 k = DSGEModel.k;
                                 r = length(DSGEModel.StateVariablePositions);
                                 f = length(DSGEModel.ForwardVariablePositions);
                                 m = BeliefSys.m;
                                 if (length(BeliefSys.G)==f)&(min(size(BeliefSys.G))==1);
                                    %
                                    % we have a proper setup of the cell array with the G_j matrices for determining
                                    % the variables in each belief equation. now we check the size.
                                    %
                                    IsCorrectSize = 1;
                                    for j=1:f;
                                       if size(BeliefSys.G{j},1)~=(k+(r*m));
                                          IsCorrectSize = 0;
                                          BadMatrixNumber = j;
                                          break;
                                       end;
                                    end;
                                    if IsCorrectSize==1;
                                       %
                                       % check if the rank of each Gj matrix is equal to
                                       % its number of columns
                                       %
                                       IsCorrectRank = 1;
                                       for j=1:f;
                                          Gj = BeliefSys.G{j};
                                          if size(Gj,2)~=rank(Gj);
                                             IsCorrectRank = 0;
                                             BadMatrixNumber = j;
                                             break;
                                          end;
                                       end;
                                       if IsCorrectRank==1;
                                          %
                                          % all G_j matrices are correct
                                          %
                                          DSGEModel.BeliefSystem = 2;
                                          DSGEModel.BeliefSystemFile = BeliefSystemFile;
                                          txt = ['The belief system file "' GetFilename(BeliefSystemFile) '" in the directory "' GetPath(BeliefSystemFile) '" is correctly specified.'];
                                          About(txt,'information',['Belief System File - "' GetFilename(BeliefSystemFile) '"'],200,600,CurrINI);
                                       else;
                                          txt = ['Element number ' int2str(BadMatrixNumber) ' doesn''t have the full column rank. It has ' int2str(size(BeliefSys.G{BadMatrixNumber},2)) ' columns, while its rank is ' int2str(rank(BeliefSys.G{BadMatrixNumber})) '. Belief system file selection is aborted.'];
                                          About(txt,'information','YADA - Error Message',300,600,CurrINI);
                                       end;
                                    else;
                                       txt = ['Element number ' int2str(BadMatrixNumber) ' doesn''t have the correct number of rows. It has ' int2str(size(BeliefSys.G{BadMatrixNumber},1)) ' rows, while it should have ' int2str(k+(r*m)) '. Belief system file selection is aborted.'];
                                       About(txt,'information','YADA - Error Message',300,600,CurrINI);
                                    end;
                                 else;
                                    txt = 'The cell array G must be a vector array and its equal to the number of forward looking variables. Belief system file selection is aborted.';
                                    About(txt,'information','YADA - Error Message',300,600,CurrINI);
                                 end;
                              elseif BeliefSys.m>2;
                                 txt = 'The belief system lag can be at most 2 in YADA. This restriction may change in the future. Belief system file selection is aborted.'
                                 About(txt,'information','YADA - Error Message',300,600,CurrINI);
                              else;
                                 txt = 'The belief system lag must at least be 0 in YADA. Belief system file selection is aborted.'
                                 About(txt,'information','YADA - Error Message',300,600,CurrINI);
                              end;
                           else;
                              txt = ['The belief system file "' GetName(BeliefSystemFile) '" does not provide as output a structure with fields "m", "G" and "Name". These fields are required in order to setup a belief system for adaptive learning in YADA.'];
                              About(txt,'information','YADA - Error Message',300,600,CurrINI);
                           end;
                        else;
                           About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
                        end;
                     end;
                  end;
               end;
            else;
               txt = 'You need to set the forward looking variables before you can select a belief system file.';
               About(txt,'information','Learning - Select Belief System',200,600,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SetBeliefParametersPriorFile'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if FileExist(DSGEModel.BeliefPriorFile)==1;
            BeliefPriorFile = DSGEModel.BeliefPriorFile;
         else;
            BeliefPriorFile = FixFilePath([pwd '\learning\data\DefaultBeliefSystemPrior.xls']);
         end;
         if MatlabNumber>=6;
            FileType = {'*.wk1;*.xls*', 'Spreadsheet Files (*.wk1, *.xls*)'; ...
                        '*.wk1', 'Lotus 1-2-3 Spreadsheet (*.wk1)'; ...
                        '*.xls*', 'Excel Spreadsheet (*.xls*)'};
            [FileName,FilePath] = uigetfile(FileType,'Belief Prior File - Select File',BeliefPriorFile);
         else;
            [FileName,FilePath] = uigetfile(BeliefPriorFile,'Belief Prior File - Select File');
         end;
         drawnow;
         pause(0.02);
         %
         if (isnumeric(FileName)==0)&(isunix==1);
            if strcmp(FilePath(1,length(FilePath)),'/')==0;
               FilePath = [FilePath '/'];
            end;
         end;
         %
         % check if a file was selected
         %
         if (isnumeric(FileName)==0)&(FileExist([FilePath FileName])==1);
            BeliefPriorFile = [FilePath FileName];
            BeliefPriorFileSheet = '';
            %
            % now check if a style sheet should be read.
            %
            [FilePath,FileName,FileExt] = fileparts(BeliefPriorFile);
            %
            if strncmp(lower(FileExt),'.xls',4)==1;
               %
               % we must matlab version 6 or later. Check the spreadsheet for sheets
               %
               ErrorValue = [];
               file = FixFilePath([FilePath '\' FileName FileExt]);
               try;
                  [fok,sheets] = xlsfinfo(file);
               catch;
                  ErrorValue = 1;
               end;
               if (isempty(ErrorValue)==1)&(isempty(fok)==0);
                  if length(sheets)>1;
                     mysheets = sheets{1};
                     for i=2:length(sheets);
                        mysheets = [mysheets '|' sheets{i}];
                     end;
                     [action,selsheet] = SheetDlg(mysheets,file,CurrINI);
                     if strcmp(action,'Cancel')==1;
                        NewSheet = '';
                     else;
                        NewSheet = sheets{selsheet};
                     end;
                  else;
                     if iscell(sheets)==1;
                        NewSheet = char(sheets);
                     else;
                        NewSheet = sheets;
                     end;
                  end;
                  if strcmp(NewSheet,'')==0;
                     BeliefPriorFileSheet = NewSheet;
                  end;
               else;
                  %
                  % The excel spreadsheet was not readable
                  %
                  txt = ['The excel spreadsheet you selected "' FileName FileExt '" cannot be read by YADA. Please select another file.'];
                  About(txt,'information','Bad Excel Spreadsheet',200,600,CurrINI);
               end;
            else;
               %
               % Check to make sure the file is a Lotus 1-2-3 spreadsheet
               %
               if strcmp(lower(FileExt),'.wk1')==0;
                  txt = ['The file you selected "' FileName FileExt '" is not a valid file for prior distribution specification. Please select a valid file for this purpose.'];
                  About(txt,'information','Invalid File Format',200,600,CurrINI);
               else;
                  ErrorValue = [];
                  try;
                     [fok,chk] = wk1finfo(FixFilePath([FilePath '\' FileName FileExt]));
                  catch;
                     ErrorValue = 1;
                  end;
                  if (isempty(ErrorValue)==1)&(isempty(fok)==0);
                     BeliefPriorFileSheet = 'None';
                  else;
                     %
                     % The Lotus spreadsheet was not readable
                     %
                     txt = ['The Lotus 1-2-3 spreadsheet you selected "' FileName FileExt '" cannot be read by YADA. Please select another file.'];
                     About(txt,'information','Bad Lotus Spreadsheet',200,600,CurrINI);
                  end;
               end;
            end;
            if isempty(BeliefPriorFileSheet)==0;
               %
               % now we check if the prior for the adaptive learning parameters is satisfactory
               %
               TmpDSGEModel = DSGEModel;
               TmpDSGEModel.BeliefPriorFile = BeliefPriorFile;
               TmpDSGEModel.BeliefPriorFileSheet = BeliefPriorFileSheet;
               [status,theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = VerifyAdaptiveLearningPriorData(TmpDSGEModel,CurrINI);
               if status==1;
                  %
                  % the spreadsheet is valid
                  %
                  DSGEModel.BeliefPriorFile = BeliefPriorFile;
                  DSGEModel.BeliefPriorFileSheet = BeliefPriorFileSheet;
                  txt = ['You have selected to use the file "' GetFilename(DSGEModel.BeliefPriorFile) '", with sheet "' DSGEModel.BeliefPriorFileSheet '" and located in the directory "' GetPath(DSGEModel.BeliefPriorFile) '", as the prior file for the parameters of the belief system when applying adaptive learning. This file contains valid names and prior distributions.'];
                  About(txt,'information','Belief Prior File',300,600,CurrINI);
               end;
               clear('TmpDSGEModel');
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'NoSmoothedStatesForBeliefs'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if DSGEModel.NoSmoothedStatesForBeliefs==1;
            DSGEModel.NoSmoothedStatesForBeliefs = 0;
         else;
            DSGEModel.NoSmoothedStatesForBeliefs = 1;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'FixedBeliefsForProjections'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if DSGEModel.FixedBeliefsForProj==1;
            DSGEModel.FixedBeliefsForProj = 0;
         else;
            DSGEModel.FixedBeliefsForProj = 1;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ImportPriorData'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % get the current posterior mode data file
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % select the posterior mode file to import prior data from
            %
            if MatlabNumber>=6;
               FileType = {'*.mat', 'Matlab mat Files (*.mat)'};
               [FileName,FilePath] = uigetfile(FileType,'Posterior Mode File - Select File',ModeFile);
            else;
               [FileName,FilePath] = uigetfile(ModeFile,'Posterior Mode File - Select File');
            end;
            drawnow;
            pause(0.02);
            %
            if (isnumeric(FileName)==0)&(isunix==1);
               if strcmp(FilePath(1,length(FilePath)),'/')==0;
                  FilePath = [FilePath '/'];
               end;
            end;
            %
            % check if a file was selected
            %
            if (isnumeric(FileName)==0)&(FileExist([FilePath FileName])==1);
               ImportModeFile = [FilePath FileName];
               %
               % check if we can import data
               %
               if MatlabNumber>=8.0;
                  try;
                     ImportModeData = load(ImportModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ImportModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ImportModeData = load(ImportModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ImportModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  %
                  % test if import file is valid
                  %
                  NumParam = length(ModeData.thetaMode);
                  if isfield(ImportModeData,'thetaMode')==1;
                     NumImportParam = length(ImportModeData.thetaMode);
                     if NumParam==NumImportParam;
                        %
                        % check if ParameterNames.estimated field exists
                        %
                        if (isfield(ImportModeData,'ParameterNames')==1)&(isfield(ImportModeData.ParameterNames,'estimated')==1);
                           %
                           % check if the correct parameter names exist
                           %
                           CheckNames = 0;
                           for ii=1:NumParam;
                              OldName = StringTrim(ModeData.ParameterNames.estimated(ii,:));
                              NewName = StringTrim(ImportModeData.ParameterNames.estimated(ii,:));
                              CheckNames = CheckNames+strcmp(OldName,NewName);
                           end;
                           if CheckNames==NumParam;
                              %
                              % all parameter names are correct.
                              % make a backup copy of the current posterior mode file
                              %
                              CheckFile = 1;
                              Num = 1;
                              while CheckFile==1;
                                 NewModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '-' int2str(Num) '.mat']);
                                 Num = Num+1;
                                 if FileExist(NewModeFile)==0;
                                    CheckFile = 0;
                                 end;
                              end;
                              %
                              copyfile(ModeFile,NewModeFile);
                              %
                              % collect all the prior distribution entries that can be imported
                              %
                              thetaIndex = ImportModeData.thetaIndex;
                              thetaDist = ImportModeData.thetaDist;
                              LowerBound = ImportModeData.LowerBound;
                              thetaPositions = ImportModeData.thetaPositions;
                              PriorDist = ImportModeData.PriorDist;
                              UniformBounds = ImportModeData.UniformBounds;
                              %
                              DSGEModel.BaseDirBeliefPriorFile = ImportModeData.DSGEModel.BaseDirBeliefPriorFile;
                              DSGEModel.BeliefPriorFile = ImportModeData.DSGEModel.BeliefPriorFile;
                              DSGEModel.BeliefPriorFileSheet = ImportModeData.DSGEModel.BeliefPriorFileSheet;
                              %
                              % recompute the prior values and the height of the log posterior density
                              %
                              LogPriorDensity = logPriorDSGE(ModeData.thetaMode,thetaDist,PriorDist,LowerBound);
                              LogJacobianValue = logJacobian(ModeData.phiMode,thetaIndex,UniformBounds);
                              LogLikeDensity = ModeData.LogLikeDensity;
                              LogSPMode = ModeData.LogSPMode;
                              LogPostDensity = LogLikeDensity+LogPriorDensity+LogJacobianValue+LogSPMode;
                              %
                              % we need to copy files first in case they are needed for the computations
                              %
                              [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
                              [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
                              if FileExist(DSGEModel.UpdateParameterFile)==1;
                                 [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
                              end;
                              if FileExist(DSGEModel.InitializeParameterFile)==1;
                                 [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
                              end;
                              if FileExist(DSGEModel.SystemPriorFile)==1;
                                 [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
                                 IsSystemPrior = 1;
                              else;
                                 IsSystemPrior = 0;
                              end;
                              %
                              % check if we should also copy the belief system file
                              %
                              if DSGEModel.BeliefSystem==2;
                                 [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                              end;
                              %
                              % check if we should copy model solver file
                              %
                              if DSGEModel.ModelSolver==4;
                                 if FileExist(DSGEModel.ModelSolverFile)==1;
                                    [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
                                 end;
                                 if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
                                    [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
                                 end;
                              end;
                              %
                              % try to load the AIMData, other wise use ModeData
                              %
                              if MatlabNumber>=8.0;
                                 try;
                                    AIMData = load(DSGEModel.AIMDataFile);
                                 catch ME;
                                    AIMData = ModeData.AIMData;
                                 end;
                              else;
                                 try;
                                    AIMData = load(DSGEModel.AIMDataFile);
                                 catch;
                                    AIMData = ModeData.AIMData;
                                 end;
                              end;
                              %
                              % get the sample from the ModeData structure
                              %
                              DSGEModel.FirstPeriod = ModeData.FirstPeriod;
                              DSGEModel.LastPeriod = ModeData.LastPeriod;
                              %
                              % determine xbar
                              %
                              ConstPos = 0;
                              if DSGEModel.k>0;
                                 ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
                                 if ConstPos==0;
                                    %
                                    % we will now check if X has a unit vector
                                    %
                                    X = DSGEModel.X(:,ModeData.FirstPeriod:ModeData.LastPeriod);
                                    T = size(X,2);
                                    for j=1:DSGEModel.k;
                                       ConstTest = (DSGEModel.X(j,ModeData.FirstPeriod:ModeData.LastPeriod)==ones(1,T));
                                       if sum(ConstTest)==T;
                                          ConstPos = j;
                                          break;
                                       end;
                                    end;
                                 end;
                                 xbar = zeros(DSGEModel.k,1);
                                 xbar(ConstPos) = 1;
                              else;
                                 xbar = [];
                              end;
                              %
                              % compute the gradient vectors at the mode with the new prior
                              %
                              ParamChange = 1e-6*ones(NumParam,1);
                              try;
                                 [gh,badg1] = YADAnumgrad('logPosteriorAdaptiveLearningPhiDSGE',ModeData.phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModeData.ModelParameters,ModeData.SMatrix,ModeData.Sperp,ModeData.BeliefSys.m,ModeData.BeliefSys.G,ModeData.Gtilde,ModeData.Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                              catch;
                                 [gh,badg1] = YADANumGradient('logPosteriorAdaptiveLearningPhiDSGE',ModeData.phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModeData.ModelParameters,ModeData.SMatrix,ModeData.Sperp,ModeData.BeliefSys.m,ModeData.BeliefSys.G,ModeData.Gtilde,ModeData.Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                              end;
                              if badg1==0;
                                 try;
                                    [g,badg2] = YADAnumgrad('logPosteriorAdaptiveLearningPhiDSGE',ModeData.phiMode-ParamChange,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModeData.ModelParameters,ModeData.SMatrix,ModeData.Sperp,ModeData.BeliefSys.m,ModeData.BeliefSys.G,ModeData.Gtilde,ModeData.Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                 catch;
                                    [g,badg2] = YADANumGradient('logPosteriorAdaptiveLearningPhiDSGE',ModeData.phiMode-ParamChange,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModeData.ModelParameters,ModeData.SMatrix,ModeData.Sperp,ModeData.BeliefSys.m,ModeData.BeliefSys.G,ModeData.Gtilde,ModeData.Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                 end;
                              else;
                                 badg2 = 1;
                              end;
                              %
                              % compute the inverse Hessian with the possibly new prior
                              %
                              if badg1+badg2==0;
                                 [InverseHessian,BFGSOutput] = YADAbfgsi(ModeData.InverseHessian,gh-g,ParamChange);
                              else;
                                 InverseHessian = [];
                              end;
                              %
                              % compute the partial of theta wrt to phi
                              %
                              ThetaPhiPartial = PartialThetaPartialPhi(ModeData.phiMode,thetaIndex,UniformBounds);
                              %
                              txt = 'Would you like to compute the inverse Hessian using finite differences and, if valid, use it to estimate the marginal likelihood with the Laplace approximation?';
                              answer = Query(txt,'question',500,'Import Prior Data - Finite Differences',600,'yes',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 %
                                 % determine some additional parameters
                                 %
                                 StepLengthStr = get(controls.optimize.steplength,'String');
                                 StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                                 %
                                 % show a wait dialog
                                 %
                                 txt = ['Please wait while YADA computes the inverse Hessian of the log posterior density with finite differences. Computations started at: ' StringTrim(datestr(now,14))];
                                 WaitHandle = WaitDLG(txt,'information','Import Prior Data',600,150,CurrINI,0);
                                 WaitControls = get(WaitHandle,'UserData');
                                 %
                                 % now we have all the inputs
                                 %
                                 VechHessian = YADAHessianMatrix('logPosteriorAdaptiveLearningPhiDSGE',ModeData.phiMode,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModeData.ModelParameters,ModeData.SMatrix,ModeData.Sperp,ModeData.BeliefSys.m,ModeData.BeliefSys.G,ModeData.Gtilde,ModeData.Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                 HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
                                 if IsPositiveDefinite(HessianMat)==1;
                                    phiInverseHessianFinDiff = HessianMat\eye(NumParam);
                                    Recomputed = 1;
                                 else;
                                    phiInverseHessianFinDiff = NaN;
                                 end;
                                 %
                                 % delete the wait dialog
                                 %
                                 set(WaitControls.text,'UserData','done');
                                 delete(WaitHandle);
                                 drawnow;
                                 pause(0.02);
                              else;
                                 phiInverseHessianFinDiff = ModeData.phiInverseHessianFinDiff;
                                 Recomputed = 0;
                              end;
                              %
                              % check if we should recheck the optimum
                              %
                              if (isempty(InverseHessian)==0)&(IsPositiveDefinite(InverseHessian)==1);
                                 txt = 'Would you like to re-check the posterior mode optimum using the transformed values?';
                                 answer = Query(txt,'question',500,'Import Prior Data - Finite Differences',600,'yes',CurrINI);
                                 if strcmp(lower(answer),'yes')==1;
                                    %
                                    % Check if GraphDirectory exists
                                    %
                                    GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
                                    MinMaxGrid = zeros(NumParam,2);
                                    stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
                                    %
                                    NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
                                    NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
                                    %
                                    % create lower and upper bounds from InverseHessian and phiMode
                                    %
                                    MinMaxGrid(:,1) = ModeData.phiMode-(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
                                    MinMaxGrid(:,2) = ModeData.phiMode+(DSGEModel.GridWidth*sqrt(diag(InverseHessian)));
                                    %
                                    NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
                                    NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
                                    phiFittedStd = VillaniCheckOptimum('logPosteriorAdaptiveLearningPhiDSGE',ModeData.phiMode,[],MinMaxGrid,InverseHessian,NumGridPoints,0,[],ModeData.ParameterNames.estimated,'Checking Curvature at Posterior Mode - Transformed Parameters',CurrINI,GraphDirectory,'phi','','',0,1,1, ...
                                       thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModeData.ModelParameters,ModeData.SMatrix,ModeData.Sperp,ModeData.BeliefSys.m,ModeData.BeliefSys.G,ModeData.Gtilde,ModeData.Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                    %
                                    % check if we should also run the optimize check for the original parameters
                                    %
                                    if (DSGEModel.CheckTransformedOptimum==0)|(IsOdd(DSGEModel.MaximizeAlgorithmValue)==0);
                                       MinMaxGrid(:,1) = PhiToTheta(MinMaxGrid(:,1),thetaIndex,UniformBounds,LowerBound);
                                       MinMaxGrid(:,2) = PhiToTheta(MinMaxGrid(:,2),thetaIndex,UniformBounds,LowerBound);
                                       %
                                       thetaFittedStd = VillaniCheckOptimum('logPosteriorAdaptiveLearningThetaDSGE',ModeData.thetaMode,[],MinMaxGrid,ThetaPhiPartial*InverseHessian*ThetaPhiPartial,NumGridPoints,0,[],ModeData.ParameterNames.estimated,'Checking Curvature at Posterior Mode - Original Parameters',CurrINI,GraphDirectory,'theta','','',0,1,1, ...
                                          thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModeData.ModelParameters,ModeData.SMatrix,ModeData.Sperp,ModeData.BeliefSys.m,ModeData.BeliefSys.G,ModeData.Gtilde,ModeData.Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                    else;
                                       thetaFittedStd = NaN;
                                    end;
                                 else;
                                    phiFittedStd = ModeData.phiFittedStd;
                                    thetaFittedStd = ModeData.thetaFittedStd;
                                 end;
                              else;
                                 phiFittedStd = ModeData.phiFittedStd;
                                 thetaFittedStd = ModeData.thetaFittedStd;
                              end;
                              %
                              % need to recompute the log posterior density
                              %
                              if isnan(phiInverseHessianFinDiff)==0;
                                 LogMargLike = MargLikeLaplace(LogPostDensity,phiInverseHessianFinDiff,1)-ModeData.LogMargLikeSystemLaplace;
                              else;
                                 if (isempty(InverseHessian)==0)&(IsPositiveDefinite(InverseHessian)==1);
                                    LogMargLike = MargLikeLaplace(LogPostDensity,InverseHessian,1)-LogMargLikeSystemLaplace;
                                    Recomputed = 2;
                                 else;
                                    LogMargLike = MargLikeLaplace(LogPostDensity,ModeData.InverseHessian,1)-LogMargLikeSystemLaplace;
                                    Recomputed = 0;
                                 end;
                              end;
                              %
                              % save to file
                              %
                              if (isempty(InverseHessian)==0)&(IsPositiveDefinite(InverseHessian)==1);
                                 save(ModeFile,'thetaIndex','thetaDist','LowerBound','thetaPositions','PriorDist','UniformBounds','DSGEModel','LogPriorDensity',...
                                      'LogJacobianValue','LogPostDensity','phiInverseHessianFinDiff','InverseHessian','phiFittedStd','thetaFittedStd','-APPEND');
                              else;
                                 save(ModeFile,'thetaIndex','thetaDist','LowerBound','thetaPositions','PriorDist','UniformBounds','DSGEModel','LogPriorDensity',...
                                      'LogJacobianValue','LogPostDensity','phiInverseHessianFinDiff','phiFittedStd','thetaFittedStd','-APPEND');
                              end;
                              %
                              % remove two variables from DSGEModel
                              %
                              DSGEModel = rmfield(DSGEModel,'FirstPeriod');
                              DSGEModel = rmfield(DSGEModel,'LastPeriod');
                              %
                              % display an about dialog
                              %
                              if Recomputed==1;
                                 txt = ['YADA has imported the prior data from the posterior mode file "' GetFilename(ImportModeFile) '" into the posterior mode file for the current model, i.e. "' GetFilename(ModeFile) '". A backup copy of the old posterior mode file is available as "' GetFilename(NewModeFile) '" in the folder "' GetPath(NewModeFile) '". A number of other variables in the posterior mode file have been recomputed, reflecting the imported prior. The log marginal likelihood has been re-estimated with finite differences and is now equal to ' num2str(LogMargLike,'%0.6f') '.'];
                              elseif Recomputed==2;
                                 txt = ['YADA has imported the prior data from the posterior mode file "' GetFilename(ImportModeFile) '" into the posterior mode file for the current model, i.e. "' GetFilename(ModeFile) '". A backup copy of the old posterior mode file is available as "' GetFilename(NewModeFile) '" in the folder "' GetPath(NewModeFile) '". A number of other variables in the posterior mode file have been recomputed, reflecting the imported prior. The log marginal likelihood has been re-estimated using the inverse Hessian from the optimization routine and is now equal to ' num2str(LogMargLike,'%0.6f') '.'];
                              else;
                                 txt = ['YADA has imported the prior data from the posterior mode file "' GetFilename(ImportModeFile) '" into the posterior mode file for the current model, i.e. "' GetFilename(ModeFile) '". A backup copy of the old posterior mode file is available as "' GetFilename(NewModeFile) '" in the folder "' GetPath(NewModeFile) '". A number of other variables in the posterior mode file have been recomputed, reflecting the imported prior. The log marginal likelihood has not been re-estimated and is still equal to ' num2str(LogMargLike,'%0.6f') '.'];
                              end;
                              About(strvcat(txt,' ','NOTE: You have to save the DSGE model file to ensure that all changes are saved.'),'information','Import Prior Data',500,600,CurrINI);
                           else;
                              txt = ['The selected import prior data file "' GetFilename(ImportModeFile) '" is not valid since it doesn''t contain the correct names of the estimated parameters for the current model.'];
                              About(txt,'information','Bad Posterior Mode File',500,600,CurrINI);
                           end;
                        else;
                           txt = ['The selected import prior data file "' GetFilename(ImportModeFile) '" is not valid since it doesn''t contain the names of the estimated parameters.'];
                           About(txt,'information','Bad Posterior Mode File',500,600,CurrINI);
                        end;
                     else;
                        txt = ['The selected import prior data file "' GetFilename(ImportModeFile) '" is not valid since it doesn''t contain the correct number of parameters to estimate.'];
                        About(txt,'information','Bad Posterior Mode File',500,600,CurrINI);
                     end;
                  else;
                     txt = ['The selected import prior data file "' GetFilename(ImportModeFile) '" is not valid since it doesn''t contain the posterior mode of theta, the original parameters.'];
                     About(txt,'information','Bad Posterior Mode File',500,600,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ModelSolver'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % the user now has to select an m-file
         %
         CancelUserSolver = 0;
         if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
            AdaptiveLearningModelSolverFile = DSGEModel.AdaptiveLearningModelSolverFile;
            %
            % allow the user to cancel the use of the user defined model solver.
            %
            txt = ['Would you like to cancel the use of your own DSGE model solver under adaptive learning?'];
            answer = Query(txt,'question',3000,'Select Model Solver - Adaptive Learning',600,'no',CurrINI);
            if strcmp(lower(answer),'yes')==1;
               CancelUserSolver = 1;
            end;
         else;
            if MatlabNumber>=6;
               AdaptiveLearningModelSolverFile = '';
            else;
               AdaptiveLearningModelSolverFile = [DSGEModel.AdaptiveLearningModelSolverFile '\*.m;*.p'];
            end;
         end;
         if CancelUserSolver==0;
            if MatlabNumber>=6;
               FileType = {'*.m', 'Matlab M-Files (*.m)'; ...
                           '*.p', 'Matlab P-Files (*.p)'};
               [FileName,FilePath] = uigetfile(FileType,'Adaptive Learning DSGE Model Solver - Select File',AdaptiveLearningModelSolverFile);
            else;
               [FileName,FilePath] = uigetfile(AdaptiveLearningModelSolverFile,'Adaptive Learning DSGE Model Solver - Select File');
            end;
            drawnow;
            pause(0.02);
            if (isnumeric(FileName)==0)&(isunix==1);
               if strcmp(FilePath(1,length(FilePath)),'/')==0;
                  FilePath = [FilePath '/'];
               end;
            end;
            AdaptiveLearningModelSolverFile = [FilePath FileName];
            if (isnumeric(FileName)==0)&(FileExist(AdaptiveLearningModelSolverFile)==1);
               DSGEModel.AdaptiveLearningModelSolverFile = AdaptiveLearningModelSolverFile;
               set(controls.learning.options.modelsolver,'Label',['&Select Model Solver (' GetName(DSGEModel.AdaptiveLearningModelSolverFile) ')']);
            end;
         else;
            DSGEModel.AdaptiveLearningModelSolverFile = pwd;
            set(controls.learning.options.modelsolver,'Label','&Select Model Solver');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PosteriorModeEstimation'
      if get(controls.about,'UserData')==0;
         %
         % This case attempts to find the posterior mode through numerical optimization of the joint
         % posterior distribution
         %
         if (strcmp(lower(DSGEModel.UpdateParameterFile),lower(pwd))==1)&(strcmp(lower(DSGEModel.InitializeParameterFile),lower(pwd))==1);
            txt = 'Are you sure that you have specified all the parameters that your DSGE model requires?';
            answer = Query(txt,'question',120,'No Additional Parameter Files',400,'no',CurrINI);
         else;
            answer = 'Yes';
         end;
         if strcmp(lower(answer),'yes')==1;
            if DSGEModel.ShowEstimationLog==1;
               oldtab = get(controls.tab1,'UserData');
               set(maingui,'CurrentObject',controls.tab6);
               YADAGUI('tab');
               if MatlabNumber>=7;
                  uicontrol(controls.tab6);
               end;
               set(controls.about,'UserData',1);
               drawnow;
            end;
            %
            % check if we should add some diving line to the log-text control
            %
            LogText = get(controls.estimation_log,'String');
            if isempty(LogText)==0;
               SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
            end;
            %
            % Run the Posterior mode estimation routine
            %
            AdaptiveLearningPosteriorModeEstimation(DSGEModel,CurrINI,controls);
            %
            % we need to enable these things
            %
            set(controls.about,'UserData',0);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PosteriorSampling'
      if get(controls.about,'UserData')==0;
         %
         % make the output tab visible
         %
         if DSGEModel.ShowEstimationLog==1;
            oldtab = get(controls.tab1,'UserData');
            set(maingui,'CurrentObject',controls.tab6);
            YADAGUI('tab');
            if MatlabNumber>=7;
               uicontrol(controls.tab6);
            end;
            set(controls.about,'UserData',1);
            drawnow;
         end;
         %
         % Fix the log text control
         %
         LogText = get(controls.estimation_log,'String');
         if isempty(LogText)==0;
            SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
         end;
         %
         % run the posterior sampler
         %
         if DSGEModel.PosteriorSampler==1;
            %
            % random walk Metropolis, with normal proposal
            %
            AdaptiveLearningDSGERWMPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==2;
            %
            % slice sampler
            %
            AdaptiveLearningDSGESlicePosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==3;
            %
            % random walk Metropolis, with Student-t proposal
            %
            AdaptiveLearningDSGERWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==4;
            %
            % fixed blocking RWM with normal proposal
            %
            AdaptiveLearningDSGEFixedBlockingRWMPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==5;
            %
            % fixed blocking RWM with a Student-t proposal
            %
            AdaptiveLearningDSGEFixedBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==6;
            %
            % random blocking RWM with a normal proposal
            %
            AdaptiveLearningDSGERandomBlockingRWMPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==7;
            %
            % random blocking RWM with a Student-t proposal
            %
            AdaptiveLearningDSGERandomBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==8;
            %
            % SMC with likelihood tempering
            %
            AdaptiveLearningDSGESMCLikelihoodTemperingPosteriorSampler(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==9;
            %
            % SMC with data tempering
            %
            AdaptiveLearningDSGESMCDataTemperingPosteriorSampler(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==10;
            %
            % IS based on MitISEM
            %
            AdaptiveLearningDSGEISMitISEMPosteriorSampler(DSGEModel,CurrINI,controls);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PriorSampling'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % get parameters and priors
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % read the prior file for the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                            thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                            LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % check if we have parameters to estimate
               %
               if isempty(ParameterNames.estimated)==0;
                  %
                  % suggest how many prior draws to compute
                  %
                  PriorDrawsStr = get(controls.posterior.postdrawsusage,'String');
                  PriorDrawsUsageValue = DSGEModel.PostDrawsUsageValue;
                  if (PriorDrawsUsageValue<10)&(isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0);
                     if FileExist(DSGEModel.SystemPriorFile)==1;
                        PriorDrawsUsageValue = 10;
                     end;
                  end;
                  %
                  % open a selection dialog
                  %
                  [action,PriorDrawsValue] = SelectionDlg(PriorDrawsStr,PriorDrawsUsageValue,'Select the number of draws from the prior distribution:','Sampling Selection','Prior Sampling','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     NumPriorDraws = str2num(strrep(StringTrim(PriorDrawsStr(PriorDrawsValue,:)),',',''));
                     %
                     % run the prior sampling function
                     %
                     if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
                        if FileExist(DSGEModel.SystemPriorFile)==1;
                           %
                           % we need to construct a number of variables that are stored in DSGEModel.AdaptiveLearning
                           % as the fields r, f, Sperp, SMatrix, Gtilde and Kgf
                           %
                           f = length(DSGEModel.ForwardVariablePositions);
                           r = length(DSGEModel.StateVariablePositions);
                           %
                           % compute the BeliefSys structure
                           %
                           if DSGEModel.BeliefSystem==1;
                              BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           else;
                              [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                              pause(0.1);
                              drawnow;
                              BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           end;
                           %
                           % get the adaptive learning input variables
                           %
                           [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
                           DSGEModel.AdaptiveLearning.r = r;
                           DSGEModel.AdaptiveLearning.f = f;
                           DSGEModel.AdaptiveLearning.Sperp = Sperp;
                           DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
                           DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
                           DSGEModel.AdaptiveLearning.Kgf = Kgf;
                           DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
                           %
                           % now run the system prior sampler
                           %
                           DSGESystemPriorSampling(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,NumPriorDraws,1,DSGEModel,CurrINI,controls);
                        else;
                           txt = ['The system prior density file "' GetFilename(DSGEModel.SystemPriorFile) '" doesn''t exist in the directory "' GetPath(DSGEModel.SystemPriorFile) '". YADA will therefore remove this setting.'];
                           About(txt,'error','Prior Sampling - Missing System Prior File',400,500,CurrINI);
                           DSGEModel.SystemPriorFile = pwd;
                           set(controls.dsge.systempriorfile,'String',DSGEModel.SystemPriorFile);
                           drawnow;
                        end;
                     else;
                        DSGEPriorSampling(PriorDist,thetaDist,LowerBound,NumPriorDraws,1,'Prior Sampling',DSGEModel,CurrINI);
                     end;
                  end;
               else;
                  txt = 'All parameters of your DSGE model and adaptive learning mechanism have status calibrated. Hence, there is no prior distribution from which YADA can sample.';
                  About(txt,'information','Prior Sampling - Calibrated DSGE Model',200,500,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '".'];
                About(txt,'information','Prior Sampling',200,500,CurrINI);
            end;
         end;     
         %
         set(controls.about,'UserData',0);
      end;
   case 'SystemPriorModeEstimation'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % get parameters and priors
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % read the prior file for the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                            thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                            LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % check if we have parameters to estimate
               %
               if isempty(ParameterNames.estimated)==0;
                  %
                  % we can now run the system prior mode estimation routine
                  %
                  SystemPriorModeEstimation(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,1,DSGEModel,CurrINI,controls);
                  %
               else;
                  txt = 'All parameters of your DSGE model and adaptive learning mechanism have status calibrated. Hence, there is no prior distribution from which YADA can sample.';
                  About(txt,'information','System Prior Mode Estimation - Calibrated DSGE Model',200,500,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '".'];
                About(txt,'information','System Prior Mode Estimation',200,500,CurrINI);
            end;
         end;         
         set(controls.about,'UserData',0);
      end;
   case 'SaveInverseHessian'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % get parameters and priors
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % read the prior file for the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                            thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                            LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % check if we have parameters to estimate
               %
               if isempty(ParameterNames.estimated)==0;
                  %
                  % select parameter values from a mat-file
                  %
                  if MatlabNumber>=6;
                     ImportValuesFile = FixFilePath([DSGEModel.OutputDirectory '\']);
                  else;
                     ImportValuesFile = FixFilePath([DSGEModel.OutputDirectory '\*.mat']);
                  end;
                  if MatlabNumber>=6;
                     FileType = {'*.mat', 'Matlab mat Files (*.mat)'};
                     [FileName,FilePath] = uigetfile(FileType,'Parameter Values File - Select File',ImportValuesFile);
                  else;
                     [FileName,FilePath] = uigetfile(ImportValuesFile,'Parameter Values File - Select File');
                  end;
                  drawnow;
                  pause(0.02);
                  %
                  if (isnumeric(FileName)==0)&(isunix==1);
                     if strcmp(FilePath(1,length(FilePath)),'/')==0;
                        FilePath = [FilePath '/'];
                     end;
                  end;
                  %
                  % check if a file was selected
                  %
                  if (isnumeric(FileName)==0)&(FileExist([FilePath FileName])==1);
                     ImportValuesFile = [FilePath FileName];
                     %
                     % try to load the file
                     %
                     ErrorStr = '';
                     if MatlabNumber>=8.0;
                        try;
                           ImportValuesData = load(ImportValuesFile);
                        catch ME;
                           ErrorStr = ['Unable to load the file "' ImportValuesFile '". Message caught is: ' YADALastError(ME)];
                        end;
                     else;
                        try;
                           ImportValuesData = load(ImportValuesFile);
                        catch;
                           ErrorStr = ['Unable to load the file "' ImportValuesFile '". Message caught is: ' YADALastError];
                        end;
                     end;
                     %
                     if isempty(ErrorStr)==1;
                        %
                        % check which type of file we have
                        %
                        SelectValues = 1;
                        if (length(FileName)>=17)&(strcmp(FileName(1,1:13),'MargLikeData-')==1);
                           %
                           % we have a file based on posterior sampling
                           %
                           ParameterValuesChoices = strvcat('thetaMargMode','ParData.SampleMean','ParData.SampleMedian');
                           DisplayChoices = strvcat('Marginal posterior theta mode','Posterior theta mean','Posterior theta median');
                        elseif (length(FileName)>=18)&(strcmp(FileName(1,1:14),'PosteriorMode-')==1);
                           %
                           % we have a posterior mode file
                           %
                           ParameterValuesChoices = strvcat('theta','thetaMode');
                           DisplayChoices = strvcat('Initial theta values','Joint posterior theta mode');
                        else;
                           %
                           % we have some other file
                           %
                           if length(fieldnames(ImportValuesData))>0;
                              DisplayChoices = char(fieldnames(ImportValuesData));
                              ParameterValuesChoices = DisplayChoices;
                           else;
                              SelectValues = 0;
                              txt = ['The file you selected "' GetFilename(ImportValuesFile) '" is empty. Hence, there are no parameters for which an inverse Hessian matrix can be computed.'];
                              About(txt,'information',['Save Inverse Hessian - ' GetFilename(ImportValuesFile)],300,600,CurrINI);
                           end;
                        end;
                        %
                        % select which variable is the appropriate choice
                        %
                        if SelectValues==1;
                           if size(DisplayChoices,1)>1;
                              [action,SelectedOption] = SelectionDlg(DisplayChoices,1,'Select variable for theta values:','Theta Values Selection','Save Inverse Hessian','','',CurrINI);
                           else;
                              action = 'ok';
                              SelectedOption = 1;
                           end;
                           if strcmp(lower(action),'ok')==1;
                              thetaMode = vec(eval(['ImportValuesData.' StringTrim(ParameterValuesChoices(SelectedOption,:))]));
                              %
                              % check if the length of thetaMode is consistent with theta
                              %
                              NumParam = length(theta);
                              if length(thetaMode)==NumParam;
                                 %
                                 % covert to phi
                                 %
                                 phiMode = ThetaToPhi(thetaMode,thetaIndex,UniformBounds,LowerBound);
                                 %
                                 % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
                                 %
                                 [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
                                 [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
                                 if FileExist(DSGEModel.UpdateParameterFile)==1;
                                    [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
                                 end;
                                 if FileExist(DSGEModel.InitializeParameterFile)==1;
                                    [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
                                 end;
                                 [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
                                 %
                                 % check if we should also copy the belief system file
                                 %
                                 if DSGEModel.BeliefSystem==2;
                                    [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                                 end;
                                 %
                                 % check if we should copy model solver file
                                 %
                                 if DSGEModel.ModelSolver==4;
                                    if FileExist(DSGEModel.ModelSolverFile)==1;
                                       [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
                                    end;
                                    if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
                                       [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
                                    end;
                                 end;
                                 try;
                                    AIMData = load(DSGEModel.AIMDataFile);
                                 catch;
                                    AIMData = '';
                                 end;
                                 if isempty(AIMData)==0;
                                    ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
                                    %
                                    % collect variables required for the calculations
                                    %
                                    StepLengthStr = get(controls.optimize.steplength,'String');
                                    StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                                    if DSGEModel.BeliefSystem==1;
                                       BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                                    else;
                                       BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                                    end;
                                    r = length(DSGEModel.StateVariablePositions);
                                    f = length(DSGEModel.ForwardVariablePositions);
                                    k = DSGEModel.k;
                                    m = BeliefSys.m;
                                    G = BeliefSys.G;
                                    SMatrix = eye(r);
                                    Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
                                    SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
                                    %
                                    % get the structural form matrices
                                    %
                                    [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
                                    %
                                    % check the solution status
                                    %
                                    [tstatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
                                    if tstatus==1;
                                       %
                                       % solve the model under rational expectations
                                       %
                                       [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                                       if mcode==1;
                                          %
                                          % get estimation sample
                                          %
                                          [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
                                          if k>0
                                             x = DSGEModel.X(:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
                                          else;
                                             x = [];
                                          end;
                                          %
                                          % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
                                          %
                                          [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
                                          %
                                          % define xbar
                                          %
                                          ConstPos = 0;
                                          if k>0;
                                             ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
                                             if ConstPos==0;
                                                %
                                                % we will now check if X has a unit vector
                                                %
                                                X = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
                                                T = size(X,2);
                                                for j=1:k;
                                                   ConstTest = (DSGEModel.X(j,DSGEModel.FirstPeriod:DSGEModel.LastPeriod)==ones(1,T));
                                                   if sum(ConstTest)==T;
                                                      ConstPos = j;
                                                      break;
                                                   end;
                                                end;
                                             end;
                                             xbar = zeros(k,1);
                                             xbar(ConstPos) = 1;
                                          else;
                                             xbar = [];
                                          end;
                                          %
                                          % try to compute the log posterior kernel
                                          %
                                          ErrorStr = '';
                                          if MatlabNumber>=8.0;
                                             try;
                                                logPost = logPosteriorAdaptiveLearningPhiDSGE(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                             catch;
                                                ErrorStr = ['Unable to calculate the log posterior at the selected theta value. Message caught is: ' YADALastError(ME)];
                                             end;
                                          else;
                                             try;
                                                logPost = logPosteriorAdaptiveLearningPhiDSGE(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                             catch;
                                                ErrorStr = ['Unable to calculate the log posterior at the selected theta value. Message caught is: ' YADALastError];
                                             end;
                                          end;
                                          if isempty(ErrorStr)==1;
                                             %
                                             % compute the inverse Hessian
                                             %
                                             txt = ['Please wait while YADA attempts to compute the inverse Hessian at the selected parameter values using finite differences. Computations started at: ' StringTrim(datestr(now,14))];
                                             WaitHandle = WaitDLG(txt,'information','Computing Inverse Hessian',600,150,CurrINI,0);
                                             WaitControls = get(WaitHandle,'UserData');
                                             drawnow;
                                             tic;
                                             %
                                             VechHessian = YADAHessianMatrix('logPosteriorAdaptiveLearningPhiDSGE',phiMode,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                                             CompTime = toc;
                                             %
                                             set(WaitControls.text,'UserData','done');
                                             delete(WaitHandle);
                                             drawnow;
                                             pause(0.02);
                                             %
                                             HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
                                             %
                                             % check if the matrix is positive definite
                                             %
                                             if IsPositiveDefinite(HessianMat)==1;
                                                InverseHessian = HessianMat\eye(NumParam);
                                                %
                                                % create the directory hessian and save the results there
                                                %
                                                OutputDir = FixFilePath([DSGEModel.OutputDirectory '\hessian']);
                                                stat = MakeDir(DSGEModel.OutputDirectory,'hessian');
                                                if stat~=1;
                                                   OutputDir = DSGEModel.OutputDirectory;
                                                end;
                                                %
                                                % save to file
                                                %
                                                SaveNo = 1;
                                                DoSave = 0;
                                                while DoSave==0;
                                                   MatFile = FixFilePath([OutputDir '\InverseHessian-AdaptiveLearning-' DSGEModel.NameOfModel '-' int2str(SelectedOption) '.' int2str(SaveNo) '.mat']);
                                                   if FileExist(MatFile)==0;
                                                      %
                                                      % save to disk
                                                      %
                                                      save(MatFile,'thetaMode','phiMode','InverseHessian','ParameterNames','DSGEModel');
                                                      DoSave = 1;
                                                      txt = ['YADA has saved the inverse Hessian at the selected theta value and the additional variables to the file "' GetFilename(MatFile) '", which is located in the directory "' OutputDir '". Computation time: ' SecondsToHoursStr(CompTime) ' (hrs:min:sec)'];
                                                      About(txt,'information','Save Inverse Hessian',300,600,CurrINI);
                                                   else;
                                                      SaveNo = SaveNo+1;
                                                   end;
                                                end;
                                             else;
                                                txt = 'The Hessian matrix computed from your selected theta vector is not positive definite. YADA is therefore unwilling to store the results.'
                                                About(txt,'information','YADA - DSGE Model Problem',500,600,CurrINI);
                                             end;
                                          else;
                                             About(ErrorStr,'error','Error - Log Posterior',500,600,CurrINI);
                                          end;
                                       else;
                                          if isempty(ErrMsg)==1;
                                             ErrMsg = 'YADA was unable to solve the model under rational expectations for the selected parameter values.';
                                          end;
                                          About(ErrMsg,'information','YADA - DSGE Model Problem',500,600,CurrINI);
                                       end;
                                    else;
                                       txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem.';
                                       About(txt,'information','YADA - DSGE Model Problem',200,600,CurrINI);
                                    end;
                                 else;
                                    if MatlabNumber>=8.0;
                                       txt = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
                                    else;
                                       txt = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
                                    end;
                                    About(txt,'information','YADA - Bad MAT File',500,600,CurrINI);
                                 end;
                              else;
                                 txt = ['The number of elements of the variable you selected "' StringTrim(DisplayChoices(SelectedOption,:)) '" is ' int2str(length(thetaMode)) ' and it doesn''t match the number of parameters in theta ' int2str(NumParam) '. YADA is therefore unable to compute and save a suitable inverse Hessian.'];
                                 About(txt,'information','Save Inverse Hessian',300,600,CurrINI);
                              end;
                           end;
                        end;
                     else;
                        About(ErrorStr,'error',['Error - ' GetFilename(ImportValuesFile)],CurrINI.scrsz(4)-50,600,CurrINI);
                     end;
                  end;
               else;
                  txt = 'All parameters of your DSGE model and adaptive learning mechanism have status calibrated. Hence, there are no parameters for which an inverse Hessian matrix can be computed.';
                  About(txt,'information','Save Inverse Hessian - Calibrated DSGE Model',200,500,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '".'];
                About(txt,'information','Save Inverse Hessian',200,500,CurrINI);
            end;
         end; 
         %
         set(controls.about,'UserData',0);
      end;
   case 'PriorInformation'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % collect the information
         %
         [InitStatus,theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = VerifyPriorData(DSGEModel,CurrINI);
         if InitStatus==1;
            %
            % Transform PriorDist.gamma and PriorDist.beta to the relevant parameters that are used by the logGammaPDF and
            % the logBetaPDF functions
            %
            if isempty(PriorDist.beta)==0;
               [a,b] = MomentToParamStdbetaPDF((PriorDist.beta(:,1)-PriorDist.beta(:,3))./(PriorDist.beta(:,4)-PriorDist.beta(:,3)),PriorDist.beta(:,2)./(PriorDist.beta(:,4)-PriorDist.beta(:,3)));
               PriorDist.beta_ab = [a b];
            else;
               PriorDist.beta_ab = [];
            end;
            %
            if isempty(PriorDist.gamma)==0;
               [a,b] = MomentToParamGammaPDF(PriorDist.gamma(:,1)-PriorDist.gamma(:,3),PriorDist.gamma(:,2));
               PriorDist.gamma_ab = [a b];
            else;
               PriorDist.gamma_ab = [];
            end;
            if isempty(PriorDist.logistic)==0;
               [a,b] = MomentToParamLogisticPDF(PriorDist.logistic(:,1),PriorDist.logistic(:,2),PriorDist.logistic(:,3));
               PriorDist.logistic_ab = [a b PriorDist.logistic(:,3)];
            else;
               PriorDist.logistic_ab = [];
            end;
            if isempty(PriorDist.gumbel)==0;
               [a,b] = MomentToParamGumbelPDF(PriorDist.gumbel(:,1),PriorDist.gumbel(:,2));
               PriorDist.gumbel_ab = [a b];
            else;
               PriorDist.gumbel_ab = [];
            end;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % collect additional information
               %
               AdditionalSystemPriorInfo = '';
               ErrorStr  = '';
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  %
                  % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
                  %
                  [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
                  [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
                  if FileExist(DSGEModel.UpdateParameterFile)==1;
                     [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
                  end;
                  if FileExist(DSGEModel.InitializeParameterFile)==1;
                     [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
                  end;
                  [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
                  if DSGEModel.BeliefSystem~=1;
                     [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                  end;
                  %
                  % check if we should copy model solver file
                  %
                  if DSGEModel.ModelSolver==4;
                     if FileExist(DSGEModel.ModelSolverFile)==1;
                        [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
                     end;
                     if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
                        [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
                     end;
                  end;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch;
                     AIMData = '';
                  end;
                  if isempty(AIMData)==0;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     %
                     if (DSGEModel.RunInitializeFirst==1)&(FileExist(DSGEModel.InitializeParameterFile)==1);
                        %
                        % we need to run the file DSGEModel.InitializeParameterFile first
                        %
                        if MatlabNumber>=8.0;
                           try;
                              ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
                           catch ME;
                              ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError(ME)];
                           end;
                        else;
                           try;
                              ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
                           catch;
                              ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError];
                           end;
                        end;
                        if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ErrorStr)==1);
                           if MatlabNumber>=8.0;
                              try;
                                 ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
                              catch ME;
                                 ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError(ME)];
                              end;
                           else;
                              try;
                                 ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
                              catch;
                                 ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError];
                              end;
                           end;
                        end;
                     else;
                        if FileExist(DSGEModel.UpdateParameterFile)==1;
                           if MatlabNumber>=8.0;
                              try;
                                 ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
                              catch ME;
                                 ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError(ME)];
                              end;
                           else;
                              try;
                                 ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
                              catch;
                                 ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError];
                              end;
                           end;
                        end;
                        if (FileExist(DSGEModel.InitializeParameterFile)==1)&(isempty(ErrorStr)==1);
                           if MatlabNumber>=8.0;
                              try;
                                 ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
                              catch ME;
                                 ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError(ME)];
                              end;
                           else;
                              try;
                                 ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
                              catch;
                                 ErrorStr = ['Error found in "' GetName(DSGEModel.InitializeParameterFile) '.m". Message caught is: ' YADALastError];
                              end;
                           end;
                        end;
                     end;
                     %
                     % and finally we can run the system prior file
                     %
                     if isempty(ErrorStr)==1;
                        if DSGEModel.BeliefSystem==1;
                           BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                        else;
                           BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                        end;
                        [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
                        DSGEModel.AdaptiveLearning.r = length(DSGEModel.StateVariablePositions);
                        DSGEModel.AdaptiveLearning.f = size(SMatrix,2);
                        DSGEModel.AdaptiveLearning.Sperp = Sperp;
                        DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
                        DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
                        DSGEModel.AdaptiveLearning.Kgf = Kgf;
                        DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
                        if MatlabNumber>=8.0;
                           try;
                              [logSP,AdditionalSystemPriorInfo] = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
                           catch ME;
                              ErrorStr = ['Error found in "' GetName(DSGEModel.SystemPriorFile) '.m". Message caught is: ' YADALastError(ME)];
                           end;
                        else;
                           try;
                              [logSP,AdditionalSystemPriorInfo] = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
                           catch;
                              ErrorStr = ['Error found in "' GetName(DSGEModel.SystemPriorFile) '.m". Message caught is: ' YADALastError];
                           end;
                        end;
                     end;
                  end;
               end;
               %
               if isempty(ErrorStr)==1;
                  %
                  % print to file
                  %
                  TextFile = PrintPriorInformation(ParameterNames,theta,thetaDist,PriorDist,AdditionalSystemPriorInfo,1,DSGEModel,CurrINI);
                  %
                  % check if file exists and display the file in dialog.
                  %
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Prior Distribution - ' GetFilename(DSGEModel.PriorFile)],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Information',40,CurrINI,1,1);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Bad File',500,600,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ParameterValuesInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % get parameters and priors
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               if FileExist(DSGEModel.UpdateParameterFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
                  %
                  % run the file as well
                  %
                  OldModelParameters = ModelParameters;
                  ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
                  %
                  % check to see if there are any "new" parameters
                  %
                  newnames = fieldnames(ModelParameters);
                  oldnames = fieldnames(OldModelParameters);
                  if length(newnames)>length(oldnames);
                     %
                     % add any valied new parameter names to the calibrated group
                     %
                     for i=length(oldnames)+1:length(newnames);
                        if (strcmp(newnames{i},'UserVariables')==0)&(strcmp(newnames{i},'YADA')==0);
                           ParameterNames.additional = strvcat(ParameterNames.additional,newnames{i});
                           ParameterNames.all = strvcat(ParameterNames.all,newnames{i});
                        end;
                     end;
                     %
                     % check if the calibrated parameters are actually updated
                     %
                     ParameterNames = CheckUpdateParameterNames(ParameterNames,ModelParameters,theta,thetaPositions,DSGEModel);
                  end;
               end;
               %
               % make sure that the belief system information is sent to the print function.
               %
               OldDSGEModel = DSGEModel;
               if DSGEModel.BeliefSystem==1;
                  DSGEModel.BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
               else;
                  if DSGEModel.BeliefSystem==2;
                     [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                  end;
                  DSGEModel.BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
               end;
               %
               % write the result to a text file
               %
               ResFile = FixFilePath([DSGEModel.OutputDirectory '\ParameterValues-AdaptiveLearning-' DSGEModel.NameOfModel '-InitialValues.txt']);
               PrintParameterValues(ResFile,DSGEModel,CurrINI,ParameterNames,ModelParameters,1,'Initial values');
               %
               DSGEModel = OldDSGEModel;
               if FileExist(ResFile)==1;
                  TextGUI(ResFile,['Adaptive Learning: Parameter Values - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,1);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to display the parameter values based on initial parameter values.'];
               About(txt,'information','Parameter Values - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ParameterValuesDSGEMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               if FileExist(DSGEModel.UpdateParameterFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
                  %
                  % run the file as well
                  %
                  OldModelParameters = ModelParameters;
                  ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
                  %
                  % check to see if there are any "new" parameters
                  %
                  newnames = fieldnames(ModelParameters);
                  oldnames = fieldnames(OldModelParameters);
                  if length(newnames)>length(oldnames);
                     %
                     % add any valied new parameter names to the calibrated group
                     %
                     for i=length(oldnames)+1:length(newnames);
                        if (strcmp(newnames{i},'UserVariables')==0)&(strcmp(newnames{i},'YADA')==0);
                           ParameterNames.additional = strvcat(ParameterNames.additional,newnames{i});
                           ParameterNames.all = strvcat(ParameterNames.all,newnames{i});
                        end;
                     end;
                     %
                     % check if the calibrated parameters are actually updated
                     %
                     ParameterNames = CheckUpdateParameterNames(ParameterNames,ModelParameters,theta,thetaPositions,DSGEModel);
                  end;
               end;
               %
               % make sure that the belief system information is sent to the print function.
               %
               OldDSGEModel = DSGEModel;
               if DSGEModel.BeliefSystem==1;
                  DSGEModel.BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
               else;
                  if DSGEModel.BeliefSystem==2;
                     [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                  end;
                  DSGEModel.BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
               end;
               %
               % write the result to a text file
               %
               ResFile = FixFilePath([DSGEModel.OutputDirectory '\ParameterValues-AdaptiveLearning-' DSGEModel.NameOfModel '-DSGEPosteriorMode.txt']);
               PrintParameterValues(ResFile,DSGEModel,CurrINI,ParameterNames,ModelParameters,1,'DSGE Posterior mode');
               %
               DSGEModel = OldDSGEModel;
               if FileExist(ResFile)==1;
                  TextGUI(ResFile,['Adaptive Learning: Parameter Values - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'DSGE Posterior Mode Values',50,CurrINI,1,1);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to display the parameter values based on the DSGE model posterior mode estimates.'];
               About(txt,'information','Parameter Values - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ParameterValuesPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
            ParameterNames = ModeData.ParameterNames;
            if FileExist(DSGEModel.UpdateParameterFile)==1;
               [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
               %
               % run the file as well
               %
               OldModelParameters = ModelParameters;
               ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
               %
               % check to see if there are any "new" parameters
               %
               newnames = fieldnames(ModelParameters);
               oldnames = fieldnames(OldModelParameters);
               if length(newnames)>length(oldnames);
                  %
                  % add any valied new parameter names to the calibrated group
                  %
                  for i=length(oldnames)+1:length(newnames);
                     if (strcmp(newnames{i},'UserVariables')==0)&(strcmp(newnames{i},'YADA')==0);
                        ParameterNames.additional = strvcat(ParameterNames.additional,newnames{i});
                        ParameterNames.all = strvcat(ParameterNames.all,newnames{i});
                     end;
                  end;
                  %
                  % check if the calibrated parameters are actually updated
                  %
                  ParameterNames = CheckUpdateParameterNames(ParameterNames,ModelParameters,theta,thetaPositions,DSGEModel);
               end;
            end;
            %
            % make sure that the belief system information is sent to the print function.
            %
            OldDSGEModel = DSGEModel;
            if DSGEModel.BeliefSystem==1;
               DSGEModel.BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
            else;
               if DSGEModel.BeliefSystem==2;
                  [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
               end;
               DSGEModel.BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
            end;
            %
            % write the result to a text file
            %
            ResFile = FixFilePath([DSGEModel.OutputDirectory '\ParameterValues-AdaptiveLearning-' DSGEModel.NameOfModel '-PosteriorMode.txt']);
            PrintParameterValues(ResFile,DSGEModel,CurrINI,ParameterNames,ModelParameters,1,'Posterior mode');
            %
            DSGEModel = OldDSGEModel;
            if FileExist(ResFile)==1;
               TextGUI(ResFile,['Adaptive Learning: Parameter Values - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Values',50,CurrINI,1,1);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PosteriorModeResults'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % get the file from the currently clicked object on the menu
         %
         ModeTextFile = get(gcbo,'UserData');
         TextGUI(ModeTextFile,['Posterior Mode Estimation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
         %
         set(controls.about,'UserData',0);
      end;
   case 'PostSampleSummary'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         PostSampleTextFile = get(gcbo,'UserData');
         TextGUI(PostSampleTextFile,['Posterior Sampling - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
         set(controls.about,'UserData',0);
      end;
   case 'RawPlots'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
            end;
            if DSGEModel.PosteriorSampler<=7;
               HeaderStr = 'MCMC Chain Selection';
            elseif (DSGEModel.PosteriorSampler==8)|(DSGEModel.PosteriorSampler==9);
               HeaderStr = 'SMC Chain Selection';
            elseif DSGEModel.PosteriorSampler==10;
               HeaderStr = 'Importance Sample Selection';
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load',HeaderStr,'Raw Posterior Draws','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               NumPostDraws = length(LogPost);
               %
               % check which parameters to use
               %
               if gcbo==controls.learning.viewmenu.postsample.rawplots.original;
                  ParamSample = thetaPostSample;
                  TypeStr = 'Original';
               elseif gcbo==controls.learning.viewmenu.postsample.rawplots.transformed;
                  ParamSample = PostSample;
                  TypeStr = 'Transformed';
               elseif gcbo==controls.learning.viewmenu.postsample.rawplots.weights;
                  ParamSample = Weights;
                  TypeStr = 'Weights';
               elseif gcbo==controls.learning.viewmenu.postsample.rawplots.logpost;
                  ParamSample = LogPost;
                  TypeStr = 'Log Posterior Kernel Values';
                  thetaPost = thetaPostSample;
               end;
               %
               % clear posterior sample data
               %
               clear('PostSample','thetaPostSample','LogPost','AcceptedDraws','Weights');
               %
               % ask if the user wants to plot a subsample
               %
               if (gcbo==controls.learning.viewmenu.postsample.rawplots.original)|(gcbo==controls.learning.viewmenu.postsample.rawplots.transformed);
                  if DSGEModel.PosteriorSampler<=7;
                     txt = 'Would you like to plot a sub-sample of the posterior draws rather than the full sample?';
                     answer = Query(txt,'question',400,['Raw Posterior Draws - ' TypeStr ' Parameters'],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        ParamSample = ParamSample(NumBurnin+1:NumPostDraws,:);
                        NumDraws = NumPostDraws-NumBurnin;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        ParamSample = ParamSample(UseDraws,:);
                     end;
                  end;
                  %
                  % send the data to the raw plotting function
                  %
                  drawnow;
                  RawPosteriorPlots(ParamSample,NumBurnin,NumPostDraws,CurrChain,TypeStr,1,DSGEModel,CurrINI);
               elseif gcbo==controls.learning.viewmenu.postsample.rawplots.weights;
                  %
                  % send the data to the raw plotting function
                  %
                  drawnow;
                  RawWeightsPosteriorPlots(ParamSample,NaN,NumPostDraws,CurrChain,DSGEModel,CurrINI,controls);
               elseif gcbo==controls.learning.viewmenu.postsample.rawplots.logpost;
                  if DSGEModel.PosteriorSampler<=7;
                     txt = 'Would you like to plot a sub-sample of the log posterior kernel values for the posterior draws rather than the full sample?';
                     answer = Query(txt,'question',400,['Raw Posterior Draws - ' TypeStr],500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        ParamSample = ParamSample(NumBurnin+1:NumPostDraws,:);
                        NumDraws = NumPostDraws-NumBurnin;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        ParamSample = ParamSample(UseDraws,:);
                        thetaPost = thetaPost(UseDraws,:);
                        drawnow;
                     end;
                  end;
                  %
                  RawLogPosteriorPlot(ParamSample,thetaPost,NumBurnin,NumPostDraws,CurrChain,1,DSGEModel,CurrINI);
                  clear('thetaPost');
               end;
               %
               % clear the used parameter draws
               %
               clear('ParamSample');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ScatterPlots'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
            end;
            if DSGEModel.PosteriorSampler<=7;
               HeaderStr = 'MCMC Chain Selection';
            elseif (DSGEModel.PosteriorSampler==8)|(DSGEModel.PosteriorSampler==9);
               HeaderStr = 'SMC Chain Selection';
            elseif DSGEModel.PosteriorSampler==10;
               HeaderStr = 'Importance Sample Selection';
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load',HeaderStr,'Scatter-Plot Posterior Draws','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            %
            % check if the data actually exists
            %
            if isempty(ErrorStr)==1;
               if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  PostSample = PostSample(NumBurnin+1:NumDraws,:);
                  LogPost = LogPost(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which parameters to use
                  %
                  if gcbo==controls.learning.viewmenu.postsample.scatterplots.original;
                     ParamSample = thetaPostSample;
                     TypeStr = 'Original';
                     ParamMode = ModeData.thetaMode;
                  elseif gcbo==controls.learning.viewmenu.postsample.scatterplots.transformed;
                     ParamSample = PostSample;
                     TypeStr = 'Transformed';
                     ParamMode = ModeData.phiMode;
                  end;
                  %
                  % clear posterior sample data
                  %
                  clear('PostSample','thetaPostSample','AcceptedDraws');
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        ParamSample = ParamSample(UseDraws,:);
                        LogPost = LogPost(UseDraws);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % send the data to the scatter-plotting function
                  %
                  drawnow;
                  ScatterPlotPosteriorDraws('init',ParamSample,LogPost,ModeData.ParameterNames.estimated,ParamMode,CurrChain,'DSGE',TypeStr,DSGEModel,CurrINI);
                  %
                  % clear the used parameter draws
                  %
                  clear('ParamSample','LogPost');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LaplaceInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % get parameters and priors
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % transform the parameters
                     %
                     phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
                     LogJacobianMode = logJacobian(phi,thetaIndex,UniformBounds);
                     LogPriorMode = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
                     Status = 1;
                     ErrorStr = '';
                     ErrorFile = '';
                     if MatlabNumber>=8.0;
                        try;
                           AIMData = load(DSGEModel.AIMDataFile);
                        catch ME;
                           ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
                           ErrorFile = DSGEModel.AIMDataFile;
                        end;
                     else;
                        try;
                           AIMData = load(DSGEModel.AIMDataFile);
                        catch;
                           ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
                           ErrorFile = DSGEModel.AIMDataFile;
                        end;
                     end;
                     if isempty(ErrorStr)==1;
                        %
                        % try to load the AIMData, other wise use ModeData
                        %
                        if MatlabNumber>=8.0;
                           try;
                              AIMData = load(DSGEModel.AIMDataFile);
                           catch ME;
                              AIMData = ModeData.AIMData;
                           end;
                        else;
                           try;
                              AIMData = load(DSGEModel.AIMDataFile);
                           catch;
                              AIMData = ModeData.AIMData;
                           end;
                        end;
                        if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
                           %
                           % we need to construct a number of variables that are stored in DSGEModel.AdaptiveLearning
                           % as the fields r, f, Sperp, SMatrix, Gtilde and Kgf
                           %
                           f = length(DSGEModel.ForwardVariablePositions);
                           r = length(DSGEModel.StateVariablePositions);
                           %
                           % compute the BeliefSys structure
                           %
                           if DSGEModel.BeliefSystem==1;
                              BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           else;
                              BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           end;
                           %
                           % get the adaptive learning input variables
                           %
                           [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
                           DSGEModel.AdaptiveLearning.r = r;
                           DSGEModel.AdaptiveLearning.f = f;
                           DSGEModel.AdaptiveLearning.Sperp = Sperp;
                           DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
                           DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
                           DSGEModel.AdaptiveLearning.Kgf = Kgf;
                           DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
                           [logPrior,LogSPMode] = logSystemPriorPhiDSGE(phi,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,1,AIMData,DSGEModel,CurrINI);
                           SystemPriorModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
                           if FileExist(SystemPriorModeFile)==1;
                              SystemPriorModeData = load(SystemPriorModeFile);
                              %
                              % we only need the Laplace approximation of the log marginal likelihood of the system prior
                              %
                              LogMargLikeSystemLaplace = SystemPriorModeData.LogMargLikeSystemLaplace;
                           else;
                              LogMargLikeSystemLaplace = NaN;
                           end;
                           IsSystemPrior = 1;
                        else;
                           LogSPMode = 0;
                           LogMargLikeSystemLaplace = 0;
                           IsSystemPrior = 0;
                        end;
                     else;
                        About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
                        drawnow;
                        Status = 0;
                     end;
                     if Status==1;
                        %
                        % compute the inverse Hessian with finite differences
                        %
                        NumParam = length(phi);
                        StepLengthStr = get(controls.optimize.steplength,'String');
                        StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                        %
                        % set the InitialStateVector value
                        %
                        if DSGEModel.UseOwnInitialState==1;
                           if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
                              DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
                           else;
                              DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                           end;
                        else;
                           DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                        end;
                        [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
                        %
                        if exist('Gtilde')==0
                           %
                           % get the adaptive learning input variables
                           %
                           [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
                        end;
                        %
                        tic;
                        VechHessian = YADAHessianMatrix('logPosteriorAdaptiveLearningPhiDSGE',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                        HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
                        CompTime = toc;
                        %
                        % compute the height of the log posterior kernel
                        %
                        LogLikeDensity = sum(StateVarStructure.lnLt);
                        LogPostDensity = LogLikeDensity+LogJacobianMode+LogPriorMode+LogSPMode;
                        %
                        if isnan(LogMargLikeSystemLaplace)==0;
                           LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0)-LogMargLikeSystemLaplace;
                        else;
                           LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0);
                        end;
                        %
                        PrtVal = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                        [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\LaplaceMarginalLikelihood-AdaptiveLearning-' DSGEModel.NameOfModel '-InitialValues.txt']);
                        fid = fopen(TextFile,'wt');
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*           L A P L A C E   A P P R O X I M A T I O N   O F   T H E            *\n');
                        fprintf(fid,'*                    M A R G I N A L   L I K E L I H O O D                     *\n');
                        fprintf(fid,'*                      A D A P T I V E   L E A R N I N G                       *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        %
                        % sample information
                        %
                        fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                        fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                        fprintf(fid,'Estimation time:                  %s (hrs:min:sec)\n\n',SecondsToHoursStr(CompTime));
                        %
                        % now the marginal likelihood
                        %
                        fprintf(fid,['Log marginal likelihood:           ' PrtVal ' \n\n'],LogMargLike);
                        fprintf(fid,'where \n\n');
                        fprintf(fid,['   Log posterior density of phi:   ' PrtVal ' \n'],LogPostDensity);
                        if isnan(LogMargLikeSystemLaplace)==0;
                           fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity+LogMargLikeSystemLaplace);
                        else;
                           fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity);
                        end;
                        if IsSystemPrior==1;
                           if isnan(LogMargLikeSystemLaplace)==0;
                              fprintf(fid,['   Correction for the log marginal\n   likelihood of the system prior: ' PrtVal ' \n\n'],LogMargLikeSystemLaplace);
                           else;
                              fprintf(fid,'   The correction term for the log\n   marginal likelihood of the system\n   prior has not been computed yet.\n   Please run system prior mode estimation.\n\n');
                           end;
                        end;
                        fprintf(fid,'================================================================================\n\n');
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        fclose(fid);
                        %
                        % save results to mat-file
                        %
                        MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-LaplaceMarginalLikelihood-' DSGEModel.NameOfModel '-InitialValues.mat']);
                        save(MatFile,'LogMargLike','LogMargLikeSystemLaplace','LogLikeDensity','LogPostDensity');
                        %
                        % display the results
                        %
                        TextGUI(TextFile,['Laplace Approximation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'DSGE Posterior Mode',100,CurrINI,0,1);
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the Laplace approximation of the log marginal likelihood based on the initial values.';
                  About(txt,'information','Laplace Approximation - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the Laplace approximation of the log marginal likelihood based on the initial values.';
                  About(txt,'information','Laplace Approximation - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the Laplace approximation of the log marginal likelihood based on the initial values.'];
                About(txt,'information','Laplace Approximation - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LaplaceDSGEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % try to load the AIMData, other wise use ModeData
            %
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  AIMData = ModeData.AIMData;
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  AIMData = ModeData.AIMData;
               end;
            end;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % we now have the log-likelihood: sum(StateVarStructure.lnLt)
                     %
                     phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
                     LogJacobianMode = logJacobian(phi,thetaIndex,UniformBounds);
                     LogPriorMode = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
                     if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
                        %
                        % we need to construct a number of variables that are stored in DSGEModel.AdaptiveLearning
                        % as the fields r, f, Sperp, SMatrix, Gtilde and Kgf
                        %
                        f = length(DSGEModel.ForwardVariablePositions);
                        r = length(DSGEModel.StateVariablePositions);
                        %
                        % compute the BeliefSys structure
                        %
                        if DSGEModel.BeliefSystem==1;
                           BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                        else;
                           BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                        end;
                        %
                        % get the adaptive learning input variables
                        %
                        [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
                        DSGEModel.AdaptiveLearning.r = r;
                        DSGEModel.AdaptiveLearning.f = f;
                        DSGEModel.AdaptiveLearning.Sperp = Sperp;
                        DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
                        DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
                        DSGEModel.AdaptiveLearning.Kgf = Kgf;
                        DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
                        %
                        [logPrior,LogSPMode] = logSystemPriorPhiDSGE(phi,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,1,AIMData,DSGEModel,CurrINI);
                        SystemPriorModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
                        if FileExist(SystemPriorModeFile)==1;
                           SystemPriorModeData = load(SystemPriorModeFile);
                           %
                           % we only need the Laplace approximation of the log marginal likelihood of the system prior
                           %
                           LogMargLikeSystemLaplace = SystemPriorModeData.LogMargLikeSystemLaplace;
                        else;
                           LogMargLikeSystemLaplace = NaN;
                        end;
                        IsSystemPrior = 1;
                     else;
                        LogSPMode = 0;
                        LogMargLikeSystemLaplace = 0;
                        IsSystemPrior = 0;
                     end;
                     %
                     % compute the inverse Hessian with finite differences
                     %
                     NumParam = length(phi);
                     StepLengthStr = get(controls.optimize.steplength,'String');
                     StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                     %
                     % set the InitialStateVector value
                     %
                     if DSGEModel.UseOwnInitialState==1;
                        if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
                           DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
                        else;
                           DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                        end;
                     else;
                        DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                     end;
                     [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
                     %
                     if exist('Gtilde')==0
                        %
                        % get the adaptive learning input variables
                        %
                        [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
                     end;
                     %
                     tic;
                     VechHessian = YADAHessianMatrix('logPosteriorAdaptiveLearningPhiDSGE',phi,StepLength,CurrINI.NumWorkers,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                     HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
                     CompTime = toc;
                     %
                     % compute the height of the log posterior kernel
                     %
                     LogLikeDensity = sum(StateVarStructure.lnLt);
                     LogPostDensity = LogLikeDensity+LogJacobianMode+LogPriorMode+LogSPMode;
                     %
                     if isnan(LogMargLikeSystemLaplace)==0;
                        LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0)-LogMargLikeSystemLaplace;
                     else;
                        LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0);
                     end;
                     %
                     PrtVal = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                     [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                     %
                     TextFile = FixFilePath([DSGEModel.OutputDirectory '\LaplaceMarginalLikelihood-AdaptiveLearning-' DSGEModel.NameOfModel '-DSGEPostMode.txt']);
                     fid = fopen(TextFile,'wt');
                     fprintf(fid,'********************************************************************************\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'*           L A P L A C E   A P P R O X I M A T I O N   O F   T H E            *\n');
                     fprintf(fid,'*                    M A R G I N A L   L I K E L I H O O D                     *\n');
                     fprintf(fid,'*                      A D A P T I V E   L E A R N I N G                       *\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'********************************************************************************\n\n');
                     %
                     % sample information
                     %
                     fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     fprintf(fid,'Estimation time:                  %s (hrs:min:sec)\n\n',SecondsToHoursStr(CompTime));
                     %
                     % now the marginal likelihood
                     %
                     fprintf(fid,['Log marginal likelihood:           ' PrtVal ' \n\n'],LogMargLike);
                     fprintf(fid,'where \n\n');
                     fprintf(fid,['   Log posterior density of phi:   ' PrtVal ' \n'],LogPostDensity);
                     if isnan(LogMargLikeSystemLaplace)==0;
                        fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity+LogMargLikeSystemLaplace);
                     else;
                        fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity);
                     end;
                     if IsSystemPrior==1;
                        if isnan(LogMargLikeSystemLaplace)==0;
                           fprintf(fid,['   Correction for the log marginal\n   likelihood of the system prior: ' PrtVal ' \n\n'],LogMargLikeSystemLaplace);
                        else;
                           fprintf(fid,'   The correction term for the log\n   marginal likelihood of the system\n   prior has not been computed yet.\n   Please run system prior mode estimation.\n\n');
                        end;
                     end;
                     fprintf(fid,'================================================================================\n\n');
                     %
                     % Some information about the filename and its location
                     %
                     fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                     fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                     fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                     fclose(fid);
                     %
                     % save results to mat-file
                     %
                     MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-LaplaceMarginalLikelihood-' DSGEModel.NameOfModel '-DSGEPostMode.mat']);
                     save(MatFile,'LogMargLike','LogMargLikeSystemLaplace','LogLikeDensity','LogPostDensity');
                     %
                     % display the results
                     %
                     TextGUI(TextFile,['Laplace Approximation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'DSGE Posterior Mode',100,CurrINI,0,1);
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the Laplace approximation of the log marginal likelihood based on the DSGE posterior mode estimates.';
                  About(txt,'information','Laplace Approximation - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the Laplace approximation of the log marginal likelihood based on the DSGE posterior mode estimates.';
                  About(txt,'information','Laplace Approximation - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the Laplace approximation of the log marginal likelihood based on the DSGE posterior mode estimates.'];
               About(txt,'information','Laplace Approximation - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LaplacePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % try to load the AIMData, other wise use ModeData
            %
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  AIMData = ModeData.AIMData;
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  AIMData = ModeData.AIMData;
               end;
            end;
            %
            % now we call the state variable estimation function which also deals with the log-likelihood
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,0);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if (BeliefStatus==1)&(TransStatus==1);
               %
               % we're may be OK
               %
               if isempty(StateVarStructure)==0;
                  %
                  % we now have the log-likelihood: sum(StateVarStructure.lnLt)
                  %
                  LogJacobianMode = logJacobian(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
                  LogPriorMode = logPriorDSGE(ModeData.thetaMode,ModeData.thetaDist,ModeData.PriorDist,ModeData.LowerBound);
                  if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
%                     %
%                     % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%                     %
%                     [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
%                     [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
%                     if FileExist(DSGEModel.UpdateParameterFile)==1;
%                        [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
%                     end;
%                     if FileExist(DSGEModel.InitializeParameterFile)==1;
%                        [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
%                     end;
%                     [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
%                     %
%                     % check if we should copy model solver file
%                     %
%                     if DSGEModel.ModelSolver==4;
%                        if FileExist(DSGEModel.ModelSolverFile)==1;
%                           [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
%                        end;
%                     end;
%                     %
%                     % check if we need to deal with th belief system file as well
%                     %
%                     if DSGEModel.BeliefSystem==2;
%                        [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
%                     end;
                     %
                     % we need to construct a number of variables that are stored in DSGEModel.AdaptiveLearning
                     % as the fields r, f, Sperp, SMatrix, Gtilde and Kgf
                     %
                     f = length(DSGEModel.ForwardVariablePositions);
                     r = length(DSGEModel.StateVariablePositions);
                     %
                     % compute the BeliefSys structure
                     %
                     if DSGEModel.BeliefSystem==1;
                        BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                     else;
                        BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                     end;
                     %
                     % get the adaptive learning input variables
                     %
                     [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
                     DSGEModel.AdaptiveLearning.r = r;
                     DSGEModel.AdaptiveLearning.f = f;
                     DSGEModel.AdaptiveLearning.Sperp = Sperp;
                     DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
                     DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
                     DSGEModel.AdaptiveLearning.Kgf = Kgf;
                     DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
                     %
                     [logPrior,LogSPMode] = logSystemPriorPhiDSGE(ModeData.phiMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.thetaDist,ModeData.PriorDist,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,1,AIMData,DSGEModel,CurrINI);
                     LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
                     IsSystemPrior = 1;
                  else;
                     LogSPMode = 0;
                     LogMargLikeSystemLaplace = 0;
                     IsSystemPrior = 0;
                  end;
                  %
                  % ask if the inverse Hessian at the mode should be recalculated
                  %
                  txt = 'Do you want YADA to recalculate the inverse Hessian at the posterior mode for the Laplace approximation of the marginal likelihood?';
                  answer = Query(strvcat(txt,' ','NOTE: The recalculation will use finite differences for the inverse Hessian. YADA will otherwise use the inverse Hessian from the posterior mode estimation output for the transformed parameters, unless it locates a previously estimated inverse Hessian based on finite differences.'),'question',400,'Laplace Approximation - Posterior Mode',500,'no',CurrINI);
                  if strcmp(lower(answer),'no')==1;
                     if isnan(ModeData.phiInverseHessianFinDiff)==0;
                        InverseHessian = ModeData.phiInverseHessianFinDiff;
                     else;
                        InverseHessian = ModeData.InverseHessian;
                     end;
                     UseOld = 1;
                  else;
                     NumParam = length(ModeData.phiMode);
                     StepLengthStr = get(controls.optimize.steplength,'String');
                     StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                     %
                     % set the InitialStateVector value
                     %
                     if DSGEModel.UseOwnInitialState==1;
                        if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
                           DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
                        else;
                           DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                        end;
                     else;
                        DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
                     end;
                     [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
                     %
                     if exist('Gtilde')==0;
                        %
                        % get the adaptive learning input variables
                        %
                        [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
                     end;
                     %
                     tic;
                     VechHessian = YADAHessianMatrix('logPosteriorAdaptiveLearningPhiDSGE',ModeData.phiMode,StepLength,CurrINI.NumWorkers,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,1);
                     HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
                     CompTime = toc;
                     UseOld = 0;
                  end;
                  %
                  % compute the height of the log posterior kernel
                  %
                  LogLikeDensity = sum(StateVarStructure.lnLt);
                  LogPostDensity = sum(LogLikeDensity)+LogJacobianMode+LogPriorMode+LogSPMode;
                  %
                  if UseOld==1;
                     LogMargLike = MargLikeLaplace(LogPostDensity,InverseHessian,1)-LogMargLikeSystemLaplace;
                  else;
                     LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0)-LogMargLikeSystemLaplace;
                  end;
                  %
                  PrtVal = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                  [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\LaplaceMarginalLikelihood-AdaptiveLearning-' DSGEModel.NameOfModel '-PostMode.txt']);
                  fid = fopen(TextFile,'wt');
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*           L A P L A C E   A P P R O X I M A T I O N   O F   T H E            *\n');
                  fprintf(fid,'*                    M A R G I N A L   L I K E L I H O O D                     *\n');
                  fprintf(fid,'*                      A D A P T I V E   L E A R N I N G                       *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  % sample information
                  %
                  fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  if UseOld==0;
                     fprintf(fid,'Estimation time:                  %s (hrs:min:sec)\n\n',SecondsToHoursStr(CompTime));
                  end;
                  %
                  % now the marginal likelihood
                  %
                  fprintf(fid,['Log marginal likelihood:           ' PrtVal ' \n\n'],LogMargLike);
                  fprintf(fid,'where \n\n');
                  fprintf(fid,['   Log posterior density of phi:   ' PrtVal ' \n'],LogPostDensity);
                  fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity+LogMargLikeSystemLaplace);
                  if IsSystemPrior==1;
                     fprintf(fid,['   Correction for the log marginal\n   likelihood of the system prior: ' PrtVal ' \n\n'],LogMargLikeSystemLaplace);
                  end;
                  fprintf(fid,'================================================================================\n\n');
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  % save results to mat-file
                  %
                  MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-AL-LaplaceMarginalLikelihood-' DSGEModel.NameOfModel '-PostMode.mat']);
                  save(MatFile,'LogMargLike','LogMargLikeSystemLaplace','LogLikeDensity','LogPostDensity');
                  %
                  % display the results
                  %
                  TextGUI(TextFile,['Laplace Approximation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'Posterior Mode',100,CurrINI,0,1);
               end;
            elseif TransStatus==0;
               txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the Laplace approximation of the log marginal likelihood based on the posterior mode estimates.';
               About(txt,'information','Laplace Approximation - Posterior Mode',300,600,CurrINI);
            elseif BeliefStatus==0;
               txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the Laplace approximation of the log marginal likelihood based on the posterior mode estimates.';
               About(txt,'information','Laplace Approximation - Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ChibJeliazkov'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Sequential Marginal Likelihood - Chib & Jeliazkov','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('thetaPostSample');
               NumPostDraws = length(LogPost);
               %
               % as if the user wants to run the sequential estimation routine or just see the end point results
               %
               txt = 'Would you like to compute the marginal likelihood for the full sample only instead of the running the sequential estimation?';
               answer = Query(txt,'question',200,'Marginal Likelihood - Chib & Jeliazkov',500,'no',CurrINI);
               drawnow;
               if strcmp(lower(answer),'no')==1;
                  PlotMargLikeChibJeliazkovAdaptiveLearning(PostSample,LogPost,NumBurnin,CurrChain,DSGEModel,CurrINI);
               else;
                  %
                  ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  %
                  % check if the user wants to recompute the log posterior
                  %
                  if isempty(ErrorStr)==1;
                     txt = 'Would you like to recompute the value of the log posterior at the posterior draws?';
                     answer = Query(strvcat(txt,' ','NOTE: This is primarily useful if the posterior draws are taken from another data sample.'),'question',400,'Marginal Likelihood - Chib & Jeliazkov',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  %
                  OldNumBurnin = NumBurnin;
                  if strcmp(lower(answer),'yes')==1;
                     NumPostDraws = length(LogPost);
                     PostSample = PostSample(NumBurnin+1:NumPostDraws,:);
                     AcceptedDraws = AcceptedDraws(NumBurnin+1:NumPostDraws);
                     NumBurnin = 0;
                     LogPost = DSGELogPosteriorAdaptiveLearning(PostSample,AcceptedDraws,ModeData,'Chib & Jeliazkov',DSGEModel,CurrINI);
                  end;
                  %
                  if isempty(LogPost)==0;
                     TmpDSGEModel = DSGEModel;
                     TmpDSGEModel.SequentialML = 0;
                     %
                     LogMargsCJ = PlotMargLikeChibJeliazkovAdaptiveLearning(PostSample,LogPost,NumBurnin,CurrChain,TmpDSGEModel,CurrINI);
                     clear('TmpDSGEModel');
                     %
                     % check if we got any results
                     %
                     if isempty(LogMargsCJ)==0;
                        NumBurnin = OldNumBurnin;
                        %
                        % retrieve estimation sample start year and period
                        %
                        [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ChibJeliazkov-AdaptiveLearning-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                        fid = fopen(TextFile,'wt');
                        prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                        %
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
                        fprintf(fid,'*                      A D A P T I V E   L E A R N I N G                       *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
                        fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
                        if NumChains>1;
                           fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
                        end;
                        %
                        % sample information
                        %
                        fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                        fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                        fprintf(fid,'==========================================================\n');
                        fprintf(fid,'                          point estimate        std. error\n');
                        fprintf(fid,'----------------------------------------------------------\n');
                        fprintf(fid,['Chib & Jeliazkov         ' prt_val '   ' prt_val '\n'],LogMargsCJ(1,2),LogMargsCJ(1,3));
                        fprintf(fid,'==========================================================\n\n');
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        %
                        fclose(fid);
                        %
                        % show in TextGUI
                        %
                        TextGUI(TextFile,'Marginal Likelihood - Chib & Jeliazkov',[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],DSGEModel.NameOfModel,100,CurrINI,0,1);
                     end;
                  end;
               end;
               %
               % clear everything
               %
               clear('PostSample','LogPost','AcceptedDraws');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ModifiedHarmonicMean'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Sequential Marginal Likelihood - Geweke''s Harmonic Mean','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('thetaPostSample');
               %
               % asK if the user wants to run the sequential estimation routine or just see the end point results
               %
               txt = 'Would you like to compute the marginal likelihood for the full sample only instead of the running the sequential estimation?';
               answer = Query(txt,'question',200,'Marginal Likelihood - Geweke''s Harmonic Mean',500,'no',CurrINI);
               %
               % run the sequential estimation routine
               %
               drawnow;
               if strcmp(lower(answer),'no')==1;
                  PlotMargLikeModifiedHarmonicAdaptiveLearning(PostSample,LogPost,NumBurnin,CurrChain,DSGEModel,CurrINI);
               else;
                  NumPostDraws = length(LogPost);
                  %
                  % Coverage probabilities for Geweke''s Harmonic Mean estimator of
                  % the marginal likelihood
                  %
                  CovStartStr = get(controls.dsge.coveragestart,'String');
                  CovStartValue = str2num(StringTrim(CovStartStr(DSGEModel.CoverageStart,:)));
                  CovIncStr = get(controls.dsge.coverageincrement,'String');
                  CovIncValue = str2num(StringTrim(CovIncStr(DSGEModel.CoverageIncrement,:)));
                  CovEndStr = get(controls.dsge.coverageend,'String');
                  CovEndValue = str2num(StringTrim(CovEndStr(DSGEModel.CoverageEnd,:)));
                  CovProb = (CovStartValue:CovIncValue:CovEndValue);
                  %
                  ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  %
                  % check if the user wants to recompute the log posterior
                  %
                  if isempty(ErrorStr)==1;
                     txt = 'Would you like to recompute the value of the log posterior at the posterior draws?';
                     answer = Query(strvcat(txt,' ','NOTE: This is primarily useful if the posterior draws are taken from another data sample.'),'question',400,'Marginal Likelihood - Geweke''s Harmonic Mean',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  %
                  % select which posterior draws to use
                  %
                  LogPost = LogPost(NumBurnin+1:NumPostDraws,:);
                  PostSample = PostSample(NumBurnin+1:NumPostDraws,:);
                  AcceptedDraws = AcceptedDraws(NumBurnin+1:NumPostDraws);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws?';
                  answersub = Query(txt,'question',400,'Marginal Likelihood - Geweke''s Harmonic Mean',500,'no',CurrINI);
                  if strcmp(lower(answersub),'yes')==1;
                     %
                     % check which draws we should use
                     %
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     LogPost = LogPost(UseDraws);
                     PostSample = PostSample(UseDraws,:);
                     AcceptedDraws = AcceptedDraws(UseDraws);
                  end;
                  NumDraws = length(AcceptedDraws);
                  %
                  if strcmp(lower(answer),'yes')==1;
                     %
                     % recompute the log posterior
                     %
                     LogPost = DSGELogPosteriorAdaptiveLearning(PostSample,AcceptedDraws,ModeData,'Geweke''s Harmonic Mean',DSGEModel,CurrINI);
                  end;
                  %
                  if isempty(LogPost)==0;
                     [LogMargsMH,CovProb,NWStdErr] = MargLikeModifiedHarmonic(PostSample,LogPost,0,CovProb,DSGEModel);
                     %
                     % write results to a text file
                     %
                     [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                     %
                     TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ModifiedHarmonicMean-AdaptiveLearning-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' int2str(NumDraws) '.' int2str(NumPostDraws) '.txt']);
                     fid = fopen(TextFile,'wt');
                     prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                     %
                     fprintf(fid,'********************************************************************************\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
                     fprintf(fid,'*                      A D A P T I V E   L E A R N I N G                       *\n');
                     fprintf(fid,'*                                                                              *\n');
                     fprintf(fid,'********************************************************************************\n\n');
                     fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
                     fprintf(fid,'Number of used posterior draws:   %0.0f\n',NumDraws);
                     fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
                     if NumChains>1;
                        fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
                     end;
                     %
                     % sample information
                     %
                     fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % Geweke's modified harmonic mean estimator
                     %
                     prt_str = 'Geweke''s Harmonic Mean';
                     top_str = '                      ';
                     std_str = 'Num. standard errors: ';
                     for i=1:length(CovProb);
                        top_str = [top_str '        ' num2str(CovProb(i)*100,'%0.0f') ' percent'];
                        prt_str = [prt_str '   ' prt_val];
                        std_str = [std_str '   ' prt_val];
                     end;
                     TopStr = MultiCharStr('=',length(top_str));
                     MidStr = MultiCharStr('-',length(top_str));
                     fprintf(fid,[TopStr '\n']);
                     fprintf(fid,'                              Coverage probabilities for the estimator\n');
                     fprintf(fid,[top_str '\n']);
                     fprintf(fid,[MidStr '\n']);
                     fprintf(fid,[prt_str '\n'],LogMargsMH(size(LogMargsMH,1),2:size(LogMargsMH,2))');
                     fprintf(fid,[std_str '\n'],NWStdErr');
                     fprintf(fid,[MidStr '\n']);
                     fprintf(fid,['Mean:                    ' prt_val '\n'],sum(LogMargsMH(1,2:size(LogMargsMH,2)))/length(CovProb));
                     fprintf(fid,['Median:                  ' prt_val '\n'],median(LogMargsMH(1,2:size(LogMargsMH,2))));
                     fprintf(fid,[TopStr '\n\n']);
                     %
                     % Some information about the filename and its location
                     %
                     fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                     fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                     fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                     %
                     fclose(fid);
                     %
                     % show in TextGUI
                     %
                     TextGUI(TextFile,'Marginal Likelihood - Geweke''s Modified Harmonic Mean',[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],DSGEModel.NameOfModel,100,CurrINI,0,1);
                  end;
               end;
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'TruncatedElliptical'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Sequential Marginal Likelihood - SWZ Harmonic Mean','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('thetaPostSample');
                  %
                  % ask if the user wants to run the sequential estimation routine or just see the end point results
                  %
                  txt = 'Would you like to compute the marginal likelihood for the full sample only instead of the running the sequential estimation?';
                  answer = Query(txt,'question',200,'Marginal Likelihood - SWZ''S Harmonic Mean',500,'no',CurrINI);
                  %
                  % select the PValue
                  %
                  PValueStrMatrix = strvcat('90 percent of posterior draws','80 percent of posterior draws','70 percent of posterior draws','60 percent of posterior draws','50 percent of posterior draws');
                  [action,SelectedOption] = SelectionDlg(PValueStrMatrix,1,'Select the truncation parameter of the elliptical density:','Truncation Parameter','Marginal Likelihood - SWZ''S Harmonic Mean',[],[],CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     %
                     PValue = str2num(StringTrim(strrep(PValueStrMatrix(SelectedOption,:),'percent of posterior draws','')));
                     %
                     % run the sequential estimation routine
                     %
                     drawnow;
                     if strcmp(lower(answer),'no')==1;
                        PlotMargLikeSWZModifiedHarmonicAdaptiveLearning(PostSample,LogPost,PValue,ModeData,NumBurnin,CurrChain,DSGEModel,CurrINI);
                     else;
                        NumPostDraws = length(LogPost);
                        %
                        % Coverage probabilities for Geweke''s Harmonic Mean estimator of
                        % the marginal likelihood
                        %
                        CovStartStr = get(controls.dsge.coveragestart,'String');
                        CovStartValue = str2num(StringTrim(CovStartStr(DSGEModel.CoverageStart,:)));
                        CovIncStr = get(controls.dsge.coverageincrement,'String');
                        CovIncValue = str2num(StringTrim(CovIncStr(DSGEModel.CoverageIncrement,:)));
                        CovEndStr = get(controls.dsge.coverageend,'String');
                        CovEndValue = str2num(StringTrim(CovEndStr(DSGEModel.CoverageEnd,:)));
                        CovProb = (CovStartValue:CovIncValue:CovEndValue);
                        %
                        % check if the user wants to recompute the log posterior
                        %
                        txt = 'Would you like to recompute the value of the log posterior at the posterior draws?';
                        answer = Query(strvcat(txt,' ','NOTE: This is primarily useful if the posterior draws are taken from another data sample.'),'question',400,'Marginal Likelihood - SWZ''s Harmonic Mean',500,'no',CurrINI);
                        %
                        % select which posterior draws to use
                        %
                        LogPost = LogPost(NumBurnin+1:NumPostDraws,:);
                        PostSample = PostSample(NumBurnin+1:NumPostDraws,:);
                        AcceptedDraws = AcceptedDraws(NumBurnin+1:NumPostDraws);
                        NumDraws = NumPostDraws-NumBurnin;
                        %
                        % check if the user wishes a sub-sample
                        %
                        txt = 'Would you like to use a sub-sample of the posterior draws?';
                        answersub = Query(txt,'question',400,'Marginal Likelihood - SWZ''s Harmonic Mean',500,'no',CurrINI);
                        if strcmp(lower(answersub),'yes')==1;
                           %
                           % check which draws we should use
                           %
                           UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                           LogPost = LogPost(UseDraws);
                           PostSample = PostSample(UseDraws,:);
                           AcceptedDraws = AcceptedDraws(UseDraws);
                        end;
                        NumDraws = length(AcceptedDraws);
                        %
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % recompute the log posterior
                           %
                           LogPost = DSGELogPosteriorAdaptiveLearning(PostSample,AcceptedDraws,ModeData,'SWZ''s Harmonic Mean',DSGEModel,CurrINI);
                        end;
                        %
                        if isempty(LogPost)==0;
                           [LogMargsSWZ,qL,LBound,NWStdErr] = MargLikeSWZModifiedHarmonic(PostSample,LogPost,NumPostDraws,PValue,0,ModeData,NaN,DSGEModel,CurrINI);
                           if isempty(LogMargsSWZ)==0;
                              %
                              % write results to a text file
                              %
                              [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
                              %
                              TextFile = FixFilePath([DSGEModel.OutputDirectory '\MarginalLikelihood-ModifiedHarmonicMean-SWZ-AdaptiveLearning-' DSGEModel.NameOfModel '-' NewStartYear '.' NewStartPeriod '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' int2str(NumDraws) '.' int2str(NumPostDraws) '.txt']);
                              fid = fopen(TextFile,'wt');
                              prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                              %
                              fprintf(fid,'********************************************************************************\n');
                              fprintf(fid,'*                                                                              *\n');
                              fprintf(fid,'*         M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N          *\n');
                              fprintf(fid,'*                      A D A P T I V E   L E A R N I N G                       *\n');
                              fprintf(fid,'*                                                                              *\n');
                              fprintf(fid,'********************************************************************************\n\n');
                              fprintf(fid,'Number of posterior draws:        %0.0f\n',NumPostDraws);
                              fprintf(fid,'Number of used posterior draws:   %0.0f\n',NumDraws);
                              fprintf(fid,'Number of burn-in draws:          %0.0f\n',NumBurnin);
                              if NumChains>1;
                                 fprintf(fid,'MCMC chain number:                %0.0f\n',CurrChain);
                              end;
                              %
                              % sample information
                              %
                              fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                              fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                              %
                              % SWZ's modified harmonic mean estimator
                              %
                              fprintf(fid,'===============================================================\n');
                              fprintf(fid,'                              point estimate        std. error\n');
                              fprintf(fid,'---------------------------------------------------------------\n');
                              fprintf(fid,['Sims, Waggoner & Zha         ' prt_val '   ' prt_val '\n'],LogMargsSWZ(1,2),NWStdErr);
                              fprintf(fid,'---------------------------------------------------------------\n');
                              fprintf(fid,['Lower Bound of log posterior kernel:           ' prt_val ' \n'],LBound);
                              fprintf(fid,['Fraction of iid draws from elliptical > lower bound:  %0.6f \n'],qL);
                              fprintf(fid,['Fraction of posterior draws > lower bound:            %0.6f \n'],PValue/100);
                              fprintf(fid,'===============================================================\n\n');
                              fprintf(fid,'NOTE: The "fraction of iid draws from elliptical > lower bound" notation refers to the fraction of\n      the parameter draws which have a log posterior kernel greater than the lower bound. The "fraction\n      of posterior draws > lower bound" notation also refers to the log posterior kernel values.\n\n');
                              %
                              fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                              fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                              fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                              %
                              fclose(fid);
                              %
                              % show in TextGUI
                              %
                              TextGUI(TextFile,'Marginal Likelihood - SWZ''s Modified Harmonic Mean',[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],DSGEModel.NameOfModel,100,CurrINI,0,1);
                           end;
                        end;
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IteratedParameters'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            if isfield(ModeData,'phiEstimates')==1;
               phiEstimates = ModeData.phiEstimates;
               thetaEstimates = zeros(size(phiEstimates));
               for i=1:size(phiEstimates,2);
                  thetaEstimates(:,i) = PhiToTheta(phiEstimates(:,i),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound);
               end;
               txt = strvcat('Would you like to save to a mat-file the iterated parameter estimates at a particular iteration of the posterior mode estimation run?',' ','NOTE: The alternative is to view plots of the iterated estimates of the individual parameters.');
               answer = Query(txt,'question',300,'Adaptive Learning - Iterated Parameter Estimates',600,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  %
                  % the log posterior kernel estimates are located in the second column of OptResults
                  %
                  LogPostKernel = ModeData.OptResults(:,2);
                  NumIter = size(phiEstimates,2);
                  if NumIter>1;
                     ParamSample = thetaEstimates(:,2:NumIter);
                  else;
                     ParamSample = thetaEstimates;
                  end;
                  %
                  % select the parameter value to use
                  %
                  SelStr = ['Iteration   1: logPost = ' num2str(LogPostKernel(1),'%0.6f')];
                  for i=2:NumIter-1;
                     SelStr = strvcat(SelStr,sprintf('Iteration %3.0f: logPost = %0.6f',i,LogPostKernel(i)));
                  end;
                  [action,SelValue] = SelectionDlg(SelStr,1,'Select the iteration number to use for showing parameter values','Parameter Value Selection','Adaptive Learning - Iterated Parameter Estimates','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     ParamValue = ParamSample(:,SelValue);
                     LPK = LogPostKernel(SelValue);
                     %
                     % save to mat-file
                     %
                     MatFile = FixFilePath([DSGEModel.OutputDirectory '\ParameterValues-AdaptiveLearning-' DSGEModel.NameOfModel '-Iteration-' int2str(SelValue) '.mat']);
                     ParameterNames = ModeData.ParameterNames.estimated;
                     save(MatFile,'ParamValue','LPK','ParameterNames');
                     %
                     % write an m-file to extract the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\ParamVals_AL_' DSGEModel.NameOfModel '_Iteration_' int2str(SelValue) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                     fprintf(fid,'%%\n%% string matrix with the parameter names\n%%\n');
                     fprintf(fid,'ParameterNames = DataStructure.ParameterNames;\n');
                     fprintf(fid,'%%\n%% vector with values for the original parameters\n%%\n');
                     fprintf(fid,'ParamValue = DataStructure.ParamValue;\n');
                     fprintf(fid,'%%\n%% value of the log posterior kernel\n%%\n');
                     fprintf(fid,'LPK = DataStructure.LPK;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The values for the original parameters have been saved to the file "' GetFilename(MatFile) '" in the directory "' GetPath(MatFile) '". The file contains 3 entries: ParameterNames (string matrix with the names of the parameters), ParamValue (vector with the values of the parameters at iteration no ' int2str(SelValue) ' of the posterior mode estimation run), and LPK (scalar with the value of the log posterior kernel at the selected parameter values). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Iterated Parameter Estimates',400,500,CurrINI);
                     drawnow;
                  end;
                  






               else;
                  %
                  % open the graph iterated parameters dialog
                  %
                  IteratedParamsGraphDLG('init',CurrINI,ModeData.ParameterNames.estimated,[ModeData.theta thetaEstimates]);
               end;
            else;
               txt = 'The data file with the posterior mode estimates didn''t include the iterated parameter estimates. YADA is therefore unable to display them.';
               About(txt,'information','YADA - Old MAT File',150,500,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CheckPosteriorMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         AdaptiveLearningCheckPosteriorMode(DSGEModel,CurrINI,controls);
         %
         set(controls.about,'UserData',0);
      end;
   case 'ThreeDeePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            if gcbo==controls.learning.viewmenu.threedeepostmode.transformed;
               UseTransformed = 1;
            elseif gcbo==controls.learning.viewmenu.threedeepostmode.original;
               UseTransformed = 0;
            end;
            %
            % display the dialog for selecting parameters
            %
            ThreeDeeParamsGraphDLG('init',DSGEModel,CurrINI,ModeData,UseTransformed,1);
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewLaplacePosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load data from file
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            thetaMode = ModeData.thetaMode;
            InverseHessian = ModeData.InverseHessian;
            if isfield(ModeData,'ThetaPhiPartial')==1;
               ThetaPhiPartial = ModeData.ThetaPhiPartial;
            else;
               ThetaPhiPartial = PartialThetaPartialPhi(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
            end;
            if isfield(ModeData,'thetaAdditionalMode')==1;
               thetaAdditionalMode = ModeData.thetaAdditionalMode;
            else;
               [thetaCalibratedMode,thetaAdditionalMode] = ReadAdditionalParameterValues(ModeData.ModelParameters,ModeData.ParameterNames);
            end;
            if (isfield(ModeData,'phiInverseHessianFinDiff')==1)&(all(~isnan(ModeData.phiInverseHessianFinDiff))==1)&(size(ModeData.phiInverseHessianFinDiff,1)==length(thetaMode));
               txt = 'When computing the Laplace approximation of the posterior densities, would you like to use the finite difference estimator of the inverse Hessian instead of the inverse Hessian from the optimization routine?';
               answer = Query(txt,'question',300,'Adaptive Learning - Laplace Posterior Densities',600,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  thetaInverseHessian = ThetaPhiPartial*ModeData.phiInverseHessianFinDiff*ThetaPhiPartial;
               else;
                  thetaInverseHessian = ThetaPhiPartial*InverseHessian*ThetaPhiPartial;
               end;
            else;
               thetaInverseHessian = ThetaPhiPartial*InverseHessian*ThetaPhiPartial;
            end;
            %
            GraphLaplacePosteriorDensity(thetaMode,thetaInverseHessian,thetaAdditionalMode,ModeData.theta,ModeData.thetaDist,ModeData.LowerBound,ModeData.PriorDist,ModeData.ParameterNames,ModeData.thetaPositions,ModeData.ModelParameters,[],'Posterior Mode',1,controls,DSGEModel,CurrINI);
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewPosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % if there are multiple chains, ask which one to load
         %
         ChainsStr = get(controls.posterior.chains,'String');
         NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
         %
         if NumChains>1;
            %
            % ask which chain to load
            %
            ChainsStr = '';
            for i=1:NumChains;
               ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
            end;
            if DSGEModel.PosteriorSampler<=7;
               HeaderStr = 'MCMC Chain Selection';
            elseif (DSGEModel.PosteriorSampler==8)|(DSGEModel.PosteriorSampler==9);
               HeaderStr = 'SMC Chain Selection';
            elseif DSGEModel.PosteriorSampler==10;
               HeaderStr = 'Importance Sample Selection';
            end;
            [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load',HeaderStr,'Posterior Densities','','',CurrINI);
         else;
            CurrChain = 1;
            action = 'ok';
         end;
         %
         if strcmp(lower(action),'cancel')==0;
            %
            % check if the data actually exists
            %
            if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
               %
               % load the data
               %
               [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
               clear('PostSample','LogPost','AcceptedDraws');
               %
               % try to load the ModeFile to get names of parameters
               %
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               try;
                  ModeData = load(ModeFile);
                  ParameterNames = ModeData.ParameterNames;
               catch;
                  ParameterNames.additional = []
               end;
               %
               % check if we have parameters that are functions of estimated parameters
               %
               if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ParameterNames.additional)==0);
                  %
                  % ask if the user wants to load additional parameters
                  %
                  txt = ['The model "' DSGEModel.NameOfModel '" has parameters that are a function of the estimated parameters. Would you also like to plot posterior densities for these additional parameters?'];
                  answer = Query(txt,'question',200,'Marginal Posterior Densities - Additional Parameters',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     [UpdatePostSample,UpdateMode,UpdateNames] = AdaptiveLearningGetUpdatePosteriorDraws(thetaPostSample,NumBurnin,CurrChain,NumChains,1,'Marginal Posterior Densities',DSGEModel,CurrINI);
                     %
                     % check if the user has the necessary tools for computing prior densitites
                     % for the additional parameters through random draws from the prior of the
                     % relevant estimated parameters
                     %
                     if isempty(UpdateMode)==0;
                        if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
                           IsSystemPrior = 0;
                        else;
                           IsSystemPrior = 1;
                        end;
                        TmpCurrINI = CurrINI;
                        TmpCurrINI.UpdateNames = UpdateNames;
                        if IsSystemPrior==0;
                           [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = AdaptiveLearningGetUpdatePriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                        else;
                           [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = AdaptiveLearningGetUpdateSystemPriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                        end;
                        %
                        % test if the results were obtained from an old mat-file
                        %
                        if max(UpdateParameters)>length(UpdateMode);
                           if DSGEModel.PosteriorSampler==1;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-AL-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==2;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Slice-AL-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==3;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-AL-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==4;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-AL-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==5;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-AL-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==6;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-AL-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==7;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-AL-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==8;
                              TempStagesStr = get(controls.posterior.temperingstages,'String');
                              TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-AL-Additional-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==9;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-AL-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(size(thetaPostSample,1)) '.mat']);
                           elseif DSGEModel.PosteriorSampler==10;
                              AddFile = FixFilePath([DSGEModel.OutputDirectory '\is\IS-AL-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(size(thetaPostSample,1)) '.mat']);
                           end;
                           delete(AddFile);
                           if IsSystemPrior==0;
                              [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = AdaptiveLearningGetUpdatePriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                           else;
                              [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters] = AdaptiveLearningGetUpdateSystemPriorDensity(DSGEModel,TmpCurrINI,controls,CurrChain,NumChains,NumBurnin,size(thetaPostSample,1));
                           end;
                        end;
                        clear('TmpCurrINI');
                        if isempty(UpdatePriorX)==0;
                           if length(UpdateMode)>length(UpdateParameters);
                              UpdateMode = UpdateMode(UpdateParameters);
                              UpdateNames = UpdateNames(UpdateParameters,:);
                              UpdatePostSample = UpdatePostSample(:,UpdateParameters);
                           end;
                        else;
                           txt = 'YADA has determined that all parameters that are supposed to be functions of the estimated parameters are, in fact, fixed. YADA will therefore skip these parameters in the plots.';
                           About(txt,'information','Marginal Posterior Densities - Additional Parameters',300,500,CurrINI);
                           UpdatePostSample = [];
                           UpdateMode = [];
                           UpdateNames = '';
                           UpdatePriorX = [];
                           UpdatePriorY = [];
                           UpdatePriorBounds = [];
                        end;
                     else;
                        UpdatePostSample = [];
                        UpdateNames = '';
                        UpdatePriorX = [];
                        UpdatePriorY = [];
                        UpdatePriorBounds = [];
                     end;
                  else;
                     UpdatePostSample = [];
                     UpdateMode = [];
                     UpdateNames = '';
                     UpdatePriorX = [];
                     UpdatePriorY = [];
                     UpdatePriorBounds = [];
                  end;
               else;
                  UpdatePostSample = [];
                  UpdateMode = [];
                  UpdateNames = '';
                  UpdatePriorX = [];
                  UpdatePriorY = [];
                  UpdatePriorBounds = [];
               end;
               %
               % send the data to the posterior densities plotting function
               %
               drawnow;
               AdaptiveLearningGraphPosteriorDensity(thetaPostSample,NumBurnin,CurrChain,UpdatePostSample,UpdateMode,UpdateNames,UpdatePriorX,UpdatePriorY,UpdatePriorBounds,DSGEModel,CurrINI,controls);
               %
               % clear everything
               %
               clear('thetaPostSample');
            else;
               txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
               About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewBivariatePosterior'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % Try to load the posterior mode estimates
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               if DSGEModel.PosteriorSampler<=7;
                  HeaderStr = 'MCMC Chain Selection';
               elseif (DSGEModel.PosteriorSampler==8)|(DSGEModel.PosteriorSampler==9);
                  HeaderStr = 'SMC Chain Selection';
               elseif DSGEModel.PosteriorSampler==10;
                  HeaderStr = 'Importance Sample Selection';
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load',HeaderStr,'Bivariate Posterior Densities','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('PostSample','LogPost','AcceptedDraws');
                  ParameterNames = ModeData.ParameterNames;
                  %
                  % check if we have parameters that are functions of estimated parameters
                  %
                  if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ParameterNames.additional)==0);
                     %
                     % ask if the user wants to load additional parameters
                     %
                     txt = ['The model "' DSGEModel.NameOfModel '" has parameters that are a function of the estimated parameters. Would you also like to plot posterior densities for these additional parameters?'];
                     answer = Query(txt,'question',200,'Bivariate Posterior Densities - Additional Parameters',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        [UpdatePostSample,UpdateMode,UpdateNames,UpdateParamPartialPhi] = AdaptiveLearningGetUpdatePosteriorDraws(thetaPostSample,NumBurnin,CurrChain,NumChains,1,'Bivariate Posterior Densities',DSGEModel,CurrINI);
                     else;
                        UpdatePostSample = [];
                        UpdateMode = [];
                        UpdateNames = '';
                        UpdateParamPartialPhi = [];
                     end;
                  else;
                     UpdatePostSample = [];
                     UpdateMode = [];
                     UpdateNames = '';
                     UpdateParamPartialPhi = [];
                  end;
                  %
                  % send the data to the bivariate posterior densities plotting function
                  %
                  drawnow;
                  if size(thetaPostSample,2)>1;
                     %
                     % deal with sub-sample
                     %
                     thetaPostSample = thetaPostSample(NumBurnin+1:size(thetaPostSample,1),:);
                     NumDraws = size(thetaPostSample,1);
                     %
                     % ask if a subset of the draws should be used
                     %
                     if DSGEModel.PosteriorSampler<=7;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        txt = ['Would you like to use a sub-sample with ' int2str(length(UseDraws)) ' of the posterior draws rather than the full sample with ' int2str(NumDraws) ' draws?'];
                        answer = Query(txt,'question',400,'Bivariate Posterior Densities',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           if length(UseDraws)<NumDraws;
                              thetaPostSample = thetaPostSample(UseDraws,:);
                              if isempty(UpdatePostSample)==0;
                                 UpdatePostSample = UpdatePostSample(UseDraws,:);
                              end;
                              UsedDraws = length(UseDraws);
                           else;
                              UsedDraws = NumDraws;
                           end;
                        else;
                           UsedDraws = NumDraws;
                        end;
                     else;
                        UsedDraws = NumDraws;
                     end;
                     %
                     % send the data to the graphing dialog for the bivariate densities
                     %
                     GraphBivariatePosteriorDensity('init',thetaPostSample,ModeData.thetaMode,ModeData,ParameterNames.estimated,NumDraws,NumBurnin,CurrChain,UpdatePostSample,UpdateMode,UpdateNames,UpdateParamPartialPhi,DSGEModel,CurrINI,controls);
                  else;
                     txt = 'The model only has one estimated parameter. YADA is threfore unable to prepare bivariate plots of its posterior distribution.';
                     About(txt,'information','Bivariate Posterior Densities - Too Few Parameters',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample','UpdatePostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            drawnow;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PosteriorCorrelations'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Try to load the posterior mode estimates
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               if DSGEModel.PosteriorSampler<=7;
                  HeaderStr = 'MCMC Chain Selection';
               elseif (DSGEModel.PosteriorSampler==8)|(DSGEModel.PosteriorSampler==9);
                  HeaderStr = 'SMC Chain Selection';
               elseif DSGEModel.PosteriorSampler==10;
                  HeaderStr = 'Importance Sample Selection';
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load',HeaderStr,'Posterior Correlations','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('PostSample','LogPost','AcceptedDraws');
                  ParameterNames = ModeData.ParameterNames;
                  %
                  % check if we have parameters that are functions of estimated parameters
                  %
                  if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ParameterNames.additional)==0);
                     %
                     % ask if the user wants to load additional parameters
                     %
                     txt = ['The model "' DSGEModel.NameOfModel '" has parameters that are a function of the estimated parameters. Would you also like to compute posterior correlations for these additional parameters?'];
                     answer = Query(txt,'question',200,'Posterior Correlations - Additional Parameters',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        [UpdatePostSample,UpdateMode,UpdateNames,UpdateParamPartialPhi] = AdaptiveLearningGetUpdatePosteriorDraws(thetaPostSample,NumBurnin,CurrChain,NumChains,1,'Posterior Correlations',DSGEModel,CurrINI);
                     else;
                        UpdatePostSample = [];
                        UpdateNames = '';
                     end;
                  else;
                     UpdatePostSample = [];
                     UpdateNames = '';
                  end;
                  %
                  % check if we should proceed
                  %
                  drawnow;
                  if size(thetaPostSample,2)>1;
                     %
                     % deal with sub-sample
                     %
                     thetaPostSample = thetaPostSample(NumBurnin+1:size(thetaPostSample,1),:);
                     NumDraws = size(thetaPostSample,1);
                     %
                     % ask if a subset of the draws should be used
                     %
                     if DSGEModel.PosteriorSampler<=7;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        txt = ['Would you like to use a sub-sample with ' int2str(length(UseDraws)) ' of the posterior draws rather than the full sample with ' int2str(NumDraws) ' draws?'];
                        answer = Query(txt,'question',400,'Posterior Correlations',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           if length(UseDraws)<NumDraws;
                              thetaPostSample = thetaPostSample(UseDraws,:);
                              if isempty(UpdatePostSample)==0;
                                 UpdatePostSample = UpdatePostSample(UseDraws,:);
                              end;
                              UsedDraws = length(UseDraws);
                           else;
                              UsedDraws = NumDraws;
                           end;
                        else;
                           UsedDraws = NumDraws;
                        end;
                     else;
                        UsedDraws = NumDraws;
                     end;
                     if isempty(Weights)==1;
                        Weights = ones(UsedDraws,1);
                     end;
                     %
                     % concatenate and transpose such that NumParam appears in the rows and
                     % the individual draws in the columns
                     %
                     thetaPostSample = [thetaPostSample UpdatePostSample]';
                     %
                     % now we compute the posterior mean
                     %
                     PosteriorMeanParam = (1/UsedDraws)*thetaPostSample*Weights;
                     thetaDeMeanDraws = thetaPostSample-(PosteriorMeanParam*ones(1,UsedDraws));
                     if (max(Weights)==min(Weights))&(max(Weights)==1);
                        PosteriorCovParam = (1/UsedDraws)*thetaDeMeanDraws*thetaDeMeanDraws';
                     else;
                        NumParam = size(thetaPostSample,1);
                        PosteriorCovParam = zeros(NumParam,NumParam);
                        for ii=1:UsedDraws;
                           PosteriorCovParam = PosteriorCovParam+((Weights(ii)/UsedDraws)*thetaDeMeanDraws(:,ii)*thetaDeMeanDraws(:,ii)');
                        end;
                     end;
                     PosteriorStdParam = sqrt(diag(PosteriorCovParam));
                     PosteriorCorrParam = PosteriorCovParam./(PosteriorStdParam*PosteriorStdParam');
                     %
                     % print to file
                     %
                     PosteriorCorrelationsFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorCorrelations-AdaptiveLearning-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(UsedDraws) '.' int2str(NumDraws) '.txt']);
                     PrintSystemPriorCorrelations(PosteriorCorrelationsFile,strvcat(ParameterNames.estimated,UpdateNames),PosteriorCorrParam,1,NumDraws,UsedDraws,NumBurnin,CurrChain,1,DSGEModel,CurrINI);
                     if FileExist(PosteriorCorrelationsFile)==1;
                        TextGUI(PosteriorCorrelationsFile,['Posterior Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Parameter Correlations',50,CurrINI,1,0);
                     end;
                  else;
                     txt = 'The model only has one estimated parameter. YADA is threfore unable to prepare bivariate plots of its posterior distribution.';
                     About(txt,'information','Posterior Correlations - Too Few Parameters',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample','UpdatePostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            drawnow;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewPriorInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % get parameters and priors
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % transform the parameters
               %
               phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
               lnjac = logJacobian(phi,thetaIndex,UniformBounds);
               logPrior = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
               Status = 1;
               if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
                  %
                  % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
                  %
                  [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
                  [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
                  if FileExist(DSGEModel.UpdateParameterFile)==1;
                     [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
                  end;
                  if FileExist(DSGEModel.InitializeParameterFile)==1;
                     [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
                  end;
                  [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
                  %
                  % check if we should copy model solver file
                  %
                  if DSGEModel.ModelSolver==4;
                     if FileExist(DSGEModel.ModelSolverFile)==1;
                        [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
                     end;
                     if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
                        [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
                     end;
                  end;
                  %
                  % check if we need to deal with the belief system file as well
                  %
                  if DSGEModel.BeliefSystem==2;
                     [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                  end;
                  ErrorStr = '';
                  ErrorFile = '';
                  if MatlabNumber>=8.0;
                     try;
                        AIMData = load(DSGEModel.AIMDataFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
                        ErrorFile = DSGEModel.AIMDataFile;
                     end;
                  else;
                     try;
                        AIMData = load(DSGEModel.AIMDataFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
                        ErrorFile = DSGEModel.AIMDataFile;
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     %
                     % we need to construct a number of variables that are stored in DSGEModel.AdaptiveLearning
                     % as the fields r, f, Sperp, SMatrix, Gtilde and Kgf
                     %
                     f = length(DSGEModel.ForwardVariablePositions);
                     r = length(DSGEModel.StateVariablePositions);
                     %
                     % compute the BeliefSys structure
                     %
                     if DSGEModel.BeliefSystem==1;
                        BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                     else;
                        BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                     end;
                     %
                     % get the adaptive learning input variables
                     %
                     [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
                     DSGEModel.AdaptiveLearning.r = r;
                     DSGEModel.AdaptiveLearning.f = f;
                     DSGEModel.AdaptiveLearning.Sperp = Sperp;
                     DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
                     DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
                     DSGEModel.AdaptiveLearning.Kgf = Kgf;
                     DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
                     %
                     % compute the system prior
                     %
                     [logPrior,logSP] = logSystemPriorPhiDSGE(phi,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,1,AIMData,DSGEModel,CurrINI);
                  else;
                     About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
                     drawnow;
                     Status = 0;
                  end;
               else;
                  logSP = [];
               end;
               if Status==1;
                  txt = '********************************************************************************';
                  txt = strvcat(txt,'*                                                                              *');
                  txt = strvcat(txt,'*        V A L U E S   O F   T H E   L O G   P R I O R   D E N S I T Y         *');
                  txt = strvcat(txt,'*                A N D   O F   T H E   L O G   J A C O B I A N                 *');
                  txt = strvcat(txt,'*                      A D A P T I V E   L E A R N I N G                       *');
                  txt = strvcat(txt,'*                                                                              *');
                  txt = strvcat(txt,'********************************************************************************',' ');
                  txt = strvcat(txt,'Parameters evaluated at their initial values.',' ');
                  prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                  if isempty(logSP)==1;
                     txt = strvcat(txt,sprintf(['log prior for theta:        ' prt_val],logPrior));
                     txt = strvcat(txt,sprintf(['log Jacobian (dtheta/dphi): ' prt_val],lnjac));
                     txt = strvcat(txt,sprintf(['log prior for phi:          ' prt_val],logPrior+lnjac),' ');
                     txt = strvcat(txt,'NOTE: The sum of the log prior for theta (original parameters) and the log','      Jacobian is equal to the log prior of phi (transformed parameters).',' ');
                  else;
                     txt = strvcat(txt,sprintf(['marginal log prior for theta:        ' prt_val],-logPrior-logSP-lnjac));
                     txt = strvcat(txt,sprintf(['log Jacobian (dtheta/dphi):          ' prt_val],lnjac));
                     txt = strvcat(txt,sprintf(['marginal log prior for phi:          ' prt_val],-logPrior-logSP));
                     txt = strvcat(txt,sprintf(['conditional log system prior:        ' prt_val],logSP),' ');
                     txt = strvcat(txt,'NOTE: The sum of the marginal log prior for theta (original parameters) and','      the log Jacobian is equal to the marginal log prior of phi (transformed','      parameters). The conditional log system prior is added to get the full','      prior up to a constant, determined by the log marginal likelihood of','      the system prior.',' ');
                  end;
                  %
                  % write to file
                  %
                  PriorJacValueFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-Prior-Jacobian-AdaptiveLearning-' DSGEModel.NameOfModel '-InitialValues.txt']);
                  fid = fopen(PriorJacValueFile,'wt');
                  for i=1:size(txt,1);
                     fprintf(fid,'%s\n',txt(i,:));
                  end;
                  fprintf(fid,'================================================================================\n\n');
                  %
                  % bottom file data
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(PriorJacValueFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(PriorJacValueFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  if FileExist(PriorJacValueFile)==1;
                     TextGUI(PriorJacValueFile,['DSGE Model Prior and Jacobian under Adaptive Learning - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'Initial Values',50,CurrINI,1,1);
                  end;
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to display the log prior and the log Jacobian based on the initial values.'];
                About(txt,'information','View Prior And Jacobian - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewPriorDSGEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % transform the parameters
               %
               phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
               lnjac = logJacobian(phi,thetaIndex,UniformBounds);
               logPrior = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
               if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
                  %
                  % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
                  %
                  [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
                  [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
                  if FileExist(DSGEModel.UpdateParameterFile)==1;
                     [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
                  end;
                  if FileExist(DSGEModel.InitializeParameterFile)==1;
                     [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
                  end;
                  [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
                  %
                  % check if we need to deal with the belief system file as well
                  %
                  if DSGEModel.BeliefSystem==2;
                     [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                  end;
                  %
                  % check if we should copy model solver file
                  %
                  if DSGEModel.ModelSolver==4;
                     if FileExist(DSGEModel.ModelSolverFile)==1;
                        [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
                     end;
                     if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
                        [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
                     end;
                  end;
                  %
                  % try to load the AIMData, other wise use ModeData
                  %
                  if MatlabNumber>=8.0;
                     try;
                        AIMData = load(DSGEModel.AIMDataFile);
                     catch ME;
                        AIMData = ModeData.AIMData;
                     end;
                  else;
                     try;
                        AIMData = load(DSGEModel.AIMDataFile);
                     catch;
                        AIMData = ModeData.AIMData;
                     end;
                  end;
                  %
                  % we need to construct a number of variables that are stored in DSGEModel.AdaptiveLearning
                  % as the fields r, f, Sperp, SMatrix, Gtilde and Kgf
                  %
                  f = length(DSGEModel.ForwardVariablePositions);
                  r = length(DSGEModel.StateVariablePositions);
                  %
                  % compute the BeliefSys structure
                  %
                  if DSGEModel.BeliefSystem==1;
                     BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                  else;
                     BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                  end;
                  %
                  % get the adaptive learning input variables
                  %
                  [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
                  DSGEModel.AdaptiveLearning.r = r;
                  DSGEModel.AdaptiveLearning.f = f;
                  DSGEModel.AdaptiveLearning.Sperp = Sperp;
                  DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
                  DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
                  DSGEModel.AdaptiveLearning.Kgf = Kgf;
                  DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
                  %
                  [logPrior,logSP] = logSystemPriorPhiDSGE(phi,thetaPositions,ModelParameters,thetaDist,PriorDist,thetaIndex,UniformBounds,LowerBound,1,AIMData,DSGEModel,CurrINI);
               else;
                  logSP = [];
               end;
               txt = '********************************************************************************';
               txt = strvcat(txt,'*                                                                              *');
               txt = strvcat(txt,'*        V A L U E S   O F   T H E   L O G   P R I O R   D E N S I T Y         *');
               txt = strvcat(txt,'*                A N D   O F   T H E   L O G   J A C O B I A N                 *');
               txt = strvcat(txt,'*                      A D A P T I V E   L E A R N I N G                       *');
               txt = strvcat(txt,'*                                                                              *');
               txt = strvcat(txt,'********************************************************************************',' ');
               txt = strvcat(txt,'Parameters evaluated at the posterior mode values from the','rational expectations version, while the adaptive learning parameters','have their initial values.',' ');
               prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
               if isempty(logSP)==1;
                  txt = strvcat(txt,sprintf(['log prior for theta:        ' prt_val],logPrior));
                  txt = strvcat(txt,sprintf(['log Jacobian (dtheta/dphi): ' prt_val],lnjac));
                  txt = strvcat(txt,sprintf(['log prior for phi:          ' prt_val],logPrior+lnjac),' ');
                  txt = strvcat(txt,'NOTE: The sum of the log prior for theta (original parameters) and the log','      Jacobian is equal to the log prior of phi (transformed parameters).',' ');
               else;
                  txt = strvcat(txt,sprintf(['marginal log prior for theta:        ' prt_val],-logPrior-logSP-lnjac));
                  txt = strvcat(txt,sprintf(['log Jacobian (dtheta/dphi):          ' prt_val],lnjac));
                  txt = strvcat(txt,sprintf(['marginal log prior for phi:          ' prt_val],-logPrior-logSP));
                  txt = strvcat(txt,sprintf(['conditional log system prior:        ' prt_val],logSP),' ');
                  txt = strvcat(txt,'NOTE: The sum of the marginal log prior for theta (original parameters) and','      the log Jacobian is equal to the marginal log prior of phi (transformed','      parameters). The conditional log system prior is added to get the full','      prior up to a constant, determined by the log marginal likelihood of','      the system prior.',' ');
               end;
               %
               % write to file
               %
               PriorJacValueFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-Prior-Jacobian-AdaptiveLearning-' DSGEModel.NameOfModel '-DSGEMode.txt']);
               fid = fopen(PriorJacValueFile,'wt');
               for i=1:size(txt,1);
                  fprintf(fid,'%s\n',txt(i,:));
               end;
               fprintf(fid,'================================================================================\n\n');
               %
               % bottom file data
               %
               fprintf(fid,'Directory for file: %s\n',GetPath(PriorJacValueFile));
               fprintf(fid,'Name of file:       %s\n',GetFilename(PriorJacValueFile));
               fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
               fclose(fid);
               %
               if FileExist(PriorJacValueFile)==1;
                  TextGUI(PriorJacValueFile,['DSGE Model Prior and Jacobian under Adaptive Learning - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'DSGE Posterior Mode',50,CurrINI,1,1);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to display the log prior and the log Jacobian.'];
               About(txt,'information','View Prior And Jacobian - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ViewPriorPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            lnjac = logJacobian(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds);
            logPrior = logPriorDSGE(ModeData.thetaMode,ModeData.thetaDist,ModeData.PriorDist,ModeData.LowerBound);
            if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==0;
               %
               % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
               %
               [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
               [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
               if FileExist(DSGEModel.UpdateParameterFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
               end;
               if FileExist(DSGEModel.InitializeParameterFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
               end;
               [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
               %
               % check if we should copy model solver file
               %
               if DSGEModel.ModelSolver==4;
                  if FileExist(DSGEModel.ModelSolverFile)==1;
                     [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
                  end;
                  if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
                     [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
                  end;
               end;
               %
               % check if we need to deal with the belief system file as well
               %
               if DSGEModel.BeliefSystem==2;
                  [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
               end;
               %
               % try to load the AIMData, other wise use ModeData
               %
               if MatlabNumber>=8.0;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch ME;
                     AIMData = ModeData.AIMData;
                  end;
               else;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch;
                     AIMData = ModeData.AIMData;
                  end;
               end;
               %
               % we need to construct a number of variables that are stored in DSGEModel.AdaptiveLearning
               % as the fields r, f, Sperp, SMatrix, Gtilde and Kgf
               %
               f = length(DSGEModel.ForwardVariablePositions);
               r = length(DSGEModel.StateVariablePositions);
               %
               % compute the BeliefSys structure
               %
               if DSGEModel.BeliefSystem==1;
                  BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
               else;
                  BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
               end;
               %
               % get the adaptive learning input variables
               %
               [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
               DSGEModel.AdaptiveLearning.r = r;
               DSGEModel.AdaptiveLearning.f = f;
               DSGEModel.AdaptiveLearning.Sperp = Sperp;
               DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
               DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
               DSGEModel.AdaptiveLearning.Kgf = Kgf;
               DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
               %
               [logPrior,logSP] = logSystemPriorPhiDSGE(ModeData.phiMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.thetaDist,ModeData.PriorDist,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,1,AIMData,DSGEModel,CurrINI);
            else;
               logSP = [];
            end;
            txt = '********************************************************************************';
            txt = strvcat(txt,'*                                                                              *');
            txt = strvcat(txt,'*        V A L U E S   O F   T H E   L O G   P R I O R   D E N S I T Y         *');
            txt = strvcat(txt,'*                A N D   O F   T H E   L O G   J A C O B I A N                 *');
            txt = strvcat(txt,'*                      A D A P T I V E   L E A R N I N G                       *');
            txt = strvcat(txt,'*                                                                              *');
            txt = strvcat(txt,'********************************************************************************',' ');
            txt = strvcat(txt,'Parameters evaluated at the posterior mode values under adaptive learning.',' ');
            prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
            if isempty(logSP)==1;
               txt = strvcat(txt,sprintf(['log prior for theta:        ' prt_val],logPrior));
               txt = strvcat(txt,sprintf(['log Jacobian (dtheta/dphi): ' prt_val],lnjac));
               txt = strvcat(txt,sprintf(['log prior for phi:          ' prt_val],logPrior+lnjac),' ');
               txt = strvcat(txt,'NOTE: The sum of the log prior for theta (original parameters) and the log','      Jacobian is equal to the log prior of phi (transformed parameters).',' ');
            else;
               txt = strvcat(txt,sprintf(['marginal log prior for theta:        ' prt_val],logPrior));
               txt = strvcat(txt,sprintf(['log Jacobian (dtheta/dphi):          ' prt_val],lnjac));
               txt = strvcat(txt,sprintf(['marginal log prior for phi:          ' prt_val],logPrior+lnjac));
               txt = strvcat(txt,sprintf(['conditional log system prior:        ' prt_val],logSP),' ');
               txt = strvcat(txt,'NOTE: The sum of the marginal log prior for theta (original parameters) and','      the log Jacobian is equal to the marginal log prior of phi (transformed','      parameters). The conditional log system prior is added to get the full','      prior up to a constant, determined by the log marginal likelihood of','      the system prior.',' ');
            end;
            %
            % write to file
            %
            PriorJacValueFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-Prior-Jacobian-AdaptiveLearning-' DSGEModel.NameOfModel '-PostMode.txt']);
            fid = fopen(PriorJacValueFile,'wt');
            for i=1:size(txt,1);
               fprintf(fid,'%s\n',txt(i,:));
            end;
            fprintf(fid,'================================================================================\n\n');
            %
            % bottom file data
            %
            fprintf(fid,'Directory for file: %s\n',GetPath(PriorJacValueFile));
            fprintf(fid,'Name of file:       %s\n',GetFilename(PriorJacValueFile));
            fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
            fclose(fid);
            %
            if FileExist(PriorJacValueFile)==1;
               TextGUI(PriorJacValueFile,['DSGE Model Prior and Jacobian under Adaptive Learning - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500],'Posterior Mode',50,CurrINI,1,1);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MonteCarloFiltering'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the user has a system prior
         %
         if strcmp(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
            IsSystemPrior = 0;
         else;
            IsSystemPrior = 1;
         end;
         %
         % there's no point in using Monte Carlo filtering if there's a system prior since
         % prior sampling will always give a unique convergent solution at each prior draw
         % due to the MCMC nature of the sampler (Metropolis-Hastings)
         %
         if IsSystemPrior==0;
            %
            % check if the posterior mode file exists
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            %
            % get parameters and priors
            %
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     thetaIndex = ModeData.thetaIndex;
                     UniformBounds = ModeData.UniformBounds;
                     LowerBound = ModeData.LowerBound;
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            % check if we can continue
            %
            if InitStatus==1;
               %
               % read the prior file for the adaptive learning parameters
               %
               [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
               if status==1;
                  [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                            thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                            LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
                  %
                  % check if we have parameters to estimate
                  %
                  if isempty(ParameterNames.estimated)==0;
                     %
                     % load the AIMData file
                     %
                     ErrorStr = '';
                     if MatlabNumber>=8.0;
                        try;
                           AIMData = load(DSGEModel.AIMDataFile);
                        catch ME;
                           ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
                        end;
                     else;
                        try;
                           AIMData = load(DSGEModel.AIMDataFile);
                        catch;
                           ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
                        end;
                     end;
                     if isempty(ErrorStr)==0;
                        About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                        set(controls.about,'UserData',0);
                        return;
                     end;
                     %
                     % suggest how many prior draws to compute
                     %
                     PriorDrawsStr = Str2Matrix('1,000|1,500|2,000|2,500|3,000|3,500|4,000|4,500|5,000|6,000|7,000|8,000|9,000|10,000|11,000|12,000|13,000|14,000|15,000|16,000|17,000|18,000|19,000|20,000|25,000|30,000|35,000|40,000|45,000|50,000|55,000|60,000|65,000|70,000|75,000|80,000|85,000|90,000|95,000|100,000|110,000|120,000|130,000|140,000|150,000|160,000|170,000|180,000|190,000|200,000|250,000|300,000|350,000|400,000|450,000|500,000','|');
                     %
                     % open a selection dialog
                     %
                     [action,PriorDrawsValue] = SelectionDlg(PriorDrawsStr,DSGEModel.MonteCarloFilterDrawsAL,'Select the number of draws from the prior distribution:','Sampling Selection','Monte Carlo Filtering','','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        NumPriorDraws = str2num(strrep(StringTrim(PriorDrawsStr(PriorDrawsValue,:)),',',''));
                        %
                        PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-' int2str(NumPriorDraws) '-' DSGEModel.NameOfModel '.mat']);
                        %
                        % create output folder if necessary
                        %
                        stat = MakeDir(DSGEModel.OutputDirectory,'priordraws');
                        %
                        % check if we should run the prior sampling function
                        %
                        if (FileExist(PriorDrawsFile)==1)&(DSGEModel.OverwriteDraws==0);
                           PD = load(PriorDrawsFile);
                           thetaDraws = PD.thetaDraws;
                           if isfield(PD,'SolutionCode')==1;
                              SolutionCode = PD.SolutionCode;
                           else;
                              SolutionCode = [];
                           end;
                           SaveDraws = 0;
                        else;
                           thetaDraws = DSGEPriorSampling(PriorDist,thetaDist,LowerBound,NumPriorDraws,1,'Monte Carlo Filtering',DSGEModel,CurrINI);
                           SolutionCode = [];
                           SaveDraws = 1;
                        end;
                        %
                        if isempty(thetaDraws)==0;
                           %
                           % check if we should save the draws to file
                           %
                           if SaveDraws==1;
                              save(PriorDrawsFile,'thetaDraws');
                           end;
                           %
                           % now run the Monte Carlo Filtering function
                           %
                           if isempty(SolutionCode)==1;
                              SolutionCode = MonteCarloFilteringAdaptiveLearning(thetaDraws,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI);
                              if isempty(SolutionCode)==0;
                                 NumUnique = sum(SolutionCode==1);
                                 if (NumUnique>=200)&(NumPriorDraws-NumUnique>=200);
                                    SaveSolutionCode = 1;
                                 elseif (NumUnique==0)|(NumUnique==NumPriorDraws);
                                    SaveSolutionCode = 0;
                                    %
                                    % a really bad case
                                    %
                                    txt = ['The number of parameter draws with a unique and convergent solution under adaptive learning is equal to ' int2str(NumUnique) ' while with number of cases with indeterminacy and no solution are ' int2str(sum(SolutionCode==2)) ' and ' int2str(sum(SolutionCode==0)) ', respectively. Monte Carlo filtering requires that both the number of unique and convergent solutions as well as the sum of the number of cases with indeterminacy and no solution are greater than zero and preferably at least equal to 200. YADA has therefore aborted from the exercise.'];
                                    About(txt,'information','Monte Carlo Filtering',300,500,CurrINI);
                                 else;
                                    if NumUnique>=200;
                                       RecNumDraws = ceil((200/(NumPriorDraws-NumUnique))*NumPriorDraws);
                                    else;
                                       RecNumDraws = ceil((200/NumUnique)*NumPriorDraws);
                                    end;
                                    txt = ['The number of parameter draws with a unique and convergent solution under adaptive learning is equal to ' int2str(NumUnique) ' while with number of cases with indeterminacy and no solution are ' int2str(sum(SolutionCode==2)) ' and ' int2str(sum(SolutionCode==0)) ', respectively. It is recommended that both the number of unique and convergent solutions and the sum of the other two cases are at least 200. To achieve this you should increase the number of draws from the prior distribution to at least ' int2str(RecNumDraws) '. Are you sure that you would like to keep these results?'];
                                    answer = Query(txt,'question',300,'Monte Carlo Filtering',500,'no',CurrINI);
                                    if strcmp(lower(answer),'yes')==1;
                                       SaveSolutionCode = 1;
                                    else;
                                       SaveSolutionCode = 0;
                                    end;
                                 end;
                                 if SaveSolutionCode==1;
                                    save(PriorDrawsFile,'SolutionCode','-APPEND');
                                 else;
                                    delete(PriorDrawsFile);
                                    SolutionCode = [];
                                 end;
                              end;
                           end;
                           %
                           if isempty(SolutionCode)==0;
                              %
                              % keep the new value
                              %
                              DSGEModel.MonteCarloFilterDrawsAL = PriorDrawsValue;
                              %
                              % process results
                              %
                              ResFile = FixFilePath([DSGEModel.OutputDirectory '\MonteCarloFiltering-AdaptiveLearning-' DSGEModel.NameOfModel '-' int2str(NumPriorDraws) '.txt']);
                              PrintMonteCarloFiltering(ResFile,SolutionCode,thetaDraws,ParameterNames.estimated,DSGEModel,CurrINI);
                              if FileExist(ResFile)==1;
                                 TextGUI(ResFile,['Monte Carlo Filtering - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Kolmogorov-Smirnov Tests',50,CurrINI,1,0);
                              end;
                              %
                              % plot the Monte Carlo filtering densities
                              %
                              GraphMonteCarloFilteringDensity(SolutionCode,thetaDraws,ParameterNames.estimated,1,DSGEModel,CurrINI);
                              %
                              % now show graphs
                              %
                              ScatterPlotMonteCarloFiltering('init',SolutionCode,thetaDraws,ParameterNames.estimated,DSGEModel,CurrINI);
                           end;
                        end;
                     end;
                  else;
                     txt = 'All parameters of your DSGE model and adaptive learning mechanism have status calibrated. Hence, there is no prior distribution from which YADA can sample.';
                     About(txt,'information','Monte Carlo Filtering - Calibrated DSGE Model',200,500,CurrINI);
                  end;
               else;
                  txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '".'];
                  About(txt,'information','Monte Carlo Filtering',200,500,CurrINI);
               end;
            end;
         else;
            txt = 'The model has a system prior and YADA has therefore skipped Monte Carlo filtering. Prior sampling with a system prior is achieved with a Metropolis-Hastings algorithm, with the effect that all prior draws imply a unique and convergent solution. This is not consistent with Monte Carlo filtering, which allows some draws to imply indeterminacy or no solution.';
            About(txt,'information','Monte Carlo Filtering',350,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'InfMatFreqDomainInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % compute the information matrix
               %
               [InformationMatrix,ParamIncluded,ParamExcluded,TransStatus,SolutionStatus] = AdaptiveLearningDSGEFrequencyDomainInfMat(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI);
               if (TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(InformationMatrix)==0;
                     %
                     % make sure we only use the estimated parameters
                     %
                     ParameterNames = ParameterNames.estimated;
                     %
                     % determine which parameters are included and which are excluded from the analysis
                     %
                     IncludedParameterNames = ParameterNames(ParamIncluded,:);
                     ExcludedParameterNames = ParameterNames(ParamExcluded,:);
                     %
                     % compute the inverse of Fisher's information if possible
                     %
                     if (rank(InformationMatrix,1.02-010)==length(InformationMatrix))&(IsPositiveDefinite(InformationMatrix)==1);
                        invIMatrix = eye(length(InformationMatrix))/InformationMatrix;
                        stdvec = sqrt(diag(invIMatrix));
                        CorrMatrix = invIMatrix./(stdvec*stdvec');
                        %
                        % compute eigenvectors and eigenvalues of the correlation amtrix
                        %
                        [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                        %
                        % check which parameters have an absolute correlation greater than or equal 0.7
                        %
                        ExtremeCorrBound = 0.7;
                        CorrMatrix = tril(CorrMatrix,-1);
                        [RowIndex,ColIndex] = find(abs(CorrMatrix)>=ExtremeCorrBound);
                        if isempty(RowIndex)==0;
                           ExtremeCorrParameters = cell(length(RowIndex),3);
                           for i=1:length(RowIndex)
                              ExtremeCorrParameters{i,1} = IncludedParameterNames(RowIndex(i),:);
                              ExtremeCorrParameters{i,2} = IncludedParameterNames(ColIndex(i),:);
                              ExtremeCorrParameters{i,3} = CorrMatrix(RowIndex(i),ColIndex(i));
                           end;
                        else;
                           ExtremeCorrParameters = [];
                        end;
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-FrequencyDomain-AdaptiveLearning-InitialValues-' DSGEModel.NameOfModel '.txt']);
                        %
                        % write the ExtremeCorrParameters to file
                        %
                        WriteString = 'wt';
                        if isempty(ExtremeCorrParameters)==0;
                           WriteString = 'at';
                           HeaderStr = strvcat('*       P A R A M E T E R   C O R R E L A T I O N S   U S I N G   T H E        *','* I N V E R S E   O F   F I S H E R '' S   I N F O R M A T I O N   M A T R I X  *','*                U N D E R    A D A P T I V E   L E A R N I N G                *');
                           PrintExtremeCorrelations(TextFile,HeaderStr,'Frequency',ExtremeCorrParameters,num2str(ExtremeCorrBound,'%0.2f'),rank(InformationMatrix),size(IncludedParameterNames,1),CurrINI);
                        end;
                        PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,IncludedParameterNames,WriteString,CurrINI);
                        fid = fopen(TextFile,'at');
                        %
                        % print data on parameter ordering
                        %
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                        fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                        fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                        fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        %
                        fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                        fprintf(fid,'==============================================================\n');
                        for i=1:length(ParamIncluded);
                           fprintf(fid,' %s\n',IncludedParameterNames(ParameterOrdering(i,2),:));
                        end;
                        fprintf(fid,'==============================================================\n\n');
                        %
                        % print which parameters were excluded from the analysis
                        %
                        if isempty(ParamExcluded)==0;
                           fprintf(fid,'NOTE: The following parameters are excluded from the analysis\n      since they don''t affect the population spectrum:\n\n');
                           ExcludedParameterNames = vecstr(ExcludedParameterNames,', ');
                           fprintf(fid,'      %s.\n\n',ExcludedParameterNames(1,1:length(ExcludedParameterNames)-1));
                           fprintf(fid,'================================================================================\n\n');
                        end;
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        fclose(fid);
                        %
                        if FileExist(TextFile)==1;
                           TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,1);
                        end;
                     else;
                        [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-FrequencyDomain-AdaptiveLearning-InitialValues-' DSGEModel.NameOfModel '.txt']);
                        PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,IncludedParameterNames,'wt',CurrINI);
                        fid = fopen(TextFile,'at');
                        fprintf(fid,'Rank of information matrix:    %0.0f\n',rank(InformationMatrix));
                        fprintf(fid,'Number of included parameters: %0.0f\n',length(ParamIncluded));
                        fprintf(fid,'Number of excluded parameters: %0.0f\n\n',length(ParamExcluded));
                        %
                        % print data on parameter ordering
                        %
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                        fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                        fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                        fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        %
                        fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                        fprintf(fid,'==============================================================\n');
                        for i=1:length(ParamIncluded);
                           fprintf(fid,' %s\n',IncludedParameterNames(ParameterOrdering(i,2),:));
                        end;
                        fprintf(fid,'==============================================================\n\n');
                        %
                        % print which parameters were excluded from the analysis
                        %
                        if isempty(ParamExcluded)==0;
                           fprintf(fid,'NOTE: The following parameters are excluded from the analysis\n      since they don''t affect the population spectrum:\n\n');
                           ExcludedParameterNames = vecstr(ExcludedParameterNames,', ');
                           fprintf(fid,'      %s.\n\n',ExcludedParameterNames(1,1:length(ExcludedParameterNames)-1));
                           fprintf(fid,'================================================================================\n\n');
                        end;
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        fclose(fid);
                        if FileExist(TextFile)==1;
                           TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,1);
                        end;
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate Fisher''s information matrix based on the initial values.';
                  About(txt,'information','Information Matrix - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate Fisher''s information matrix based on the initial values.'];
                About(txt,'information','Information Matrix - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'InfMatFreqDomainDSGEMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % compute the information matrix
               %
               [InformationMatrix,ParamIncluded,ParamExcluded,TransStatus] = AdaptiveLearningDSGEFrequencyDomainInfMat(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI);
               if TransStatus==1;
                  %
                  % we're may be OK
                  %
                  if isempty(InformationMatrix)==0;
                     %
                     % make sure we only use the estimated parameters
                     %
                     ParameterNames = ParameterNames.estimated;
                     %
                     % determine which parameters are included and which are excluded from the analysis
                     %
                     IncludedParameterNames = ParameterNames(ParamIncluded,:);
                     ExcludedParameterNames = ParameterNames(ParamExcluded,:);
                     %
                     % compute the inverse of Fisher's information if possible
                     %
                     if (rank(InformationMatrix,1.02-010)==length(InformationMatrix))&(IsPositiveDefinite(InformationMatrix)==1);
                        invIMatrix = eye(length(InformationMatrix))/InformationMatrix;
                        stdvec = sqrt(diag(invIMatrix));
                        CorrMatrix = invIMatrix./(stdvec*stdvec');
                        %
                        % compute eigenvectors and eigenvalues of the correlation amtrix
                        %
                        [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                        %
                        % check which parameters have an absolute correlation greater than or equal 0.7
                        %
                        ExtremeCorrBound = 0.7;
                        CorrMatrix = tril(CorrMatrix,-1);
                        [RowIndex,ColIndex] = find(abs(CorrMatrix)>=ExtremeCorrBound);
                        if isempty(RowIndex)==0;
                           ExtremeCorrParameters = cell(length(RowIndex),3);
                           for i=1:length(RowIndex)
                              ExtremeCorrParameters{i,1} = IncludedParameterNames(RowIndex(i),:);
                              ExtremeCorrParameters{i,2} = IncludedParameterNames(ColIndex(i),:);
                              ExtremeCorrParameters{i,3} = CorrMatrix(RowIndex(i),ColIndex(i));
                           end;
                        else;
                           ExtremeCorrParameters = [];
                        end;
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-FrequencyDomain-AdaptiveLearning-DSGEPosteriorMode-' DSGEModel.NameOfModel '.txt']);
                        %
                        % write the ExtremeCorrParameters to file
                        %
                        WriteString = 'wt';
                        if isempty(ExtremeCorrParameters)==0;
                           WriteString = 'at';
                           HeaderStr = strvcat('*       P A R A M E T E R   C O R R E L A T I O N S   U S I N G   T H E        *','* I N V E R S E   O F   F I S H E R '' S   I N F O R M A T I O N   M A T R I X  *','*                U N D E R    A D A P T I V E   L E A R N I N G                *');
                           PrintExtremeCorrelations(TextFile,HeaderStr,'Frequency',ExtremeCorrParameters,num2str(ExtremeCorrBound,'%0.2f'),rank(InformationMatrix),size(IncludedParameterNames,1),CurrINI);
                        end;
                        PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,IncludedParameterNames,WriteString,CurrINI);
                        fid = fopen(TextFile,'at');
                        %
                        % print data on parameter ordering
                        %
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                        fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                        fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                        fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        %
                        fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                        fprintf(fid,'==============================================================\n');
                        for i=1:length(ParamIncluded);
                           fprintf(fid,' %s\n',IncludedParameterNames(ParameterOrdering(i,2),:));
                        end;
                        fprintf(fid,'==============================================================\n\n');
                        %
                        % print which parameters were excluded from the analysis
                        %
                        if isempty(ParamExcluded)==0;
                           fprintf(fid,'NOTE: The following parameters are excluded from the analysis\n      since they don''t affect the population spectrum:\n\n');
                           ExcludedParameterNames = vecstr(ExcludedParameterNames,', ');
                           fprintf(fid,'      %s.\n\n',ExcludedParameterNames(1,1:length(ExcludedParameterNames)-1));
                           fprintf(fid,'================================================================================\n\n');
                        end;
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        fclose(fid);
                        %
                        if FileExist(TextFile)==1;
                           TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'DSGE Posterior Mode Parameter Values',50,CurrINI,1,1);
                        end;
                     else;
                        [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-FrequencyDomain-AdaptiveLearning-DSGEPosteriorMode-' DSGEModel.NameOfModel '.txt']);
                        PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,IncludedParameterNames,'wt',CurrINI);
                        fid = fopen(TextFile,'at');
                        fprintf(fid,'Rank of information matrix:    %0.0f\n',rank(InformationMatrix));
                        fprintf(fid,'Number of included parameters: %0.0f\n',length(ParamIncluded));
                        fprintf(fid,'Number of excluded parameters: %0.0f\n\n',length(ParamExcluded));
                        %
                        % print data on parameter ordering
                        %
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                        fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                        fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                        fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        %
                        fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                        fprintf(fid,'==============================================================\n');
                        for i=1:length(ParamIncluded);
                           fprintf(fid,' %s\n',IncludedParameterNames(ParameterOrdering(i,2),:));
                        end;
                        fprintf(fid,'==============================================================\n\n');
                        %
                        % print which parameters were excluded from the analysis
                        %
                        if isempty(ParamExcluded)==0;
                           fprintf(fid,'NOTE: The following parameters are excluded from the analysis\n      since they don''t affect the population spectrum:\n\n');
                           ExcludedParameterNames = vecstr(ExcludedParameterNames,', ');
                           fprintf(fid,'      %s.\n\n',ExcludedParameterNames(1,1:length(ExcludedParameterNames)-1));
                           fprintf(fid,'================================================================================\n\n');
                        end;
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        fclose(fid);
                        if FileExist(TextFile)==1;
                           TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'DSGE Posterior Mode Parameter Values',50,CurrINI,1,1);
                        end;
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate Fisher''s information matrix based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Information Matrix - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate Fisher''s information matrix based on the DSGE model posterior mode estimates.'];
               About(txt,'information','Information Matrix - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'InfMatFreqDomainPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
            %
            % now we call the information matrix estimation function which also deals with the log-likelihood
            %
            % calculate Fisher's information matrix in the frequency domain for the DSGE model
            %
            [InformationMatrix,ParamIncluded,ParamExcluded] = AdaptiveLearningDSGEFrequencyDomainInfMat(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI);
            if isempty(InformationMatrix)==0;
               %
               % determine which parameters are included and which are excluded from the analysis
               %
               IncludedParameterNames = ParameterNames(ParamIncluded,:);
               ExcludedParameterNames = ParameterNames(ParamExcluded,:);
               %
               % compute the inverse of Fisher's information if possible
               %
               if (rank(InformationMatrix,1.02-010)==length(InformationMatrix))&(IsPositiveDefinite(InformationMatrix)==1);
                  invIMatrix = eye(length(InformationMatrix))/InformationMatrix;
                  stdvec = sqrt(diag(invIMatrix));
                  CorrMatrix = invIMatrix./(stdvec*stdvec');
                  %
                  % compute eigenvectors and eigenvalues of the correlation amtrix
                  %
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  % check which parameters have an absolute correlation greater than or equal 0.7
                  %
                  ExtremeCorrBound = 0.7;
                  CorrMatrix = tril(CorrMatrix,-1);
                  [RowIndex,ColIndex] = find(abs(CorrMatrix)>=ExtremeCorrBound);
                  if isempty(RowIndex)==0;
                     ExtremeCorrParameters = cell(length(RowIndex),3);
                     for i=1:length(RowIndex)
                        ExtremeCorrParameters{i,1} = IncludedParameterNames(RowIndex(i),:);
                        ExtremeCorrParameters{i,2} = IncludedParameterNames(ColIndex(i),:);
                        ExtremeCorrParameters{i,3} = CorrMatrix(RowIndex(i),ColIndex(i));
                     end;
                  else;
                     ExtremeCorrParameters = [];
                  end;
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-FrequencyDomain-AdaptiveLearning-PosteriorMode-' DSGEModel.NameOfModel '.txt']);
                  %
                  % write the ExtremeCorrParameters to file
                  %
                  WriteString = 'wt';
                  if isempty(ExtremeCorrParameters)==0;
                     WriteString = 'at';
                     HeaderStr = strvcat('*       P A R A M E T E R   C O R R E L A T I O N S   U S I N G   T H E        *','* I N V E R S E   O F   F I S H E R '' S   I N F O R M A T I O N   M A T R I X  *','*                U N D E R    A D A P T I V E   L E A R N I N G                *');
                     PrintExtremeCorrelations(TextFile,HeaderStr,'Frequency',ExtremeCorrParameters,num2str(ExtremeCorrBound,'%0.2f'),rank(InformationMatrix),size(IncludedParameterNames,1),CurrINI);
                  end;
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,IncludedParameterNames,WriteString,CurrINI);
                  fid = fopen(TextFile,'at');
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(ParamIncluded);
                     fprintf(fid,' %s\n',IncludedParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % print which parameters were excluded from the analysis
                  %
                  if isempty(ParamExcluded)==0;
                     fprintf(fid,'NOTE: The following parameters are excluded from the analysis\n      since they don''t affect the population spectrum:\n\n');
                     ExcludedParameterNames = vecstr(ExcludedParameterNames,', ');
                     fprintf(fid,'      %s.\n\n',ExcludedParameterNames(1,1:length(ExcludedParameterNames)-1));
                     fprintf(fid,'================================================================================\n\n');
                  end;
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               else;
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-FrequencyDomain-AdaptiveLearning-PosteriorMode-' DSGEModel.NameOfModel '.txt']);
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,IncludedParameterNames,'wt',CurrINI);
                  fid = fopen(TextFile,'at');
                  fprintf(fid,'Rank of information matrix:    %0.0f\n',rank(InformationMatrix));
                  fprintf(fid,'Number of included parameters: %0.0f\n',length(ParamIncluded));
                  fprintf(fid,'Number of excluded parameters: %0.0f\n\n',length(ParamExcluded));
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(ParamIncluded);
                     fprintf(fid,' %s\n',IncludedParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % print which parameters were excluded from the analysis
                  %
                  if isempty(ParamExcluded)==0;
                     fprintf(fid,'NOTE: The following parameters are excluded from the analysis\n      since they don''t affect the population spectrum:\n\n');
                     ExcludedParameterNames = vecstr(ExcludedParameterNames,', ');
                     fprintf(fid,'      %s.\n\n',ExcludedParameterNames(1,1:length(ExcludedParameterNames)-1));
                     fprintf(fid,'================================================================================\n\n');
                  end;
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'InfMatTimeDomainInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % compute the information matrix
               %
               [InformationMatrix,BeliefStatus,TransStatus,SolutionStatus] = AdaptiveLearningDSGEInformationMatrix(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI);
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(InformationMatrix)==0;
                     %
                     % make sure we only use the estimated parameters
                     %
                     ParameterNames = ParameterNames.estimated;
                     %
                     % check the rank
                     %
                     if rank(InformationMatrix,1.02-010)==length(theta);
                        invIMatrix = eye(length(theta))/InformationMatrix;
                        stdvec = sqrt(diag(invIMatrix));
                        CorrMatrix = invIMatrix./(stdvec*stdvec');
                        %
                        % compute eigenvectors and eigenvalues of the correlation amtrix
                        %
                        [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                        %
                        % check which parameters have an absolute correlation greater than or equal 0.7
                        %
                        ExtremeCorrBound = 0.7;
                        CorrMatrix = tril(CorrMatrix,-1);
                        [RowIndex,ColIndex] = find(abs(CorrMatrix)>=ExtremeCorrBound);
                        if isempty(RowIndex)==0;
                           ExtremeCorrParameters = cell(length(RowIndex),3);
                           for i=1:length(RowIndex)
                              ExtremeCorrParameters{i,1} = ParameterNames(RowIndex(i),:);
                              ExtremeCorrParameters{i,2} = ParameterNames(ColIndex(i),:);
                              ExtremeCorrParameters{i,3} = CorrMatrix(RowIndex(i),ColIndex(i));
                           end;
                        else;
                           ExtremeCorrParameters = [];
                        end;
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-TimeDomain-AdaptiveLearning-InitialValues-' DSGEModel.NameOfModel '.txt']);
                        %
                        % write the ExtremeCorrParameters to file
                        %
                        WriteString = 'wt';
                        if isempty(ExtremeCorrParameters)==0;
                           WriteString = 'at';
                           HeaderStr = strvcat('*       P A R A M E T E R   C O R R E L A T I O N S   U S I N G   T H E        *','* I N V E R S E   O F   F I S H E R '' S   I N F O R M A T I O N   M A T R I X  *');
                           PrintExtremeCorrelations(TextFile,HeaderStr,'Time',ExtremeCorrParameters,num2str(ExtremeCorrBound,'%0.2f'),rank(InformationMatrix),length(theta),CurrINI);
                        end;
                        PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,ParameterNames,WriteString,CurrINI);
                        fid = fopen(TextFile,'at');
                        %
                        % print data on parameter ordering
                        %
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                        fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                        fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                        fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        %
                        fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                        fprintf(fid,'==============================================================\n');
                        for i=1:length(theta);
                           fprintf(fid,' %s\n',ParameterNames(ParameterOrdering(i,2),:));
                        end;
                        fprintf(fid,'==============================================================\n\n');
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        fclose(fid);
                        %
                        if FileExist(TextFile)==1;
                           TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,1);
                        end;
                     else;
                        [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-TimeDomain-AdaptiveLearning-InitialValues-' DSGEModel.NameOfModel '.txt']);
                        PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,ParameterNames,'wt',CurrINI);
                        fid = fopen(TextFile,'at');
                        fprintf(fid,'Rank of information matrix: %0.0f\n',rank(InformationMatrix));
                        fprintf(fid,'Number of parameters:       %0.0f\n\n',length(theta));
                        %
                        % print data on parameter ordering
                        %
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                        fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                        fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                        fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        %
                        fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                        fprintf(fid,'==============================================================\n');
                        for i=1:length(theta);
                           fprintf(fid,' %s\n',ParameterNames(ParameterOrdering(i,2),:));
                        end;
                        fprintf(fid,'==============================================================\n\n');
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        fclose(fid);
                        if FileExist(TextFile)==1;
                           TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,1);
                        end;
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate Fisher''s information matrix based on the initial values.';
                  About(txt,'information','Information Matrix - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate Fisher''s information matrix based on the initial values.';
                  About(txt,'information','Information Matrix - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate Fisher''s information matrix based on the initial values.'];
                About(txt,'information','Information Matrix - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'InfMatTimeDomainDSGEMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % compute the information matrix
               %
               [InformationMatrix,BeliefStatus,TransStatus] = AdaptiveLearningDSGEInformationMatrix(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI);
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(InformationMatrix)==0;
                     %
                     % make sure we only use the estimated parameters
                     %
                     ParameterNames = ParameterNames.estimated;
                     %
                     % check the rank
                     %
                     if rank(InformationMatrix,1.02-010)==length(theta);
                        invIMatrix = eye(length(theta))/InformationMatrix;
                        stdvec = sqrt(diag(invIMatrix));
                        CorrMatrix = invIMatrix./(stdvec*stdvec');
                        %
                        % compute eigenvectors and eigenvalues of the correlation amtrix
                        %
                        [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                        %
                        % check which parameters have an absolute correlation greater than or equal 0.7
                        %
                        ExtremeCorrBound = 0.7;
                        CorrMatrix = tril(CorrMatrix,-1);
                        [RowIndex,ColIndex] = find(abs(CorrMatrix)>=ExtremeCorrBound);
                        if isempty(RowIndex)==0;
                           ExtremeCorrParameters = cell(length(RowIndex),3);
                           for i=1:length(RowIndex)
                              ExtremeCorrParameters{i,1} = ParameterNames(RowIndex(i),:);
                              ExtremeCorrParameters{i,2} = ParameterNames(ColIndex(i),:);
                              ExtremeCorrParameters{i,3} = CorrMatrix(RowIndex(i),ColIndex(i));
                           end;
                        else;
                           ExtremeCorrParameters = [];
                        end;
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-TimeDomain-AdaptiveLearning-DSGEPosteriorMode-' DSGEModel.NameOfModel '.txt']);
                        %
                        % write the ExtremeCorrParameters to file
                        %
                        WriteString = 'wt';
                        if isempty(ExtremeCorrParameters)==0;
                           WriteString = 'at';
                           HeaderStr = strvcat('*       P A R A M E T E R   C O R R E L A T I O N S   U S I N G   T H E        *','* I N V E R S E   O F   F I S H E R '' S   I N F O R M A T I O N   M A T R I X  *');
                           PrintExtremeCorrelations(TextFile,HeaderStr,'Time',ExtremeCorrParameters,num2str(ExtremeCorrBound,'%0.2f'),rank(InformationMatrix),length(theta),CurrINI);
                        end;
                        PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,ParameterNames,WriteString,CurrINI);
                        fid = fopen(TextFile,'at');
                        %
                        % print data on parameter ordering
                        %
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                        fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                        fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                        fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        %
                        fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                        fprintf(fid,'==============================================================\n');
                        for i=1:length(theta);
                           fprintf(fid,' %s\n',ParameterNames(ParameterOrdering(i,2),:));
                        end;
                        fprintf(fid,'==============================================================\n\n');
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        fclose(fid);
                        %
                        if FileExist(TextFile)==1;
                           TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'DSGE Posterior Mode Parameter Values',50,CurrINI,1,1);
                        end;
                     else;
                        [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                        %
                        TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-TimeDomain-AdaptiveLearning-DSGEPosteriorMode-' DSGEModel.NameOfModel '.txt']);
                        PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,ParameterNames,'wt',CurrINI);
                        fid = fopen(TextFile,'at');
                        fprintf(fid,'Rank of information matrix: %0.0f\n',rank(InformationMatrix));
                        fprintf(fid,'Number of parameters:       %0.0f\n\n',length(theta));
                        %
                        % print data on parameter ordering
                        %
                        fprintf(fid,'********************************************************************************\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                        fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                        fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                        fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                        fprintf(fid,'*                                                                              *\n');
                        fprintf(fid,'********************************************************************************\n\n');
                        %
                        fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                        fprintf(fid,'==============================================================\n');
                        for i=1:length(theta);
                           fprintf(fid,' %s\n',ParameterNames(ParameterOrdering(i,2),:));
                        end;
                        fprintf(fid,'==============================================================\n\n');
                        %
                        % Some information about the filename and its location
                        %
                        fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                        fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                        fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                        fclose(fid);
                        if FileExist(TextFile)==1;
                           TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'DSGE Posterior Mode Parameter Values',50,CurrINI,1,1);
                        end;
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate Fisher''s information matrix based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Information Matrix - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate Fisher''s information matrix based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Information Matrix - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate Fisher''s information matrix based on the DSGE model posterior mode estimates.'];
               About(txt,'information','Information Matrix - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'InfMatTimeDomainPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
            %
            % now we call the information matrix estimation function which also deals with the log-likelihood
            %
            InformationMatrix = AdaptiveLearningDSGEInformationMatrix(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI);
            if isempty(InformationMatrix)==0;
               %
               % check the rank
               %
               if rank(InformationMatrix,1.02-010)==length(theta);
                  invIMatrix = eye(length(theta))/InformationMatrix;
                  stdvec = sqrt(diag(invIMatrix));
                  CorrMatrix = invIMatrix./(stdvec*stdvec');
                  %
                  % compute eigenvectors and eigenvalues of the correlation amtrix
                  %
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  % check which parameters have an absolute correlation greater than or equal 0.7
                  %
                  ExtremeCorrBound = 0.7;
                  CorrMatrix = tril(CorrMatrix,-1);
                  [RowIndex,ColIndex] = find(abs(CorrMatrix)>=ExtremeCorrBound);
                  if isempty(RowIndex)==0;
                     ExtremeCorrParameters = cell(length(RowIndex),3);
                     for i=1:length(RowIndex)
                        ExtremeCorrParameters{i,1} = ParameterNames(RowIndex(i),:);
                        ExtremeCorrParameters{i,2} = ParameterNames(ColIndex(i),:);
                        ExtremeCorrParameters{i,3} = CorrMatrix(RowIndex(i),ColIndex(i));
                     end;
                  else;
                     ExtremeCorrParameters = [];
                  end;
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-TimeDomain-AdaptiveLearning-PosteriorMode-' DSGEModel.NameOfModel '.txt']);
                  %
                  % write the ExtremeCorrParameters to file
                  %
                  WriteString = 'wt';
                  if isempty(ExtremeCorrParameters)==0;
                     WriteString = 'at';
                     HeaderStr = strvcat('*       P A R A M E T E R   C O R R E L A T I O N S   U S I N G   T H E        *','* I N V E R S E   O F   F I S H E R '' S   I N F O R M A T I O N   M A T R I X  *');
                     PrintExtremeCorrelations(TextFile,HeaderStr,'Time',ExtremeCorrParameters,num2str(ExtremeCorrBound,'%0.2f'),rank(InformationMatrix),length(theta),CurrINI);
                  end;
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,ParameterNames,WriteString,CurrINI);
                  fid = fopen(TextFile,'at');
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(theta);
                     fprintf(fid,' %s\n',ParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               else;
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-TimeDomain-AdaptiveLearning-PosteriorMode-' DSGEModel.NameOfModel '.txt']);
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,ParameterNames,'wt',CurrINI);
                  fid = fopen(TextFile,'at');
                  fprintf(fid,'Rank of information matrix: %0.0f\n',rank(InformationMatrix));
                  fprintf(fid,'Number of parameters:       %0.0f\n\n',length(theta));
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                U N D E R    A D A P T I V E   L E A R N I N G                *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(theta);
                     fprintf(fid,' %s\n',ParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Adaptive Learning: Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVariablesInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save estimates of the state variables for the adaptive learning version at the initial parameter values to file?';
                     answer = Query(txt,'question',140,'Save - State Variables',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        StateVariableForecast = StateVarStructure.Ksitt1;
                        StateVariableUpdate = StateVarStructure.Ksitt;
                        StateVariableSmooth = StateVarStructure.KsitT;
                        %
                        txt = 'Would you also like to save the covariance matrices of the state variables based on the three estimators?';
                        covanswer = Query(txt,'question',140,'Save - State Variable Covariance Matrices',500,'no',CurrINI);
                        if strcmp(lower(covanswer),'yes')==1;
                           ForecastCovMatrix = StateVarStructure.Ptt1;
                           UpdateCovMatrix = StateVarStructure.Ptt;
                           SmoothCovMatrix = StateVarStructure.PtT;
                           CovStr = 'ForecastCovMatrix (covariance matrix based on the forecasted state variables), UpdateCovMatrix (covariance matrix based on the update estimates), SmoothCovMatrix (covariance matrix based on the smooth estimates), ';
                           AddEntry = 3;
                        else;
                           CovStr = '';
                           AddEntry = 0;
                        end;
                        NumEntries = 5+AddEntry;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                        if length(DateVector)>size(StateVariableForecast,2);
                           DateVector = DateVector(1,length(DateVector)-size(StateVariableForecast,2)+1:length(DateVector));
                        end;
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateVariables-AdaptiveLearning-InitialValues.mat']);
                        if AddEntry==0;
                           save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','StateVariableNames','DateVector');
                        else;
                           save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','ForecastCovMatrix','UpdateCovMatrix','SmoothCovMatrix','StateVariableNames','DateVector');
                        end;
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\SVALInitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% matrix with estimates of the forecasted state variables\n%%\n');
                        fprintf(fid,'StateVariableForecast = DataStructure.StateVariableForecast;\n');
                        fprintf(fid,'%%\n%% matrix with estimates of the updated state variable projections\n%%\n');
                        fprintf(fid,'StateVariableUpdate = DataStructure.StateVariableUpdate;\n');
                        fprintf(fid,'%%\n%% matrix with estimates of the smoothed state variable projections\n%%\n');
                        fprintf(fid,'StateVariableSmooth = DataStructure.StateVariableSmooth;\n');
                        if AddEntry==3;
                           fprintf(fid,'%%\n%% matrix with covariances of the estimates of the forecasted state variables\n%%\n');
                           fprintf(fid,'ForecastCovMatrix = DataStructure.ForecastCovMatrix;\n');
                           fprintf(fid,'%%\n%% matrix with covariances of the estimates of the updated state variable projections\n%%\n');
                           fprintf(fid,'UpdateCovMatrix = DataStructure.UpdateCovMatrix;\n');
                           fprintf(fid,'%%\n%% matrix with covariances of the estimates of the smoothed state variable projections\n%%\n');
                           fprintf(fid,'SmoothCovMatrix = DataStructure.SmoothCovMatrix;\n');
                        end;
                        fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['Estimates of the state variables at the initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: StateVariableForecast (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the forecasted state variables), StateVariableUpdate (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the updated state variable projections), StateVariableSmooth (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the smoothed state variable projections), ' CovStr 'StateVariableNames (string matrix with the names of the state variables), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','State Variables - Initial Values',500,600,CurrINI);
                     end;
                     %
                     % Display the graphs
                     %
                     StateGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Initial Values','State');
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the state variables based on the initial values.';
                  About(txt,'information','State Variables - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the state variables based on the initial values.';
                  About(txt,'information','State Variables - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the state variables based on the initial values.'];
                About(txt,'information','State Variables - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVariablesDSGEMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save estimates of the state variables for the adaptive learning version at the DSGE posterior mode values to file?';
                     answer = Query(txt,'question',140,'Save - State Variables',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        StateVariableForecast = StateVarStructure.Ksitt1;
                        StateVariableUpdate = StateVarStructure.Ksitt;
                        StateVariableSmooth = StateVarStructure.KsitT;
                        %
                        txt = 'Would you also like to save the covariance matrices of the state variables based on the three estimators?';
                        covanswer = Query(txt,'question',140,'Save - State Variable Covariance Matrices',500,'no',CurrINI);
                        if strcmp(lower(covanswer),'yes')==1;
                           ForecastCovMatrix = StateVarStructure.Ptt1;
                           UpdateCovMatrix = StateVarStructure.Ptt;
                           SmoothCovMatrix = StateVarStructure.PtT;
                           CovStr = 'ForecastCovMatrix (covariance matrix based on the forecasted state variables), UpdateCovMatrix (covariance matrix based on the update estimates), SmoothCovMatrix (covariance matrix based on the smooth estimates), ';
                           AddEntry = 3;
                        else;
                           CovStr = '';
                           AddEntry = 0;
                        end;
                        NumEntries = 5+AddEntry;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                        if length(DateVector)>size(StateVariableForecast,2);
                           DateVector = DateVector(1,length(DateVector)-size(StateVariableForecast,2)+1:length(DateVector));
                        end;
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateVariables-AdaptiveLearning-DSGEPosteriorMode.mat']);
                        if AddEntry==0;
                           save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','StateVariableNames','DateVector');
                        else;
                           save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','ForecastCovMatrix','UpdateCovMatrix','SmoothCovMatrix','StateVariableNames','DateVector');
                        end;
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\SVALDSGEMode.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% matrix with estimates of the forecasted state variables\n%%\n');
                        fprintf(fid,'StateVariableForecast = DataStructure.StateVariableForecast;\n');
                        fprintf(fid,'%%\n%% matrix with estimates of the updated state variable projections\n%%\n');
                        fprintf(fid,'StateVariableUpdate = DataStructure.StateVariableUpdate;\n');
                        fprintf(fid,'%%\n%% matrix with estimates of the smoothed state variable projections\n%%\n');
                        fprintf(fid,'StateVariableSmooth = DataStructure.StateVariableSmooth;\n');
                        if AddEntry==3;
                           fprintf(fid,'%%\n%% matrix with covariances of the estimates of the forecasted state variables\n%%\n');
                           fprintf(fid,'ForecastCovMatrix = DataStructure.ForecastCovMatrix;\n');
                           fprintf(fid,'%%\n%% matrix with covariances of the estimates of the updated state variable projections\n%%\n');
                           fprintf(fid,'UpdateCovMatrix = DataStructure.UpdateCovMatrix;\n');
                           fprintf(fid,'%%\n%% matrix with covariances of the estimates of the smoothed state variable projections\n%%\n');
                           fprintf(fid,'SmoothCovMatrix = DataStructure.SmoothCovMatrix;\n');
                        end;
                        fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['Estimates of the state variables at the DSGE posterior mode values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: StateVariableForecast (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the forecasted state variables), StateVariableUpdate (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the updated state variable projections), StateVariableSmooth (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the smoothed state variable projections), ' CovStr 'StateVariableNames (string matrix with the names of the state variables), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','State Variables - DSGE Posterior Mode',500,600,CurrINI);
                     end;
                     %
                     % Display the graphs
                     %
                     StateGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'DSGE Posterior Mode','State');
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the state variables based on the DSGE model posterior mode estimates.';
                  About(txt,'information','State Variables - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the state variables based on the DSGE model posterior mode estimates.';
                  About(txt,'information','State Variables - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the state variables based on the DSGE model posterior mode estimates.'];
               About(txt,'information','State Variables - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVariablesPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % now we call the state variable estimation function which also deals with the log-likelihood
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,0);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if (BeliefStatus==1)&(TransStatus==1);
               %
               % we're may be OK
               %
               if isempty(StateVarStructure)==0;
                  %
                  % ask if we should save to disk
                  %
                  txt = 'Would you like to save estimates of the state variables for the adaptive learning version at posterior mode to file?';
                  answer = Query(txt,'question',140,'Save - State Variables',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     StateVariableForecast = StateVarStructure.Ksitt1;
                     StateVariableUpdate = StateVarStructure.Ksitt;
                     StateVariableSmooth = StateVarStructure.KsitT;
                     %
                     txt = 'Would you also like to save the covariance matrices of the state variables based on the three estimators?';
                     covanswer = Query(txt,'question',140,'Save - State Variable Covariance Matrices',500,'no',CurrINI);
                     if strcmp(lower(covanswer),'yes')==1;
                        ForecastCovMatrix = StateVarStructure.Ptt1;
                        UpdateCovMatrix = StateVarStructure.Ptt;
                        SmoothCovMatrix = StateVarStructure.PtT;
                        CovStr = 'ForecastCovMatrix (covariance matrix based on the forecasted state variables), UpdateCovMatrix (covariance matrix based on the update estimates), SmoothCovMatrix (covariance matrix based on the smooth estimates), ';
                        AddEntry = 3;
                     else;
                        CovStr = '';
                        AddEntry = 0;
                     end;
                     NumEntries = 5+AddEntry;
                     StateVariableNames = DSGEModel.StateVariableNames;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     if length(DateVector)>size(StateVariableForecast,2);
                        DateVector = DateVector(1,length(DateVector)-size(StateVariableForecast,2)+1:length(DateVector));
                     end;
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateVariables-AdaptiveLearning-PosteriorMode.mat']);
                     if AddEntry==0;
                        save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','StateVariableNames','DateVector');
                     else;
                        save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','ForecastCovMatrix','UpdateCovMatrix','SmoothCovMatrix','StateVariableNames','DateVector');
                     end;
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SVALPostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% matrix with estimates of the forecasted state variables\n%%\n');
                     fprintf(fid,'StateVariableForecast = DataStructure.StateVariableForecast;\n');
                     fprintf(fid,'%%\n%% matrix with estimates of the updated state variable projections\n%%\n');
                     fprintf(fid,'StateVariableUpdate = DataStructure.StateVariableUpdate;\n');
                     fprintf(fid,'%%\n%% matrix with estimates of the smoothed state variable projections\n%%\n');
                     fprintf(fid,'StateVariableSmooth = DataStructure.StateVariableSmooth;\n');
                     if AddEntry==3;
                        fprintf(fid,'%%\n%% matrix with covariances of the estimates of the forecasted state variables\n%%\n');
                        fprintf(fid,'ForecastCovMatrix = DataStructure.ForecastCovMatrix;\n');
                        fprintf(fid,'%%\n%% matrix with covariances of the estimates of the updated state variable projections\n%%\n');
                        fprintf(fid,'UpdateCovMatrix = DataStructure.UpdateCovMatrix;\n');
                        fprintf(fid,'%%\n%% matrix with covariances of the estimates of the smoothed state variable projections\n%%\n');
                        fprintf(fid,'SmoothCovMatrix = DataStructure.SmoothCovMatrix;\n');
                     end;
                     fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                     fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                     fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['Estimates of the state variables at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: StateVariableForecast (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the forecasted state variables), StateVariableUpdate (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the updated state variable projections), StateVariableSmooth (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the smoothed state variable projections), ' CovStr 'StateVariableNames (string matrix with the names of the state variables), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','State Variables - Posterior Mode',500,600,CurrINI);
                  end;
                  %
                  % Display the graphs
                  %
                  StateGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Posterior Mode','State');
               end;
            elseif TransStatus==0;
               txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the state variables based on the posterior mode estimates.';
               About(txt,'information','State Variables - Posterior Mode',300,600,CurrINI);
            elseif BeliefStatus==0;
               txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the state variables based on the posterior mode estimates.';
               About(txt,'information','State Variables - Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateShocksInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're may be OK
                  %
                  if (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.etatT)==0);
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save the smooth and update estimates of the state shocks for the adaptive learning version at the initial parameter values to file?';
                     answer = Query(txt,'question',140,'Save - State Shocks',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        SmoothStateShocks = StateVarStructure.etatT;
                        UpdateStateShocks = StateVarStructure.etatt;
                        NumEntries = 4;
                        StateShockNames = DSGEModel.StateShockNames(StateVarStructure.KeepVar,:);
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                        if length(DateVector)>size(SmoothStateShocks,2);
                           DateVector = DateVector(1,length(DateVector)-size(SmoothStateShocks,2)+1:length(DateVector));
                        end;
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateShocks-AdaptiveLearning-InitialValues.mat']);
                        save(file,'SmoothStateShocks','UpdateStateShocks','StateShockNames','DateVector');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\SSALInitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% matrix with the smooth estimates of the non-zero economic shocks\n%%\n');
                        fprintf(fid,'SmoothStateShocks = DataStructure.SmoothStateShocks;\n');
                        fprintf(fid,'%%\n%% matrix with the update estimates of the non-zero economic shocks\n%%\n');
                        fprintf(fid,'UpdateStateShocks = DataStructure.UpdateStateShocks;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the non-zero economic shocks\n%%\n');
                        fprintf(fid,'StateShockNames = DataStructure.StateShockNames;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The smooth and update estimates of the state shocks at the initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: SmoothStateShocks and UpdateStateShocks (' num2str(size(SmoothStateShocks,1),'%0.0f') 'x' num2str(size(SmoothStateShocks,2),'%0.0f') ' matrix with the smooth and update estimates of the non-zero economic shocks), StateShockNames (string matrix with the names of the non-zero economic shocks), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','State Shocks - Initial Values',200,500,CurrINI);
                     end;
                     %
                     % Display the graphs
                     %
                     ShockGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Initial Values','State Shock');
                  elseif (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.etatT)==1);
                     txt = 'YADA couldn''t estimate the state shocks. Please make sure that your model has state shocks!';
                     About(txt,'information','YADA - State Shocks',120,500,CurrINI);
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the state shocks based on the initial values.';
                  About(txt,'information','State Shocks - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the state shocks based on the initial values.';
                  About(txt,'information','State Shocks - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the state shocks based on the initial values.'];
                About(txt,'information','State Shocks - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateShocksDSGEMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're may be OK
                  %
                  if (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.etatT)==0);
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save the smooth and update estimates of the state shocks for the adaptive learning version at the DSGE posterior mode values to file?';
                     answer = Query(txt,'question',140,'Save - State Shocks',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        SmoothStateShocks = StateVarStructure.etatT;
                        UpdateStateShocks = StateVarStructure.etatt;
                        NumEntries = 4;
                        StateShockNames = DSGEModel.StateShockNames(StateVarStructure.KeepVar,:);
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                        if length(DateVector)>size(SmoothStateShocks,2);
                           DateVector = DateVector(1,length(DateVector)-size(SmoothStateShocks,2)+1:length(DateVector));
                        end;
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateShocks-AdaptiveLearning-DSGEPosteriorMode.mat']);
                        save(file,'SmoothStateShocks','UpdateStateShocks','StateShockNames','DateVector');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\SSALDSGEMode.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% matrix with the smooth estimates of the non-zero economic shocks\n%%\n');
                        fprintf(fid,'SmoothStateShocks = DataStructure.SmoothStateShocks;\n');
                        fprintf(fid,'%%\n%% matrix with the update estimates of the non-zero economic shocks\n%%\n');
                        fprintf(fid,'UpdateStateShocks = DataStructure.UpdateStateShocks;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the non-zero economic shocks\n%%\n');
                        fprintf(fid,'StateShockNames = DataStructure.StateShockNames;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The smooth and update estimates of the state shocks at the DSGE posterior mode values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: SmoothStateShocks and UpdateStateShocks (' num2str(size(SmoothStateShocks,1),'%0.0f') 'x' num2str(size(SmoothStateShocks,2),'%0.0f') ' matrix with the smooth and update estimates of the non-zero economic shocks), StateShockNames (string matrix with the names of the non-zero economic shocks), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','State Shocks - DSGE Posterior Mode',200,500,CurrINI);
                     end;
                     %
                     % Display the graphs
                     %
                     ShockGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'DSGE Posterior Mode','State Shock');
                  elseif (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.etatT)==1);
                     txt = 'YADA couldn''t estimate the state shocks. Please make sure that your model has state shocks!';
                     About(txt,'information','YADA - State Shocks',120,500,CurrINI);
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the state shocks based on the DSGE model posterior mode estimates.';
                  About(txt,'information','State Shocks - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the state shocks based on the DSGE model posterior mode estimates.';
                  About(txt,'information','State Shocks - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the state shocks based on the DSGE model posterior mode estimates.'];
               About(txt,'information','State Shocks - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateShocksPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % now we call the state variable estimation function which also deals with the log-likelihood
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,0);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if (BeliefStatus==1)&(TransStatus==1);
               %
               % we're may be OK
               %
               if (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.etatT)==0);
                  %
                  % ask if we should save to disk
                  %
                  txt = 'Would you like to save the smooth and update estimates of the state shocks for the adaptive learning version at posterior mode to file?';
                  answer = Query(txt,'question',140,'Save - State Shocks',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     SmoothStateShocks = StateVarStructure.etatT;
                     UpdateStateShocks = StateVarStructure.etatt;
                     NumEntries = 4;
                     StateShockNames = DSGEModel.StateShockNames(StateVarStructure.KeepVar,:);
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     if length(DateVector)>size(SmoothStateShocks,2);
                        DateVector = DateVector(1,length(DateVector)-size(SmoothStateShocks,2)+1:length(DateVector));
                     end;
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateShocks-AdaptiveLearning-PosteriorMode.mat']);
                     save(file,'SmoothStateShocks','UpdateStateShocks','StateShockNames','DateVector');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SSALPostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% matrix with the smooth estimates of the non-zero economic shocks\n%%\n');
                     fprintf(fid,'SmoothStateShocks = DataStructure.SmoothStateShocks;\n');
                     fprintf(fid,'%%\n%% matrix with the update estimates of the non-zero economic shocks\n%%\n');
                     fprintf(fid,'UpdateStateShocks = DataStructure.UpdateStateShocks;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the non-zero economic shocks\n%%\n');
                     fprintf(fid,'StateShockNames = DataStructure.StateShockNames;\n');
                     fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The smooth and update estimates of the state shocks at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: SmoothStateShocks and UpdateStateShocks (' num2str(size(SmoothStateShocks,1),'%0.0f') 'x' num2str(size(SmoothStateShocks,2),'%0.0f') ' matrix with the smooth and update estimates of the non-zero economic shocks), StateShockNames (string matrix with the names of the non-zero economic shocks), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','State Shocks - Posterior Mode',200,500,CurrINI);
                  end;
                  %
                  % Display the graphs
                  %
                  ShockGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Posterior Mode','State Shock');
               elseif (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.etatT)==1);
                  txt = 'YADA couldn''t estimate the state shocks. Please make sure that your model has state shocks!';
                  About(txt,'information','YADA - State Shocks',120,500,CurrINI);
               end;
            elseif TransStatus==0;
               txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the state shocks based on the posterior mode estimates.';
               About(txt,'information','State Shocks - Posterior Mode',300,600,CurrINI);
            elseif BeliefStatus==0;
               txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the state shocks based on the posterior mode estimates.';
               About(txt,'information','State Shocks - Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MeasurementErrorsInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.wtT)==0);
                  if (BeliefStatus==1)&(TransStatus==1);
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save smooth and update estimates of the measurement errors for the adaptive learning version at the initial values to file?';
                     answer = Query(txt,'question',140,'Save - Measurement Errors',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        SmoothMeasurementError = StateVarStructure.wtT;
                        UpdateMeasurementError = StateVarStructure.wtt;
                        MeasurementErrorNames = StateVarStructure.wtNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                        if length(DateVector)>size(SmoothMeasurementError,2);
                           DateVector = DateVector(1,length(DateVector)-size(SmoothMeasurementError,2)+1:length(DateVector));
                        end;
                        file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrors-AdaptiveLearning-InitialValues.mat']);
                        save(file,'SmoothMeasurementError','UpdateMeasurementError','MeasurementErrorNames','DateVector');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\MEALInitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% smoothed estimates of the non-zero measurement errors\n%%\n');
                        fprintf(fid,'SmoothMeasurementError = DataStructure.SmoothMeasurementError;\n');
                        fprintf(fid,'%%\n%% update estimates of the non-zero measurement errors\n%%\n');
                        fprintf(fid,'UpdateMeasurementError = DataStructure.UpdateMeasurementError;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the non-zero measurement errors\n%%\n');
                        fprintf(fid,'MeasurementErrorNames = DataStructure.MeasurementErrorNames;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['Smooth and update estimates of the measurement errors for the adaptive learning version at the initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: SmoothMeasurementError and UpdateMeasurementError (' num2str(size(SmoothMeasurementError,1),'%0.0f') 'x' num2str(size(SmoothMeasurementError,2),'%0.0f') ' matrix with smoothed and update estimates of the non-zero measurement errors), MeasurementErrorNames (string matrix with the names of the non-zero measurement errors), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Measurement Errors - Initial Values',300,500,CurrINI);
                     end;
                     %
                     % Display the graphs
                     %
                     ShockGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Initial Values','Measurement Error');
                  elseif TransStatus==0;
                     txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the measurement errors based on the initial values estimates.';
                     About(txt,'information','Measurement Errors - Initial Values',300,600,CurrINI);
                  elseif BeliefStatus==0;
                     txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the measurement errors based on the initial values estimates.';
                     About(txt,'information','Measurement Errors - Initial Values',300,600,CurrINI);
                  end;
               elseif (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.wtT)==1);
                  txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
                  About(txt,'information','Measurement Errors - Initial Values',120,500,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the measurement errors based on the initial values.'];
                About(txt,'information','Measurement Errors - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MeasurementErrorsDSGEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.wtT)==0);
                  if (BeliefStatus==1)&(TransStatus==1);
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save smooth and update estimates of the measurement errors for the adaptive learning version at the DSGE posterior mode values to file?';
                     answer = Query(txt,'question',140,'Save - Measurement Errors',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        SmoothMeasurementError = StateVarStructure.wtT;
                        UpdateMeasurementError = StateVarStructure.wtt;
                        MeasurementErrorNames = StateVarStructure.wtNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                        if length(DateVector)>size(SmoothMeasurementError,2);
                           DateVector = DateVector(1,length(DateVector)-size(SmoothMeasurementError,2)+1:length(DateVector));
                        end;
                        file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrors-AdaptiveLearning-DSGEPosteriorMode.mat']);
                        save(file,'SmoothMeasurementError','UpdateMeasurementError','MeasurementErrorNames','DateVector');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\MEALDSGEMode.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% smoothed estimates of the non-zero measurement errors\n%%\n');
                        fprintf(fid,'SmoothMeasurementError = DataStructure.SmoothMeasurementError;\n');
                        fprintf(fid,'%%\n%% update estimates of the non-zero measurement errors\n%%\n');
                        fprintf(fid,'UpdateMeasurementError = DataStructure.UpdateMeasurementError;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the non-zero measurement errors\n%%\n');
                        fprintf(fid,'MeasurementErrorNames = DataStructure.MeasurementErrorNames;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['Smooth and update estimates of the measurement errors for the adaptive learning version at the DSGE posterior mode values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: SmoothMeasurementError and UpdateMeasurementError (' num2str(size(SmoothMeasurementError,1),'%0.0f') 'x' num2str(size(SmoothMeasurementError,2),'%0.0f') ' matrix with smoothed and update estimates of the non-zero measurement errors), MeasurementErrorNames (string matrix with the names of the non-zero measurement errors), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Measurement Errors - DSGE Posterior Mode',300,500,CurrINI);
                     end;
                     %
                     % Display the graphs
                     %
                     ShockGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'DSGE Posterior Mode','Measurement Error');
                  elseif TransStatus==0;
                     txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the measurement errors based on the DSGE posterior mode values estimates.';
                     About(txt,'information','Measurement Errors - DSGE Posterior Mode',300,600,CurrINI);
                  elseif BeliefStatus==0;
                     txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the measurement errors based on the DSGE posterior mode values estimates.';
                     About(txt,'information','Measurement Errors - DSGE Posterior Mode',300,600,CurrINI);
                  end;
               elseif (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.wtT)==1);
                  txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
                  About(txt,'information','Measurement Errors - DSGE Posterior Mode',120,500,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the measurement errors based on the DSGE model posterior mode estimates.'];
               About(txt,'information','Measurement Errors - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MeasurementErrorsPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % now we call the state variable estimation function which also deals with the log-likelihood
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,0);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.wtT)==0);
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % ask if we should save to disk
                  %
                  txt = 'Would you like to save smooth and update estimates of the measurement errors for the adaptive learning version at posterior mode to file?';
                  answer = Query(txt,'question',140,'Save - Measurement Errors',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     SmoothMeasurementError = StateVarStructure.wtT;
                     UpdateMeasurementError = StateVarStructure.wtt;
                     MeasurementErrorNames = StateVarStructure.wtNames;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     if length(DateVector)>size(SmoothMeasurementError,2);
                        DateVector = DateVector(1,length(DateVector)-size(SmoothMeasurementError,2)+1:length(DateVector));
                     end;
                     file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrors-AdaptiveLearning-PosteriorMode.mat']);
                     save(file,'SmoothMeasurementError','UpdateMeasurementError','MeasurementErrorNames','DateVector');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\MEALPostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% smoothed estimates of the non-zero measurement errors\n%%\n');
                     fprintf(fid,'SmoothMeasurementError = DataStructure.SmoothMeasurementError;\n');
                     fprintf(fid,'%%\n%% update estimates of the non-zero measurement errors\n%%\n');
                     fprintf(fid,'UpdateMeasurementError = DataStructure.UpdateMeasurementError;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the non-zero measurement errors\n%%\n');
                     fprintf(fid,'MeasurementErrorNames = DataStructure.MeasurementErrorNames;\n');
                     fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['Smooth and update estimates of the measurement errors for the adaptive learning version at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: SmoothMeasurementError and UpdateMeasurementError (' num2str(size(SmoothMeasurementError,1),'%0.0f') 'x' num2str(size(SmoothMeasurementError,2),'%0.0f') ' matrix with smoothed and update estimates of the non-zero measurement errors), MeasurementErrorNames (string matrix with the names of the non-zero measurement errors), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Measurement Errors - Posterior Mode',300,500,CurrINI);
                  end;
                  %
                  % Display the graphs
                  %
                  ShockGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Posterior Mode','Measurement Error');
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the measurement errors based on the posterior mode estimates.';
                  About(txt,'information','Measurement Errors - Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the measurement errors based on the posterior mode estimates.';
                  About(txt,'information','Measurement Errors - Posterior Mode',300,600,CurrINI);
               end;
            elseif (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.wtT)==1);
               txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
               About(txt,'information','Measurement Errors - Posterior Mode',120,500,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'BeliefCoefficientsInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're OK
                  %
                  [BeliefVariables,gjvec] = GetBeliefCoefficientNames(StateVarStructure.BeliefSys.G,StateVarStructure.BeliefSys.m,StateVarStructure.ConstPos,DSGEModel.BeliefSystem,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                  %
                  % take into account that betatt is lagged one period
                  %
                  StateVarStructure.betatt = [StateVarStructure.betatt(:,2:size(StateVarStructure.betatt,2)) StateVarStructure.LastSolution.betaTT];
                  %
                  % check if we should rewrite the betatt matrix to account for persistence and growth rate
                  %
                  if DSGEModel.BeliefSystem==1;
                     if StateVarStructure.ConstPos>0;
                        AddPos = 1;
                     else;
                        AddPos = 0;
                     end;
                     for j=1:StateVarStructure.f;
                        if j==1;
                           StartPos = 1+AddPos;
                           EndPos = 2+AddPos;
                        else;
                           StartPos = 1+AddPos+((2+AddPos)*(j-1));
                           EndPos = (2+AddPos)*j;
                        end;
                        betatj = StateVarStructure.betatt(StartPos:EndPos,:);                     %
                        % add element 1+AddPos and 2+AddPos
                        %
                        betaPersistence = betatj(1,:)+betatj(2,:);
                        betaGrowth = -betatj(2,:);
                        StateVarStructure.betatt(StartPos:EndPos,:) = [betaPersistence;betaGrowth];
                     end;
                  end;
                  %
                  % ask if we should save to disk
                  %
                  txt = 'Would you like to save the estimated belief coefficients of the adaptive learning perceived law of motion at the initial parameter values to file?';
                  answer = Query(txt,'question',140,'Save - Belief Coefficients',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     if DSGEModel.KalmanFirstObservation>1;
                        DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
                     end;
                     betatt = StateVarStructure.betatt;
                     ForwardVariableNames = DSGEModel.ForwardVariableNames;
                     file = FixFilePath([DSGEModel.OutputDirectory '\BeliefCoefficients-AdaptiveLearning-InitialValues.mat']);
                     save(file,'betatt','BeliefVariables','ForwardVariableNames','DateVector');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\BeliefCoeffALInitVal.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% matrix with the update estimates of the belief coefficients\n%%\n');
                     fprintf(fid,'betatt = DataStructure.betatt;\n');
                     fprintf(fid,'%%\n%% cell array with string matrices of the belief equation explanatory variables\n%%\n');
                     fprintf(fid,'BeliefVariables = DataStructure.BeliefVariables;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the forward looking variables\n%%\n');
                     fprintf(fid,'ForwardVariableNames = DataStructure.ForwardVariableNames;\n');
                     fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The update estimates of the belief coefficients at the initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: betatt (matrix with update estimates of the belief coefficients), BeliefVariables (cell array with string matrices with the names of the explanatory variables in each belief equation), ForwardVariableNames (string matrix with the names of the forward looking variables), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Belief Coefficients - Initial Values',200,500,CurrINI);
                  end;
                  %
                  % Display the graphs
                  %
                  BeliefCoefficientGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,BeliefVariables,gjvec,'Initial Values');
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the belief coefficients based on the initial values.';
                  About(txt,'information','Belief Coefficients - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the belief coefficients based on the initial values.';
                  About(txt,'information','Belief Coefficients - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the belief coefficients based on the initial values.'];
                About(txt,'information','Belief Coefficients - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'BeliefCoefficientsDSGEMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're OK
                  %
                  [BeliefVariables,gjvec] = GetBeliefCoefficientNames(StateVarStructure.BeliefSys.G,StateVarStructure.BeliefSys.m,StateVarStructure.ConstPos,DSGEModel.BeliefSystem,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                  %
                  % take into account that betatt is lagged one period
                  %
                  StateVarStructure.betatt = [StateVarStructure.betatt(:,2:size(StateVarStructure.betatt,2)) StateVarStructure.LastSolution.betaTT];
                  %
                  % check if we should rewrite the betatt matrix to account for persistence and growth rate
                  %
                  if DSGEModel.BeliefSystem==1;
                     if StateVarStructure.ConstPos>0;
                        AddPos = 1;
                     else;
                        AddPos = 0;
                     end;
                     for j=1:StateVarStructure.f;
                        if j==1;
                           StartPos = 1+AddPos;
                           EndPos = 2+AddPos;
                        else;
                           StartPos = 1+AddPos+((2+AddPos)*(j-1));
                           EndPos = (2+AddPos)*j;
                        end;
                        betatj = StateVarStructure.betatt(StartPos:EndPos,:);                     %
                        % add element 1+AddPos and 2+AddPos
                        %
                        betaPersistence = betatj(1,:)+betatj(2,:);
                        betaGrowth = -betatj(2,:);
                        StateVarStructure.betatt(StartPos:EndPos,:) = [betaPersistence;betaGrowth];
                     end;
                  end;
                  %
                  % ask if we should save to disk
                  %
                  txt = 'Would you like to save the estimated belief coefficients of the adaptive learning perceived law of motion at the DSGE posterior mode values to file?';
                  answer = Query(txt,'question',140,'Save - Belief Coefficients',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     if DSGEModel.KalmanFirstObservation>1;
                        DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
                     end;
                     betatt = StateVarStructure.betatt;
                     ForwardVariableNames = DSGEModel.ForwardVariableNames;
                     file = FixFilePath([DSGEModel.OutputDirectory '\BeliefCoefficients-AdaptiveLearning-DSGEPosteriorMode.mat']);
                     save(file,'betatt','BeliefVariables','ForwardVariableNames','DateVector');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\BeliefCoeffALDSGEMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% matrix with the update estimates of the belief coefficients\n%%\n');
                     fprintf(fid,'betatt = DataStructure.betatt;\n');
                     fprintf(fid,'%%\n%% cell array with string matrices of the belief equation explanatory variables\n%%\n');
                     fprintf(fid,'BeliefVariables = DataStructure.BeliefVariables;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the forward looking variables\n%%\n');
                     fprintf(fid,'ForwardVariableNames = DataStructure.ForwardVariableNames;\n');
                     fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The update estimates of the belief coefficients at the DSGE posterior mode values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: betatt (matrix with update estimates of the belief coefficients), BeliefVariables (cell array with string matrices with the names of the explanatory variables in each belief equation), ForwardVariableNames (string matrix with the names of the forward looking variables), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Belief Coefficients - DSGE Posterior Mode',200,500,CurrINI);
                  end;
                  %
                  % Display the graphs
                  %
                  BeliefCoefficientGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,BeliefVariables,gjvec,'DSGE Posterior Mode');
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the belief coefficients based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Belief Coefficients - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the belief coefficients based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Belief Coefficients - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the belief coefficients based on the DSGE model posterior mode estimates.'];
               About(txt,'information','Belief Coefficients - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'BeliefCoefficientsPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % now we call the state variable estimation function which also deals with the log-likelihood
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,0);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if (BeliefStatus==1)&(TransStatus==1);
               %
               % we're OK
               %
               [BeliefVariables,gjvec] = GetBeliefCoefficientNames(StateVarStructure.BeliefSys.G,StateVarStructure.BeliefSys.m,StateVarStructure.ConstPos,DSGEModel.BeliefSystem,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
               %
               % take into account that betatt is lagged one period
               %
               StateVarStructure.betatt = [StateVarStructure.betatt(:,2:size(StateVarStructure.betatt,2)) StateVarStructure.LastSolution.betaTT];
               %
               % check if we should rewrite the betatt matrix to account for persistence and growth rate
               %
               if DSGEModel.BeliefSystem==1;
                  if StateVarStructure.ConstPos>0;
                     AddPos = 1;
                  else;
                     AddPos = 0;
                  end;
                  for j=1:StateVarStructure.f;
                     if j==1;
                        StartPos = 1+AddPos;
                        EndPos = 2+AddPos;
                     else;
                        StartPos = 1+AddPos+((2+AddPos)*(j-1));
                        EndPos = (2+AddPos)*j;
                     end;
                     betatj = StateVarStructure.betatt(StartPos:EndPos,:);                     %
                     % add element 1+AddPos and 2+AddPos
                     %
                     betaPersistence = betatj(1,:)+betatj(2,:);
                     betaGrowth = -betatj(2,:);
                     StateVarStructure.betatt(StartPos:EndPos,:) = [betaPersistence;betaGrowth];
                  end;
               end;
               %
               % ask if we should save to disk
               %
               txt = 'Would you like to save the estimated belief coefficients of the adaptive learning perceived law of motion at posterior mode to file?';
               answer = Query(txt,'question',140,'Save - Belief Coefficients',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                  if DSGEModel.KalmanFirstObservation>1;
                     DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
                  end;
                  betatt = StateVarStructure.betatt;
                  ForwardVariableNames = DSGEModel.ForwardVariableNames;
                  file = FixFilePath([DSGEModel.OutputDirectory '\BeliefCoefficients-AdaptiveLearning-PosteriorMode.mat']);
                  save(file,'betatt','BeliefVariables','ForwardVariableNames','DateVector');
                  %
                  % create a matlab file to read the data
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\BeliefCoeffALPostMode.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% matrix with the update estimates of the belief coefficients\n%%\n');
                  fprintf(fid,'betatt = DataStructure.betatt;\n');
                  fprintf(fid,'%%\n%% cell array with string matrices of the belief equation explanatory variables\n%%\n');
                  fprintf(fid,'BeliefVariables = DataStructure.BeliefVariables;\n');
                  fprintf(fid,'%%\n%% string matrix with the names of the forward looking variables\n%%\n');
                  fprintf(fid,'ForwardVariableNames = DataStructure.ForwardVariableNames;\n');
                  fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                  fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  txt = ['The update estimates of the belief coefficients at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: betatt (matrix with update estimates of the belief coefficients), BeliefVariables (cell array with string matrices with the names of the explanatory variables in each belief equation), ForwardVariableNames (string matrix with the names of the forward looking variables), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Belief Coefficients - Posterior Mode',200,500,CurrINI);
               end;
               %
               % Display the graphs
               %
               BeliefCoefficientGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,BeliefVariables,gjvec,'Posterior Mode');
            elseif TransStatus==0;
               txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the belief coefficients based on the posterior mode estimates.';
               About(txt,'information','Belief Coefficients - Posterior Mode',300,600,CurrINI);
            elseif BeliefStatus==0;
               txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the belief coefficients based on the posterior mode estimates.';
               About(txt,'information','Belief Coefficients - Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LawsOfMotionInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % determine the laws of motion
                     %
                     ForwardVariableNames = DSGEModel.ForwardVariableNames;
                     ALMProjections = StateVarStructure.SMatrix'*StateVarStructure.Ksitt1;
                     ForwardVariableValues = StateVarStructure.SMatrix'*StateVarStructure.Ksitt;
                     PLMProjections = zeros(size(ForwardVariableValues));
                     [f,T] = size(ForwardVariableValues);
                     g = size(StateVarStructure.betatt,1);
                     for t=1:T;
                        PLMProjections(:,t) = reshape(StateVarStructure.Kgf*StateVarStructure.Gtilde*StateVarStructure.Zlag(:,t),f,g)*StateVarStructure.betatt(:,t);
                     end;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save estimates of the forward variables and the two laws of motion at the initial values to file?';
                     answer = Query(txt,'question',140,'Save - Laws Of Motion',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\LawsOfMotion-AdaptiveLearning-InitialValues.mat']);
                        save(file,'ForwardVariableNames','ForwardVariableValues','ALMProjections','PLMProjections','DateVector');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\BeliefCoeffALInitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the forward looking variables\n%%\n');
                        fprintf(fid,'ForwardVariableNames = DataStructure.ForwardVariableNames;\n');
                        fprintf(fid,'%%\n%% \n%%\n');
                        fprintf(fid,'ForwardVariableValues = DataStructure.ForwardVariableValues;\n');
                        fprintf(fid,'%%\n%% \n%%\n');
                        fprintf(fid,'ALMProjections = DataStructure.ALMProjections;\n');
                        fprintf(fid,'%%\n%% \n%%\n');
                        fprintf(fid,'PLMProjections = DataStructure.PLMProjections;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The actual and perceived laws of motion and the update estimates of the forward looking variables at the DSGE posterior mode values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: ForwardVariableNames (string matrix with the f names of the forward looking variables), ForwardVariableValues (matrix of dimension f x T with the Kalman update estimates of the forward looking variable, t|t), ALMProjections (matrix of dimension f x T with the actual law of motion, represented by the Kalman forecast estimates of the forward looking variables, t|t-1), PLMProjections (matrix of dimension f x T with the perceived law of motion of the forward looking variables), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Laws Of Motion - Initial Values',400,600,CurrINI);
                     end;
                     %
                     % display the results
                     %
                     LawsOfMotionGraphDLG('init',CurrINI,DSGEModel,ForwardVariableValues,ALMProjections,PLMProjections,DateVector,'Initial Values');
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the laws of motion based on the initial values.';
                  About(txt,'information','Laws Of Motion - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the laws of motion based on the initial values.';
                  About(txt,'information','Laws Of Motion - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the laws of motion based on the initial values.'];
                About(txt,'information','Laws Of Motion - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LawsOfMotionDSGEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % determine the laws of motion
                     %
                     ForwardVariableNames = DSGEModel.ForwardVariableNames;
                     ALMProjections = StateVarStructure.SMatrix'*StateVarStructure.Ksitt1;
                     ForwardVariableValues = StateVarStructure.SMatrix'*StateVarStructure.Ksitt;
                     PLMProjections = zeros(size(ForwardVariableValues));
                     [f,T] = size(ForwardVariableValues);
                     g = size(StateVarStructure.betatt,1);
                     for t=1:T;
                        PLMProjections(:,t) = reshape(StateVarStructure.Kgf*StateVarStructure.Gtilde*StateVarStructure.Zlag(:,t),f,g)*StateVarStructure.betatt(:,t);
                     end;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save estimates of the forward variables and the two laws of motion at the DSGE posterior mode values to file?';
                     answer = Query(txt,'question',140,'Save - Laws Of Motion',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\LawsOfMotion-AdaptiveLearning-DSGEPosteriorMode.mat']);
                        save(file,'ForwardVariableNames','ForwardVariableValues','ALMProjections','PLMProjections','DateVector');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\BeliefCoeffALDSGEMode.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the forward looking variables\n%%\n');
                        fprintf(fid,'ForwardVariableNames = DataStructure.ForwardVariableNames;\n');
                        fprintf(fid,'%%\n%% \n%%\n');
                        fprintf(fid,'ForwardVariableValues = DataStructure.ForwardVariableValues;\n');
                        fprintf(fid,'%%\n%% \n%%\n');
                        fprintf(fid,'ALMProjections = DataStructure.ALMProjections;\n');
                        fprintf(fid,'%%\n%% \n%%\n');
                        fprintf(fid,'PLMProjections = DataStructure.PLMProjections;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The actual and perceived laws of motion and the update estimates of the forward looking variables at the DSGE posterior mode values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: ForwardVariableNames (string matrix with the f names of the forward looking variables), ForwardVariableValues (matrix of dimension f x T with the Kalman update estimates of the forward looking variable, t|t), ALMProjections (matrix of dimension f x T with the actual law of motion, represented by the Kalman forecast estimates of the forward looking variables, t|t-1), PLMProjections (matrix of dimension f x T with the perceived law of motion of the forward looking variables), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Laws Of Motion - DSGE Posterior Mode',400,600,CurrINI);
                     end;
                     %
                     % display the results
                     %
                     LawsOfMotionGraphDLG('init',CurrINI,DSGEModel,ForwardVariableValues,ALMProjections,PLMProjections,DateVector,'DSGE Posterior Mode');
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the laws of motion based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Laws Of Motion - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the laws of motion based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Laws Of Motion - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the laws of motion based on the DSGE model posterior mode estimates.'];
               About(txt,'information','Laws Of Motion - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LawsOfMotionPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % now we call the state variable estimation function which also deals with the log-likelihood
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,0);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if (BeliefStatus==1)&(TransStatus==1);
               %
               % we're OK
               %
               if isempty(StateVarStructure)==0;
                  %
                  % determine the laws of motion
                  %
                  ForwardVariableNames = DSGEModel.ForwardVariableNames;
                  ALMProjections = StateVarStructure.SMatrix'*StateVarStructure.Ksitt1;
                  ForwardVariableValues = StateVarStructure.SMatrix'*StateVarStructure.Ksitt;
                  PLMProjections = zeros(size(ForwardVariableValues));
                  [f,T] = size(ForwardVariableValues);
                  g = size(StateVarStructure.betatt,1);
                  for t=1:T;
                     PLMProjections(:,t) = reshape(StateVarStructure.Kgf*StateVarStructure.Gtilde*StateVarStructure.Zlag(:,t),f,g)*StateVarStructure.betatt(:,t);
                  end;
                  DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                  DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
                  %
                  % ask if we should save to disk
                  %
                  txt = 'Would you like to save estimates of the forward variables and the two laws of motion at posterior mode to file?';
                  answer = Query(txt,'question',140,'Save - Laws Of Motion',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\LawsOfMotion-AdaptiveLearning-PosteriorMode.mat']);
                     save(file,'ForwardVariableNames','ForwardVariableValues','ALMProjections','PLMProjections','DateVector');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\BeliefCoeffALPostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the forward looking variables\n%%\n');
                     fprintf(fid,'ForwardVariableNames = DataStructure.ForwardVariableNames;\n');
                     fprintf(fid,'%%\n%% \n%%\n');
                     fprintf(fid,'ForwardVariableValues = DataStructure.ForwardVariableValues;\n');
                     fprintf(fid,'%%\n%% \n%%\n');
                     fprintf(fid,'ALMProjections = DataStructure.ALMProjections;\n');
                     fprintf(fid,'%%\n%% \n%%\n');
                     fprintf(fid,'PLMProjections = DataStructure.PLMProjections;\n');
                     fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The actual and perceived laws of motion and the update estimates of the forward looking variables at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: ForwardVariableNames (string matrix with the f names of the forward looking variables), ForwardVariableValues (matrix of dimension f x T with the Kalman update estimates of the forward looking variable, t|t), ALMProjections (matrix of dimension f x T with the actual law of motion, represented by the Kalman forecast estimates of the forward looking variables, t|t-1), PLMProjections (matrix of dimension f x T with the perceived law of motion of the forward looking variables), and DateVector (vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Laws Of Motion - Posterior Mode',400,600,CurrINI);
                  end;
                  %
                  % display the results
                  %
                  LawsOfMotionGraphDLG('init',CurrINI,DSGEModel,ForwardVariableValues,ALMProjections,PLMProjections,DateVector,'Posterior Mode');
               end;
            elseif TransStatus==0;
               txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the laws of motion based on the posterior mode estimates.';
               About(txt,'information','Laws Of Motion - Posterior Mode',300,600,CurrINI);
            elseif BeliefStatus==0;
               txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the laws of motion based on the posterior mode estimates.';
               About(txt,'information','Laws Of Motion - Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'DSGEEigenvaluesInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % get the AiM data
               %
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  set(controls.about,'UserData',0);
                  return;
               end;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % determine the eigenvalues
                     %
                     T = size(StateVarStructure.Ftilde,3);
                     Feig = flipud(sort(abs(eig(StateVarStructure.F))));
                     Fsseig = flipud(sort(abs(eig(StateVarStructure.Fss))));
                     r = size(StateVarStructure.F,1);
                     Fteig = zeros(r,T);
                     for t=1:T;
                        Fteig(:,t) = flipud(sort(abs(eig(StateVarStructure.Ftilde(:,:,t)))));
                     end;
                     Fteig = max(Fteig,[],2);
                     %
                     % write to text file
                     %
                     EigenValueFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-Eigenvalues-Adaptive-Learning-' DSGEModel.NameOfModel '-InitialValues.txt']);
                     fid = fopen(EigenValueFile,'wt');
                     %
                     % Some generic model information
                     %
                     txt = strvcat(' ',['AiM Model File:                            ' GetFilename(DSGEModel.AIMFile)],' ');
                     if DSGEModel.ModelSolver==1;
                        txt = strvcat(txt,['Number of equations:                       ' int2str(AIMData.neq)]);
                     else;
                        txt = strvcat(txt,['Number of equations:                       ' int2str(length(DSGEModel.StateVariablePositions))]);
                     end;
                     txt = strvcat(txt,['Number of lags:                            ' int2str(AIMData.nlag)]);
                     txt = strvcat(txt,['Number of leads:                           ' int2str(AIMData.nlead)]);
                     txt = strvcat(txt,['Number of parameters to estimate:          ' int2str(length(theta))]);
                     txt = strvcat(txt,['Total number of parameters:                ' int2str(size(ParameterNames.estimated,1)+size(ParameterNames.calibrated,1)+size(ParameterNames.additional,1))],' ');
                     %
                     % and now the eigenvalues
                     %
                     txt = strvcat(txt,'=================================================================================');
                     txt = strvcat(txt,'Number   RE solution   AL steady-state solution   AL time-varying solution (max)');
                     txt = strvcat(txt,'---------------------------------------------------------------------------------');
                     %
                     PrtVal = ['%' int2str(3+CurrINI.decimals) '.' int2str(CurrINI.decimals) 'f'];
                     PrtStr = [' %3.0f      ' PrtVal '           ' PrtVal '                     ' PrtVal ' '];
                     for i=1:r;
                        txt = strvcat(txt,sprintf(PrtStr,i,Feig(i),Fsseig(i),Fteig(i)));
                     end;
                     %
                     for i=1:size(txt,1);
                        fprintf(fid,'%s\n',txt(i,:));
                     end;
                     fprintf(fid,'================================================================================\n\n');
                     %
                     % bottom file data
                     %
                     fprintf(fid,'Directory for file: %s\n',GetPath(EigenValueFile));
                     fprintf(fid,'Name of file:       %s\n',GetFilename(EigenValueFile));
                     fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                     fclose(fid);
                     %
                     % Display information
                     %
                     if FileExist(EigenValueFile)==1;
                        TextGUI(EigenValueFile,['DSGE Model Solution Eigenvalues: Adaptive Learning - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-750)/2 32 750 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the DSGE model solution eigenvalues based on the initial values.';
                  About(txt,'information','DSGE Model Solution Eigenvalues - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the DSGE model solution eigenvalues based on the initial values.';
                  About(txt,'information','DSGE Model Solution Eigenvalues - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the DSGE model solution eigenvalues based on the initial values.'];
                About(txt,'information','DSGE Model Solution Eigenvalues - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'DSGEEigenvaluesDSGEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % get the AiM data
                  %
                  if MatlabNumber>=8.0;
                     try;
                        AIMData = load(DSGEModel.AIMDataFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        AIMData = load(DSGEModel.AIMDataFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==0;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                     set(controls.about,'UserData',0);
                     return;
                  end;
                  %
                  % we're OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % determine the eigenvalues
                     %
                     T = size(StateVarStructure.Ftilde,3);
                     Feig = flipud(sort(abs(eig(StateVarStructure.F))));
                     Fsseig = flipud(sort(abs(eig(StateVarStructure.Fss))));
                     r = size(StateVarStructure.F,1);
                     Fteig = zeros(r,T);
                     for t=1:T;
                        Fteig(:,t) = flipud(sort(abs(eig(StateVarStructure.Ftilde(:,:,t)))));
                     end;
                     Fteig = max(Fteig,[],2);
                     %
                     % write to text file
                     %
                     EigenValueFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-Eigenvalues-Adaptive-Learning-' DSGEModel.NameOfModel '-DSGEPostMode.txt']);
                     fid = fopen(EigenValueFile,'wt');
                     %
                     % Some generic model information
                     %
                     txt = strvcat(' ',['AiM Model File:                            ' GetFilename(DSGEModel.AIMFile)],' ');
                     if DSGEModel.ModelSolver==1;
                        txt = strvcat(txt,['Number of equations:                       ' int2str(AIMData.neq)]);
                     else;
                        txt = strvcat(txt,['Number of equations:                       ' int2str(length(DSGEModel.StateVariablePositions))]);
                     end;
                     txt = strvcat(txt,['Number of lags:                            ' int2str(AIMData.nlag)]);
                     txt = strvcat(txt,['Number of leads:                           ' int2str(AIMData.nlead)]);
                     txt = strvcat(txt,['Number of parameters to estimate:          ' int2str(length(theta))]);
                     txt = strvcat(txt,['Total number of parameters:                ' int2str(size(ParameterNames.estimated,1)+size(ParameterNames.calibrated,1)+size(ParameterNames.additional,1))],' ');
                     %
                     % and now the eigenvalues
                     %
                     txt = strvcat(txt,'=================================================================================');
                     txt = strvcat(txt,'Number   RE solution   AL steady-state solution   AL time-varying solution (max)');
                     txt = strvcat(txt,'---------------------------------------------------------------------------------');
                     %
                     PrtVal = ['%' int2str(3+CurrINI.decimals) '.' int2str(CurrINI.decimals) 'f'];
                     PrtStr = [' %3.0f      ' PrtVal '           ' PrtVal '                     ' PrtVal ' '];
                     for i=1:r;
                        txt = strvcat(txt,sprintf(PrtStr,i,Feig(i),Fsseig(i),Fteig(i)));
                     end;
                     %
                     for i=1:size(txt,1);
                        fprintf(fid,'%s\n',txt(i,:));
                     end;
                     fprintf(fid,'================================================================================\n\n');
                     %
                     % bottom file data
                     %
                     fprintf(fid,'Directory for file: %s\n',GetPath(EigenValueFile));
                     fprintf(fid,'Name of file:       %s\n',GetFilename(EigenValueFile));
                     fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                     fclose(fid);
                     %
                     % Display information
                     %
                     if FileExist(EigenValueFile)==1;
                        TextGUI(EigenValueFile,['DSGE Model Solution Eigenvalues: Adaptive Learning - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-750)/2 32 750 CurrINI.scrsz(4)-100],'DSGE Posterior Mode Parameter Values',50,CurrINI,1,1);
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the DSGE model solution eigenvalues based on the DSGE posterior mode estimates.';
                  About(txt,'information','DSGE Model Solution Eigenvalues - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the DSGE model solution eigenvalues based on the DSGE posterior mode estimates.';
                  About(txt,'information','DSGE Model Solution Eigenvalues - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the DSGE model solution eigenvalues based on the DSGE posterior mode estimates.'];
               About(txt,'information','DSGE Model Solution Eigenvalues - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'DSGEEigenvaluesPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % get the AiM data
            %
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
            if isempty(ErrorStr)==0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               set(controls.about,'UserData',0);
               return;
            end;
            %
            % now we call the state variable estimation function which also deals with the log-likelihood
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,0);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if (BeliefStatus==1)&(TransStatus==1);
               %
               % we're OK
               %
               if isempty(StateVarStructure)==0;
                  %
                  % determine the eigenvalues
                  %
                  T = size(StateVarStructure.Ftilde,3);
                  Feig = flipud(sort(abs(eig(StateVarStructure.F))));
                  Fsseig = flipud(sort(abs(eig(StateVarStructure.Fss))));
                  r = size(StateVarStructure.F,1);
                  Fteig = zeros(r,T);
                  for t=1:T;
                     Fteig(:,t) = flipud(sort(abs(eig(StateVarStructure.Ftilde(:,:,t)))));
                  end;
                  Fteig = max(Fteig,[],2);
                  %
                  % write to text file
                  %
                  EigenValueFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-Eigenvalues-Adaptive-Learning-' DSGEModel.NameOfModel '-PostMode.txt']);
                  fid = fopen(EigenValueFile,'wt');
                  %
                  % Some generic model information
                  %
                  txt = strvcat(' ',['AiM Model File:                            ' GetFilename(DSGEModel.AIMFile)],' ');
                  if DSGEModel.ModelSolver==1;
                     txt = strvcat(txt,['Number of equations:                       ' int2str(AIMData.neq)]);
                  else;
                     txt = strvcat(txt,['Number of equations:                       ' int2str(length(DSGEModel.StateVariablePositions))]);
                  end;
                  txt = strvcat(txt,['Number of lags:                            ' int2str(AIMData.nlag)]);
                  txt = strvcat(txt,['Number of leads:                           ' int2str(AIMData.nlead)]);
                  txt = strvcat(txt,['Number of parameters to estimate:          ' int2str(length(ModeData.theta))]);
                  txt = strvcat(txt,['Total number of parameters:                ' int2str(size(ModeData.ParameterNames.estimated,1)+size(ModeData.ParameterNames.calibrated,1)+size(ModeData.ParameterNames.additional,1))],' ');
                  %
                  % and now the eigenvalues
                  %
                  txt = strvcat(txt,'=================================================================================');
                  txt = strvcat(txt,'Number   RE solution   AL steady-state solution   AL time-varying solution (max)');
                  txt = strvcat(txt,'---------------------------------------------------------------------------------');
                  %
                  PrtVal = ['%' int2str(3+CurrINI.decimals) '.' int2str(CurrINI.decimals) 'f'];
                  PrtStr = [' %3.0f      ' PrtVal '           ' PrtVal '                     ' PrtVal ' '];
                  for i=1:r;
                     txt = strvcat(txt,sprintf(PrtStr,i,Feig(i),Fsseig(i),Fteig(i)));
                  end;
                  %
                  for i=1:size(txt,1);
                     fprintf(fid,'%s\n',txt(i,:));
                  end;
                  fprintf(fid,'================================================================================\n\n');
                  %
                  % bottom file data
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(EigenValueFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(EigenValueFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  % Display information
                  %
                  if FileExist(EigenValueFile)==1;
                     TextGUI(EigenValueFile,['DSGE Model Solution Eigenvalues: Adaptive Learning - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-750)/2 32 750 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               end;
            elseif TransStatus==0;
               txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the DSGE model solution eigenvalues based on the posterior mode estimates.';
               About(txt,'information','DSGE Model Solution Eigenvalues - Posterior Mode',300,600,CurrINI);
            elseif BeliefStatus==0;
               txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the DSGE model solution eigenvalues based on the posterior mode estimates.';
               About(txt,'information','DSGE Model Solution Eigenvalues - Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PersistenceInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % determine the persistence data
                     %
                     T = size(StateVarStructure.Ftilde,3);
                     Feig = sum(abs(eig(StateVarStructure.F)));
                     Fsseig = sum(abs(eig(StateVarStructure.Fss)));
                     Fteig = zeros(1,T);
                     for t=1:T;
                        Fteig(t) = sum(abs(eig(StateVarStructure.Ftilde(:,:,t))));
                     end;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
                     FigHandle = figure('Visible','off', ...
                                        'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                                        'Tag','PersistenceInitialValues', ...
                                        'Name','Persistence - Initial Values');
                     %
                     SetFigureProperties(FigHandle,CurrINI);
                     %
                     % plot the RE based max eigenvalue
                     %
                     FeigHandle = plot([floor(DateVector(1)) ceil(DateVector(T))],Feig*ones(1,2),'-','LineWidth',1,'Color',[0.4 0.4 0.4]);
                     hold('on');
                     %
                     FsseigHandle = plot([floor(DateVector(1)) ceil(DateVector(T))],Fsseig*ones(1,2),'-','LineWidth',1,'Color',[0.7 0.7 0.7]);
                     %
                     FteigHandle = plot(DateVector,Fteig,'-.','LineWidth',2,'Color',[0.8 0 0]);
                     %
                     set(gca,'XLim',[floor(DateVector(1)) ceil(DateVector(T))],'YLim',[min([Feig Fsseig Fteig])-0.1 max([Feig Fsseig Fteig])+0.1]);
                     %
                     SetAxesFonts(CurrINI,'Title','Accumulated modulus of the eigenvalues of state transition matrices at the initial values');
                     %
                     % add the legend
                     %
                     if MatlabNumber>=7;
                        legHandle = legend([FteigHandle FsseigHandle FeigHandle],strvcat('Adaptive learning','Steady-state AL','Rational expectations'),'Location','SouthWest');
                     else;
                        legHandle = legend([FteigHandle FsseigHandle FeigHandle],strvcat('Adaptive learning','Steady-state AL','Rational expectations'),0);
                     end;
                     SetAxesFonts(CurrINI,legHandle);
                     %
                     SetXTickIntegers(gca);
                     %
                     % display figures
                     %
                     set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
                     drawnow;
                     pause(0.02);
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the persistence (sum of the modulus of the eigenvalues of the Ftilde matrices) based on the initial values.';
                  About(txt,'information','Persistence - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the persistence (sum of the modulus of the eigenvalues of the Ftilde matrices) based on the initial values.';
                  About(txt,'information','Persistence - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the persistence (sum of the modulus of the eigenvalues of the Ftilde matrices) based on the initial values.'];
                About(txt,'information','Persistence - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PersistenceDSGEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % determine the persistence data
                     %
                     T = size(StateVarStructure.Ftilde,3);
                     Feig = sum(abs(eig(StateVarStructure.F)));
                     Fsseig = sum(abs(eig(StateVarStructure.Fss)));
                     Fteig = zeros(1,T);
                     for t=1:T;
                        Fteig(t) = sum(abs(eig(StateVarStructure.Ftilde(:,:,t))));
                     end;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
                     FigHandle = figure('Visible','off', ...
                                        'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                                        'Tag','PersistenceDSGEPostMode', ...
                                        'Name','Persistence - DSGE Posterior Mode');
                     %
                     SetFigureProperties(FigHandle,CurrINI);
                     %
                     % plot the RE based max eigenvalue
                     %
                     FeigHandle = plot([floor(DateVector(1)) ceil(DateVector(T))],Feig*ones(1,2),'-','LineWidth',1,'Color',[0.4 0.4 0.4]);
                     hold('on');
                     %
                     FsseigHandle = plot([floor(DateVector(1)) ceil(DateVector(T))],Fsseig*ones(1,2),'-','LineWidth',1,'Color',[0.7 0.7 0.7]);
                     %
                     FteigHandle = plot(DateVector,Fteig,'-.','LineWidth',2,'Color',[0.8 0 0]);
                     %
                     set(gca,'XLim',[floor(DateVector(1)) ceil(DateVector(T))],'YLim',[min([Feig Fsseig Fteig])-0.1 max([Feig Fsseig Fteig])+0.1]);
                     %
                     SetAxesFonts(CurrINI,'Title','Accumulated modulus of the eigenvalues of state transition matrices at the DSGE posterior mode');
                     %
                     % add the legend
                     %
                     if MatlabNumber>=7;
                        legHandle = legend([FteigHandle FsseigHandle FeigHandle],strvcat('Adaptive learning','Steady-state AL','Rational expectations'),'Location','SouthWest');
                     else;
                        legHandle = legend([FteigHandle FsseigHandle FeigHandle],strvcat('Adaptive learning','Steady-state AL','Rational expectations'),0);
                     end;
                     SetAxesFonts(CurrINI,legHandle);
                     %
                     SetXTickIntegers(gca);
                     %
                     % display figures
                     %
                     set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
                     drawnow;
                     pause(0.02);
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the persistence (sum of the modulus of the eigenvalues of the Ftilde matrices) based on the DSGE posterior mode estimates.';
                  About(txt,'information','Persistence - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the persistence (sum of the modulus of the eigenvalues of the Ftilde matrices) based on the DSGE posterior mode estimates.';
                  About(txt,'information','Persistence - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA was therefore unable to estimate the persistence (sum of the modulus of the eigenvalues of the Ftilde matrices) based on the DSGE posterior mode estimates.'];
               About(txt,'information','Persistence - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PersistencePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % now we call the state variable estimation function which also deals with the log-likelihood
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,0);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if (BeliefStatus==1)&(TransStatus==1);
               %
               % we're OK
               %
               if isempty(StateVarStructure)==0;
                  %
                  % determine the persistence data
                  %
                  T = size(StateVarStructure.Ftilde,3);
                  Feig = sum(abs(eig(StateVarStructure.F)));
                  Fsseig = sum(abs(eig(StateVarStructure.Fss)));
                  Fteig = zeros(1,T);
                  for t=1:T;
                     Fteig(t) = sum(abs(eig(StateVarStructure.Ftilde(:,:,t))));
                  end;
                  DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                  DateVector = DateVector(1,DSGEModel.KalmanFirstObservation:length(DateVector));
                  FigHandle = figure('Visible','off', ...
                                     'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                                     'Tag','PersistencePostMode', ...
                                     'Name','Persistence - Posterior Mode');
                  %
                  SetFigureProperties(FigHandle,CurrINI);
                  %
                  % plot the RE based max eigenvalue
                  %
                  FeigHandle = plot([floor(DateVector(1)) ceil(DateVector(T))],Feig*ones(1,2),'-','LineWidth',1,'Color',[0.4 0.4 0.4]);
                  hold('on');
                  %
                  FsseigHandle = plot([floor(DateVector(1)) ceil(DateVector(T))],Fsseig*ones(1,2),'-','LineWidth',1,'Color',[0.7 0.7 0.7]);
                  %
                  FteigHandle = plot(DateVector,Fteig,'-.','LineWidth',2,'Color',[0.8 0 0]);
                  %
                  set(gca,'XLim',[floor(DateVector(1)) ceil(DateVector(T))],'YLim',[min([Feig Fsseig Fteig])-0.1 max([Feig Fsseig Fteig])+0.1]);
                  %
                  SetAxesFonts(CurrINI,'Title','Accumulated modulus of the eigenvalues of state transition matrices at posterior mode');
                  %
                  % add the legend
                  %
                  if MatlabNumber>=7;
                     legHandle = legend([FteigHandle FsseigHandle FeigHandle],strvcat('Adaptive learning','Steady-state AL','Rational expectations'),'Location','SouthWest');
                  else;
                     legHandle = legend([FteigHandle FsseigHandle FeigHandle],strvcat('Adaptive learning','Steady-state AL','Rational expectations'),0);
                  end;
                  SetAxesFonts(CurrINI,legHandle);
                  %
                  SetXTickIntegers(gca);
                  %
                  % display figures
                  %
                  set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
                  drawnow;
                  pause(0.02);
               end;
            elseif TransStatus==0;
               txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the persistence (sum of the modulus of the eigenvalues of the Ftilde matrices) based on the posterior mode estimates.';
               About(txt,'information','Persistence - Posterior Mode',300,600,CurrINI);
            elseif BeliefStatus==0;
               txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the persistence (sum of the modulus of the eigenvalues of the Ftilde matrices) based on the posterior mode estimates.';
               About(txt,'information','Persistence - Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LogLikelihoodInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're may be OK
                  %
                  if (isempty(StateVarStructure)==0)&(isfield(StateVarStructure,'lnLt')==1);
                     if (isempty(StateVarStructure.lnLt)==0)&(sum(isinf(StateVarStructure.lnLt))==0);
                        if StateVarStructure.nonsingular==1;
                           GraphLogLikelihood(StateVarStructure.lnLt,StateVarStructure.KalmanTime,DSGEModel,CurrINI,'Adaptive Learning: ','Initial Values');
                        else;
                           txt = 'The model has fewer structural shocks and measurement errors than observed variables. The log-likelihood function can therefore not be computed.';
                           About(txt,'information','Log-Likelihood Function - Initial Values',300,500,CurrINI);
                        end;
                     elseif sum(isinf(StateVarStructure.lnLt))>0;
                        txt = ['There are ' int2str(sum(isinf(StateVarStructure.lnLt))) ' infinite values when computing the log-likelihood function. The most likely reason is that the 1-step ahead forecast error covariance matrix of the observed variables is singular.'];
                        About(txt,'information','Log-Likelihood Function - Initial Values',300,500,CurrINI);
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the log-likelihood calculation based on the initial values.';
                  About(txt,'information','Log-likelihood Function - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the log-likelihood calculation based on the initial values.';
                  About(txt,'information','Log-likelihood Function - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the log-likelihood calculation based on the initial values.'];
                About(txt,'information','Log-likelihood Function - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LogLikelihoodDSGEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're may be OK
                  %
                  if (isempty(StateVarStructure)==0)&(isfield(StateVarStructure,'lnLt')==1);
                     if (isempty(StateVarStructure.lnLt)==0)&(sum(isinf(StateVarStructure.lnLt))==0);
                        if StateVarStructure.nonsingular==1;
                           GraphLogLikelihood(StateVarStructure.lnLt,StateVarStructure.KalmanTime,DSGEModel,CurrINI,'Adaptive Learning: ','DSGE Posterior Mode');
                        else;
                           txt = 'The model has fewer structural shocks and measurement errors than observed variables. The log-likelihood function can therefore not be computed.';
                           About(txt,'information','Log-Likelihood Function - DSGE Posterior Mode',300,500,CurrINI);
                        end;
                     elseif sum(isinf(StateVarStructure.lnLt))>0;
                        txt = ['There are ' int2str(sum(isinf(StateVarStructure.lnLt))) ' infinite values when computing the log-likelihood function. The most likely reason is that the 1-step ahead forecast error covariance matrix of the observed variables is singular.'];
                        About(txt,'information','Log-Likelihood Function - DSGE Posterior Mode',300,500,CurrINI);
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the log-likelihood calculation based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Log-likelihood Function - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the log-likelihood calculation based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Log-likelihood Function - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the log-likelihood calculation based on the DSGE model posterior mode estimates.'];
               About(txt,'information','Log-likelihood Function - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'LogLikelihoodPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % now we call the state variable estimation function which also deals with the log-likelihood
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,0);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if (BeliefStatus==1)&(TransStatus==1);
               %
               % we're may be OK
               %
               if (isempty(StateVarStructure)==0)&(isfield(StateVarStructure,'lnLt')==1);
                  if (isempty(StateVarStructure.lnLt)==0)&(sum(isinf(StateVarStructure.lnLt))==0);
                     if StateVarStructure.nonsingular==1;
                        GraphLogLikelihood(StateVarStructure.lnLt,StateVarStructure.KalmanTime,DSGEModel,CurrINI,'Adaptive Learning: ','Posterior Mode');
                     else;
                        txt = 'The model has fewer structural shocks and measurement errors than observed variables. The log-likelihood function can therefore not be computed.';
                        About(txt,'information','Log-Likelihood Function - Posterior Mode',300,500,CurrINI);
                     end;
                  elseif sum(isinf(StateVarStructure.lnLt))>0;
                     txt = ['There are ' int2str(sum(isinf(StateVarStructure.lnLt))) ' infinite values when computing the log-likelihood function. The most likely reason is that the 1-step ahead forecast error covariance matrix of the observed variables is singular.'];
                     About(txt,'information','Log-Likelihood Function - Posterior Mode',300,500,CurrINI);
                  end;
               end;
            elseif TransStatus==0;
               txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the log-likelihood calculation based on the posterior mode estimates.';
               About(txt,'information','Log-likelihood Function - Posterior Mode',300,600,CurrINI);
            elseif BeliefStatus==0;
               txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the log-likelihood calculation based on the posterior mode estimates.';
               About(txt,'information','Log-likelihood Function - Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredLikePriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Predictive Likelihood - Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % load the data
               %
               if FileExist(DSGEModel.SystemPriorFile)==0;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               else;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               end;
               PriorData = load(PriorDrawsFile);
               thetaDraws = PriorData.thetaDraws;
               NumPriorDraws = size(thetaDraws,2);
               %
               % check if we have a system prior and if so, use a subset of the prior draws
               %
               OldDSGEModel = DSGEModel;
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  if DSGEModel.PosteriorSampler>7;
                     [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','Predictive Likelihood - Prior Distribution','','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PostDrawsPercentValue = SelectedOption;
                     end;
                  else;
                     action = 'ok';
                  end;
                  if strcmp(lower(action),'ok')==1;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                     if length(UseDraws)<NumPriorDraws;
                        thetaDraws = thetaDraws(:,UseDraws);
                        NumPriorDraws = length(UseDraws);
                     end;
                  end;
               end;
               %
               % initialize the random number generator
               %
               if DSGEModel.RandomNumberValue==1;
                  randn('state',0);
                  rand('state',0);
               else;
                  randn('state',sum(100*clock));
                  rand('state',sum(100*clock));
               end;
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               %
               % check how many time periods to use for predictive distributions
               %
               MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
               %
               % check the maximum when we take NaNs into account
               %
               YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
               BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
               if isempty(DSGEModel.Actuals)==1;
                  SelectedYData = 1;
                  DataAction = 'ok';
                  YDataStr = 'Actuals from original sample';
               else;
                  YDataStr = 'Actuals from original sample';
                  m = length(DSGEModel.Actuals);
                  for i=1:m;
                     YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                  end;
                  [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - Prior Distribution','','',CurrINI);
                  if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                     YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                  end;
               end;
               %
               if strcmp(lower(DataAction),'ok')==1;
                  MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                  [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                  MaxFH = min(MaxFH,MaxPH);
                  YData = YData(:,1:MaxFH);
                  %
                  % matrix with exogenous variables
                  %
                  X = [];
                  if DSGEModel.k>0;
                     if NumPredX<MaxFH;
                        X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                     end;
                     if NumPredX>0;
                        X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                     end;
                  end;
                  %
                  % selection of variable 
                  %
                  if isempty(DSGEModel.PredictionVariables)==1;
                     PredictionVariables = ones(1,DSGEModel.n);
                     PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                  else;
                     PredictionVariables = DSGEModel.PredictionVariables;
                  end;
                  %
                  [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                  if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                     %
                     % we may continue and store the new selection of prediction variables
                     %
                     DSGEModel.PredictionVariables = PredictionVariables;
                     %
                     % check if we should predict the original data or annualized data
                     %
                     if gcbo==controls.learningtools.predictivedist.unconditional.original.height.prior;
                        IsOriginal = 1;
                        DataStr = 'Original';
                     elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.height.prior;
                        IsOriginal = 0;
                        DataStr = 'Annualized';
                     end;
                     %
                     % check if the user wants to compute the marginal or the joint predictive likelihood
                     %
                     txt = 'Would you like to compute the marginal predictive likelikehood or the joint predictive likelihood?';
                     answer = Query(txt,'question',400,'Predictive Likelihood - Prior Distribution',500,'yes',CurrINI,{'Marginal' 'Joint'});
                     if strcmp(lower(answer),'marginal')==1;
                        IsMarginal = 1;
                        MargStr = 'Marginal';
                     else;
                        IsMarginal = 0;
                        MargStr = 'Joint';
                     end;
                     if ALPredictiveLikelihoodExist(DSGEModel,SelectedYData,IsOriginal,IsMarginal,0,MaxFH,NumPriorDraws,0,NumPriorDraws,[])==1;
                        txt = ['YADA has located ' lower(MargStr) ' log posterior calculations for the ' lower(DataStr) ' data for the predictive likelihood on disk. Would you like to use these results?'];
                        answer = Query(txt,'question',200,[MargStr ' Predictive Likelihood - Prior Distribution'],500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     tic;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % compute the predictive density height
                        %
                        DoneCalc = AdaptiveLearningDSGEPredictiveLikelihood(thetaDraws',theta,thetaPositions,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds,ModelParameters,YData,X,BackYData,SelectedYData,IsOriginal,0,IsMarginal,FirstPeriod,LastPeriod,NumPriorDraws,0,[],DSGEModel,CurrINI);
                     end;
                     if DoneCalc==1;
                        %
                        % we can now do the remaining computations.
                        %
                        [LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd,LogPredLikeBackNow,StdErrBackNow,MinMaxCondLikeBackNow,TestStdBackNow] = AdaptiveLearningPredLikeMCI(SelectedYData,NumPriorDraws,MaxFH,IsOriginal,IsMarginal,0,[],0,NumPriorDraws,[],DSGEModel);
                        CompTime = toc;
                        if isempty(LogPredLike)==0;
                           %
                           % ask the user if results should be saved to file
                           %
                           SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                           SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                           %
                           PredictionSampleStrMat = CreateFullSample(DSGEModel);
                           HistSampleStrMat = PredictionSampleStrMat(LastPeriod-1:LastPeriod,:);
                           PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                           txt = ['Would you like to save the prior draws results for the ' lower(DataStr) ' predictions under adaptive learning to disk?'];
                           answer = Query(txt,'question',200,['Save - ' MargStr ' Predictive Likelihood'],500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              YNames = DSGEModel.VariableNames;
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGE' MargStr 'AdaptiveLearningPriorPredictiveLikelihood-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataStr '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' int2str(NumPriorDraws) '.mat']);
                              if (IsOriginal==1)&(IsMarginal==1);
                                 save(file,'LogPredLike','StdErr','LogPredLikeBackNow','StdErrBackNow','PredVars','YNames','PredictionSampleStrMat','HistSampleStrMat');
                              else;
                                 save(file,'LogPredLike','StdErr','PredVars','YNames','PredictionSampleStrMat');
                              end;
                              %
                              % create a matlab m-file to read the output.
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\' MargStr(1) 'ALPriorPLike' int2str(SelectedYData) '_' DataStr(1) SelectedVarStr strrep(PredictionSampleStrMat(1,:),':','_') strrep(PredictionSampleStrMat(MaxFH,:),':','_') int2str(NumPriorDraws) '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% Log predictive likelihood of %s predictions \n%%\n',lower(MargStr));
                              fprintf(fid,'LogPredLike = DataStructure.LogPredLike;\n');
                              fprintf(fid,'%%\n%% numerical standard errors of the log predictive likelihood of %s predictions \n%%\n',lower(MargStr));
                              fprintf(fid,'StdErr = DataStructure.StdErr;\n');
                              if (IsOriginal==1)&(IsMarginal==1);
                                 fprintf(fid,'%%\n%% Log predictive likelihood of predictions for the back and nowcast \n%%\n',lower(MargStr));
                                 fprintf(fid,'LogPredLikeBackNow = DataStructure.LogPredLikeBackNow;\n');
                                 fprintf(fid,'%%\n%% numerical standard errors of the log predictive likelihood of the back and nowcasts \n%%\n',lower(MargStr));
                                 fprintf(fid,'StdErrBackNow = DataStructure.StdErrBackNow;\n');
                                 fprintf(fid,'%%\n%% string matrix with back- and nowcast sample dates \n%%\n');
                                 fprintf(fid,'HistSampleStrMat = DataStructure.HistSampleStrMat;\n');
                                 AddTxtStr = 'LogPredLikeBackNow (vector of dimension 2 with the values of the log predictive likelihood for the back- and nowcast), StdErrBackNow (vector of dimension 2 with the numerical standard errors of the log predictive likelihood for the back- and nowcast), HistSampleStrMat (string matrix with the back- and nowcast sample dates), ';
                                 NumEntries = '8';
                              else;
                                 AddTxtStr = '';
                                 NumEntries = '5';
                              end;
                              fprintf(fid,'%%\n%% positions of predicted variables in YNames \n%%\n');
                              fprintf(fid,'PredVars = DataStructure.PredVars;\n');
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                              fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The ' lower(MargStr) ' predictive likelihood results for the prior draws under adaptive learning have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: LogPredLike (vector of dimension hx1 with the values of the ' lower(MargStr) ' log predictive likelihood), StdErr (vector of dimension h with the numerical standard errors of the log predictive likelihood), ' AddTxtStr 'PredVars (cell array with the locations of the predicted variables), YNames (string matrix with the names of the observed variables), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information',[MargStr ' Predictive Likelihood - Prior Distribution'],500,600,CurrINI);
                           end;
                           %
                           % write the results to text file
                           %
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE' MargStr 'AdaptiveLearningPriorPredictiveLikelihood-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataStr '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' int2str(NumPriorDraws) '.txt']);
                           if DSGEModel.BeliefSystem==1;
                              BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           else;
                              [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                              pause(0.2);
                              BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           end;
                           DSGEModel.BeliefSys = BeliefSys;
                           %
                           PrintDSGEPredictiveLikelihood(ResFile,LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd,LogPredLikeBackNow,StdErrBackNow,MinMaxCondLikeBackNow,TestStdBackNow,PredictionSampleStrMat,HistSampleStrMat,YDataStr(SelectedYData,:),IsOriginal,IsMarginal,0,1,NumPriorDraws,0,NumPriorDraws,[],CompTime,DSGEModel,CurrINI);
                           %
                           % display the results
                           %
                           if FileExist(ResFile)==1;
                              TextGUI(ResFile,[MargStr ' Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution',50,CurrINI,1,0);
                           end;
                        end;
                     else;
                        CompTime = toc;
                     end;
                  elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                     clear('thetaPostSample');
                     txt = 'You didn''t select any variables to predict when computing the predictive likelihood. YADA is therefore unable to help you!';
                     About(txt,'information','Predictive Likelihood - Prior Distribution',150,500,CurrINI);
                  end;
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the predictive likelihood based on the prior draws.'];
                About(txt,'information','Predictive Likelihood - Prior Distribution',500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredLikePosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Predictive Likelihood - Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     txt = 'Would you like to use a sub-sample of the posterior draws when estimating the predictive likelihood?';
                     answer = Query(txt,'question',400,'Predictive Likelihood - Posterior Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        NumDraws = length(UseDraws);
                     end;
                  end;
                  %
                  % collect some required data
                  %
                  theta = ModeData.thetaMode;
                  thetaPositions = ModeData.thetaPositions;
                  thetaIndex = ModeData.thetaIndex;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  LowerBound = ModeData.LowerBound;
                  UniformBounds = ModeData.UniformBounds;
                  ModelParameters = ModeData.ModelParameters;
                  %
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  %
                  % check how many time periods to use for predictive distributions
                  %
                  MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % check the maximum when we take NaNs into account
                  %
                  YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                  BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                  if isempty(DSGEModel.Actuals)==1;
                     SelectedYData = 1;
                     DataAction = 'ok';
                     YDataStr = 'Actuals from original sample';
                  else;
                     YDataStr = 'Actuals from original sample';
                     m = length(DSGEModel.Actuals);
                     for i=1:m;
                        YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                     end;
                     [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - Posterior Distribution','','',CurrINI);
                     if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                        YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                     end;
                  end;
                  %
                  if strcmp(lower(DataAction),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxFH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                        end;
                     end;
                     %
                     % selection of variable 
                     %
                     if isempty(DSGEModel.PredictionVariables)==1;
                        PredictionVariables = ones(1,DSGEModel.n);
                        PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                     else;
                        PredictionVariables = DSGEModel.PredictionVariables;
                     end;
                     %
                     [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                     if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                        %
                        % we may continue and store the new selection of prediction variables
                        %
                        DSGEModel.PredictionVariables = PredictionVariables;
                        %
                        % check if we should predict the original data or annualized data
                        %
                        if gcbo==controls.learningtools.predictivedist.unconditional.original.height.posterior;
                           IsOriginal = 1;
                           DataStr = 'Original';
                        elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.height.posterior;
                           IsOriginal = 0;
                           DataStr = 'Annualized';
                        end;
                        %
                        % check if the user wants to compute the marginal or the joint predictive likelihood
                        %
                        txt = 'Would you like to compute the marginal predictive likelikehood or the joint predictive likelihood?';
                        answer = Query(txt,'question',400,'Predictive Likelihood - Posterior Distribution',500,'yes',CurrINI,{'Marginal' 'Joint'});
                        if strcmp(lower(answer),'marginal')==1;
                           IsMarginal = 1;
                           MargStr = 'Marginal';
                        else;
                           IsMarginal = 0;
                           MargStr = 'Joint';
                        end;
                        if ALPredictiveLikelihoodExist(DSGEModel,SelectedYData,IsOriginal,IsMarginal,1,MaxFH,NumDraws,NumBurnin,NumPostDraws,CurrChain)==1;
                           txt = ['YADA has located ' lower(MargStr) ' log posterior calculations for the ' lower(DataStr) ' data for the predictive likelihood on disk. Would you like to use these results?'];
                           answer = Query(txt,'question',200,[MargStr ' Predictive Likelihood - Posterior Distribution'],500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        tic;
                        if strcmp(lower(answer),'yes')==1;
                           DoneCalc = 1;
                        else;
                           %
                           % compute the predictive density height
                           %
                           DoneCalc = AdaptiveLearningDSGEPredictiveLikelihood(thetaPostSample,theta,thetaPositions,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds,ModelParameters,YData,X,BackYData,SelectedYData,IsOriginal,1,IsMarginal,FirstPeriod,LastPeriod,NumPostDraws,NumBurnin,CurrChain,DSGEModel,CurrINI);
                        end;
                        clear('thetaPostSample');
                        if DoneCalc==1;
                           %
                           % we can now do the remaining computations.
                           %
                           [LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd,LogPredLikeBackNow,StdErrBackNow,MinMaxCondLikeBackNow,TestStdBackNow] = AdaptiveLearningPredLikeMCI(SelectedYData,NumDraws,MaxFH,IsOriginal,IsMarginal,1,Weights,NumBurnin,NumPostDraws,CurrChain,DSGEModel);
                           CompTime = toc;
                           %
                           if isempty(LogPredLike)==0;
                              %
                              % ask the user if results should be saved to file
                              %
                              SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                              SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                              %
                              PredictionSampleStrMat = CreateFullSample(DSGEModel);
                              HistSampleStrMat = PredictionSampleStrMat(LastPeriod-1:LastPeriod,:);
                              PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                              txt = ['Would you like to save the posterior draws results for the ' lower(DataStr) ' predictions under adaptive learning to disk?'];
                              answer = Query(txt,'question',200,['Save - ' MargStr ' Predictive Likelihood'],500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 YNames = DSGEModel.VariableNames;
                                 file = FixFilePath([DSGEModel.OutputDirectory '\DSGE' MargStr 'AdaptiveLearningPredictiveLikelihood-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataStr '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' int2str(NumDraws) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '-' int2str(CurrChain) '.mat']);
                                 if (IsOriginal==1)&(IsMarginal==1);
                                    save(file,'LogPredLike','StdErr','LogPredLikeBackNow','StdErrBackNow','PredVars','YNames','PredictionSampleStrMat','HistSampleStrMat');
                                 else;
                                    save(file,'LogPredLike','StdErr','PredVars','YNames','PredictionSampleStrMat');
                                 end;
                                 %
                                 % create a matlab m-file to read the output.
                                 %
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\' MargStr(1) 'ALPLike' int2str(SelectedYData) '_' DataStr(1) SelectedVarStr strrep(PredictionSampleStrMat(1,:),':','_') strrep(PredictionSampleStrMat(MaxFH,:),':','_') int2str(NumDraws) int2str(NumBurnin) int2str(NumPostDraws) int2str(CurrChain) '.m']);
                                 fid = fopen(mfile,'wt');
                                 fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                 fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                 fprintf(fid,'%%\n%% Log predictive likelihood of %s predictions \n%%\n',lower(MargStr));
                                 fprintf(fid,'LogPredLike = DataStructure.LogPredLike;\n');
                                 fprintf(fid,'%%\n%% numerical standard errors of the log predictive likelihood of %s predictions \n%%\n',lower(MargStr));
                                 fprintf(fid,'StdErr = DataStructure.StdErr;\n');
                                 if (IsOriginal==1)&(IsMarginal==1);
                                    fprintf(fid,'%%\n%% Log predictive likelihood of predictions for the back and nowcast \n%%\n',lower(MargStr));
                                    fprintf(fid,'LogPredLikeBackNow = DataStructure.LogPredLikeBackNow;\n');
                                    fprintf(fid,'%%\n%% numerical standard errors of the log predictive likelihood of the back and nowcasts \n%%\n',lower(MargStr));
                                    fprintf(fid,'StdErrBackNow = DataStructure.StdErrBackNow;\n');
                                    fprintf(fid,'%%\n%% string matrix with back- and nowcast sample dates \n%%\n');
                                    fprintf(fid,'HistSampleStrMat = DataStructure.HistSampleStrMat;\n');
                                    AddTxtStr = 'LogPredLikeBackNow (vector of dimension 2 with the values of the log predictive likelihood for the back- and nowcast), StdErrBackNow (vector of dimension 2 with the numerical standard errors of the log predictive likelihood for the back- and nowcast), HistSampleStrMat (string matrix with the back- and nowcast sample dates), ';
                                    NumEntries = '8';
                                 else;
                                    AddTxtStr = '';
                                    NumEntries = '5';
                                 end;
                                 fprintf(fid,'%%\n%% positions of predicted variables in YNames \n%%\n');
                                 fprintf(fid,'PredVars = DataStructure.PredVars;\n');
                                 fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                 fprintf(fid,'YNames = DataStructure.YNames;\n');
                                 fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                                 fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                                 fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                 %
                                 % end the script file
                                 %
                                 fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                 fclose(fid);
                                 %
                                 txt = ['The ' lower(MargStr) ' predictive likelihood results for the posterior draws under adaptive learning have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: LogPredLike (vector of dimension hx1 with the values of the ' lower(MargStr) ' log predictive likelihood), StdErr (vector of dimension h with the numerical standard errors of the log predictive likelihood), ' AddTxtStr 'PredVars (cell array with the locations of the predicted variables), YNames (string matrix with the names of the observed variables), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                 About(txt,'information',[MargStr ' Predictive Likelihood - Posterior Distribution'],500,600,CurrINI);
                              end;
                              %
                              % write the results to text file
                              %
                              ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE' MargStr 'AdaptiveLearningPredictiveLikelihood-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataStr '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' int2str(NumDraws) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '-' int2str(CurrChain) '.txt']);
                              if DSGEModel.BeliefSystem==1;
                                 BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                              else;
                                 [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                                 pause(0.2);
                                 BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                              end;
                              DSGEModel.BeliefSys = BeliefSys;
                              %
                              PrintDSGEPredictiveLikelihood(ResFile,LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd,LogPredLikeBackNow,StdErrBackNow,MinMaxCondLikeBackNow,TestStdBackNow,PredictionSampleStrMat,HistSampleStrMat,YDataStr(SelectedYData,:),IsOriginal,IsMarginal,1,1,NumDraws,NumBurnin,NumPostDraws,[],CompTime,DSGEModel,CurrINI);
                              %
                              % display the results
                              %
                              if FileExist(ResFile)==1;
                                 TextGUI(ResFile,[MargStr ' Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,0);
                              end;
                           end;
                        else;
                           CompTime = toc;
                        end;
                     elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                        clear('thetaPostSample');
                        txt = 'You didn''t select any variables to predict when computing the predictive likelihood. YADA is therefore unable to help you!';
                        About(txt,'information','Predictive Likelihood - Posterior Distribution',150,500,CurrINI);
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconMargPredMomentsPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Marginal Predictive Moments - Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % load the data
               %
               if FileExist(DSGEModel.SystemPriorFile)==0;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               else;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               end;
               PriorData = load(PriorDrawsFile);
               thetaDraws = PriorData.thetaDraws;
               NumPriorDraws = size(thetaDraws,2);
               %
               % check if we have a system prior and if so, use a subset of the prior draws
               %
               OldDSGEModel = DSGEModel;
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  if DSGEModel.PosteriorSampler>7;
                     [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','CRPS - Prior Distribution','','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PostDrawsPercentValue = SelectedOption;
                     end;
                  else;
                     action = 'ok';
                  end;
                  if strcmp(lower(action),'ok')==1;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                     if length(UseDraws)<NumPriorDraws;
                        thetaDraws = thetaDraws(:,UseDraws);
                        NumPriorDraws = length(UseDraws);
                     end;
                  end;
               end;
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               DSGEModel.FirstPeriod = FirstPeriod;
               DSGEModel.LastPeriod = LastPeriod;
               %
               % check how many time periods to use for predictive distributions
               %
               MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
               %
               % check the maximum when we take NaNs into account
               %
               YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               MaxFH = min(MaxFH,MaxPH);
               YData = YData(:,1:MaxFH);
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxFH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                  end;
               end;
               %
               % Determine the prediction sample
               %
               SampleStrMat = CreateFullSample(DSGEModel);
               PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
               BNSampleStrMat = SampleStrMat(LastPeriod-1:LastPeriod,:);
               SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
               %
               % check if older calculations exists given that such may be used
               %
               MatFile = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningDSGE-PredictiveMoments-Prior-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(MaxFH) '.' int2str(NumPriorDraws) '.mat']);
               if FileExist(MatFile)==1;
                  txt = ['YADA has located prior draws based estimates of the marginal predictive moments under adaptive learning on disk. Would you like to use these results?'];
                  answer = Query(txt,'question',200,'Marginal Predictive Moments - Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               tic;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  DoneCalc = AdaptiveLearningDSGEPredictiveMoments(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxFH,NumPriorDraws,[],0,[],SmpStr,0);
               end;
               CompTime = toc;
               if DoneCalc==1;
                  %
                  % load the results
                  %
                  MPM = load(MatFile);
                  PredY = MPM.MeanPredictY;
                  n = DSGEModel.n;
                  CovPred = zeros(n,n,MaxFH);
                  for i=1:MaxFH;
                     CovPred(:,:,i) = MPM.MeasureCov+MPM.ShockCov(:,:,i)+MPM.StateCov(:,:,i)+MPM.ParameterCov(:,:,i);
                  end;
                  BNCov = [];
                  BNPredictY = [];
                  if isfield(MPM,'BNPredictY')==1;
                     BNPredictY = MPM.BNPredictY;
                     if isempty(MPM.BNPredictY)==0;
                        if isfield(MPM,'BNCov')==1;
                           BNCov = MPM.BNCov;
                        else;
                           BNCov = MPM.BNObsCov+MPM.BNParamCov;
                           save(MatFile,'BNCov','-APPEND');
                        end;
                        IsBackCast = 1;
                     else;
                        IsBackCast = 0;
                     end;
                  else;
                     IsBackCast = 0;
                  end;
                  if isfield(MPM,'BNSampleStrMat')==0;
                     save(MatFile,'BNSampleStrMat','-APPEND');
                  end;
                  %
                  if isfield(MPM,'CovPred')==0;
                     save(MatFile,'CovPred','PredictionSampleStrMat','-APPEND');
                  end;
                  if isfield(MPM,'YData')==0;
                     save(MatFile,'YData','-APPEND');
                  end;
                  %
                  % aske if the user wants to have easy access to the results
                  %
                  txt = ['The marginal predictive moments using the prior draws have been saved to the file "' GetFilename(MatFile) '" in the "' GetPath(MatFile) '" directory. Would you like YADA to write an m-file to give you direct access to these results?'];
                  answer = Query(txt,'question',300,'Marginal Predictive Moments - Prior Distribution',600,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     YNames = DSGEModel.VariableNames;
                     if isfield(MPM,'YNames')==0;
                        save(MatFile,'YNames','-APPEND');
                     end;
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\' strrep(strrep(strrep(GetName(MatFile),'AdaptiveLearning','AL'),'-',''),'.','') '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStructure.YNames;\n');
                     fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                     fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                     fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables \n%%\n');
                     fprintf(fid,'MeanPredictY = DataStructure.MeanPredictY;\n');
                     fprintf(fid,'%%\n%% 3D matrix with the marginal prediction covariances \n%%\n');
                     fprintf(fid,'CovPred = DataStructure.CovPred;\n');
                     fprintf(fid,'%%\n%% 2D matrix with the part due to the measurement errors \n%%\n');
                     fprintf(fid,'MeasureCov = DataStructure.MeasureCov;\n');
                     fprintf(fid,'%%\n%% 3D matrix with the part due to the shock uncertainty \n%%\n');
                     fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                     fprintf(fid,'%%\n%% 3D matrix with the part due to the state uncertainty \n%%\n');
                     fprintf(fid,'StateCov = DataStructure.StateCov;\n');
                     fprintf(fid,'%%\n%% 3D matrix with the part due to parameter uncertainty \n%%\n');
                     fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                     fprintf(fid,'%%\n%% matrix with the actuals for the prediction sample \n%%\n');
                     fprintf(fid,'YData = DataStructure.YData;\n');
                     if IsBackCast==1;
                        fprintf(fid,'%%\n%% string matrix with back- and nowcast sample dates \n%%\n');
                        fprintf(fid,'BNSampleStrMat = DataStructure.BNSampleStrMat;\n');
                        fprintf(fid,'%%\n%% matrix with the mean back- and nowcasts of the observed variables \n%%\n');
                        fprintf(fid,'BNPredictY = DataStructure.BNPredictY;\n');
                        fprintf(fid,'%%\n%% 3D matrix with the marginal back- and nowcasts covariances \n%%\n');
                        fprintf(fid,'BNCov = DataStructure.BNCov;\n');
                        fprintf(fid,'%%\n%% 3D matrix with the part due to shocks and states \n%%\n');
                        fprintf(fid,'BNObsCov = DataStructure.BNObsCov;\n');
                        fprintf(fid,'%%\n%% 3D matrix with the part due to parameter uncertainty\n%%\n');
                        fprintf(fid,'BNParamCov = DataStructure.BNParamCov;\n');
                     end;
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     %
                     txt = ['You can access the marginal predictive moments estimated from the prior distribution through the file: "' GetFilename(mfile) '", located in the "' GetPath(mfile) '" directory.'];
                     About(txt,'information','Marginal Predictive Moments - Prior Distribution',500,600,CurrINI);
                  end;
                  %
                  % write to a text file
                  %
                  if DSGEModel.BeliefSystem==1;
                     BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                  else;
                     [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                     pause(0.2);
                     BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                  end;
                  DSGEModel.BeliefSys = BeliefSys;
                  ResFile = strrep(MatFile,'.mat','.txt');
                  PrintPredictiveMoments(ResFile,PredY,CovPred,PredictionSampleStrMat,BNPredictY,BNCov,BNSampleStrMat,[],NumPriorDraws,NumPriorDraws,0,0,1,CompTime,DSGEModel,CurrINI);
                  if FileExist(ResFile)==1;
                     TextGUI(ResFile,['Marginal Predictive Moments - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution',50,CurrINI,1,0);
                  end;
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the marginal predictive moments using the prior draws.'];
                About(txt,'information','Marginal Predictive Moments - Prior Distribution',500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconMargPredMomentsPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Marginal Predictive Moments - Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws when estimating the marginal predictive moments?';
                  answer = Query(txt,'question',400,'Marginal Predictive Moments - Posterior Distribution',500,'no',CurrINI);
                  if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        NumDraws = length(UseDraws);
                     end;
                  else;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'Marginal Predictive Moments - Posterior Distribution',Weights,[]);
                        if isempty(UseDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        Weights = Weights(UseDraws,1);
                        Weights = Weights*length(Weights)/sum(Weights);
                        NumDraws = length(UseDraws);
                     end;
                  end;
                  %
                  % collect some required data
                  %
                  theta = ModeData.thetaMode;
                  thetaPositions = ModeData.thetaPositions;
                  thetaIndex = ModeData.thetaIndex;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  LowerBound = ModeData.LowerBound;
                  UniformBounds = ModeData.UniformBounds;
                  ModelParameters = ModeData.ModelParameters;
                  %
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  DSGEModel.FirstPeriod = FirstPeriod;
                  DSGEModel.LastPeriod = LastPeriod;
                  %
                  % check how many time periods to use for predictive distributions
                  %
                  MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % check the maximum when we take NaNs into account
                  %
                  YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                  [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                  MaxFH = min(MaxFH,MaxPH);
                  YData = YData(:,1:MaxFH);
                  %
                  % matrix with exogenous variables
                  %
                  X = [];
                  if DSGEModel.k>0;
                     if NumPredX<MaxFH;
                        X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                     end;
                     if NumPredX>0;
                        X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                     end;
                  end;
                  %
                  % Determine the prediction sample
                  %
                  SampleStrMat = CreateFullSample(DSGEModel);
                  PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                  BNSampleStrMat = SampleStrMat(LastPeriod-1:LastPeriod,:);
                  SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                  %
                  % check if older calculations exists given that such may be used
                  %
                  MatFile = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningDSGE-PredictiveMoments-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(CurrChain) '-' int2str(MaxFH) '.' int2str(NumDraws) '-' int2str(NumPostDraws) '.mat']);
                  if FileExist(MatFile)==1;
                     txt = ['YADA has located posterior draws based estimates of the marginal predictive moments under adaptive learning on disk. Would you like to use these results?'];
                     answer = Query(txt,'question',200,'Marginal Predictive Moments - Posterior Distribution',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  tic;
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     DoneCalc = AdaptiveLearningDSGEPredictiveMoments(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxFH,NumPostDraws,CurrChain,1,Weights,SmpStr,0);
                  end;
                  clear('thetaPostSample');
                  CompTime = toc;
                  if DoneCalc==1;
                     %
                     % load the results
                     %
                     MPM = load(MatFile);
                     PredY = MPM.MeanPredictY;
                     n = DSGEModel.n;
                     CovPred = zeros(n,n,MaxFH);
                     for i=1:MaxFH;
                        CovPred(:,:,i) = MPM.MeasureCov+MPM.ShockCov(:,:,i)+MPM.StateCov(:,:,i)+MPM.ParameterCov(:,:,i);
                     end;
                     BNCov = [];
                     BNPredictY = [];
                     if isfield(MPM,'BNPredictY')==1;
                        BNPredictY = MPM.BNPredictY;
                        if isempty(MPM.BNPredictY)==0;
                           if isfield(MPM,'BNCov')==1;
                              BNCov = MPM.BNCov;
                           else;
                              BNCov = MPM.BNObsCov+MPM.BNParamCov;
                              save(MatFile,'BNCov','-APPEND');
                           end;
                           IsBackCast = 1;
                        else;
                           IsBackCast = 0;
                        end;
                     else;
                        IsBackCast = 0;
                     end;
                     if isfield(MPM,'BNSampleStrMat')==0;
                        save(MatFile,'BNSampleStrMat','-APPEND');
                     end;
                     %
                     if isfield(MPM,'CovPred')==0;
                        save(MatFile,'CovPred','PredictionSampleStrMat','-APPEND');
                     end;
                     if isfield(MPM,'YData')==0;
                        save(MatFile,'YData','-APPEND');
                     end;
                     %
                     % aske if the user wants to have easy access to the results
                     %
                     txt = ['The marginal predictive moments using the posterior draws have been saved to the file "' GetFilename(MatFile) '" in the "' GetPath(MatFile) '" directory. Would you like YADA to write an m-file to give you direct access to these results?'];
                     answer = Query(txt,'question',300,'Marginal Predictive Moments - Posterior Distribution',600,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        YNames = DSGEModel.VariableNames;
                        if isfield(MPM,'YNames')==0;
                           save(MatFile,'YNames','-APPEND');
                        end;
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\' strrep(strrep(strrep(GetName(MatFile),'AdaptiveLearning','AL'),'-',''),'.','') '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                        fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                        fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables \n%%\n');
                        fprintf(fid,'MeanPredictY = DataStructure.MeanPredictY;\n');
                        fprintf(fid,'%%\n%% 3D matrix with the marginal prediction covariances \n%%\n');
                        fprintf(fid,'CovPred = DataStructure.CovPred;\n');
                        fprintf(fid,'%%\n%% 2D matrix with the part due to the measurement errors \n%%\n');
                        fprintf(fid,'MeasureCov = DataStructure.MeasureCov;\n');
                        fprintf(fid,'%%\n%% 3D matrix with the part due to the shock uncertainty \n%%\n');
                        fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                        fprintf(fid,'%%\n%% 3D matrix with the part due to the state uncertainty \n%%\n');
                        fprintf(fid,'StateCov = DataStructure.StateCov;\n');
                        fprintf(fid,'%%\n%% 3D matrix with the part due to parameter uncertainty \n%%\n');
                        fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                        fprintf(fid,'%%\n%% matrix with the actuals for the prediction sample \n%%\n');
                        fprintf(fid,'YData = DataStructure.YData;\n');
                        if IsBackCast==1;
                           fprintf(fid,'%%\n%% string matrix with back- and nowcast sample dates \n%%\n');
                           fprintf(fid,'BNSampleStrMat = DataStructure.BNSampleStrMat;\n');
                           fprintf(fid,'%%\n%% matrix with the mean back- and nowcasts of the observed variables \n%%\n');
                           fprintf(fid,'BNPredictY = DataStructure.BNPredictY;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the marginal back- and nowcasts covariances \n%%\n');
                           fprintf(fid,'BNCov = DataStructure.BNCov;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the part due to shocks and states \n%%\n');
                           fprintf(fid,'BNObsCov = DataStructure.BNObsCov;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the part due to parameter uncertainty\n%%\n');
                           fprintf(fid,'BNParamCov = DataStructure.BNParamCov;\n');
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        txt = ['You can access the marginal predictive moments estimated from the posterior distribution through the file: "' GetFilename(mfile) '", located in the "' GetPath(mfile) '" directory.'];
                        About(txt,'information','Marginal Predictive Moments - Posterior Distribution',500,600,CurrINI);
                     end;
                     %
                     % write to a text file
                     %
                     if DSGEModel.BeliefSystem==1;
                        BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                     else;
                        [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                        pause(0.2);
                        BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                     end;
                     DSGEModel.BeliefSys = BeliefSys;
                     ResFile = strrep(MatFile,'.mat','.txt');
                     PrintPredictiveMoments(ResFile,PredY,CovPred,PredictionSampleStrMat,BNPredictY,BNCov,BNSampleStrMat,[],NumDraws,NumPostDraws,NumBurnin,1,1,CompTime,DSGEModel,CurrINI);
                     if FileExist(ResFile)==1;
                        TextGUI(ResFile,['Marginal Predictive Moments - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,0);
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPITPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Probability Integral Transform - Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % load the data
               %
               if FileExist(DSGEModel.SystemPriorFile)==0;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               else;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               end;
               PriorData = load(PriorDrawsFile);
               thetaDraws = PriorData.thetaDraws;
               NumPriorDraws = size(thetaDraws,2);
               %
               % check if we have a system prior and if so, use a subset of the prior draws
               %
               OldDSGEModel = DSGEModel;
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  if DSGEModel.PosteriorSampler>7;
                     [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','CRPS - Prior Distribution','','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PostDrawsPercentValue = SelectedOption;
                     end;
                  else;
                     action = 'ok';
                  end;
                  if strcmp(lower(action),'ok')==1;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                     if length(UseDraws)<NumPriorDraws;
                        thetaDraws = thetaDraws(:,UseDraws);
                        NumPriorDraws = length(UseDraws);
                     end;
                  end;
               end;
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               DSGEModel.FirstPeriod = FirstPeriod;
               DSGEModel.LastPeriod = LastPeriod;
               %
               % check how many time periods to use for predictive distributions
               %
               MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
               %
               % check the maximum when we take NaNs into account
               %
               YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
               BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
               if isempty(DSGEModel.Actuals)==1;
                  SelectedYData = 1;
                  DataAction = 'ok';
                  YDataStr = 'Actuals from original sample';
               else;
                  YDataStr = 'Actuals from original sample';
                  m = length(DSGEModel.Actuals);
                  for i=1:m;
                     YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                  end;
                  [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Probability Integral Transform - Prior Distribution','','',CurrINI);
                  if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                     YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                  end;
               end;
               %
               if strcmp(lower(DataAction),'ok')==1;
                  MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                  [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                  MaxFH = min(MaxFH,MaxPH);
                  YData = YData(:,1:MaxFH);
                  %
                  % matrix with exogenous variables
                  %
                  X = [];
                  XP = [];
                  if DSGEModel.k>0;
                     if NumPredX<MaxFH;
                        X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                     end;
                     if NumPredX>0;
                        X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                     end;
                     if NumPredX<MaxPH;
                        XP = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                     end;
                     if NumPredX>0;
                        XP = [XP DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                     end;
                  end;
                  %
                  % selection of variable 
                  %
                  if isempty(DSGEModel.PredictionVariables)==1;
                     PredictionVariables = ones(1,DSGEModel.n);
                     PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                  else;
                     PredictionVariables = DSGEModel.PredictionVariables;
                  end;
                  %
                  [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                  if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                     DSGEModel.PredictionVariables = PredictionVariables;
                     %
                     % Determine the prediction sample
                     %
                     SampleStrMat = CreateFullSample(DSGEModel);
                     PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                     BNSampleStrMat = SampleStrMat(LastPeriod-1:LastPeriod,:);
                     SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                     %
                     % check if older calculations exists given that such may be used
                     %
                     MatFile = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningDSGE-PredictiveMoments-Prior-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(MaxFH) '.' int2str(NumPriorDraws) '.mat']);
                     PITMomentExist = 0;
                     if FileExist(MatFile)==1;
                        CheckData = load(MatFile);
                        if isfield(CheckData,'PITMoments')==1;
                           if isempty(CheckData.PITMoments)==0;
                              PITMomentExist = 1;
                           end;
                        end;
                        clear('CheckData');
                     end;
                     if PITMomentExist==1;
                        txt = 'YADA has located prior draws based estimates of the moments under adaptive learning on disk that can be used for the probability integral transform (PIT). Would you like to use these results?';
                        answer = Query(txt,'question',200,'Probability Integral Transform - Prior Distribution',500,'yes',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     tic;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        DoneCalc = AdaptiveLearningDSGEPredictiveMoments(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxFH,NumPriorDraws,[],0,[],SmpStr,1);
                     end;
                     if DoneCalc==1;
                        %
                        % we can now do the PIT calculations based on the moments
                        %
                        [PITMatrix,BNPITMatrix] = ComputePITs(YData,DSGEModel.Y(:,LastPeriod-1:LastPeriod),BackYData,PredictionVariables,MatFile,[],0,(exist('mvncdf')~=0),CurrINI);
                        CompTimeStepOne = toc;
                        %
                        % check if the user also wants to study PITs based on simulated paths
                        %
                        txt = 'YADA can also estimate probability integral transform (PIT) values under adaptive learning based on simulated paths from the predictive distribution. Would you like to run these calculations as well?';
                        answer = Query(txt,'question',200,'Probability Integral Transform - Prior Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % setup the necessary variables for computing or loading simulated paths from the predictive distribution
                           %
                           % determine how many prediction paths to use
                           %
                           NumPredPathsStr = get(controls.forecast.predpaths,'String');
                           NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                           %
                           TransData = 0;
                           AnnualizeData = 0;
                           AnnualStr = 'original';
                           QueryHead = 'Predictive Distributions - Original Data';
                           if PredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,0,[],0)==1;
                              txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                              answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           %
                           % check if we shall recompute
                           %
                           if strcmp(lower(answer),'yes')==1;
                              DoneCalc2 = 1;
                              tic;
                           else;
                              %
                              % ask if we should conduct prediction event calculations
                              %
                              txt = strvcat('Would you like YADA to calculate prediction event probabilities and perform risk analysis?',' ','NOTE: The results will only be stored to disk and will not be displayed.');
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                    DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                                 end;
                                 %
                                 % make sure that the prediction event length does not exceed the maximum prediction horizon
                                 %
                                 for i=1:DSGEModel.n;
                                    if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                       DSGEModel.PredictionEvent(i,3) = MaxPH;
                                    end;
                                 end;
                                 %
                                 % display a dialog for setting up an prediction event
                                 %
                                 [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                                 if strcmp(lower(action),'ok')==1;
                                    DSGEModel.PredictionEvent = PredictionEvent;
                                    DSGEModel.RunPredictionEvent = 1;
                                 else;
                                    DSGEModel.RunPredictionEvent = 0;
                                 end;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                              %
                              % we can now compute the predictive distribution for the endogenous variables
                              % up to period MaxPH
                              %
                              tic;
                              [DoneCalc2,PredEventData,StateCov,ShockCov,MeasureCov,ParameterCov] = AdaptiveLearningDSGEPredictionPaths(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,XP,MaxPH,NumPredPaths,AnnualizeData,0,[],0,[]);
                           end;
                           %
                           %
                           if DoneCalc2==1;
                              %
                              % compute PITs based on the paths from the predictive distribution
                              %
                              PathPITMatrix = ComputePITsAdaptiveLearningDSGEPaths(YData,NumPriorDraws,NumPredPaths,MaxPH,[],0,[],DSGEModel,CurrINI);
                              CompTimeStepTwo = toc;
                           else;
                              %
                              % make sure that path based estimates are empty
                              %
                              PathPITMatrix = [];
                              CompTimeStepTwo = [];
                           end;
                        else;
                           %
                           % make sure that path based estimates are empty
                           %
                           PathPITMatrix = [];
                           CompTimeStepTwo = [];
                           NumPredPaths = 0;
                        end;
                        %
                        % check if the user wants to store the results to disk
                        %
                        txt = 'Would you like to save the probability integral transform estimates under adaptive learning to file?';
                        answer = Query(txt,'question',300,'Probability Integral Transform - Prior Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           file = FixFilePath([DSGEModel.OutputDirectory '\PITs-AdaptiveLearning-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                           YNames = DSGEModel.VariableNames;
                           save(file,'YNames','PredictionVariables','PITMatrix','PredDateVector');
                           if isempty(BNPITMatrix)==0;
                              save(file,'BNPITMatrix','-APPEND')
                           end;
                           if isempty(PathPITMatrix)==0;
                              save(file,'PathPITMatrix','-APPEND');
                           end;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\PITsALPrior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' int2str(MaxPH) '_' int2str(NumPriorDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                           fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                           fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable uniform PITs and the joint uniform PIT\n%%\n%% for each forecast horizon. Use norminv to transform to normal.\n%%\n');
                           fprintf(fid,'PITMatrix = DataStructure.PITMatrix;\n');
                           NumEntries = 3;
                           if isempty(BNPITMatrix)==0;
                              fprintf(fid,'%%\n%% n+1 x 2 matrix with the individual variable uniform PITs and the joint uniform PIT\n%%\n%% for the backcast and the nowcast. Use norminv to transform to normal.\n%%\n');
                              fprintf(fid,'BNPITMatrix = DataStructure.BNPITMatrix;\n');
                              NumEntries = NumEntries+1;
                              BNTxt = 'BNPITMatrix (matrix with the individual variable uniform PITs and the joint predicted variable uniform PIT for the backcast and the nowcast), ';
                           else;
                              BNTxt = '';
                           end;
                           if isempty(PathPITMatrix)==0;
                              fprintf(fid,'%%\n%% n x h matrix with the individual variable uniform PITs using the prediction\n%%\n%% paths for each forecast horizon. Use norminv to transform to normal.\n%%\n');
                              fprintf(fid,'PathPITMatrix = DataStructure.PathPITMatrix;\n');
                              NumEntries = NumEntries+1;
                              PathTxt = 'PathPITMatrix (matrix with the individual variable uniform PITs using the prediction paths for each forecast horizon), ';
                           else;
                              PathTxt = '';
                           end;
                           fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                           fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           NumEntries = NumEntries+1;
                           %
                           txt = ['The probability integral transform (PIT) estimates (uniform under the assumption of well-calibrated) under adaptive learning for the prior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), PITMatrix (matrix with the individual variable uniform PITs and the joint predicted variable uniform PIT for each forecast horizon), ' BNTxt PathTxt 'and PredDateVector (vector with dates for the forecast horizon). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Probability Integral Transform - Prior Distribution',500,600,CurrINI);
                        end;
                        %
                        % write the results to a text file
                        %
                        if DSGEModel.BeliefSystem==1;
                           BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                        else;
                           [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                           pause(0.2);
                           BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                        end;
                        DSGEModel.BeliefSys = BeliefSys;
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\PITs-AdaptiveLearning-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                        PrintPITs(ResFile,'DSGE with learning',PredictionVariables,PredictionSampleStrMat,BNSampleStrMat,YDataStr(SelectedYData,:),PITMatrix,BNPITMatrix,PathPITMatrix,0,1,NumPriorDraws,0,NumPriorDraws,[],CompTimeStepOne,CompTimeStepTwo,DSGEModel,CurrINI);
                        %
                        % check if file exists and display it then
                        %
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Probability Integral Transform - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution',50,CurrINI,1,0);
                        end;
                     end;
                  elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                     txt = 'You didn''t select any variables to predict when computing the probability integral transform. YADA is therefore unable to help you!';
                     About(txt,'information','Probability Integral Transform - Prior Distribution',150,500,CurrINI);
                  end;
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the probability integral transform using the prior draws.'];
                About(txt,'information','Probability Integral Transform - Prior Distribution',500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPITPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Probability Integral Transform - Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws when estimating the probability integral transform?';
                  answer = Query(txt,'question',400,'Probability Integral Transform - Posterior Distribution',500,'no',CurrINI);
                  if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        NumDraws = length(UseDraws);
                     end;
                  else;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'Probability Integral Transform - Posterior Distribution',Weights,[]);
                        if isempty(UseDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        Weights = Weights(UseDraws,1);
                        Weights = Weights*length(Weights)/sum(Weights);
                        NumDraws = length(UseDraws);
                     end;
                  end;
                  %
                  % collect some required data
                  %
                  theta = ModeData.thetaMode;
                  thetaPositions = ModeData.thetaPositions;
                  thetaIndex = ModeData.thetaIndex;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  LowerBound = ModeData.LowerBound;
                  UniformBounds = ModeData.UniformBounds;
                  ModelParameters = ModeData.ModelParameters;
                  %
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  DSGEModel.FirstPeriod = FirstPeriod;
                  DSGEModel.LastPeriod = LastPeriod;
                  %
                  % check how many time periods to use for predictive distributions
                  %
                  MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % check the maximum when we take NaNs into account
                  %
                  YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                  BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                  if isempty(DSGEModel.Actuals)==1;
                     SelectedYData = 1;
                     DataAction = 'ok';
                     YDataStr = 'Actuals from original sample';
                  else;
                     YDataStr = 'Actuals from original sample';
                     m = length(DSGEModel.Actuals);
                     for i=1:m;
                        YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                     end;
                     [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Probability Integral Transform - Posterior Distribution','','',CurrINI);
                     if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                        YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                     end;
                  end;
                  %
                  if strcmp(lower(DataAction),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     XP = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxFH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                        end;
                        if NumPredX<MaxPH;
                           XP = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           XP = [XP DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                        end;
                     end;
                     %
                     % selection of variable 
                     %
                     if isempty(DSGEModel.PredictionVariables)==1;
                        PredictionVariables = ones(1,DSGEModel.n);
                        PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                     else;
                        PredictionVariables = DSGEModel.PredictionVariables;
                     end;
                     %
                     [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                     if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                        DSGEModel.PredictionVariables = PredictionVariables;
                        %
                        % Determine the prediction sample
                        %
                        SampleStrMat = CreateFullSample(DSGEModel);
                        PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                        BNSampleStrMat = SampleStrMat(LastPeriod-1:LastPeriod,:);
                        SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                        %
                        % check if older calculations exists given that such may be used
                        %
                        MatFile = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningDSGE-PredictiveMoments-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(CurrChain) '-' int2str(MaxFH) '.' int2str(NumDraws) '-' int2str(NumPostDraws) '.mat']);
                        PITMomentExist = 0;
                        if FileExist(MatFile)==1;
                           CheckData = load(MatFile);
                           if isfield(CheckData,'PITMoments')==1;
                              if isempty(CheckData.PITMoments)==0;
                                 PITMomentExist = 1;
                              end;
                           end;
                           clear('CheckData');
                        end;
                        if PITMomentExist==1;
                           txt = 'YADA has located posterior draws based estimates of the moments under adaptive learning on disk that can be used for the probability integral transform (PIT). Would you like to use these results?';
                           answer = Query(txt,'question',200,'Probability Integral Transform - Posterior Distribution',500,'yes',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        tic;
                        if strcmp(lower(answer),'yes')==1;
                           DoneCalc = 1;
                        else;
                           DoneCalc = AdaptiveLearningDSGEPredictiveMoments(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxFH,NumPostDraws,CurrChain,1,Weights,SmpStr,1);
                        end;
                        %
                        if DoneCalc==1;
                           %
                           % we can now do the PIT calculations based on the moments
                           %
                           [PITMatrix,BNPITMatrix] = ComputePITs(YData,DSGEModel.Y(:,LastPeriod-1:LastPeriod),BackYData,PredictionVariables,MatFile,Weights,1,(exist('mvncdf')~=0),CurrINI);
                           CompTimeStepOne = toc;
                           %
                           % check if the user also wants to study PITs based on simulated paths
                           %
                           txt = 'YADA can also estimate probability integral transform (PIT) values under adaptive learning based on simulated paths from the predictive distribution. Would you like to run these calculations as well?';
                           answer = Query(txt,'question',200,'Probability Integral Transform - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              %
                              % setup the necessary variables for computing or loading simulated paths from the predictive distribution
                              %
                              % determine how many prediction paths to use
                              %
                              NumPredPathsStr = get(controls.forecast.predpaths,'String');
                              NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                              %
                              TransData = 0;
                              AnnualizeData = 0;
                              AnnualStr = 'original';
                              QueryHead = 'Predictive Distributions - Original Data';
                              if PredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,MaxPH,NumDraws,NumPredPaths,AnnualizeData,TransData,CurrChain,1)==1;
                                 txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                                 answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
                              else;
                                 answer = 'No';
                              end;
                              %
                              % check if we shall recompute
                              %
                              if strcmp(lower(answer),'yes')==1;
                                 DoneCalc2 = 1;
                                 tic;
                              else;
                                 %
                                 % ask if we should conduct prediction event calculations
                                 %
                                 txt = strvcat('Would you like YADA to calculate prediction event probabilities and perform risk analysis?',' ','NOTE: The results will only be stored to disk and will not be displayed.');
                                 answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                                 if strcmp(lower(answer),'yes')==1;
                                    if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                       DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                                    end;
                                    %
                                    % make sure that the prediction event length does not exceed the maximum prediction horizon
                                    %
                                    for i=1:DSGEModel.n;
                                       if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                          DSGEModel.PredictionEvent(i,3) = MaxPH;
                                       end;
                                    end;
                                    %
                                    % display a dialog for setting up an prediction event
                                    %
                                    [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                                    if strcmp(lower(action),'ok')==1;
                                       DSGEModel.PredictionEvent = PredictionEvent;
                                       DSGEModel.RunPredictionEvent = 1;
                                    else;
                                       DSGEModel.RunPredictionEvent = 0;
                                    end;
                                 else;
                                    DSGEModel.RunPredictionEvent = 0;
                                 end;
                                 %
                                 % we can now compute the predictive distribution for the endogenous variables
                                 % up to period MaxPH
                                 %
                                 tic;
                                 [DoneCalc2,PredEventData,StateCov,ShockCov,MeasureCov,ParameterCov] = AdaptiveLearningDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,XP,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
                              end;
                              %
                              %
                              if DoneCalc2==1;
                                 %
                                 % compute PITs based on the paths from the predictive distribution
                                 %
                                 PathPITMatrix = ComputePITsAdaptiveLearningDSGEPaths(YData,NumDraws,NumPredPaths,MaxPH,CurrChain,1,Weights,DSGEModel,CurrINI);
                                 CompTimeStepTwo = toc;
                              else;
                                 %
                                 % make sure that path based estimates are empty
                                 %
                                 PathPITMatrix = [];
                                 CompTimeStepTwo = [];
                              end;
                           else;
                              %
                              % make sure that path based estimates are empty
                              %
                              PathPITMatrix = [];
                              CompTimeStepTwo = [];
                              NumPredPaths = 0;
                           end;
                           clear('thetaPostSample');
                           %
                           % check if the user wants to store the results to disk
                           %
                           txt = 'Would you like to save the probability integral transform estimates under adaptive learning to file?';
                           answer = Query(txt,'question',300,'Probability Integral Transform - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              file = FixFilePath([DSGEModel.OutputDirectory '\PITs-AdaptiveLearning-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                              YNames = DSGEModel.VariableNames;
                              save(file,'YNames','PredictionVariables','PITMatrix','PredDateVector');
                              if isempty(BNPITMatrix)==0;
                                 save(file,'BNPITMatrix','-APPEND')
                              end;
                              if isempty(PathPITMatrix)==0;
                                 save(file,'PathPITMatrix','-APPEND');
                              end;
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\PITsALPosterior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' int2str(CurrChain) '_' int2str(MaxPH) '_' int2str(NumDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                              fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                              fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable uniform PITs and the joint uniform PIT\n%%\n%% for each forecast horizon. Use norminv to transform to normal.\n%%\n');
                              fprintf(fid,'PITMatrix = DataStructure.PITMatrix;\n');
                              NumEntries = 3;
                              if isempty(BNPITMatrix)==0;
                                 fprintf(fid,'%%\n%% n+1 x 2 matrix with the individual variable uniform PITs and the joint uniform PIT\n%%\n%% for the backcast and the nowcast. Use norminv to transform to normal.\n%%\n');
                                 fprintf(fid,'BNPITMatrix = DataStructure.BNPITMatrix;\n');
                                 NumEntries = NumEntries+1;
                                 BNTxt = 'BNPITMatrix (matrix with the individual variable uniform PITs and the joint predicted variable uniform PIT for the backcast and the nowcast), ';
                              else;
                                 BNTxt = '';
                              end;
                              if isempty(PathPITMatrix)==0;
                                 fprintf(fid,'%%\n%% n x h matrix with the individual variable uniform PITs using the prediction\n%%\n%% paths for each forecast horizon. Use norminv to transform to normal.\n%%\n');
                                 fprintf(fid,'PathPITMatrix = DataStructure.PathPITMatrix;\n');
                                 NumEntries = NumEntries+1;
                                 PathTxt = 'PathPITMatrix (matrix with the individual variable uniform PITs using the prediction paths for each forecast horizon), ';
                              else;
                                 PathTxt = '';
                              end;
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              NumEntries = NumEntries+1;
                              %
                              txt = ['The probability integral transform (PIT) estimates (uniform under the assumption of well-calibrated) under adaptive learning for the posterior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), PITMatrix (matrix with the individual variable uniform PITs and the joint predicted variable uniform PIT for each forecast horizon), ' BNTxt PathTxt 'and PredDateVector (vector with dates for the forecast horizon). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','Probability Integral Transform - Posterior Distribution',500,600,CurrINI);
                           end;
                           %
                           % write the results to a text file
                           %
                           if DSGEModel.BeliefSystem==1;
                              BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           else;
                              [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                              pause(0.2);
                              BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           end;
                           DSGEModel.BeliefSys = BeliefSys;
                           %
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\PITs-AdaptiveLearning-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                           PrintPITs(ResFile,'DSGE with learning',PredictionVariables,PredictionSampleStrMat,BNSampleStrMat,YDataStr(SelectedYData,:),PITMatrix,BNPITMatrix,PathPITMatrix,1,1,NumDraws,NumBurnin,NumPostDraws,[],CompTimeStepOne,CompTimeStepTwo,DSGEModel,CurrINI);
                           %
                           % check if file exists and display it then
                           %
                           if FileExist(ResFile)==1;
                              TextGUI(ResFile,['Probability Integral Transform - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,0);
                           end;
                        else;
                           CompTime = toc;
                           clear('thetaPostSample');
                        end;
                     elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                        clear('thetaPostSample');
                        txt = 'You didn''t select any variables to predict when computing the probability integral transform. YADA is therefore unable to help you!';
                        About(txt,'information','Probability Integral Transform - Posterior Distribution',150,500,CurrINI);
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconCRPSPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'CRPS - Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % load the data
               %
               if FileExist(DSGEModel.SystemPriorFile)==0;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               else;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               end;
               PriorData = load(PriorDrawsFile);
               thetaDraws = PriorData.thetaDraws;
               NumPriorDraws = size(thetaDraws,2);
               %
               % check if we have a system prior and if so, use a subset of the prior draws
               %
               OldDSGEModel = DSGEModel;
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  if DSGEModel.PosteriorSampler>7;
                     [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','CRPS - Prior Distribution','','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PostDrawsPercentValue = SelectedOption;
                     end;
                  else;
                     action = 'ok';
                  end;
                  if strcmp(lower(action),'ok')==1;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                     if length(UseDraws)<NumPriorDraws;
                        thetaDraws = thetaDraws(:,UseDraws);
                        NumPriorDraws = length(UseDraws);
                     end;
                  end;
               end;
               %
               % initialize the random number generator
               %
               if DSGEModel.RandomNumberValue==1;
                  randn('state',0);
                  rand('state',0);
               else;
                  randn('state',sum(100*clock));
                  rand('state',sum(100*clock));
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               UsePaths = 1;
               if NumPredPaths*NumPriorDraws>100000;
                  txt = ['The total number of paths that the CRPS calculations will use is equal ' int2str(NumPredPaths*NumDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
                  answer = Query(txt,'question',400,'CRPS - Posterior Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'no')==1;
                     UsePaths = 0;
                  end;
               end;
               %
               % check if we should continue
               %
               if UsePaths==1;
                  %
                  % determine the sample
                  %
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  DSGEModel.FirstPeriod = FirstPeriod;
                  DSGEModel.LastPeriod = LastPeriod;
                  %
                  % check how many time periods to use for predictive distributions
                  %
                  MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % check the maximum when we take NaNs into account
                  %
                  YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                  BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                  if isempty(DSGEModel.Actuals)==1;
                     SelectedYData = 1;
                     DataAction = 'ok';
                     YDataStr = 'Actuals from original sample';
                  else;
                     YDataStr = 'Actuals from original sample';
                     m = length(DSGEModel.Actuals);
                     for i=1:m;
                        YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                     end;
                     [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','CRPS - Prior Distribution','','',CurrINI);
                     if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                        YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                     end;
                  end;
               else;
                  DataAction = 'cancel';
               end;
               %
               if strcmp(lower(DataAction),'ok')==1;
                  MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                  [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                  MaxFH = min(MaxFH,MaxPH);
                  YData = YData(:,1:MaxFH);
                  %
                  % matrix with exogenous variables
                  %
                  X = [];
                  if DSGEModel.k>0;
                     if NumPredX<MaxPH;
                        X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                     end;
                     if NumPredX>0;
                        X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                     end;
                  end;
                  %
                  % selection of variable 
                  %
                  if isempty(DSGEModel.PredictionVariables)==1;
                     PredictionVariables = ones(1,DSGEModel.n);
                     PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                  else;
                     PredictionVariables = DSGEModel.PredictionVariables;
                  end;
                  %
                  [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                  if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                     DSGEModel.PredictionVariables = PredictionVariables;
                     %
                     % Determine the prediction sample
                     %
                     SampleStrMat = CreateFullSample(DSGEModel);
                     PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                     SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                     %
                     if gcbo==controls.learningtools.predictivedist.unconditional.original.crps.prior;
                        TransData = 0;
                        AnnualizeData = 0;
                        AnnualStr = 'original';
                        QueryHead = 'Predictive Distributions (CRPS) - Original Data';
                     elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.crps.prior;
                        TransData = 0;
                        AnnualizeData = 1;
                        AnnualStr = 'annual';
                        QueryHead = 'Predictive Distributions (CRPS) - Annualized Data';
                     end;
                     %
                     if PredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,[],0)==1;
                        txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                        tic;
                     else;
                        %
                        % ask if we should conduct prediction event calculations
                        %
                        txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                        answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                              DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                           end;
                           %
                           % make sure that the prediction event length does not exceed the maximum prediction horizon
                           %
                           for i=1:DSGEModel.n;
                              if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                 DSGEModel.PredictionEvent(i,3) = MaxPH;
                              end;
                           end;
                           %
                           % display a dialog for setting up an prediction event
                           %
                           [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                           if strcmp(lower(action),'ok')==1;
                              DSGEModel.PredictionEvent = PredictionEvent;
                              DSGEModel.RunPredictionEvent = 1;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPH
                        %
                        tic;
                        [DoneCalc,PredEventData,StateCov,ShockCov,MeasureCov,ParameterCov] = AdaptiveLearningDSGEPredictionPaths(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,[],0,[]);
                     end;
                     if DoneCalc==1;
                        %
                        % we can now compute the continuous ranked probability scores
                        %
                        CRPSMatrix = ComputeCRPSAdaptiveLearningDSGEPaths(YData,PredictionVariables,NumPriorDraws,NumPredPaths,MaxPH,AnnualizeData,[],0,[],DSGEModel,CurrINI);
                        CompTime = toc;
                        %
                        % check if the user wants to store the results to disk
                        %
                        txt = 'Would you like to save the continuous ranked probability score (CRPS) and energy score (ES) estimates under adaptive learning to file?';
                        answer = Query(txt,'question',300,'CRPS - Prior Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           file = FixFilePath([DSGEModel.OutputDirectory '\CRPS-AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                           YNames = DSGEModel.VariableNames;
                           save(file,'YNames','PredictionVariables','CRPSMatrix','PredDateVector');
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\CRPSAdaptiveLearning' int2str(DSGEModel.FixedBeliefsForProj) '_Prior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' int2str(AnnualizeData) '_' int2str(MaxPH) '_' int2str(NumPriorDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                           fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                           fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable CRPS and the joint energy score\n%%\n for each forecast horizon.\n%%\n');
                           fprintf(fid,'CRPSMatrix = DataStructure.CRPSMatrix;\n');
                           fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                           fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The continuous probability score (CRPS) estimates and the energy scores for the predicted variables based on the posterior draws under adaptive learning have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), and PredDateVector (vector with dates for the forecast horizon). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','CRPS - Prior Distribution',500,600,CurrINI);
                        end;
                        if DSGEModel.BeliefSystem==1;
                           BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                        else;
                           [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                           pause(0.2);
                           BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                        end;
                        DSGEModel.BeliefSys = BeliefSys;
                        %
                        % write the results to a text file
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\CRPS-AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                        PrintCRPS(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),CRPSMatrix,AnnualizeData,0,0,1,0,NumPriorDraws,0,NumPriorDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                        %
                        % check if file exists and display it then
                        %
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['CRPS & ES - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution',50,CurrINI,1,0);
                        end;
                     end;
                  elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                     txt = 'You didn''t select any variables to predict when computing the continuous ranked probability score. YADA is therefore unable to help you!';
                     About(txt,'information','CRPS - Prior Distribution',150,500,CurrINI);
                  end;
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the CRPS and ES calculations using the prior draws.'];
                About(txt,'information','CRPS - Prior Distribution',500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconCRPSPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','CRPS - Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws when estimating the continuous ranked probability score (CRPS)?';
                  answer = Query(txt,'question',400,'CRPS - Posterior Distribution',500,'no',CurrINI);
                  if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        NumDraws = length(UseDraws);
                     end;
                  else;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'CRPS - Posterior Distribution',Weights,[]);
                        if isempty(UseDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        Weights = Weights(UseDraws,1);
                        Weights = Weights*length(Weights)/sum(Weights);
                        NumDraws = length(UseDraws);
                     end;
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  UsePaths = 1;
                  if NumPredPaths*NumDraws>100000;
                     txt = ['The total number of paths that the CRPS calculations will use is equal ' int2str(NumPredPaths*NumDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
                     answer = Query(txt,'question',400,'CRPS - Posterior Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        UsePaths = 0;
                     end;
                  end;
                  %
                  % check if we should continue
                  %
                  if UsePaths==1;
                     %
                     % collect some required data
                     %
                     theta = ModeData.thetaMode;
                     thetaPositions = ModeData.thetaPositions;
                     thetaIndex = ModeData.thetaIndex;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     LowerBound = ModeData.LowerBound;
                     UniformBounds = ModeData.UniformBounds;
                     ModelParameters = ModeData.ModelParameters;
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        DataAction = 'ok';
                        YDataStr = 'Actuals from original sample';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','CRPS - Posterior Distribution','','',CurrINI);
                        if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                           BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                        end;
                     end;
                  else;
                     DataAction = 'cancel';
                  end;
                  %
                  if strcmp(lower(DataAction),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxPH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                        end;
                     end;
                     %
                     % selection of variable 
                     %
                     if isempty(DSGEModel.PredictionVariables)==1;
                        PredictionVariables = ones(1,DSGEModel.n);
                        PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                     else;
                        PredictionVariables = DSGEModel.PredictionVariables;
                     end;
                     %
                     [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                     if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                        DSGEModel.PredictionVariables = PredictionVariables;
                        %
                        % Determine the prediction sample
                        %
                        SampleStrMat = CreateFullSample(DSGEModel);
                        PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                        SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                        %
                        if gcbo==controls.learningtools.predictivedist.unconditional.original.crps.posterior;
                           TransData = 0;
                           AnnualizeData = 0;
                           AnnualStr = 'original';
                           QueryHead = 'Predictive Distributions (CRPS) - Original Data';
                        elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.crps.posterior;
                           TransData = 0;
                           AnnualizeData = 1;
                           AnnualStr = 'annual';
                           QueryHead = 'Predictive Distributions (CRPS) - Annualized Data';
                        end;
                        %
                        if PredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,MaxPH,NumDraws,NumPredPaths,AnnualizeData,TransData,CurrChain,1)==1;
                           txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           DoneCalc = 1;
                           tic;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPH
                           %
                           tic;
                           [DoneCalc,PredEventData,StateCov,ShockCov,MeasureCov,ParameterCov] = AdaptiveLearningDSGEPredictionPaths(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
                        end;
                        clear('thetaPostSample');
                        if DoneCalc==1;
                           %
                           % we can now compute the continuous ranked probability scores
                           %
                           CRPSMatrix = ComputeCRPSAdaptiveLearningDSGEPaths(YData,PredictionVariables,NumDraws,NumPredPaths,MaxPH,AnnualizeData,CurrChain,1,Weights,DSGEModel,CurrINI);
                           CompTime = toc;
                           %
                           % check if the user wants to store the results to disk
                           %
                           txt = 'Would you like to save the continuous ranked probability score (CRPS) and energy score (ES) estimates under adaptive learning to file?';
                           answer = Query(txt,'question',300,'CRPS - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              file = FixFilePath([DSGEModel.OutputDirectory '\CRPS-AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                              YNames = DSGEModel.VariableNames;
                              save(file,'YNames','PredictionVariables','CRPSMatrix','PredDateVector');
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\CRPSAdaptiveLearning' int2str(DSGEModel.FixedBeliefsForProj) '_Posterior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' int2str(AnnualizeData) '_' int2str(CurrChain) '_' int2str(MaxPH) '_' int2str(NumDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                              fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                              fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable CRPS and the joint energy score\n%%\n for each forecast horizon.\n%%\n');
                              fprintf(fid,'CRPSMatrix = DataStructure.CRPSMatrix;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The continuous probability score (CRPS) estimates and the energy scores for the predicted variables based on the posterior draws under adaptive learning have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), and PredDateVector (vector with dates for the forecast horizon). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','CRPS - Posterior Distribution',500,600,CurrINI);
                           end;
                           if DSGEModel.BeliefSystem==1;
                              BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           else;
                              [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                              pause(0.2);
                              BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           end;
                           DSGEModel.BeliefSys = BeliefSys;
                           %
                           % write the results to a text file
                           %
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\CRPS-AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                           PrintCRPS(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),CRPSMatrix,AnnualizeData,1,0,1,0,NumDraws,NumBurnin,NumPostDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                           %
                           % check if file exists and display it then
                           %
                           if FileExist(ResFile)==1;
                              TextGUI(ResFile,['CRPS & ES - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,0);
                           end;
                        else;
                           CompTime = toc;
                        end;
                     elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                        clear('thetaPostSample');
                        txt = 'You didn''t select any variables to predict when computing the continuous ranked probability score. YADA is therefore unable to help you!';
                        About(txt,'information','CRPS - Posterior Distribution',150,500,CurrINI);
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % check if we should check transformations functions
               %
               ErrorStr = '';
               if (isempty(ErrorStr)==1)&(gcbo==controls.learningtools.predictivedist.unconditional.transformed.initialvalues);
                  DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
                  if DTStatus==1;
                     [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                  else;
                     ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
                  end;
               end;
               if isempty(ErrorStr)==1;
                  %
                  % get sample data
                  %
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  %
                  % check how many time periods to use for predictive distributions
                  %
                  [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                  %
                  % matrix with exogenous variables
                  %
                  X = [];
                  if DSGEModel.k>0;
                     if NumPredX<MaxPH;
                        X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                     end;
                     if NumPredX>0;
                        X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                     end;
                  end;
                  %
                  % check if we should annualize or not
                  %
                  if gcbo==controls.learningtools.predictivedist.unconditional.original.initialvalues;
                     TransData = 0;
                     AnnualizeData = 0;
                     AnnualStr = 'original';
                  elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.initialvalues;
                     TransData = 0;
                     AnnualizeData = 1;
                     AnnualStr = 'annual';
                  elseif gcbo==controls.learningtools.predictivedist.unconditional.transformed.initialvalues;
                     TransData = 1;
                     AnnualizeData = 0;
                     AnnualStr = 'transformed';
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  %
                  % ask if we should conduct prediction event calculations
                  %
                  if NumPredPaths>1;
                     txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                     answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                        DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                     end;
                     %
                     % make sure that the prediction event length does not exceed the maximum prediction horizon
                     %
                     for i=1:DSGEModel.n;
                        if DSGEModel.PredictionEvent(i,3)>MaxPH;
                           DSGEModel.PredictionEvent(i,3) = MaxPH;
                        end;
                     end;
                     %
                     % display a dialog for setting up an prediction event
                     %
                     [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PredictionEvent = PredictionEvent;
                        DSGEModel.RunPredictionEvent = 1;
                     else;
                        DSGEModel.RunPredictionEvent = 0;
                     end;
                  else;
                     DSGEModel.RunPredictionEvent = 0;
                  end;
                  %
                  % we can now compute the predictive distribution for the endogenous variables
                  % up to period MaxPh
                  %
                  if DSGEModel.RandomNumberValue==1;
                     randn('state',0);
                  else;
                     randn('state',sum(100*clock));
                  end;
                  %
                  % run the unconditional forecasts
                  %
                  [PredPaths,PredData,PredEventData,YObsEventData,BeliefStatus,TransStatus,SolutionStatus] = AdaptiveLearningDSGEPredictionPathsTheta(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,' At Initial Values');
                  %
                  if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                     if isempty(PredPaths)==0;
                        %
                        % the computations went OK. Compute point estimates from predictive distribution and
                        % quantiles
                        %
                        [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                        %
                        % compute the mean error bias in percent
                        %
                        MeanErrorBias = max(max(abs((PredData.PredMean-MeanPrediction)./PredData.PredMean)));
                        if DSGEModel.AdjustPredictionPaths==0;
                           %
                           % ask if the prediction distribution should be corrected
                           %
                           txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust that predictive distribution such that it matches the theoretical mean?'];
                           answer = Query(txt,'question',140,'Mean Error Bias - Predictive Distributions',500,'yes',CurrINI);
                        else;
                           answer = 'Yes';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % adjust the quantiles to match
                           %
                           for i=1:length(Quantiles);
                              OlVal = Quantiles(i).Quantile;
                              Quantiles(i).Quantile = Quantiles(i).Quantile+(PredData.PredMean-MeanPrediction);
                           end;
                           MeanPrediction = PredData.PredMean;
                        end;
                        %
                        % compute the forecast errors based on the mean predictions
                        %
                        PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                        %
                        % check if the user wants to save the data
                        %
                        txt = ['Would you like to save the predictive distributions under adaptive learning for the ' AnnualStr ' data at the initial values to file?'];
                        answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                           file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-AdaptiveLearning-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-InitialValues.mat']);
                           YNames = DSGEModel.VariableNames;
                           DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                           Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           if (length(DateVector)>size(Y,2))&(TransData==0);
                              DateVector = DateVector(1,1:size(Y,2));
                           end;
                           EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                           EstimationStartPeriodOrig = EstimationStartPeriod;
                           YOrig = Y;
                           DateVectorOrig = DateVector;
                           if AnnualizeData==1;
                              Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                              EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                              DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                           end;
                           if TransData==1;
                              [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                              Y = YData';
                              if size(Y,2)==DSGEModel.T;
                                 Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 T2 = size(Y,2);
                                 if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                    Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 end;
                              end;
                           end;
                           PredMeanKsiStates = PredData.xiStates;
                           PredMean = PredData.PredMean;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           %
                           A = PredData.A;
                           H = PredData.H;
                           R = PredData.R;
                           MT1 = PredData.MT1;
                           FT1 = PredData.FT1;
                           BT1 = PredData.BT1;
                           %
                           ShockPaths = PredData.Shocks;
                           HistState = PredData.HistState;
                           MeasureErrorPaths = PredData.MeasError;
                           ShockNames = PredData.ShockNames;
                           KernelX = PredData.KernelX;
                           KernelY = PredData.KernelY;
                           if isempty(PredEventData)==1;
                              save(file,'A','H','R','MT1','FT1','BT1','PredPaths','ShockPaths','MeasureErrorPaths','HistState','MeanPrediction','PredMean','PredMeanKsiStates','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateVariableNames','ShockNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                           else;
                              save(file,'A','H','R','MT1','FT1','BT1','PredPaths','ShockPaths','MeasureErrorPaths','HistState','MeanPrediction','PredMean','PredMeanKsiStates','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateVariableNames','ShockNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData','YObsEventData');
                           end;
                           %
                           % create a matlab m-file to read the output.
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\PDAL' AnnualStr num2str(MaxPH,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod 'InitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% M E A S U R E M E N T   E Q U A T I O N\n');
                           fprintf(fid,'%%\n%% matrix with parameters on exogenous variables\n%%\n');
                           fprintf(fid,'A = DataStructure.A;\n');
                           fprintf(fid,'%%\n%% matrix with parameters on state variables\n%%\n');
                           fprintf(fid,'H = DataStructure.H;\n');
                           fprintf(fid,'%%\n%% matrix with measurement error covariances\n%%\n');
                           fprintf(fid,'R = DataStructure.R;\n');
                           fprintf(fid,'%%\n%% S T A T E   E Q U A T I O N\n');
                           fprintf(fid,'%%\n%% matrix with parameters on deterministic variables during the forecast sample\n%%\n');
                           fprintf(fid,'MT1 = DataStructure.MT1;\n');
                           fprintf(fid,'%%\n%% matrix with state transition parameters during the forecast sample\n%%\n');
                           fprintf(fid,'FT1 = DataStructure.FT1;\n');
                           fprintf(fid,'%%\n%% matrix with shock parameters during the forecast sample\n%%\n');
                           fprintf(fid,'BT1 = DataStructure.BT1;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the state variables variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%%  3D matrix with all the prediction paths for the observed variables\n%%\n');
                           fprintf(fid,'PredPaths = DataStructure.PredPaths;\n');
                           fprintf(fid,'%%\n%% Underlying draws of the economic shocks for all paths\n%%\n');
                           fprintf(fid,'ShockPaths = DataStructure.ShockPaths;\n');
                           fprintf(fid,'%%\n%% Underlying draws of the neasurement errors for all paths\n%%\n');
                           fprintf(fid,'MeasureErrorPaths = DataStructure.MeasureErrorPaths;\n');
                           fprintf(fid,'%%\n%% Underlying draws of the state variables before the predictions\n%%\n');
                           fprintf(fid,'HistState = DataStructure.HistState;\n');
                           fprintf(fid,'%%\n%% matrix with the sample mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                           fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the observed variables\n%%\n');
                           fprintf(fid,'PredMean = DataStructure.PredMean;\n');
                           fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the state variables\n%%\n');
                           fprintf(fid,'PredMeanKsiStates = DataStructure.PredMeanKsiStates;\n');
                           fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                           fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                           fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                           if isempty(KernelX)==0;
                              fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                              fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                              KStr = 'KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), ';
                              KVal = 2;
                           else;
                              KStr = '';
                              KVal = 0;
                           end;
                           fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                           fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                           fprintf(fid,'%%\n%% observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           if isempty(PredEventData)==0;
                              fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                              fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                              fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                              fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                              fprintf(fid,'%%\n%% matrix with observed event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                              fprintf(fid,'YObsEventData = DataStructure.YObsEventData;\n');
                              NumEntries = int2str(24+KVal);
                              ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true prediction events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), YObsEventData (matrix with the number of true events for prediction paths whose mean is the observed data, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared of deviations from lower bound when below lower bound, the sum and sum of squared of deviations from upper bound when above upper bound), ';
                           else;
                              NumEntries = int2str(21+KVal);
                              ExtraEntry = '';
                           end;
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: the 6 state space matrices (A, H, R, MT1, FT1, BT1), YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), ShockNames (string matrix with the names of the economic shocks), PredPaths (3D matrix with the prediction paths for the observed variables), ShockPaths (3D matrix with the underlying economic shocks for the prediction paths), MeasureErrorPaths (3D matrix with the underlying draws of the measurement errors for the prediction paths), HistState (matrix with the underlying draws of the state variables from the period prior to the predictions), MeanPrediction (matrix with the sample mean predictions), PredMean (matrix with the population mean predictions), PredMeanKsiStates (matrix with the population mean state predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), ' KStr 'PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Predictive Distributions - Initial Values',700,600,CurrINI);
                        end;
                        if isempty(PredEventData)==0;
                           %
                           % write prediction event results to file
                           %
                           file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-AdaptiveLearning-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-InitialValues.txt']);
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,YObsEventData,DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                           %
                           % displays the file if it exists
                           %
                           if FileExist(file)==1;
                              TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,0);
                           end;
                        end;
                        %
                        if size(PredData.A,1)==1;
                           ObsVarMean = PredData.A';
                        else;
                           ObsVarMean = [];
                        end;
                        %
                        % Open a dialog for displaying predictive distributions
                        %
                        PredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,ObsVarMean,Quantiles,PredData.KernelX,PredData.KernelY,'Initial Values','DSGE',AnnualStr,1);
                     end;
                  elseif TransStatus==0;
                     txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the predictive distributions based on the initial values.';
                     About(txt,'information','Predictive Distributions - Initial Values',500,600,CurrINI);
                  elseif BeliefStatus==0;
                     txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the predictive distributions based on the initial values.';
                     About(txt,'information','Predictive Distributions - Initial Values',500,600,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','Missing Transformation Information',500,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the predictive distributions based on the initial values.'];
                About(txt,'information','Predictive Distributions - Initial Values',500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredDSGEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.learningtools.predictivedist.unconditional.transformed.dsgemode);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % get sample data
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               %
               % check how many time periods to use for predictive distributions
               %
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                  end;
               end;
               %
               % check if we should annualize or not
               %
               if gcbo==controls.learningtools.predictivedist.unconditional.original.dsgemode;
                  TransData = 0;
                  AnnualizeData = 0;
                  AnnualStr = 'original';
               elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.dsgemode;
                  TransData = 0;
                  AnnualizeData = 1;
                  AnnualStr = 'annual';
               elseif gcbo==controls.learningtools.predictivedist.unconditional.transformed.dsgemode;
                  TransData = 1;
                  AnnualizeData = 0;
                  AnnualStr = 'transformed';
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % ask if we should conduct prediction event calculations
               %
               if NumPredPaths>1;
                  txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                  answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                     DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                  end;
                  %
                  % make sure that the prediction event length does not exceed the maximum prediction horizon
                  %
                  for i=1:DSGEModel.n;
                     if DSGEModel.PredictionEvent(i,3)>MaxPH;
                        DSGEModel.PredictionEvent(i,3) = MaxPH;
                     end;
                  end;
                  %
                  % display a dialog for setting up an prediction event
                  %
                  [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PredictionEvent = PredictionEvent;
                     DSGEModel.RunPredictionEvent = 1;
                  else;
                     DSGEModel.RunPredictionEvent = 0;
                  end;
               else;
                  DSGEModel.RunPredictionEvent = 0;
               end;
               %
               % we can now compute the predictive distribution for the endogenous variables
               % up to period MaxPh
               %
               if DSGEModel.RandomNumberValue==1;
                  randn('state',0);
               else;
                  randn('state',sum(100*clock));
               end;
               %
               % run the unconditional forecasts
               %
               [PredPaths,PredData,PredEventData,YObsEventData,BeliefStatus,TransStatus] = AdaptiveLearningDSGEPredictionPathsTheta(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,' At DSGE Posterior Mode');
               %
               if (BeliefStatus==1)&(TransStatus==1);
                  if isempty(PredPaths)==0;
                     %
                     % the computations went OK. Compute point estimates from predictive distribution and
                     % quantiles
                     %
                     [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                     %
                     % compute the mean error bias in percent
                     %
                     MeanErrorBias = max(max(abs((PredData.PredMean-MeanPrediction)./PredData.PredMean)));
                     if DSGEModel.AdjustPredictionPaths==0;
                        %
                        % ask if the prediction distribution should be corrected
                        %
                        txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust that predictive distribution such that it matches the theoretical mean?'];
                        answer = Query(txt,'question',140,'Mean Error Bias - Predictive Distributions',500,'yes',CurrINI);
                     else;
                        answer = 'Yes';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % adjust the quantiles to match
                        %
                        for i=1:length(Quantiles);
                           OlVal = Quantiles(i).Quantile;
                           Quantiles(i).Quantile = Quantiles(i).Quantile+(PredData.PredMean-MeanPrediction);
                        end;
                        MeanPrediction = PredData.PredMean;
                     end;
                     %
                     % compute the forecast errors based on the mean predictions
                     %
                     PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                     %
                     % check if the user wants to save the data
                     %
                     txt = ['Would you like to save the predictive distributions under adaptive learning for the ' AnnualStr ' data at the DSGE model posterior mode to file?'];
                     answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                        file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-AdaptiveLearning-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-DSGEPosteriorMode.mat']);
                        YNames = DSGEModel.VariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                        Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        if (length(DateVector)>size(Y,2))&(TransData==0);
                           DateVector = DateVector(1,1:size(Y,2));
                        end;
                        EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                        EstimationStartPeriodOrig = EstimationStartPeriod;
                        YOrig = Y;
                        DateVectorOrig = DateVector;
                        if AnnualizeData==1;
                           Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                           EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                           DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                        end;
                        if TransData==1;
                           [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                           Y = YData';
                           if size(Y,2)==DSGEModel.T;
                              Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           else;
                              T2 = size(Y,2);
                              if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                 Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           end;
                        end;
                        PredMeanKsiStates = PredData.xiStates;
                        PredMean = PredData.PredMean;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        %
                        A = PredData.A;
                        H = PredData.H;
                        R = PredData.R;
                        MT1 = PredData.MT1;
                        FT1 = PredData.FT1;
                        BT1 = PredData.BT1;
                        %
                        ShockPaths = PredData.Shocks;
                        HistState = PredData.HistState;
                        MeasureErrorPaths = PredData.MeasError;
                        ShockNames = PredData.ShockNames;
                        KernelX = PredData.KernelX;
                        KernelY = PredData.KernelY;
                        if isempty(PredEventData)==1;
                           save(file,'A','H','R','MT1','FT1','BT1','PredPaths','ShockPaths','MeasureErrorPaths','HistState','MeanPrediction','PredMean','PredMeanKsiStates','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateVariableNames','ShockNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                        else;
                           save(file,'A','H','R','MT1','FT1','BT1','PredPaths','ShockPaths','MeasureErrorPaths','HistState','MeanPrediction','PredMean','PredMeanKsiStates','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateVariableNames','ShockNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData','YObsEventData');
                        end;
                        %
                        % create a matlab m-file to read the output.
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\PDAL' AnnualStr num2str(MaxPH,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod 'DSGEMode.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% M E A S U R E M E N T   E Q U A T I O N\n');
                        fprintf(fid,'%%\n%% matrix with parameters on exogenous variables\n%%\n');
                        fprintf(fid,'A = DataStructure.A;\n');
                        fprintf(fid,'%%\n%% matrix with parameters on state variables\n%%\n');
                        fprintf(fid,'H = DataStructure.H;\n');
                        fprintf(fid,'%%\n%% matrix with measurement error covariances\n%%\n');
                        fprintf(fid,'R = DataStructure.R;\n');
                        fprintf(fid,'%%\n%% S T A T E   E Q U A T I O N\n');
                        fprintf(fid,'%%\n%% matrix with parameters on deterministic variables during the forecast sample\n%%\n');
                        fprintf(fid,'MT1 = DataStructure.MT1;\n');
                        fprintf(fid,'%%\n%% matrix with state transition parameters during the forecast sample\n%%\n');
                        fprintf(fid,'FT1 = DataStructure.FT1;\n');
                        fprintf(fid,'%%\n%% matrix with shock parameters during the forecast sample\n%%\n');
                        fprintf(fid,'BT1 = DataStructure.BT1;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the state variables variables\n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                        fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                        fprintf(fid,'%%\n%%  3D matrix with all the prediction paths for the observed variables\n%%\n');
                        fprintf(fid,'PredPaths = DataStructure.PredPaths;\n');
                        fprintf(fid,'%%\n%% Underlying draws of the economic shocks for all paths\n%%\n');
                        fprintf(fid,'ShockPaths = DataStructure.ShockPaths;\n');
                        fprintf(fid,'%%\n%% Underlying draws of the neasurement errors for all paths\n%%\n');
                        fprintf(fid,'MeasureErrorPaths = DataStructure.MeasureErrorPaths;\n');
                        fprintf(fid,'%%\n%% Underlying draws of the state variables before the predictions\n%%\n');
                        fprintf(fid,'HistState = DataStructure.HistState;\n');
                        fprintf(fid,'%%\n%% matrix with the sample mean predictions of the observed variables\n%%\n');
                        fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                        fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the observed variables\n%%\n');
                        fprintf(fid,'PredMean = DataStructure.PredMean;\n');
                        fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the state variables\n%%\n');
                        fprintf(fid,'PredMeanKsiStates = DataStructure.PredMeanKsiStates;\n');
                        fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                        fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                        fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                        fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                        if isempty(KernelX)==0;
                           fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                           fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                           fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                           fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                           KStr = 'KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), ';
                           KVal = 2;
                        else;
                           KStr = '';
                           KVal = 0;
                        end;
                        fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                        fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                        fprintf(fid,'%%\n%% observed variables\n%%\n');
                        fprintf(fid,'Y = DataStructure.Y;\n');
                        fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        if isempty(PredEventData)==0;
                           fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                           fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                           fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                           fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                           fprintf(fid,'%%\n%% matrix with observed event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                           fprintf(fid,'YObsEventData = DataStructure.YObsEventData;\n');
                           NumEntries = int2str(24+KVal);
                           ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true prediction events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), YObsEventData (matrix with the number of true events for prediction paths whose mean is the observed data, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared of deviations from lower bound when below lower bound, the sum and sum of squared of deviations from upper bound when above upper bound), ';
                        else;
                           NumEntries = int2str(21+KVal);
                           ExtraEntry = '';
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        txt = ['The predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: the 6 state space matrices (A, H, R, MT1, FT1, BT1), YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), ShockNames (string matrix with the names of the economic shocks), PredPaths (3D matrix with the prediction paths for the observed variables), ShockPaths (3D matrix with the underlying economic shocks for the prediction paths), MeasureErrorPaths (3D matrix with the underlying draws of the measurement errors for the prediction paths), HistState (matrix with the underlying draws of the state variables from the period prior to the predictions), MeanPrediction (matrix with the sample mean predictions), PredMean (matrix with the population mean predictions), PredMeanKsiStates (matrix with the population mean state predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), ' KStr 'PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Predictive Distributions - DSGE Posterior Mode',700,600,CurrINI);
                     end;
                     if isempty(PredEventData)==0;
                        %
                        % write prediction event results to file
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-AdaptiveLearning-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-DSGEPosteriorMode.txt']);
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                        PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,YObsEventData,DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                        %
                        % displays the file if it exists
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'DSGE Posterior Mode Parameter Values',50,CurrINI,1,0);
                        end;
                     end;
                     %
                     if size(PredData.A,1)==1;
                        ObsVarMean = PredData.A';
                     else;
                        ObsVarMean = [];
                     end;
                     %
                     % Open a dialog for displaying predictive distributions
                     %
                     PredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,ObsVarMean,Quantiles,PredData.KernelX,PredData.KernelY,'DSGE Posterior Mode','DSGE',AnnualStr,1);
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the predictive distribution based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Predictive Distributions - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the predictive distribution based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Predictive Distributions - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the predictive distribution based on the DSGE model posterior mode estimates.'];
               About(txt,'information','Predictive Distributions - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.learningtools.predictivedist.unconditional.transformed.postmode);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % get sample data
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % check if we should annualize or not
            %
            if gcbo==controls.learningtools.predictivedist.unconditional.original.postmode;
               TransData = 0;
               AnnualizeData = 0;
               AnnualStr = 'original';
            elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.postmode;
               TransData = 0;
               AnnualizeData = 1;
               AnnualStr = 'annual';
            elseif gcbo==controls.learningtools.predictivedist.unconditional.transformed.postmode;
               TransData = 1;
               AnnualizeData = 0;
               AnnualStr = 'transformed';
            end;
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % ask if we should conduct prediction event calculations
            %
            if NumPredPaths>1;
               txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
               answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                  DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
               end;
               %
               % make sure that the prediction event length does not exceed the maximum prediction horizon
               %
               for i=1:DSGEModel.n;
                  if DSGEModel.PredictionEvent(i,3)>MaxPH;
                     DSGEModel.PredictionEvent(i,3) = MaxPH;
                  end;
               end;
               %
               % display a dialog for setting up an prediction event
               %
               [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.PredictionEvent = PredictionEvent;
                  DSGEModel.RunPredictionEvent = 1;
               else;
                  DSGEModel.RunPredictionEvent = 0;
               end;
            else;
               DSGEModel.RunPredictionEvent = 0;
            end;
            %
            % we can now compute the predictive distribution for the endogenous variables
            % up to period MaxPh
            %
            if DSGEModel.RandomNumberValue==1;
               randn('state',0);
            else;
               randn('state',sum(100*clock));
            end;
            %
            [PredPaths,PredData,PredEventData,YObsEventData,BeliefStatus,TransStatus] = AdaptiveLearningDSGEPredictionPathsTheta(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,' At Posterior Mode');
            if (BeliefStatus==1)&(TransStatus==1);
               if isempty(PredPaths)==0;
                  %
                  % the computations went OK. Compute point estimates from predictive distribution and
                  % quantiles
                  %
                  [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                  %
                  % compute the mean error bias in percent
                  %
                  MeanErrorBias = max(max(abs((PredData.PredMean-MeanPrediction)./PredData.PredMean)));
                  if DSGEModel.AdjustPredictionPaths==0;
                     %
                     % ask if the prediction distribution should be corrected
                     %
                     txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust that predictive distribution such that it matches the theoretical mean?'];
                     answer = Query(txt,'question',140,'Mean Error Bias - Predictive Distributions',500,'yes',CurrINI);
                  else;
                     answer = 'Yes';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     %
                     % adjust the quantiles to match
                     %
                     for i=1:length(Quantiles);
                        OlVal = Quantiles(i).Quantile;
                        Quantiles(i).Quantile = Quantiles(i).Quantile+(PredData.PredMean-MeanPrediction);
                     end;
                     MeanPrediction = PredData.PredMean;
                  end;
                  %
                  % compute the forecast errors based on the mean predictions
                  %
                  PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                  %
                  % check if the user wants to save the data
                  %
                  txt = ['Would you like to save the predictive distributions under adaptive learning for the ' AnnualStr ' data at posterior mode to file?'];
                  answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                     file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-AdaptiveLearning-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-PosteriorMode.mat']);
                     YNames = DSGEModel.VariableNames;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                     Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                     if (length(DateVector)>size(Y,2))&(TransData==0);
                        DateVector = DateVector(1,1:size(Y,2));
                     end;
                     EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                     EstimationStartPeriodOrig = EstimationStartPeriod;
                     YOrig = Y;
                     DateVectorOrig = DateVector;
                     if AnnualizeData==1;
                        Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                        EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                        DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                     end;
                     if TransData==1;
                        [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                        Y = YData';
                        if size(Y,2)==DSGEModel.T;
                           Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        else;
                           T2 = size(Y,2);
                           if (FirstPeriod-(DSGEModel.T-T2))>=1;
                              Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           else;
                              Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                              DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                           end;
                        end;
                     end;
                     PredMeanKsiStates = PredData.xiStates;
                     PredMean = PredData.PredMean;
                     StateVariableNames = DSGEModel.StateVariableNames;
                     %
                     A = PredData.A;
                     H = PredData.H;
                     R = PredData.R;
                     MT1 = PredData.MT1;
                     FT1 = PredData.FT1;
                     BT1 = PredData.BT1;
                     %
                     ShockPaths = PredData.Shocks;
                     HistState = PredData.HistState;
                     MeasureErrorPaths = PredData.MeasError;
                     ShockNames = PredData.ShockNames;
                     KernelX = PredData.KernelX;
                     KernelY = PredData.KernelY;
                     if isempty(PredEventData)==1;
                        save(file,'A','H','R','MT1','FT1','BT1','PredPaths','ShockPaths','MeasureErrorPaths','HistState','MeanPrediction','PredMean','PredMeanKsiStates','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateVariableNames','ShockNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                     else;
                        save(file,'A','H','R','MT1','FT1','BT1','PredPaths','ShockPaths','MeasureErrorPaths','HistState','MeanPrediction','PredMean','PredMeanKsiStates','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateVariableNames','ShockNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData','YObsEventData');
                     end;
                     %
                     % create a matlab m-file to read the output.
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\PDAL' AnnualStr num2str(MaxPH,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod 'PostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% M E A S U R E M E N T   E Q U A T I O N\n');
                     fprintf(fid,'%%\n%% matrix with parameters on exogenous variables\n%%\n');
                     fprintf(fid,'A = DataStructure.A;\n');
                     fprintf(fid,'%%\n%% matrix with parameters on state variables\n%%\n');
                     fprintf(fid,'H = DataStructure.H;\n');
                     fprintf(fid,'%%\n%% matrix with measurement error covariances\n%%\n');
                     fprintf(fid,'R = DataStructure.R;\n');
                     fprintf(fid,'%%\n%% S T A T E   E Q U A T I O N\n');
                     fprintf(fid,'%%\n%% matrix with parameters on deterministic variables during the forecast sample\n%%\n');
                     fprintf(fid,'MT1 = DataStructure.MT1;\n');
                     fprintf(fid,'%%\n%% matrix with state transition parameters during the forecast sample\n%%\n');
                     fprintf(fid,'FT1 = DataStructure.FT1;\n');
                     fprintf(fid,'%%\n%% matrix with shock parameters during the forecast sample\n%%\n');
                     fprintf(fid,'BT1 = DataStructure.BT1;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStructure.YNames;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the state variables variables\n%%\n');
                     fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                     fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                     fprintf(fid,'%%\n%%  3D matrix with all the prediction paths for the observed variables\n%%\n');
                     fprintf(fid,'PredPaths = DataStructure.PredPaths;\n');
                     fprintf(fid,'%%\n%% Underlying draws of the economic shocks for all paths\n%%\n');
                     fprintf(fid,'ShockPaths = DataStructure.ShockPaths;\n');
                     fprintf(fid,'%%\n%% Underlying draws of the neasurement errors for all paths\n%%\n');
                     fprintf(fid,'MeasureErrorPaths = DataStructure.MeasureErrorPaths;\n');
                     fprintf(fid,'%%\n%% Underlying draws of the state variables before the predictions\n%%\n');
                     fprintf(fid,'HistState = DataStructure.HistState;\n');
                     fprintf(fid,'%%\n%% matrix with the sample mean predictions of the observed variables\n%%\n');
                     fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                     fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the observed variables\n%%\n');
                     fprintf(fid,'PredMean = DataStructure.PredMean;\n');
                     fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the state variables\n%%\n');
                     fprintf(fid,'PredMeanKsiStates = DataStructure.PredMeanKsiStates;\n');
                     fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                     fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                     fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                     fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                     if isempty(KernelX)==0;
                        fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                        fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                        fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                        fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                        KStr = 'KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), ';
                        KVal = 2;
                     else;
                        KStr = '';
                        KVal = 0;
                     end;
                     fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                     fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                     fprintf(fid,'%%\n%% observed variables\n%%\n');
                     fprintf(fid,'Y = DataStructure.Y;\n');
                     fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     if isempty(PredEventData)==0;
                        fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                        fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                        fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                        fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                        fprintf(fid,'%%\n%% matrix with observed event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                        fprintf(fid,'YObsEventData = DataStructure.YObsEventData;\n');
                        NumEntries = int2str(24+KVal);
                        ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true prediction events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), YObsEventData (matrix with the number of true events for prediction paths whose mean is the observed data, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared of deviations from lower bound when below lower bound, the sum and sum of squared of deviations from upper bound when above upper bound), ';
                     else;
                        NumEntries = int2str(21+KVal);
                        ExtraEntry = '';
                     end;
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     %
                     txt = ['The predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: the 6 state space matrices (A, H, R, MT1, FT1, BT1), YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), ShockNames (string matrix with the names of the economic shocks), PredPaths (3D matrix with the prediction paths for the observed variables), ShockPaths (3D matrix with the underlying economic shocks for the prediction paths), MeasureErrorPaths (3D matrix with the underlying draws of the measurement errors for the prediction paths), HistState (matrix with the underlying draws of the state variables from the period prior to the predictions), MeanPrediction (matrix with the sample mean predictions), PredMean (matrix with the population mean predictions), PredMeanKsiStates (matrix with the population mean state predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), ' KStr 'PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Predictive Distributions - Posterior Mode',700,600,CurrINI);
                  end;
                  if isempty(PredEventData)==0;
                     %
                     % write prediction event results to file
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveDist-AdaptiveLearning-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-PosteriorMode.txt']);
                     [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                     PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,YObsEventData,DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                     %
                     % displays the file if it exists
                     %
                     if FileExist(file)==1;
                        TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,0);
                     end;
                  end;
                  %
                  if size(PredData.A,1)==1;
                     ObsVarMean = PredData.A';
                  else;
                     ObsVarMean = [];
                  end;
                  %
                  % Open a dialog for displaying predictive distributions
                  %
                  PredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,ObsVarMean,Quantiles,PredData.KernelX,PredData.KernelY,'Posterior Mode','DSGE',AnnualStr,1);
               end;
            elseif TransStatus==0;
               txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the predictive distribution based on the posterior mode estimates.';
               About(txt,'information','Predictive Distributions - Posterior Mode',300,600,CurrINI);
            elseif BeliefStatus==0;
               txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the predictive distribution based on the posterior mode estimates.';
               About(txt,'information','Predictive Distributions - Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % check if we should check transformations functions
         %
         ErrorStr = '';
         if gcbo==controls.learningtools.predictivedist.unconditional.transformed.prior;
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            %
            % are we using annualized data, transformed or the original
            %
            if gcbo==controls.learningtools.predictivedist.unconditional.original.prior;
               TransData = 0;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'Predictive Distributions - Original Data';
            elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.prior;
               TransData = 0;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'Predictive Distributions - Annualized Data';
            elseif gcbo==controls.learningtools.predictivedist.unconditional.transformed.prior;
               TransData = 1;
               AnnualizeData = 0;
               AnnualStr = 'transformed';
               QueryHead = 'Predictive Distributions - Transformed Data';
            end;
            %
            % check if the posterior mode file exists
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  if FileExist(DSGEModel.PriorFile)==1;
                     txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                     answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                  else;
                     answer = 'Yes';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     InitStatus = 1;
                     theta = ModeData.theta;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     LowerBound = ModeData.LowerBound;
                     ModelParameters = ModeData.ModelParameters;
                     thetaPositions = ModeData.thetaPositions;
                     thetaIndex = ModeData.thetaIndex;
                     UniformBounds = ModeData.UniformBounds;
                  else;
                     [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                  end;
               else;
                  InitStatus = 0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            else;
               [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            end;
            if InitStatus==1;
               %
               % get the necessary information about the adaptive learning parameters
               %
               [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
               if status==1;
                  %
                  % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
                  % data from the DSGE model with its posterior mode estimates
                  %
                  [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                   thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                   LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
                  %
                  % load the data
                  %
                  if FileExist(DSGEModel.SystemPriorFile)==0;
                     PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
                  else;
                     PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
                  end;
                  PriorData = load(PriorDrawsFile);
                  thetaDraws = PriorData.thetaDraws;
                  NumPriorDraws = size(thetaDraws,2);
                  %
                  % check if we have a system prior and if so, use a subset of the prior draws
                  %
                  OldDSGEModel = DSGEModel;
                  if FileExist(DSGEModel.SystemPriorFile)==1;
                     if DSGEModel.PosteriorSampler>7;
                        [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection',QueryHead,'','',CurrINI);
                        if strcmp(lower(action),'ok')==1;
                           DSGEModel.PostDrawsPercentValue = SelectedOption;
                        end;
                     else;
                        action = 'ok';
                     end;
                     if strcmp(lower(action),'ok')==1;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                        if length(UseDraws)<NumPriorDraws;
                           thetaDraws = thetaDraws(:,UseDraws);
                           NumPriorDraws = length(UseDraws);
                        end;
                     end;
                  end;
                  %
                  % initialize the random number generator
                  %
                  if DSGEModel.RandomNumberValue==1;
                     randn('state',0);
                     rand('state',0);
                  else;
                     randn('state',sum(100*clock));
                     rand('state',sum(100*clock));
                  end;
                  %
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  %
                  % check how many time periods to use for predictive distributions
                  %
                  [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                  %
                  % matrix with exogenous variables
                  %
                  X = [];
                  if DSGEModel.k>0;
                     if NumPredX<MaxPH;
                        X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                     end;
                     if NumPredX>0;
                        X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                     end;
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  %
                  %
                  if PredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,[],0)==1;
                     txt = 'YADA has located results under adaptive learning for the predictive distribution on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     [PredEventData,PredictionEvent,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = LoadAdaptiveLearningDSGEPredictionUncertainty(DSGEModel.OutputDirectory,'','',DSGEModel.NameOfModel,DSGEModel.FixedBeliefsForProj,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,[],0);
                     DoneCalc = 1;
                  else;
                     %
                     % ask if we should conduct prediction event calculations
                     %
                     txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                     answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                           DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                        end;
                        %
                        % make sure that the prediction event length does not exceed the maximum prediction horizon
                        %
                        for i=1:DSGEModel.n;
                           if DSGEModel.PredictionEvent(i,3)>MaxPH;
                              DSGEModel.PredictionEvent(i,3) = MaxPH;
                           end;
                        end;
                        %
                        % display a dialog for setting up an prediction event
                        %
                        [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                        if strcmp(lower(action),'ok')==1;
                           DSGEModel.PredictionEvent = PredictionEvent;
                           DSGEModel.RunPredictionEvent = 1;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                     else;
                        DSGEModel.RunPredictionEvent = 0;
                     end;
                     %
                     % we can now compute the predictive distribution for the endogenous variables
                     % up to period MaxPH
                     %
                     [DoneCalc,PredEventData,StateCov,ShockCov,MeasureCov,ParameterCov] = AdaptiveLearningDSGEPredictionPaths(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,[],0,[]);
                  end;
                  %
                  % check if we should show any results
                  %
                  if DoneCalc==1;
                     %
                     % yes, now we need to calculate some distribution properties
                     %
                     [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = AdaptiveLearningDSGEPredictionDistribution(DSGEModel.n,DSGEModel.OutputDirectory,'','',DSGEModel.NameOfModel,DSGEModel.FixedBeliefsForProj,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,[],0,[],DSGEModel.Percentiles,DSGEModel.VariableNames,DSGEModel.ConfidenceRegionMethod,CurrINI);
                     %
                     % compute the forecast errors based on the mean predictions
                     %
                     PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                     %
                     % check if the user wants to save the data
                     %
                     txt = ['Would you like to save the adaptive learning based predictive distributions for the ' AnnualStr ' data to file?'];
                     answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                        file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningPriorPredictiveDist-' int2str(DSGEModel.FixedBeliefsForProj) '-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPriorDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                        YNames = DSGEModel.VariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                        Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        if (length(DateVector)>size(Y,2))&(TransData==0);
                           DateVector = DateVector(1,1:size(Y,2));
                        end;
                        EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                        EstimationStartPeriodOrig = EstimationStartPeriod;
                        YOrig = Y;
                        DateVectorOrig = DateVector;
                        if AnnualizeData==1;
                           Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                           EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                           DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                        end;
                        if TransData==1;
                           [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                           Y = YData';
                           if size(Y,2)==DSGEModel.T;
                              Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           else;
                              T2 = size(Y,2);
                              if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                 Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           end;
                        end;
                        if isempty(PredEventData)==1;
                           save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateCov','ShockCov','MeasureCov','ParameterCov','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                        else;
                           save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateCov','ShockCov','MeasureCov','ParameterCov','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData');
                        end;
                        %
                        % create a matlab m-file to read the output.
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\ALPriorPD' int2str(DSGEModel.FixedBeliefsForProj) AnnualStr num2str(MaxPH,'%0.0f') num2str(NumPriorDraws,'%0.0f') num2str(NumPredPaths,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                        fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                        fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                        fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                        fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                        fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                        fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                        fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                        fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                        fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                        fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                        fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                        fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                        fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                        fprintf(fid,'%%\n%% observed variables\n%%\n');
                        fprintf(fid,'Y = DataStructure.Y;\n');
                        fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        if (AnnualizeData==0)&(TransData==0);
                           fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to state uncertainty\n%%\n');
                           fprintf(fid,'StateCov = DataStructure.StateCov;\n');
                           fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to shock uncertainty\n%%\n');
                           fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                           fprintf(fid,'%%\n%% 2D Error covariance matrix (nxn) part due to measurement error uncertainty\n%%\n');
                           fprintf(fid,'MeasureCov = DataStructure.MeasureCov;\n');
                           fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                           fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                           CovDecompStr = 'StateCov (3D error covariance matrix with part due to state uncertainty), ShockCov (3D error covariance matrix with part due to shock uncertainty), MeasureCov (2D error covariance matrix with part due to measurement error uncertainty), ParameterCov (3D covariance matrix with part due to parameter uncertainty), ';
                        else;
                           CovDecompStr = '';
                        end;
                        if isempty(PredEventData)==0;
                           fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                           fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                           fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                           fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                           if (AnnualizeData==0)&(TransData==0);
                              NumEntries = '16';
                           else;
                              NumEntries = '12';
                           end;
                           ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                        else;
                           if (AnnualizeData==0)&(TransData==0);
                              NumEntries = '14';
                           else;
                              NumEntries = '10';
                           end;
                           ExtraEntry = '';
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        txt = ['The predictive distributions under adaptive learning and based on the prior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Predictive Distributions',500,600,CurrINI);
                     end;
                     if isempty(PredEventData)==0;
                        %
                        % write prediction event results to file
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningPriorPredictiveDist-' int2str(DSGEModel.FixedBeliefsForProj) '-' AnnualStr '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPriorDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                        PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPriorDraws,CurrINI);
                        %
                        % displays the file if it exists
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                        end;
                     end;
                     %
                     % summarize the information in the uncertainty variables more conveniently
                     %
                     if isempty(StateCov)==0;
                        [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(StateCov,ShockCov,MeasureCov,ParameterCov);
                        %
                        % Open a dialog for displaying predictive distributions
                        %
                        UData.StateVar = StateVar;
                        UData.ShockVar = ShockVar;
                        UData.MeasureVar = MeasureVar;
                        UData.ParameterVar = ParameterVar;
                        [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                     else;
                        UData = [];
                     end;
                     PredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,[],Quantiles,KernelX,KernelY,'','DSGE',AnnualStr,0,UData);
                  end;
               else;
                   txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the predictive distributions based on the prior draws.'];
                   About(txt,'information',QueryHead,500,600,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.learningtools.predictivedist.unconditional.transformed.posterior);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            thetaMode = ModeData.thetaMode;
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Predictive Distributions','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % initialize the random number generator
                  %
                  if DSGEModel.RandomNumberValue==1;
                     randn('state',0);
                     rand('state',0);
                  else;
                     randn('state',sum(100*clock));
                     rand('state',sum(100*clock));
                  end;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check how many draws we should pick among these
                  %
                  PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
                  NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
                  %
                  % get the draws to use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     if DSGEModel.RandomizeDraws==0;
                        DrawFreq = round(NumDraws/NumPostDrawsUsage);
                        SelectDraws = (1:DrawFreq:NumDraws);
                        if length(SelectDraws)+1==NumPostDrawsUsage;
                           SelectDraws = [SelectDraws NumDraws];
                        elseif length(SelectDraws)>NumPostDrawsUsage;
                           SelectDraws = SelectDraws(1:NumPostDrawsUsage);
                        end;
                        thetaPostSample = thetaPostSample(SelectDraws,:);
                     else;
                        thetaPostSample = thetaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
                     end;
                  else;
                     txt = 'Would you like to use a sub-sample of the posterior draws when computing the predictive distribution?';
                     answer = Query(txt,'question',400,'Predictive Distributions - Posterior',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % select how many draws to use
                        %
                        SelectDraws = SampledPosteriorDraws(DSGEModel,CurrINI,'Predictive Distributions - Posterior',Weights,NumDraws,[]);
                        if isempty(SelectDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(SelectDraws,:);
                        if DSGEModel.PosteriorSampler~=9;
                           %
                           % fix the weights
                           %
                           Weights = Weights(SelectDraws,1);
                           Weights = Weights*length(Weights)/sum(Weights);
                        else;
                           if isempty(Weights)==0;
                              Weights = Weights(SelectDraws);
                           end;
                        end;
                     end;
                  end;
                  %
                  % are we using annualized data, transformed or the original
                  %
                  if gcbo==controls.learningtools.predictivedist.unconditional.original.posterior;
                     TransData = 0;
                     AnnualizeData = 0;
                     AnnualStr = 'original';
                     QueryHead = 'Predictive Distributions - Original Data';
                  elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.posterior;
                     TransData = 0;
                     AnnualizeData = 1;
                     AnnualStr = 'annual';
                     QueryHead = 'Predictive Distributions - Annualized Data';
                  elseif gcbo==controls.learningtools.predictivedist.unconditional.transformed.posterior;
                     TransData = 1;
                     AnnualizeData = 0;
                     AnnualStr = 'transformed';
                     QueryHead = 'Predictive Distributions - Transformed Data';
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  %
                  if PredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1)==1;
                     txt = 'YADA has located results under adaptive learning for the predictive distribution on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     [PredEventData,PredictionEvent,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = LoadAdaptiveLearningDSGEPredictionUncertainty(DSGEModel.OutputDirectory,'','',DSGEModel.NameOfModel,DSGEModel.FixedBeliefsForProj,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1);
                     DoneCalc = 1;
                  else;
                     %
                     % ask if we should conduct prediction event calculations
                     %
                     txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                     answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                           DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                        end;
                        %
                        % make sure that the prediction event length does not exceed the maximum prediction horizon
                        %
                        for i=1:DSGEModel.n;
                           if DSGEModel.PredictionEvent(i,3)>MaxPH;
                              DSGEModel.PredictionEvent(i,3) = MaxPH;
                           end;
                        end;
                        %
                        % display a dialog for setting up an prediction event
                        %
                        [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                        if strcmp(lower(action),'ok')==1;
                           DSGEModel.PredictionEvent = PredictionEvent;
                           DSGEModel.RunPredictionEvent = 1;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                     else;
                        DSGEModel.RunPredictionEvent = 0;
                     end;
                     %
                     % we can now compute the predictive distribution for the endogenous variables
                     % up to period MaxPH
                     %
                     [DoneCalc,PredEventData,StateCov,ShockCov,MeasureCov,ParameterCov] = AdaptiveLearningDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
                  %
                  % check if we should show any results
                  %
                  if DoneCalc==1;
                     %
                     % yes, now we need to calculate some distribution properties
                     %
                     [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = AdaptiveLearningDSGEPredictionDistribution(DSGEModel.n,DSGEModel.OutputDirectory,'','',DSGEModel.NameOfModel,DSGEModel.FixedBeliefsForProj,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,DSGEModel.Percentiles,DSGEModel.VariableNames,DSGEModel.ConfidenceRegionMethod,CurrINI);
                     %
                     % compute the forecast errors based on the mean predictions
                     %
                     PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                     %
                     % check if the user wants to save the data
                     %
                     txt = ['Would you like to save the adaptive learning based predictive distributions for the ' AnnualStr ' data to file?'];
                     answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                        file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningPredictiveDist-' int2str(DSGEModel.FixedBeliefsForProj) '-' AnnualStr '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPostDrawsUsage,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                        YNames = DSGEModel.VariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                        Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        if (length(DateVector)>size(Y,2))&(TransData==0);
                           DateVector = DateVector(1,1:size(Y,2));
                        end;
                        EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                        EstimationStartPeriodOrig = EstimationStartPeriod;
                        YOrig = Y;
                        DateVectorOrig = DateVector;
                        if AnnualizeData==1;
                           Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                           EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                           DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                        end;
                        if TransData==1;
                           [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                           Y = YData';
                           if size(Y,2)==DSGEModel.T;
                              Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           else;
                              T2 = size(Y,2);
                              if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                 Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           end;
                        end;
                        if isempty(PredEventData)==1;
                           save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateCov','ShockCov','MeasureCov','ParameterCov','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                        else;
                           save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','StateCov','ShockCov','MeasureCov','ParameterCov','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData');
                        end;
                        %
                        % create a matlab m-file to read the output.
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\ALPD' int2str(DSGEModel.FixedBeliefsForProj) num2str(CurrChain,'%0.0f') AnnualStr num2str(MaxPH,'%0.0f') num2str(NumPostDrawsUsage,'%0.0f') num2str(NumPredPaths,'%0.0f') DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                        fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                        fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                        fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                        fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                        fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                        fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                        fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                        fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                        fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                        fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                        fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                        fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                        fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                        fprintf(fid,'%%\n%% observed variables\n%%\n');
                        fprintf(fid,'Y = DataStructure.Y;\n');
                        fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        if (AnnualizeData==0)&(TransData==0);
                           fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to state uncertainty\n%%\n');
                           fprintf(fid,'StateCov = DataStructure.StateCov;\n');
                           fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to shock uncertainty\n%%\n');
                           fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                           fprintf(fid,'%%\n%% 2D Error covariance matrix (nxn) part due to measurement error uncertainty\n%%\n');
                           fprintf(fid,'MeasureCov = DataStructure.MeasureCov;\n');
                           fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                           fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                           CovDecompStr = 'StateCov (3D error covariance matrix with part due to state uncertainty), ShockCov (3D error covariance matrix with part due to shock uncertainty), MeasureCov (2D error covariance matrix with part due to measurement error uncertainty), ParameterCov (3D covariance matrix with part due to parameter uncertainty), ';
                        else;
                           CovDecompStr = '';
                        end;
                        if isempty(PredEventData)==0;
                           fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                           fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                           fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                           fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                           if (AnnualizeData==0)&(TransData==0);
                              NumEntries = '16';
                           else;
                              NumEntries = '12';
                           end;
                           ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                        else;
                           if (AnnualizeData==0)&(TransData==0);
                              NumEntries = '14';
                           else;
                              NumEntries = '10';
                           end;
                           ExtraEntry = '';
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        txt = ['The predictive distributions under adaptive learning have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Predictive Distributions',500,600,CurrINI);
                     end;
                     if isempty(PredEventData)==0;
                        %
                        % write prediction event results to file
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningPredictiveDist-' int2str(DSGEModel.FixedBeliefsForProj) '-' AnnualStr '-' num2str(CurrChain,'%0.0f') '-' num2str(MaxPH,'%0.0f') '-' num2str(NumPostDrawsUsage,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                        PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                        %
                        % displays the file if it exists
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                        end;
                     end;
                     %
                     % summarize the information in the uncertainty variables more conveniently
                     %
                     if isempty(StateCov)==0;
                        [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(StateCov,ShockCov,MeasureCov,ParameterCov);
                        %
                        % Open a dialog for displaying predictive distributions
                        %
                        UData.StateVar = StateVar;
                        UData.ShockVar = ShockVar;
                        UData.MeasureVar = MeasureVar;
                        UData.ParameterVar = ParameterVar;
                        [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                     else;
                        UData = [];
                     end;
                     PredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,[],Quantiles,KernelX,KernelY,'','DSGE',AnnualStr,1,UData);
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarDecompInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,1);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(StateVarStructure)==0;
                     if gcbo==controls.learningtools.decompositions.obsvar.decomp.original.initialvalues;
                        AnnualStr = 'Original';
                     elseif gcbo==controls.learningtools.decompositions.obsvar.decomp.annual.initialvalues;
                        AnnualStr = 'Annualized';
                     end;
                     %
                     txt = ['Would you like to produce scatter plots of the decompositions for the ' lower(AnnualStr) ' data instead of time series based bar charts?'];
                     answer = Query(txt,'question',200,'Observed Variable Decomposition - Initial Values',500,'no',CurrINI);
                     %
                     % we can now send the data for graphing
                     %
                     if strcmp(lower(answer),'no')==1;
                        ObsVarDecompDLG('init',StateVarStructure,'Initial Values',AnnualStr,CurrINI,DSGEModel);
                     else;
                        ScatterDecompDLG('init',StateVarStructure,'Initial Values',AnnualStr,CurrINI,DSGEModel);
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the observed variable decompositions based on the initial values.';
                  About(txt,'information','Observed Variable Decompositions - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the observed variable decompositions based on the initial values.';
                  About(txt,'information','Observed Variable Decompositions - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the observed variable decompositions based on the initial values.'];
                About(txt,'information','Observed Variable Decompositions - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarDecompDSGEMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,1);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(StateVarStructure)==0;
                     if gcbo==controls.learningtools.decompositions.obsvar.decomp.original.dsgemode;
                        AnnualStr = 'Original';
                     elseif gcbo==controls.learningtools.decompositions.obsvar.decomp.annual.dsgemode;
                        AnnualStr = 'Annualized';
                     end;
                     %
                     txt = ['Would you like to produce scatter plots of the decompositions for the ' lower(AnnualStr) ' data instead of time series based bar charts?'];
                     answer = Query(txt,'question',200,'Observed Variable Decomposition - DSGE Posterior Mode',500,'no',CurrINI);
                     %
                     % we can now send the data for graphing
                     %
                     if strcmp(lower(answer),'no')==1;
                        ObsVarDecompDLG('init',StateVarStructure,'DSGE Posterior Mode',AnnualStr,CurrINI,DSGEModel);
                     else;
                        ScatterDecompDLG('init',StateVarStructure,'DSGE Posterior Mode',AnnualStr,CurrINI,DSGEModel);
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the observed variable decomposition based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Observed Variable Decomposition - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the observed variable decomposition based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Observed Variable Decomposition - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the observed variable decomposition based on the DSGE model posterior mode estimates.'];
               About(txt,'information','Observed Variable Decomposition - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarDecompPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % now we call the state variable estimation function
            %
%            if (isfield(controls,'svs')==1)&(CheckStateScenarioData(DSGEModel)==1);
%               txt = 'Would you like to estimate structural shocks using a constrained Kalman filter routine instead of the standard Kalman filter?';
%               answer = Query(txt,'question',200,'Observed Variable Decomposition - Posterior Mode',500,'no',CurrINI);
%               if strcmp(lower(answer),'yes')==1;
%                  ConEst = 1;
%               else;
%                  ConEst = 0;
%               end;
%            else;
%               ConEst = 0;
%            end;
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,1);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if isempty(StateVarStructure)==0;
               if gcbo==controls.learningtools.decompositions.obsvar.decomp.original.postmode;
                  AnnualStr = 'Original';
               elseif gcbo==controls.learningtools.decompositions.obsvar.decomp.annual.postmode;
                  AnnualStr = 'Annualized';
               end;
               %
               txt = ['Would you like to produce scatter plots of the decompositions for the ' lower(AnnualStr) ' data instead of time series based bar charts?'];
               answer = Query(txt,'question',200,'Observed Variable Decomposition - Posterior Mode',500,'no',CurrINI);
               %
               % we can now send the data for graphing
               %
               if strcmp(lower(answer),'no')==1;
                  ObsVarDecompDLG('init',StateVarStructure,'Posterior Mode',AnnualStr,CurrINI,DSGEModel);
               else;
                  ScatterDecompDLG('init',StateVarStructure,'Posterior Mode',AnnualStr,CurrINI,DSGEModel);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVarDecompInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,1);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % we can now send the data for graphing
                     %
                     StateVarDecompDLG('init',StateVarStructure,'Initial Values',CurrINI,DSGEModel);
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the state variable decompositions based on the initial values.';
                  About(txt,'information','State Variable Decompositions - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the state variable decompositions based on the initial values.';
                  About(txt,'information','State Variable Decompositions - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the state variable decompositions based on the initial values.'];
                About(txt,'information','State Variable Decompositions - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVarDecompDSGEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,1);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % we can now send the data for graphing
                     %
                     StateVarDecompDLG('init',StateVarStructure,'DSGE Posterior Mode',CurrINI,DSGEModel);
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the state variable decomposition based on the DSGE model posterior mode estimates.';
                  About(txt,'information','State Variable Decomposition - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the state variable decomposition based on the DSGE model posterior mode estimates.';
                  About(txt,'information','State Variable Decomposition - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the state variable decomposition based on the DSGE model posterior mode estimates.'];
               About(txt,'information','State Variable Decomposition - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVarDecompPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % now we call the state variable estimation function
            %
%            if (isfield(controls,'svs')==1)&(CheckStateScenarioData(DSGEModel)==1);
%               txt = 'Would you like to estimate structural shocks using a constrained Kalman filter routine instead of the standard Kalman filter?';
%               answer = Query(txt,'question',200,'Observed Variable Decomposition - Posterior Mode',500,'no',CurrINI);
%               if strcmp(lower(answer),'yes')==1;
%                  ConEst = 1;
%               else;
%                  ConEst = 0;
%               end;
%            else;
%               ConEst = 0;
%            end;
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,1);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if isempty(StateVarStructure)==0;
               %
               % we can now send the data for graphing
               %
               StateVarDecompDLG('init',StateVarStructure,'Posterior Mode',CurrINI,DSGEModel);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRFSInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  thetaIndex = ModeData.thetaIndex;
                  UniformBounds = ModeData.UniformBounds;
                  LowerBound = ModeData.LowerBound;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         % check if we can continue
         %
         if InitStatus==1;
            %
            % get the necessary information about the adaptive learning parameters
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the log-likelihood
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(4);
               end;
               [StateVarStructure,BeliefStatus,TransStatus,SolutionStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,1);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               if (BeliefStatus==1)&(TransStatus==1)&(SolutionStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % take into account that the solution matrices in Mtilde, Ftilde and Btilde are lagged one period
                     %
                     [r,q,T] = size(StateVarStructure.Btilde);
                     Ftilde = zeros(r,r,T);
                     Ftilde(:,:,1:T-1) = StateVarStructure.Ftilde(:,:,2:T);
                     Ftilde(:,:,T) = StateVarStructure.LastSolution.FTT;
                     Btilde = zeros(r,q,T);
                     Btilde(:,:,1:T-1) = StateVarStructure.Btilde(:,:,2:T);
                     Btilde(:,:,T) = StateVarStructure.LastSolution.BTT;
                     %
                     % check which transformation we should use
                     %
                     if gcbo==controls.learningtools.irfs.original.initialvalues;
                        UseLevels = 0;
                     elseif gcbo==controls.learningtools.irfs.levels.initialvalues;
                        UseLevels = 1;
                     elseif gcbo==controls.learningtools.irfs.annual.initialvalues;
                        UseLevels = 2;
                     end;
                     %
                     % compute the impulse responses
                     %
                     IRStructure = AdaptiveLearningIRFtheta(Ftilde,Btilde,StateVarStructure.Fss,StateVarStructure.Bss,StateVarStructure.H,StateVarStructure.F,StateVarStructure.B0,DSGEModel,UseLevels);
                     if isempty(IRStructure)==0;
                        txt = 'Would you like to save the impulse response functions for the adaptive learning and rational expectations solutions at the initial parameter values to file?';
                        answer = Query(txt,'question',140,'Save - Impulse Responses',500,'no',CurrINI);
                        drawnow;
                        if strcmp(lower(answer),'yes')==1;
                           YNames = DSGEModel.VariableNames;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           ShockNames = DSGEModel.StateShockNames;
                           %
                           % rational expectations
                           %
                           RE.IRFState = IRStructure.RE.Ksi;
                           RE.IRFY = IRStructure.RE.Y;
                           %
                           % adaptive learning
                           %
                           ALSS.IRFState = IRStructure.ALSS.Ksi;
                           ALSS.IRFY = IRStructure.ALSS.Y;
                           for t=1:T;
                              AL(t).IRFState = IRStructure.AL(t).Ksi;
                              AL(t).IRFY = IRStructure.AL(t).Y;
                           end;
                           file = FixFilePath([DSGEModel.OutputDirectory '\ImpulseResponses-AdaptiveLearning-InitialValues-']);
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\IRALInitVal']);
                           if UseLevels==0;
                              file = [file 'Original.mat'];
                              mfile = [mfile 'Original.m'];
                           elseif UseLevels==1;
                              file = [file 'Levels.mat'];
                              mfile = [mfile 'Levels.m'];
                           else;
                              file = [file 'Annual.mat'];
                              mfile = [mfile 'Annual.m'];
                           end;
                           save(file,'YNames','StateVariableNames','ShockNames','RE','AL','ALSS');
                           %
                           % create a matlab file to read the data
                           %
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% a structure with fields IRFState and IRFY of 3D matrices of dimension rxqxh%%\n%% and nxqxh respectively with the responses under rational expectations%%\n%% for the r state variables and n observed variables from the q shocks from%%\n%% period 0 to period h-1\n%%\n');
                           fprintf(fid,'RE = DataStructure.RE;\n');
                           fprintf(fid,'%%\n%% a structure with fields IRFState and IRFY of 3D matrices of dimension rxqxh%%\n%% and nxqxh respectively with the responses under adaptive learning (steady-state)%%\n%% for the r state variables and n observed variables from the q shocks from%%\n%% period 0 to period h-1\n%%\n');
                           fprintf(fid,'ALSS = DataStructure.ALSS;\n');
                           fprintf(fid,'%%\n%% a structure of dimension T with fields IRFState and IRFY of 3D matrices of%%\n%% dimension rxqxh and nxqxh respectively with the responses under adaptive%%\n%% learning for T time periods for the r state variables and n observed%%\n%% variables from the q shocks from period 0 to period h-1\n%%\n');
                           fprintf(fid,'AL = DataStructure.AL;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The impulse response data under adaptive learning and rational expectations at the initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), ShockNames (string matrix with the names of the economic shocks), RE (a structure with fields IRFState and IRFY with 3D matrices of dimension rxqxh and nxqxh with the responses for the r state and n observed variables under rational expectations from the q shocks from period 0 to period h-1), and AL (a structure of dimension T with fields IRFState and IRFY with 3D matrices of dimension rxqxh and nxqxh with the responses for the r state and n observed variables under adaptive learning). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Impulse Responses - Initial Values',400,600,CurrINI);
                           drawnow;
                        end;
                        %
                        % display selected impulse responses
                        %
                        AdaptiveLearningIRGraphDLG('init',CurrINI,DSGEModel,IRStructure,'DSGE Posterior Mode',UseLevels);
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with impulse responses based on the initial values.';
                  About(txt,'information','Impulse Responses - Initial Values',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the impulse responses based on the initial values.';
                  About(txt,'information','Impulse Responses - Initial Values',300,600,CurrINI);
               end;
            else;
                txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the impulse responses based on the initial values.'];
                About(txt,'information','Impulse Responses - Initial Values',300,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRFSDSGEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % we be doing OK
            %
            [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
            if status==1;
               %
               % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
               % data from the DSGE model with its posterior mode estimates
               %
               [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(ModeData.thetaMode, ...
                      ModeData.thetaIndex,ModeData.thetaDist,ModeData.LowerBound,ModeData.ModelParameters,ModeData.thetaPositions,ModeData.PriorDist,ModeData.ParameterNames, ...
                      ModeData.UniformBounds,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
               %
               % now we call the state variable estimation function which also deals with the solution to the model
               %
               if DSGEModel.n>=20;
                  drawnow;
                  pause(2);
               end;
               [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,[],DSGEModel,CurrINI,0);
               if DSGEModel.n>=20;
                  drawnow;
                  pause(1);
               end;
               if (BeliefStatus==1)&(TransStatus==1);
                  %
                  % we're may be OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % take into account that the solution matrices in Mtilde, Ftilde and Btilde are lagged one period
                     %
                     [r,q,T] = size(StateVarStructure.Btilde);
                     Ftilde = zeros(r,r,T);
                     Ftilde(:,:,1:T-1) = StateVarStructure.Ftilde(:,:,2:T);
                     Ftilde(:,:,T) = StateVarStructure.LastSolution.FTT;
                     Btilde = zeros(r,q,T);
                     Btilde(:,:,1:T-1) = StateVarStructure.Btilde(:,:,2:T);
                     Btilde(:,:,T) = StateVarStructure.LastSolution.BTT;
                     %
                     % check which transformation we should use
                     %
                     if gcbo==controls.learningtools.irfs.original.dsgemode;
                        UseLevels = 0;
                     elseif gcbo==controls.learningtools.irfs.levels.dsgemode;
                        UseLevels = 1;
                     elseif gcbo==controls.learningtools.irfs.annual.dsgemode;
                        UseLevels = 2;
                     end;
                     %
                     % compute the impulse responses
                     %
                     IRStructure = AdaptiveLearningIRFtheta(Ftilde,Btilde,StateVarStructure.Fss,StateVarStructure.Bss,StateVarStructure.H,StateVarStructure.F,StateVarStructure.B0,DSGEModel,UseLevels);
                     if isempty(IRStructure)==0;
                        txt = 'Would you like to save the impulse response functions for the adaptive learning and rational expectations solutions at the DSGE posterior mode values to file?';
                        answer = Query(txt,'question',140,'Save - Impulse Responses',500,'no',CurrINI);
                        drawnow;
                        if strcmp(lower(answer),'yes')==1;
                           YNames = DSGEModel.VariableNames;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           ShockNames = DSGEModel.StateShockNames;
                           %
                           % rational expectations
                           %
                           RE.IRFState = IRStructure.RE.Ksi;
                           RE.IRFY = IRStructure.RE.Y;
                           %
                           % adaptive learning
                           %
                           ALSS.IRFState = IRStructure.ALSS.Ksi;
                           ALSS.IRFY = IRStructure.ALSS.Y;
                           for t=1:T;
                              AL(t).IRFState = IRStructure.AL(t).Ksi;
                              AL(t).IRFY = IRStructure.AL(t).Y;
                           end;
                           file = FixFilePath([DSGEModel.OutputDirectory '\ImpulseResponses-AdaptiveLearning-DSGEPosteriorMode-']);
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\IRALDSGEMode']);
                           if UseLevels==0;
                              file = [file 'Original.mat'];
                              mfile = [mfile 'Original.m'];
                           elseif UseLevels==1;
                              file = [file 'Levels.mat'];
                              mfile = [mfile 'Levels.m'];
                           else;
                              file = [file 'Annual.mat'];
                              mfile = [mfile 'Annual.m'];
                           end;
                           save(file,'YNames','StateVariableNames','ShockNames','RE','AL','ALSS');
                           %
                           % create a matlab file to read the data
                           %
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% a structure with fields IRFState and IRFY of 3D matrices of dimension rxqxh%%\n%% and nxqxh respectively with the responses under rational expectations%%\n%% for the r state variables and n observed variables from the q shocks from%%\n%% period 0 to period h-1\n%%\n');
                           fprintf(fid,'RE = DataStructure.RE;\n');
                           fprintf(fid,'%%\n%% a structure with fields IRFState and IRFY of 3D matrices of dimension rxqxh%%\n%% and nxqxh respectively with the responses under adaptive learning (steady-state)%%\n%% for the r state variables and n observed variables from the q shocks from%%\n%% period 0 to period h-1\n%%\n');
                           fprintf(fid,'ALSS = DataStructure.ALSS;\n');
                           fprintf(fid,'%%\n%% a structure of dimension T with fields IRFState and IRFY of 3D matrices of%%\n%% dimension rxqxh and nxqxh respectively with the responses under adaptive%%\n%% learning for T time periods for the r state variables and n observed%%\n%% variables from the q shocks from period 0 to period h-1\n%%\n');
                           fprintf(fid,'AL = DataStructure.AL;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The impulse response data under adaptive learning and rational expectations at the DSGE posterior mode values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), ShockNames (string matrix with the names of the economic shocks), RE (a structure with fields IRFState and IRFY with 3D matrices of dimension rxqxh and nxqxh with the responses for the r state and n observed variables under rational expectations from the q shocks from period 0 to period h-1), and AL (a structure of dimension T with fields IRFState and IRFY with 3D matrices of dimension rxqxh and nxqxh with the responses for the r state and n observed variables under adaptive learning). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Impulse Responses - DSGE Posterior Mode',400,600,CurrINI);
                           drawnow;
                        end;
                        %
                        % display selected impulse responses
                        %
                        AdaptiveLearningIRGraphDLG('init',CurrINI,DSGEModel,IRStructure,'DSGE Posterior Mode',UseLevels);
                     end;
                  end;
               elseif TransStatus==0;
                  txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to continue with the impulse responses based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Impulse Responses - DSGE Posterior Mode',300,600,CurrINI);
               elseif BeliefStatus==0;
                  txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the impulse responses based on the DSGE model posterior mode estimates.';
                  About(txt,'information','Impulse Responses - DSGE Posterior Mode',300,600,CurrINI);
               end;
            else;
               txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the impulse responses based on the DSGE model posterior mode estimates.'];
               About(txt,'information','Impulse Responses - DSGE Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRFSPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the posterior mode estimates from the adaptive learning version of the DSGE model
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % compute the solution matrices based on the Kalman filter routine
            %
            if DSGEModel.n>=20;
               drawnow;
               pause(2);
            end;
            [StateVarStructure,BeliefStatus,TransStatus] = CalculateAdaptiveLearningStateVariables(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,[],DSGEModel,CurrINI,0);
            if DSGEModel.n>=20;
               drawnow;
               pause(1);
            end;
            if (isempty(StateVarStructure)==0)&(BeliefStatus==1)&(TransStatus==1);
               %
               % take into account that the solution matrices in Mtilde, Ftilde and Btilde are lagged one period
               %
               [r,q,T] = size(StateVarStructure.Btilde);
               Ftilde = zeros(r,r,T);
               Ftilde(:,:,1:T-1) = StateVarStructure.Ftilde(:,:,2:T);
               Ftilde(:,:,T) = StateVarStructure.LastSolution.FTT;
               Btilde = zeros(r,q,T);
               Btilde(:,:,1:T-1) = StateVarStructure.Btilde(:,:,2:T);
               Btilde(:,:,T) = StateVarStructure.LastSolution.BTT;
               %
               % check which transformation we should use
               %
               if gcbo==controls.learningtools.irfs.original.postmode;
                  UseLevels = 0;
               elseif gcbo==controls.learningtools.irfs.levels.postmode;
                  UseLevels = 1;
               elseif gcbo==controls.learningtools.irfs.annual.postmode;
                  UseLevels = 2;
               end;
               %
               % compute the impulse responses
               %
               IRStructure = AdaptiveLearningIRFtheta(Ftilde,Btilde,StateVarStructure.Fss,StateVarStructure.Bss,StateVarStructure.H,StateVarStructure.F,StateVarStructure.B0,DSGEModel,UseLevels);
               if isempty(IRStructure)==0;
                  txt = 'Would you like to save the impulse response functions for the adaptive learning and rational expectations solutions at posterior mode to file?';
                  answer = Query(txt,'question',140,'Save - Impulse Responses',500,'no',CurrINI);
                  drawnow;
                  if strcmp(lower(answer),'yes')==1;
                     YNames = DSGEModel.VariableNames;
                     StateVariableNames = DSGEModel.StateVariableNames;
                     ShockNames = DSGEModel.StateShockNames;
                     %
                     % rational expectations
                     %
                     RE.IRFState = IRStructure.RE.Ksi;
                     RE.IRFY = IRStructure.RE.Y;
                     %
                     % adaptive learning
                     %
                     ALSS.IRFState = IRStructure.ALSS.Ksi;
                     ALSS.IRFY = IRStructure.ALSS.Y;
                     for t=1:T;
                        AL(t).IRFState = IRStructure.AL(t).Ksi;
                        AL(t).IRFY = IRStructure.AL(t).Y;
                     end;
                     file = FixFilePath([DSGEModel.OutputDirectory '\ImpulseResponses-AdaptiveLearning-PosteriorMode-']);
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\IRALPostMode']);
                     if UseLevels==0;
                        file = [file 'Original.mat'];
                        mfile = [mfile 'Original.m'];
                     elseif UseLevels==1;
                        file = [file 'Levels.mat'];
                        mfile = [mfile 'Levels.m'];
                     else;
                        file = [file 'Annual.mat'];
                        mfile = [mfile 'Annual.m'];
                     end;
                     save(file,'YNames','StateVariableNames','ShockNames','RE','AL','ALSS');
                     %
                     % create a matlab file to read the data
                     %
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStructure.YNames;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                     fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                     fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                     fprintf(fid,'%%\n%% a structure with fields IRFState and IRFY of 3D matrices of dimension rxqxh%%\n%% and nxqxh respectively with the responses under rational expectations%%\n%% for the r state variables and n observed variables from the q shocks from%%\n%% period 0 to period h-1\n%%\n');
                     fprintf(fid,'RE = DataStructure.RE;\n');
                     fprintf(fid,'%%\n%% a structure with fields IRFState and IRFY of 3D matrices of dimension rxqxh%%\n%% and nxqxh respectively with the responses under adaptive learning (steady-state)%%\n%% for the r state variables and n observed variables from the q shocks from%%\n%% period 0 to period h-1\n%%\n');
                     fprintf(fid,'ALSS = DataStructure.ALSS;\n');
                     fprintf(fid,'%%\n%% a structure of dimension T with fields IRFState and IRFY of 3D matrices of%%\n%% dimension rxqxh and nxqxh respectively with the responses under adaptive%%\n%% learning for T time periods for the r state variables and n observed%%\n%% variables from the q shocks from period 0 to period h-1\n%%\n');
                     fprintf(fid,'AL = DataStructure.AL;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The impulse response data under adaptive learning and rational expectations at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), ShockNames (string matrix with the names of the economic shocks), RE (a structure with fields IRFState and IRFY with 3D matrices of dimension rxqxh and nxqxh with the responses for the r state and n observed variables under rational expectations from the q shocks from period 0 to period h-1), and AL (a structure of dimension T with fields IRFState and IRFY with 3D matrices of dimension rxqxh and nxqxh with the responses for the r state and n observed variables under adaptive learning). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Impulse Responses - Posterior Mode',400,600,CurrINI);
                     drawnow;
                  end;
                  %
                  % display selected impulse responses
                  %
                  AdaptiveLearningIRGraphDLG('init',CurrINI,DSGEModel,IRStructure,'Posterior Mode',UseLevels);
               end;
            elseif TransStatus==0;
               txt = 'The transformation of the DSGE model to remove expected future non-forward looking variables was not successful. Rewriting the model equations by substitution of such variables may solve the problem. YADA was therefore unable to estimate the impulse responses based on the posterior mode estimates.';
               About(txt,'information','Impulse Responses - Posterior Mode',300,600,CurrINI);
            elseif BeliefStatus==0;
               txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to estimate the impulse responses based on the posterior mode estimates.';
               About(txt,'information','Impulse Responses - Posterior Mode',300,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of LearningMenuFunctions.m
%
